@@ -1,9 +1,10 @@
+/* $Id: GuestControlSvc.h $ */
 /** @file
  * Guest control service - Common header for host service and guest clients.
  */
 
 /*
- * Copyright (C) 2011-2013 Oracle Corporation
+ * Copyright (C) 2011-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -26,13 +27,10 @@
 #ifndef ___VBox_HostService_GuestControlService_h
 #define ___VBox_HostService_GuestControlService_h
 
-#include <VBox/types.h>
-#include <VBox/VMMDev.h>
-#include <VBox/VBoxGuest2.h>
+#include <VBox/VMMDevCoreTypes.h>
+#include <VBox/VBoxGuestCoreTypes.h>
 #include <VBox/hgcmsvc.h>
-#include <VBox/log.h>
 #include <iprt/assert.h>
-#include <iprt/string.h>
 
 /* Everything defined in this file lives in this namespace. */
 namespace guestControl {
@@ -80,132 +78,6 @@ namespace guestControl {
 #define VBOX_GUESTCTRL_FILTER_BY_SESSION(uSession) \
     (VBOX_GUESTCTRL_CONTEXTID_MAKE_SESSION(uSession) | 0xF8000000)
 
-/**
- * Process status when executed in the guest.
- */
-enum eProcessStatus
-{
-    /** Process is in an undefined state. */
-    PROC_STS_UNDEFINED = 0,
-    /** Process has been started. */
-    PROC_STS_STARTED = 1,
-    /** Process terminated normally. */
-    PROC_STS_TEN = 2,
-    /** Process terminated via signal. */
-    PROC_STS_TES = 3,
-    /** Process terminated abnormally. */
-    PROC_STS_TEA = 4,
-    /** Process timed out and was killed. */
-    PROC_STS_TOK = 5,
-    /** Process timed out and was not killed successfully. */
-    PROC_STS_TOA = 6,
-    /** Service/OS is stopping, process was killed. */
-    PROC_STS_DWN = 7,
-    /** Something went wrong (error code in flags). */
-    PROC_STS_ERROR = 8
-};
-
-/**
- * Input flags, set by the host. This is needed for
- * handling flags on the guest side.
- * Note: Has to match Main's ProcessInputFlag_* flags!
- */
-#define INPUT_FLAG_NONE                     0x0
-#define INPUT_FLAG_EOF                      RT_BIT(0)
-
-/**
- * Guest session creation flags.
- * Only handled internally at the moment.
- */
-#define SESSIONCREATIONFLAG_NONE            0x0
-
-/**
- * Guest directory removement flags.
- * Essentially using what IPRT's RTDIRRMREC_F_
- * defines have to offer.
- */
-#define DIRREMOVE_FLAG_RECURSIVE            RT_BIT(0)
-/** Delete the content of the directory and the directory itself. */
-#define DIRREMOVE_FLAG_CONTENT_AND_DIR      RT_BIT(1)
-/** Only delete the content of the directory, omit the directory it self. */
-#define DIRREMOVE_FLAG_CONTENT_ONLY         RT_BIT(2)
-/** Mask of valid flags. */
-#define DIRREMOVE_FLAG_VALID_MASK           UINT32_C(0x00000003)
-
-/**
- * Guest process creation flags.
- * Note: Has to match Main's ProcessCreateFlag_* flags!
- */
-#define EXECUTEPROCESSFLAG_NONE             0x0
-#define EXECUTEPROCESSFLAG_WAIT_START       RT_BIT(0)
-#define EXECUTEPROCESSFLAG_IGNORE_ORPHANED  RT_BIT(1)
-#define EXECUTEPROCESSFLAG_HIDDEN           RT_BIT(2)
-#define EXECUTEPROCESSFLAG_NO_PROFILE       RT_BIT(3)
-#define EXECUTEPROCESSFLAG_WAIT_STDOUT      RT_BIT(4)
-#define EXECUTEPROCESSFLAG_WAIT_STDERR      RT_BIT(5)
-#define EXECUTEPROCESSFLAG_EXPAND_ARGUMENTS RT_BIT(6)
-
-/**
- * Pipe handle IDs used internally for referencing to
- * a certain pipe buffer.
- */
-#define OUTPUT_HANDLE_ID_STDOUT_DEPRECATED  0 /* Needed for VBox hosts < 4.1.0. */
-#define OUTPUT_HANDLE_ID_STDOUT             1
-#define OUTPUT_HANDLE_ID_STDERR             2
-
-/**
- * Guest path rename flags.
- * Essentially using what IPRT's RTPATHRENAME_FLAGS_
- * defines have to offer.
- */
-/** Do not replace anything. */
-#define PATHRENAME_FLAG_NO_REPLACE          UINT32_C(0)
-/** This will replace attempt any target which isn't a directory. */
-#define PATHRENAME_FLAG_REPLACE             RT_BIT(0)
-/** Don't allow symbolic links as part of the path. */
-#define PATHRENAME_FLAG_NO_SYMLINKS         RT_BIT(1)
-/** Mask of valid flags. */
-#define PATHRENAME_FLAG_VALID_MASK          UINT32_C(0x00000002)
-
-/**
- * Defines for guest process array lengths.
- */
-#define GUESTPROCESS_MAX_CMD_LEN            _1K
-#define GUESTPROCESS_MAX_ARGS_LEN           _1K
-#define GUESTPROCESS_MAX_ENV_LEN            _64K
-#define GUESTPROCESS_MAX_USER_LEN           128
-#define GUESTPROCESS_MAX_PASSWORD_LEN       128
-#define GUESTPROCESS_MAX_DOMAIN_LEN         256
-
-/** @name Internal tools built into VBoxService which are used in order to
- *        accomplish tasks host<->guest.
- * @{
- */
-#define VBOXSERVICE_TOOL_CAT        "vbox_cat"
-#define VBOXSERVICE_TOOL_LS         "vbox_ls"
-#define VBOXSERVICE_TOOL_RM         "vbox_rm"
-#define VBOXSERVICE_TOOL_MKDIR      "vbox_mkdir"
-#define VBOXSERVICE_TOOL_MKTEMP     "vbox_mktemp"
-#define VBOXSERVICE_TOOL_STAT       "vbox_stat"
-/** @} */
-
-/**
- * Input status, reported by the client.
- */
-enum eInputStatus
-{
-    /** Input is in an undefined state. */
-    INPUT_STS_UNDEFINED = 0,
-    /** Input was written (partially, see cbProcessed). */
-    INPUT_STS_WRITTEN = 1,
-    /** Input failed with an error (see flags for rc). */
-    INPUT_STS_ERROR = 20,
-    /** Process has abandoned / terminated input handling. */
-    INPUT_STS_TERMINATED = 21,
-    /** Too much input data. */
-    INPUT_STS_OVERFLOW = 30
-};
-
 /**
  * Structure keeping the context of a host callback.
  */
@@ -313,6 +185,10 @@ enum eHostFn
      * Gets the current file position of an opened guest file.
      */
     HOST_FILE_TELL = 271,
+    /**
+     * Changes the file size.
+     */
+    HOST_MSG_FILE_SET_SIZE,
     /**
      * Removes a directory on the guest.
      */
@@ -332,6 +208,17 @@ enum eGuestFn
     /**
      * Guest waits for a new message the host wants to process on the guest side.
      * This is a blocking call and can be deferred.
+     *
+     * @note This command is rather odd.  The above description isn't really
+     *       correct.  Yes, it (1) waits for a new message and will return the
+     *       mesage number and parameter count when one is available.   However, it
+     *       is also (2) used to retrieve the message parameters.   For some weird
+     *       reasons it was decided that it should always return VERR_TOO_MUCH_DATA
+     *       when used in the first capacity.
+     *
+     * @todo r=bird: Next time this interface gets a major adjustment, please split
+     *       this function up into two calls and, for heavens sake, make them return
+     *       VINF_SUCCESS on success.
      */
     GUEST_MSG_WAIT = 1,
     /**
@@ -379,6 +266,37 @@ enum eGuestFn
      * Guest wants to close a specific guest session.
      */
     GUEST_SESSION_CLOSE = 21,
+    /** Report guest side feature flags and retrieve the host ones.
+     *
+     * VBoxService makes this call right after becoming master to indicate to the
+     * host what features it support in addition.  In return the host will return
+     * features the host supports.  Two 64-bit parameters are passed in from the
+     * guest with the guest features (VBOX_GUESTCTRL_GF_XXX), the host replies by
+     * replacing the parameter values with the host ones (VBOX_GUESTCTRL_HF_XXX).
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_ACCESS_DENIED it not master.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0.10, 5.2.32
+     */
+    GUEST_MSG_REPORT_FEATURES,
+    /** Query the host ones feature masks.
+     *
+     * This is for the session sub-process so that it can get hold of the features
+     * from the host.  Again, it is prudent to set the 127 bit and observe it being
+     * cleared on success, as older hosts might return success without doing
+     * anything.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0.10, 5.2.32
+     */
+    GUEST_MSG_QUERY_FEATURES,
+
     /**
      * Guests sends output from an executed process.
      */
@@ -465,9 +383,12 @@ enum GUEST_FILE_NOTIFYTYPE
     GUEST_FILE_NOTIFYTYPE_OPEN = 10,
     GUEST_FILE_NOTIFYTYPE_CLOSE = 20,
     GUEST_FILE_NOTIFYTYPE_READ = 30,
+    GUEST_FILE_NOTIFYTYPE_READ_OFFSET,  /**< @since 6.0.10, 5.2.32 - VBOX_GUESTCTRL_HF_0_NOTIFY_RDWR_OFFSET */
     GUEST_FILE_NOTIFYTYPE_WRITE = 40,
+    GUEST_FILE_NOTIFYTYPE_WRITE_OFFSET, /**< @since 6.0.10, 5.2.32 - VBOX_GUESTCTRL_HF_0_NOTIFY_RDWR_OFFSET */
     GUEST_FILE_NOTIFYTYPE_SEEK = 50,
-    GUEST_FILE_NOTIFYTYPE_TELL = 60
+    GUEST_FILE_NOTIFYTYPE_TELL = 60,
+    GUEST_FILE_NOTIFYTYPE_SET_SIZE
 };
 
 /**
@@ -481,6 +402,25 @@ enum GUEST_FILE_SEEKTYPE
     GUEST_FILE_SEEKTYPE_END = 8
 };
 
+/** @name VBOX_GUESTCTRL_GF_XXX - Guest features.
+ * @sa GUEST_MSG_REPORT_FEATURES
+ * @{ */
+/** Supports HOST_MSG_FILE_SET_SIZE. */
+#define VBOX_GUESTCTRL_GF_0_SET_SIZE     RT_BIT_64(0)
+/** Bit that must be set in the 2nd parameter, will be cleared if the host reponds
+ * correctly (old hosts might not). */
+#define VBOX_GUESTCTRL_GF_1_MUST_BE_ONE RT_BIT_64(63)
+/** @} */
+
+/** @name VBOX_GUESTCTRL_HF_XXX - Host features.
+ * @sa GUEST_MSG_REPORT_FEATURES
+ * @{ */
+/** Host supports the GUEST_FILE_NOTIFYTYPE_READ_OFFSET and
+ *  GUEST_FILE_NOTIFYTYPE_WRITE_OFFSET notification types. */
+#define VBOX_GUESTCTRL_HF_0_NOTIFY_RDWR_OFFSET      RT_BIT_64(0)
+/** @} */
+
+
 /*
  * HGCM parameter structures.
  */
@@ -493,7 +433,7 @@ enum GUEST_FILE_SEEKTYPE
  */
 typedef struct HGCMMsgCmdWaitFor
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /**
      * The returned command the host wants to
      * run on the guest.
@@ -511,7 +451,7 @@ typedef struct HGCMMsgCmdWaitFor
  */
 typedef struct HGCMMsgCmdFilterSet
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Value to filter for after filter mask
      *  was applied. */
     HGCMFunctionParameter value;         /* IN uint32_t */
@@ -529,7 +469,7 @@ typedef struct HGCMMsgCmdFilterSet
  */
 typedef struct HGCMMsgCmdFilterUnset
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Unset flags; currently unused. */
     HGCMFunctionParameter flags;    /* IN uint32_t */
 } HGCMMsgCmdFilterUnset;
@@ -541,7 +481,7 @@ typedef struct HGCMMsgCmdFilterUnset
  */
 typedef struct HGCMMsgCmdSkip
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Skip flags; currently unused. */
     HGCMFunctionParameter flags;    /* IN uint32_t */
 } HGCMMsgCmdSkip;
@@ -552,12 +492,12 @@ typedef struct HGCMMsgCmdSkip
  */
 typedef struct HGCMMsgCancelPendingWaits
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
 } HGCMMsgCancelPendingWaits;
 
 typedef struct HGCMMsgCmdReply
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Message type. */
@@ -573,7 +513,7 @@ typedef struct HGCMMsgCmdReply
  */
 typedef struct HGCMMsgSessionOpen
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The guest control protocol version this
@@ -594,7 +534,7 @@ typedef struct HGCMMsgSessionOpen
  */
 typedef struct HGCMMsgSessionClose
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Session termination flags. */
@@ -606,7 +546,7 @@ typedef struct HGCMMsgSessionClose
  */
 typedef struct HGCMMsgSessionNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -617,7 +557,7 @@ typedef struct HGCMMsgSessionNotify
 
 typedef struct HGCMMsgPathRename
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** Source to rename. */
@@ -633,7 +573,7 @@ typedef struct HGCMMsgPathRename
  */
 typedef struct HGCMMsgProcExec
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The command to execute on the guest. */
@@ -688,7 +628,7 @@ typedef struct HGCMMsgProcExec
  */
 typedef struct HGCMMsgProcInput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID) to send the input to. */
@@ -707,7 +647,7 @@ typedef struct HGCMMsgProcInput
  */
 typedef struct HGCMMsgProcOutput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -725,7 +665,7 @@ typedef struct HGCMMsgProcOutput
  */
 typedef struct HGCMMsgProcStatus
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -743,7 +683,7 @@ typedef struct HGCMMsgProcStatus
  */
 typedef struct HGCMMsgProcStatusInput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -765,7 +705,7 @@ typedef struct HGCMMsgProcStatusInput
  */
 typedef struct HGCMMsgProcTerminate
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -777,7 +717,7 @@ typedef struct HGCMMsgProcTerminate
  */
 typedef struct HGCMMsgProcWaitFor
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -790,7 +730,7 @@ typedef struct HGCMMsgProcWaitFor
 
 typedef struct HGCMMsgDirRemove
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** Directory to remove. */
@@ -804,7 +744,7 @@ typedef struct HGCMMsgDirRemove
  */
 typedef struct HGCMMsgFileOpen
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** File to open. */
@@ -826,7 +766,7 @@ typedef struct HGCMMsgFileOpen
  */
 typedef struct HGCMMsgFileClose
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to close. */
@@ -838,7 +778,7 @@ typedef struct HGCMMsgFileClose
  */
 typedef struct HGCMMsgFileRead
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to read from. */
@@ -852,7 +792,7 @@ typedef struct HGCMMsgFileRead
  */
 typedef struct HGCMMsgFileReadAt
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to read from. */
@@ -868,7 +808,7 @@ typedef struct HGCMMsgFileReadAt
  */
 typedef struct HGCMMsgFileWrite
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to write to. */
@@ -884,7 +824,7 @@ typedef struct HGCMMsgFileWrite
  */
 typedef struct HGCMMsgFileWriteAt
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to write to. */
@@ -902,7 +842,7 @@ typedef struct HGCMMsgFileWriteAt
  */
 typedef struct HGCMMsgFileSeek
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to seek. */
@@ -918,13 +858,28 @@ typedef struct HGCMMsgFileSeek
  */
 typedef struct HGCMMsgFileTell
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to get the current position for. */
     HGCMFunctionParameter handle;
 } HGCMMsgFileTell;
 
+/**
+ * Changes the file size.
+ */
+typedef struct HGCMMsgFileSetSize
+{
+    VBGLIOCHGCMCALL         Hdr;
+    /** Context ID. */
+    HGCMFunctionParameter   id32Context;
+    /** File handle to seek. */
+    HGCMFunctionParameter   id32Handle;
+    /** The new file size. */
+    HGCMFunctionParameter   cb64NewSize;
+} HGCMMsgFileSetSize;
+
+
 /******************************************************************************
 * HGCM replies from the guest. These are handled in Main's low-level HGCM     *
 * callbacks and dispatched to the appropriate guest object.                   *
@@ -932,7 +887,7 @@ typedef struct HGCMMsgFileTell
 
 typedef struct HGCMReplyFileNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -953,11 +908,25 @@ typedef struct HGCMReplyFileNotify
             HGCMFunctionParameter data;
         } read;
         struct
+        {
+            /** Actual data read (if any). */
+            HGCMFunctionParameter pvData;
+            /** The new file offset (signed).  Negative value if non-seekable files. */
+            HGCMFunctionParameter off64New;
+        } ReadOffset;
+        struct
         {
             /** How much data (in bytes) have been successfully written. */
             HGCMFunctionParameter written;
         } write;
         struct
+        {
+            /** Number of bytes that was successfully written. */
+            HGCMFunctionParameter cb32Written;
+            /** The new file offset (signed).  Negative value if non-seekable files. */
+            HGCMFunctionParameter off64New;
+        } WriteOffset;
+        struct
         {
             HGCMFunctionParameter offset;
         } seek;
@@ -965,12 +934,16 @@ typedef struct HGCMReplyFileNotify
         {
             HGCMFunctionParameter offset;
         } tell;
+        struct
+        {
+            HGCMFunctionParameter cb64Size;
+        } SetSize;
     } u;
 } HGCMReplyFileNotify;
 
 typedef struct HGCMReplyDirNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -1179,6 +1152,11 @@ typedef struct CALLBACKDATA_FILE_NOTIFY
             /** New file offset after successful tell. */
             uint64_t uOffActual;
         } tell;
+        struct
+        {
+            /** The new file siz.e */
+            uint64_t cbSize;
+        } SetSize;
     } u;
 } CALLBACKDATA_FILE_NOTIFY, *PCALLBACKDATA_FILE_NOTIFY;
 
