@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -93,7 +93,7 @@ typedef uint32_t VRDEAUDIOFORMAT;
 typedef struct VRDEAUDIOINBEGIN
 {
     VRDEAUDIOFORMAT fmt; /* Actual format of data, which will be sent in VRDE_AUDIOIN_DATA events. */
-} VRDEAUDIOINBEGIN;
+} VRDEAUDIOINBEGIN, *PVRDEAUDIOINBEGIN;
 
 
 /*
@@ -632,10 +632,10 @@ typedef struct _VRDEUSBREQNEGOTIATERET_3
 
 
 /** Hints what has been intercepted by the application. */
-#define VRDE_CLIENT_INTERCEPT_AUDIO     (0x1)
-#define VRDE_CLIENT_INTERCEPT_USB       (0x2)
-#define VRDE_CLIENT_INTERCEPT_CLIPBOARD (0x4)
-#define VRDE_CLIENT_INTERCEPT_AUDIO_INPUT (0x8)
+#define VRDE_CLIENT_INTERCEPT_AUDIO       RT_BIT(0)
+#define VRDE_CLIENT_INTERCEPT_USB         RT_BIT(1)
+#define VRDE_CLIENT_INTERCEPT_CLIPBOARD   RT_BIT(2)
+#define VRDE_CLIENT_INTERCEPT_AUDIO_INPUT RT_BIT(3)
 
 
 /** The version of the VRDE server interface. */
@@ -1135,12 +1135,11 @@ typedef struct _VRDEENTRYPOINTS_3
                                         * which tells the server to bind to either of ports:
                                         * 3000, 3010, 3011, 3012, 4000.
                                         */
-#ifdef VBOX_WITH_VRDP_VIDEO_CHANNEL
 #define VRDE_QP_VIDEO_CHANNEL         (5)
 #define VRDE_QP_VIDEO_CHANNEL_QUALITY (6)
 #define VRDE_QP_VIDEO_CHANNEL_SUNFLSH (7)
-#endif /* VBOX_WITH_VRDP_VIDEO_CHANNEL */
 #define VRDE_QP_FEATURE           (8) /* VRDEFEATURE structure. Generic interface to query named VRDE properties. */
+#define VRDE_QP_UNIX_SOCKET_PATH  (9) /* Path to a UNIX Socket for incoming connections */
 
 #define VRDE_SP_BASE 0x1000
 #define VRDE_SP_NETWORK_BIND_PORT (VRDE_SP_BASE + 1) /* 32 bit. The port number actually used by the server.
