@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -100,7 +100,11 @@ typedef enum SUPPAGINGMODE
  * @{
  */
 /** GDT is read-only. */
-#define SUPKERNELFEATURES_GDT_READ_ONLY     RT_BIT(0)
+#define SUPKERNELFEATURES_GDT_READ_ONLY       RT_BIT(0)
+/** SMAP is possibly enabled. */
+#define SUPKERNELFEATURES_SMAP                RT_BIT(1)
+/** GDT is read-only but the writable GDT can be fetched by SUPR0GetCurrentGdtRw(). */
+#define SUPKERNELFEATURES_GDT_NEED_WRITABLE   RT_BIT(2)
 /** @} */
 
 
@@ -227,61 +231,85 @@ typedef struct SUPGIPCPU
      * thusly that odd numbers indicates update in progress, while even numbers
      * indicate stable data. Use this to make sure that the data items you fetch
      * are consistent. */
-    volatile uint32_t   u32TransactionId;
+    volatile uint32_t       u32TransactionId;
     /** The interval in TSC ticks between two NanoTS updates.
      * This is the average interval over the last 2, 4 or 8 updates + a little slack.
      * The slack makes the time go a tiny tiny bit slower and extends the interval enough
      * to avoid ending up with too many 1ns increments. */
-    volatile uint32_t   u32UpdateIntervalTSC;
+    volatile uint32_t       u32UpdateIntervalTSC;
     /** Current nanosecond timestamp. */
-    volatile uint64_t   u64NanoTS;
+    volatile uint64_t       u64NanoTS;
     /** The TSC at the time of u64NanoTS. */
-    volatile uint64_t   u64TSC;
+    volatile uint64_t       u64TSC;
     /** Current CPU Frequency. */
-    volatile uint64_t   u64CpuHz;
+    volatile uint64_t       u64CpuHz;
     /** The TSC delta with reference to the master TSC, subtract from RDTSC. */
-    volatile int64_t    i64TSCDelta;
+    volatile int64_t        i64TSCDelta;
     /** Number of errors during updating.
      * Typical errors are under/overflows. */
-    volatile uint32_t   cErrors;
+    volatile uint32_t       cErrors;
     /** Index of the head item in au32TSCHistory. */
-    volatile uint32_t   iTSCHistoryHead;
+    volatile uint32_t       iTSCHistoryHead;
     /** Array of recent TSC interval deltas.
      * The most recent item is at index iTSCHistoryHead.
      * This history is used to calculate u32UpdateIntervalTSC.
      */
-    volatile uint32_t   au32TSCHistory[8];
+    volatile uint32_t       au32TSCHistory[8];
     /** The interval between the last two NanoTS updates. (experiment for now) */
-    volatile uint32_t   u32PrevUpdateIntervalNS;
+    volatile uint32_t       u32PrevUpdateIntervalNS;
 
     /** Reserved for future per processor data. */
-    volatile uint32_t   au32Reserved0[5];
-
+    volatile uint32_t       u32Reserved;
     /** The TSC value read while doing TSC delta measurements across CPUs. */
-    volatile uint64_t   u64TSCSample;
-
+    volatile uint64_t       u64TSCSample;
     /** Reserved for future per processor data. */
-    volatile uint32_t   au32Reserved1[1];
+    volatile uint32_t       au32Reserved1[3];
 
-    /** @todo Add topology/NUMA info. */
     /** The CPU state. */
     SUPGIPCPUSTATE volatile enmState;
     /** The host CPU ID of this CPU (the SUPGIPCPU is indexed by APIC ID). */
     RTCPUID                 idCpu;
     /** The CPU set index of this CPU. */
     int16_t                 iCpuSet;
+    /** CPU group number (always zero, except on windows). */
+    uint16_t                iCpuGroup;
+    /** CPU group member number (same as iCpuSet, except on windows). */
+    uint16_t                iCpuGroupMember;
     /** The APIC ID of this CPU. */
     uint16_t                idApic;
+    /** @todo Add topology/NUMA info. */
+    uint32_t                iReservedForNumaNode;
 } SUPGIPCPU;
 AssertCompileSize(RTCPUID, 4);
 AssertCompileSize(SUPGIPCPU, 128);
 AssertCompileMemberAlignment(SUPGIPCPU, u64NanoTS, 8);
 AssertCompileMemberAlignment(SUPGIPCPU, u64TSC, 8);
+AssertCompileMemberAlignment(SUPGIPCPU, u64TSCSample, 8);
 
 /** Pointer to per cpu data.
  * @remark there is no const version of this typedef, see g_pSUPGlobalInfoPage for details. */
 typedef SUPGIPCPU *PSUPGIPCPU;
 
+/**
+ * CPU group information.
+ * @remarks Windows only.
+ */
+typedef struct SUPGIPCPUGROUP
+{
+    /** Current number of CPUs in this group. */
+    uint16_t volatile       cMembers;
+    /** Maximum number of CPUs in the group. */
+    uint16_t                cMaxMembers;
+    /** The CPU set index of the members. This table has cMaxMembers entries.
+     * @note For various reasons, entries from cMembers and up to cMaxMembers are
+     *       may change as the host OS does set dynamic assignments during CPU
+     *       hotplugging. */
+    int16_t                 aiCpuSetIdxs[1];
+} SUPGIPCPUGROUP;
+/** Pointer to a GIP CPU group structure. */
+typedef SUPGIPCPUGROUP *PSUPGIPCPUGROUP;
+/** Pointer to a const GIP CPU group structure. */
+typedef SUPGIPCPUGROUP const *PCSUPGIPCPUGROUP;
 
 /**
  * The rules concerning the applicability of SUPGIPCPU::i64TscDelta.
@@ -311,6 +339,12 @@ typedef enum SUPGIPUSETSCDELTA
 
 
 /** @name SUPGIPGETCPU_XXX - methods that aCPUs can be indexed.
+ *
+ * @note    Linux offers information via selector 0x78, and Windows via selector
+ *          0x53.  But since they both support RDTSCP as well, and because most
+ *          CPUs now have RDTSCP, we prefer it over LSL.  We can implement more
+ *          alternatives if it becomes necessary.
+ *
  * @{
  */
 /** Use ASMGetApicId (or equivalent) and translate the result via
@@ -332,10 +366,26 @@ typedef enum SUPGIPUSETSCDELTA
  * value in the IDT limit.  The masking is a precaution against what linux
  * does with RDTSCP. */
 #define SUPGIPGETCPU_IDTR_LIMIT_MASK_MAX_SET_CPUS   RT_BIT_32(2)
-/* Linux also offers information via selector 0x78, but we'll settle for
-   RDTSCP for now. */
+/** Windows specific RDTSCP variant, where CH gives you the group and CL gives
+ * you the CPU number within that group.
+ *
+ * Use SUPGLOBALINFOPAGE::aidFirstCpuFromCpuGroup to get the group base CPU set
+ * index, then translate the sum of thru aiCpuFromCpuSetIdx to find the aCPUs
+ * entry.
+ *
+ * @note The group number is actually 16-bit wide (ECX[23:8]), but we simplify
+ *       it since we only support 256 CPUs/groups at the moment.
+ */
+#define SUPGIPGETCPU_RDTSCP_GROUP_IN_CH_NUMBER_IN_CL RT_BIT_32(3)
 /** @} */
 
+/** @def SUPGIP_MAX_CPU_GROUPS
+ * Maximum number of CPU groups.  */
+#if RTCPUSET_MAX_CPUS >= 256
+# define SUPGIP_MAX_CPU_GROUPS 256
+#else
+# define SUPGIP_MAX_CPU_GROUPS RTCPUSET_MAX_CPUS
+#endif
 
 /**
  * Global Information Page.
@@ -378,7 +428,8 @@ typedef struct SUPGLOBALINFOPAGE
     volatile uint16_t   cPresentCpus;
     /** The highest number of CPUs possible. */
     uint16_t            cPossibleCpus;
-    uint16_t            u16Padding0;
+    /** The highest number of CPU groups possible. */
+    uint16_t            cPossibleCpuGroups;
     /** The max CPU ID (RTMpGetMaxCpuId). */
     RTCPUID             idCpuMax;
     /** The applicability of SUPGIPCPU::i64TscDelta. */
@@ -386,14 +437,21 @@ typedef struct SUPGLOBALINFOPAGE
     /** Mask of SUPGIPGETCPU_XXX values that indicates different ways that aCPU
      * can be accessed from ring-3 and raw-mode context. */
     uint32_t            fGetGipCpu;
+    /** GIP flags, see SUPGIP_FLAGS_XXX. */
+    volatile uint32_t   fFlags;
 
     /** Padding / reserved space for future data. */
-    uint32_t            au32Padding1[25];
+    uint32_t            au32Padding1[24];
 
     /** Table indexed by the CPU APIC ID to get the CPU table index. */
     uint16_t            aiCpuFromApicId[256];
     /** CPU set index to CPU table index. */
     uint16_t            aiCpuFromCpuSetIdx[RTCPUSET_MAX_CPUS];
+    /** Table indexed by CPU group to containing offsets to SUPGIPCPUGROUP
+     * structures, invalid entries are set to UINT16_MAX.  The offsets are relative
+     * to the start of this structure.
+     * @note Windows only. The other hosts sets all entries to UINT16_MAX! */
+    uint16_t            aoffCpuGroup[SUPGIP_MAX_CPU_GROUPS];
 
     /** Array of per-cpu data.
      * This is index by ApicId via the aiCpuFromApicId table.
@@ -410,6 +468,7 @@ AssertCompileMemberAlignment(SUPGLOBALINFOPAGE, aCPUs, 32);
 #else
 AssertCompileMemberAlignment(SUPGLOBALINFOPAGE, aCPUs, 256);
 #endif
+AssertCompile(sizeof(SUPGLOBALINFOPAGE) <= 0x1000); /* Keeping it less or equal to a page for raw-mode (saved state). */
 
 /** Pointer to the global info page.
  * @remark there is no const version of this typedef, see g_pSUPGlobalInfoPage for details. */
@@ -421,7 +480,7 @@ typedef SUPGLOBALINFOPAGE *PSUPGLOBALINFOPAGE;
 /** The GIP version.
  * Upper 16 bits is the major version. Major version is only changed with
  * incompatible changes in the GIP. */
-#define SUPGLOBALINFOPAGE_VERSION   0x00060000
+#define SUPGLOBALINFOPAGE_VERSION   0x00080000
 
 /**
  * SUPGLOBALINFOPAGE::u32Mode values.
@@ -488,9 +547,25 @@ extern DECLIMPORT(SUPGLOBALINFOPAGE)    g_SUPGlobalInfoPage;
  */
 SUPDECL(PSUPGLOBALINFOPAGE)             SUPGetGIP(void);
 
+/** @name SUPGIP_FLAGS_XXX - SUPR3GipSetFlags flags.
+ * @{ */
+/** Enable GIP test mode. */
+#define SUPGIP_FLAGS_TESTING_ENABLE          RT_BIT_32(0)
+/** Valid mask of flags that can be set through the ioctl. */
+#define SUPGIP_FLAGS_VALID_MASK              RT_BIT_32(0)
+/** GIP test mode needs to be checked (e.g. when enabled or being disabled). */
+#define SUPGIP_FLAGS_TESTING                 RT_BIT_32(24)
+/** Prepare to start GIP test mode. */
+#define SUPGIP_FLAGS_TESTING_START           RT_BIT_32(25)
+/** Prepare to stop GIP test mode. */
+#define SUPGIP_FLAGS_TESTING_STOP            RT_BIT_32(26)
+/** @} */
 
 /** @internal  */
 SUPDECL(uint64_t) SUPGetCpuHzFromGipForAsyncMode(PSUPGLOBALINFOPAGE pGip);
+SUPDECL(bool)     SUPIsTscFreqCompatible(uint64_t uCpuHz, uint64_t *puGipCpuHz, bool fRelax);
+SUPDECL(bool)     SUPIsTscFreqCompatibleEx(uint64_t uBaseCpuHz, uint64_t uCpuHz, bool fRelax);
+
 
 /**
  * Gets the TSC frequency of the calling CPU.
@@ -551,34 +626,28 @@ DECLINLINE(uint64_t) SUPGetCpuHzFromGipBySetIndex(PSUPGLOBALINFOPAGE pGip, uint3
 }
 
 
-#if 0 /* Not used anywhere.  Unsure where this would be useful. */
 /**
- * Checks if the provided TSC frequency is close enough to the computed TSC
- * frequency of the host.
+ * Gets the pointer to the per CPU data for a CPU given by its set index.
  *
- * @returns true if it's compatible, false otherwise.
+ * @returns Pointer to the corresponding per CPU structure, or NULL if invalid.
+ * @param   pGip        The GIP pointer.
+ * @param   iCpuSet     The CPU set index of the CPU which we want.
  */
-DECLINLINE(bool) SUPIsTscFreqCompatible(uint64_t u64CpuHz)
+DECLINLINE(PSUPGIPCPU) SUPGetGipCpuBySetIndex(PSUPGLOBALINFOPAGE pGip, uint32_t iCpuSet)
 {
-    PSUPGLOBALINFOPAGE pGip = g_pSUPGlobalInfoPage;
-    if (   pGip
-        && pGip->u32Mode == SUPGIPMODE_INVARIANT_TSC)
+    if (RT_LIKELY(   pGip
+                  && pGip->u32Magic == SUPGLOBALINFOPAGE_MAGIC))
     {
-        if (pGip->u64CpuHz != u64CpuHz)
+        if (RT_LIKELY(iCpuSet < RT_ELEMENTS(pGip->aiCpuFromCpuSetIdx)))
         {
-            /* Arbitrary tolerance threshold, tweak later if required, perhaps
-               more tolerance on lower frequencies and less tolerance on higher. */
-            uint64_t uLo = (pGip->u64CpuHz << 10) / 1025;
-            uint64_t uHi = pGip->u64CpuHz + (pGip->u64CpuHz - uLo);
-            if (   u64CpuHz < uLo
-                || u64CpuHz > uHi)
-                return false;
+            uint16_t iCpu = pGip->aiCpuFromCpuSetIdx[iCpuSet];
+            if (RT_LIKELY(iCpu < pGip->cCpus))
+                return &pGip->aCPUs[iCpu];
         }
-        return true;
     }
-    return false;
+    return NULL;
 }
-#endif
+
 
 #if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
 
@@ -1018,7 +1087,8 @@ typedef enum SUPINITOP
  * @param   pszMsgFmt       Error message format string.
  * @param   va              The message format arguments.
  */
-typedef DECLCALLBACK(void) FNSUPTRUSTEDERROR(const char *pszWhere, SUPINITOP enmWhat, int rc, const char *pszMsgFmt, va_list va);
+typedef DECLCALLBACK(void) FNSUPTRUSTEDERROR(const char *pszWhere, SUPINITOP enmWhat, int rc,
+                                             const char *pszMsgFmt, va_list va) RT_IPRT_FORMAT_ATTR(4, 0);
 /** Pointer to FNSUPTRUSTEDERROR. */
 typedef FNSUPTRUSTEDERROR *PFNSUPTRUSTEDERROR;
 
@@ -1050,16 +1120,23 @@ DECLHIDDEN(int) SUPR3HardenedMain(const char *pszProgName, uint32_t fFlags, int
 #define SUPSECMAIN_FLAGS_DONT_OPEN_DEV      RT_BIT_32(0)
 /** The hardened DLL has a "TrustedError" function (see FNSUPTRUSTEDERROR). */
 #define SUPSECMAIN_FLAGS_TRUSTED_ERROR      RT_BIT_32(1)
-/** Hack for making VirtualBoxVM use VirtualBox.dylib on Mac OS X. */
+/** Hack for making VirtualBoxVM use VirtualBox.dylib on Mac OS X.
+ * @note Not used since 6.0  */
 #define SUPSECMAIN_FLAGS_OSX_VM_APP         RT_BIT_32(2)
 /** Program binary location mask. */
-#define SUPSECMAIN_FLAGS_LOC_MASK           UINT32_C(0x00000010)
+#define SUPSECMAIN_FLAGS_LOC_MASK           UINT32_C(0x00000030)
 /** Default binary location is the application binary directory.  Does
  * not need to be given explicitly (it's 0).  */
 #define SUPSECMAIN_FLAGS_LOC_APP_BIN        UINT32_C(0x00000000)
 /** The binary is located in the testcase directory instead of the
  * default application binary directory. */
 #define SUPSECMAIN_FLAGS_LOC_TESTCASE       UINT32_C(0x00000010)
+/** The binary is located in a nested application bundle under Resources/ in the
+ * main Mac OS X application (think Resources/VirtualBoxVM.app).  */
+#define SUPSECMAIN_FLAGS_LOC_OSX_HLP_APP    UINT32_C(0x00000020)
+/** The first process.
+ * @internal  */
+#define SUPSECMAIN_FLAGS_FIRST_PROCESS      UINT32_C(0x00000100)
 /** @} */
 
 /**
@@ -1073,7 +1150,6 @@ DECLHIDDEN(int) SUPR3HardenedMain(const char *pszProgName, uint32_t fFlags, int
  */
 SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession);
 
-
 /**
  * Initializes the support library, extended version.
  *
@@ -1158,7 +1234,7 @@ SUPR3DECL(int) SUPR3CallVMMR0Ex(PVMR0 pVMR0, VMCPUID idCpu, unsigned uOperation,
  * @returns error code specific to uFunction.
  * @param   pszService  The service name.
  * @param   cchService  The length of the service name.
- * @param   uReq        The request number.
+ * @param   uOperation  The request number.
  * @param   u64Arg      Constant argument.
  * @param   pReqHdr     Pointer to a request header. Optional.
  *                      This will be copied in and out of kernel space. There currently is a size
@@ -1381,7 +1457,7 @@ SUPR3DECL(int) SUPR3LoadServiceModule(const char *pszFilename, const char *pszMo
  * Frees a R0 HC module.
  *
  * @returns VBox status code.
- * @param   pszModule       The module to free.
+ * @param   pvImageBase     The base address of the image to free.
  * @remark  This will not actually 'free' the module, there are of course usage counting.
  */
 SUPR3DECL(int) SUPR3FreeModule(void *pvImageBase);
@@ -1402,7 +1478,7 @@ SUPR3DECL(int) SUPR3LockDownLoader(PRTERRINFO pErrInfo);
  * Get the address of a symbol in a ring-0 module.
  *
  * @returns VBox status code.
- * @param   pszModule       The module name.
+ * @param   pvImageBase     The base address of the image to search.
  * @param   pszSymbol       Symbol name. If it's value is less than 64k it's treated like a
  *                          ordinal value rather than a string pointer.
  * @param   ppvValue        Where to store the symbol value.
@@ -1648,7 +1724,6 @@ SUPR3DECL(int) SUPR3TracerDeregisterModule(struct VTGOBJHDR *pVtgHdr);
 SUPDECL(void)  SUPTracerFireProbe(struct VTGPROBELOC *pVtgProbeLoc, uintptr_t uArg0, uintptr_t uArg1, uintptr_t uArg2,
                                   uintptr_t uArg3, uintptr_t uArg4);
 
-
 /**
  * Attempts to read the value of an MSR.
  *
@@ -1713,7 +1788,6 @@ SUPR3DECL(int) SUPR3MsrProberModifyEx(uint32_t uMsr, RTCPUID idCpu, uint64_t fAn
  */
 SUPR3DECL(int) SUPR3ResumeSuspendedKeyboards(void);
 
-
 /**
  * Measure the TSC-delta for the specified CPU.
  *
@@ -1740,6 +1814,23 @@ SUPR3DECL(int) SUPR3TscDeltaMeasure(RTCPUID idCpu, bool fAsync, bool fForce, uin
  */
 SUPR3DECL(int) SUPR3ReadTsc(uint64_t *puTsc, uint16_t *pidApic);
 
+/**
+ * Modifies the GIP flags.
+ *
+ * @returns VBox status code.
+ * @param   fOrMask         The OR mask of the GIP flags, see SUPGIP_FLAGS_XXX.
+ * @param   fAndMask        The AND mask of the GIP flags, see SUPGIP_FLAGS_XXX.
+ */
+SUPR3DECL(int) SUPR3GipSetFlags(uint32_t fOrMask, uint32_t fAndMask);
+
+/**
+ * Return processor microcode revision, if applicable.
+ *
+ * @returns VINF_SUCCESS if supported, error code indicating why if not.
+ * @param   puMicrocodeRev  Pointer to microcode revision dword (out).
+ */
+SUPR3DECL(int) SUPR3QueryMicrocodeRev(uint32_t *puMicrocodeRev);
+
 /** @} */
 #endif /* IN_RING3 */
 
@@ -1803,6 +1894,10 @@ SUPR0DECL(int) SUPR0ObjAddRefEx(void *pvObj, PSUPDRVSESSION pSession, bool fNoBl
 SUPR0DECL(int) SUPR0ObjRelease(void *pvObj, PSUPDRVSESSION pSession);
 SUPR0DECL(int) SUPR0ObjVerifyAccess(void *pvObj, PSUPDRVSESSION pSession, const char *pszObjName);
 
+SUPR0DECL(PVM) SUPR0GetSessionVM(PSUPDRVSESSION pSession);
+SUPR0DECL(PGVM) SUPR0GetSessionGVM(PSUPDRVSESSION pSession);
+SUPR0DECL(int) SUPR0SetSessionVM(PSUPDRVSESSION pSession, PGVM pGVM, PVM pVM);
+
 SUPR0DECL(int) SUPR0LockMem(PSUPDRVSESSION pSession, RTR3PTR pvR3, uint32_t cPages, PRTHCPHYS paPages);
 SUPR0DECL(int) SUPR0UnlockMem(PSUPDRVSESSION pSession, RTR3PTR pvR3);
 SUPR0DECL(int) SUPR0ContAlloc(PSUPDRVSESSION pSession, uint32_t cPages, PRTR0PTR ppvR0, PRTR3PTR ppvR3, PRTHCPHYS pHCPhys);
@@ -1819,11 +1914,12 @@ SUPR0DECL(int) SUPR0PageFree(PSUPDRVSESSION pSession, RTR3PTR pvR3);
 SUPR0DECL(int) SUPR0GipMap(PSUPDRVSESSION pSession, PRTR3PTR ppGipR3, PRTHCPHYS pHCPhysGip);
 SUPR0DECL(int) SUPR0GetSvmUsability(bool fInitSvm);
 SUPR0DECL(int) SUPR0GetVmxUsability(bool *pfIsSmxModeAmbiguous);
+SUPR0DECL(int) SUPR0GetRawModeUsability(void);
+SUPR0DECL(int) SUPR0GetCurrentGdtRw(RTHCUINTPTR *pGdtRw);
 SUPR0DECL(int) SUPR0QueryVTCaps(PSUPDRVSESSION pSession, uint32_t *pfCaps);
 SUPR0DECL(int) SUPR0GipUnmap(PSUPDRVSESSION pSession);
-SUPR0DECL(int) SUPR0Printf(const char *pszFormat, ...);
+SUPR0DECL(int) SUPR0QueryUcodeRev(PSUPDRVSESSION pSession, uint32_t *puMicrocodeRev);
 SUPR0DECL(SUPPAGINGMODE) SUPR0GetPagingMode(void);
-SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void);
 SUPR0DECL(RTCCUINTREG) SUPR0ChangeCR4(RTCCUINTREG fOrMask, RTCCUINTREG fAndMask);
 SUPR0DECL(int) SUPR0EnableVTx(bool fEnable);
 SUPR0DECL(bool) SUPR0SuspendVTxOnCpu(void);
@@ -1834,6 +1930,28 @@ SUPR0DECL(void) SUPR0ResumeVTxOnCpu(bool fSuspended);
 SUPR0DECL(int) SUPR0TscDeltaMeasureBySetIndex(PSUPDRVSESSION pSession, uint32_t iCpuSet, uint32_t fFlags,
                                               RTMSINTERVAL cMsWaitRetry, RTMSINTERVAL cMsWaitThread, uint32_t cTries);
 
+SUPR0DECL(void) SUPR0BadContext(PSUPDRVSESSION pSession, const char *pszFile, uint32_t uLine, const char *pszExpr);
+
+/**
+ * Writes to the debugger and/or kernel log.
+ *
+ * The length of the formatted message is somewhat limited, so keep things short
+ * and to the point.
+ *
+ * @returns Number of bytes written, mabye.
+ * @param   pszFormat       IPRT format string.
+ * @param   ...             Arguments referenced by the format string.
+ */
+SUPR0DECL(int)  SUPR0Printf(const char *pszFormat, ...) RT_IPRT_FORMAT_ATTR(1, 2);
+
+/**
+ * Returns configuration flags of the host kernel.
+ *
+ * @returns Combination of SUPKERNELFEATURES_XXX flags.
+ */
+SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void);
+
+
 /** @name Absolute symbols
  * Take the address of these, don't try call them.
  * @{ */
@@ -1964,7 +2082,7 @@ typedef struct SUPDRVVDTPROVIDERCORE
     uint8_t                     cbProbeLocsEn;
     /** The actual module bit count (corresponds to cbProbeLocsEn). */
     uint8_t                     cBits;
-    /** Set if this is a Umod, otherwise clear.. */
+    /** Set if this is a Umod, otherwise clear. */
     bool                        fUmod;
     /** Explicit alignment padding (paranoia). */
     uint8_t                     abAlignment[ARCH_BITS == 32 ? 1 : 5];
@@ -2149,6 +2267,7 @@ SUPR0DECL(void) SUPR0TracerUmodProbeFire(PSUPDRVSESSION pSession, PSUPDRVTRACERU
  *
  * @returns VBox status code.
  * @param   pSession    The caller's session.
+ * @param   uOperation  The operation identifier.
  * @param   u64Arg      64-bit integer argument.
  * @param   pReqHdr     The request header. Input / Output. Optional.
  */
@@ -2246,6 +2365,11 @@ extern SUPTAENTRY const             g_aSUPTimestampTAs[];
 /** Number of entries in g_aSUPTimestampTAs. */
 extern unsigned const               g_cSUPTimestampTAs;
 
+/** Root certificates trusted by Apple code signing. */
+extern SUPTAENTRY const             g_aSUPAppleRootTAs[];
+/** Number of entries in g_cSUPAppleRootTAs. */
+extern unsigned const               g_cSUPAppleRootTAs;
+
 /** TAs we trust (the build certificate, Oracle VirtualBox). */
 extern SUPTAENTRY const             g_aSUPTrustedTAs[];
 /** Number of entries in g_aSUPTrustedTAs. */
