@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -29,6 +29,7 @@
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
 #include <iprt/fs.h>
+#include <iprt/symlink.h>
 
 
 RT_C_DECLS_BEGIN
@@ -58,9 +59,11 @@ RTDECL(bool) RTDirExists(const char *pszPath);
 /** Set the not-content-indexed flag (default).  Windows only atm. */
 #define RTDIRCREATE_FLAGS_NOT_CONTENT_INDEXED_DONT_SET      RT_BIT(1)
 /** Do not set the not-content-indexed flag.  Windows only atm. */
-#define RTDIRCREATE_FLAGS_NOT_CONTENT_INDEXED_DONT_SET      RT_BIT(1)
+#define RTDIRCREATE_FLAGS_NOT_CONTENT_INDEXED_SET           UINT32_C(0)
 /** Ignore errors setting the not-content-indexed flag.  Windows only atm. */
 #define RTDIRCREATE_FLAGS_NOT_CONTENT_INDEXED_NOT_CRITICAL  RT_BIT(2)
+/** Valid mask. */
+#define RTDIRCREATE_FLAGS_VALID_MASK                        UINT32_C(0x00000007)
 /** @} */
 
 /**
@@ -129,22 +132,30 @@ RTDECL(int) RTDirCreateTempSecure(char *pszTemplate);
 /**
  * Creates a new directory with a unique name by appending a number.
  *
- * First it is tried to create the directory without any numbers appended.
- * When this fails a number string is appended (starting with 1) separated by
- * the optional separator. The numbers are zero padded.
+ * This API differs from RTDirCreateTemp & RTDirCreateTempSecure in that it
+ * first tries to create the directory without any random bits, thus the best
+ * case result will be prettier.  It also differs in that it does not take a
+ * template, but is instead given a template description, and will only use
+ * digits for the filling.
+ *
+ * For sake of convenience and debugging , the current implementation
+ * starts at 0 and will increment sequentally for a while before switching to
+ * random numbers.
  *
  * On success @a pszPath contains the path created.
  *
  * @returns iprt status code.
- * @param   pszPath     Path to the directory to create.
- * @param   cbSize      The size of pszPath. Needs enough space for holding the
- *                      digits and the optional separator.
+ * @param   pszPath     The path to the directory.  On input the base template
+ *                      name.  On successful return, the unique directory we
+ *                      created.
+ * @param   cbSize      The size of the pszPath buffer.  Needs enough space for
+ *                      holding the digits and the optional separator.
  * @param   fMode       The mode of the new directory.
- * @param   cchDigits   How many digits should the number maximal have.
+ * @param   cchDigits   How many digits should the number have (zero padded).
  * @param   chSep       The separator used between the path and the number. Can
  *                      be zero. (optional)
  */
-RTDECL(int) RTDirCreateUniqueNumbered(char *pszPath, size_t cbSize, RTFMODE fMode, signed int cchDigits, char chSep);
+RTDECL(int) RTDirCreateUniqueNumbered(char *pszPath, size_t cbSize, RTFMODE fMode, size_t cchDigits, char chSep);
 
 /**
  * Removes a directory if empty.
@@ -201,9 +212,6 @@ RTDECL(int) RTDirFlush(const char *pszPath);
 RTDECL(int) RTDirFlushParent(const char *pszChild);
 
 
-/** Pointer to an open directory (sort of handle). */
-typedef struct RTDIR *PRTDIR;
-
 
 /**
  * Filter option for RTDirOpenFiltered().
@@ -235,7 +243,7 @@ typedef enum RTDIRFILTER
  * Directory entry type.
  *
  * This is the RTFS_TYPE_MASK stuff shifted down 12 bits and
- * identical to the BSD/LINUX ABI.
+ * identical to the BSD/LINUX ABI.  See RTFS_TYPE_DIRENTRYTYPE_SHIFT.
  */
 typedef enum RTDIRENTRYTYPE
 {
@@ -275,11 +283,13 @@ typedef struct RTDIRENTRY
      * available. */
     RTINODE         INodeId;
     /** The entry type. (d_type)
-     * RTDIRENTRYTYPE_UNKNOWN is a common return value here since not all file
-     * systems (or Unixes) stores the type of a directory entry and instead
-     * expects the user to use stat() to get it.  So, when you see this you
-     * should use RTDirQueryUnknownType or RTDirQueryUnknownTypeEx to get the type,
-     * or if if you're lazy, use RTDirReadEx. */
+     *
+     * @warning RTDIRENTRYTYPE_UNKNOWN is a common return value here since not all
+     *          file systems (or Unixes) stores the type of a directory entry and
+     *          instead expects the user to use stat() to get it.  So, when you see
+     *          this you should use RTDirQueryUnknownType or RTDirQueryUnknownTypeEx
+     *          to get the type, or if if you're lazy, use RTDirReadEx.
+     */
     RTDIRENTRYTYPE  enmType;
     /** The length of the filename, excluding the terminating nul character. */
     uint16_t        cbName;
@@ -329,37 +339,51 @@ typedef RTDIRENTRYEX const *PCRTDIRENTRYEX;
  * Opens a directory.
  *
  * @returns iprt status code.
- * @param   ppDir       Where to store the open directory pointer.
+ * @param   phDir       Where to store the open directory handle.
  * @param   pszPath     Path to the directory to open.
  */
-RTDECL(int) RTDirOpen(PRTDIR *ppDir, const char *pszPath);
+RTDECL(int) RTDirOpen(RTDIR *phDir, const char *pszPath);
 
-/** @name RTDirOpenFiltered  flags.
+/** @name RTDIR_F_XXX - RTDirOpenFiltered flags.
  * @{ */
 /** Don't allow symbolic links as part of the path.
  * @remarks this flag is currently not implemented and will be ignored. */
-#define RTDIROPEN_FLAGS_NO_SYMLINKS  RT_BIT(0)
+#define RTDIR_F_NO_SYMLINKS     RT_BIT_32(0)
+/** Deny relative opening of anything above this directory. */
+#define RTDIR_F_DENY_ASCENT     RT_BIT_32(1)
+/** Valid flag mask.   */
+#define RTDIR_F_VALID_MASK      UINT32_C(0x00000003)
 /** @} */
 
 /**
- * Opens a directory filtering the entries using dos style wildcards.
+ * Opens a directory with flags and optional filtering.
  *
  * @returns iprt status code.
- * @param   ppDir       Where to store the open directory pointer.
+ * @param   phDir       Where to store the open directory handle.
  * @param   pszPath     Path to the directory to search, this must include wildcards.
  * @param   enmFilter   The kind of filter to apply. Setting this to RTDIRFILTER_NONE makes
  *                      this function behave like RTDirOpen.
- * @param   fOpen       Open flags, RTDIROPENFILTERED_FLAGS_*.
+ * @param   fFlags      Open flags, RTDIR_F_XXX.
  */
-RTDECL(int) RTDirOpenFiltered(PRTDIR *ppDir, const char *pszPath, RTDIRFILTER enmFilter, uint32_t fOpen);
+RTDECL(int) RTDirOpenFiltered(RTDIR *phDir, const char *pszPath, RTDIRFILTER enmFilter, uint32_t fFlags);
 
 /**
  * Closes a directory.
  *
  * @returns iprt status code.
- * @param   pDir        Pointer to open directory returned by RTDirOpen() or RTDirOpenFiltered().
+ * @param   hDir        Handle to open directory returned by RTDirOpen() or
+ *                      RTDirOpenFiltered().
  */
-RTDECL(int) RTDirClose(PRTDIR pDir);
+RTDECL(int) RTDirClose(RTDIR hDir);
+
+/**
+ * Checks if the supplied directory handle is valid.
+ *
+ * @returns true if valid.
+ * @returns false if invalid.
+ * @param   hDir        The directory handle.
+ */
+RTDECL(bool) RTDirIsValid(RTDIR hDir);
 
 /**
  * Reads the next entry in the directory.
@@ -370,7 +394,7 @@ RTDECL(int) RTDirClose(PRTDIR pDir);
  *          pcbDirEntry is specified it will be updated with the required buffer size.
  * @returns suitable iprt status code on other errors.
  *
- * @param   pDir        Pointer to the open directory.
+ * @param   hDir        Handle to the open directory.
  * @param   pDirEntry   Where to store the information about the next
  *                      directory entry on success.
  * @param   pcbDirEntry Optional parameter used for variable buffer size.
@@ -386,7 +410,7 @@ RTDECL(int) RTDirClose(PRTDIR pDir);
  *
  *                      The value is unchanged in all other cases.
  */
-RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry);
+RTDECL(int) RTDirRead(RTDIR hDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry);
 
 /**
  * Reads the next entry in the directory returning extended information.
@@ -397,7 +421,7 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry);
  *          pcbDirEntry is specified it will be updated with the required buffer size.
  * @returns suitable iprt status code on other errors.
  *
- * @param   pDir        Pointer to the open directory.
+ * @param   hDir        Handle to the open directory.
  * @param   pDirEntry   Where to store the information about the next
  *                      directory entry on success.
  * @param   pcbDirEntry Optional parameter used for variable buffer size.
@@ -417,7 +441,32 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry);
  *                      Use RTFSOBJATTRADD_NOTHING if this doesn't matter.
  * @param   fFlags      RTPATH_F_ON_LINK or RTPATH_F_FOLLOW_LINK.
  */
-RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntry, RTFSOBJATTRADD enmAdditionalAttribs, uint32_t fFlags);
+RTDECL(int) RTDirReadEx(RTDIR hDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntry, RTFSOBJATTRADD enmAdditionalAttribs, uint32_t fFlags);
+
+/**
+ * Wrapper around RTDirReadEx that does the directory entry buffer handling.
+ *
+ * Call RTDirReadExAFree to free the buffers allocated by this function.
+ *
+ * @returns IPRT status code, see RTDirReadEx() for details.
+ *
+ * @param   hDir        Handle to the open directory.
+ * @param   ppDirEntry  Pointer to the directory entry pointer.  Initialize this
+ *                      to NULL before the first call.
+ * @param   pcbDirEntry Where the API caches the allocation size.  Set this to
+ *                      zero before the first call.
+ * @param   enmAddAttr  See RTDirReadEx.
+ * @param   fFlags      See RTDirReadEx.
+ */
+RTDECL(int) RTDirReadExA(RTDIR hDir, PRTDIRENTRYEX *ppDirEntry, size_t *pcbDirEntry, RTFSOBJATTRADD enmAddAttr, uint32_t fFlags);
+
+/**
+ * Frees the buffer allocated by RTDirReadExA.
+ *
+ * @param   ppDirEntry  Pointer to the directory entry pointer.
+ * @param   pcbDirEntry Where the API caches the allocation size.
+ */
+RTDECL(void) RTDirReadExAFree(PRTDIRENTRYEX *ppDirEntry, size_t *pcbDirEntry);
 
 /**
  * Resolves RTDIRENTRYTYPE_UNKNOWN values returned by RTDirRead.
@@ -490,12 +539,12 @@ RTDECL(int) RTDirRename(const char *pszSrc, const char *pszDst, unsigned fRename
  *
  * @returns iprt status code.
  *
- * @param   pDir        Pointer to the open directory.
+ * @param   hDir                    Handle to the open directory.
  * @param   pObjInfo                Object information structure to be filled on successful return.
  * @param   enmAdditionalAttribs    Which set of additional attributes to request.
  *                                  Use RTFSOBJATTRADD_NOTHING if this doesn't matter.
  */
-RTR3DECL(int) RTDirQueryInfo(PRTDIR pDir, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs);
+RTR3DECL(int) RTDirQueryInfo(RTDIR hDir, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs);
 
 
 /**
@@ -504,7 +553,7 @@ RTR3DECL(int) RTDirQueryInfo(PRTDIR pDir, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD
  * @returns iprt status code.
  * @returns VERR_NOT_SUPPORTED is returned if the operation isn't supported by the OS.
  *
- * @param   pDir        Pointer to the open directory.
+ * @param   hDir                Handle to the open directory.
  * @param   pAccessTime         Pointer to the new access time. NULL if not to be changed.
  * @param   pModificationTime   Pointer to the new modifcation time. NULL if not to be changed.
  * @param   pChangeTime         Pointer to the new change time. NULL if not to be changed.
@@ -521,9 +570,258 @@ RTR3DECL(int) RTDirQueryInfo(PRTDIR pDir, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD
  *
  * @remark  POSIX can only set Access & Modification and will always set both.
  */
-RTR3DECL(int) RTDirSetTimes(PRTDIR pDir, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
+RTR3DECL(int) RTDirSetTimes(RTDIR hDir, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
                             PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime);
 
+
+/** @defgroup grp_rt_dir_rel    Directory relative APIs
+ *
+ * This group of APIs allows working with paths that are relative to an open
+ * directory, therebye eliminating some classic path related race conditions on
+ * systems with native support for these kinds of operations.
+ *
+ * On NT (Windows) there is native support for addressing files, directories and
+ * stuff _below_ the open directory.  It is not possible to go upwards
+ * (hDir:../../grandparent), at least not with NTFS, forcing us to use the
+ * directory path as a fallback and opening us to potential races.
+ *
+ * On most unix-like systems here is now native support for all of this.
+ *
+ * @{ */
+
+/**
+ * Open a file relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory to open relative to.
+ * @param   pszRelFilename  The relative path to the file.
+ * @param   fOpen           Open flags, i.e a combination of the RTFILE_O_XXX
+ *                          defines.  The ACCESS, ACTION and DENY flags are
+ *                          mandatory!
+ * @param   phFile          Where to store the handle to the opened file.
+ *
+ * @sa      RTFileOpen
+ */
+RTDECL(int)  RTDirRelFileOpen(RTDIR hDir, const char *pszRelFilename, uint64_t fOpen, PRTFILE phFile);
+
+
+
+/**
+ * Opens a directory relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory to open relative to.
+ * @param   pszDir          The relative path to the directory to open.
+ * @param   phDir           Where to store the directory handle.
+ *
+ * @sa      RTDirOpen
+ */
+RTDECL(int) RTDirRelDirOpen(RTDIR hDir, const char *pszDir, RTDIR *phDir);
+
+/**
+ * Opens a directory relative to @a hDir, with flags and optional filtering.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory to open relative to.
+ * @param   pszDirAndFilter The relative path to the directory to search, this
+ *                          must include wildcards.
+ * @param   enmFilter       The kind of filter to apply. Setting this to
+ *                          RTDIRFILTER_NONE makes this function behave like
+ *                          RTDirOpen.
+ * @param   fFlags          Open flags, RTDIR_F_XXX.
+ * @param   phDir           Where to store the directory handle.
+ *
+ * @sa      RTDirOpenFiltered
+ */
+RTDECL(int) RTDirRelDirOpenFiltered(RTDIR hDir, const char *pszDirAndFilter, RTDIRFILTER enmFilter,
+                                    uint32_t fFlags, RTDIR *phDir);
+
+/**
+ * Creates a directory relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory @a pszRelPath is relative to.
+ * @param   pszRelPath      The relative path to the directory to create.
+ * @param   fMode           The mode of the new directory.
+ * @param   fCreate         Create flags, RTDIRCREATE_FLAGS_XXX.
+ * @param   phSubDir        Where to return the handle of the created directory.
+ *                          Optional.
+ *
+ * @sa      RTDirCreate
+ */
+RTDECL(int) RTDirRelDirCreate(RTDIR hDir, const char *pszRelPath, RTFMODE fMode, uint32_t fCreate, RTDIR *phSubDir);
+
+/**
+ * Removes a directory relative to @a hDir if empty.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory @a pszRelPath is relative to.
+ * @param   pszRelPath      The relative path to the directory to remove.
+ *
+ * @sa      RTDirRemove
+ */
+RTDECL(int) RTDirRelDirRemove(RTDIR hDir, const char *pszRelPath);
+
+
+/**
+ * Query information about a file system object relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @retval  VINF_SUCCESS if the object exists, information returned.
+ * @retval  VERR_PATH_NOT_FOUND if any but the last component in the specified
+ *          path was not found or was not a directory.
+ * @retval  VERR_FILE_NOT_FOUND if the object does not exist (but path to the
+ *          parent directory exists).
+ *
+ * @param   hDir            The directory @a pszRelPath is relative to.
+ * @param   pszRelPath      The relative path to the file system object.
+ * @param   pObjInfo        Object information structure to be filled on successful
+ *                          return.
+ * @param   enmAddAttr      Which set of additional attributes to request.
+ *                          Use RTFSOBJATTRADD_NOTHING if this doesn't matter.
+ * @param   fFlags          RTPATH_F_ON_LINK or RTPATH_F_FOLLOW_LINK.
+ *
+ * @sa      RTPathQueryInfoEx
+ */
+RTDECL(int) RTDirRelPathQueryInfo(RTDIR hDir, const char *pszRelPath, PRTFSOBJINFO pObjInfo,
+                                  RTFSOBJATTRADD enmAddAttr, uint32_t fFlags);
+
+/**
+ * Changes the mode flags of a file system object relative to @a hDir.
+ *
+ * The API requires at least one of the mode flag sets (Unix/Dos) to
+ * be set. The type is ignored.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory @a pszRelPath is relative to.
+ * @param   pszRelPath      The relative path to the file system object.
+ * @param   fMode           The new file mode, see @ref grp_rt_fs for details.
+ * @param   fFlags          RTPATH_F_ON_LINK or RTPATH_F_FOLLOW_LINK.
+ *
+ * @sa      RTPathSetMode
+ */
+RTDECL(int) RTDirRelPathSetMode(RTDIR hDir, const char *pszRelPath, RTFMODE fMode, uint32_t fFlags);
+
+/**
+ * Changes one or more of the timestamps associated of file system object
+ * relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @param   hDir                The directory @a pszRelPath is relative to.
+ * @param   pszRelPath          The relative path to the file system object.
+ * @param   pAccessTime         Pointer to the new access time.
+ * @param   pModificationTime   Pointer to the new modification time.
+ * @param   pChangeTime         Pointer to the new change time. NULL if not to be changed.
+ * @param   pBirthTime          Pointer to the new time of birth. NULL if not to be changed.
+ * @param   fFlags              RTPATH_F_ON_LINK or RTPATH_F_FOLLOW_LINK.
+ *
+ * @remark  The file system might not implement all these time attributes,
+ *          the API will ignore the ones which aren't supported.
+ *
+ * @remark  The file system might not implement the time resolution
+ *          employed by this interface, the time will be chopped to fit.
+ *
+ * @remark  The file system may update the change time even if it's
+ *          not specified.
+ *
+ * @remark  POSIX can only set Access & Modification and will always set both.
+ *
+ * @sa      RTPathSetTimesEx
+ */
+RTDECL(int) RTDirRelPathSetTimes(RTDIR hDir, const char *pszRelPath, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
+                                 PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime, uint32_t fFlags);
+
+/**
+ * Changes the owner and/or group of a file system object relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory @a pszRelPath is relative to.
+ * @param   pszRelPath      The relative path to the file system object.
+ * @param   uid             The new file owner user id.  Pass NIL_RTUID to leave
+ *                          this unchanged.
+ * @param   gid             The new group id.  Pass NIL_RTGID to leave this
+ *                          unchanged.
+ * @param   fFlags          RTPATH_F_ON_LINK or RTPATH_F_FOLLOW_LINK.
+ *
+ * @sa      RTPathSetOwnerEx
+ */
+RTDECL(int) RTDirRelPathSetOwner(RTDIR hDir, const char *pszRelPath, uint32_t uid, uint32_t gid, uint32_t fFlags);
+
+/**
+ * Renames a directory relative path within a filesystem.
+ *
+ * This will rename symbolic links.  If RTPATHRENAME_FLAGS_REPLACE is used and
+ * pszDst is a symbolic link, it will be replaced and not its target.
+ *
+ * @returns IPRT status code.
+ * @param   hDirSrc         The directory the source path is relative to.
+ * @param   pszSrc          The source path, relative to @a hDirSrc.
+ * @param   hDirDst         The directory the destination path is relative to.
+ * @param   pszDst          The destination path, relative to @a hDirDst.
+ * @param   fRename         Rename flags, RTPATHRENAME_FLAGS_XXX.
+ *
+ * @sa      RTPathRename
+ */
+RTDECL(int) RTDirRelPathRename(RTDIR hDirSrc, const char *pszSrc, RTDIR hDirDst, const char *pszDst, unsigned fRename);
+
+/**
+ * Removes the last component of the directory relative path.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory @a pszRelPath is relative to.
+ * @param   pszRelPath      The relative path to the file system object.
+ * @param   fUnlink         Unlink flags, RTPATHUNLINK_FLAGS_XXX.
+ *
+ * @sa      RTPathUnlink
+ */
+RTDECL(int) RTDirRelPathUnlink(RTDIR hDir, const char *pszRelPath, uint32_t fUnlink);
+
+
+
+/**
+ * Creates a symbolic link (@a pszSymlink) relative to @a hDir targeting @a
+ * pszTarget.
+ *
+ * @returns IPRT status code.
+ * @param   hDir            The directory @a pszSymlink is relative to.
+ * @param   pszSymlink      The relative path of the symbolic link.
+ * @param   pszTarget       The path to the symbolic link target.  This is
+ *                          relative to @a pszSymlink or an absolute path.
+ * @param   enmType         The symbolic link type.  For Windows compatability
+ *                          it is very important to set this correctly.  When
+ *                          RTSYMLINKTYPE_UNKNOWN is used, the API will try
+ *                          make a guess and may attempt query information
+ *                          about @a pszTarget in the process.
+ * @param   fCreate         Create flags, RTSYMLINKCREATE_FLAGS_XXX.
+ *
+ * @sa      RTSymlinkCreate
+ */
+RTDECL(int) RTDirRelSymlinkCreate(RTDIR hDir, const char *pszSymlink, const char *pszTarget,
+                                  RTSYMLINKTYPE enmType, uint32_t fCreate);
+
+/**
+ * Read the symlink target relative to @a hDir.
+ *
+ * @returns IPRT status code.
+ * @retval  VERR_NOT_SYMLINK if @a pszSymlink does not specify a symbolic link.
+ * @retval  VERR_BUFFER_OVERFLOW if the link is larger than @a cbTarget.  The
+ *          buffer will contain what all we managed to read, fully terminated
+ *          if @a cbTarget > 0.
+ *
+ * @param   hDir            The directory @a pszSymlink is relative to.
+ * @param   pszSymlink      The relative path to the symbolic link that should
+ *                          be read.
+ * @param   pszTarget       The target buffer.
+ * @param   cbTarget        The size of the target buffer.
+ * @param   fRead           Read flags, RTSYMLINKREAD_FLAGS_XXX.
+ *
+ * @sa      RTSymlinkRead
+ */
+RTDECL(int) RTDirRelSymlinkRead(RTDIR hDir, const char *pszSymlink, char *pszTarget, size_t cbTarget, uint32_t fRead);
+
+/** @} */
+
+
 /** @} */
 
 RT_C_DECLS_END
