@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -173,7 +173,7 @@ typedef struct RTLDRREADER
      * @returns size of raw image bits in bytes.
      * @param   pReader     Pointer to the reader instance.
      */
-    DECLCALLBACKMEMBER(RTFOFF, pfnSize)(PRTLDRREADER pReader);
+    DECLCALLBACKMEMBER(uint64_t, pfnSize)(PRTLDRREADER pReader);
 
     /**
      * Map the bits into memory.
@@ -278,14 +278,18 @@ RTDECL(int) RTLdrLoadEx(const char *pszFilename, PRTLDRMOD phLdrMod, uint32_t fF
  * @{ */
 /** Symbols defined in this library are not made available to resolve
  * references in subsequently loaded libraries (default). */
-#define RTLDRLOAD_FLAGS_LOCAL       UINT32_C(0)
+#define RTLDRLOAD_FLAGS_LOCAL                   UINT32_C(0)
 /** Symbols defined in this library will be made available for symbol
  * resolution of subsequently loaded libraries. */
-#define RTLDRLOAD_FLAGS_GLOBAL      RT_BIT_32(0)
+#define RTLDRLOAD_FLAGS_GLOBAL                  RT_BIT_32(0)
 /** Do not unload the library upon RTLdrClose. (For system libs.) */
-#define RTLDRLOAD_FLAGS_NO_UNLOAD   RT_BIT_32(1)
+#define RTLDRLOAD_FLAGS_NO_UNLOAD               RT_BIT_32(1)
+/** Windows/NT: Search the DLL load directory for imported DLLs - W7,
+ *  Vista, and W2K8 requires KB2533623 to be installed to support this; not
+ *  supported on XP, W2K3 or earlier.  Ignored on other platforms. */
+#define RTLDRLOAD_FLAGS_NT_SEARCH_DLL_LOAD_DIR  RT_BIT_32(2)
 /** The mask of valid flag bits. */
-#define RTLDRLOAD_FLAGS_VALID_MASK  UINT32_C(0x00000003)
+#define RTLDRLOAD_FLAGS_VALID_MASK              UINT32_C(0x00000007)
 /** @} */
 
 /**
@@ -359,15 +363,19 @@ typedef RTLDRARCH *PRTLDRARCH;
  * @{ */
 /** Open for debugging or introspection reasons.
  * This will skip a few of the stricter validations when loading images. */
-#define RTLDR_O_FOR_DEBUG       RT_BIT_32(0)
+#define RTLDR_O_FOR_DEBUG                   RT_BIT_32(0)
 /** Open for signature validation. */
-#define RTLDR_O_FOR_VALIDATION  RT_BIT_32(1)
+#define RTLDR_O_FOR_VALIDATION              RT_BIT_32(1)
+/** The arch specification is just a guideline for FAT binaries. */
+#define RTLDR_O_WHATEVER_ARCH               RT_BIT_32(2)
+/** Ignore the architecture specification if there is no code. */
+#define RTLDR_O_IGNORE_ARCH_IF_NO_CODE      RT_BIT_32(3)
 /** Mask of valid flags. */
-#define RTLDR_O_VALID_MASK      UINT32_C(0x00000003)
+#define RTLDR_O_VALID_MASK                  UINT32_C(0x0000000f)
 /** @} */
 
 /**
- * Open a binary image file, extended version.
+ * Open a binary image file.
  *
  * @returns iprt status code.
  * @param   pszFilename Image filename.
@@ -378,16 +386,16 @@ typedef RTLDRARCH *PRTLDRARCH;
 RTDECL(int) RTLdrOpen(const char *pszFilename, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod);
 
 /**
- * Opens a binary image file using kLdr.
+ * Open a binary image file, extended version.
  *
  * @returns iprt status code.
  * @param   pszFilename Image filename.
- * @param   phLdrMod    Where to store the handle to the loaded module.
  * @param   fFlags      Valid RTLDR_O_XXX combination.
  * @param   enmArch     CPU architecture specifier for the image to be loaded.
- * @remark  Primarily for testing the loader.
+ * @param   phLdrMod    Where to store the handle to the loader module.
+ * @param   pErrInfo    Where to return extended error information. Optional.
  */
-RTDECL(int) RTLdrOpenkLdr(const char *pszFilename, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod);
+RTDECL(int) RTLdrOpenEx(const char *pszFilename, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod, PRTERRINFO pErrInfo);
 
 /**
  * Open part with reader.
@@ -423,8 +431,9 @@ typedef FNRTLDRRDRMEMREAD *PFNRTLDRRDRMEMREAD;
  *
  * @returns IPRT status code
  * @param   pvUser      The user parameter.
+ * @param   cbImage     The image size.
  */
-typedef DECLCALLBACK(void) FNRTLDRRDRMEMDTOR(void *pvUser);
+typedef DECLCALLBACK(void) FNRTLDRRDRMEMDTOR(void *pvUser, size_t cbImage);
 /** Pointer to a RTLdrOpenInMemory destructor callback. */
 typedef FNRTLDRRDRMEMDTOR *PFNRTLDRRDRMEMDTOR;
 
@@ -579,10 +588,10 @@ RTDECL(size_t) RTLdrSize(RTLDRMOD hLdrMod);
  * @param   pValue          Where to store the symbol value (address).
  * @param   pvUser          User argument.
  */
-typedef DECLCALLBACK(int) RTLDRIMPORT(RTLDRMOD hLdrMod, const char *pszModule, const char *pszSymbol, unsigned uSymbol,
-                                      PRTLDRADDR pValue, void *pvUser);
+typedef DECLCALLBACK(int) FNRTLDRIMPORT(RTLDRMOD hLdrMod, const char *pszModule, const char *pszSymbol, unsigned uSymbol,
+                                        PRTLDRADDR pValue, void *pvUser);
 /** Pointer to a FNRTLDRIMPORT() callback function. */
-typedef RTLDRIMPORT *PFNRTLDRIMPORT;
+typedef FNRTLDRIMPORT *PFNRTLDRIMPORT;
 
 /**
  * Loads the image into a buffer provided by the user and applies fixups
@@ -626,9 +635,9 @@ RTDECL(int) RTLdrRelocate(RTLDRMOD hLdrMod, void *pvBits, RTLDRADDR NewBaseAddre
  * @param   Value           Symbol value.
  * @param   pvUser          The user argument specified to RTLdrEnumSymbols().
  */
-typedef DECLCALLBACK(int) RTLDRENUMSYMS(RTLDRMOD hLdrMod, const char *pszSymbol, unsigned uSymbol, RTLDRADDR Value, void *pvUser);
-/** Pointer to a RTLDRENUMSYMS() callback function. */
-typedef RTLDRENUMSYMS *PFNRTLDRENUMSYMS;
+typedef DECLCALLBACK(int) FNRTLDRENUMSYMS(RTLDRMOD hLdrMod, const char *pszSymbol, unsigned uSymbol, RTLDRADDR Value, void *pvUser);
+/** Pointer to a FNRTLDRENUMSYMS() callback function. */
+typedef FNRTLDRENUMSYMS *PFNRTLDRENUMSYMS;
 
 /**
  * Enumerates all symbols in a module.
@@ -681,7 +690,7 @@ typedef enum RTLDRDBGINFOTYPE
     RTLDRDBGINFOTYPE_COFF,
     /** Watcom debug info. */
     RTLDRDBGINFOTYPE_WATCOM,
-    /** IBM High Level Language debug info.. */
+    /** IBM High Level Language debug info. */
     RTLDRDBGINFOTYPE_HLL,
     /** The end of the valid debug info values (exclusive). */
     RTLDRDBGINFOTYPE_END,
@@ -922,9 +931,9 @@ RTDECL(int) RTLdrLinkAddressToRva(RTLDRMOD hLdrMod, RTLDRADDR LinkAddress, PRTLD
  * @returns IPRT status code.
  *
  * @param   hLdrMod         The module handle.
- * @param   Rva             The link address to convert.
- * @param   piSeg           Where to return the segment index.
- * @param   poffSeg         Where to return the segment offset.
+ * @param   iSeg            The segment index.
+ * @param   offSeg          The segment offset.
+ * @param   pRva            Where to return the RVA.
  */
 RTDECL(int) RTLdrSegOffsetToRva(RTLDRMOD hLdrMod, uint32_t iSeg, RTLDRADDR offSeg, PRTLDRADDR pRva);
 
@@ -934,9 +943,9 @@ RTDECL(int) RTLdrSegOffsetToRva(RTLDRMOD hLdrMod, uint32_t iSeg, RTLDRADDR offSe
  * @returns IPRT status code.
  *
  * @param   hLdrMod         The module handle.
- * @param   iSeg            The segment index.
- * @param   offSeg          The segment offset.
- * @param   pRva            Where to return the RVA.
+ * @param   Rva             The link address to convert.
+ * @param   piSeg           Where to return the segment index.
+ * @param   poffSeg         Where to return the segment offset.
  */
 RTDECL(int) RTLdrRvaToSegOffset(RTLDRMOD hLdrMod, RTLDRADDR Rva, uint32_t *piSeg, PRTLDRADDR poffSeg);
 
@@ -997,6 +1006,7 @@ typedef enum RTLDRPROP
      * @remarks This generally starts with a PKCS \#7 Content structure, the
      *          SignedData bit is found a few levels down into this as per RFC. */
     RTLDRPROP_PKCS7_SIGNED_DATA,
+
     /** Query whether code signature checks are enabled.  */
     RTLDRPROP_SIGNATURE_CHECKS_ENFORCED,
 
@@ -1004,6 +1014,14 @@ typedef enum RTLDRPROP
     RTLDRPROP_IMPORT_COUNT,
     /** Import module by index (32-bit) stored in the buffer. */
     RTLDRPROP_IMPORT_MODULE,
+    /** The file offset of the main executable header.
+     * This is mainly for PE, NE and LX headers, but also Mach-O FAT. */
+    RTLDRPROP_FILE_OFF_HEADER,
+    /** The internal module name.
+     * This is the SONAME for ELF, export table name for PE, and zero'th resident
+     * name table entry for LX.
+     * Returns zero terminated string. */
+    RTLDRPROP_INTERNAL_NAME,
 
     /** End of valid properties.  */
     RTLDRPROP_END,
@@ -1026,7 +1044,7 @@ typedef enum RTLDRPROP
  * @retval  VERR_INVALID_HANDLE if the handle is invalid.
  *
  * @param   hLdrMod         The module handle.
- * @param   enmLdrProp      The property to query.
+ * @param   enmProp         The property to query.
  * @param   pvBuf           Pointer to the input / output buffer.  In most cases
  *                          it's only used for returning data.
  * @param   cbBuf           The size of the buffer.
@@ -1049,7 +1067,7 @@ RTDECL(int) RTLdrQueryProp(RTLDRMOD hLdrMod, RTLDRPROP enmProp, void *pvBuf, siz
  * @retval  VERR_INVALID_HANDLE if the handle is invalid.
  *
  * @param   hLdrMod         The module handle.
- * @param   enmLdrProp      The property to query.
+ * @param   enmProp         The property to query.
  * @param   pvBits          Optional pointer to bits returned by
  *                          RTLdrGetBits().  This can be utilized by some module
  *                          interpreters to reduce memory consumption and file
@@ -1061,7 +1079,7 @@ RTDECL(int) RTLdrQueryProp(RTLDRMOD hLdrMod, RTLDRPROP enmProp, void *pvBuf, siz
  *                          buffer size errors, this is set to the correct size.
  *                          Optional.
  */
-RTDECL(int) RTLdrQueryPropEx(RTLDRMOD hLdrMod, RTLDRPROP enmProp, void *pvBits, void *pvBuf, size_t cbBuf, size_t *pcbBuf);
+RTDECL(int) RTLdrQueryPropEx(RTLDRMOD hLdrMod, RTLDRPROP enmProp, void *pvBits, void *pvBuf, size_t cbBuf, size_t *pcbRet);
 
 
 /**
@@ -1090,11 +1108,17 @@ typedef enum RTLDRSIGNATURETYPE
  * @param   enmSignature    The signature format.
  * @param   pvSignature     The signature data. Format given by @a enmSignature.
  * @param   cbSignature     The size of the buffer @a pvSignature points to.
+ * @param   pvExternalData  Pointer to the signed data, if external. NULL if the
+ *                          data is internal to the signature structure.
+ * @param   cbExternalData Size of the signed data, if external.  0 if
+ *                          internal to the signature structure.
  * @param   pErrInfo        Pointer to an error info buffer, optional.
  * @param   pvUser          User argument.
  *
  */
-typedef DECLCALLBACK(int) FNRTLDRVALIDATESIGNEDDATA(RTLDRMOD hLdrMod, RTLDRSIGNATURETYPE enmSignature, void const *pvSignature, size_t cbSignature,
+typedef DECLCALLBACK(int) FNRTLDRVALIDATESIGNEDDATA(RTLDRMOD hLdrMod, RTLDRSIGNATURETYPE enmSignature,
+                                                    void const *pvSignature, size_t cbSignature,
+                                                    void const *pvExternalData, size_t cbExternalData,
                                                     PRTERRINFO pErrInfo, void *pvUser);
 /** Pointer to a signature verification callback. */
 typedef FNRTLDRVALIDATESIGNEDDATA *PFNRTLDRVALIDATESIGNEDDATA;
