@@ -10,8 +10,6 @@
 #include "cr_string.h"
 #include "packspu_proto.h"
 
-#define MAGIC_OFFSET 3000
-
 /*
  * Allocate a new ThreadInfo structure, setup a connection to the
  * server, allocate/init a packer context, bind this ThreadInfo to
@@ -22,17 +20,15 @@
 ThreadInfo *packspuNewThread(
 #if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
                 struct VBOXUHGSMI *pHgsmi
+#else
+                void
 #endif
 )
 {
     ThreadInfo *thread=NULL;
     int i;
 
-#ifdef CHROMIUM_THREADSAFE
     crLockMutex(&_PackMutex);
-#else
-    CRASSERT(pack_spu.numThreads == 0);
-#endif
 
 #if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
     CRASSERT(!CRPACKSPU_IS_WDDM_CRHGSMI() == !pHgsmi);
@@ -68,7 +64,7 @@ ThreadInfo *packspuNewThread(
     CRASSERT(thread->netServer.conn);
     /* packer setup */
     CRASSERT(thread->packer == NULL);
-    thread->packer = crPackNewContext( pack_spu.swap );
+    thread->packer = crPackNewContext();
     CRASSERT(thread->packer);
     crPackInitBuffer( &(thread->buffer), crNetAlloc(thread->netServer.conn),
                 thread->netServer.conn->buffer_size, thread->netServer.conn->mtu );
@@ -84,18 +80,14 @@ ThreadInfo *packspuNewThread(
     }
 
 
-#ifdef CHROMIUM_THREADSAFE
     if (!CRPACKSPU_IS_WDDM_CRHGSMI())
     {
         crSetTSD(&_PackTSD, thread);
     }
-#endif
 
     pack_spu.numThreads++;
 
-#ifdef CHROMIUM_THREADSAFE
     crUnlockMutex(&_PackMutex);
-#endif
     return thread;
 }
 
@@ -117,6 +109,8 @@ packspu_VBoxConCreate(struct VBOXUHGSMI *pHgsmi)
         return thread->id;
     }
     crError("packspuNewThread failed");
+#else
+    RT_NOREF(pHgsmi);
 #endif
     return 0;
 }
@@ -131,6 +125,7 @@ packspu_VBoxConFlush(GLint con)
     CRASSERT(thread->packer);
     packspuFlush((void *) thread);
 #else
+    RT_NOREF(con);
     crError("VBoxConFlush not implemented!");
 #endif
 }
@@ -181,6 +176,8 @@ packspu_VBoxConDestroy(GLint con)
     }
 #endif
     crUnlockMutex(&_PackMutex);
+#else
+    RT_NOREF(con);
 #endif
 }
 
@@ -190,14 +187,10 @@ packspu_VBoxConChromiumParameteriCR(GLint con, GLenum param, GLint value)
     GET_THREAD(thread);
     CRPackContext * curPacker = crPackGetContext();
     ThreadInfo *curThread = thread;
-    int writeback = 1;
-    GLint serverCtx = (GLint) -1;
 
     CRASSERT(!curThread == !curPacker);
     CRASSERT(!curThread || !curPacker || curThread->packer == curPacker);
-#ifdef CHROMIUM_THREADSAFE
     crLockMutex(&_PackMutex);
-#endif
 
 #if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
     CRASSERT(!con == !CRPACKSPU_IS_WDDM_CRHGSMI());
@@ -231,10 +224,58 @@ packspu_VBoxConChromiumParameteriCR(GLint con, GLenum param, GLint value)
 
     packspu_ChromiumParameteriCR(param, value);
 
-#ifdef CHROMIUM_THREADSAFE
     crUnlockMutex(&_PackMutex);
+    if (CRPACKSPU_IS_WDDM_CRHGSMI())
+    {
+        /* restore the packer context to the tls */
+        crPackSetContext(curPacker);
+    }
+}
+
+GLvoid PACKSPU_APIENTRY
+packspu_VBoxConChromiumParametervCR(GLint con, GLenum target, GLenum type, GLsizei count, const GLvoid *values)
+{
+    GET_THREAD(thread);
+    CRPackContext * curPacker = crPackGetContext();
+    ThreadInfo *curThread = thread;
+
+    CRASSERT(!curThread == !curPacker);
+    CRASSERT(!curThread || !curPacker || curThread->packer == curPacker);
+    crLockMutex(&_PackMutex);
+
+#if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
+    CRASSERT(!con == !CRPACKSPU_IS_WDDM_CRHGSMI());
 #endif
 
+    if (CRPACKSPU_IS_WDDM_CRHGSMI())
+    {
+        if (!con)
+        {
+            crError("connection should be specified!");
+            return;
+        }
+        thread = GET_THREAD_VAL_ID(con);
+    }
+    else
+    {
+        CRASSERT(!con);
+        if (!thread)
+        {
+            thread = packspuNewThread(
+#if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
+                NULL
+#endif
+                );
+        }
+    }
+    CRASSERT(thread);
+    CRASSERT(thread->packer);
+
+    crPackSetContext( thread->packer );
+
+    packspu_ChromiumParametervCR(target, type, count, values);
+
+    crUnlockMutex(&_PackMutex);
     if (CRPACKSPU_IS_WDDM_CRHGSMI())
     {
         /* restore the packer context to the tls */
@@ -254,9 +295,7 @@ packspu_VBoxCreateContext( GLint con, const char *dpyName, GLint visual, GLint s
 
     CRASSERT(!curThread == !curPacker);
     CRASSERT(!curThread || !curPacker || curThread->packer == curPacker);
-#ifdef CHROMIUM_THREADSAFE
     crLockMutex(&_PackMutex);
-#endif
 
 #if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
     CRASSERT(!con == !CRPACKSPU_IS_WDDM_CRHGSMI());
@@ -297,10 +336,7 @@ packspu_VBoxCreateContext( GLint con, const char *dpyName, GLint visual, GLint s
     crPackSetContext( thread->packer );
 
     /* Pack the command */
-    if (pack_spu.swap)
-        crPackCreateContextSWAP( dpyName, visual, shareCtx, &serverCtx, &writeback );
-    else
-        crPackCreateContext( dpyName, visual, shareCtx, &serverCtx, &writeback );
+    crPackCreateContext( dpyName, visual, shareCtx, &serverCtx, &writeback );
 
     /* Flush buffer and get return value */
     packspuFlush(thread);
@@ -315,9 +351,9 @@ packspu_VBoxCreateContext( GLint con, const char *dpyName, GLint visual, GLint s
          * some sort of #define'd constants somewhere so the client and the
          * server could be aware of how each other were numbering things in
          * cases like file networking where they actually
-         * care. 
+         * care.
          *
-         *  -Humper 
+         *  -Humper
          *
          */
         serverCtx = 5000;
@@ -325,13 +361,8 @@ packspu_VBoxCreateContext( GLint con, const char *dpyName, GLint visual, GLint s
     else {
         CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
 
-        if (pack_spu.swap) {
-            serverCtx = (GLint) SWAP32(serverCtx);
-        }
         if (serverCtx < 0) {
-#ifdef CHROMIUM_THREADSAFE
             crUnlockMutex(&_PackMutex);
-#endif
             crWarning("Failure in packspu_CreateContext");
 
             if (CRPACKSPU_IS_WDDM_CRHGSMI())
@@ -362,14 +393,11 @@ packspu_VBoxCreateContext( GLint con, const char *dpyName, GLint visual, GLint s
 
     /* Fill in the new context info */
     /* XXX fix-up sharedCtx param here */
-    pack_spu.context[slot].clientState = crStateCreateContext(NULL, visual, NULL);
+    pack_spu.context[slot].clientState = crStateCreateContext(&pack_spu.StateTracker, NULL, visual, NULL);
     pack_spu.context[slot].clientState->bufferobject.retainBufferData = GL_TRUE;
     pack_spu.context[slot].serverCtx = serverCtx;
 
-#ifdef CHROMIUM_THREADSAFE
     crUnlockMutex(&_PackMutex);
-#endif
-
     if (CRPACKSPU_IS_WDDM_CRHGSMI())
     {
         /* restore the packer context to the tls */
@@ -397,27 +425,30 @@ void PACKSPU_APIENTRY packspu_DestroyContext( GLint ctx )
     CRASSERT(slot >= 0);
     CRASSERT(slot < pack_spu.numContexts);
 
-    context = &(pack_spu.context[slot]);
-
-    if (CRPACKSPU_IS_WDDM_CRHGSMI())
-    {
-        thread = context->currentThread;
-        crPackSetContext(thread->packer);
-        CRASSERT(!(thread->packer == curPacker) == !(thread == curThread));
-    }
-    CRASSERT(thread);
+    context = (slot >= 0 && slot < pack_spu.numContexts) ? &(pack_spu.context[slot]) : NULL;
     curContext = curThread ? curThread->currentContext : NULL;
 
-    if (pack_spu.swap)
-        crPackDestroyContextSWAP( context->serverCtx );
-    else
+    if (context)
+    {
+        if (CRPACKSPU_IS_WDDM_CRHGSMI())
+        {
+            thread = context->currentThread;
+            if (thread)
+            {
+                crPackSetContext(thread->packer);
+                CRASSERT(!(thread->packer == curPacker) == !(thread == curThread));
+            }
+        }
+
         crPackDestroyContext( context->serverCtx );
+        crStateDestroyContext(&pack_spu.StateTracker, context->clientState );
 
-    crStateDestroyContext( context->clientState );
+        context->clientState = NULL;
+        context->serverCtx = 0;
+        context->currentThread = NULL;
 
-    context->clientState = NULL;
-    context->serverCtx = 0;
-    context->currentThread = NULL;
+        crMemset (&context->zvaBufferInfo, 0, sizeof (context->zvaBufferInfo));
+    }
 
     if (curContext == context)
     {
@@ -431,7 +462,7 @@ void PACKSPU_APIENTRY packspu_DestroyContext( GLint ctx )
             crSetTSD(&_PackTSD, NULL);
             crPackSetContext(NULL);
         }
-        crStateMakeCurrent( NULL );
+        crStateMakeCurrent(&pack_spu.StateTracker, NULL);
     }
     else
     {
@@ -444,9 +475,9 @@ void PACKSPU_APIENTRY packspu_DestroyContext( GLint ctx )
 
 void PACKSPU_APIENTRY packspu_MakeCurrent( GLint window, GLint nativeWindow, GLint ctx )
 {
-    ThreadInfo *thread;
+    ThreadInfo *thread = NULL;
     GLint serverCtx;
-    ContextInfo *newCtx;
+    ContextInfo *newCtx = NULL;
 
     if (!CRPACKSPU_IS_WDDM_CRHGSMI())
     {
@@ -469,6 +500,7 @@ void PACKSPU_APIENTRY packspu_MakeCurrent( GLint window, GLint nativeWindow, GLi
         CRASSERT(slot < pack_spu.numContexts);
 
         newCtx = &pack_spu.context[slot];
+        CRASSERT(newCtx);
         CRASSERT(newCtx->clientState);  /* verify valid */
 
         if (CRPACKSPU_IS_WDDM_CRHGSMI())
@@ -480,6 +512,7 @@ void PACKSPU_APIENTRY packspu_MakeCurrent( GLint window, GLint nativeWindow, GLi
         }
         else
         {
+            CRASSERT(thread);
             if (newCtx->fAutoFlush)
             {
                 if (newCtx->currentThread && newCtx->currentThread != thread)
@@ -499,16 +532,19 @@ void PACKSPU_APIENTRY packspu_MakeCurrent( GLint window, GLint nativeWindow, GLi
                 newCtx->currentThread = thread;
             }
 
+            if (thread->currentContext && newCtx != thread->currentContext && thread->currentContext->fCheckZerroVertAttr)
+                crStateCurrentRecoverNew(thread->currentContext->clientState, &thread->packer->current);
+
             thread->currentContext = newCtx;
             crPackSetContext( thread->packer );
         }
 
-        crStateMakeCurrent( newCtx->clientState );
+        crStateMakeCurrent(&pack_spu.StateTracker, newCtx->clientState );
         //crStateSetCurrentPointers(newCtx->clientState, &thread->packer->current);
         serverCtx = pack_spu.context[slot].serverCtx;
     }
     else {
-        crStateMakeCurrent( NULL );
+        crStateMakeCurrent(&pack_spu.StateTracker, NULL );
         if (CRPACKSPU_IS_WDDM_CRHGSMI())
         {
             thread = GET_THREAD_VAL();
@@ -528,10 +564,11 @@ void PACKSPU_APIENTRY packspu_MakeCurrent( GLint window, GLint nativeWindow, GLi
         serverCtx = 0;
     }
 
-    if (pack_spu.swap)
-        crPackMakeCurrentSWAP( window, nativeWindow, serverCtx );
-    else
-        crPackMakeCurrent( window, nativeWindow, serverCtx );
+    crPackMakeCurrent( window, nativeWindow, serverCtx );
+    if (serverCtx)
+    {
+        packspuInitStrings();
+    }
 
     {
         GET_THREAD(t);
