@@ -16,76 +16,81 @@ static GLubyte gpszExtensions[10000];
 #ifdef CR_OPENGL_VERSION_2_0
 static GLubyte gpszShadingVersion[255]="";
 #endif
+static uint32_t g_uOpenGlVersMaj = 0;
+static uint32_t g_uOpenGlVersMin = 0;
+
 
 static void GetString(GLenum name, GLubyte *pszStr)
 {
     GET_THREAD(thread);
     int writeback = 1;
 
-    if (pack_spu.swap)
-        crPackGetStringSWAP(name, pszStr, &writeback);
-    else
-        crPackGetString(name, pszStr, &writeback);
+    crPackGetString(name, pszStr, &writeback);
     packspuFlush( (void *) thread );
 
     CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
 }
 
-static GLfloat
+static void
 GetVersionString(void)
 {
-    GLubyte return_value[100];
-    GLfloat version;
+    static GLboolean fInitialized = GL_FALSE;
 
-    GetString(GL_VERSION, return_value);
-    CRASSERT(crStrlen((char *)return_value) < 100);
+    if (!fInitialized)
+    {
+        int iGlVersion = 0;
+        GLubyte return_value[100];
 
-    version = crStrToFloat((char *) return_value);
-    version = crStateComputeVersion(version);
+        GetString(GL_VERSION, return_value);
+        CRASSERT(crStrlen((char *)return_value) < 100);
 
-    return version;
+        iGlVersion = crStrParseGlVersion((const char *) return_value);
+        g_uOpenGlVersMaj = CR_GLVERSION_GET_MAJOR(iGlVersion);
+        g_uOpenGlVersMin = CR_GLVERSION_GET_MINOR(iGlVersion);
+        crStateComputeVersion(&g_uOpenGlVersMaj, &g_uOpenGlVersMin);
+
+        fInitialized = GL_TRUE;
+    }
 }
 
 static const GLubyte *
 GetExtensions(void)
 {
-    GLubyte return_value[10*1000];
-    const GLubyte *extensions, *ext;
-    GET_THREAD(thread);
-    int writeback = 1;
+    static GLboolean fInitialized = GL_FALSE;
+    if (!fInitialized)
+    {
+        GLubyte return_value[10*1000];
+        const GLubyte *extensions, *ext;
+        GET_THREAD(thread);
+        int writeback = 1;
 
-    if (pack_spu.swap)
-    {
-        crPackGetStringSWAP( GL_EXTENSIONS, return_value, &writeback );
-    }
-    else
-    {
         crPackGetString( GL_EXTENSIONS, return_value, &writeback );
-    }
-    packspuFlush( (void *) thread );
+        packspuFlush( (void *) thread );
 
-    CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
+        CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
 
-    CRASSERT(crStrlen((char *)return_value) < 10*1000);
+        CRASSERT(crStrlen((char *)return_value) < 10*1000);
 
-    /* OK, we got the result from the server.  Now we have to
-     * intersect is with the set of extensions that Chromium understands
-     * and tack on the Chromium-specific extensions.
-     */
-    extensions = return_value;
-    ext = crStateMergeExtensions(1, &extensions);
+        /* OK, we got the result from the server.  Now we have to
+         * intersect is with the set of extensions that Chromium understands
+         * and tack on the Chromium-specific extensions.
+         */
+        extensions = return_value;
+        ext = crStateMergeExtensions(1, &extensions);
 
 #ifdef Linux
-    /*@todo 
-     *That's a hack to allow running Unity, it uses libnux which is calling extension functions
-     *without checking if it's being supported/exported.
-     *glActiveStencilFaceEXT seems to be actually supported but the extension string isn't exported (for ex. on ATI HD4870),
-     *which leads to libglew setting function pointer to NULL and crashing Unity.
-     */
-    sprintf((char*)gpszExtensions, "%s GL_EXT_stencil_two_side", ext);
+        /** @todo
+         *That's a hack to allow running Unity, it uses libnux which is calling extension functions
+         *without checking if it's being supported/exported.
+         *glActiveStencilFaceEXT seems to be actually supported but the extension string isn't exported (for ex. on ATI HD4870),
+         *which leads to libglew setting function pointer to NULL and crashing Unity.
+         */
+        sprintf((char*)gpszExtensions, "%s GL_EXT_stencil_two_side", ext);
 #else
-    sprintf((char*)gpszExtensions, "%s", ext);
+        sprintf((char*)gpszExtensions, "%s", ext);
 #endif
+        fInitialized = GL_TRUE;
+    }
 
     return gpszExtensions;
 }
@@ -110,20 +115,19 @@ const GLubyte * PACKSPU_APIENTRY packspu_GetString( GLenum name )
             if (packspuRunningUnderWine())
             {
                 GetString(GL_REAL_VERSION, ctx->pszRealVersion);
-                return ctx->pszRealVersion;               
+                return ctx->pszRealVersion;
             }
             else
 #endif
             {
                 char *oldlocale;
-                float version;
 
                 oldlocale = setlocale(LC_NUMERIC, NULL);
                 oldlocale = crStrdup(oldlocale);
                 setlocale(LC_NUMERIC, "C");
 
-                version = GetVersionString();
-                sprintf((char*)ctx->glVersion, "%.1f Chromium %s", version, CR_VERSION_STRING);
+                GetVersionString();
+                sprintf((char*)ctx->glVersion, "%u.%u Chromium %s", g_uOpenGlVersMaj, g_uOpenGlVersMin, CR_VERSION_STRING);
 
                 if (oldlocale)
                 {
@@ -143,7 +147,7 @@ const GLubyte * PACKSPU_APIENTRY packspu_GetString( GLenum name )
             else
 #endif
             {
-                return crStateGetString(name);
+                return crStateGetString(&pack_spu.StateTracker, name);
             }
         case GL_RENDERER:
 #ifdef WINDOWS
@@ -155,13 +159,20 @@ const GLubyte * PACKSPU_APIENTRY packspu_GetString( GLenum name )
             else
 #endif
             {
-                return crStateGetString(name);
+                return crStateGetString(&pack_spu.StateTracker, name);
             }
 
 #ifdef CR_OPENGL_VERSION_2_0
         case GL_SHADING_LANGUAGE_VERSION:
-            GetString(GL_SHADING_LANGUAGE_VERSION, gpszShadingVersion);
+        {
+            static GLboolean fInitialized = GL_FALSE;
+            if (!fInitialized)
+            {
+                GetString(GL_SHADING_LANGUAGE_VERSION, gpszShadingVersion);
+                fInitialized = GL_TRUE;
+            }
             return gpszShadingVersion;
+        }
 #endif
 #ifdef GL_CR_real_vendor_strings
         case GL_REAL_VENDOR:
@@ -175,6 +186,18 @@ const GLubyte * PACKSPU_APIENTRY packspu_GetString( GLenum name )
             return ctx->pszRealRenderer;
 #endif
         default:
-            return crStateGetString(name);
+            return crStateGetString(&pack_spu.StateTracker, name);
+    }
+}
+
+void packspuInitStrings()
+{
+    static GLboolean fInitialized = GL_FALSE;
+
+    if (!fInitialized)
+    {
+        packspu_GetString(GL_EXTENSIONS);
+        packspu_GetString(GL_VERSION);
+        fInitialized = GL_TRUE;
     }
 }
