@@ -13,6 +13,8 @@
 #include "packspu.h"
 #include "packspu_proto.h"
 
+uint32_t g_u32VBoxHostCaps = 0;
+
 static void
 packspuWriteback( const CRMessageWriteback *wb )
 {
@@ -28,12 +30,12 @@ packspuWriteback( const CRMessageWriteback *wb )
 static void
 packspuReadback( const CRMessageReadback *rb, unsigned int len )
 {
-    /* minus the header, the destination pointer, 
+    /* minus the header, the destination pointer,
      * *and* the implicit writeback pointer at the head. */
 
     int payload_len = len - sizeof( *rb );
     int *writeback;
-    void *dest_ptr; 
+    void *dest_ptr;
     crMemcpy( &writeback, &(rb->writeback_ptr), sizeof( writeback ) );
     crMemcpy( &dest_ptr, &(rb->readback_ptr), sizeof( dest_ptr ) );
 
@@ -51,6 +53,7 @@ packspuReadPixels( const CRMessageReadPixels *rp, unsigned int len )
 static int
 packspuReceiveData( CRConnection *conn, CRMessage *msg, unsigned int len )
 {
+    RT_NOREF(conn);
     if (msg->header.type == CR_MESSAGE_REDIR_PTR)
         msg = (CRMessage*) msg->redirptr.pMessage;
 
@@ -77,6 +80,7 @@ __prependHeader( CRPackBuffer *buf, unsigned int *len, unsigned int senderID )
 {
     int num_opcodes;
     CRMessageOpcodes *hdr;
+    RT_NOREF(senderID);
 
     CRASSERT( buf );
     CRASSERT( buf->opcode_current < buf->opcode_start );
@@ -85,21 +89,13 @@ __prependHeader( CRPackBuffer *buf, unsigned int *len, unsigned int senderID )
     CRASSERT( buf->data_current <= buf->data_end );
 
     num_opcodes = buf->opcode_start - buf->opcode_current;
-    hdr = (CRMessageOpcodes *) 
+    hdr = (CRMessageOpcodes *)
         ( buf->data_start - ( ( num_opcodes + 3 ) & ~0x3 ) - sizeof(*hdr) );
 
     CRASSERT( (void *) hdr >= buf->pack );
 
-    if (pack_spu.swap)
-    {
-        hdr->header.type = (CRMessageType) SWAP32(CR_MESSAGE_OPCODES);
-        hdr->numOpcodes  = SWAP32(num_opcodes);
-    }
-    else
-    {
-        hdr->header.type = CR_MESSAGE_OPCODES;
-        hdr->numOpcodes  = num_opcodes;
-    }
+    hdr->header.type = CR_MESSAGE_OPCODES;
+    hdr->numOpcodes  = num_opcodes;
 
     *len = buf->data_current - (unsigned char *) hdr;
 
@@ -121,13 +117,14 @@ void packspuFlush(void *arg )
 
     /* we should _always_ pass a valid <arg> value */
     CRASSERT(thread && thread->inUse);
-#ifdef CHROMIUM_THREADSAFE
     CR_LOCK_PACKER_CONTEXT(thread->packer);
-#endif
     ctx = thread->currentContext;
     buf = &(thread->buffer);
     CRASSERT(buf);
 
+    if (ctx && ctx->fCheckZerroVertAttr)
+        crStateCurrentRecoverNew(ctx->clientState, &thread->packer->current);
+
     /* We're done packing into the current buffer, unbind it */
     crPackReleaseBuffer( thread->packer );
 
@@ -145,9 +142,7 @@ void packspuFlush(void *arg )
            /* XXX these calls seem to help, but might be appropriate */
            crPackSetBuffer( thread->packer, buf );
            crPackResetPointers(thread->packer);
-#ifdef CHROMIUM_THREADSAFE
            CR_UNLOCK_PACKER_CONTEXT(thread->packer);
-#endif
            return;
     }
 
@@ -174,10 +169,7 @@ void packspuFlush(void *arg )
     crPackSetBuffer( thread->packer, buf );
 
     crPackResetPointers(thread->packer);
-
-#ifdef CHROMIUM_THREADSAFE
     CR_UNLOCK_PACKER_CONTEXT(thread->packer);
-#endif
 }
 
 
@@ -198,11 +190,6 @@ void packspuHuge( CROpcode opcode, void *buf )
        includes an additional word for the opcode (with alignment) and
        a header */
     len = ((unsigned int *) buf)[-1];
-    if (pack_spu.swap)
-    {
-        /* It's already been swapped, swap it back. */
-        len = SWAP32(len);
-    }
     len += 4 + sizeof(CRMessageOpcodes);
 
     /* write the opcode in just before the length */
@@ -213,17 +200,8 @@ void packspuHuge( CROpcode opcode, void *buf )
     src = (unsigned char *) buf - 8 - sizeof(CRMessageOpcodes);
 
     msg = (CRMessageOpcodes *) src;
-
-    if (pack_spu.swap)
-    {
-        msg->header.type = (CRMessageType) SWAP32(CR_MESSAGE_OPCODES);
-        msg->numOpcodes  = SWAP32(1);
-    }
-    else
-    {
-        msg->header.type = CR_MESSAGE_OPCODES;
-        msg->numOpcodes  = 1;
-    }
+    msg->header.type = CR_MESSAGE_OPCODES;
+    msg->numOpcodes  = 1;
 
     CRASSERT( thread->netServer.conn );
     crNetSend( thread->netServer.conn, NULL, src, len );
@@ -241,6 +219,11 @@ static void packspuFirstConnectToServer( CRNetServer *server
                 , pHgsmi
 #endif
             );
+    if (server->conn)
+    {
+        g_u32VBoxHostCaps = crNetHostCapsGet();
+        crPackCapsSet(g_u32VBoxHostCaps);
+    }
 }
 
 void packspuConnectToServer( CRNetServer *server
@@ -259,7 +242,6 @@ void packspuConnectToServer( CRNetServer *server
             crError("packspuConnectToServer: no connection on first create!");
             return;
         }
-        pack_spu.swap = server->conn->swap;
     }
     else {
         /* a new pthread */
