@@ -2,7 +2,7 @@
    All rights reserved.
 
    See the file LICENSE.txt for information on redistributing this software. */
-    
+
 #include "cr_packfunctions.h"
 #include "cr_glstate.h"
 #include "cr_pixeldata.h"
@@ -139,7 +139,7 @@ static GLboolean packspu_CheckTexImageInternalFormat(GLint internalformat)
         && internalformat!=GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT
 # endif
 #endif
-        /*@todo ARB_texture_float*/
+        /** @todo ARB_texture_float*/
         && internalformat!=GL_RGBA32F_ARB
         && internalformat!=GL_RGB32F_ARB
         && internalformat!=GL_ALPHA32F_ARB
@@ -197,7 +197,7 @@ static const CRPixelPackState _defaultPacking = {
         crPackPixelStorei(enum, _defaultPacking.field); \
     }
 
-static void packspu_ApplyUnpackState()
+static void packspu_ApplyUnpackState(void)
 {
     GET_THREAD(thread);
     ContextInfo *ctx = thread->currentContext;
@@ -213,7 +213,7 @@ static void packspu_ApplyUnpackState()
     APPLY_IF_NEQ(clientState->unpack, psLSBFirst, GL_UNPACK_LSB_FIRST);
 }
 
-static void packspu_RestoreUnpackState()
+static void packspu_RestoreUnpackState(void)
 {
     GET_THREAD(thread);
     ContextInfo *ctx = thread->currentContext;
@@ -229,7 +229,7 @@ static void packspu_RestoreUnpackState()
     RESTORE_IF_NEQ(clientState->unpack, psLSBFirst, GL_UNPACK_LSB_FIRST);
 }
 
-static void packspu_ApplyPackState()
+static void packspu_ApplyPackState(void)
 {
     GET_THREAD(thread);
     ContextInfo *ctx = thread->currentContext;
@@ -245,7 +245,7 @@ static void packspu_ApplyPackState()
     APPLY_IF_NEQ(clientState->pack, psLSBFirst, GL_PACK_LSB_FIRST);
 }
 
-static void packspu_RestorePackState()
+static void packspu_RestorePackState(void)
 {
     GET_THREAD(thread);
     ContextInfo *ctx = thread->currentContext;
@@ -268,12 +268,12 @@ void PACKSPU_APIENTRY packspu_PixelStoref( GLenum pname, GLfloat param )
      * the user's pixel store parameters at that time and pack the
      * image in a canonical layout (see util/pixel.c).
      */
-    crStatePixelStoref( pname, param );
+    crStatePixelStoref(&pack_spu.StateTracker, pname, param );
 }
 
 void PACKSPU_APIENTRY packspu_PixelStorei( GLenum pname, GLint param )
 {
-    crStatePixelStorei( pname, param );
+    crStatePixelStorei(&pack_spu.StateTracker, pname, param );
 }
 
 void PACKSPU_APIENTRY packspu_DrawPixels( GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels )
@@ -282,14 +282,14 @@ void PACKSPU_APIENTRY packspu_DrawPixels( GLsizei width, GLsizei height, GLenum
     ContextInfo *ctx = thread->currentContext;
     CRClientState *clientState = &(ctx->clientState->client);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackDrawPixels( width, height, format, type, pixels, &(clientState->unpack) );
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -300,40 +300,36 @@ void PACKSPU_APIENTRY packspu_ReadPixels( GLint x, GLint y, GLsizei width, GLsiz
     GET_THREAD(thread);
     ContextInfo *ctx = thread->currentContext;
     CRClientState *clientState = &(ctx->clientState->client);
-    int writeback;    
+    int writeback;
 
-    if (crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
     {
         packspu_ApplyPackState();
     }
 
     crPackReadPixels(x, y, width, height, format, type, pixels, &(clientState->pack), &writeback);
 
-    if (crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
     {
         packspu_RestorePackState();
     }
 
 #ifdef CR_ARB_pixel_buffer_object
-    if (!crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (!crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
 #endif
     {
         pack_spu.ReadPixels++;
 
         packspuFlush((void *) thread);
-        while (pack_spu.ReadPixels) 
-            crNetRecv();
+        CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
     }
 }
 
-/*@todo check with pbo's*/
+/** @todo check with pbo's*/
 void PACKSPU_APIENTRY packspu_CopyPixels( GLint x, GLint y, GLsizei width, GLsizei height, GLenum type )
 {
     GET_THREAD(thread);
-    if (pack_spu.swap)
-        crPackCopyPixelsSWAP( x, y, width, height, type );
-    else
-        crPackCopyPixels( x, y, width, height, type );
+    crPackCopyPixels( x, y, width, height, type );
     /* XXX why flush here? */
     packspuFlush( (void *) thread );
 }
@@ -343,14 +339,14 @@ void PACKSPU_APIENTRY packspu_Bitmap( GLsizei width, GLsizei height, GLfloat xor
     GET_CONTEXT(ctx);
     CRClientState *clientState = &(ctx->clientState->client);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackBitmap(width, height, xorig, yorig, xmove, ymove, bitmap, &(clientState->unpack));
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -363,7 +359,7 @@ void PACKSPU_APIENTRY packspu_TexImage1D( GLenum target, GLint level, GLint inte
 
     if (!packspu_CheckTexImageParams(internalformat, format, type))
     {
-        if (pixels || crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+        if (pixels || crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
         {
             crWarning("packspu_TexImage1D invalid internalFormat(%x)/format(%x)/type(%x)", internalformat, format, type);
             return;
@@ -373,17 +369,13 @@ void PACKSPU_APIENTRY packspu_TexImage1D( GLenum target, GLint level, GLint inte
         type = packspu_CheckTexImageType(type) ? type:GL_UNSIGNED_BYTE;
     }
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexImage1DSWAP( target, level, internalformat, width, border, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexImage1D( target, level, internalformat, width, border, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexImage1D( target, level, internalformat, width, border, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -396,7 +388,7 @@ void PACKSPU_APIENTRY packspu_TexImage2D( GLenum target, GLint level, GLint inte
 
     if (!packspu_CheckTexImageParams(internalformat, format, type))
     {
-        if (pixels || crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+        if (pixels || crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
         {
             crWarning("packspu_TexImage2D invalid internalFormat(%x)/format(%x)/type(%x)", internalformat, format, type);
             return;
@@ -406,17 +398,13 @@ void PACKSPU_APIENTRY packspu_TexImage2D( GLenum target, GLint level, GLint inte
         type = packspu_CheckTexImageType(type) ? type:GL_UNSIGNED_BYTE;
     }
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexImage2DSWAP( target, level, internalformat, width, height, border, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexImage2D( target, level, internalformat, width, height, border, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexImage2D( target, level, internalformat, width, height, border, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -428,17 +416,13 @@ void PACKSPU_APIENTRY packspu_TexImage3DEXT( GLenum target, GLint level, GLenum
     GET_CONTEXT(ctx);
     CRClientState *clientState = &(ctx->clientState->client);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexImage3DEXTSWAP( target, level, internalformat, width, height, depth, border, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexImage3DEXT( target, level, internalformat, width, height, depth, border, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexImage3DEXT( target, level, internalformat, width, height, depth, border, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -451,17 +435,13 @@ void PACKSPU_APIENTRY packspu_TexImage3D(GLenum target, GLint level, GLint inter
     GET_CONTEXT(ctx);
     CRClientState *clientState = &(ctx->clientState->client);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexImage3DSWAP( target, level, internalformat, width, height, depth, border, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexImage3D( target, level, internalformat, width, height, depth, border, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexImage3D( target, level, internalformat, width, height, depth, border, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -479,17 +459,13 @@ void PACKSPU_APIENTRY packspu_TexSubImage1D( GLenum target, GLint level, GLint x
         return;
     }
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexSubImage1DSWAP( target, level, xoffset, width, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexSubImage1D( target, level, xoffset, width, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexSubImage1D( target, level, xoffset, width, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -506,17 +482,13 @@ void PACKSPU_APIENTRY packspu_TexSubImage2D( GLenum target, GLint level, GLint x
         return;
     }
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexSubImage2DSWAP( target, level, xoffset, yoffset, width, height, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexSubImage2D( target, level, xoffset, yoffset, width, height, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexSubImage2D( target, level, xoffset, yoffset, width, height, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -528,17 +500,13 @@ void PACKSPU_APIENTRY packspu_TexSubImage3D( GLenum target, GLint level, GLint x
     GET_CONTEXT(ctx);
     CRClientState *clientState = &(ctx->clientState->client);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
-    if (pack_spu.swap)
-        crPackTexSubImage3DSWAP( target, level, xoffset, yoffset, zoffset, width, height, depth, format, type, pixels, &(clientState->unpack) );
-    else
-        crPackTexSubImage3D( target, level, xoffset, yoffset, zoffset, width, height, depth, format, type, pixels, &(clientState->unpack) );
-
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    crPackTexSubImage3D( target, level, xoffset, yoffset, zoffset, width, height, depth, format, type, pixels, &(clientState->unpack) );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -549,10 +517,7 @@ void PACKSPU_APIENTRY packspu_ZPixCR( GLsizei width, GLsizei height, GLenum form
 {
     GET_CONTEXT(ctx);
     CRClientState *clientState = &(ctx->clientState->client);
-    if (pack_spu.swap)
-        crPackZPixCRSWAP( width, height, format, type, ztype, zparm, length, pixels, &(clientState->unpack) );
-    else
-        crPackZPixCR( width, height, format, type, ztype, zparm, length, pixels, &(clientState->unpack) );
+    crPackZPixCR( width, height, format, type, ztype, zparm, length, pixels, &(clientState->unpack) );
 }
 
 void PACKSPU_APIENTRY packspu_GetTexImage (GLenum target, GLint level, GLenum format, GLenum type, GLvoid *pixels)
@@ -567,28 +532,23 @@ void PACKSPU_APIENTRY packspu_GetTexImage (GLenum target, GLint level, GLenum fo
      * and copy the image to the user's buffer using the pixel pack params.
      */
 
-    if (crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
     {
         packspu_ApplyPackState();
     }
 
-    if (pack_spu.swap)
-        crPackGetTexImageSWAP( target, level, format, type, pixels, &(clientState->pack), &writeback );
-    else
-        crPackGetTexImage( target, level, format, type, pixels, &(clientState->pack), &writeback );
-
-    if (crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    crPackGetTexImage( target, level, format, type, pixels, &(clientState->pack), &writeback );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
     {
         packspu_RestorePackState();
     }
 
 #ifdef CR_ARB_pixel_buffer_object
-    if (!crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (!crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
 #endif
     {
         packspuFlush( (void *) thread );
-        while (writeback) 
-            crNetRecv();
+        CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
     }
 }
 
@@ -597,32 +557,23 @@ void PACKSPU_APIENTRY packspu_GetCompressedTexImageARB( GLenum target, GLint lev
     GET_THREAD(thread);
     int writeback = 1;
 
-    if (crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
     {
         packspu_ApplyPackState();
     }
 
-    if (pack_spu.swap)
-    {
-        crPackGetCompressedTexImageARBSWAP( target, level, img, &writeback );
-    }
-    else
-    {
-        crPackGetCompressedTexImageARB( target, level, img, &writeback );
-    }
-
-    if (crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    crPackGetCompressedTexImageARB( target, level, img, &writeback );
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
     {
         packspu_RestorePackState();
     }
 
 #ifdef CR_ARB_pixel_buffer_object
-    if (!crStateIsBufferBound(GL_PIXEL_PACK_BUFFER_ARB))
+    if (!crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_PACK_BUFFER_ARB))
 #endif
     {
         packspuFlush( (void *) thread );
-        while (writeback)
-            crNetRecv();
+        CRPACKSPU_WRITEBACK_WAIT(thread, writeback);
     }
 }
 
@@ -630,14 +581,14 @@ void PACKSPU_APIENTRY
 packspu_CompressedTexImage1DARB(GLenum target, GLint level, GLenum internalformat, GLsizei width,
                                 GLint border, GLsizei imagesize, const GLvoid *data)
 {
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackCompressedTexImage1DARB(target, level, internalformat, width, border, imagesize, data);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -647,14 +598,14 @@ void PACKSPU_APIENTRY
 packspu_CompressedTexImage2DARB(GLenum target, GLint level, GLenum internalformat, GLsizei width,
                                 GLsizei height, GLint border, GLsizei imagesize, const GLvoid *data)
 {
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackCompressedTexImage2DARB(target, level, internalformat, width, height, border, imagesize, data);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -664,14 +615,14 @@ void PACKSPU_APIENTRY
 packspu_CompressedTexImage3DARB(GLenum target, GLint level, GLenum internalformat, GLsizei width,
                                 GLsizei height, GLsizei depth, GLint border, GLsizei imagesize, const GLvoid *data)
 {
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackCompressedTexImage3DARB(target, level, internalformat, width, height, depth, border, imagesize, data);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -681,14 +632,14 @@ void PACKSPU_APIENTRY
 packspu_CompressedTexSubImage1DARB(GLenum target, GLint level, GLint xoffset, GLsizei width,
                                    GLenum format, GLsizei imagesize, const GLvoid *data)
 {
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackCompressedTexSubImage1DARB(target, level, xoffset, width, format, imagesize, data);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -698,14 +649,14 @@ void PACKSPU_APIENTRY
 packspu_CompressedTexSubImage2DARB(GLenum target, GLint level, GLint xoffset, GLint yoffset,
                                    GLsizei width, GLsizei height, GLenum format, GLsizei imagesize, const GLvoid *data)
 {
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackCompressedTexSubImage2DARB(target, level, xoffset, yoffset, width, height, format, imagesize, data);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
@@ -716,14 +667,14 @@ packspu_CompressedTexSubImage3DARB(GLenum target, GLint level, GLint xoffset, GL
                                    GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format,
                                    GLsizei imagesize, const GLvoid *data)
 {
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_ApplyUnpackState();
     }
 
     crPackCompressedTexSubImage3DARB(target, level, xoffset, yoffset, zoffset, width, height, depth, format, imagesize, data);
 
-    if (crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB))
+    if (crStateIsBufferBound(&pack_spu.StateTracker, GL_PIXEL_UNPACK_BUFFER_ARB))
     {
         packspu_RestoreUnpackState();
     }
