@@ -1,12 +1,10 @@
 /* $Id: main.cpp $ */
 /** @file
- *
- * VirtualBox Guest Service:
- * Linux guest.
+ * VirtualBox Guest Additions - X11 Client.
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,6 +16,7 @@
  */
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <stdlib.h>       /* For exit */
 #include <stdio.h>
 #include <string.h>
@@ -45,14 +44,14 @@
 
 #include "VBoxClient.h"
 
-static int (*gpfnOldIOErrorHandler)(Display *) = NULL;
+/*static int (*gpfnOldIOErrorHandler)(Display *) = NULL; - unused */
 
 /** Object representing the service we are running.  This has to be global
  * so that the cleanup routine can access it. */
 struct VBCLSERVICE **g_pService;
 /** The name of our pidfile.  It is global for the benefit of the cleanup
  * routine. */
-static char g_szPidFile[RTPATH_MAX];
+static char g_szPidFile[RTPATH_MAX] = "";
 /** The file handle of our pidfile.  It is global for the benefit of the
  * cleanup routine. */
 static RTFILE g_hPidFile;
@@ -64,22 +63,45 @@ RTCRITSECT g_critSect;
 /** Counter of how often our deamon has been respawned. */
 unsigned cRespawn = 0;
 
-/** Exit with a fatal error. */
+/**
+ * Exit with a fatal error.
+ *
+ * This is used by the VBClFatalError macro and thus needs to be external.
+ */
 void vbclFatalError(char *pszMessage)
 {
     char *pszCommand;
+    int status;
     if (pszMessage && cRespawn == 0)
     {
         pszCommand = RTStrAPrintf2("notify-send \"VBoxClient: %s\"", pszMessage);
         if (pszCommand)
-            system(pszCommand);
+        {
+            status = system(pszCommand);
+            if (WEXITSTATUS(status) != 0)  /* Utility or extension not available. */
+            {
+                pszCommand = RTStrAPrintf2("xmessage -buttons OK:0 -center \"VBoxClient: %s\"",
+                                           pszMessage);
+                if (pszCommand)
+                {
+                    status = system(pszCommand);
+                    if (WEXITSTATUS(status) != 0)  /* Utility or extension not available. */
+                    {
+                        RTPrintf("VBoxClient: %s", pszMessage);
+                    }
+                }
+            }
+        }
     }
-    _exit(1);
+    _exit(RTEXITCODE_FAILURE);
 }
 
-/** Clean up if we get a signal or something.  This is extern so that we
- * can call it from other compilation units. */
-void VBClCleanUp()
+/**
+ * Clean up if we get a signal or something.
+ *
+ * This is extern so that we can call it from other compilation units.
+ */
+void VBClCleanUp(bool fExit /*=true*/)
 {
     /* We never release this, as we end up with a call to exit(3) which is not
      * async-safe.  Unless we fix this application properly, we should be sure
@@ -91,7 +113,8 @@ void VBClCleanUp()
         (*g_pService)->cleanup(g_pService);
     if (g_szPidFile[0] && g_hPidFile)
         VbglR3ClosePidFile(g_szPidFile, g_hPidFile);
-    exit(0);
+    if (fExit)
+        exit(RTEXITCODE_SUCCESS);
 }
 
 /**
@@ -114,7 +137,7 @@ static int vboxClientXLibErrorHandler(Display *pDisplay, XErrorEvent *pError)
 
     XGetErrorText(pDisplay, pError->error_code, errorText, sizeof(errorText));
     LogRelFlow(("VBoxClient: an X Window protocol error occurred: %s (error code %d).  Request code: %d, minor code: %d, serial number: %d\n", errorText, pError->error_code, pError->request_code, pError->minor_code, pError->serial));
-    return 0;  /* We should never reach this. */
+    return 0;
 }
 
 /**
@@ -123,6 +146,7 @@ static int vboxClientXLibErrorHandler(Display *pDisplay, XErrorEvent *pError)
  */
 static int vboxClientXLibIOErrorHandler(Display *pDisplay)
 {
+    RT_NOREF1(pDisplay);
     LogRel(("VBoxClient: a fatal guest X Window error occurred.  This may just mean that the Window system was shut down while the client was still running.\n"));
     VBClCleanUp();
     return 0;  /* We should never reach this. */
@@ -151,95 +175,10 @@ static void vboxClientSetSignalHandlers(void)
     LogRelFlowFunc(("returning\n"));
 }
 
-/** Check whether X.Org has acquired or lost the current virtual terminal and
- * call the service @a pause() or @a resume() call-back if appropriate.
- * The functionality is provided by the vboxvideo driver for pre-1.16 X servers
- * and by 1.16 and later series servers.
- * This can either be called directly from a service's event loop or the service
- * can call VBClStartVTMonitor() to start an event loop in a separate thread.
- * Property notification for the root window should be selected first.  Services
- * are not required to check VT changes if they do not need the information.
- * @param  pEvent an event received on a display connection which will be
- *                checked to see if it is change to the XFree86_has_VT property
- */
-void VBClCheckXOrgVT(union _XEvent *pEvent)
-{
-    Atom actualType;
-    int actualFormat;
-    unsigned long cItems, cbLeft;
-    bool fHasVT = false;
-    unsigned long *pValue;
-    int rc;
-    Display *pDisplay = pEvent->xany.display;
-    Atom hasVT = XInternAtom(pDisplay, "XFree86_has_VT", False);
-
-    if (   pEvent->type != PropertyNotify
-        || pEvent->xproperty.window != DefaultRootWindow(pDisplay)
-        || pEvent->xproperty.atom != hasVT)
-        return;
-    XGetWindowProperty(pDisplay, DefaultRootWindow(pDisplay), hasVT, 0, 1,
-                       False, XA_INTEGER, &actualType, &actualFormat, &cItems,
-                       &cbLeft, (unsigned char **)&pValue);
-    if (cItems && actualFormat == 32)
-    {
-        fHasVT = *pValue != 0;
-        XFree(pValue);
-    }
-    else
-        return;
-    if (fHasVT)
-    {
-        rc = (*g_pService)->resume(g_pService);
-        if (RT_FAILURE(rc))
-            VBClFatalError(("Error resuming the service: %Rrc\n"));
-    }
-    if (!fHasVT)
-    {
-        rc = (*g_pService)->pause(g_pService);
-        if (RT_FAILURE(rc))
-            VBClFatalError(("Error pausing the service: %Rrc\n"));
-    }
-}
-
-/**
- * Thread which notifies the service when we switch to a different VT or back
- * and cleans up when the X server exits.
- * @note runs until programme exit.
- */
-static int pfnMonitorThread(RTTHREAD self, void *pvUser)
-{
-    Display *pDisplay;
-    bool fHasVT = true;
-
-    pDisplay = XOpenDisplay(NULL);
-    if (!pDisplay)
-        VBClFatalError(("Failed to open the X11 display\n"));
-    XSelectInput(pDisplay, DefaultRootWindow(pDisplay), PropertyChangeMask);
-    while (true)
-    {
-        XEvent event;
-
-        XNextEvent(pDisplay, &event);
-        VBClCheckXOrgVT(&event);
-    }
-    return VINF_SUCCESS;  /* Should never be reached. */
-}
-
-/**
- * Start a thread which notifies the service when we switch to a different
- * VT or back, and terminates us when the X server exits.  This should be called
- * by most services which do not regularly run an X11 event loop.
- */
-int VBClStartVTMonitor()
-{
-    return RTThreadCreate(NULL, pfnMonitorThread, NULL, 0,
-                          RTTHREADTYPE_INFREQUENT_POLLER, 0, "MONITOR");
-}
-
 /**
  * Print out a usage message and exit with success.
  */
-void vboxClientUsage(const char *pcszFileName)
+static void vboxClientUsage(const char *pcszFileName)
 {
     RTPrintf("Usage: %s --clipboard|"
 #ifdef VBOX_WITH_DRAG_AND_DROP
@@ -249,8 +188,10 @@ void vboxClientUsage(const char *pcszFileName)
 # ifdef VBOX_WITH_GUEST_PROPS
              "--checkhostversion|"
 #endif
-             "--seamless [-d|--nodaemon]\n", pcszFileName);
-    RTPrintf("Starts the VirtualBox X Window System guest services.\n\n");
+             "--seamless|check3d|"
+             "--vmsvga|--vmsvga-x11"
+             "[-d|--nodaemon]\n", pcszFileName);
+    RTPrintf("Starts the VirtualBox DRM/X Window System guest services.\n\n");
     RTPrintf("Options:\n");
     RTPrintf("  --clipboard        starts the shared clipboard service\n");
 #ifdef VBOX_WITH_DRAG_AND_DROP
@@ -260,12 +201,26 @@ void vboxClientUsage(const char *pcszFileName)
 #ifdef VBOX_WITH_GUEST_PROPS
     RTPrintf("  --checkhostversion starts the host version notifier service\n");
 #endif
+    RTPrintf("  --check3d          tests whether 3D pass-through is enabled\n");
     RTPrintf("  --seamless         starts the seamless windows service\n");
+    RTPrintf("  --vmsvga           starts VMSVGA dynamic resizing for DRM\n");
+    RTPrintf("  --vmsvga-x11       starts VMSVGA dynamic resizing for X11\n");
+    RTPrintf("  -f, --foreground   run in the foreground (no daemonizing)\n");
     RTPrintf("  -d, --nodaemon     continues running as a system service\n");
     RTPrintf("  -h, --help         shows this help text\n");
     RTPrintf("  -V, --version      shows version information\n");
     RTPrintf("\n");
-    exit(0);
+}
+
+/**
+ * Complains about seeing more than one service specification.
+ *
+ * @returns RTEXITCODE_SYNTAX.
+ */
+static int vbclSyntaxOnlyOneService(void)
+{
+    RTMsgError("More than one service specified! Only one, please.");
+    return RTEXITCODE_SYNTAX;
 }
 
 /**
@@ -274,39 +229,48 @@ void vboxClientUsage(const char *pcszFileName)
  */
 int main(int argc, char *argv[])
 {
-    bool fDaemonise = true, fRespawn = true;
-    int rc;
-    const char *pcszFileName;
-
     /* Initialise our runtime before all else. */
-    rc = RTR3InitExe(argc, &argv, 0);
+    int rc = RTR3InitExe(argc, &argv, 0);
     if (RT_FAILURE(rc))
         return RTMsgInitFailure(rc);
+
     /* This should never be called twice in one process - in fact one Display
      * object should probably never be used from multiple threads anyway. */
     if (!XInitThreads())
         VBClFatalError(("Failed to initialize X11 threads\n"));
-    /* Get our file name for error output. */
-    pcszFileName = RTPathFilename(argv[0]);
+
+    /* Get our file name for usage info and hints. */
+    const char *pcszFileName = RTPathFilename(argv[0]);
     if (!pcszFileName)
         pcszFileName = "VBoxClient";
 
     /* Parse our option(s) */
     /** @todo Use RTGetOpt() if the arguments become more complex. */
+    bool fDaemonise = true;
+    bool fRespawn = true;
     for (int i = 1; i < argc; ++i)
     {
-        rc = VERR_INVALID_PARAMETER;
-        if (!strcmp(argv[i], "-d") || !strcmp(argv[i], "--nodaemon"))
+        if (   !strcmp(argv[i], "-f")
+            || !strcmp(argv[i], "--foreground")
+            || !strcmp(argv[i], "-d")
+            || !strcmp(argv[i], "--nodaemon"))
         {
             /* If the user is running in "no daemon" mode anyway, send critical
              * logging to stdout as well. */
+            /** @todo r=bird: Since the release logger isn't created until the service
+             *        calls VbglR3InitUser or VbglR3Init or RTLogCreate, this whole
+             *        exercise is pointless.  Added --init-vbgl-user and --init-vbgl-full
+             *        for getting some work done. */
             PRTLOGGER pReleaseLog = RTLogRelGetDefaultInstance();
             if (pReleaseLog)
                 rc = RTLogDestinations(pReleaseLog, "stdout");
             if (pReleaseLog && RT_FAILURE(rc))
-                RTPrintf("%s: failed to redivert error output, rc=%Rrc\n",
-                         pcszFileName, rc);
+                return RTMsgErrorExitFailure("failed to redivert error output, rc=%Rrc", rc);
+
             fDaemonise = false;
+            if (   !strcmp(argv[i], "-f")
+                || !strcmp(argv[i], "--foreground"))
+                fRespawn = false;
         }
         else if (!strcmp(argv[i], "--no-respawn"))
         {
@@ -315,79 +279,112 @@ int main(int argc, char *argv[])
         else if (!strcmp(argv[i], "--clipboard"))
         {
             if (g_pService)
-                break;
+                return vbclSyntaxOnlyOneService();
             g_pService = VBClGetClipboardService();
         }
         else if (!strcmp(argv[i], "--display"))
         {
             if (g_pService)
-                break;
+                return vbclSyntaxOnlyOneService();
             g_pService = VBClGetDisplayService();
         }
         else if (!strcmp(argv[i], "--seamless"))
         {
             if (g_pService)
-                break;
+                return vbclSyntaxOnlyOneService();
             g_pService = VBClGetSeamlessService();
         }
         else if (!strcmp(argv[i], "--checkhostversion"))
         {
             if (g_pService)
-                break;
+                return vbclSyntaxOnlyOneService();
             g_pService = VBClGetHostVersionService();
         }
 #ifdef VBOX_WITH_DRAG_AND_DROP
         else if (!strcmp(argv[i], "--draganddrop"))
         {
             if (g_pService)
-                break;
+                return vbclSyntaxOnlyOneService();
             g_pService = VBClGetDragAndDropService();
         }
 #endif /* VBOX_WITH_DRAG_AND_DROP */
+        else if (!strcmp(argv[i], "--check3d"))
+        {
+            if (g_pService)
+                return vbclSyntaxOnlyOneService();
+            g_pService = VBClCheck3DService();
+        }
+        else if (!strcmp(argv[i], "--vmsvga"))
+        {
+            if (g_pService)
+                return vbclSyntaxOnlyOneService();
+            g_pService = VBClDisplaySVGAService();
+        }
+        else if (!strcmp(argv[i], "--vmsvga-x11"))
+        {
+            if (g_pService)
+                break;
+            g_pService = VBClDisplaySVGAX11Service();
+        }
+        /* bird: this is just a quick hack to get something out of the LogRel statements in the code. */
+        else if (!strcmp(argv[i], "--init-vbgl-user"))
+        {
+            rc = VbglR3InitUser();
+            if (RT_FAILURE(rc))
+                return RTMsgErrorExitFailure("VbglR3InitUser failed: %Rrc", rc);
+        }
+        else if (!strcmp(argv[i], "--init-vbgl-full"))
+        {
+            rc = VbglR3Init();
+            if (RT_FAILURE(rc))
+                return RTMsgErrorExitFailure("VbglR3Init failed: %Rrc", rc);
+        }
         else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))
         {
             vboxClientUsage(pcszFileName);
-            return 0;
+            return RTEXITCODE_SUCCESS;
         }
         else if (!strcmp(argv[i], "-V") || !strcmp(argv[i], "--version"))
         {
             RTPrintf("%sr%s\n", RTBldCfgVersion(), RTBldCfgRevisionStr());
-            return 0;
+            return RTEXITCODE_SUCCESS;
         }
         else
         {
-            RTPrintf("%s: unrecognized option `%s'\n", pcszFileName, argv[i]);
-            RTPrintf("Try `%s --help' for more information\n", pcszFileName);
-            return 1;
+            RTMsgError("unrecognized option `%s'", argv[i]);
+            RTMsgInfo("Try `%s --help' for more information", pcszFileName);
+            return RTEXITCODE_SYNTAX;
         }
-        rc = VINF_SUCCESS;
     }
-    if (RT_FAILURE(rc) || !g_pService)
+    if (!g_pService)
     {
-        vboxClientUsage(pcszFileName);
-        return 1;
+        RTMsgError("No service specified. Quitting because nothing to do!");
+        return RTEXITCODE_SYNTAX;
     }
 
     rc = RTCritSectInit(&g_critSect);
     if (RT_FAILURE(rc))
-        VBClFatalError(("Initialising critical section: %Rrc\n", rc));
-    rc = RTPathUserHome(g_szPidFile, sizeof(g_szPidFile));
-    if (RT_FAILURE(rc))
-        VBClFatalError(("Getting home directory for pid-file: %Rrc\n", rc));
-    rc = RTPathAppend(g_szPidFile, sizeof(g_szPidFile),
-                      (*g_pService)->getPidFilePath());
-    if (RT_FAILURE(rc))
-        VBClFatalError(("Creating pid-file path: %Rrc\n", rc));
-    if (fDaemonise)
-        rc = VbglR3Daemonize(false /* fNoChDir */, false /* fNoClose */, fRespawn, &cRespawn);
-    if (RT_FAILURE(rc))
-        VBClFatalError(("Daemonizing: %Rrc\n", rc));
-    if (g_szPidFile[0])
-        rc = VbglR3PidFile(g_szPidFile, &g_hPidFile);
-    if (rc == VERR_FILE_LOCK_VIOLATION)  /* Already running. */
-        return 0;
-    if (RT_FAILURE(rc))
-        VBClFatalError(("Creating pid-file: %Rrc\n", rc));
+        VBClFatalError(("Initialising critical section failed: %Rrc\n", rc));
+    if ((*g_pService)->getPidFilePath)
+    {
+        rc = RTPathUserHome(g_szPidFile, sizeof(g_szPidFile));
+        if (RT_FAILURE(rc))
+            VBClFatalError(("Getting home directory for PID file failed: %Rrc\n", rc));
+        rc = RTPathAppend(g_szPidFile, sizeof(g_szPidFile),
+                          (*g_pService)->getPidFilePath());
+        if (RT_FAILURE(rc))
+            VBClFatalError(("Creating PID file path failed: %Rrc\n", rc));
+        if (fDaemonise)
+            rc = VbglR3Daemonize(false /* fNoChDir */, false /* fNoClose */, fRespawn, &cRespawn);
+        if (RT_FAILURE(rc))
+            VBClFatalError(("Daemonizing failed: %Rrc\n", rc));
+        if (g_szPidFile[0])
+            rc = VbglR3PidFile(g_szPidFile, &g_hPidFile);
+        if (rc == VERR_FILE_LOCK_VIOLATION)  /* Already running. */
+            return RTEXITCODE_SUCCESS;
+        if (RT_FAILURE(rc))
+            VBClFatalError(("Creating PID file failed: %Rrc\n", rc));
+    }
     /* Set signal handlers to clean up on exit. */
     vboxClientSetSignalHandlers();
 #ifndef VBOXCLIENT_WITHOUT_X11
@@ -399,11 +396,19 @@ int main(int argc, char *argv[])
     XSetIOErrorHandler(vboxClientXLibIOErrorHandler);
 #endif
     rc = (*g_pService)->init(g_pService);
-    if (RT_FAILURE(rc))
-        VBClFatalError(("Initialising service: %Rrc\n", rc));
-    rc = (*g_pService)->run(g_pService, fDaemonise);
-    if (RT_FAILURE(rc))
-        VBClFatalError(("Service main loop failed: %Rrc\n", rc));
-    VBClCleanUp();
-    return 0;
+    if (RT_SUCCESS(rc))
+    {
+        rc = (*g_pService)->run(g_pService, fDaemonise);
+        if (RT_FAILURE(rc))
+            LogRel2(("Running service failed: %Rrc\n", rc));
+    }
+    else
+    {
+        /** @todo r=andy Should we return an appropriate exit code if the service failed to init?
+         *               Must be tested carefully with our init scripts first. */
+        LogRel2(("Initializing service failed: %Rrc\n", rc));
+    }
+    VBClCleanUp(false /*fExit*/);
+    return RTEXITCODE_SUCCESS;
 }
+
