@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -29,6 +29,13 @@
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
 
+// VirtualBox COM interfaces declarations (generated header)
+#ifdef VBOX_WITH_XPCOM
+# include <VirtualBox_XPCOM.h>
+#else
+# include <VirtualBox.h>
+#endif
+
 
 class VBoxDbgConsoleOutput : public QTextEdit
 {
@@ -39,9 +46,10 @@ public:
      * Constructor.
      *
      * @param   pParent     Parent Widget.
+     * @param   pVirtualBox VirtualBox object for storing extra data.
      * @param   pszName     Widget name.
      */
-    VBoxDbgConsoleOutput(QWidget *pParent = NULL, const char *pszName = NULL);
+    VBoxDbgConsoleOutput(QWidget *pParent = NULL, IVirtualBox *pVirtualBox = NULL, const char *pszName = NULL);
 
     /**
      * Destructor
@@ -60,20 +68,65 @@ public:
      */
     virtual void appendText(const QString &rStr, bool fClearSelection);
 
+    /** The action to switch to black-on-white color scheme. */
+    QAction *m_pBlackOnWhiteAction;
+    /** The action to switch to green-on-black color scheme. */
+    QAction *m_pGreenOnBlackAction;
+
+    /** The action to switch to Courier font. */
+    QAction *m_pCourierFontAction;
+    /** The action to switch to Monospace font. */
+    QAction *m_pMonospaceFontAction;
+
 protected:
+    typedef enum  { kGreenOnBlack, kBlackOnWhite } VBoxDbgConsoleColor;
+
+    /**
+     * Context menu event.
+     * This adds custom menu items for the output view.
+     *
+     * @param pEvent   Pointer to the event.
+     */
+    virtual void contextMenuEvent(QContextMenuEvent *pEvent);
+
     /** The current line (paragraph) number. */
     unsigned m_uCurLine;
     /** The position in the current line. */
     unsigned m_uCurPos;
     /** The handle to the GUI thread. */
     RTNATIVETHREAD m_hGUIThread;
+    /** The current color scheme (foreground on background). */
+    VBoxDbgConsoleColor m_enmColorScheme;
+    /** The IVirtualBox object */
+    IVirtualBox *m_pVirtualBox;
+
+private slots:
+    /**
+     * The green-on-black color scheme context-menu item was triggered.
+     */
+    void        setColorGreenOnBlack();
+
+    /**
+     * The black-on-white color scheme context-menu item was triggered.
+     */
+    void        setColorBlackOnWhite();
+
+    /**
+     * The courier font family context-menu item was triggered.
+     */
+    void        setFontCourier();
+
+    /**
+     * The monospace font family context-menu item was triggered.
+     */
+    void        setFontMonospace();
 };
 
 
 /**
  * The Debugger Console Input widget.
  *
- * This is a combobox which only responds to <return>.
+ * This is a combobox which only responds to \<return\>.
  */
 class VBoxDbgConsoleInput : public QComboBox
 {
@@ -135,8 +188,9 @@ public:
      *
      * @param   a_pDbgGui       Pointer to the debugger gui object.
      * @param   a_pParent       Parent Widget.
+     * @param   a_pVirtualBox   VirtualBox object for storing extra data.
      */
-    VBoxDbgConsole(VBoxDbgGui *a_pDbgGui, QWidget *a_pParent = NULL);
+    VBoxDbgConsole(VBoxDbgGui *a_pDbgGui, QWidget *a_pParent = NULL, IVirtualBox *a_pVirtualBox = NULL);
 
     /**
      * Destructor
@@ -254,6 +308,13 @@ protected:
      */
     bool event(QEvent *pEvent);
 
+    /**
+     * For implementing keyboard shortcuts.
+     *
+     * @param   pEvent      The key event.
+     */
+    void keyReleaseEvent(QKeyEvent *pEvent);
+
 protected:
     /** The output widget. */
     VBoxDbgConsoleOutput *m_pOutput;
