@@ -1,7 +1,7 @@
 ## @file
 # Parse FV image
 #
-# Copyright (c) 2008 - 2010, Intel Corporation. All rights reserved.<BR>
+# Copyright (c) 2008 - 2014, Intel Corporation. All rights reserved.<BR>
 # This program and the accompanying materials
 # are licensed and made available under the terms and conditions of the BSD License
 # which accompanies this distribution.  The full text of the license may be found at
@@ -13,7 +13,7 @@
 
 ## Import Modules
 #
-import os
+import Common.LongFilePathOs as os
 import re
 import sys
 import uuid
@@ -24,7 +24,7 @@ import copy
 from UserDict import IterableUserDict
 from cStringIO import StringIO
 from array import array
-
+from Common.LongFilePathSupport import OpenLongFilePath as open
 from CommonDataClass import *
 from Common.Misc import sdict, GuidStructureStringToGuidString
 
