@@ -1,71 +1,3 @@
-/*
- *
- * BIOS Memory
- *
- */
-#define BIOSMEM_SEG 0x40
-
-#define BIOSMEM_INITIAL_MODE  0x10
-#define BIOSMEM_CURRENT_MODE  0x49
-#define BIOSMEM_NB_COLS       0x4A
-#define BIOSMEM_PAGE_SIZE     0x4C
-#define BIOSMEM_CURRENT_START 0x4E
-#define BIOSMEM_CURSOR_POS    0x50
-#define BIOSMEM_CURSOR_TYPE   0x60
-#define BIOSMEM_CURRENT_PAGE  0x62
-#define BIOSMEM_CRTC_ADDRESS  0x63
-#define BIOSMEM_CURRENT_MSR   0x65
-#define BIOSMEM_CURRENT_PAL   0x66
-#define BIOSMEM_NB_ROWS       0x84
-#define BIOSMEM_CHAR_HEIGHT   0x85
-#define BIOSMEM_VIDEO_CTL     0x87
-#define BIOSMEM_SWITCHES      0x88
-#define BIOSMEM_MODESET_CTL   0x89
-#define BIOSMEM_DCC_INDEX     0x8A
-#define BIOSMEM_VS_POINTER    0xA8
-#define BIOSMEM_VBE_FLAG      0xB9
-#define BIOSMEM_VBE_MODE      0xBA
-
-
-/*
- *
- * VGA registers
- *
- */
-#define VGAREG_ACTL_ADDRESS            0x3c0
-#define VGAREG_ACTL_WRITE_DATA         0x3c0
-#define VGAREG_ACTL_READ_DATA          0x3c1
-
-#define VGAREG_INPUT_STATUS            0x3c2
-#define VGAREG_WRITE_MISC_OUTPUT       0x3c2
-#define VGAREG_VIDEO_ENABLE            0x3c3
-#define VGAREG_SEQU_ADDRESS            0x3c4
-#define VGAREG_SEQU_DATA               0x3c5
-
-#define VGAREG_PEL_MASK                0x3c6
-#define VGAREG_DAC_STATE               0x3c7
-#define VGAREG_DAC_READ_ADDRESS        0x3c7
-#define VGAREG_DAC_WRITE_ADDRESS       0x3c8
-#define VGAREG_DAC_DATA                0x3c9
-
-#define VGAREG_READ_FEATURE_CTL        0x3ca
-#define VGAREG_READ_MISC_OUTPUT        0x3cc
-
-#define VGAREG_GRDC_ADDRESS            0x3ce
-#define VGAREG_GRDC_DATA               0x3cf
-
-#define VGAREG_MDA_CRTC_ADDRESS        0x3b4
-#define VGAREG_MDA_CRTC_DATA           0x3b5
-#define VGAREG_VGA_CRTC_ADDRESS        0x3d4
-#define VGAREG_VGA_CRTC_DATA           0x3d5
-
-#define VGAREG_MDA_WRITE_FEATURE_CTL   0x3ba
-#define VGAREG_VGA_WRITE_FEATURE_CTL   0x3da
-#define VGAREG_ACTL_RESET              0x3da
-
-#define VGAREG_MDA_MODECTL             0x3b8
-#define VGAREG_CGA_MODECTL             0x3d8
-#define VGAREG_CGA_PALETTE             0x3d9
 
 /* Video memory */
 #define VGAMEM_GRAPH 0xA000
@@ -95,13 +27,13 @@
 #define LINEAR32   0x13
 
 typedef struct
-{Bit8u  svgamode;
- Bit8u  class;    /* TEXT, GRAPH */
- Bit8u  memmodel; /* CTEXT,MTEXT,CGA,PL1,PL2,PL4,P8,P15,P16,P24,P32 */
- Bit8u  pixbits;
- Bit16u sstart;
- Bit8u  pelmask;
- Bit8u  dacmodel; /* 0 1 2 3 */
+{uint8_t    svgamode;
+ uint8_t    class;    /* TEXT, GRAPH */
+ uint8_t    memmodel; /* CTEXT,MTEXT,CGA,PL1,PL2,PL4,P8,P15,P16,P24,P32 */
+ uint8_t    pixbits;
+ uint16_t   sstart;
+ uint8_t    pelmask;
+ uint8_t    dacmodel; /* 0 1 2 3 */
 } VGAMODES;
 
 static VGAMODES vga_modes[MODE_MAX+1]=
@@ -125,7 +57,7 @@ static VGAMODES vga_modes[MODE_MAX+1]=
 };
 
 /* convert index in vga_modes[] to index in video_param_table[] */
-static Bit8u line_to_vpti[MODE_MAX+1]={
+static uint8_t line_to_vpti[MODE_MAX+1]={
     0x17, 0x17, 0x18, 0x18, 0x04, 0x05, 0x06, 0x07,
     0x0d, 0x0e, 0x11, 0x12, 0x1a, 0x1b, 0x1c, 0x1d,
 };
@@ -133,21 +65,21 @@ static Bit8u line_to_vpti[MODE_MAX+1]={
 /* Default Palette */
 #define DAC_MAX_MODEL 3
 
-static Bit8u dac_regs[DAC_MAX_MODEL+1]=
+static uint8_t dac_regs[DAC_MAX_MODEL+1]=
 {0x3f,0x3f,0x3f,0xff};
 
 /* standard BIOS Video Parameter Table */
 typedef struct {
-    Bit8u  twidth;
-    Bit8u  theightm1;
-    Bit8u  cheight;
-    Bit8u  slength_l;
-    Bit8u  slength_h;
-    Bit8u  sequ_regs[4];
-    Bit8u  miscreg;
-    Bit8u  crtc_regs[25];
-    Bit8u  actl_regs[20];
-    Bit8u  grdc_regs[9];
+    uint8_t     twidth;
+    uint8_t     theightm1;
+    uint8_t     cheight;
+    uint8_t     slength_l;
+    uint8_t     slength_h;
+    uint8_t     sequ_regs[4];
+    uint8_t     miscreg;
+    uint8_t     crtc_regs[25];
+    uint8_t     actl_regs[20];
+    uint8_t     grdc_regs[9];
 } VideoParamTableEntry;
 
 static VideoParamTableEntry video_param_table[30] = {
@@ -418,7 +350,7 @@ static VideoParamTableEntry video_param_table[30] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 },
 {
-    /* index=0x18 ega mode 0x03 */
+    /* index=0x16 ega mode 0x03 */
     80, 24, 14, 0x00, 0x10, /* tw, th-1, ch, slength */
     0x00, 0x03, 0x00, 0x02, /* sequ_regs */
     0x67, /* miscreg */
@@ -532,7 +464,7 @@ static VideoParamTableEntry video_param_table[30] = {
 };
 
 /* Mono */
-static Bit8u palette0[63+1][3]=
+static uint8_t palette0[63+1][3]=
 {
   0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00,
   0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a,
@@ -544,7 +476,7 @@ static Bit8u palette0[63+1][3]=
   0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f
 };
 
-static Bit8u palette1[63+1][3]=
+static uint8_t palette1[63+1][3]=
 {
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a, 0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x15,0x00, 0x2a,0x2a,0x2a,
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a, 0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x15,0x00, 0x2a,0x2a,0x2a,
@@ -556,7 +488,7 @@ static Bit8u palette1[63+1][3]=
   0x15,0x15,0x15, 0x15,0x15,0x3f, 0x15,0x3f,0x15, 0x15,0x3f,0x3f, 0x3f,0x15,0x15, 0x3f,0x15,0x3f, 0x3f,0x3f,0x15, 0x3f,0x3f,0x3f
 };
 
-static Bit8u palette2[63+1][3]=
+static uint8_t palette2[63+1][3]=
 {
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a, 0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x2a,0x00, 0x2a,0x2a,0x2a,
   0x00,0x00,0x15, 0x00,0x00,0x3f, 0x00,0x2a,0x15, 0x00,0x2a,0x3f, 0x2a,0x00,0x15, 0x2a,0x00,0x3f, 0x2a,0x2a,0x15, 0x2a,0x2a,0x3f,
@@ -568,7 +500,7 @@ static Bit8u palette2[63+1][3]=
   0x15,0x15,0x15, 0x15,0x15,0x3f, 0x15,0x3f,0x15, 0x15,0x3f,0x3f, 0x3f,0x15,0x15, 0x3f,0x15,0x3f, 0x3f,0x3f,0x15, 0x3f,0x3f,0x3f
 };
 
-static Bit8u palette3[256][3]=
+static uint8_t palette3[256][3]=
 {
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a, 0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x15,0x00, 0x2a,0x2a,0x2a,
   0x15,0x15,0x15, 0x15,0x15,0x3f, 0x15,0x3f,0x15, 0x15,0x3f,0x3f, 0x3f,0x15,0x15, 0x3f,0x15,0x3f, 0x3f,0x3f,0x15, 0x3f,0x3f,0x3f,
@@ -607,14 +539,14 @@ static Bit8u palette3[256][3]=
   0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00
 };
 
-static Bit8u static_functionality[0x10]=
+static uint8_t static_functionality[0x10]=
 {
  /* 0 */ 0xff,  // All modes supported #1
  /* 1 */ 0xe0,  // All modes supported #2
  /* 2 */ 0x0f,  // All modes supported #3
  /* 3 */ 0x00, 0x00, 0x00, 0x00,  // reserved
  /* 7 */ 0x07,  // 200, 350, 400 scan lines
- /* 8 */ 0x02,  // mamimum number of visible charsets in text mode
+ /* 8 */ 0x02,  // maximum number of visible charsets in text mode
  /* 9 */ 0x08,  // total number of charset blocks in text mode
  /* a */ 0xe7,  // Change to add new functions
  /* b */ 0x0c,  // Change to add new functions
