@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -128,7 +128,7 @@ ip_output0(PNATState pData, struct socket *so, struct mbuf *m0, int urg)
     STAM_PROFILE_START(&pData->StatIP_output, a);
 
 #ifdef LOG_ENABLED
-    LogFlowFunc(("ip_output: so = %R[natsock], m0 = %lx\n", so, (long)m0));
+    LogFlowFunc(("ip_output: so = %R[natsock], m0 = %p\n", so, m0));
 #else
     NOREF(so);
 #endif
@@ -150,8 +150,9 @@ ip_output0(PNATState pData, struct socket *so, struct mbuf *m0, int urg)
      */
     ip->ip_v = IPVERSION;
     ip->ip_off &= IP_DF;
-    ip->ip_id = RT_H2N_U16(ip_currid++);
+    ip->ip_id = RT_H2N_U16(ip_currid);
     ip->ip_hl = hlen >> 2;
+    ip_currid++;
     ipstat.ips_localout++;
 
     /* Current TCP/IP stack hasn't routing information at
@@ -173,15 +174,8 @@ ip_output0(PNATState pData, struct socket *so, struct mbuf *m0, int urg)
         ip->ip_sum = cksum(m, hlen);
 
         if (!(m->m_flags & M_SKIP_FIREWALL)){
-            struct m_tag *t;
             STAM_PROFILE_START(&pData->StatALIAS_output, b);
-            if ((t = m_tag_find(m, PACKET_TAG_ALIAS, NULL)) != 0)
-                rc = LibAliasOut((struct libalias *)&t[1], mtod(m, char *),
-                                 m_length(m, NULL));
-            else
-                rc = LibAliasOut(pData->proxy_alias, mtod(m, char *),
-                                 m_length(m, NULL));
-
+            rc = LibAliasOut(pData->proxy_alias, mtod(m, char *), m_length(m, NULL));
             if (rc == PKT_ALIAS_IGNORED)
             {
                 Log(("NAT: packet was droppped\n"));
@@ -237,7 +231,7 @@ ip_output0(PNATState pData, struct socket *so, struct mbuf *m0, int urg)
             {
                 error = -1;
                 ipstat.ips_odropped++;
-                goto send_or_free;
+                goto exit_drop_package;
             }
             m->m_data += if_maxlinkhdr;
             mhip = mtod(m, struct ip *);
@@ -296,9 +290,8 @@ ip_output0(PNATState pData, struct socket *so, struct mbuf *m0, int urg)
         ip->ip_sum = 0;
         ip->ip_sum = cksum(m, mhlen);
 
-send_or_free:
         if (!(m->m_flags & M_SKIP_FIREWALL)){
-            /* @todo: We can't alias all fragments because the way libalias processing
+            /** @todo We can't alias all fragments because the way libalias processing
              * the fragments brake the sequence. libalias put alias_address to the source
              * address of IP header of fragment, while IP header of the first packet is
              * is unmodified. That confuses guest's TCP/IP stack and guest drop the sequence.
@@ -307,13 +300,9 @@ send_or_free:
              * Here we need investigate what should be done to avoid such behavior and find right
              * solution.
              */
-            struct m_tag *t;
             int rcLa;
-            if ((t = m_tag_find(m, PACKET_TAG_ALIAS, NULL)) != 0)
-                rcLa = LibAliasOut((struct libalias *)&t[1], mtod(m, char *), m->m_len);
-            else
-                rcLa = LibAliasOut(pData->proxy_alias, mtod(m, char *), m->m_len);
 
+            rcLa = LibAliasOut(pData->proxy_alias, mtod(m, char *), m->m_len);
             if (rcLa == PKT_ALIAS_IGNORED)
             {
                 Log(("NAT: packet was droppped\n"));
