@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,7 +19,7 @@
 #define _LIBSLIRP_H
 
 #ifdef RT_OS_WINDOWS
-# include <winsock2.h>
+# include <iprt/win/winsock2.h>
 # ifdef __cplusplus
 extern "C" {
 # endif
@@ -37,6 +37,7 @@ int inet_aton(const char *cp, struct in_addr *ia);
 #endif
 
 #include <VBox/types.h>
+#include <iprt/req.h>
 
 typedef struct NATState *PNATState;
 struct mbuf;
@@ -55,14 +56,13 @@ void slirp_link_down(PNATState);
 #if defined(RT_OS_WINDOWS)
 void slirp_select_fill(PNATState pData, int *pndfs);
 
-void slirp_select_poll(PNATState pData, int fTimeout, int fIcmp);
+void slirp_select_poll(PNATState pData, int fTimeout);
 #else /* RT_OS_WINDOWS */
 void slirp_select_fill(PNATState pData, int *pnfds, struct pollfd *polls);
 void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs);
 #endif /* !RT_OS_WINDOWS */
 
 void slirp_input(PNATState pData, struct mbuf *m, size_t cbBuf);
-void slirp_set_ethaddr_and_activate_port_forwarding(PNATState pData, const uint8_t *ethaddr, uint32_t GuestIP);
 
 /* you must provide the following functions: */
 void slirp_arm_fast_timer(void *pvUser);
@@ -72,9 +72,18 @@ void slirp_output_pending(void * pvUser);
 void slirp_urg_output(void *pvUser, struct mbuf *, const uint8_t *pu8Buf, int cb);
 void slirp_post_sent(PNATState pData, void *pvArg);
 
+int slirp_call(void *pvUser, PRTREQ *ppReq, RTMSINTERVAL cMillies,
+               unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...);
+
+int slirp_call_hostres(void *pvUser, PRTREQ *ppReq, RTMSINTERVAL cMillies,
+                       unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...);
+
+
+void slirp_update_guest_addr_guess(PNATState pData, uint32_t guess, const char *msg);
+
 int slirp_add_redirect(PNATState pData, int is_udp, struct in_addr host_addr,
                 int host_port, struct in_addr guest_addr,
-                int guest_port, const uint8_t *);
+                int guest_port);
 int slirp_remove_redirect(PNATState pData, int is_udp, struct in_addr host_addr,
                 int host_port, struct in_addr guest_addr,
                 int guest_port);
@@ -95,25 +104,32 @@ void slirp_set_mtu(PNATState, int);
 void slirp_info(PNATState pData, const void *pvArg, const char *pszArgs);
 void slirp_set_somaxconn(PNATState pData, int iSoMaxConn);
 
+/**
+ * This macrodefinition is shortcut for check of hosts where Slirp,
+ * receives notifications from host. For now it's Darwin only. But
+ * Main API has primitives for listening DNS change event since 4.3.
+ */
+#if defined(RT_OS_DARWIN) || defined(RT_OS_WINDOWS)
+# define HAVE_NOTIFICATION_FOR_DNS_UPDATE 1
+#else
+# define HAVE_NOTIFICATION_FOR_DNS_UPDATE 0
+#endif
+
+
 /**
  * This method help DrvNAT to select strategy: about VMRESUMEREASON_HOST_RESUME:
  * - proceed with link termination (we let guest track host DNS settings)
- *    VBOX_NAT_HNCE_EXPOSED_NAME_RESOLVING_INFO
+ *    VBOX_NAT_DNS_EXTERNAL
  * - enforce internal DNS update (we are using dnsproxy and track but don't export DNS host settings)
- *    VBOX_NAT_HNCE_DNSPROXY
- * - flap link and trigger guest to request new DHCP configuration (means that NAT was temporary in
- *   host resolver mode due to temporary DNS data outage)
- *    VBOX_NAT_HNCE_HOSTRESOLVER_TEMPORARY
+ *    VBOX_NAT_DNS_DNSPROXY
  * - ignore (NAT configured to use hostresolver - we aren't track any host DNS changes)
- *    VBOX_NAT_HNCE_HOSTRESOLVER
- * @note: It's safe to call this method from any thread, because settings we're checking 
+ *    VBOX_NAT_DNS_HOSTRESOLVER
+ * @note: It's safe to call this method from any thread, because settings we're checking
  * are immutable at runtime.
  */
-#define VBOX_NAT_HNCE_EXSPOSED_NAME_RESOLUTION_INFO 0
-#define VBOX_NAT_HNCE_DNSPROXY 1
-#define VBOX_NAT_HNCE_HOSTRESOLVER_TEMPORARY 2
-#define VBOX_NAT_HNCE_HOSTRESOLVER 3
-
+#define VBOX_NAT_DNS_EXTERNAL 0
+#define VBOX_NAT_DNS_DNSPROXY 1
+#define VBOX_NAT_DNS_HOSTRESOLVER 2
 int slirp_host_network_configuration_change_strategy_selector(const PNATState);
 #if defined(RT_OS_WINDOWS)
 
@@ -162,7 +178,9 @@ int slirp_get_nsock(PNATState pData);
 # endif
 
 #ifdef VBOX_WITH_DNSMAPPING_IN_HOSTRESOLVER
-void  slirp_add_host_resolver_mapping(PNATState pData, const char *pszHostName, const char *pszHostNamePattern, uint32_t u32HostIP);
+void slirp_add_host_resolver_mapping(PNATState pData,
+                                     const char *pszHostName, bool fPattern,
+                                     uint32_t u32HostIP);
 #endif
 
 #ifdef __cplusplus
