@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,8 +21,8 @@
 #include <VBox/vmm/stam.h>
 
 #ifdef RT_OS_WINDOWS
-# include <winsock2.h>
-# include <ws2tcpip.h>
+# include <iprt/win/winsock2.h>
+# include <iprt/win/ws2tcpip.h>
 typedef int socklen_t;
 #endif
 #ifdef RT_OS_OS2 /* temporary workaround, see ticket #127 */
@@ -46,7 +46,7 @@ typedef int socklen_t;
 #include <VBox/log.h>
 #include <iprt/mem.h>
 #ifdef RT_OS_WINDOWS
-# include <windows.h>
+# include <iprt/win/windows.h>
 # include <io.h>
 #endif
 #include <iprt/asm.h>
@@ -76,24 +76,37 @@ typedef int socklen_t;
 
 
 # include <sys/timeb.h>
-# include <iphlpapi.h>
+# include <iprt/win/iphlpapi.h>
 
 /* We don't want the errno.h versions of these error defines. */
 # if defined(_MSC_VER) && _MSC_VER >= 1600
 #  include <errno.h>
-#  undef EWOULDBLOCK
-#  undef EINPROGRESS
-#  undef ENOTCONN
-#  undef EHOSTUNREACH
-#  undef ENETUNREACH
 #  undef ECONNREFUSED
+#  undef ECONNRESET
+#  undef EHOSTDOWN
+#  undef EHOSTUNREACH
+#  undef EINPROGRESS
+#  undef ENETDOWN
+#  undef ENETUNREACH
+#  undef ENOTCONN
+#  undef ESHUTDOWN
+#  undef EWOULDBLOCK
 # endif
-# define EWOULDBLOCK WSAEWOULDBLOCK
-# define EINPROGRESS WSAEINPROGRESS
-# define ENOTCONN WSAENOTCONN
-# define EHOSTUNREACH WSAEHOSTUNREACH
-# define ENETUNREACH WSAENETUNREACH
 # define ECONNREFUSED WSAECONNREFUSED
+# define ECONNRESET WSAECONNRESET
+# define EHOSTDOWN WSAEHOSTDOWN
+# define EHOSTUNREACH WSAEHOSTUNREACH
+# define EINPROGRESS WSAEINPROGRESS
+# define ENETDOWN WSAENETDOWN
+# define ENETUNREACH WSAENETUNREACH
+# define ENOTCONN WSAENOTCONN
+# define ESHUTDOWN WSAESHUTDOWN
+# define EWOULDBLOCK WSAEWOULDBLOCK
+
+/* standard names for the shutdown() "how" argument */
+#define SHUT_RD SD_RECEIVE
+#define SHUT_WR SD_SEND
+#define SHUT_RDWR SD_BOTH
 
 typedef uint8_t u_int8_t;
 typedef uint16_t u_int16_t;
@@ -171,6 +184,9 @@ typedef unsigned char u_int8_t;
 # include <sys/time.h>
 # include <time.h>
 #else
+# ifndef HAVE_SYS_TIME_H
+#  define HAVE_SYS_TIME_H 0
+# endif
 # if HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
@@ -320,6 +336,7 @@ int ip_output0 (PNATState, struct socket *, struct mbuf *, int urg);
 /* tcp_input.c */
 int tcp_reass (PNATState, struct tcpcb *, struct tcphdr *, int *, struct mbuf *);
 void tcp_input (PNATState, register struct mbuf *, int, struct socket *);
+void tcp_fconnect_failed(PNATState, struct socket *, int);
 void tcp_dooptions (PNATState, struct tcpcb *, u_char *, int, struct tcpiphdr *);
 void tcp_xmit_timer (PNATState, register struct tcpcb *, int);
 int tcp_mss (PNATState, register struct tcpcb *, u_int);
@@ -343,6 +360,9 @@ u_int8_t tcp_tos (struct socket *);
 int tcp_ctl (PNATState, struct socket *);
 struct tcpcb *tcp_drop(PNATState, struct tcpcb *tp, int err);
 
+/* hostres.c */
+struct mbuf *hostresolver(PNATState, struct mbuf *, uint32_t src, uint16_t sport);
+
 /*slirp.c*/
 void slirp_arp_who_has(PNATState pData, uint32_t dst);
 int slirp_arp_cache_update_or_add(PNATState pData, uint32_t dst, const uint8_t *mac);
@@ -457,12 +477,10 @@ int ftp_alias_load(PNATState);
 int ftp_alias_unload(PNATState);
 int nbt_alias_load(PNATState);
 int nbt_alias_unload(PNATState);
-int dns_alias_load(PNATState);
-int dns_alias_unload(PNATState);
 int slirp_arp_lookup_ip_by_ether(PNATState, const uint8_t *, uint32_t *);
 int slirp_arp_lookup_ether_by_ip(PNATState, uint32_t, uint8_t *);
 
-static inline size_t slirp_size(PNATState pData)
+DECLINLINE(unsigned) slirp_size(PNATState pData)
 {
         if (if_mtu < MSIZE)
             return MCLBYTES;
@@ -521,7 +539,7 @@ static inline struct mbuf *slirpDnsMbufAlloc(PNATState pData)
 
 DECLINLINE(bool) slirpIsWideCasting(PNATState pData, uint32_t u32Addr)
 {
-    bool fWideCasting = false;
+    bool fWideCasting;
     LogFlowFunc(("Enter: u32Addr:%RTnaipv4\n", u32Addr));
     fWideCasting =  (   u32Addr == INADDR_BROADCAST
                     || (u32Addr & RT_H2N_U32_C(~pData->netmask)) == RT_H2N_U32_C(~pData->netmask));
