@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,9 +15,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 //#define DEBUG
 #define LOG_GROUP LOG_GROUP_DEV_BUSLOGIC /** @todo Create extra group. */
 
@@ -27,6 +28,7 @@
 
 #include <VBox/vmm/pdmdev.h>
 #include <VBox/vmm/pgm.h>
+#include <VBox/version.h>
 #include <iprt/asm.h>
 #include <iprt/mem.h>
 #include <iprt/thread.h>
@@ -67,10 +69,37 @@ static void vboxscsiReset(PVBOXSCSI pVBoxSCSI, bool fEverything)
  */
 int vboxscsiInitialize(PVBOXSCSI pVBoxSCSI)
 {
-    pVBoxSCSI->pbBuf = NULL;
-    vboxscsiReset(pVBoxSCSI, true /*fEverything*/);
+    int rc = RTCritSectInit(&pVBoxSCSI->CritSect);
+    if (RT_SUCCESS(rc))
+    {
+        pVBoxSCSI->pbBuf = NULL;
+        vboxscsiReset(pVBoxSCSI, true /*fEverything*/);
+    }
 
-    return VINF_SUCCESS;
+    return rc;
+}
+
+/**
+ * Frees all allocated resources.
+ *
+ * @returns nothing.
+ * @param   pVBoxSCSI    Pointer to the SCSI state,
+ */
+void vboxscsiDestroy(PVBOXSCSI pVBoxSCSI)
+{
+    if (RTCritSectIsInitialized(&pVBoxSCSI->CritSect))
+        RTCritSectDelete(&pVBoxSCSI->CritSect);
+}
+
+/**
+ * Performs a hardware reset.
+ *
+ * @returns nothing.
+ * @param   pVBoxSCSI    Pointer to the SCSI state,
+ */
+void vboxscsiHwReset(PVBOXSCSI pVBoxSCSI)
+{
+    vboxscsiReset(pVBoxSCSI, true /*fEverything*/);
 }
 
 /**
@@ -85,6 +114,7 @@ int vboxscsiReadRegister(PVBOXSCSI pVBoxSCSI, uint8_t iRegister, uint32_t *pu32V
 {
     uint8_t uVal = 0;
 
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
     switch (iRegister)
     {
         case 0:
@@ -135,6 +165,7 @@ int vboxscsiReadRegister(PVBOXSCSI pVBoxSCSI, uint8_t iRegister, uint32_t *pu32V
     }
 
     *pu32Value = uVal;
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
 
     return VINF_SUCCESS;
 }
@@ -152,6 +183,7 @@ int vboxscsiWriteRegister(PVBOXSCSI pVBoxSCSI, uint8_t iRegister, uint8_t uVal)
 {
     int rc = VINF_SUCCESS;
 
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
     switch (iRegister)
     {
         case 0:
@@ -174,6 +206,9 @@ int vboxscsiWriteRegister(PVBOXSCSI pVBoxSCSI, uint8_t iRegister, uint8_t uVal)
             else if (pVBoxSCSI->enmState == VBOXSCSISTATE_READ_CDB_SIZE_BUFHI)
             {
                 uint8_t cbCDB = uVal & 0x0F;
+
+                if (cbCDB == 0)
+                    cbCDB = 16;
                 if (cbCDB > VBOXSCSI_CDB_SIZE_MAX)
                     vboxscsiReset(pVBoxSCSI, true /*fEverything*/);
                 else
@@ -263,6 +298,7 @@ int vboxscsiWriteRegister(PVBOXSCSI pVBoxSCSI, uint8_t iRegister, uint8_t uVal)
             AssertMsgFailed(("Invalid register to write to %u\n", iRegister));
     }
 
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
     return rc;
 }
 
@@ -271,15 +307,20 @@ int vboxscsiWriteRegister(PVBOXSCSI pVBoxSCSI, uint8_t iRegister, uint8_t uVal)
  *
  * @returns VBox status code.
  * @param   pVBoxSCSI      Pointer to the SCSI state.
- * @param   pScsiRequest   Pointer to a scsi request to setup.
+ * @param   puLun          Where to store the LUN on success.
+ * @param   ppbCdb         Where to store the pointer to the CDB on success.
+ * @param   pcbCdb         Where to store the size of the CDB on success.
+ * @param   pcbBuf         Where to store th size of the data buffer on success.
  * @param   puTargetDevice Where to store the target device ID.
  */
-int vboxscsiSetupRequest(PVBOXSCSI pVBoxSCSI, PPDMSCSIREQUEST pScsiRequest, uint32_t *puTargetDevice)
+int vboxscsiSetupRequest(PVBOXSCSI pVBoxSCSI, uint32_t *puLun, uint8_t **ppbCdb,
+                         size_t *pcbCdb, size_t *pcbBuf, uint32_t *puTargetDevice)
 {
     int rc = VINF_SUCCESS;
 
-    LogFlowFunc(("pVBoxSCSI=%#p pScsiRequest=%#p puTargetDevice=%#p\n", pVBoxSCSI, pScsiRequest, puTargetDevice));
+    LogFlowFunc(("pVBoxSCSI=%#p puTargetDevice=%#p\n", pVBoxSCSI, puTargetDevice));
 
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
     AssertMsg(pVBoxSCSI->enmState == VBOXSCSISTATE_COMMAND_READY, ("Invalid state %u\n", pVBoxSCSI->enmState));
 
     /* Clear any errors from a previous request. */
@@ -295,29 +336,12 @@ int vboxscsiSetupRequest(PVBOXSCSI pVBoxSCSI, PPDMSCSIREQUEST pScsiRequest, uint
             return VERR_NO_MEMORY;
     }
 
-    /* Allocate scatter gather element. */
-    pScsiRequest->paScatterGatherHead = (PRTSGSEG)RTMemAllocZ(sizeof(RTSGSEG) * 1); /* Only one element. */
-    if (!pScsiRequest->paScatterGatherHead)
-    {
-        RTMemFree(pVBoxSCSI->pbBuf);
-        pVBoxSCSI->pbBuf = NULL;
-        return VERR_NO_MEMORY;
-    }
-
-    /* Allocate sense buffer. */
-    pScsiRequest->cbSenseBuffer = 18;
-    pScsiRequest->pbSenseBuffer = (uint8_t *)RTMemAllocZ(pScsiRequest->cbSenseBuffer);
-
-    pScsiRequest->cbCDB = pVBoxSCSI->cbCDB;
-    pScsiRequest->pbCDB = pVBoxSCSI->abCDB;
-    pScsiRequest->uLogicalUnit = 0;
-    pScsiRequest->cbScatterGather = pVBoxSCSI->cbBuf;
-    pScsiRequest->cScatterGatherEntries = 1;
-
-    pScsiRequest->paScatterGatherHead[0].cbSeg = pVBoxSCSI->cbBuf;
-    pScsiRequest->paScatterGatherHead[0].pvSeg = pVBoxSCSI->pbBuf;
-
+    *puLun = 0;
+    *ppbCdb = &pVBoxSCSI->abCDB[0];
+    *pcbCdb = pVBoxSCSI->cbCDB;
+    *pcbBuf = pVBoxSCSI->cbBuf;
     *puTargetDevice = pVBoxSCSI->uTargetDevice;
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
 
     return rc;
 }
@@ -326,25 +350,52 @@ int vboxscsiSetupRequest(PVBOXSCSI pVBoxSCSI, PPDMSCSIREQUEST pScsiRequest, uint
  * Notifies the device that a request finished and the incoming data
  * is ready at the incoming data port.
  */
-int vboxscsiRequestFinished(PVBOXSCSI pVBoxSCSI, PPDMSCSIREQUEST pScsiRequest, int rcCompletion)
+int vboxscsiRequestFinished(PVBOXSCSI pVBoxSCSI, int rcCompletion)
 {
-    LogFlowFunc(("pVBoxSCSI=%#p pScsiRequest=%#p\n", pVBoxSCSI, pScsiRequest));
-    RTMemFree(pScsiRequest->paScatterGatherHead);
-    RTMemFree(pScsiRequest->pbSenseBuffer);
+    LogFlowFunc(("pVBoxSCSI=%#p\n", pVBoxSCSI));
 
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
     if (pVBoxSCSI->uTxDir == VBOXSCSI_TXDIR_TO_DEVICE)
         vboxscsiReset(pVBoxSCSI, false /*fEverything*/);
 
     pVBoxSCSI->rcCompletion = rcCompletion;
 
     ASMAtomicXchgBool(&pVBoxSCSI->fBusy, false);
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
 
     return VINF_SUCCESS;
 }
 
+size_t vboxscsiCopyToBuf(PVBOXSCSI pVBoxSCSI, PRTSGBUF pSgBuf, size_t cbSkip, size_t cbCopy)
+{
+    AssertPtrReturn(pVBoxSCSI->pbBuf, 0);
+    AssertReturn(cbSkip + cbCopy <= pVBoxSCSI->cbBuf, 0);
+
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
+    void *pvBuf = pVBoxSCSI->pbBuf + cbSkip;
+    size_t cbCopied = RTSgBufCopyToBuf(pSgBuf, pvBuf, cbCopy);
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
+
+    return cbCopied;
+}
+
+size_t vboxscsiCopyFromBuf(PVBOXSCSI pVBoxSCSI, PRTSGBUF pSgBuf, size_t cbSkip, size_t cbCopy)
+{
+    AssertPtrReturn(pVBoxSCSI->pbBuf, 0);
+    AssertReturn(cbSkip + cbCopy <= pVBoxSCSI->cbBuf, 0);
+
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
+    void *pvBuf = pVBoxSCSI->pbBuf + cbSkip;
+    size_t cbCopied = RTSgBufCopyFromBuf(pSgBuf, pvBuf, cbCopy);
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
+
+    return cbCopied;
+}
+
 int vboxscsiReadString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegister,
                        uint8_t *pbDst, uint32_t *pcTransfers, unsigned cb)
 {
+    RT_NOREF(pDevIns);
     LogFlowFunc(("pDevIns=%#p pVBoxSCSI=%#p iRegister=%d cTransfers=%u cb=%u\n",
                  pDevIns, pVBoxSCSI, iRegister, *pcTransfers, cb));
 
@@ -363,6 +414,7 @@ int vboxscsiReadString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegiste
     AssertReturn(pVBoxSCSI->enmState == VBOXSCSISTATE_COMMAND_READY, VINF_SUCCESS);
     Assert(!pVBoxSCSI->fBusy);
 
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
     /*
      * Also ignore attempts to read more data than is available.
      */
@@ -395,6 +447,7 @@ int vboxscsiReadString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegiste
         memset(pbDst, 0, cbTransfer);
     }
     *pcTransfers = 0;
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
 
     return rc;
 }
@@ -402,6 +455,8 @@ int vboxscsiReadString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegiste
 int vboxscsiWriteString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegister,
                         uint8_t const *pbSrc, uint32_t *pcTransfers, unsigned cb)
 {
+    RT_NOREF(pDevIns);
+
     /*
      * Check preconditions, fall back to non-string I/O handler.
      */
@@ -416,6 +471,7 @@ int vboxscsiWriteString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegist
     AssertReturn(pVBoxSCSI->enmState == VBOXSCSISTATE_COMMAND_READY, VINF_SUCCESS);
     AssertReturn(pVBoxSCSI->uTxDir == VBOXSCSI_TXDIR_TO_DEVICE, VINF_SUCCESS);
 
+    RTCritSectEnter(&pVBoxSCSI->CritSect);
     /*
      * Ignore excess data (not supposed to happen).
      */
@@ -439,17 +495,15 @@ int vboxscsiWriteString(PPDMDEVINS pDevIns, PVBOXSCSI pVBoxSCSI, uint8_t iRegist
     else
         AssertFailed();
     *pcTransfers = 0;
+    RTCritSectLeave(&pVBoxSCSI->CritSect);
 
     return rc;
 }
 
-void vboxscsiSetRequestRedo(PVBOXSCSI pVBoxSCSI, PPDMSCSIREQUEST pScsiRequest)
+void vboxscsiSetRequestRedo(PVBOXSCSI pVBoxSCSI)
 {
     AssertMsg(pVBoxSCSI->fBusy, ("No request to redo\n"));
 
-    RTMemFree(pScsiRequest->paScatterGatherHead);
-    RTMemFree(pScsiRequest->pbSenseBuffer);
-
     if (pVBoxSCSI->uTxDir == VBOXSCSI_TXDIR_FROM_DEVICE)
     {
         AssertPtr(pVBoxSCSI->pbBuf);
@@ -462,7 +516,24 @@ DECLHIDDEN(int) vboxscsiR3LoadExec(PVBOXSCSI pVBoxSCSI, PSSMHANDLE pSSM)
     SSMR3GetU8  (pSSM, &pVBoxSCSI->uTargetDevice);
     SSMR3GetU8  (pSSM, &pVBoxSCSI->uTxDir);
     SSMR3GetU8  (pSSM, &pVBoxSCSI->cbCDB);
-    SSMR3GetMem (pSSM, &pVBoxSCSI->abCDB[0], sizeof(pVBoxSCSI->abCDB));
+
+    /*
+     * The CDB buffer was increased with r104155 in trunk (backported to 5.0
+     * in r104311) without bumping the SSM state versions which leaves us
+     * with broken saved state restoring for older VirtualBox releases
+     * (up to 5.0.10).
+     */
+    if (   (   SSMR3HandleRevision(pSSM) < 104311
+            && SSMR3HandleVersion(pSSM)  < VBOX_FULL_VERSION_MAKE(5, 0, 12))
+        || (   SSMR3HandleRevision(pSSM) < 104155
+            && SSMR3HandleVersion(pSSM)  >= VBOX_FULL_VERSION_MAKE(5, 0, 51)))
+    {
+        memset(&pVBoxSCSI->abCDB[0], 0, sizeof(pVBoxSCSI->abCDB));
+        SSMR3GetMem (pSSM, &pVBoxSCSI->abCDB[0], 12);
+    }
+    else
+        SSMR3GetMem (pSSM, &pVBoxSCSI->abCDB[0], sizeof(pVBoxSCSI->abCDB));
+
     SSMR3GetU8  (pSSM, &pVBoxSCSI->iCDB);
     SSMR3GetU32 (pSSM, &pVBoxSCSI->cbBufLeft);
     SSMR3GetU32 (pSSM, &pVBoxSCSI->iBuf);
