@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2015 Oracle Corporation
+ * Copyright (C) 2010-2017 Oracle Corporation
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -21,7 +21,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
@@ -29,22 +29,21 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <VBox/ExtPack/ExtPack.h>
 
 #include <VBox/err.h>
 #include <VBox/version.h>
-#include <VBox/vmm/cfgm.h>
 #include <iprt/string.h>
 #include <iprt/param.h>
 #include <iprt/path.h>
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** Pointer to the extension pack helpers. */
 static PCVBOXEXTPACKHLP g_pHlp;
 
@@ -52,100 +51,51 @@ static PCVBOXEXTPACKHLP g_pHlp;
 // /**
 //  * @interface_method_impl{VBOXEXTPACKREG,pfnInstalled}
 //  */
-// static DECLCALLBACK(void) vboxSkeletonExtPack_Installed(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox);
+// static DECLCALLBACK(void) vboxBusMouseExtPack_Installed(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox, PRTERRINFO pErrInfo);
+//
 // /**
 //  * @interface_method_impl{VBOXEXTPACKREG,pfnUninstall}
 //  */
-// static DECLCALLBACK(int)  vboxSkeletonExtPack_Uninstall(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox);
+// static DECLCALLBACK(int)  vboxBusMouseExtPack_Uninstall(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox);
 //
 // /**
 //  * @interface_method_impl{VBOXEXTPACKREG,pfnVirtualBoxReady}
 //  */
-// static DECLCALLBACK(void)  vboxSkeletonExtPack_VirtualBoxReady(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox);
+// static DECLCALLBACK(void)  vboxBusMouseExtPack_VirtualBoxReady(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox);
 //
 // /**
 //  * @interface_method_impl{VBOXEXTPACKREG,pfnUnload}
 //  */
-// static DECLCALLBACK(void) vboxSkeletonExtPack_Unload(PCVBOXEXTPACKREG pThis);
+// static DECLCALLBACK(void) vboxBusMouseExtPack_Unload(PCVBOXEXTPACKREG pThis);
+//
 // /**
 //  * @interface_method_impl{VBOXEXTPACKREG,pfnVMCreated}
 //  */
-// static DECLCALLBACK(int)  vboxSkeletonExtPack_VMCreated(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox, IMachine *pMachine);
+// static DECLCALLBACK(int)  vboxBusMouseExtPack_VMCreated(PCVBOXEXTPACKREG pThis, VBOXEXTPACK_IF_CS(IVirtualBox) *pVirtualBox, VBOXEXTPACK_IF_CS(IMachine) *pMachine);
 //
-
-/**
- * @interface_method_impl{VBOXEXTPACKREG,pfnVMConfigureVMM
- */
-static DECLCALLBACK(int)  vboxBusMouseExtPack_VMConfigureVMM(PCVBOXEXTPACKREG pThis, IConsole *pConsole, PVM pVM)
-{
-    /*
-     * Find the bus mouse module and tell PDM to load it.
-     * ASSUME /PDM/Devices exists.
-     */
-    char szPath[RTPATH_MAX];
-    int rc = g_pHlp->pfnFindModule(g_pHlp, "VBoxBusMouseR3", NULL, VBOXEXTPACKMODKIND_R3, szPath, sizeof(szPath), NULL);
-    if (RT_FAILURE(rc))
-        return rc;
-
-    PCFGMNODE pCfgRoot = CFGMR3GetRoot(pVM);
-    AssertReturn(pCfgRoot, VERR_INTERNAL_ERROR_3);
-
-    PCFGMNODE pCfgDevices = CFGMR3GetChild(pCfgRoot, "PDM/Devices");
-    AssertReturn(pCfgDevices, VERR_INTERNAL_ERROR_3);
-
-    PCFGMNODE pCfgMine;
-    rc = CFGMR3InsertNode(pCfgDevices, "VBoxBusMouse", &pCfgMine);
-    AssertRCReturn(rc, rc);
-    rc = CFGMR3InsertString(pCfgMine, "Path", szPath);
-    AssertRCReturn(rc, rc);
-
-    /*
-     * Tell PDM where to find the R0 and RC modules for the bus mouse device.
-     */
-#ifdef VBOX_WITH_RAW_MODE
-    rc = g_pHlp->pfnFindModule(g_pHlp, "VBoxBusMouseRC", NULL, VBOXEXTPACKMODKIND_RC, szPath, sizeof(szPath), NULL);
-    AssertRCReturn(rc, rc);
-    RTPathStripFilename(szPath);
-    rc = CFGMR3InsertString(pCfgMine, "RCSearchPath", szPath);
-    AssertRCReturn(rc, rc);
-#endif
-
-    rc = g_pHlp->pfnFindModule(g_pHlp, "VBoxBusMouseR0", NULL, VBOXEXTPACKMODKIND_R0, szPath, sizeof(szPath), NULL);
-    AssertRCReturn(rc, rc);
-    RTPathStripFilename(szPath);
-    rc = CFGMR3InsertString(pCfgMine, "R0SearchPath", szPath);
-    AssertRCReturn(rc, rc);
-
-    return VINF_SUCCESS;
-}
-
 // /**
-//  * @interface_method_impl{VBOXEXTPACKREG,pfnVMPowerOn}
+//  * @interface_method_impl{VBOXEXTPACKREG,pfnQueryObject}
 //  */
-// static DECLCALLBACK(int)  vboxSkeletonExtPack_VMPowerOn(PCVBOXEXTPACKREG pThis, IConsole *pConsole, PVM pVM);
-// /**
-//  * @interface_method_impl{VBOXEXTPACKREG,pfnVMPowerOff}
-//  */
-// static DECLCALLBACK(void) vboxSkeletonExtPack_VMPowerOff(PCVBOXEXTPACKREG pThis, IConsole *pConsole, PVM pVM);
-// /**
-//  * @interface_method_impl{VBOXEXTPACKREG,pfnVMPowerOff}
-//  */
-// static DECLCALLBACK(void) vboxSkeletonExtPack_QueryObject(PCVBOXEXTPACKREG pThis, PCRTUUID pObjectId);
+// static DECLCALLBACK(void) vboxBusMouseExtPack_QueryObject(PCVBOXEXTPACKREG pThis, PCRTUUID pObjectId);
 
 
 static const VBOXEXTPACKREG g_vboxBusMouseExtPackReg =
 {
     VBOXEXTPACKREG_VERSION,
+    /* .uVBoxFullVersion =  */  VBOX_FULL_VERSION,
     /* .pfnInstalled =      */  NULL,
     /* .pfnUninstall =      */  NULL,
     /* .pfnVirtualBoxReady =*/  NULL,
-    /* .pfnConsoleReady =   */  NULL,
     /* .pfnUnload =         */  NULL,
     /* .pfnVMCreated =      */  NULL,
-    /* .pfnVMConfigureVMM = */  vboxBusMouseExtPack_VMConfigureVMM,
-    /* .pfnVMPowerOn =      */  NULL,
-    /* .pfnVMPowerOff =     */  NULL,
     /* .pfnQueryObject =    */  NULL,
+    /* .pfnReserved1 =      */  NULL,
+    /* .pfnReserved2 =      */  NULL,
+    /* .pfnReserved3 =      */  NULL,
+    /* .pfnReserved4 =      */  NULL,
+    /* .pfnReserved5 =      */  NULL,
+    /* .pfnReserved6 =      */  NULL,
+    /* .u32Reserved7 =      */  0,
     VBOXEXTPACKREG_VERSION
 };
 
