@@ -1,11 +1,10 @@
+/* $Id: Framebuffer.cpp $ */
 /** @file
- *
- * VBox frontends: VBoxSDL (simple frontend based on SDL):
- * Implementation of VBoxSDLFB (SDL framebuffer) class
+ * VBoxSDL - Implementation of VBoxSDLFB (SDL framebuffer) class
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,10 +16,10 @@
  */
 
 #include <VBox/com/com.h>
+#include <VBox/com/array.h>
 #include <VBox/com/string.h>
 #include <VBox/com/Guid.h>
 #include <VBox/com/ErrorInfo.h>
-#include <VBox/com/EventQueue.h>
 #include <VBox/com/VirtualBox.h>
 
 #include <iprt/stream.h>
@@ -43,13 +42,20 @@ using namespace com;
 #include "Ico64x01.h"
 
 #if defined(RT_OS_WINDOWS) || defined(RT_OS_LINUX)
-#include <SDL_syswm.h>           /* for SDL_GetWMInfo() */
+# ifdef _MSC_VER
+#  pragma warning(push)
+#  pragma warning(disable: 4121) /* warning C4121: 'SDL_SysWMmsg' : alignment of a member was sensitive to packing*/
+# endif
+# include <SDL_syswm.h>           /* for SDL_GetWMInfo() */
+# ifdef _MSC_VER
+#  pragma warning(pop)
+# endif
 #endif
 
 #if defined(VBOX_WITH_XPCOM)
 NS_IMPL_THREADSAFE_ISUPPORTS1_CI(VBoxSDLFB, IFramebuffer)
 NS_DECL_CLASSINFO(VBoxSDLFB)
-NS_IMPL_THREADSAFE_ISUPPORTS1_CI(VBoxSDLFBOverlay, IFramebufferOverlay)
+NS_IMPL_THREADSAFE_ISUPPORTS2_CI(VBoxSDLFBOverlay, IFramebufferOverlay, IFramebuffer)
 NS_DECL_CLASSINFO(VBoxSDLFBOverlay)
 #endif
 
@@ -74,6 +80,20 @@ static RTNATIVETHREAD gSdlNativeThread = NIL_RTNATIVETHREAD; /**< the SDL thread
 // Constructor / destructor
 //
 
+VBoxSDLFB::VBoxSDLFB()
+{
+}
+
+HRESULT VBoxSDLFB::FinalConstruct()
+{
+    return 0;
+}
+
+void VBoxSDLFB::FinalRelease()
+{
+    return;
+}
+
 /**
  * SDL framebuffer constructor. It is called from the main
  * (i.e. SDL) thread. Therefore it is safe to use SDL calls
@@ -86,19 +106,17 @@ static RTNATIVETHREAD gSdlNativeThread = NIL_RTNATIVETHREAD; /**< the SDL thread
  * @param iFixedWidth    fixed SDL width (-1 means not set)
  * @param iFixedHeight   fixed SDL height (-1 means not set)
  */
-VBoxSDLFB::VBoxSDLFB(uint32_t uScreenId,
+HRESULT VBoxSDLFB::init(uint32_t uScreenId,
                      bool fFullscreen, bool fResizable, bool fShowSDLConfig,
                      bool fKeepHostRes, uint32_t u32FixedWidth,
-                     uint32_t u32FixedHeight, uint32_t u32FixedBPP)
+                     uint32_t u32FixedHeight, uint32_t u32FixedBPP,
+                     bool fUpdateImage)
 {
     int rc;
     LogFlow(("VBoxSDLFB::VBoxSDLFB\n"));
 
-#if defined (RT_OS_WINDOWS)
-    refcnt = 0;
-#endif
-
     mScreenId       = uScreenId;
+    mfUpdateImage   = fUpdateImage;
     mScreen         = NULL;
 #ifdef VBOX_WITH_SDL13
     mWindow         = 0;
@@ -119,8 +137,6 @@ VBoxSDLFB::VBoxSDLFB(uint32_t uScreenId,
     /* Start with standard screen dimensions. */
     mGuestXRes      = 640;
     mGuestYRes      = 480;
-    mPixelFormat    = FramebufferPixelFormat_Opaque;
-    mUsesGuestVRAM  = FALSE;
     mPtrVRAM        = NULL;
     mBitsPerPixel   = 0;
     mBytesPerLine   = 0;
@@ -131,12 +147,20 @@ VBoxSDLFB::VBoxSDLFB(uint32_t uScreenId,
     mLabelOffs      = 0;
 #endif
 
+    mfUpdates = false;
+
     rc = RTCritSectInit(&mUpdateLock);
     AssertMsg(rc == VINF_SUCCESS, ("Error from RTCritSectInit!\n"));
 
     resizeGuest();
     Assert(mScreen);
     mfInitialized = true;
+#ifdef RT_OS_WINDOWS
+    HRESULT hr = CoCreateFreeThreadedMarshaler(this, m_pUnkMarshaler.asOutParam());
+    Log(("CoCreateFreeThreadedMarshaler hr %08X\n", hr)); NOREF(hr);
+#endif
+
+    return 0;
 }
 
 VBoxSDLFB::~VBoxSDLFB()
@@ -168,7 +192,7 @@ bool VBoxSDLFB::init(bool fShowSDLConfig)
 
 #ifdef RT_OS_WINDOWS
     /* default to DirectX if nothing else set */
-    if (!RTEnvGet("SDL_VIDEODRIVER"))
+    if (!RTEnvExist("SDL_VIDEODRIVER"))
     {
         _putenv("SDL_VIDEODRIVER=directx");
 //        _putenv("SDL_VIDEODRIVER=windib");
@@ -283,54 +307,6 @@ STDMETHODIMP VBoxSDLFB::COMGETTER(Height)(ULONG *height)
     return S_OK;
 }
 
-/**
- * Lock the framebuffer (make its address immutable).
- *
- * @returns COM status code
- */
-STDMETHODIMP VBoxSDLFB::Lock()
-{
-    LogFlow(("VBoxSDLFB::Lock\n"));
-    RTCritSectEnter(&mUpdateLock);
-    return S_OK;
-}
-
-/**
- * Unlock the framebuffer.
- *
- * @returns COM status code
- */
-STDMETHODIMP VBoxSDLFB::Unlock()
-{
-    LogFlow(("VBoxSDLFB::Unlock\n"));
-    RTCritSectLeave(&mUpdateLock);
-    return S_OK;
-}
-
-/**
- * Return the framebuffer start address.
- *
- * @returns COM status code.
- * @param   address Pointer to result variable.
- */
-STDMETHODIMP VBoxSDLFB::COMGETTER(Address)(BYTE **address)
-{
-    LogFlow(("VBoxSDLFB::GetAddress\n"));
-    if (!address)
-        return E_INVALIDARG;
-
-    if (!mSurfVRAM)
-    {
-        /* That's actually rather bad. */
-        AssertMsgFailed(("mSurfVRAM is NULL!\n"));
-        return E_FAIL;
-    }
-
-    *address = (BYTE *) mSurfVRAM->pixels;
-    LogFlow(("VBoxSDL::GetAddress returning %p\n", *address));
-    return S_OK;
-}
-
 /**
  * Return the current framebuffer color depth.
  *
@@ -365,19 +341,11 @@ STDMETHODIMP VBoxSDLFB::COMGETTER(BytesPerLine)(ULONG *bytesPerLine)
     return S_OK;
 }
 
-STDMETHODIMP VBoxSDLFB::COMGETTER(PixelFormat) (ULONG *pixelFormat)
+STDMETHODIMP VBoxSDLFB::COMGETTER(PixelFormat) (BitmapFormat_T *pixelFormat)
 {
     if (!pixelFormat)
         return E_POINTER;
-    *pixelFormat = mPixelFormat;
-    return S_OK;
-}
-
-STDMETHODIMP VBoxSDLFB::COMGETTER(UsesGuestVRAM) (BOOL *usesGuestVRAM)
-{
-    if (!usesGuestVRAM)
-        return E_POINTER;
-    *usesGuestVRAM = mUsesGuestVRAM;
+    *pixelFormat = BitmapFormat_BGR;
     return S_OK;
 }
 
@@ -422,14 +390,39 @@ STDMETHODIMP VBoxSDLFB::COMGETTER(Overlay)(IFramebufferOverlay **aOverlay)
  * @returns COM status code.
  * @param   winId Handle of associated window.
  */
-STDMETHODIMP VBoxSDLFB::COMGETTER(WinId)(int64_t *winId)
+STDMETHODIMP VBoxSDLFB::COMGETTER(WinId)(LONG64 *winId)
 {
     if (!winId)
         return E_POINTER;
+#ifdef RT_OS_DARWIN
+    if (mWinId == NULL) /* (In case it failed the first time.) */
+        mWinId = (intptr_t)VBoxSDLGetDarwinWindowId();
+#endif
     *winId = mWinId;
     return S_OK;
 }
 
+STDMETHODIMP VBoxSDLFB::COMGETTER(Capabilities)(ComSafeArrayOut(FramebufferCapabilities_T, aCapabilities))
+{
+    if (ComSafeArrayOutIsNull(aCapabilities))
+        return E_POINTER;
+
+    com::SafeArray<FramebufferCapabilities_T> caps;
+
+    if (mfUpdateImage)
+    {
+        caps.resize(1);
+        caps[0] = FramebufferCapabilities_UpdateImage;
+    }
+    else
+    {
+        /* No caps to return. */
+    }
+
+    caps.detachTo(ComSafeArrayOutArg(aCapabilities));
+    return S_OK;
+}
+
 /**
  * Notify framebuffer of an update.
  *
@@ -465,8 +458,8 @@ STDMETHODIMP VBoxSDLFB::NotifyUpdate(ULONG x, ULONG y,
     event.user.code  = mScreenId;
     event.user.type  = SDL_USER_EVENT_UPDATERECT;
     // 16 bit is enough for coordinates
-    event.user.data1 = (void*)(x << 16 | y);
-    event.user.data2 = (void*)(w << 16 | h);
+    event.user.data1 = (void*)(uintptr_t)(x << 16 | y);
+    event.user.data2 = (void*)(uintptr_t)(w << 16 | h);
     PushNotifyUpdateEvent(&event);
 #else /* !VBOXSDL_WITH_X11 */
     update(x, y, w, h, true /* fGuestRelative */);
@@ -475,81 +468,89 @@ STDMETHODIMP VBoxSDLFB::NotifyUpdate(ULONG x, ULONG y,
     return S_OK;
 }
 
-/**
- * Request a display resize from the framebuffer.
- *
- * @returns COM status code.
- * @param   pixelFormat The requested pixel format.
- * @param   vram        Pointer to the guest VRAM buffer (can be NULL).
- * @param   bitsPerPixel Color depth in bits.
- * @param   bytesPerLine Size of a scanline in bytes.
- * @param   w           New display width in pixels.
- * @param   h           New display height in pixels.
- * @param   finished    Address of output flag whether the update
- *                      could be fully processed in this call (which
- *                      has to return immediately) or VBox should wait
- *                      for all call to the resize complete API before
- *                      continuing with display updates.
- */
-STDMETHODIMP VBoxSDLFB::RequestResize(ULONG aScreenId, ULONG pixelFormat, BYTE *vram,
-                                      ULONG bitsPerPixel, ULONG bytesPerLine,
-                                      ULONG w, ULONG h, BOOL *finished)
+STDMETHODIMP VBoxSDLFB::NotifyUpdateImage(ULONG aX,
+                                          ULONG aY,
+                                          ULONG aWidth,
+                                          ULONG aHeight,
+                                          ComSafeArrayIn(BYTE, aImage))
 {
-    LogFlowFunc (("w=%d, h=%d, pixelFormat=0x%08lX, vram=%p, "
-                  "bpp=%d, bpl=%d\n",
-                  w, h, pixelFormat, vram, bitsPerPixel, bytesPerLine));
+    LogFlow(("NotifyUpdateImage: %d,%d %dx%d\n", aX, aY, aWidth, aHeight));
 
-    /*
-     * SDL does not allow us to make this call from any other thread than
-     * the main thread (the one which initialized the video mode). So we
-     * have to send an event to the main SDL thread and tell VBox to wait.
-     */
-    if (!finished)
+    com::SafeArray<BYTE> image(ComSafeArrayInArg(aImage));
+
+    /* Copy to mSurfVRAM. */
+    SDL_Rect srcRect;
+    SDL_Rect dstRect;
+    srcRect.x = 0;
+    srcRect.y = 0;
+    srcRect.w = (uint16_t)aWidth;
+    srcRect.h = (uint16_t)aHeight;
+    dstRect.x = (int16_t)aX;
+    dstRect.y = (int16_t)aY;
+    dstRect.w = (uint16_t)aWidth;
+    dstRect.h = (uint16_t)aHeight;
+
+    const uint32_t Rmask = 0x00FF0000, Gmask = 0x0000FF00, Bmask = 0x000000FF, Amask = 0;
+    SDL_Surface *surfSrc = SDL_CreateRGBSurfaceFrom(image.raw(), aWidth, aHeight, 32, aWidth * 4,
+                                                    Rmask, Gmask, Bmask, Amask);
+    if (surfSrc)
     {
-        AssertMsgFailed(("RequestResize requires the finished flag!\n"));
-        return E_FAIL;
+        RTCritSectEnter(&mUpdateLock);
+        if (mfUpdates)
+            SDL_BlitSurface(surfSrc, &srcRect, mSurfVRAM, &dstRect);
+        RTCritSectLeave(&mUpdateLock);
+
+        SDL_FreeSurface(surfSrc);
     }
 
-    /*
-     * Optimize the case when the guest has changed only the VRAM ptr
-     * and the framebuffer uses the guest VRAM as the source bitmap.
-     */
-    if (   mGuestXRes    == w
-        && mGuestYRes    == h
-        && mPixelFormat  == pixelFormat
-        && mBitsPerPixel == bitsPerPixel
-        && mBytesPerLine == bytesPerLine
-        && mUsesGuestVRAM
-       )
+    return NotifyUpdate(aX, aY, aWidth, aHeight);
+}
+
+extern ComPtr<IDisplay> gpDisplay;
+
+STDMETHODIMP VBoxSDLFB::NotifyChange(ULONG aScreenId,
+                                     ULONG aXOrigin,
+                                     ULONG aYOrigin,
+                                     ULONG aWidth,
+                                     ULONG aHeight)
+{
+    LogRel(("NotifyChange: %d %d,%d %dx%d\n",
+            aScreenId, aXOrigin, aYOrigin, aWidth, aHeight));
+
+    ComPtr<IDisplaySourceBitmap> pSourceBitmap;
+    if (!mfUpdateImage)
+        gpDisplay->QuerySourceBitmap(aScreenId, pSourceBitmap.asOutParam());
+
+    RTCritSectEnter(&mUpdateLock);
+
+    /* Disable screen updates. */
+    mfUpdates = false;
+
+    if (mfUpdateImage)
     {
-        mfSameSizeRequested = true;
+        mGuestXRes   = aWidth;
+        mGuestYRes   = aHeight;
+        mPtrVRAM     = NULL;
+        mBitsPerPixel = 0;
+        mBytesPerLine = 0;
     }
     else
     {
-        mfSameSizeRequested = false;
+        /* Save the new bitmap. */
+        mpPendingSourceBitmap = pSourceBitmap;
     }
 
-    mGuestXRes   = w;
-    mGuestYRes   = h;
-    mPixelFormat = pixelFormat;
-    mPtrVRAM     = vram;
-    mBitsPerPixel = bitsPerPixel;
-    mBytesPerLine = bytesPerLine;
-    mUsesGuestVRAM = FALSE; /* yet */
+    RTCritSectLeave(&mUpdateLock);
 
     SDL_Event event;
     event.type       = SDL_USEREVENT;
-    event.user.type  = SDL_USER_EVENT_RESIZE;
+    event.user.type  = SDL_USER_EVENT_NOTIFYCHANGE;
     event.user.code  = mScreenId;
 
-    /* Try multiple times if necessary */
     PushSDLEventForSure(&event);
 
-    /* we want this request to be processed quickly, so yield the CPU */
     RTThreadYield();
 
-    *finished = false;
-
     return S_OK;
 }
 
@@ -564,6 +565,8 @@ STDMETHODIMP VBoxSDLFB::RequestResize(ULONG aScreenId, ULONG pixelFormat, BYTE *
  */
 STDMETHODIMP VBoxSDLFB::VideoModeSupported(ULONG width, ULONG height, ULONG bpp, BOOL *supported)
 {
+    RT_NOREF(bpp);
+
     if (!supported)
         return E_POINTER;
 
@@ -617,14 +620,89 @@ STDMETHODIMP VBoxSDLFB::SetVisibleRegion(BYTE *aRectangles, ULONG aCount)
     return S_OK;
 }
 
-STDMETHODIMP VBoxSDLFB::ProcessVHWACommand(BYTE *pCommand)
+STDMETHODIMP VBoxSDLFB::ProcessVHWACommand(BYTE *pCommand, LONG enmCmd, BOOL fGuestCmd)
 {
+    RT_NOREF(pCommand, enmCmd, fGuestCmd);
     return E_NOTIMPL;
 }
+
+STDMETHODIMP VBoxSDLFB::Notify3DEvent(ULONG uType, ComSafeArrayIn(BYTE, aData))
+{
+    RT_NOREF(uType); ComSafeArrayNoRef(aData);
+    return E_NOTIMPL;
+}
+
 //
 // Internal public methods
 //
 
+/* This method runs on the main SDL thread. */
+void VBoxSDLFB::notifyChange(ULONG aScreenId)
+{
+    /* Disable screen updates. */
+    RTCritSectEnter(&mUpdateLock);
+
+    if (!mfUpdateImage && mpPendingSourceBitmap.isNull())
+    {
+        /* Do nothing. Change event already processed. */
+        RTCritSectLeave(&mUpdateLock);
+        return;
+    }
+
+    /* Release the current bitmap and keep the pending one. */
+    mpSourceBitmap = mpPendingSourceBitmap;
+    mpPendingSourceBitmap.setNull();
+
+    RTCritSectLeave(&mUpdateLock);
+
+    if (mpSourceBitmap.isNull())
+    {
+        mPtrVRAM      = NULL;
+        mBitsPerPixel = 32;
+        mBytesPerLine = mGuestXRes * 4;
+    }
+    else
+    {
+        BYTE *pAddress = NULL;
+        ULONG ulWidth = 0;
+        ULONG ulHeight = 0;
+        ULONG ulBitsPerPixel = 0;
+        ULONG ulBytesPerLine = 0;
+        BitmapFormat_T bitmapFormat = BitmapFormat_Opaque;
+
+        mpSourceBitmap->QueryBitmapInfo(&pAddress,
+                                        &ulWidth,
+                                        &ulHeight,
+                                        &ulBitsPerPixel,
+                                        &ulBytesPerLine,
+                                        &bitmapFormat);
+
+        if (   mGuestXRes    == ulWidth
+            && mGuestYRes    == ulHeight
+            && mBitsPerPixel == ulBitsPerPixel
+            && mBytesPerLine == ulBytesPerLine
+            && mPtrVRAM == pAddress
+           )
+        {
+            mfSameSizeRequested = true;
+        }
+        else
+        {
+            mfSameSizeRequested = false;
+        }
+
+        mGuestXRes   = ulWidth;
+        mGuestYRes   = ulHeight;
+        mPtrVRAM     = pAddress;
+        mBitsPerPixel = ulBitsPerPixel;
+        mBytesPerLine = ulBytesPerLine;
+    }
+
+    resizeGuest();
+
+    gpDisplay->InvalidateAndUpdateScreen(aScreenId);
+}
+
 /**
  * Method that does the actual resize of the guest framebuffer and
  * then changes the SDL framebuffer setup.
@@ -635,42 +713,9 @@ void VBoxSDLFB::resizeGuest()
     AssertMsg(gSdlNativeThread == RTThreadNativeSelf(),
               ("Wrong thread! SDL is not threadsafe!\n"));
 
-    uint32_t Rmask, Gmask, Bmask, Amask = 0;
+    RTCritSectEnter(&mUpdateLock);
 
-    mUsesGuestVRAM = FALSE;
-
-    /* pixel characteristics. if we don't support the format directly, we will
-     * fallback to the indirect 32bpp buffer (mUsesGuestVRAM will remain
-     * FALSE) */
-    if (mPixelFormat == FramebufferPixelFormat_FOURCC_RGB)
-    {
-        switch (mBitsPerPixel)
-        {
-            case 16:
-            case 24:
-            case 32:
-                mUsesGuestVRAM = TRUE;
-                break;
-            default:
-                /* the fallback buffer is always 32bpp */
-                mBitsPerPixel = 32;
-                mBytesPerLine = mGuestXRes * 4;
-                break;
-        }
-    }
-    else
-    {
-        /* the fallback buffer is always RGB, 32bpp */
-        mPixelFormat = FramebufferPixelFormat_FOURCC_RGB;
-        mBitsPerPixel = 32;
-        mBytesPerLine = mGuestXRes * 4;
-    }
-
-    switch (mBitsPerPixel)
-    {
-        case 16: Rmask = 0x0000F800; Gmask = 0x000007E0; Bmask = 0x0000001F; break;
-        default: Rmask = 0x00FF0000; Gmask = 0x0000FF00; Bmask = 0x000000FF; break;
-    }
+    const uint32_t Rmask = 0x00FF0000, Gmask = 0x0000FF00, Bmask = 0x000000FF, Amask = 0;
 
     /* first free the current surface */
     if (mSurfVRAM)
@@ -679,36 +724,38 @@ void VBoxSDLFB::resizeGuest()
         mSurfVRAM = NULL;
     }
 
-    /* is the guest in a linear framebuffer mode we support? */
-    if (mUsesGuestVRAM)
+    if (mPtrVRAM)
     {
-        /* Create a source surface from guest VRAM. */
+        /* Create a source surface from the source bitmap. */
         mSurfVRAM = SDL_CreateRGBSurfaceFrom(mPtrVRAM, mGuestXRes, mGuestYRes, mBitsPerPixel,
                                              mBytesPerLine, Rmask, Gmask, Bmask, Amask);
-        LogRel(("mSurfVRAM from guest %d x %d\n", mGuestXRes,  mGuestYRes));
+        LogFlow(("VBoxSDL:: using the source bitmap\n"));
     }
     else
     {
-        /* Create a software surface for which SDL allocates the RAM */
         mSurfVRAM = SDL_CreateRGBSurface(SDL_SWSURFACE, mGuestXRes, mGuestYRes, mBitsPerPixel,
                                          Rmask, Gmask, Bmask, Amask);
-        LogRel(("mSurfVRAM from SDL %d x %d\n", mGuestXRes,  mGuestYRes));
+        LogFlow(("VBoxSDL:: using SDL_SWSURFACE\n"));
     }
     LogFlow(("VBoxSDL:: created VRAM surface %p\n", mSurfVRAM));
 
-    if (mfSameSizeRequested && mUsesGuestVRAM)
+    if (mfSameSizeRequested)
     {
-        /*
-         * Same size has been requested and the framebuffer still uses the guest VRAM.
-         * Reset the condition and return.
-         */
         mfSameSizeRequested = false;
         LogFlow(("VBoxSDL:: the same resolution requested, skipping the resize.\n"));
-        return;
+    }
+    else
+    {
+        /* now adjust the SDL resolution */
+        resizeSDL();
     }
 
-    /* now adjust the SDL resolution */
-    resizeSDL();
+    /* Enable screen updates. */
+    mfUpdates = true;
+
+    RTCritSectLeave(&mUpdateLock);
+
+    repaint();
 }
 
 /**
@@ -878,12 +925,14 @@ void VBoxSDLFB::resizeSDL(void)
     SDL_SysWMinfo info;
     SDL_VERSION(&info.version);
     if (SDL_GetWMInfo(&info))
-        mWinId = (LONG64) info.window;
+        mWinId = (intptr_t) info.window;
 # elif defined (RT_OS_LINUX)
     SDL_SysWMinfo info;
     SDL_VERSION(&info.version);
     if (SDL_GetWMInfo(&info))
         mWinId = (LONG64) info.info.x11.wmwindow;
+# elif defined(RT_OS_DARWIN)
+    mWinId = (intptr_t)VBoxSDLGetDarwinWindowId();
 # else
     /* XXX ignore this for other architectures */
 # endif
@@ -931,7 +980,6 @@ void VBoxSDLFB::resizeSDL(void)
             RTPrintf("Resized to %dx%d, screen surface type: %s\n", mScreen->w, mScreen->h,
                      ((mScreen->flags & SDL_HWSURFACE) == 0) ? "software" : "hardware");
     }
-    repaint();
 }
 
 /**
@@ -950,10 +998,21 @@ void VBoxSDLFB::update(int x, int y, int w, int h, bool fGuestRelative)
 #ifdef VBOXSDL_WITH_X11
     AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
 #endif
+    RTCritSectEnter(&mUpdateLock);
+    Log(("Updates %d, %d,%d %dx%d\n", mfUpdates, x, y, w, h));
+    if (!mfUpdates)
+    {
+        RTCritSectLeave(&mUpdateLock);
+        return;
+    }
+
     Assert(mScreen);
     Assert(mSurfVRAM);
     if (!mScreen || !mSurfVRAM)
+    {
+        RTCritSectLeave(&mUpdateLock);
         return;
+    }
 
     /* the source and destination rectangles */
     SDL_Rect srcRect;
@@ -1033,6 +1092,7 @@ void VBoxSDLFB::update(int x, int y, int w, int h, bool fGuestRelative)
     if (fPaintLabel)
         paintSecureLabel(0, 0, 0, 0, false);
 #endif
+    RTCritSectLeave(&mUpdateLock);
 }
 
 /**
@@ -1059,6 +1119,7 @@ void VBoxSDLFB::setFullscreen(bool fFullscreen)
     mfFullscreen = fFullscreen;
     /* only change the SDL resolution, do not touch the guest framebuffer */
     resizeSDL();
+    repaint();
 }
 
 /**
@@ -1098,6 +1159,7 @@ void VBoxSDLFB::getFullscreenGeometry(uint32_t *width, uint32_t *height)
 }
 
 #ifdef VBOX_SECURELABEL
+
 /**
  * Setup the secure labeling parameters
  *
@@ -1160,9 +1222,10 @@ void VBoxSDLFB::setSecureLabelColor(uint32_t colorFG, uint32_t colorBG)
  */
 void VBoxSDLFB::paintSecureLabel(int x, int y, int w, int h, bool fForce)
 {
-#ifdef VBOXSDL_WITH_X11
+    RT_NOREF(x, w, h);
+# ifdef VBOXSDL_WITH_X11
     AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
-#endif
+# endif
     /* only when the function is present */
     if (!pTTF_RenderUTF8_Solid)
         return;
@@ -1197,6 +1260,7 @@ void VBoxSDLFB::paintSecureLabel(int x, int y, int w, int h, bool fForce)
     /* make sure to update the screen */
     SDL_UpdateRect(mScreen, 0, 0, mScreen->w, mLabelHeight);
 }
+
 #endif /* VBOX_SECURELABEL */
 
 // IFramebufferOverlay
@@ -1358,6 +1422,7 @@ STDMETHODIMP VBoxSDLFBOverlay::COMSETTER(Visible)(BOOL visible)
  */
 STDMETHODIMP VBoxSDLFBOverlay::COMGETTER(Alpha)(ULONG *alpha)
 {
+    RT_NOREF(alpha);
     LogFlow(("VBoxSDLFBOverlay::GetAlpha\n"));
     return E_NOTIMPL;
 }
@@ -1370,25 +1435,11 @@ STDMETHODIMP VBoxSDLFBOverlay::COMGETTER(Alpha)(ULONG *alpha)
  */
 STDMETHODIMP VBoxSDLFBOverlay::COMSETTER(Alpha)(ULONG alpha)
 {
+    RT_NOREF(alpha);
     LogFlow(("VBoxSDLFBOverlay::SetAlpha\n"));
     return E_NOTIMPL;
 }
 
-/**
- * Returns the address of the framebuffer bits for writing to.
- *
- * @returns COM status code
- * @param   alpha Address of result buffer.
- */
-STDMETHODIMP VBoxSDLFBOverlay::COMGETTER(Address)(ULONG *address)
-{
-    LogFlow(("VBoxSDLFBOverlay::GetAddress\n"));
-    if (!address)
-        return E_INVALIDARG;
-    *address = (uintptr_t) mOverlayBits->pixels;
-    return S_OK;
-}
-
 /**
  * Returns the current colour depth.  In fact, this is always 32bpp.
  *
@@ -1415,7 +1466,7 @@ STDMETHODIMP VBoxSDLFBOverlay::COMGETTER(PixelFormat)(ULONG *pixelFormat)
     LogFlow(("VBoxSDLFBOverlay::GetPixelFormat\n"));
     if (!pixelFormat)
         return E_INVALIDARG;
-    *pixelFormat = FramebufferPixelFormat_FOURCC_RGB;
+    *pixelFormat = BitmapFormat_BGR;
     return S_OK;
 }
 
@@ -1545,7 +1596,7 @@ STDMETHODIMP VBoxSDLFBOverlay::NotifyUpdate(ULONG x, ULONG y,
  * Change the dimensions of the overlay.
  *
  * @returns COM status code
- * @param   pixelFormat Must be FramebufferPixelFormat_PixelFormatRGB32.
+ * @param   pixelFormat Must be BitmapFormat_BGR.
  * @param   vram        Must be NULL.
  * @param   lineSize    Ignored.
  * @param   w           New overlay width.
@@ -1556,7 +1607,8 @@ STDMETHODIMP VBoxSDLFBOverlay::RequestResize(ULONG aScreenId, ULONG pixelFormat,
                                              ULONG bitsPerPixel, ULONG bytesPerLine,
                                              ULONG w, ULONG h, BOOL *finished)
 {
-    AssertReturn(pixelFormat == FramebufferPixelFormat_FOURCC_RGB, E_INVALIDARG);
+    RT_NOREF(aScreenId, bytesPerLine, finished);
+    AssertReturn(pixelFormat == BitmapFormat_BGR, E_INVALIDARG);
     AssertReturn(vram == 0, E_INVALIDARG);
     AssertReturn(bitsPerPixel == 32, E_INVALIDARG);
     mOverlayWidth = w;
@@ -1585,9 +1637,9 @@ STDMETHODIMP VBoxSDLFBOverlay::RequestResize(ULONG aScreenId, ULONG pixelFormat,
  *
  * Basically, we support anything with 32bpp.
  */
-STDMETHODIMP VBoxSDLFBOverlay::VideoModeSupported(ULONG width, ULONG height, ULONG bpp,
-                                               BOOL *supported)
+STDMETHODIMP VBoxSDLFBOverlay::VideoModeSupported(ULONG width, ULONG height, ULONG bpp, BOOL *supported)
 {
+    RT_NOREF(width, height);
     if (!supported)
         return E_POINTER;
     if (bpp == 32)
