@@ -1,11 +1,10 @@
+/* $Id: VBoxFBOverlay.h $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * VBoxFrameBuffer Overly classes declarations
+ * VBox Qt GUI - VBoxFrameBuffer Overly classes declarations.
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,7 +16,8 @@
  */
 #ifndef __VBoxFBOverlay_h__
 #define __VBoxFBOverlay_h__
-#if defined (VBOX_GUI_USE_QGL) || defined(VBOX_WITH_VIDEOHWACCEL)
+
+#if defined(VBOX_GUI_USE_QGL) || defined(VBOX_WITH_VIDEOHWACCEL)
 
 /* Defines: */
 //#define VBOXQGL_PROF_BASE 1
@@ -26,25 +26,38 @@
 #define VBOXVHWA_ALLOW_PRIMARY_AND_OVERLAY_ONLY 1
 
 /* Qt includes: */
+#ifdef RT_OS_WINDOWS
+# include <iprt/win/windows.h> /* QGLWidget drags in Windows.h; -Wall forces us to use wrapper. */
+# include <iprt/stdint.h>      /* QGLWidget drags in stdint.h; -Wall forces us to use wrapper. */
+#endif
 #include <QGLWidget>
 
 /* GUI includes: */
 #include "UIDefs.h"
 #include "VBoxFBOverlayCommon.h"
+#include "runtime/UIFrameBuffer.h"
+#include "runtime/UIMachineView.h"
 
 /* COM includes: */
 #include "COMEnums.h"
 
+#include "CDisplay.h"
+
 /* Other VBox includes: */
 #include <iprt/assert.h>
 #include <iprt/critsect.h>
 #include <iprt/asm.h>
 #include <iprt/err.h>
+#include <iprt/list.h>
 #include <VBox/VBoxGL2D.h>
 #ifdef VBOXVHWA_PROFILE_FPS
 # include <iprt/stream.h>
 #endif /* VBOXVHWA_PROFILE_FPS */
 
+#ifndef S_FALSE
+# define S_FALSE ((HRESULT)1L)
+#endif
+
 #ifdef DEBUG_misha
 # define VBOXVHWA_PROFILE_FPS
 #endif /* DEBUG_misha */
@@ -76,7 +89,8 @@ private:
 class VBoxVHWASettings
 {
 public:
-    VBoxVHWASettings (CSession &session);
+    VBoxVHWASettings ();
+    void init(CSession &session);
 
     int fourccEnabledCount() const { return mFourccEnabledCount; }
     const uint32_t * fourccEnabledList() const { return mFourccEnabledList; }
@@ -262,10 +276,10 @@ public:
             switch (mFormat)
             {
                 case GL_BGRA_EXT:
-                    return FramebufferPixelFormat_FOURCC_RGB;
+                    return KBitmapFormat_BGR;
             }
         }
-        return FramebufferPixelFormat_Opaque;
+        return KBitmapFormat_Opaque;
     }
 
 private:
@@ -736,7 +750,7 @@ private:
 class VBoxVHWAHandleTable
 {
 public:
-    VBoxVHWAHandleTable(uint32_t initialSize);
+    VBoxVHWAHandleTable(uint32_t maxSize);
     ~VBoxVHWAHandleTable();
     uint32_t put(void * data);
     bool mapPut(uint32_t h, void * data);
@@ -1008,6 +1022,18 @@ class VBoxVHWASurfList
 public:
 
     VBoxVHWASurfList() : mCurrent(NULL) {}
+
+    void moveTo(VBoxVHWASurfList *pDst)
+    {
+        for (SurfList::iterator it = mSurfaces.begin();
+             it != mSurfaces.end(); it = mSurfaces.begin())
+        {
+            pDst->add((*it));
+        }
+
+        Assert(empty());
+    }
+
     void add(VBoxVHWASurfaceBase *pSurf)
     {
         VBoxVHWASurfList * pOld = pSurf->getComplexList();
@@ -1018,7 +1044,7 @@ public:
         mSurfaces.push_back(pSurf);
         pSurf->setComplexList(this);
     }
-
+/*
     void clear()
     {
         for (SurfList::iterator it = mSurfaces.begin();
@@ -1029,7 +1055,7 @@ public:
         mSurfaces.clear();
         mCurrent = NULL;
     }
-
+*/
     size_t size() const {return mSurfaces.size(); }
 
     void remove(VBoxVHWASurfaceBase *pSurf)
@@ -1070,7 +1096,12 @@ public:
     {
         VBoxVHWASurfaceBase * old = mSurfVGA;
         mSurfVGA = pVga;
-        mPrimary.clear();
+        if (!mPrimary.empty())
+        {
+            VBoxVHWASurfList *pNewList = new VBoxVHWASurfList();
+            mPrimary.moveTo(pNewList);
+            Assert(mPrimary.empty());
+        }
         if(pVga)
         {
             Assert(!pVga->getComplexList());
@@ -1188,14 +1219,12 @@ typedef struct VBOXVHWAFUNCCALLBACKINFO
 class VBoxVHWACommandElement
 {
 public:
-    VBoxVHWACommandElement() :
-        bNewEvent(false)
-    {}
-
-    void setVHWACmd(struct VBOXVHWACMD * pCmd)
+    void setVHWACmd(struct VBOXVHWACMD RT_UNTRUSTED_VOLATILE_GUEST *pCmd, int enmCmd, bool fGuestCmd)
     {
         mType = VBOXVHWA_PIPECMD_VHWA;
-        u.mpCmd = pCmd;
+        u.s.mpCmd = pCmd;
+        u.s.menmCmd = enmCmd;
+        u.s.mfGuestCmd = fGuestCmd;
     }
 
     void setPaintCmd(const QRect & aRect)
@@ -1210,177 +1239,47 @@ public:
         u.mFuncCallback = aOp;
     }
 
-    void setData(VBOXVHWA_PIPECMD_TYPE aType, void * pvData)
+    void setData(VBOXVHWA_PIPECMD_TYPE aType, void *pvData, int /*VBOXVHWACMD_TYPE*/ enmCmd, bool fGuestCmd = false)
     {
-        switch(aType)
+        switch (aType)
         {
         case VBOXVHWA_PIPECMD_PAINT:
-            setPaintCmd(*((QRect*)pvData));
+            setPaintCmd(*((QRect *)pvData));
             break;
         case VBOXVHWA_PIPECMD_VHWA:
-            setVHWACmd((struct VBOXVHWACMD *)pvData);
+            setVHWACmd((struct VBOXVHWACMD *)pvData, enmCmd, fGuestCmd);
             break;
         case VBOXVHWA_PIPECMD_FUNC:
             setFunc(*((VBOXVHWAFUNCCALLBACKINFO *)pvData));
             break;
         default:
-            Assert(0);
+            AssertFailed();
+            mType = (VBOXVHWA_PIPECMD_TYPE)0;
             break;
         }
     }
 
-    void setNewEvent(bool bNew) {bNewEvent = bNew;}
-    bool isNewEvent() const { return bNewEvent; }
-
     VBOXVHWA_PIPECMD_TYPE type() const {return mType;}
     const QRect & rect() const {return mRect;}
-    struct VBOXVHWACMD * vhwaCmd() const {return u.mpCmd;}
+    struct VBOXVHWACMD RT_UNTRUSTED_VOLATILE_GUEST *vhwaCmdPtr() const      { return u.s.mpCmd; }
+    int /*VBOXVHWACMD_TYPE*/                        vhwaCmdType() const     { return u.s.menmCmd; }
+    bool                                            vhwaIsGuestCmd() const  { return u.s.mfGuestCmd; }
     const VBOXVHWAFUNCCALLBACKINFO & func() const {return u.mFuncCallback; }
 
-    VBoxVHWACommandElement * mpNext;
+    RTLISTNODE ListNode;
 private:
     VBOXVHWA_PIPECMD_TYPE mType;
     union
     {
-        struct VBOXVHWACMD * mpCmd;
+        struct
+        {
+            struct VBOXVHWACMD RT_UNTRUSTED_VOLATILE_GUEST *mpCmd;
+            int /*VBOXVHWACMD_TYPE*/                        menmCmd;
+            bool                                            mfGuestCmd;
+        } s;
         VBOXVHWAFUNCCALLBACKINFO mFuncCallback;
-    }u;
+    } u;
     QRect                 mRect;
-    bool bNewEvent;
-};
-
-class VBoxVHWACommandElementPipe
-{
-public:
-    VBoxVHWACommandElementPipe() :
-        mpFirst(NULL),
-        mpLast(NULL)
-    {}
-
-    void put(VBoxVHWACommandElement *pCmd)
-    {
-        if (mpLast)
-        {
-            Assert(mpFirst);
-            mpLast->mpNext = pCmd;
-            mpLast = pCmd;
-        }
-        else
-        {
-            Assert(!mpFirst);
-            mpFirst = pCmd;
-            mpLast = pCmd;
-        }
-        pCmd->mpNext= NULL;
-
-    }
-
-    void setFrom(VBoxVHWACommandElementPipe *pOther)
-    {
-        mpFirst = pOther->detachList(&mpLast);
-    }
-
-    void set(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        mpFirst = pFirst;
-        mpLast = pLast;
-        if (mpLast)
-            mpLast->mpNext = NULL;
-    }
-
-    void prepend(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        if (!mpFirst)
-            set(pFirst, pLast);
-        else if (pLast)
-        {
-            pLast->mpNext = mpFirst;
-            mpFirst = pFirst;
-        }
-    }
-
-    void prependFrom(VBoxVHWACommandElementPipe *pOther)
-    {
-        VBoxVHWACommandElement *pFirst;
-        VBoxVHWACommandElement *pLast;
-        pFirst = pOther->detachList(&pLast);
-        prepend(pFirst, pLast);
-    }
-
-    void append(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        if (!mpLast)
-            set(pFirst, pLast);
-        else if (pLast)
-        {
-            mpLast->mpNext = pFirst;
-            mpLast = pLast;
-            pLast->mpNext = NULL;
-        }
-    }
-
-    VBoxVHWACommandElement * detachList(VBoxVHWACommandElement **ppLast)
-    {
-        if (mpLast)
-        {
-            VBoxVHWACommandElement * pHead = mpFirst;
-            if (ppLast)
-                *ppLast = mpLast;
-            mpFirst = NULL;
-            mpLast = NULL;
-            return pHead;
-        }
-        if (ppLast)
-            *ppLast = NULL;
-        return NULL;
-    }
-
-
-
-    const VBoxVHWACommandElement * contentsRo (const VBoxVHWACommandElement **ppLast) const
-    {
-        if (ppLast)
-            *ppLast = mpLast;
-        return mpFirst;
-    }
-
-    bool isEmpty() const { return !mpLast; }
-
-private:
-    VBoxVHWACommandElement *mpFirst;
-    VBoxVHWACommandElement *mpLast;
-};
-
-class VBoxVHWACommandElementStack
-{
-public:
-    VBoxVHWACommandElementStack() :
-        mpFirst(NULL) {}
-
-    void push(VBoxVHWACommandElement *pCmd)
-    {
-        pCmd->mpNext = mpFirst;
-        mpFirst = pCmd;
-    }
-
-    void pusha(VBoxVHWACommandElement *pFirst, VBoxVHWACommandElement *pLast)
-    {
-        pLast->mpNext = mpFirst;
-        mpFirst = pFirst;
-    }
-
-    VBoxVHWACommandElement * pop()
-    {
-        if(mpFirst)
-        {
-            VBoxVHWACommandElement * ret = mpFirst;
-            mpFirst = ret->mpNext;
-            return ret;
-        }
-        return NULL;
-    }
-private:
-    VBoxVHWACommandElement *mpFirst;
 };
 
 class VBoxVHWARefCounter
@@ -1424,20 +1323,17 @@ private:
     volatile uint32_t m_cRefs;
 };
 
-#define VBOXVHWACMDPIPEC_NEWEVENT      0x00000001
-#define VBOXVHWACMDPIPEC_COMPLETEEVENT 0x00000002
+class VBoxVHWAEntriesCache;
 class VBoxVHWACommandElementProcessor
 {
 public:
-    VBoxVHWACommandElementProcessor(QObject *pNotifyObject);
+    VBoxVHWACommandElementProcessor();
+    void init(QObject *pNotifyObject);
     ~VBoxVHWACommandElementProcessor();
-    void postCmd(VBOXVHWA_PIPECMD_TYPE aType, void * pvData, uint32_t flags);
-    bool completeCurrentEvent();
-    class VBoxVHWACommandElement * detachCmdList(class VBoxVHWACommandElement ** ppLast,
-            class VBoxVHWACommandElement * pFirst2Free, VBoxVHWACommandElement * pLast2Free);
-    void putBack(class VBoxVHWACommandElement * pFirst2Put, VBoxVHWACommandElement * pLast2Put,
-            class VBoxVHWACommandElement * pFirst2Free, VBoxVHWACommandElement * pLast2Free);
-    void reset(class VBoxVHWACommandElement ** ppHead, class VBoxVHWACommandElement ** ppTail);
+    void postCmd(VBOXVHWA_PIPECMD_TYPE aType, void *pvData, int /*VBOXVHWACMD_TYPE*/ enmCmdInt, bool fGuestCmd);
+    VBoxVHWACommandElement *getCmd();
+    void doneCmd();
+    void reset(CDisplay *pDisplay);
     void setNotifyObject(QObject *pNotifyObject);
     int loadExec (struct SSMHANDLE * pSSM, uint32_t u32Version, void *pvVRAM);
     void saveExec (struct SSMHANDLE * pSSM, void *pvVRAM);
@@ -1445,53 +1341,66 @@ public:
     void enable();
     void lock();
     void unlock();
-#ifdef DEBUG_misha
-    void checkConsistence(uint32_t cEvents2Submit = 0, const VBoxVHWACommandElementPipe *pPipe = NULL);
-#endif
 private:
     RTCRITSECT mCritSect;
-    VBoxVHWACommandElementPipe m_CmdPipe;
+    RTLISTNODE mCommandList;
     QObject *m_pNotifyObject;
     VBoxVHWARefCounter m_NotifyObjectRefs;
-    bool mbNewEvent;
-    bool mbProcessingList;
+    VBoxVHWACommandElement *mpCurCmd;
+    bool mbResetting;
     uint32_t mcDisabled;
-    VBoxVHWACommandElementStack mFreeElements;
-    VBoxVHWACommandElement mElementsBuffer[2048];
+    VBoxVHWAEntriesCache *m_pCmdEntryCache;
 };
 
 /* added to workaround this ** [VBox|UI] duplication */
 class VBoxFBSizeInfo
 {
 public:
-    VBoxFBSizeInfo() {}
-    template<class T> VBoxFBSizeInfo(T * fb) :
-        mPixelFormat(fb->pixelFormat()), mVRAM(fb->address()), mBitsPerPixel(fb->bitsPerPixel()),
-        mBytesPerLine(fb->bytesPerLine()), mWidth(fb->width()), mHeight(fb->height()),
-        mUsesGuestVram(fb->usesGuestVRAM()) {}
 
-    VBoxFBSizeInfo(ulong aPixelFormat, uchar *aVRAM,
-                     ulong aBitsPerPixel, ulong aBytesPerLine,
-                     ulong aWidth, ulong aHeight,
-                     bool bUsesGuestVram) :
-        mPixelFormat (aPixelFormat), mVRAM (aVRAM), mBitsPerPixel (aBitsPerPixel),
-        mBytesPerLine (aBytesPerLine), mWidth (aWidth), mHeight (aHeight),
+    VBoxFBSizeInfo() {}
+    template<class T> VBoxFBSizeInfo(T *pFb) :
+        m_visualState(pFb->visualState()),
+        mPixelFormat(pFb->pixelFormat()), mVRAM(pFb->address()), mBitsPerPixel(pFb->bitsPerPixel()),
+        mBytesPerLine(pFb->bytesPerLine()), mWidth(pFb->width()), mHeight(pFb->height()),
+        m_dScaleFactor(pFb->scaleFactor()), m_scaledSize(pFb->scaledSize()), m_fUseUnscaledHiDPIOutput(pFb->useUnscaledHiDPIOutput()),
+        mUsesGuestVram(true) {}
+
+    VBoxFBSizeInfo(UIVisualStateType visualState,
+                   ulong aPixelFormat, uchar *aVRAM,
+                   ulong aBitsPerPixel, ulong aBytesPerLine,
+                   ulong aWidth, ulong aHeight,
+                   double dScaleFactor, const QSize &scaledSize, bool fUseUnscaledHiDPIOutput,
+                   bool bUsesGuestVram) :
+        m_visualState(visualState),
+        mPixelFormat(aPixelFormat), mVRAM(aVRAM), mBitsPerPixel(aBitsPerPixel),
+        mBytesPerLine(aBytesPerLine), mWidth(aWidth), mHeight(aHeight),
+        m_dScaleFactor(dScaleFactor), m_scaledSize(scaledSize), m_fUseUnscaledHiDPIOutput(fUseUnscaledHiDPIOutput),
         mUsesGuestVram(bUsesGuestVram) {}
+
+    UIVisualStateType visualState() const { return m_visualState; }
     ulong pixelFormat() const { return mPixelFormat; }
     uchar *VRAM() const { return mVRAM; }
     ulong bitsPerPixel() const { return mBitsPerPixel; }
     ulong bytesPerLine() const { return mBytesPerLine; }
     ulong width() const { return mWidth; }
     ulong height() const { return mHeight; }
+    double scaleFactor() const { return m_dScaleFactor; }
+    QSize scaledSize() const { return m_scaledSize; }
+    bool useUnscaledHiDPIOutput() const { return m_fUseUnscaledHiDPIOutput; }
     bool usesGuestVram() const {return mUsesGuestVram;}
 
 private:
+
+    UIVisualStateType m_visualState;
     ulong mPixelFormat;
     uchar *mVRAM;
     ulong mBitsPerPixel;
     ulong mBytesPerLine;
     ulong mWidth;
     ulong mHeight;
+    double m_dScaleFactor;
+    QSize m_scaledSize;
+    bool m_fUseUnscaledHiDPIOutput;
     bool mUsesGuestVram;
 };
 
@@ -1511,22 +1420,22 @@ public:
     static void vhwaSaveExecVoid(struct SSMHANDLE * pSSM);
     static int vhwaLoadExec(VHWACommandList * pCmdList, struct SSMHANDLE * pSSM, uint32_t u32Version);
 
-    int vhwaSurfaceCanCreate(struct VBOXVHWACMD_SURF_CANCREATE *pCmd);
-    int vhwaSurfaceCreate(struct VBOXVHWACMD_SURF_CREATE *pCmd);
+    int vhwaSurfaceCanCreate(struct VBOXVHWACMD_SURF_CANCREATE RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceCreate(struct VBOXVHWACMD_SURF_CREATE RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
 #ifdef VBOX_WITH_WDDM
-    int vhwaSurfaceGetInfo(struct VBOXVHWACMD_SURF_GETINFO *pCmd);
+    int vhwaSurfaceGetInfo(struct VBOXVHWACMD_SURF_GETINFO RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
 #endif
-    int vhwaSurfaceDestroy(struct VBOXVHWACMD_SURF_DESTROY *pCmd);
-    int vhwaSurfaceLock(struct VBOXVHWACMD_SURF_LOCK *pCmd);
-    int vhwaSurfaceUnlock(struct VBOXVHWACMD_SURF_UNLOCK *pCmd);
-    int vhwaSurfaceBlt(struct VBOXVHWACMD_SURF_BLT *pCmd);
-    int vhwaSurfaceFlip(struct VBOXVHWACMD_SURF_FLIP *pCmd);
-    int vhwaSurfaceColorFill(struct VBOXVHWACMD_SURF_COLORFILL *pCmd);
-    int vhwaSurfaceOverlayUpdate(struct VBOXVHWACMD_SURF_OVERLAY_UPDATE *pCmf);
-    int vhwaSurfaceOverlaySetPosition(struct VBOXVHWACMD_SURF_OVERLAY_SETPOSITION *pCmd);
-    int vhwaSurfaceColorkeySet(struct VBOXVHWACMD_SURF_COLORKEY_SET *pCmd);
-    int vhwaQueryInfo1(struct VBOXVHWACMD_QUERYINFO1 *pCmd);
-    int vhwaQueryInfo2(struct VBOXVHWACMD_QUERYINFO2 *pCmd);
+    int vhwaSurfaceDestroy(struct VBOXVHWACMD_SURF_DESTROY RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceLock(struct VBOXVHWACMD_SURF_LOCK RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceUnlock(struct VBOXVHWACMD_SURF_UNLOCK RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceBlt(struct VBOXVHWACMD_SURF_BLT RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceFlip(struct VBOXVHWACMD_SURF_FLIP RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceColorFill(struct VBOXVHWACMD_SURF_COLORFILL RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceOverlayUpdate(struct VBOXVHWACMD_SURF_OVERLAY_UPDATE RT_UNTRUSTED_VOLATILE_GUEST *pCmf);
+    int vhwaSurfaceOverlaySetPosition(struct VBOXVHWACMD_SURF_OVERLAY_SETPOSITION RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaSurfaceColorkeySet(struct VBOXVHWACMD_SURF_COLORKEY_SET RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaQueryInfo1(struct VBOXVHWACMD_QUERYINFO1 RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
+    int vhwaQueryInfo2(struct VBOXVHWACMD_QUERYINFO2 RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
     int vhwaConstruct(struct VBOXVHWACMD_HH_CONSTRUCT *pCmd);
 
     void *vramBase() { return mpvVRAM; }
@@ -1630,7 +1539,8 @@ private:
     static int vhwaLoadOverlayData(VHWACommandList * pCmdList, struct SSMHANDLE * pSSM, uint32_t u32Version);
     static int vhwaLoadVHWAEnable(VHWACommandList * pCmdList);
 
-    void vhwaDoSurfaceOverlayUpdate(VBoxVHWASurfaceBase *pDstSurf, VBoxVHWASurfaceBase *pSrcSurf, struct VBOXVHWACMD_SURF_OVERLAY_UPDATE *pCmd);
+    void vhwaDoSurfaceOverlayUpdate(VBoxVHWASurfaceBase *pDstSurf, VBoxVHWASurfaceBase *pSrcSurf,
+                                    struct VBOXVHWACMD_SURF_OVERLAY_UPDATE RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
 #endif
 
     VBoxVHWADisplay mDisplay;
@@ -1798,7 +1708,8 @@ private:
 class VBoxQGLOverlay
 {
 public:
-    VBoxQGLOverlay(QWidget *pViewport, QObject *pPostEventObject, CSession * aSession, uint32_t id);
+    VBoxQGLOverlay();
+    void init(QWidget *pViewport, QObject *pPostEventObject, CSession * aSession, uint32_t id);
     ~VBoxQGLOverlay()
     {
         if (mpShareWgt)
@@ -1807,7 +1718,8 @@ public:
 
     void updateAttachment(QWidget *pViewport, QObject *pPostEventObject);
 
-    int onVHWACommand (struct VBOXVHWACMD * pCommand);
+    int onVHWACommand(struct VBOXVHWACMD RT_UNTRUSTED_VOLATILE_GUEST *pCommand,
+                      int /*VBOXVHWACMD_TYPE*/ enmCmdInt, bool fGuestCmd);
 
     void onVHWACommandEvent (QEvent * pEvent);
 
@@ -1818,32 +1730,14 @@ public:
     bool onNotifyUpdate (ULONG aX, ULONG aY,
                              ULONG aW, ULONG aH);
 
-    /**
-     * to be called on RequestResize framebuffer call
-     * @return true if the request was processed & should not be forwarded to the framebuffer
-     * false - otherwise */
-    bool onRequestResize (ULONG aScreenId, ULONG uPixelFormat,
-                          BYTE * pVRAM, ULONG uBitsPerPixel, ULONG uBytesPerLine,
-                          ULONG uWidth, ULONG uHeight,
-                          HRESULT *pResult,
-                          BOOL * pbFinished)
+    void onNotifyUpdateIgnore (ULONG aX, ULONG aY,
+                             ULONG aW, ULONG aH)
     {
-        Q_UNUSED(aScreenId);
-        Q_UNUSED(uPixelFormat);
-        Q_UNUSED(pVRAM);
-        Q_UNUSED(uBitsPerPixel);
-        Q_UNUSED(uBytesPerLine);
-        Q_UNUSED(uWidth);
-        Q_UNUSED(uHeight);
-        Q_UNUSED(pbFinished);
-
-        if (mCmdPipe.completeCurrentEvent())
-            return false;
-
-        /* TODO: more graceful resize handling */
-        *pResult = E_FAIL;
-
-        return true;
+        Q_UNUSED(aX);
+        Q_UNUSED(aY);
+        Q_UNUSED(aW);
+        Q_UNUSED(aH);
+        /* @todo: we actually should not miss notify updates, since we need to update the texture on it */
     }
 
     void onResizeEventPostprocess (const VBoxFBSizeInfo &re, const QPoint & topLeft);
@@ -1871,7 +1765,7 @@ public:
     int vhwaLoadExec (struct SSMHANDLE * pSSM, uint32_t u32Version);
     void vhwaSaveExec (struct SSMHANDLE * pSSM);
 private:
-    int vhwaSurfaceUnlock (struct VBOXVHWACMD_SURF_UNLOCK *pCmd);
+    int vhwaSurfaceUnlock (struct VBOXVHWACMD_SURF_UNLOCK RT_UNTRUSTED_VOLATILE_GUEST *pCmd);
 
     void repaintMain();
     void repaintOverlay()
@@ -1915,13 +1809,13 @@ private:
     void vboxSetGlOn (bool on);
     bool vboxGetGlOn() { return mGlOn; }
     bool vboxSynchGl();
-    void vboxDoVHWACmdExec(void *cmd);
+    void vboxDoVHWACmdExec(void RT_UNTRUSTED_VOLATILE_GUEST *pvCmd, int /*VBOXVHWACMD_TYPE*/ enmCmdInt, bool fGuestCmd);
     void vboxShowOverlay (bool show);
     void vboxDoCheckUpdateViewport();
-    void vboxDoVHWACmd (void *cmd);
+    void vboxDoVHWACmd(void RT_UNTRUSTED_VOLATILE_GUEST *pvCmd, int /*VBOXVHWACMD_TYPE*/ enmCmd, bool fGuestCmd);
     void addMainDirtyRect (const QRect & aRect);
     void vboxCheckUpdateOverlay (const QRect & rect);
-    VBoxVHWACommandElement * processCmdList (VBoxVHWACommandElement * pCmd, bool bFirst);
+    void processCmd (VBoxVHWACommandElement * pCmd);
 
     int vhwaConstruct (struct VBOXVHWACMD_HH_CONSTRUCT *pCmd);
 
@@ -1962,118 +1856,6 @@ private:
     uint32_t m_id;
 };
 
-/* these two additional class V, class R are to workaround the [VBox|UI] duplication,
- * @todo: remove them once VBox stuff is removed */
-template <class T, class V, class R>
-class VBoxOverlayFrameBuffer : public T
-{
-public:
-    VBoxOverlayFrameBuffer (V *pView, CSession * aSession, uint32_t id)
-        : T (pView),
-          mOverlay(pView->viewport(), pView, aSession, id),
-          mpView (pView)
-    {
-        /* sync with framebuffer */
-        mOverlay.onResizeEventPostprocess (VBoxFBSizeInfo(this), QPoint(mpView->contentsX(), mpView->contentsY()));
-    }
-
-    STDMETHOD(ProcessVHWACommand)(BYTE *pCommand)
-    {
-        return mOverlay.onVHWACommand ((struct VBOXVHWACMD*)pCommand);
-    }
-
-    void doProcessVHWACommand (QEvent * pEvent)
-    {
-        mOverlay.onVHWACommandEvent (pEvent);
-    }
-
-    STDMETHOD(RequestResize) (ULONG aScreenId, ULONG aPixelFormat,
-                              BYTE *aVRAM, ULONG aBitsPerPixel, ULONG aBytesPerLine,
-                              ULONG aWidth, ULONG aHeight,
-                              BOOL *aFinished)
-   {
-        HRESULT result;
-        if (mOverlay.onRequestResize (aScreenId, aPixelFormat,
-                aVRAM, aBitsPerPixel, aBytesPerLine,
-                aWidth, aHeight,
-                &result,
-                aFinished))
-        {
-            return result;
-        }
-        return T::RequestResize (aScreenId, aPixelFormat,
-                aVRAM, aBitsPerPixel, aBytesPerLine,
-                aWidth, aHeight,
-                aFinished);
-   }
-
-    STDMETHOD(NotifyUpdate) (ULONG aX, ULONG aY,
-                             ULONG aW, ULONG aH)
-    {
-        if (mOverlay.onNotifyUpdate (aX, aY, aW, aH))
-            return S_OK;
-        return T::NotifyUpdate (aX, aY, aW, aH);
-    }
-
-    STDMETHOD(VideoModeSupported) (ULONG uWidth, ULONG uHeight, ULONG uBPP,
-                                       BOOL *pbSupported)
-    {
-        /* TODO: tmp workaround: the lock should be moved to the calling code??
-         * otherwise we may end up calling a null View */
-        /* Todo: can we call VideoModeSupported with the lock held?
-         * if not we can introduce a ref counting for the mpView usage
-         * to ensure it stays alive till we need it*/
-        HRESULT hr = T::Lock();
-        HRESULT retHr = S_OK;
-        Assert(hr == S_OK);
-        if (SUCCEEDED(hr))
-        {
-            if (mpView)
-                retHr = T::VideoModeSupported(uWidth, uHeight, uBPP, pbSupported);
-            hr = T::Unlock();
-            Assert(hr == S_OK);
-        }
-        return retHr;
-    }
-
-    void resizeEvent (R *re)
-    {
-        T::resizeEvent (re);
-        mOverlay.onResizeEventPostprocess (VBoxFBSizeInfo(this),
-                QPoint(mpView->contentsX(), mpView->contentsY()));
-    }
-
-    void viewportResized (QResizeEvent * re)
-    {
-        mOverlay.onViewportResized (re);
-        T::viewportResized (re);
-    }
-
-    void viewportScrolled (int dx, int dy)
-    {
-        mOverlay.onViewportScrolled (QPoint(mpView->contentsX(), mpView->contentsY()));
-        T::viewportScrolled (dx, dy);
-    }
-
-    void setView(V * pView)
-    {
-        /* lock to ensure we do not collide with the EMT thread passing commands to us */
-        HRESULT hr = T::Lock();
-        Assert(hr == S_OK);
-        if (SUCCEEDED(hr))
-        {
-            T::setView(pView);
-            mpView = pView;
-            mOverlay.updateAttachment(pView ? pView->viewport() : NULL, pView);
-            hr = T::Unlock();
-            Assert(hr == S_OK);
-        }
-    }
-private:
-    VBoxQGLOverlay mOverlay;
-    V *mpView;
-};
-
-#endif
+#endif /* defined(VBOX_GUI_USE_QGL) || defined(VBOX_WITH_VIDEOHWACCEL) */
 
 #endif /* #ifndef __VBoxFBOverlay_h__ */
