@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2015 Oracle Corporation
+ * Copyright (C) 2010-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,12 +19,19 @@
 # include <precomp.h>
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
+/* Qt includes: */
+# include <QThread>
+# include <QMutex>
+
 /* GUI includes: */
 # include "UIMainEventListener.h"
+# include "VBoxGlobal.h"
 
 /* COM includes: */
 # include "COMEnums.h"
 # include "CEvent.h"
+# include "CEventSource.h"
+# include "CEventListener.h"
 # include "CVBoxSVCAvailabilityChangedEvent.h"
 # include "CVirtualBoxErrorInfo.h"
 # include "CMachineStateChangedEvent.h"
@@ -42,6 +49,7 @@
 # include "CKeyboardLedsChangedEvent.h"
 # include "CStateChangedEvent.h"
 # include "CNetworkAdapterChangedEvent.h"
+# include "CStorageDeviceChangedEvent.h"
 # include "CMediumChangedEvent.h"
 # include "CUSBDevice.h"
 # include "CUSBDeviceStateChangedEvent.h"
@@ -49,8 +57,116 @@
 # include "CRuntimeErrorEvent.h"
 # include "CCanShowWindowEvent.h"
 # include "CShowWindowEvent.h"
+# include "CProgressPercentageChangedEvent.h"
+# include "CProgressTaskCompletedEvent.h"
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
+
+/** Private QThread extension allowing to listen for Main events in separate thread.
+  * This thread listens for a Main events infinitely unless creator calls for #setShutdown. */
+class UIMainEventListeningThread : public QThread
+{
+    Q_OBJECT;
+
+public:
+
+    /** Constructs Main events listener thread redirecting events from @a source to @a listener. */
+    UIMainEventListeningThread(const CEventSource &source, const CEventListener &listener);
+    /** Destructs Main events listener thread. */
+    ~UIMainEventListeningThread();
+
+protected:
+
+    /** Contains the thread excution body. */
+    virtual void run() /* override */;
+
+    /** Returns whether the thread asked to shutdown prematurely. */
+    bool isShutdown() const;
+    /** Defines whether the thread asked to @a fShutdown prematurely. */
+    void setShutdown(bool fShutdown);
+
+private:
+
+    /** Holds the Main event source reference. */
+    CEventSource m_source;
+    /** Holds the Main event listener reference. */
+    CEventListener m_listener;
+
+    /** Holds the mutex instance which protects thread access. */
+    mutable QMutex m_mutex;
+    /** Holds whether the thread asked to shutdown prematurely. */
+    bool m_fShutdown;
+};
+
+
+/*********************************************************************************************************************************
+*   Class UIMainEventListeningThread implementation.                                                                             *
+*********************************************************************************************************************************/
+
+UIMainEventListeningThread::UIMainEventListeningThread(const CEventSource &source, const CEventListener &listener)
+    : m_source(source)
+    , m_listener(listener)
+    , m_fShutdown(false)
+{
+}
+
+UIMainEventListeningThread::~UIMainEventListeningThread()
+{
+    /* Make a request to shutdown: */
+    setShutdown(true);
+
+    /* And wait 30 seconds for run() to finish: */
+    wait(30000);
+}
+
+void UIMainEventListeningThread::run()
+{
+    /* Initialize COM: */
+    COMBase::InitializeCOM(false);
+
+    /* Copy source wrapper to this thread: */
+    CEventSource source = m_source;
+    /* Copy listener wrapper to this thread: */
+    CEventListener listener = m_listener;
+
+    /* While we are not in shutdown: */
+    while (!isShutdown())
+    {
+        /* Fetch the event from the queue: */
+        CEvent event = source.GetEvent(listener, 500);
+        if (!event.isNull())
+        {
+            /* Process the event and tell the listener: */
+            listener.HandleEvent(event);
+            if (event.GetWaitable())
+                source.EventProcessed(listener, event);
+        }
+    }
+
+    /* Cleanup COM: */
+    COMBase::CleanupCOM();
+}
+
+bool UIMainEventListeningThread::isShutdown() const
+{
+    m_mutex.lock();
+    bool fShutdown = m_fShutdown;
+    m_mutex.unlock();
+    return fShutdown;
+}
+
+void UIMainEventListeningThread::setShutdown(bool fShutdown)
+{
+    m_mutex.lock();
+    m_fShutdown = fShutdown;
+    m_mutex.unlock();
+}
+
+
+/*********************************************************************************************************************************
+*   Class UIMainEventListener implementation.                                                                                    *
+*********************************************************************************************************************************/
+
 UIMainEventListener::UIMainEventListener()
 {
     /* Register meta-types for required enums. */
@@ -64,8 +180,30 @@ UIMainEventListener::UIMainEventListener()
     qRegisterMetaType<KGuestMonitorChangedEventType>("KGuestMonitorChangedEventType");
 }
 
+void UIMainEventListener::registerSource(const CEventSource &source, const CEventListener &listener)
+{
+    /* Make sure source and listener are valid: */
+    AssertReturnVoid(!source.isNull());
+    AssertReturnVoid(!listener.isNull());
+
+    /* Create thread for passed source: */
+    m_threads << new UIMainEventListeningThread(source, listener);
+    /* And start it: */
+    m_threads.last()->start();
+}
+
+void UIMainEventListener::unregisterSources()
+{
+    /* Wipe out the threads: */
+    qDeleteAll(m_threads);
+}
+
 STDMETHODIMP UIMainEventListener::HandleEvent(VBoxEventType_T /* type */, IEvent *pEvent)
 {
+    /* Try to acquire COM cleanup protection token first: */
+    if (!vboxGlobal().comTokenTryLockForRead())
+        return S_OK;
+
     CEvent event(pEvent);
     // printf("Event received: %d\n", event.GetType());
     switch (event.GetType())
@@ -181,6 +319,12 @@ STDMETHODIMP UIMainEventListener::HandleEvent(VBoxEventType_T /* type */, IEvent
             emit sigNetworkAdapterChange(es.GetNetworkAdapter());
             break;
         }
+        case KVBoxEventType_OnStorageDeviceChanged:
+        {
+            CStorageDeviceChangedEvent es(pEvent);
+            emit sigStorageDeviceChange(es.GetStorageDevice(), es.GetRemoved(), es.GetSilent());
+            break;
+        }
         case KVBoxEventType_OnMediumChanged:
         {
             CMediumChangedEvent es(pEvent);
@@ -249,20 +393,39 @@ STDMETHODIMP UIMainEventListener::HandleEvent(VBoxEventType_T /* type */, IEvent
         {
             CShowWindowEvent es(pEvent);
             /* Has to be done in place to give an answer: */
-            LONG64 winId = es.GetWinId();
+            qint64 winId = es.GetWinId();
             if (winId != 0)
                 break; /* Already set by some listener. */
             emit sigShowWindow(winId);
             es.SetWinId(winId);
             break;
         }
-//        case KVBoxEventType_OnSerialPortChanged:
-//        case KVBoxEventType_OnParallelPortChanged:
-//        case KVBoxEventType_OnStorageControllerChanged:
-//        case KVBoxEventType_OnCPUChange:
+        case KVBoxEventType_OnAudioAdapterChanged:
+        {
+            emit sigAudioAdapterChange();
+            break;
+        }
+        case KVBoxEventType_OnProgressPercentageChanged:
+        {
+            CProgressPercentageChangedEvent es(pEvent);
+            emit sigProgressPercentageChange(es.GetProgressId(), (int)es.GetPercent());
+            break;
+        }
+        case KVBoxEventType_OnProgressTaskCompleted:
+        {
+            CProgressTaskCompletedEvent es(pEvent);
+            emit sigProgressTaskComplete(es.GetProgressId());
+            break;
+        }
 
         default: break;
     }
+
+    /* Unlock COM cleanup protection token: */
+    vboxGlobal().comTokenUnlock();
+
     return S_OK;
 }
 
+#include "UIMainEventListener.moc"
+
