@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2013 Oracle Corporation
+ * Copyright (C) 2010-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -20,25 +20,29 @@
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 /* Qt includes: */
-# include <QDesktopWidget>
 # include <QMenu>
 # include <QTimer>
+# ifdef VBOX_WS_WIN
+#  include <QWindow>
+# endif
 
 /* GUI includes: */
 # include "VBoxGlobal.h"
+# include "UIDesktopWidgetWatchdog.h"
 # include "UIExtraDataManager.h"
 # include "UISession.h"
 # include "UIActionPoolRuntime.h"
 # include "UIMachineLogicFullscreen.h"
 # include "UIMachineWindowFullscreen.h"
 # include "UIMachineView.h"
-# include "UIFrameBuffer.h"
-# include "UIMachineDefs.h"
-# include "UIMiniToolBar.h"
-# ifdef Q_WS_MAC
+# if   defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+#  include "UIMachineDefs.h"
+#  include "UIMiniToolBar.h"
+# elif defined(VBOX_WS_MAC)
+#  include "UIFrameBuffer.h"
 #  include "VBoxUtils-darwin.h"
 #  include "UICocoaApplication.h"
-# endif /* Q_WS_MAC */
+# endif /* VBOX_WS_MAC */
 
 /* COM includes: */
 # include "CSnapshot.h"
@@ -48,16 +52,21 @@
 
 UIMachineWindowFullscreen::UIMachineWindowFullscreen(UIMachineLogic *pMachineLogic, ulong uScreenId)
     : UIMachineWindow(pMachineLogic, uScreenId)
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
     , m_pMiniToolBar(0)
-#endif /* !Q_WS_MAC */
-#ifdef Q_WS_MAC
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+#ifdef VBOX_WS_MAC
     , m_fIsInFullscreenTransition(false)
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
+    , m_fWasMinimized(false)
+#ifdef VBOX_WS_X11
+    , m_fIsMinimizationRequested(false)
+    , m_fIsMinimized(false)
+#endif
 {
 }
 
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
 void UIMachineWindowFullscreen::handleNativeNotification(const QString &strNativeNotificationName)
 {
     /* Make sure this method is only used for ML and next: */
@@ -65,7 +74,7 @@ void UIMachineWindowFullscreen::handleNativeNotification(const QString &strNativ
 
     /* Log all arrived notifications: */
     LogRel(("UIMachineWindowFullscreen::handleNativeNotification: Notification '%s' received.\n",
-            strNativeNotificationName.toAscii().constData()));
+            strNativeNotificationName.toLatin1().constData()));
 
     /* Handle 'NSWindowWillEnterFullScreenNotification' notification: */
     if (strNativeNotificationName == "NSWindowWillEnterFullScreenNotification")
@@ -81,6 +90,8 @@ void UIMachineWindowFullscreen::handleNativeNotification(const QString &strNativ
         m_fIsInFullscreenTransition = false;
         LogRel(("UIMachineWindowFullscreen::handleNativeNotification: "
                 "Native fullscreen mode entered, notifying listener...\n"));
+        /* Update console's display viewport and 3D overlay: */
+        machineView()->updateViewport();
         emit sigNotifyAboutNativeFullscreenDidEnter();
     }
     /* Handle 'NSWindowWillExitFullScreenNotification' notification: */
@@ -97,6 +108,8 @@ void UIMachineWindowFullscreen::handleNativeNotification(const QString &strNativ
         m_fIsInFullscreenTransition = false;
         LogRel(("UIMachineWindowFullscreen::handleNativeNotification: "
                 "Native fullscreen mode exited, notifying listener...\n"));
+        /* Update console's display viewport and 3D overlay: */
+        machineView()->updateViewport();
         emit sigNotifyAboutNativeFullscreenDidExit();
     }
     /* Handle 'NSWindowDidFailToEnterFullScreenNotification' notification: */
@@ -109,9 +122,9 @@ void UIMachineWindowFullscreen::handleNativeNotification(const QString &strNativ
         emit sigNotifyAboutNativeFullscreenFailToEnter();
     }
 }
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
 void UIMachineWindowFullscreen::sltMachineStateChanged()
 {
     /* Call to base-class: */
@@ -120,9 +133,30 @@ void UIMachineWindowFullscreen::sltMachineStateChanged()
     /* Update mini-toolbar: */
     updateAppearanceOf(UIVisualElement_MiniToolBar);
 }
-#endif /* !Q_WS_MAC */
 
-#ifdef Q_WS_MAC
+void UIMachineWindowFullscreen::sltRevokeWindowActivation()
+{
+#ifdef VBOX_WS_X11
+    // WORKAROUND:
+    // We could be asked to minimize already, but just
+    // not yet executed that order to current moment.
+    if (m_fIsMinimizationRequested)
+        return;
+#endif
+
+    /* Make sure window is visible: */
+    if (!isVisible() || isMinimized())
+        return;
+
+    /* Revoke stolen activation: */
+#ifdef VBOX_WS_X11
+    raise();
+#endif /* VBOX_WS_X11 */
+    activateWindow();
+}
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+
+#ifdef VBOX_WS_MAC
 void UIMachineWindowFullscreen::sltEnterNativeFullscreen(UIMachineWindow *pMachineWindow)
 {
     /* Make sure this slot is called only under ML and next: */
@@ -171,36 +205,16 @@ void UIMachineWindowFullscreen::sltExitNativeFullscreen(UIMachineWindow *pMachin
         && darwinIsInFullscreenMode(this))
         darwinToggleFullscreenMode(this);
 }
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-void UIMachineWindowFullscreen::sltRevokeFocus()
+void UIMachineWindowFullscreen::sltShowMinimized()
 {
-    /* Make sure window is visible: */
-    if (!isVisible())
-        return;
+#ifdef VBOX_WS_X11
+    /* Remember that we are asked to minimize: */
+    m_fIsMinimizationRequested = true;
+#endif
 
-#if   defined(Q_WS_WIN)
-    /* Revoke stolen focus: */
-    m_pMachineView->setFocus();
-#elif defined(Q_WS_MAC) || defined(Q_WS_X11)
-    /* Revoke stolen activation: */
-    activateWindow();
-#endif /* Q_WS_MAC || Q_WS_X11 */
-}
-
-void UIMachineWindowFullscreen::showMinimized()
-{
-#ifdef Q_WS_X11
-    /* If there is mini-toolbar: */
-    if (m_pMiniToolBar)
-    {
-        /* Minimize it first: */
-        m_pMiniToolBar->showMinimized();
-    }
-#endif /* Q_WS_X11 */
-
-    /* Call to base-class: */
-    UIMachineWindow::showMinimized();
+    showMinimized();
 }
 
 void UIMachineWindowFullscreen::prepareVisualState()
@@ -215,12 +229,12 @@ void UIMachineWindowFullscreen::prepareVisualState()
     centralWidget()->setAutoFillBackground(true);
     setAutoFillBackground(true);
 
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
     /* Prepare mini-toolbar: */
     prepareMiniToolbar();
-#endif /* !Q_WS_MAC */
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
 
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
     /* Native fullscreen stuff on ML and next: */
     if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
     {
@@ -244,11 +258,11 @@ void UIMachineWindowFullscreen::prepareVisualState()
                                                                        UIMachineWindow::handleNativeNotification);
         UICocoaApplication::instance()->registerToNotificationOfWindow("NSWindowDidFailToEnterFullScreenNotification", this,
                                                                        UIMachineWindow::handleNativeNotification);
-    }
-#endif /* Q_WS_MAC */
+}
+#endif /* VBOX_WS_MAC */
 }
 
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
 void UIMachineWindowFullscreen::prepareMiniToolbar()
 {
     /* Make sure mini-toolbar is not restricted: */
@@ -256,22 +270,28 @@ void UIMachineWindowFullscreen::prepareMiniToolbar()
         return;
 
     /* Create mini-toolbar: */
-    m_pMiniToolBar = new UIRuntimeMiniToolBar(this,
-                                              GeometryType_Full,
-                                              gEDataManager->miniToolbarAlignment(vboxGlobal().managedVMUuid()),
-                                              gEDataManager->autoHideMiniToolbar(vboxGlobal().managedVMUuid()));
-    m_pMiniToolBar->addMenus(actionPool()->menus());
-    connect(m_pMiniToolBar, SIGNAL(sigMinimizeAction()), this, SLOT(showMinimized()), Qt::QueuedConnection);
-    connect(m_pMiniToolBar, SIGNAL(sigExitAction()),
-            actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen), SLOT(trigger()));
-    connect(m_pMiniToolBar, SIGNAL(sigCloseAction()),
-            actionPool()->action(UIActionIndex_M_Application_S_Close), SLOT(trigger()));
-    connect(m_pMiniToolBar, SIGNAL(sigNotifyAboutFocusStolen()),
-            this, SLOT(sltRevokeFocus()), Qt::QueuedConnection);
+    m_pMiniToolBar = new UIMiniToolBar(this,
+                                       GeometryType_Full,
+                                       gEDataManager->miniToolbarAlignment(vboxGlobal().managedVMUuid()),
+                                       gEDataManager->autoHideMiniToolbar(vboxGlobal().managedVMUuid()),
+                                       screenId());
+    AssertPtrReturnVoid(m_pMiniToolBar);
+    {
+        /* Configure mini-toolbar: */
+        m_pMiniToolBar->addMenus(actionPool()->menus());
+        connect(m_pMiniToolBar, SIGNAL(sigMinimizeAction()),
+                this, SLOT(sltShowMinimized()), Qt::QueuedConnection);
+        connect(m_pMiniToolBar, SIGNAL(sigExitAction()),
+                actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen), SLOT(trigger()));
+        connect(m_pMiniToolBar, SIGNAL(sigCloseAction()),
+                actionPool()->action(UIActionIndex_M_Application_S_Close), SLOT(trigger()));
+        connect(m_pMiniToolBar, SIGNAL(sigNotifyAboutWindowActivationStolen()),
+                this, SLOT(sltRevokeWindowActivation()), Qt::QueuedConnection);
+    }
 }
-#endif /* !Q_WS_MAC */
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
 
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
 void UIMachineWindowFullscreen::cleanupMiniToolbar()
 {
     /* Make sure mini-toolbar was created: */
@@ -284,11 +304,11 @@ void UIMachineWindowFullscreen::cleanupMiniToolbar()
     delete m_pMiniToolBar;
     m_pMiniToolBar = 0;
 }
-#endif /* !Q_WS_MAC */
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
 
 void UIMachineWindowFullscreen::cleanupVisualState()
 {
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
     /* Native fullscreen stuff on ML and next: */
     if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
     {
@@ -299,12 +319,12 @@ void UIMachineWindowFullscreen::cleanupVisualState()
         UICocoaApplication::instance()->unregisterFromNotificationOfWindow("NSWindowDidExitFullScreenNotification", this);
         UICocoaApplication::instance()->unregisterFromNotificationOfWindow("NSWindowDidFailToEnterFullScreenNotification", this);
     }
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
     /* Cleanup mini-toolbar: */
     cleanupMiniToolbar();
-#endif /* !Q_WS_MAC */
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
 
     /* Call to base-class: */
     UIMachineWindow::cleanupVisualState();
@@ -312,26 +332,30 @@ void UIMachineWindowFullscreen::cleanupVisualState()
 
 void UIMachineWindowFullscreen::placeOnScreen()
 {
-    /* Get corresponding screen: */
-    const int iScreen = qobject_cast<UIMachineLogicFullscreen*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
-    /* And corresponding working area: */
-    const QRect workingArea = QApplication::desktop()->screenGeometry(iScreen);
-
-    /* Move to the appropriate position: */
-    move(workingArea.topLeft());
-
-#ifdef Q_WS_MAC
     /* Make sure this window has fullscreen logic: */
     UIMachineLogicFullscreen *pFullscreenLogic = qobject_cast<UIMachineLogicFullscreen*>(machineLogic());
     AssertPtrReturnVoid(pFullscreenLogic);
 
-    /* Resize to the appropriate size on Lion and previous: */
+    /* Get corresponding host-screen: */
+    const int iHostScreen = pFullscreenLogic->hostScreenForGuestScreen(m_uScreenId);
+    /* And corresponding working area: */
+    const QRect workingArea = gpDesktop->screenGeometry(iHostScreen);
+    Q_UNUSED(workingArea);
+
+#if defined(VBOX_WS_MAC)
+
+    /* Move window to the appropriate position: */
+    move(workingArea.topLeft());
+
+    /* Resize window to the appropriate size on Lion and previous: */
     if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
         resize(workingArea.size());
-    /* Resize to the appropriate size on ML and next
-     * only if that screen has no own user-space: */
+    /* Resize window to the appropriate size on ML and next if it's screen has no own user-space: */
     else if (!pFullscreenLogic->screensHaveSeparateSpaces() && m_uScreenId != 0)
         resize(workingArea.size());
+    /* Resize the window if we are already in the full screen mode. This covers cases like host-resolution changes while in full screen mode: */
+    else if(darwinIsInFullscreenMode(this))
+        resize(workingArea.size());
     else
     {
         /* Load normal geometry first of all: */
@@ -342,153 +366,174 @@ void UIMachineWindowFullscreen::placeOnScreen()
             const UIFrameBuffer *pFrameBuffer = uisession()->frameBuffer(m_uScreenId);
             geo = QRect(QPoint(0, 0), QSize(pFrameBuffer->width(), pFrameBuffer->height()).boundedTo(workingArea.size()));
         }
-        /* If frame-buffer size is null => use default size: */
+        /* If normal geometry still null => use default size: */
         if (geo.isNull())
             geo = QRect(QPoint(0, 0), QSize(800, 600).boundedTo(workingArea.size()));
         /* Move window to the center of working-area: */
         geo.moveCenter(workingArea.center());
-        setGeometry(geo);
+        VBoxGlobal::setTopLevelGeometry(this, geo);
     }
-#else /* !Q_WS_MAC */
-    /* Resize to the appropriate size: */
+
+#elif defined(VBOX_WS_WIN)
+
+    /* Map window onto required screen: */
+    windowHandle()->setScreen(qApp->screens().at(iHostScreen));
+    /* Set appropriate window size: */
     resize(workingArea.size());
-#endif /* !Q_WS_MAC */
+
+#elif defined(VBOX_WS_X11)
+
+    /* Determine whether we should use the native full-screen mode: */
+    const bool fUseNativeFullScreen = VBoxGlobal::supportsFullScreenMonitorsProtocolX11() &&
+                                      !gEDataManager->legacyFullscreenModeRequested();
+    if (fUseNativeFullScreen)
+    {
+        /* Tell recent window managers which host-screen this window should be mapped to: */
+        VBoxGlobal::setFullScreenMonitorX11(this, pFullscreenLogic->hostScreenForGuestScreen(m_uScreenId));
+    }
+
+    /* Set appropriate window geometry: */
+    resize(workingArea.size());
+    move(workingArea.topLeft());
+
+#else
+
+# warning "port me"
+
+#endif
 }
 
 void UIMachineWindowFullscreen::showInNecessaryMode()
 {
-    /* Make sure this window has fullscreen logic: */
+    /* Make sure window has fullscreen logic: */
     UIMachineLogicFullscreen *pFullscreenLogic = qobject_cast<UIMachineLogicFullscreen*>(machineLogic());
     AssertPtrReturnVoid(pFullscreenLogic);
 
-#ifdef Q_WS_MAC
-    /* ML and next using native stuff: */
-    const bool fSupportsNativeFullScreen = vboxGlobal().osRelease() > MacOSXRelease_Lion;
-#endif /* Q_WS_MAC */
+#if defined(VBOX_WS_MAC)
 
-    /* Make sure this window should be shown and mapped to some host-screen: */
+    /* If window shouldn't be shown or mapped to some host-screen: */
     if (!uisession()->isScreenVisible(m_uScreenId) ||
         !pFullscreenLogic->hasHostScreenForGuestScreen(m_uScreenId))
     {
-#ifndef Q_WS_MAC
-        /* If there is mini-toolbar: */
-        if (m_pMiniToolBar)
-        {
-            /* Just hide mini-toolbar: */
-            m_pMiniToolBar->hide();
-        }
-#endif /* !Q_WS_MAC */
         /* Hide window: */
         hide();
-        return;
+    }
+    /* If window should be shown and mapped to some host-screen: */
+    else
+    {
+        /* Make sure window have appropriate geometry: */
+        placeOnScreen();
+
+        /* Simple show() for ML and next, showFullScreen() otherwise: */
+        if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
+            show();
+        else
+            showFullScreen();
+
+        /* Adjust machine-view size if necessary: */
+        adjustMachineViewSize();
+
+        /* Make sure machine-view have focus: */
+        m_pMachineView->setFocus();
     }
 
-    /* Make sure this window is not minimized: */
-    if (isMinimized())
-        return;
+#elif defined(VBOX_WS_WIN)
 
-#ifdef Q_WS_X11
-    /* On X11 calling placeOnScreen() is only needed for legacy window managers
-     * which we do not test, so this is 'best effort' code. With window managers which
-     * support the _NET_WM_FULLSCREEN_MONITORS protocol this would interfere unreliable. */
-    const bool fSupportsNativeFullScreen = VBoxGlobal::supportsFullScreenMonitorsProtocolX11() &&
-                                           !gEDataManager->legacyFullscreenModeRequested();
-    if (!fSupportsNativeFullScreen)
-        placeOnScreen();
-#else /* !Q_WS_X11 */
-    /* Make sure this window is maximized and placed on valid screen: */
-    placeOnScreen();
-#endif /* !Q_WS_X11 */
+    /* If window shouldn't be shown or mapped to some host-screen: */
+    if (!uisession()->isScreenVisible(m_uScreenId) ||
+        !pFullscreenLogic->hasHostScreenForGuestScreen(m_uScreenId))
+    {
+        /* Remember whether the window was minimized: */
+        if (isMinimized())
+            m_fWasMinimized = true;
 
-#ifdef Q_WS_MAC
-    /* ML and next using native stuff, so we can call for simple show(),
-     * Lion and previous using Qt stuff, so we should call for showFullScreen(): */
-    if (fSupportsNativeFullScreen)
-        show();
+        /* Hide window and reset it's state to NONE: */
+        setWindowState(Qt::WindowNoState);
+        hide();
+    }
+    /* If window should be shown and mapped to some host-screen: */
     else
+    {
+        /* Check whether window was minimized: */
+        const bool fWasMinimized = isMinimized() && isVisible();
+        /* And reset it's state in such case before exposing: */
+        if (fWasMinimized)
+            setWindowState(Qt::WindowNoState);
+
+        /* Make sure window have appropriate geometry: */
+        placeOnScreen();
+
+        /* Show window: */
         showFullScreen();
-#else /* !Q_WS_MAC */
-    /* Show in fullscreen mode: */
-    showFullScreen();
-#endif /* !Q_WS_MAC */
 
-#ifdef Q_WS_X11
-    if (fSupportsNativeFullScreen)
-    {
-        /* Tell recent window managers which screen this window should be mapped to.
-         * Apparently some window managers will not respond to requests for
-         * unmapped windows, so do this *after* the call to showFullScreen(). */
-        VBoxGlobal::setFullScreenMonitorX11(this, pFullscreenLogic->hostScreenForGuestScreen(m_uScreenId));
+        /* Restore minimized state if necessary: */
+        if (m_fWasMinimized || fWasMinimized)
+        {
+            m_fWasMinimized = false;
+            QMetaObject::invokeMethod(this, "showMinimized", Qt::QueuedConnection);
+        }
+
+        /* Adjust machine-view size if necessary: */
+        adjustMachineViewSize();
+
+        /* Make sure machine-view have focus: */
+        m_pMachineView->setFocus();
     }
+
+#elif defined(VBOX_WS_X11)
+
+    /* If window shouldn't be shown or mapped to some host-screen: */
+    if (!uisession()->isScreenVisible(m_uScreenId) ||
+        !pFullscreenLogic->hasHostScreenForGuestScreen(m_uScreenId))
+    {
+        /* Remember whether the window was minimized: */
+        if (isMinimized())
+            m_fWasMinimized = true;
+
+        /* Hide window and reset it's state to NONE: */
+        setWindowState(Qt::WindowNoState);
+        hide();
+    }
+    /* If window should be shown and mapped to some host-screen: */
     else
     {
-        /* On X11 calling placeOnScreen() is only needed for legacy window managers
-         * which we do not test, so this is 'best effort' code. With window managers which
-         * support the _NET_WM_FULLSCREEN_MONITORS protocol this would interfere unreliable. */
+        /* Check whether window was minimized: */
+        const bool fWasMinimized = isMinimized() && isVisible();
+        /* And reset it's state in such case before exposing: */
+        if (fWasMinimized)
+            setWindowState(Qt::WindowNoState);
+
+        /* Show window: */
+        showFullScreen();
+
+        /* Make sure window have appropriate geometry: */
         placeOnScreen();
-    }
-#endif /* Q_WS_X11 */
 
-    /* Adjust machine-view size if necessary: */
-    adjustMachineViewSize();
+        /* Restore full-screen state after placeOnScreen() call: */
+        setWindowState(Qt::WindowFullScreen);
 
-#ifndef Q_WS_MAC
-    /* If there is mini-toolbar: */
-    if (m_pMiniToolBar)
-    {
-# if   defined(Q_WS_WIN)
-        /* Just show mini-toolbar: */
-        m_pMiniToolBar->show();
-# elif defined(Q_WS_X11)
-        /* Allow mini-toolbar to be located on full-screen area: */
-        m_pMiniToolBar->showFullScreen();
-        /* On modern window managers: */
-        if (fSupportsNativeFullScreen)
+        /* Restore minimized state if necessary: */
+        if (m_fWasMinimized || fWasMinimized)
         {
-            /* We also can map mini-toolbar directly on corresponding machine-window: */
-            VBoxGlobal::setFullScreenMonitorX11(m_pMiniToolBar, pFullscreenLogic->hostScreenForGuestScreen(m_uScreenId));
+            m_fWasMinimized = false;
+            QMetaObject::invokeMethod(this, "showMinimized", Qt::QueuedConnection);
         }
-        /* Make sure mini-toolbar is always on top of machine-window: */
-        VBoxGlobal::setTransientFor(m_pMiniToolBar, this);
-# endif /* Q_WS_X11 */
-    }
-#endif /* !Q_WS_MAC */
 
-    /* Make sure machine-view have focus: */
-    m_pMachineView->setFocus();
+        /* Adjust machine-view size if necessary: */
+        adjustMachineViewSize();
+
+        /* Make sure machine-view have focus: */
+        m_pMachineView->setFocus();
+    }
+
+#else
+
+# warning "port me"
+
+#endif
 }
 
-void UIMachineWindowFullscreen::adjustMachineViewSize()
-{
-    /* Call to base-class: */
-    UIMachineWindow::adjustMachineViewSize();
-
-#ifndef Q_WS_MAC
-    /* If mini-toolbar present: */
-    if (m_pMiniToolBar)
-    {
-        /* Make sure this window has fullscreen logic: */
-        const UIMachineLogicFullscreen *pFullscreenLogic = qobject_cast<UIMachineLogicFullscreen*>(machineLogic());
-        AssertPtrReturnVoid(pFullscreenLogic);
-
-        /* Which host-screen should that machine-window located on? */
-        const int iHostScreen = pFullscreenLogic->hostScreenForGuestScreen(m_uScreenId);
-
-#ifndef Q_WS_X11
-        /* Move mini-toolbar into appropriate place: */
-        m_pMiniToolBar->adjustGeometry(iHostScreen);
-#else /* Q_WS_X11 */
-        /* On modern WMs we are mapping mini-toolbar to corresponding host-screen directly. */
-        const bool fSupportsNativeFullScreen = VBoxGlobal::supportsFullScreenMonitorsProtocolX11() &&
-                                               !gEDataManager->legacyFullscreenModeRequested();
-        /* Adjust mini-toolbar and move into appropriate place if necessary: */
-        m_pMiniToolBar->adjustGeometry(fSupportsNativeFullScreen ? -1 : iHostScreen);
-#endif /* Q_WS_X11 */
-    }
-#endif /* !Q_WS_MAC */
-}
-
-#ifndef Q_WS_MAC
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
 void UIMachineWindowFullscreen::updateAppearanceOf(int iElement)
 {
     /* Call to base-class: */
@@ -497,6 +542,7 @@ void UIMachineWindowFullscreen::updateAppearanceOf(int iElement)
     /* Update mini-toolbar: */
     if (iElement & UIVisualElement_MiniToolBar)
     {
+        /* If there is a mini-toolbar: */
         if (m_pMiniToolBar)
         {
             /* Get snapshot(s): */
@@ -511,5 +557,61 @@ void UIMachineWindowFullscreen::updateAppearanceOf(int iElement)
         }
     }
 }
-#endif /* !Q_WS_MAC */
+#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+
+#ifdef VBOX_WS_X11
+void UIMachineWindowFullscreen::changeEvent(QEvent *pEvent)
+{
+    switch (pEvent->type())
+    {
+        case QEvent::WindowStateChange:
+        {
+            /* Watch for window state changes: */
+            QWindowStateChangeEvent *pChangeEvent = static_cast<QWindowStateChangeEvent*>(pEvent);
+            LogRel2(("GUI: UIMachineWindowFullscreen::changeEvent: Window state changed from %d to %d\n",
+                     (int)pChangeEvent->oldState(), (int)windowState()));
+            if (   windowState() == Qt::WindowMinimized
+                && pChangeEvent->oldState() == Qt::WindowNoState
+                && !m_fIsMinimized)
+            {
+                /* Mark window minimized, isMinimized() is not enough due to Qt5vsX11 fight: */
+                LogRel2(("GUI: UIMachineWindowFullscreen::changeEvent: Window minimized\n"));
+                m_fIsMinimized = true;
+            }
+            else
+            if (   windowState() == Qt::WindowNoState
+                && pChangeEvent->oldState() == Qt::WindowMinimized
+                && m_fIsMinimized)
+            {
+                /* Mark window restored, and do manual restoring with showInNecessaryMode(): */
+                LogRel2(("GUI: UIMachineWindowFullscreen::changeEvent: Window restored\n"));
+                m_fIsMinimized = false;
+                /* Remember that we no more asked to minimize: */
+                m_fIsMinimizationRequested = false;
+                showInNecessaryMode();
+            }
+            break;
+        }
+        default:
+            break;
+    }
+
+    /* Call to base-class: */
+    UIMachineWindow::changeEvent(pEvent);
+}
+#endif /* VBOX_WS_X11 */
+
+#ifdef VBOX_WS_WIN
+void UIMachineWindowFullscreen::showEvent(QShowEvent *pEvent)
+{
+    /* Expose workaround again,
+     * Qt devs will never fix that it seems.
+     * This time they forget to set 'Mapped'
+     * attribute for initially frame-less window. */
+    setAttribute(Qt::WA_Mapped);
+
+    /* Call to base-class: */
+    UIMachineWindow::showEvent(pEvent);
+}
+#endif /* VBOX_WS_WIN */
 
