@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2013 Oracle Corporation
+ * Copyright (C) 2012-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -28,6 +28,7 @@
 # include <QPropertyAnimation>
 # include <QScrollBar>
 # include <QTimer>
+# include <QDrag>
 
 /* GUI includes: */
 # include "UIGChooser.h"
@@ -44,6 +45,7 @@
 # include "UIWizardNewVM.h"
 # include "UISelectorWindow.h"
 # include "UIVirtualBoxEventHandler.h"
+# include "UIModalWindowManager.h"
 
 /* COM includes: */
 # include "CVirtualBox.h"
@@ -52,12 +54,13 @@
 
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
+/* Qt includes: */
 #include <QParallelAnimationGroup>
 
-
 /* Type defs: */
 typedef QSet<QString> UIStringSet;
 
+
 UIGChooserModel::UIGChooserModel(UIGChooser *pParent)
     : QObject(pParent)
     , m_pChooser(pParent)
@@ -460,6 +463,7 @@ void UIGChooserModel::indentRoot(UIGChooserItem *pNewRootItem)
     m_pRightRoot->resize(root()->geometry().size());
 
     /* Indent root: */
+    root()->setRoot(false);
     m_rootStack << pNewRootItem;
     root()->setRoot(true);
     m_pAfterSlidingFocus = root()->items().first();
@@ -496,6 +500,7 @@ void UIGChooserModel::unindentRoot()
     /* Unindent root: */
     m_pAfterSlidingFocus = root();
     m_rootStack.removeLast();
+    root()->setRoot(true);
 
     /* Slide root: */
     slideRoot(false);
@@ -666,9 +671,13 @@ void UIGChooserModel::sltMachineRegistered(QString strId, bool fRegistered)
             /* And update model: */
             updateNavigation();
             updateLayout();
-            setCurrentItem(mainRoot()->searchForItem(machine.GetName(),
-                                                     UIGChooserItemSearchFlag_Machine |
-                                                     UIGChooserItemSearchFlag_ExactName));
+            /* Change current-item only if VM was created from the GUI side: */
+            if (strId == m_strLastCreatedMachineId)
+            {
+                setCurrentItem(mainRoot()->searchForItem(machine.GetName(),
+                                                         UIGChooserItemSearchFlag_Machine |
+                                                         UIGChooserItemSearchFlag_ExactName));
+            }
         }
     }
     /* Existing VM unregistered? */
@@ -739,6 +748,9 @@ void UIGChooserModel::sltSlidingComplete()
     /* We are no more sliding: */
     m_fSliding = false;
 
+    /* Update root geometry: */
+    root()->updateGeometry();
+
     /* Update model: */
     cleanupGroupTree();
     updateNavigation();
@@ -877,12 +889,27 @@ void UIGChooserModel::sltCreateNewMachine()
     if (pGroup)
         strGroupName = pGroup->fullName();
 
-    /* Start the new vm wizard: */
-    UISafePointerWizard pWizard = new UIWizardNewVM(&vboxGlobal().selectorWnd(), strGroupName);
+    /* Lock the action preventing cascade calls: */
+    actionPool()->action(UIActionIndexST_M_Machine_S_New)->setEnabled(false);
+    actionPool()->action(UIActionIndexST_M_Group_S_New)->setEnabled(false);
+
+    /* Use the "safe way" to open stack of Mac OS X Sheets: */
+    QWidget *pWizardParent = windowManager().realParentWindow(m_pChooser->selector());
+    UISafePointerWizardNewVM pWizard = new UIWizardNewVM(pWizardParent, strGroupName);
+    windowManager().registerNewParent(pWizard, pWizardParent);
     pWizard->prepare();
-    pWizard->exec();
+
+    /* Execute wizard and store created VM Id
+     * on success for current-item handling: */
+    if (pWizard->exec() == QDialog::Accepted)
+        m_strLastCreatedMachineId = pWizard->createdMachineId();
+
     if (pWizard)
         delete pWizard;
+
+    /* Unlock the action allowing further calls: */
+    actionPool()->action(UIActionIndexST_M_Machine_S_New)->setEnabled(true);
+    actionPool()->action(UIActionIndexST_M_Group_S_New)->setEnabled(true);
 }
 
 void UIGChooserModel::sltGroupSelectedMachines()
@@ -1090,9 +1117,10 @@ void UIGChooserModel::sltStartScrolling()
     QGraphicsView *pView = scene()->views()[0];
     QScrollBar *pVerticalScrollBar = pView->verticalScrollBar();
 
-    /* Request still valid? */
+    /* Convert mouse position to view co-ordinates: */
     QPoint mousePos = pView->mapFromGlobal(QCursor::pos());
-    if (mousePos.y() < m_iScrollingTokenSize)
+    /* Mouse position is at the top of view? */
+    if (mousePos.y() < m_iScrollingTokenSize && mousePos.y() > 0)
     {
         int iValue = mousePos.y();
         if (!iValue) iValue = 1;
@@ -1105,7 +1133,8 @@ void UIGChooserModel::sltStartScrolling()
             QTimer::singleShot(10, this, SLOT(sltStartScrolling()));
         }
     }
-    else if (mousePos.y() > pView->height() - m_iScrollingTokenSize)
+    /* Mouse position is at the bottom of view? */
+    else if (mousePos.y() > pView->height() - m_iScrollingTokenSize && mousePos.y() < pView->height())
     {
         int iValue = pView->height() - mousePos.y();
         if (!iValue) iValue = 1;
@@ -1125,11 +1154,6 @@ void UIGChooserModel::sltCurrentDragObjectDestroyed()
     root()->resetDragToken();
 }
 
-void UIGChooserModel::sltActionHovered(QAction *pAction)
-{
-    emit sigShowStatusMessage(pAction->statusTip());
-}
-
 void UIGChooserModel::sltEraseLookupTimer()
 {
     m_pLookupTimer->stop();
@@ -1228,9 +1252,6 @@ void UIGChooserModel::prepareContextMenu()
     m_pContextMenuMachine->addSeparator();
     m_pContextMenuMachine->addAction(actionPool()->action(UIActionIndexST_M_Machine_S_SortParent));
 
-    connect(m_pContextMenuGroup, SIGNAL(hovered(QAction*)), this, SLOT(sltActionHovered(QAction*)));
-    connect(m_pContextMenuMachine, SIGNAL(hovered(QAction*)), this, SLOT(sltActionHovered(QAction*)));
-
     connect(actionPool()->action(UIActionIndexST_M_Group_S_New), SIGNAL(triggered()),
             this, SLOT(sltCreateNewMachine()));
     connect(actionPool()->action(UIActionIndexST_M_Machine_S_New), SIGNAL(triggered()),
@@ -1370,9 +1391,13 @@ bool UIGChooserModel::eventFilter(QObject *pWatched, QEvent *pEvent)
         /* Context-menu handler: */
         case QEvent::GraphicsSceneContextMenu:
             return processContextMenuEvent(static_cast<QGraphicsSceneContextMenuEvent*>(pEvent));
-        /* Drag&drop scroll-event handler: */
+        /* Drag&drop scroll-event (drag-move) handler: */
         case QEvent::GraphicsSceneDragMove:
             return processDragMoveEvent(static_cast<QGraphicsSceneDragDropEvent*>(pEvent));
+        /* Drag&drop scroll-event (drag-leave) handler: */
+        case QEvent::GraphicsSceneDragLeave:
+            return processDragLeaveEvent(static_cast<QGraphicsSceneDragDropEvent*>(pEvent));
+        default: break; /* Shut up MSC */
     }
 
     /* Call to base-class: */
@@ -1580,6 +1605,7 @@ bool UIGChooserModel::processContextMenuEvent(QGraphicsSceneContextMenuEvent *pE
                             return true;
                         }
                     }
+                    RT_FALL_THRU();
                     case UIGChooserItemType_Machine:
                     {
                         /* Machine context menu for other Group/Machine cases: */
@@ -1610,6 +1636,7 @@ bool UIGChooserModel::processContextMenuEvent(QGraphicsSceneContextMenuEvent *pE
                             return true;
                         }
                     }
+                    RT_FALL_THRU();
                     case UIGChooserItemType_Machine:
                     {
                         /* Machine context menu for other Group/Machine cases: */
@@ -1647,8 +1674,6 @@ void UIGChooserModel::popupContextMenu(UIGraphicsSelectorContextMenuType type, Q
             break;
         }
     }
-    /* Clear status-bar: */
-    emit sigClearStatusMessage();
 }
 
 bool UIGChooserModel::processDragMoveEvent(QGraphicsSceneDragDropEvent *pEvent)
@@ -1675,6 +1700,19 @@ bool UIGChooserModel::processDragMoveEvent(QGraphicsSceneDragDropEvent *pEvent)
     return false;
 }
 
+bool UIGChooserModel::processDragLeaveEvent(QGraphicsSceneDragDropEvent *pEvent)
+{
+    /* Event object is not required here: */
+    Q_UNUSED(pEvent);
+
+    /* Make sure to stop scrolling as drag-leave event happened: */
+    if (m_fIsScrollingInProgress)
+        m_fIsScrollingInProgress = false;
+
+    /* Pass event: */
+    return false;
+}
+
 void UIGChooserModel::loadGroupTree()
 {
     /* Add all the approved machines we have into the group-tree: */
@@ -1696,27 +1734,27 @@ void UIGChooserModel::addMachineIntoTheTree(const CMachine &machine, bool fMakeI
     AssertReturnVoid(!machine.isNull());
 
     /* Which VM we are loading: */
-    LogRelFlow(("UIGChooserModel: Loading VM with ID={%s}...\n", machine.GetId().toAscii().constData()));
+    LogRelFlow(("UIGChooserModel: Loading VM with ID={%s}...\n", machine.GetId().toUtf8().constData()));
     /* Is that machine accessible? */
     if (machine.GetAccessible())
     {
         /* VM is accessible: */
         QString strName = machine.GetName();
-        LogRelFlow(("UIGChooserModel:  VM {%s} is accessible.\n", strName.toAscii().constData()));
+        LogRelFlow(("UIGChooserModel:  VM {%s} is accessible.\n", strName.toUtf8().constData()));
         /* Which groups passed machine attached to? */
         QVector<QString> groups = machine.GetGroups();
         QStringList groupList = groups.toList();
         QString strGroups = groupList.join(", ");
-        LogRelFlow(("UIGChooserModel:  VM {%s} has groups: {%s}.\n", strName.toAscii().constData(),
-                                                                     strGroups.toAscii().constData()));
+        LogRelFlow(("UIGChooserModel:  VM {%s} has groups: {%s}.\n", strName.toUtf8().constData(),
+                                                                     strGroups.toUtf8().constData()));
         foreach (QString strGroup, groups)
         {
             /* Remove last '/' if any: */
             if (strGroup.right(1) == "/")
                 strGroup.truncate(strGroup.size() - 1);
             /* Create machine-item with found group-item as parent: */
-            LogRelFlow(("UIGChooserModel:   Creating item for VM {%s} in group {%s}.\n", strName.toAscii().constData(),
-                                                                                         strGroup.toAscii().constData()));
+            LogRelFlow(("UIGChooserModel:   Creating item for VM {%s} in group {%s}.\n", strName.toUtf8().constData(),
+                                                                                         strGroup.toUtf8().constData()));
             createMachineItem(machine, getGroupItem(strGroup, mainRoot(), fMakeItVisible));
         }
         /* Update group definitions: */
@@ -1726,7 +1764,7 @@ void UIGChooserModel::addMachineIntoTheTree(const CMachine &machine, bool fMakeI
     else
     {
         /* VM is accessible: */
-        LogRelFlow(("UIGChooserModel:  VM {%s} is inaccessible.\n", machine.GetId().toAscii().constData()));
+        LogRelFlow(("UIGChooserModel:  VM {%s} is inaccessible.\n", machine.GetId().toUtf8().constData()));
         /* Create machine-item with main-root group-item as parent: */
         createMachineItem(machine, mainRoot());
     }
