@@ -1,12 +1,10 @@
 /* $Id: UIHostComboEditor.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * VirtualBox Qt extensions: UIHostComboEditor class implementation
+ * VBox Qt GUI - VirtualBox Qt extensions: UIHostComboEditor class implementation.
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,61 +15,157 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* Qt includes: */
-#include <QApplication>
-#include <QStyleOption>
-#include <QStylePainter>
-#include <QKeyEvent>
-#include <QTimer>
+# include <QApplication>
+# include <QKeyEvent>
+# include <QStyleOption>
+# include <QStylePainter>
+# include <QTimer>
+# ifdef VBOX_WS_X11
+#  include <QX11Info>
+# endif /* VBOX_WS_X11 */
 
 /* GUI includes: */
-#include "UIHostComboEditor.h"
-#include "VBoxGlobal.h"
-#include "QIToolButton.h"
-#include "UIIconPool.h"
+# include "UIHostComboEditor.h"
+# include "UIExtraDataManager.h"
+# include "UIIconPool.h"
+# include "VBoxGlobal.h"
+# include "QIToolButton.h"
+# ifdef VBOX_WS_MAC
+#  include "UICocoaApplication.h"
+#  include "VBoxUtils-darwin.h"
+# endif /* VBOX_WS_MAC */
 
-#ifdef Q_WS_WIN
-# undef LOWORD
-# undef HIWORD
-# undef LOBYTE
-# undef HIBYTE
-# include <windows.h>
-#endif /* Q_WS_WIN */
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
-#ifdef Q_WS_X11
+/* Qt includes: */
+#if defined(VBOX_WS_MAC) || defined(VBOX_WS_WIN)
+# include <QAbstractNativeEventFilter>
+#endif /* VBOX_WS_MAC || VBOX_WS_WIN */
+
+/* GUI includes: */
+#if defined(VBOX_WS_MAC)
+# include "DarwinKeyboard.h"
+#elif defined(VBOX_WS_WIN)
+# include "WinKeyboard.h"
+#elif defined(VBOX_WS_X11)
+# include "XKeyboard.h"
+#endif /* VBOX_WS_X11 */
+
+/* Other VBox includes: */
+#if defined(VBOX_WS_X11)
+# include <VBox/VBoxKeyboard.h>
+#endif /* VBOX_WS_X11 */
+
+/* External includes: */
+#if defined(VBOX_WS_MAC)
+# include <Carbon/Carbon.h>
+#elif defined(VBOX_WS_X11)
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # include <X11/keysym.h>
-# ifdef KeyPress
-   const int XKeyPress = KeyPress;
-   const int XKeyRelease = KeyRelease;
-#  undef KeyPress
-#  undef KeyRelease
-# endif /* KeyPress */
-# include "XKeyboard.h"
-# include <QX11Info>
-#endif /* Q_WS_X11 */
+# include <xcb/xcb.h>
+#endif /* VBOX_WS_X11 */
 
-#ifdef Q_WS_MAC
-# include "UICocoaApplication.h"
-# include "DarwinKeyboard.h"
-# include "VBoxUtils.h"
-# include <Carbon/Carbon.h>
-#endif /* Q_WS_MAC */
+/* Namespaces: */
+using namespace UIExtraDataDefs;
 
 
-#ifdef Q_WS_X11
+#if defined(VBOX_WS_MAC) || defined(VBOX_WS_WIN)
+/** QAbstractNativeEventFilter extension
+  * allowing to handle native platform events.
+  * Why do we need it? It's because Qt5 have unhandled
+  * well .. let's call it 'a bug' about native keyboard events
+  * which come to top-level widget (window) instead of focused sub-widget
+  * which actually supposed to get them. The strange thing is that target of
+  * those events on at least Windows host (MSG::hwnd) is indeed window itself,
+  * not the sub-widget we expect, so that's probably the reason Qt devs
+  * haven't fixed that bug so far for Windows and Mac OS X hosts. */
+class ComboEditorEventFilter : public QAbstractNativeEventFilter
+{
+public:
+
+    /** Constructor which takes the passed @a pParent to redirect events to. */
+    ComboEditorEventFilter(UIHostComboEditorPrivate *pParent)
+        : m_pParent(pParent)
+    {}
+
+    /** Handles all native events. */
+    bool nativeEventFilter(const QByteArray &eventType, void *pMessage, long *pResult)
+    {
+        /* Redirect event to parent: */
+        return m_pParent->nativeEvent(eventType, pMessage, pResult);
+    }
+
+private:
+
+    /** Holds the passed parent reference. */
+    UIHostComboEditorPrivate *m_pParent;
+};
+#endif /* VBOX_WS_MAC || VBOX_WS_WIN */
+
+
+/*********************************************************************************************************************************
+*   Namespace UINativeHotKey implementation.                                                                                     *
+*********************************************************************************************************************************/
+
+#ifdef VBOX_WS_X11
 namespace UINativeHotKey
 {
     QMap<QString, QString> m_keyNames;
 }
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
 
 QString UINativeHotKey::toString(int iKeyCode)
 {
     QString strKeyName;
 
-#ifdef Q_WS_WIN
+#if defined(VBOX_WS_MAC)
+
+    UInt32 modMask = DarwinKeyCodeToDarwinModifierMask(iKeyCode);
+    switch (modMask)
+    {
+        case shiftKey:
+        case optionKey:
+        case controlKey:
+        case cmdKey:
+            strKeyName = UIHostComboEditor::tr("Left %1");
+            break;
+        case rightShiftKey:
+        case rightOptionKey:
+        case rightControlKey:
+        case kEventKeyModifierRightCmdKeyMask:
+            strKeyName = UIHostComboEditor::tr("Right %1");
+            break;
+        default:
+            AssertMsgFailedReturn(("modMask=%#x\n", modMask), QString());
+    }
+    switch (modMask)
+    {
+        case shiftKey:
+        case rightShiftKey:
+            strKeyName = strKeyName.arg(QChar(kShiftUnicode));
+            break;
+        case optionKey:
+        case rightOptionKey:
+            strKeyName = strKeyName.arg(QChar(kOptionUnicode));
+            break;
+        case controlKey:
+        case rightControlKey:
+            strKeyName = strKeyName.arg(QChar(kControlUnicode));
+            break;
+        case cmdKey:
+        case kEventKeyModifierRightCmdKeyMask:
+            strKeyName = strKeyName.arg(QChar(kCommandUnicode));
+            break;
+    }
+
+#elif defined(VBOX_WS_WIN)
+
     /* MapVirtualKey doesn't distinguish between right and left vkeys,
      * even under XP, despite that it stated in MSDN. Do it by hands.
      * Besides that it can't recognize such virtual keys as
@@ -102,9 +196,9 @@ QString UINativeHotKey::toString(int iKeyCode)
         strKeyName = UIHostComboEditor::tr("<key_%1>").arg(iKeyCode);
     }
     delete[] pKeyName;
-#endif /* Q_WS_WIN */
 
-#ifdef Q_WS_X11
+#elif defined(VBOX_WS_X11)
+
     if (char *pNativeKeyName = ::XKeysymToString((KeySym)iKeyCode))
     {
         strKeyName = m_keyNames[pNativeKeyName].isEmpty() ?
@@ -115,73 +209,20 @@ QString UINativeHotKey::toString(int iKeyCode)
         AssertMsgFailed(("That key have no name!\n"));
         strKeyName = UIHostComboEditor::tr("<key_%1>").arg(iKeyCode);
     }
-#endif /* Q_WS_X11 */
 
-#ifdef Q_WS_MAC
-    UInt32 modMask = DarwinKeyCodeToDarwinModifierMask(iKeyCode);
-    switch (modMask)
-    {
-        case shiftKey:
-        case optionKey:
-        case controlKey:
-        case cmdKey:
-            strKeyName = UIHostComboEditor::tr("Left ");
-            break;
-        case rightShiftKey:
-        case rightOptionKey:
-        case rightControlKey:
-        case kEventKeyModifierRightCmdKeyMask:
-            strKeyName = UIHostComboEditor::tr("Right ");
-            break;
-        default:
-            AssertMsgFailedReturn(("modMask=%#x\n", modMask), QString());
-    }
-    switch (modMask)
-    {
-        case shiftKey:
-        case rightShiftKey:
-            strKeyName += QChar(kShiftUnicode);
-            break;
-        case optionKey:
-        case rightOptionKey:
-            strKeyName += QChar(kOptionUnicode);
-            break;
-        case controlKey:
-        case rightControlKey:
-            strKeyName += QChar(kControlUnicode);
-            break;
-        case cmdKey:
-        case kEventKeyModifierRightCmdKeyMask:
-            strKeyName += QChar(kCommandUnicode);
-            break;
-    }
-#endif /* Q_WS_MAC */
+#else
+
+# warning "port me!"
+
+#endif
 
     return strKeyName;
 }
 
 bool UINativeHotKey::isValidKey(int iKeyCode)
 {
-#ifdef Q_WS_WIN
-    return ((iKeyCode >= VK_SHIFT && iKeyCode <= VK_CAPITAL) ||
-            (iKeyCode >= VK_LSHIFT && iKeyCode <= VK_RMENU) ||
-            (iKeyCode >= VK_F1 && iKeyCode <= VK_F24) ||
-            iKeyCode == VK_NUMLOCK || iKeyCode == VK_SCROLL ||
-            iKeyCode == VK_LWIN || iKeyCode == VK_RWIN ||
-            iKeyCode == VK_APPS ||
-            iKeyCode == VK_PRINT);
-#endif /* Q_WS_WIN */
+#if defined(VBOX_WS_MAC)
 
-#ifdef Q_WS_X11
-    return (IsModifierKey(iKeyCode) /* allow modifiers */ ||
-            IsFunctionKey(iKeyCode) /* allow function keys */ ||
-            IsMiscFunctionKey(iKeyCode) /* allow miscellaneous function keys */ ||
-            iKeyCode == XK_Scroll_Lock /* allow 'Scroll Lock' missed in IsModifierKey() */) &&
-           (iKeyCode != NoSymbol /* ignore some special symbol */ &&
-            iKeyCode != XK_Insert /* ignore 'insert' included into IsMiscFunctionKey */);
-#endif /* Q_WS_X11 */
-
-#ifdef Q_WS_MAC
     UInt32 modMask = ::DarwinKeyCodeToDarwinModifierMask(iKeyCode);
     switch (modMask)
     {
@@ -197,12 +238,97 @@ bool UINativeHotKey::isValidKey(int iKeyCode)
         default:
             return false;
     }
-#endif /* Q_WS_MAC */
+
+#elif defined(VBOX_WS_WIN)
+
+    return (   iKeyCode >= VK_SHIFT  && iKeyCode <= VK_CAPITAL
+            && iKeyCode != VK_PAUSE)
+        || (iKeyCode >= VK_LSHIFT && iKeyCode <= VK_RMENU)
+        || (iKeyCode >= VK_F1     && iKeyCode <= VK_F24)
+        || iKeyCode == VK_NUMLOCK
+        || iKeyCode == VK_SCROLL
+        || iKeyCode == VK_LWIN
+        || iKeyCode == VK_RWIN
+        || iKeyCode == VK_APPS
+        || iKeyCode == VK_PRINT;
+
+#elif defined(VBOX_WS_X11)
+
+    return (IsModifierKey(iKeyCode) /* allow modifiers */ ||
+            IsFunctionKey(iKeyCode) /* allow function keys */ ||
+            IsMiscFunctionKey(iKeyCode) /* allow miscellaneous function keys */ ||
+            iKeyCode == XK_Scroll_Lock /* allow 'Scroll Lock' missed in IsModifierKey() */) &&
+           (iKeyCode != NoSymbol /* ignore some special symbol */ &&
+            iKeyCode != XK_Insert /* ignore 'insert' included into IsMiscFunctionKey */);
+
+#else
+
+# warning "port me!"
 
     return false;
+#endif
 }
 
-#ifdef Q_WS_WIN
+unsigned UINativeHotKey::modifierToSet1ScanCode(int iKeyCode)
+{
+    switch(iKeyCode)
+    {
+#if defined(VBOX_WS_MAC)
+
+        case controlKey:                        return 0x1D;
+        case rightControlKey:                   return 0x11D;
+        case shiftKey:                          return 0x2A;
+        case rightShiftKey:                     return 0x36;
+        case optionKey:                         return 0x38;
+        case rightOptionKey:                    return 0x138;
+        case cmdKey:                            return 0x15B;
+        case kEventKeyModifierRightCmdKeyMask:  return 0x15C;
+        default:                                return 0;
+
+#elif defined(VBOX_WS_WIN)
+
+        case VK_CONTROL:
+        case VK_LCONTROL:  return 0x1D;
+        case VK_RCONTROL:  return 0x11D;
+        case VK_SHIFT:
+        case VK_LSHIFT:    return 0x2A;
+        case VK_RSHIFT:    return 0x36;
+        case VK_MENU:
+        case VK_LMENU:     return 0x38;
+        case VK_RMENU:     return 0x138;
+        case VK_LWIN:      return 0x15B;
+        case VK_RWIN:      return 0x15C;
+        case VK_APPS:      return 0x15D;
+        default:           return 0;
+
+#elif defined(VBOX_WS_X11)
+
+        case XK_Control_L:         return 0x1D;
+        case XK_Control_R:         return 0x11D;
+        case XK_Shift_L:           return 0x2A;
+        case XK_Shift_R:           return 0x36;
+        case XK_Alt_L:             return 0x38;
+        case XK_ISO_Level3_Shift:
+        case XK_Alt_R:             return 0x138;
+        case XK_Meta_L:
+        case XK_Super_L:           return 0x15B;
+        case XK_Meta_R:
+        case XK_Super_R:           return 0x15C;
+        case XK_Menu:              return 0x15D;
+        default:                   return 0;
+
+#else
+
+# warning "port me!"
+
+        default:  return 0;
+
+#endif
+    }
+}
+
+#if defined(VBOX_WS_WIN)
+
 int UINativeHotKey::distinguishModifierVKey(int wParam, int lParam)
 {
     int iKeyCode = wParam;
@@ -241,9 +367,9 @@ int UINativeHotKey::distinguishModifierVKey(int wParam, int lParam)
     }
     return iKeyCode;
 }
-#endif /* Q_WS_WIN */
 
-#ifdef Q_WS_X11
+#elif defined(VBOX_WS_X11)
+
 void UINativeHotKey::retranslateKeyNames()
 {
     m_keyNames["Shift_L"]          = UIHostComboEditor::tr("Left Shift");
@@ -259,8 +385,13 @@ void UINativeHotKey::retranslateKeyNames()
     m_keyNames["Caps_Lock"]        = UIHostComboEditor::tr("Caps Lock");
     m_keyNames["Scroll_Lock"]      = UIHostComboEditor::tr("Scroll Lock");
 }
-#endif /* Q_WS_X11 */
 
+#endif /* VBOX_WS_X11 */
+
+
+/*********************************************************************************************************************************
+*   Namespace UIHostCombo implementation.                                                                                        *
+*********************************************************************************************************************************/
 
 namespace UIHostCombo
 {
@@ -302,6 +433,17 @@ QList<int> UIHostCombo::toKeyCodeList(const QString &strKeyCombo)
     return keyCodeList;
 }
 
+QList<unsigned> UIHostCombo::modifiersToScanCodes(const QString &strKeyCombo)
+{
+    QStringList encodedKeyList = strKeyCombo.split(',');
+    QList<unsigned> scanCodeList;
+    for (int i = 0; i < encodedKeyList.size(); ++i)
+        if (unsigned idxScanCode = UINativeHotKey::modifierToSet1ScanCode(encodedKeyList[i].toInt()))
+            if (idxScanCode != 0)
+                scanCodeList << idxScanCode;
+    return scanCodeList;
+}
+
 bool UIHostCombo::isValidKeyCombo(const QString &strKeyCombo)
 {
     QList<int> keyCodeList = toKeyCodeList(strKeyCombo);
@@ -314,6 +456,10 @@ bool UIHostCombo::isValidKeyCombo(const QString &strKeyCombo)
 }
 
 
+/*********************************************************************************************************************************
+*   Class UIHostComboEditor implementation.                                                                                      *
+*********************************************************************************************************************************/
+
 UIHostComboEditor::UIHostComboEditor(QWidget *pParent)
     : QIWithRetranslateUI<QWidget>(pParent)
 {
@@ -335,7 +481,11 @@ void UIHostComboEditor::prepare()
     QHBoxLayout *pLayout = new QHBoxLayout(this);
     {
         /* Configure layout: */
-        pLayout->setSpacing(4);
+#ifdef VBOX_WS_MAC
+        pLayout->setSpacing(5);
+#else
+        pLayout->setSpacing(qApp->style()->pixelMetric(QStyle::PM_LayoutVerticalSpacing) / 2);
+#endif
         pLayout->setContentsMargins(0, 0, 0, 0);
         /* Create UIHostComboEditorPrivate instance: */
         m_pEditor = new UIHostComboEditorPrivate;
@@ -379,9 +529,19 @@ UIHostComboWrapper UIHostComboEditor::combo() const
 }
 
 
+/*********************************************************************************************************************************
+*   Class UIHostComboEditorPrivate implementation.                                                                               *
+*********************************************************************************************************************************/
+
 UIHostComboEditorPrivate::UIHostComboEditorPrivate()
     : m_pReleaseTimer(0)
     , m_fStartNewSequence(true)
+#if defined(VBOX_WS_MAC) || defined(VBOX_WS_WIN)
+    , m_pPrivateEventFilter(0)
+#endif /* VBOX_WS_MAC || VBOX_WS_WIN */
+#ifdef VBOX_WS_WIN
+    , m_pAltGrMonitor(0)
+#endif /* VBOX_WS_WIN */
 {
     /* Configure widget: */
     setAttribute(Qt::WA_NativeWindow);
@@ -394,24 +554,37 @@ UIHostComboEditorPrivate::UIHostComboEditorPrivate()
     m_pReleaseTimer->setInterval(200);
     connect(m_pReleaseTimer, SIGNAL(timeout()), this, SLOT(sltReleasePendingKeys()));
 
-#ifdef Q_WS_X11
-    /* Initialize the X keyboard subsystem: */
-    initMappedX11Keyboard(QX11Info::display(), vboxGlobal().settings().publicProperty("GUI/RemapScancodes"));
-#endif /* Q_WS_X11 */
+#if defined(VBOX_WS_MAC) || defined(VBOX_WS_WIN)
+    /* Prepare private event filter: */
+    m_pPrivateEventFilter = new ComboEditorEventFilter(this);
+    qApp->installNativeEventFilter(m_pPrivateEventFilter);
+#endif /* VBOX_WS_MAC || VBOX_WS_WIN */
 
-#ifdef Q_WS_MAC
+#if defined(VBOX_WS_MAC)
     m_uDarwinKeyModifiers = 0;
-    UICocoaApplication::instance()->registerForNativeEvents(RT_BIT_32(10) | RT_BIT_32(11) | RT_BIT_32(12) /* NSKeyDown  | NSKeyUp | | NSFlagsChanged */, UIHostComboEditorPrivate::darwinEventHandlerProc, this);
-    ::DarwinGrabKeyboard(false /* just modifiers */);
-#endif /* Q_WS_MAC */
+#elif defined(VBOX_WS_WIN)
+    /* Prepare AltGR monitor: */
+    m_pAltGrMonitor = new WinAltGrMonitor;
+#elif defined(VBOX_WS_X11)
+    /* Initialize the X keyboard subsystem: */
+    initMappedX11Keyboard(QX11Info::display(), gEDataManager->remappedScanCodes());
+#endif /* VBOX_WS_X11 */
 }
 
 UIHostComboEditorPrivate::~UIHostComboEditorPrivate()
 {
-#ifdef Q_WS_MAC
-    ::DarwinReleaseKeyboard();
-    UICocoaApplication::instance()->unregisterForNativeEvents(RT_BIT_32(10) | RT_BIT_32(11) | RT_BIT_32(12) /* NSKeyDown  | NSKeyUp | | NSFlagsChanged */, UIHostComboEditorPrivate::darwinEventHandlerProc, this);
-#endif /* Q_WS_MAC */
+#if defined(VBOX_WS_WIN)
+    /* Cleanup AltGR monitor: */
+    delete m_pAltGrMonitor;
+    m_pAltGrMonitor = 0;
+#endif /* VBOX_WS_WIN */
+
+#if defined(VBOX_WS_MAC) || defined(VBOX_WS_WIN)
+    /* Cleanup private event filter: */
+    qApp->removeNativeEventFilter(m_pPrivateEventFilter);
+    delete m_pPrivateEventFilter;
+    m_pPrivateEventFilter = 0;
+#endif /* VBOX_WS_MAC || VBOX_WS_WIN */
 }
 
 void UIHostComboEditorPrivate::setCombo(const UIHostComboWrapper &strCombo)
@@ -455,189 +628,151 @@ void UIHostComboEditorPrivate::sltClear()
     emit sigDataChanged();
 }
 
-#ifdef Q_WS_WIN
-/**
- * @brief isSyntheticLCtrl
- * @param   pMsg  Windows WM_[SYS]KEY* event message structure
- * @return  true if this is a synthetic LCtrl event, false otherwise
- * This function is a heuristic to tell whether a key event is the first in
- * a synthetic LCtrl+RAlt sequence which Windows uses to signal AltGr.  Our
- * heuristic is in two parts.  First of all, we check whether there is a pending
- * RAlt key event matching this LCtrl event (i.e. both key up or both key down)
- * and if there is, we check whether the current layout has an AltGr key.  We
- * check this by looking to see if any of the layout-dependent keys has a symbol
- * associated when AltGr is pressed.
- */
-static bool isSyntheticLCtrl(MSG *pMsg)
+bool UIHostComboEditorPrivate::nativeEvent(const QByteArray &eventType, void *pMessage, long *pResult)
 {
-    MSG peekMsg;
-    /** Keyboard state array with VK_CONTROL and VK_MENU depressed. */
-    const BYTE auKeyStates[256] =
-    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0x80, 0x80 };
-    WORD ach;
-    unsigned i;
+# if defined(VBOX_WS_MAC)
 
-    Assert(   pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN
-           || pMsg->message == WM_KEYUP || pMsg->message == WM_SYSKEYUP);
-    if (   ((RT_HIWORD(pMsg->lParam) & 0xFF) != 0x1d /* scan code: Control */)
-        || RT_HIWORD(pMsg->lParam) & KF_EXTENDED)
-        return false;
-    if (!PeekMessage(&peekMsg, NULL, WM_KEYFIRST, WM_KEYLAST, PM_NOREMOVE))
-        return false;
-    if (   (pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN)
-        && (peekMsg.message != WM_KEYDOWN && peekMsg.message != WM_SYSKEYDOWN))
-        return false;
-    if (   (pMsg->message == WM_KEYUP || pMsg->message == WM_SYSKEYUP)
-        && (peekMsg.message != WM_KEYUP && peekMsg.message != WM_SYSKEYUP))
-        return false;
-    if (   ((RT_HIWORD(peekMsg.lParam) & 0xFF) != 0x38 /* scan code: Alt */)
-        || !(RT_HIWORD(peekMsg.lParam) & KF_EXTENDED))
-        return false;
-    /* If we got this far then we have a key event which could potentially
-     * be a synthetic left control.  Now we check to see whether the current
-     * keyboard layout actually has an AltGr key by checking whether any of
-     * the keys which might do produce a symbol when AltGr (Control + Alt) is
-     * depressed.  Generally this loop will exit pretty early (it exits on the
-     * first iteration for my German layout).  If there is no AltGr key in the
-     * layout then it will run right through, but that should not happen very
-     * often as we should hardly ever reach the loop in that case.
-     *
-     * In theory we could do this once and cache the result, but that involves
-     * tracking layout switches to invalidate the cache, and I don't think
-     * that the added complexity is worth the price.
-     */
-    for (i = '0'; i <= VK_OEM_102; ++i)
-    {
-        if (ToAscii(i, 0, auKeyStates, &ach, 0))
-            break;
-        /* Skip ranges of virtual keys which are undefined or not relevant. */
-        if (i == '9')
-            i = 'A' - 1;
-        if (i == 'Z')
-            i = VK_OEM_1 - 1;
-        if (i == VK_OEM_3)
-            i = VK_OEM_4 - 1;
-        if (i == VK_OEM_8)
-            i = VK_OEM_102 - 1;
-    }
-    if (i > VK_OEM_102)
-        return false;
-    return true;
-}
+    /* Make sure it's generic NSEvent: */
+    if (eventType != "mac_generic_NSEvent")
+        return QLineEdit::nativeEvent(eventType, pMessage, pResult);
+    EventRef event = static_cast<EventRef>(darwinCocoaToCarbonEvent(pMessage));
 
-bool UIHostComboEditorPrivate::winEvent(MSG *pMsg, long* /* pResult */)
-{
-    switch (pMsg->message)
+    /* Check if some NSEvent should be filtered out.
+     * Returning @c true means filtering-out,
+     * Returning @c false means passing event to Qt. */
+    switch(::GetEventClass(event))
     {
-        case WM_KEYDOWN:
-        case WM_SYSKEYDOWN:
-        case WM_KEYUP:
-        case WM_SYSKEYUP:
+        /* Watch for keyboard-events: */
+        case kEventClassKeyboard:
         {
-            /* Get key-code: */
-            int iKeyCode = UINativeHotKey::distinguishModifierVKey((int)pMsg->wParam, (int)pMsg->lParam);
-
-            /* If this is the first event in a synthetic AltGr = LCtrl+RAlt
-             * sequence then swallow it. */
-            if (isSyntheticLCtrl(pMsg))
-                return true;
-
-            /* Process the key event: */
-            return processKeyEvent(iKeyCode, pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN);
-        }
-        default:
-            break;
-    }
-
-    return false;
-}
-#endif /* Q_WS_WIN */
-
-#ifdef Q_WS_X11
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-bool UIHostComboEditorPrivate::x11Event(XEvent *pEvent)
-{
-    switch (pEvent->type)
-    {
-        case XKeyPress:
-        case XKeyRelease:
-        {
-            /* Get key-code: */
-            XKeyEvent *pKeyEvent = (XKeyEvent*)pEvent;
-            KeySym ks = ::XKeycodeToKeysym(pKeyEvent->display, pKeyEvent->keycode, 0);
-
-            int iKeySym = (int)ks;
-
-            /* Process the key event: */
-            return processKeyEvent(iKeySym, pEvent->type == XKeyPress);
-        }
-        default:
-            break;
-    }
-
-    return false;
-}
-#pragma GCC diagnostic pop
-#endif /* Q_WS_X11 */
-
-#ifdef Q_WS_MAC
-/* static */
-bool UIHostComboEditorPrivate::darwinEventHandlerProc(const void *pvCocoaEvent, const void *pvCarbonEvent, void *pvUser)
-{
-    UIHostComboEditorPrivate *pEditor = static_cast<UIHostComboEditorPrivate*>(pvUser);
-    EventRef inEvent = (EventRef)pvCarbonEvent;
-    UInt32 EventClass = ::GetEventClass(inEvent);
-    if (EventClass == kEventClassKeyboard)
-        return pEditor->darwinKeyboardEvent(pvCocoaEvent, inEvent);
-    return false;
-}
-
-bool UIHostComboEditorPrivate::darwinKeyboardEvent(const void *pvCocoaEvent, EventRef inEvent)
-{
-    /* Ignore key changes unless we're the focus widget: */
-    if (!hasFocus())
-        return false;
-
-    UInt32 eventKind = ::GetEventKind(inEvent);
-    switch (eventKind)
-    {
-        //case kEventRawKeyDown:
-        //case kEventRawKeyUp:
-        //case kEventRawKeyRepeat:
-        case kEventRawKeyModifiersChanged:
-        {
-            /* Get modifier mask: */
-            UInt32 modifierMask = 0;
-            ::GetEventParameter(inEvent, kEventParamKeyModifiers, typeUInt32, NULL,
-                                sizeof(modifierMask), NULL, &modifierMask);
-            modifierMask = ::DarwinAdjustModifierMask(modifierMask, pvCocoaEvent);
-            UInt32 changed = m_uDarwinKeyModifiers ^ modifierMask;
-
-            if (!changed)
-                break;
-
-            /* Convert to keycode: */
-            unsigned uKeyCode = ::DarwinModifierMaskToDarwinKeycode(changed);
-
-            if (!uKeyCode || uKeyCode == ~0U)
-                return false;
-
-            /* Process the key event: */
-            if (processKeyEvent(uKeyCode, changed & modifierMask))
+            switch(::GetEventKind(event))
             {
-                /* Save the new modifier mask state. */
-                m_uDarwinKeyModifiers = modifierMask;
-                return true;
+                /* Watch for keyboard-modifier-events: */
+                case kEventRawKeyModifiersChanged:
+                {
+                    /* Get modifier mask: */
+                    UInt32 modifierMask = 0;
+                    ::GetEventParameter(event, kEventParamKeyModifiers, typeUInt32,
+                                        NULL, sizeof(modifierMask), NULL, &modifierMask);
+                    modifierMask = ::DarwinAdjustModifierMask(modifierMask, pMessage);
+
+                    /* Do not handle unchanged masks: */
+                    UInt32 uChanged = m_uDarwinKeyModifiers ^ modifierMask;
+                    if (!uChanged)
+                        break;
+
+                    /* Convert to keycode: */
+                    unsigned uKeyCode = ::DarwinModifierMaskToDarwinKeycode(uChanged);
+
+                    /* Do not handle empty and multiple modifier changes: */
+                    if (!uKeyCode || uKeyCode == ~0U)
+                        break;
+
+                    /* Handle key-event: */
+                    if (processKeyEvent(uKeyCode, uChanged & modifierMask))
+                    {
+                        /* Save the new modifier mask state: */
+                        m_uDarwinKeyModifiers = modifierMask;
+                        return true;
+                    }
+                    break;
+                }
+                default:
+                    break;
             }
             break;
         }
         default:
             break;
     }
-    return false;
+
+# elif defined(VBOX_WS_WIN)
+
+    /* Make sure it's generic MSG event: */
+    if (eventType != "windows_generic_MSG")
+        return QLineEdit::nativeEvent(eventType, pMessage, pResult);
+    MSG *pEvent = static_cast<MSG*>(pMessage);
+
+    /* Check if some MSG event should be filtered out.
+     * Returning @c true means filtering-out,
+     * Returning @c false means passing event to Qt. */
+    switch (pEvent->message)
+    {
+        /* Watch for key-events: */
+        case WM_KEYDOWN:
+        case WM_SYSKEYDOWN:
+        case WM_KEYUP:
+        case WM_SYSKEYUP:
+        {
+            /* Parse key-event: */
+            int iKeyCode = UINativeHotKey::distinguishModifierVKey((int)pEvent->wParam, (int)pEvent->lParam);
+            unsigned iDownScanCode = (pEvent->lParam >> 16) & 0x7F;
+            const bool fPressed = !(pEvent->lParam & 0x80000000);
+            const bool fExtended = pEvent->lParam & 0x1000000;
+
+            /* If present - why not just assert this? */
+            if (m_pAltGrMonitor)
+            {
+                /* Update AltGR monitor state from key-event: */
+                m_pAltGrMonitor->updateStateFromKeyEvent(iDownScanCode, fPressed, fExtended);
+                /* And release left Ctrl key early (if required): */
+                if (m_pAltGrMonitor->isLeftControlReleaseNeeded())
+                {
+                    m_pressedKeys.remove(VK_LCONTROL);
+                    m_shownKeys.remove(VK_LCONTROL);
+                }
+                /* Fake LCtrl release events can also end up in the released
+                 * key set.  Detect them on the immediately following RAlt up. */
+                if (!m_pressedKeys.contains(VK_LCONTROL))
+                    m_releasedKeys.remove(VK_LCONTROL);
+            }
+
+            /* Handle key-event: */
+            return processKeyEvent(iKeyCode, (pEvent->message == WM_KEYDOWN || pEvent->message == WM_SYSKEYDOWN));
+        }
+        default:
+            break;
+    }
+
+# elif defined(VBOX_WS_X11)
+
+    /* Make sure it's generic XCB event: */
+    if (eventType != "xcb_generic_event_t")
+        return QLineEdit::nativeEvent(eventType, pMessage, pResult);
+    xcb_generic_event_t *pEvent = static_cast<xcb_generic_event_t*>(pMessage);
+
+    /* Check if some XCB event should be filtered out.
+     * Returning @c true means filtering-out,
+     * Returning @c false means passing event to Qt. */
+    switch (pEvent->response_type & ~0x80)
+    {
+        /* Watch for key-events: */
+        case XCB_KEY_PRESS:
+        case XCB_KEY_RELEASE:
+        {
+            /* Parse key-event: */
+            xcb_key_press_event_t *pKeyEvent = static_cast<xcb_key_press_event_t*>(pMessage);
+            RT_GCC_NO_WARN_DEPRECATED_BEGIN
+            const KeySym ks = ::XKeycodeToKeysym(QX11Info::display(), pKeyEvent->detail, 0);
+            RT_GCC_NO_WARN_DEPRECATED_END
+            const int iKeySym = static_cast<const int>(ks);
+
+            /* Handle key-event: */
+            return processKeyEvent(iKeySym, (pEvent->response_type & ~0x80) == XCB_KEY_PRESS);
+        }
+        default:
+            break;
+    }
+
+# else
+
+#  warning "port me!"
+
+# endif
+
+    /* Call to base-class: */
+    return QLineEdit::nativeEvent(eventType, pMessage, pResult);
 }
-#endif /* Q_WS_MAC */
 
 void UIHostComboEditorPrivate::keyPressEvent(QKeyEvent *pEvent)
 {
