@@ -1,12 +1,10 @@
 /* $Id: VBoxMediaComboBox.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * VBoxMediaComboBox class implementation
+ * VBox Qt GUI - VBoxMediaComboBox class implementation.
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,14 +15,21 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* Qt includes: */
-#include <QFileInfo>
-#include <QDir>
-#include <QAbstractItemView>
+# include <QFileInfo>
+# include <QDir>
+# include <QAbstractItemView>
 
 /* GUI includes: */
-#include "VBoxMediaComboBox.h"
-#include "UIMedium.h"
+# include "VBoxMediaComboBox.h"
+# include "UIMedium.h"
+
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 
 VBoxMediaComboBox::VBoxMediaComboBox (QWidget *aParent)
     : QComboBox (aParent)
