@@ -1,12 +1,10 @@
 /* $Id: UIWizardNewVDPageBasic1.cpp $ */
 /** @file
- *
- * VBox frontends: Qt4 GUI ("VirtualBox"):
- * UIWizardNewVDPageBasic1 class implementation
+ * VBox Qt GUI - UIWizardNewVDPageBasic1 class implementation.
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,25 +15,32 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* Qt includes: */
-#include <QVBoxLayout>
-#include <QButtonGroup>
-#include <QRadioButton>
+# include <QVBoxLayout>
+# include <QButtonGroup>
+# include <QRadioButton>
 
 /* GUI includes: */
-#include "UIWizardNewVDPageBasic1.h"
-#include "UIWizardNewVD.h"
-#include "VBoxGlobal.h"
-#include "QIRichTextLabel.h"
+# include "UIWizardNewVDPageBasic1.h"
+# include "UIWizardNewVD.h"
+# include "VBoxGlobal.h"
+# include "QIRichTextLabel.h"
 
 /* COM includes: */
-#include "CSystemProperties.h"
+# include "CSystemProperties.h"
+
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 
 UIWizardNewVDPage1::UIWizardNewVDPage1()
 {
 }
 
-void UIWizardNewVDPage1::addFormatButton(QWidget *pParent, QVBoxLayout *pFormatLayout, CMediumFormat medFormat)
+void UIWizardNewVDPage1::addFormatButton(QWidget *pParent, QVBoxLayout *pFormatLayout, CMediumFormat medFormat, bool fPreferred /* = false */)
 {
     /* Check that medium format supports creation: */
     ULONG uFormatCapabilities = 0;
@@ -44,8 +49,8 @@ void UIWizardNewVDPage1::addFormatButton(QWidget *pParent, QVBoxLayout *pFormatL
     for (int i = 0; i < capabilities.size(); i++)
         uFormatCapabilities |= capabilities[i];
 
-    if (!(uFormatCapabilities & MediumFormatCapabilities_CreateFixed ||
-          uFormatCapabilities & MediumFormatCapabilities_CreateDynamic))
+    if (!(uFormatCapabilities & KMediumFormatCapabilities_CreateFixed ||
+          uFormatCapabilities & KMediumFormatCapabilities_CreateDynamic))
         return;
 
     /* Check that medium format supports creation of virtual hard-disks: */
@@ -57,10 +62,20 @@ void UIWizardNewVDPage1::addFormatButton(QWidget *pParent, QVBoxLayout *pFormatL
 
     /* Create/add corresponding radio-button: */
     QRadioButton *pFormatButton = new QRadioButton(pParent);
-    pFormatLayout->addWidget(pFormatButton);
-    m_formats << medFormat;
-    m_formatNames << medFormat.GetName();
-    m_pFormatButtonGroup->addButton(pFormatButton, m_formatNames.size() - 1);
+    AssertPtrReturnVoid(pFormatButton);
+    {
+        /* Make the preferred button font bold: */
+        if (fPreferred)
+        {
+            QFont font = pFormatButton->font();
+            font.setBold(true);
+            pFormatButton->setFont(font);
+        }
+        pFormatLayout->addWidget(pFormatButton);
+        m_formats << medFormat;
+        m_formatNames << medFormat.GetName();
+        m_pFormatButtonGroup->addButton(pFormatButton, m_formatNames.size() - 1);
+    }
 }
 
 CMediumFormat UIWizardNewVDPage1::mediumFormat() const
@@ -88,26 +103,36 @@ UIWizardNewVDPageBasic1::UIWizardNewVDPageBasic1()
         {
             m_pFormatButtonGroup = new QButtonGroup(this);
             {
-                CSystemProperties systemProperties = vboxGlobal().virtualBox().GetSystemProperties();
-                const QVector<CMediumFormat> &medFormats = systemProperties.GetMediumFormats();
-                for (int i = 0; i < medFormats.size(); ++i)
+                /* Enumerate medium formats in special order: */
+                CSystemProperties properties = vboxGlobal().virtualBox().GetSystemProperties();
+                const QVector<CMediumFormat> &formats = properties.GetMediumFormats();
+                QMap<QString, CMediumFormat> vdi, preferred;
+                foreach (const CMediumFormat &format, formats)
                 {
-                    const CMediumFormat &medFormat = medFormats[i];
-                    if (medFormat.GetName() == "VDI")
-                        addFormatButton(this, pFormatLayout, medFormat);
+                    /* VDI goes first: */
+                    if (format.GetName() == "VDI")
+                        vdi[format.GetId()] = format;
+                    else
+                    {
+                        const QVector<KMediumFormatCapabilities> &capabilities = format.GetCapabilities();
+                        /* Then preferred: */
+                        if (capabilities.contains(KMediumFormatCapabilities_Preferred))
+                            preferred[format.GetId()] = format;
+                    }
                 }
-                for (int i = 0; i < medFormats.size(); ++i)
+
+                /* Create buttons for VDI and preferred: */
+                foreach (const QString &strId, vdi.keys())
+                    addFormatButton(this, pFormatLayout, vdi.value(strId));
+                foreach (const QString &strId, preferred.keys())
+                    addFormatButton(this, pFormatLayout, preferred.value(strId));
+
+                if (!m_pFormatButtonGroup->buttons().isEmpty())
                 {
-                    const CMediumFormat &medFormat = medFormats[i];
-                    if (medFormat.GetName() != "VDI")
-                        addFormatButton(this, pFormatLayout, medFormat);
+                    m_pFormatButtonGroup->button(0)->click();
+                    m_pFormatButtonGroup->button(0)->setFocus();
                 }
             }
-            if (!m_pFormatButtonGroup->buttons().isEmpty())
-            {
-                m_pFormatButtonGroup->button(0)->click();
-                m_pFormatButtonGroup->button(0)->setFocus();
-            }
         }
         pMainLayout->addWidget(m_pLabel);
         pMainLayout->addLayout(pFormatLayout);
@@ -115,7 +140,7 @@ UIWizardNewVDPageBasic1::UIWizardNewVDPageBasic1()
     }
 
     /* Setup connections: */
-    connect(m_pFormatButtonGroup, SIGNAL(buttonClicked(QAbstractButton*)), this, SIGNAL(completeChanged()));
+    connect(m_pFormatButtonGroup, SIGNAL(buttonClicked(QAbstractButton *)), this, SIGNAL(completeChanged()));
 
     /* Register classes: */
     qRegisterMetaType<CMediumFormat>();
@@ -126,11 +151,11 @@ UIWizardNewVDPageBasic1::UIWizardNewVDPageBasic1()
 void UIWizardNewVDPageBasic1::retranslateUi()
 {
     /* Translate page: */
-    setTitle(UIWizardNewVD::tr("Hard drive file type"));
+    setTitle(UIWizardNewVD::tr("Hard disk file type"));
 
     /* Translate widgets: */
     m_pLabel->setText(UIWizardNewVD::tr("Please choose the type of file that you would like to use "
-                                        "for the new virtual hard drive. If you do not need to use it "
+                                        "for the new virtual hard disk. If you do not need to use it "
                                         "with other virtualization software you can leave this setting unchanged."));
     QList<QAbstractButton*> buttons = m_pFormatButtonGroup->buttons();
     for (int i = 0; i < buttons.size(); ++i)
