@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2014-2015 Oracle Corporation
+ * Copyright (C) 2014-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,14 +15,16 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/******************************************************************************
- *   Header Files                                                             *
- ******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 
 #include <iprt/dir.h>
 #include <iprt/file.h>
 #include <iprt/fs.h>
 #include <iprt/path.h>
+#include <iprt/string.h>
 #include <iprt/symlink.h>
 #include <iprt/uri.h>
 
@@ -42,6 +44,7 @@ DnDURIList::DnDURIList(void)
 
 DnDURIList::~DnDURIList(void)
 {
+    Clear();
 }
 
 int DnDURIList::addEntry(const char *pcszSource, const char *pcszTarget, uint32_t fFlags)
@@ -49,7 +52,7 @@ int DnDURIList::addEntry(const char *pcszSource, const char *pcszTarget, uint32_
     AssertPtrReturn(pcszSource, VERR_INVALID_POINTER);
     AssertPtrReturn(pcszTarget, VERR_INVALID_POINTER);
 
-    LogFlowFunc(("pcszSource=%s, pcszTarget=%s\n", pcszSource, pcszTarget));
+    LogFlowFunc(("pcszSource=%s, pcszTarget=%s, fFlags=0x%x\n", pcszSource, pcszTarget, fFlags));
 
     RTFSOBJINFO objInfo;
     int rc = RTPathQueryInfo(pcszSource, &objInfo, RTFSOBJATTRADD_NOTHING);
@@ -57,20 +60,46 @@ int DnDURIList::addEntry(const char *pcszSource, const char *pcszTarget, uint32_
     {
         if (RTFS_IS_FILE(objInfo.Attr.fMode))
         {
-            LogFlowFunc(("File '%s' -> '%s'\n", pcszSource, pcszTarget));
+            LogFlowFunc(("File '%s' -> '%s' (%RU64 bytes, file mode 0x%x)\n",
+                         pcszSource, pcszTarget, (uint64_t)objInfo.cbObject, objInfo.Attr.fMode));
 
-            m_lstTree.append(DnDURIObject(DnDURIObject::File, pcszSource, pcszTarget,
-                             objInfo.Attr.fMode, (uint64_t)objInfo.cbObject));
-            m_cTotal++;
-            m_cbTotal += (uint64_t)objInfo.cbObject;
+            DnDURIObject *pObjFile = new DnDURIObject(DnDURIObject::File, pcszSource, pcszTarget);
+            if (pObjFile)
+            {
+                if (fFlags & DNDURILIST_FLAGS_KEEP_OPEN) /* Shall we keep the file open while being added to this list? */
+                {
+                    /** @todo Add a standard fOpen mode for this list. */
+                    rc = pObjFile->Open(DnDURIObject::Source, RTFILE_O_OPEN | RTFILE_O_READ | RTFILE_O_DENY_WRITE, objInfo.Attr.fMode);
+                }
+
+                if (RT_SUCCESS(rc))
+                {
+                    m_lstTree.append(pObjFile);
+
+                    m_cTotal++;
+                    m_cbTotal += pObjFile->GetSize();
+                }
+                else
+                    delete pObjFile;
+            }
+            else
+                rc = VERR_NO_MEMORY;
         }
         else if (RTFS_IS_DIRECTORY(objInfo.Attr.fMode))
         {
-            LogFlowFunc(("Directory '%s' -> '%s' \n", pcszSource, pcszTarget));
+            LogFlowFunc(("Directory '%s' -> '%s' (file mode 0x%x)\n", pcszSource, pcszTarget, objInfo.Attr.fMode));
 
-            m_lstTree.append(DnDURIObject(DnDURIObject::Directory, pcszSource, pcszTarget,
-                             objInfo.Attr.fMode, 0 /* Size */));
-            m_cTotal++;
+            DnDURIObject *pObjDir = new DnDURIObject(DnDURIObject::Directory, pcszSource, pcszTarget,
+                                                     objInfo.Attr.fMode, 0 /* Size */);
+            if (pObjDir)
+            {
+                m_lstTree.append(pObjDir);
+
+                /** @todo Add DNDURILIST_FLAGS_KEEP_OPEN handling? */
+                m_cTotal++;
+            }
+            else
+                rc = VERR_NO_MEMORY;
         }
         /* Note: Symlinks already should have been resolved at this point. */
         else
@@ -98,109 +127,112 @@ int DnDURIList::appendPathRecursive(const char *pcszSrcPath,
         if (RTFS_IS_DIRECTORY(objInfo.Attr.fMode))
         {
             rc = addEntry(pcszSrcPath, &pcszDstPath[cchDstBase], fFlags);
-
-            PRTDIR hDir;
-            if (RT_SUCCESS(rc))
-                rc = RTDirOpen(&hDir, pcszSrcPath);
             if (RT_SUCCESS(rc))
             {
-                do
+                RTDIR hDir;
+                rc = RTDirOpen(&hDir, pcszSrcPath);
+                if (RT_SUCCESS(rc))
                 {
-                    RTDIRENTRY DirEntry;
-                    rc = RTDirRead(hDir, &DirEntry, NULL);
-                    if (RT_FAILURE(rc))
+                    size_t        cbDirEntry = 0;
+                    PRTDIRENTRYEX pDirEntry  = NULL;
+                    do
                     {
-                        if (rc == VERR_NO_MORE_FILES)
-                            rc = VINF_SUCCESS;
-                        break;
-                    }
-
-                    switch (DirEntry.enmType)
-                    {
-                        case RTDIRENTRYTYPE_DIRECTORY:
+                        /* Retrieve the next directory entry. */
+                        rc = RTDirReadExA(hDir, &pDirEntry, &cbDirEntry, RTFSOBJATTRADD_NOTHING, RTPATH_F_ON_LINK);
+                        if (RT_FAILURE(rc))
                         {
-                            /* Skip "." and ".." entries. */
-                            if (   RTStrCmp(DirEntry.szName, ".")  == 0
-                                || RTStrCmp(DirEntry.szName, "..") == 0)
-                                break;
-
-                            char *pszSrc = RTPathJoinA(pcszSrcPath, DirEntry.szName);
-                            if (pszSrc)
-                            {
-                                char *pszDst = RTPathJoinA(pcszDstPath, DirEntry.szName);
-                                if (pszDst)
-                                {
-                                    rc = appendPathRecursive(pszSrc, pszDst, pcszDstBase, cchDstBase, fFlags);
-                                    RTStrFree(pszDst);
-                                }
-                                else
-                                    rc = VERR_NO_MEMORY;
-
-                                RTStrFree(pszSrc);
-                            }
-                            else
-                                rc = VERR_NO_MEMORY;
+                            if (rc == VERR_NO_MORE_FILES)
+                                rc = VINF_SUCCESS;
                             break;
                         }
 
-                        case RTDIRENTRYTYPE_FILE:
+                        switch (pDirEntry->Info.Attr.fMode & RTFS_TYPE_MASK)
                         {
-                            char *pszSrc = RTPathJoinA(pcszSrcPath, DirEntry.szName);
-                            if (pszSrc)
+                            case RTFS_TYPE_DIRECTORY:
                             {
-                                char *pszDst = RTPathJoinA(pcszDstPath, DirEntry.szName);
-                                if (pszDst)
-                                {
-                                    rc = addEntry(pszSrc, &pszDst[cchDstBase], fFlags);
-                                    RTStrFree(pszDst);
-                                }
-                                else
-                                    rc = VERR_NO_MEMORY;
-                                RTStrFree(pszSrc);
-                            }
-                            else
-                                rc = VERR_NO_MEMORY;
-                            break;
-                        }
-                        case RTDIRENTRYTYPE_SYMLINK:
-                        {
-                            if (fFlags & DNDURILIST_FLAGS_RESOLVE_SYMLINKS)
-                            {
-                                char *pszSrc = RTPathRealDup(pcszDstBase);
+                                /* Skip "." and ".." entries. */
+                                if (RTDirEntryExIsStdDotLink(pDirEntry))
+                                    break;
+
+                                char *pszSrc = RTPathJoinA(pcszSrcPath, pDirEntry->szName);
                                 if (pszSrc)
                                 {
-                                    rc = RTPathQueryInfo(pszSrc, &objInfo, RTFSOBJATTRADD_NOTHING);
-                                    if (RT_SUCCESS(rc))
+                                    char *pszDst = RTPathJoinA(pcszDstPath, pDirEntry->szName);
+                                    if (pszDst)
                                     {
-                                        if (RTFS_IS_DIRECTORY(objInfo.Attr.fMode))
-                                        {
-                                            LogFlowFunc(("Directory entry is symlink to directory\n"));
-                                            rc = appendPathRecursive(pszSrc, pcszDstPath, pcszDstBase, cchDstBase, fFlags);
-                                        }
-                                        else if (RTFS_IS_FILE(objInfo.Attr.fMode))
-                                        {
-                                            LogFlowFunc(("Directory entry is symlink to file\n"));
-                                            rc = addEntry(pszSrc, &pcszDstPath[cchDstBase], fFlags);
-                                        }
-                                        else
-                                            rc = VERR_NOT_SUPPORTED;
+                                        rc = appendPathRecursive(pszSrc, pszDst, pcszDstBase, cchDstBase, fFlags);
+                                        RTStrFree(pszDst);
                                     }
+                                    else
+                                        rc = VERR_NO_MEMORY;
 
                                     RTStrFree(pszSrc);
                                 }
                                 else
                                     rc = VERR_NO_MEMORY;
+                                break;
                             }
-                            break;
+
+                            case RTFS_TYPE_FILE:
+                            {
+                                char *pszSrc = RTPathJoinA(pcszSrcPath, pDirEntry->szName);
+                                if (pszSrc)
+                                {
+                                    char *pszDst = RTPathJoinA(pcszDstPath, pDirEntry->szName);
+                                    if (pszDst)
+                                    {
+                                        rc = addEntry(pszSrc, &pszDst[cchDstBase], fFlags);
+                                        RTStrFree(pszDst);
+                                    }
+                                    else
+                                        rc = VERR_NO_MEMORY;
+                                    RTStrFree(pszSrc);
+                                }
+                                else
+                                    rc = VERR_NO_MEMORY;
+                                break;
+                            }
+                            case RTFS_TYPE_SYMLINK:
+                            {
+                                if (fFlags & DNDURILIST_FLAGS_RESOLVE_SYMLINKS)
+                                {
+                                    char *pszSrc = RTPathRealDup(pcszDstBase);
+                                    if (pszSrc)
+                                    {
+                                        rc = RTPathQueryInfo(pszSrc, &objInfo, RTFSOBJATTRADD_NOTHING);
+                                        if (RT_SUCCESS(rc))
+                                        {
+                                            if (RTFS_IS_DIRECTORY(objInfo.Attr.fMode))
+                                            {
+                                                LogFlowFunc(("Directory entry is symlink to directory\n"));
+                                                rc = appendPathRecursive(pszSrc, pcszDstPath, pcszDstBase, cchDstBase, fFlags);
+                                            }
+                                            else if (RTFS_IS_FILE(objInfo.Attr.fMode))
+                                            {
+                                                LogFlowFunc(("Directory entry is symlink to file\n"));
+                                                rc = addEntry(pszSrc, &pcszDstPath[cchDstBase], fFlags);
+                                            }
+                                            else
+                                                rc = VERR_NOT_SUPPORTED;
+                                        }
+
+                                        RTStrFree(pszSrc);
+                                    }
+                                    else
+                                        rc = VERR_NO_MEMORY;
+                                }
+                                break;
+                            }
+
+                            default:
+                                break;
                         }
 
-                        default:
-                            break;
-                    }
+                    } while (RT_SUCCESS(rc));
 
-                } while (RT_SUCCESS(rc));
-
-                RTDirClose(hDir);
+                    RTDirReadExAFree(&pDirEntry, &cbDirEntry);
+                    RTDirClose(hDir);
+                }
             }
         }
         else if (RTFS_IS_FILE(objInfo.Attr.fMode))
@@ -255,7 +287,7 @@ int DnDURIList::AppendNativePath(const char *pszPath, uint32_t fFlags)
     {
         RTPathChangeToUnixSlashes(pszPathNative, true /* fForce */);
 
-        char *pszPathURI = RTUriCreate("file" /* pszScheme */, "/" /* pszAuthority */,
+        char *pszPathURI = RTUriCreate("file" /* pszScheme */, NULL /* pszAuthority */,
                                        pszPathNative, NULL /* pszQuery */, NULL /* pszFragment */);
         if (pszPathURI)
         {
@@ -304,17 +336,42 @@ int DnDURIList::AppendNativePathsFromList(const RTCList<RTCString> &lstNativePat
 int DnDURIList::AppendURIPath(const char *pszURI, uint32_t fFlags)
 {
     AssertPtrReturn(pszURI, VERR_INVALID_POINTER);
-
+    AssertReturn(!(fFlags & ~DNDURILIST_FLAGS_VALID_MASK), VERR_INVALID_FLAGS);
     /** @todo Check for string termination?  */
-#ifdef DEBUG_andy
-    LogFlowFunc(("pszPath=%s, fFlags=0x%x\n", pszURI, fFlags));
-#endif
-    int rc = VINF_SUCCESS;
 
-    /* Query the path component of a file URI. If this hasn't a
-     * file scheme NULL is returned. */
-    char *pszSrcPath = RTUriFilePath(pszURI, URI_FILE_FORMAT_AUTO);
-    if (pszSrcPath)
+    RTURIPARSED Parsed;
+    int rc = RTUriParse(pszURI, &Parsed);
+    if (RT_FAILURE(rc))
+        return rc;
+
+    char *pszSrcPath = NULL;
+
+    /* file://host.example.com/path/to/file.txt */
+    const char *pszParsedAuthority = RTUriParsedAuthority(pszURI, &Parsed);
+    if (   pszParsedAuthority
+        && pszParsedAuthority[0] != '\0') /* Authority present? */
+    {
+        const char *pszParsedPath = RTUriParsedPath(pszURI, &Parsed);
+        if (pszParsedPath)
+        {
+            /* Always use UNIXy paths internally. */
+            if (RTStrAPrintf(&pszSrcPath,  "//%s%s", pszParsedAuthority, pszParsedPath) == -1)
+                rc = VERR_NO_MEMORY;
+        }
+        else
+            rc = VERR_INVALID_PARAMETER;
+    }
+    else
+    {
+        pszSrcPath = RTUriFilePath(pszURI);
+        if (!pszSrcPath)
+            rc = VERR_INVALID_PARAMETER;
+    }
+
+    LogFlowFunc(("pszURI=%s, fFlags=0x%x -> pszParsedAuthority=%s, pszSrcPath=%s, rc=%Rrc\n",
+                 pszURI, fFlags, pszParsedAuthority ? pszParsedAuthority : "<None>", pszSrcPath, rc));
+
+    if (RT_SUCCESS(rc))
     {
         /* Add the path to our internal file list (recursive in
          * the case of a directory). */
@@ -331,7 +388,7 @@ int DnDURIList::AppendURIPath(const char *pszURI, uint32_t fFlags)
                 char *pszDstPath = &pszSrcPath[cchDstBase];
                 m_lstRoot.append(pszDstPath);
 
-                LogFlowFunc(("pszFilePath=%s, pszFileName=%s, pszRoot=%s\n",
+                LogFlowFunc(("pszSrcPath=%s, pszFileName=%s, pszRoot=%s\n",
                              pszSrcPath, pszFileName, pszDstPath));
 
                 rc = appendPathRecursive(pszSrcPath, pszSrcPath, pszSrcPath, cchDstBase, fFlags);
@@ -341,11 +398,9 @@ int DnDURIList::AppendURIPath(const char *pszURI, uint32_t fFlags)
         }
         else
             rc = VERR_INVALID_PARAMETER;
-
-        RTStrFree(pszSrcPath);
     }
-    else
-        rc = VERR_INVALID_PARAMETER;
+
+    RTStrFree(pszSrcPath);
 
     LogFlowFuncLeaveRC(rc);
     return rc;
@@ -383,8 +438,16 @@ int DnDURIList::AppendURIPathsFromList(const RTCList<RTCString> &lstURI,
 void DnDURIList::Clear(void)
 {
     m_lstRoot.clear();
+
+    for (size_t i = 0; i < m_lstTree.size(); i++)
+    {
+        DnDURIObject *pCurObj = m_lstTree.at(i);
+        AssertPtr(pCurObj);
+        RTMemFree(pCurObj);
+    }
     m_lstTree.clear();
 
+    m_cTotal  = 0;
     m_cbTotal = 0;
 }
 
@@ -393,22 +456,30 @@ void DnDURIList::RemoveFirst(void)
     if (m_lstTree.isEmpty())
         return;
 
-    DnDURIObject &curPath = m_lstTree.first();
+    DnDURIObject *pCurObj = m_lstTree.first();
+    AssertPtr(pCurObj);
 
-    uint64_t cbSize = curPath.GetSize();
+    uint64_t cbSize = pCurObj->GetSize();
     Assert(m_cbTotal >= cbSize);
     m_cbTotal -= cbSize; /* Adjust total size. */
 
+    pCurObj->Close();
+    RTMemFree(pCurObj);
+
     m_lstTree.removeFirst();
 }
 
 int DnDURIList::RootFromURIData(const void *pvData, size_t cbData, uint32_t fFlags)
 {
+    Assert(fFlags == 0); RT_NOREF1(fFlags);
     AssertPtrReturn(pvData, VERR_INVALID_POINTER);
     AssertReturn(cbData, VERR_INVALID_PARAMETER);
 
+    if (!RTStrIsValidEncoding(static_cast<const char *>(pvData)))
+        return VERR_INVALID_PARAMETER;
+
     RTCList<RTCString> lstURI =
-        RTCString(static_cast<const char*>(pvData), cbData - 1).split("\r\n");
+        RTCString(static_cast<const char *>(pvData), cbData - 1).split("\r\n");
     if (lstURI.isEmpty())
         return VINF_SUCCESS;
 
@@ -419,8 +490,7 @@ int DnDURIList::RootFromURIData(const void *pvData, size_t cbData, uint32_t fFla
         /* Query the path component of a file URI. If this hasn't a
          * file scheme, NULL is returned. */
         const char *pszURI = lstURI.at(i).c_str();
-        char *pszFilePath = RTUriFilePath(pszURI,
-                                          URI_FILE_FORMAT_AUTO);
+        char *pszFilePath = RTUriFilePath(pszURI);
 #ifdef DEBUG_andy
         LogFlowFunc(("pszURI=%s, pszFilePath=%s\n", pszURI, pszFilePath));
 #endif
@@ -446,7 +516,7 @@ int DnDURIList::RootFromURIData(const void *pvData, size_t cbData, uint32_t fFla
 }
 
 RTCString DnDURIList::RootToString(const RTCString &strPathBase /* = "" */,
-                                   const RTCString &strSeparator /* = "\r\n" */)
+                                   const RTCString &strSeparator /* = "\r\n" */) const
 {
     RTCString strRet;
     for (size_t i = 0; i < m_lstRoot.size(); i++)
@@ -464,7 +534,7 @@ RTCString DnDURIList::RootToString(const RTCString &strPathBase /* = "" */,
                 if (pszPathURI)
                 {
                     strRet += RTCString(pszPathURI) + strSeparator;
-                    LogFlowFunc(("URI: %s\n", strRet.c_str()));
+                    LogFlowFunc(("URI (Base): %s\n", strRet.c_str()));
                     RTStrFree(pszPathURI);
                 }
 
