@@ -27,10 +27,15 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
 {
     unsigned char *p = array->a[attr].p + index * array->a[attr].stride;
 
+#ifdef DEBUG_misha
+    Assert(index >= 0);
+#endif
+
 #ifdef CR_ARB_vertex_buffer_object
     if (array->a[attr].buffer && array->a[attr].buffer->data)
     {
-        p = (unsigned char *)(array->a[attr].buffer->data) + (unsigned long)p;
+        Assert(((uintptr_t)p) < array->a[attr].buffer->size);
+        p = (unsigned char *)(array->a[attr].buffer->data) + (uintptr_t)p;
     }
 #endif
 
@@ -103,8 +108,8 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
                     case 4:
                         crPackVertexAttrib4usvARB(attr, usPtr);
                         break;
-                    case 3: usv[2] = usPtr[2];
-                    case 2: usv[1] = usPtr[1];
+                    case 3: usv[2] = usPtr[2]; RT_FALL_THRU();
+                    case 2: usv[1] = usPtr[1]; RT_FALL_THRU();
                     case 1:
                         usv[0] = usPtr[0];
                         crPackVertexAttrib4usvARB(attr, usv);
@@ -142,8 +147,8 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
                     case 4:
                         crPackVertexAttrib4ivARB(attr, iPtr);
                         break;
-                    case 3: iv[2] = iPtr[2];
-                    case 2: iv[1] = iPtr[1];
+                    case 3: iv[2] = iPtr[2]; RT_FALL_THRU();
+                    case 2: iv[1] = iPtr[1]; RT_FALL_THRU();
                     case 1:
                         iv[0] = iPtr[0];
                         crPackVertexAttrib4ivARB(attr, iv);
@@ -181,8 +186,8 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
                     case 4:
                         crPackVertexAttrib4uivARB(attr, uiPtr);
                         break;
-                    case 3: uiv[2] = uiPtr[2];
-                    case 2: uiv[1] = uiPtr[1];
+                    case 3: uiv[2] = uiPtr[2]; RT_FALL_THRU();
+                    case 2: uiv[1] = uiPtr[1]; RT_FALL_THRU();
                     case 1:
                         uiv[0] = uiPtr[0];
                         crPackVertexAttrib4uivARB(attr, uiv);
@@ -238,8 +243,8 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
                     case 4:
                         crPackVertexAttrib4bvARB(attr, bPtr);
                         break;
-                    case 3: bv[2] = bPtr[2];
-                    case 2: bv[1] = bPtr[1];
+                    case 3: bv[2] = bPtr[2]; RT_FALL_THRU();
+                    case 2: bv[1] = bPtr[1]; RT_FALL_THRU();
                     case 1:
                         bv[0] = bPtr[0];
                         crPackVertexAttrib4bvARB(attr, bv);
@@ -277,8 +282,8 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
                     case 4:
                         crPackVertexAttrib4ubvARB(attr, ubPtr);
                         break;
-                    case 3: ubv[2] = ubPtr[2];
-                    case 2: ubv[1] = ubPtr[1];
+                    case 3: ubv[2] = ubPtr[2]; RT_FALL_THRU();
+                    case 2: ubv[1] = ubPtr[1]; RT_FALL_THRU();
                     case 1:
                         ubv[0] = ubPtr[0];
                         crPackVertexAttrib4ubvARB(attr, ubv);
@@ -297,12 +302,12 @@ static void crPackVertexAttrib(const CRVertexArrays *array, unsigned int attr, G
  * Expand glArrayElement into crPackVertex/Color/Normal/etc.
  */
 void
-crPackExpandArrayElement(GLint index, CRClientState *c)
+crPackExpandArrayElement(GLint index, CRClientState *c, const GLfloat *pZva)
 {
     unsigned char *p;
     unsigned int unit, attr;
     const CRVertexArrays *array = &(c->array);
-    const GLboolean vpEnabled = crStateGetCurrent()->program.vpEnabled;
+    const GLboolean vpEnabled = crStateGetCurrent(g_pStateTracker)->program.vpEnabled;
 
     /*crDebug("crPackExpandArrayElement(%i)", index);*/
 
@@ -313,7 +318,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->n.buffer && array->n.buffer->data)
         {
-            p = (unsigned char *)(array->n.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->n.buffer->data) + (uintptr_t)p;
         }
 #endif
 
@@ -336,7 +341,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->c.buffer && array->c.buffer->data)
         {
-            p = (unsigned char *)(array->c.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->c.buffer->data) + (uintptr_t)p;
         }
 #endif
 
@@ -411,7 +416,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->s.buffer && array->s.buffer->data)
         {
-            p = (unsigned char *)(array->s.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->s.buffer->data) + (uintptr_t)p;
         }
 #endif
 
@@ -437,7 +442,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
                 crWarning("Unhandled: crPackExpandArrayElement, array->s.type 0x%x", array->s.type);
         }
     }
-#endif // CR_EXT_secondary_color
+#endif /* CR_EXT_secondary_color */
 
 
 #ifdef CR_EXT_fog_coord
@@ -448,12 +453,12 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->f.buffer && array->f.buffer->data)
         {
-            p = (unsigned char *)(array->f.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->f.buffer->data) + (uintptr_t)p;
         }
 #endif
         crPackFogCoordfEXT( *((GLfloat *) p) );
     }
-#endif // CR_EXT_fog_coord
+#endif /* CR_EXT_fog_coord */
 
     for (unit = 0 ; unit < CR_MAX_TEXTURE_UNITS ; unit++)
     {
@@ -464,7 +469,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
             if (array->t[unit].buffer && array->t[unit].buffer->data)
             {
-                p = (unsigned char *)(array->t[unit].buffer->data) + (unsigned long)p;
+                p = (unsigned char *)(array->t[unit].buffer->data) + (uintptr_t)p;
             }
 #endif
 
@@ -519,7 +524,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->i.buffer && array->i.buffer->data)
         {
-            p = (unsigned char *)(array->i.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->i.buffer->data) + (uintptr_t)p;
         }
 #endif
 
@@ -541,7 +546,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->e.buffer && array->e.buffer->data)
         {
-            p = (unsigned char *)(array->e.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->e.buffer->data) + (uintptr_t)p;
         }
 #endif
 
@@ -560,6 +565,10 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
     {
         crPackVertexAttrib(array, VERT_ATTRIB_POS, index);
     }
+    else if (pZva)
+    {
+        crPackVertexAttrib4fvARB(VERT_ATTRIB_POS, pZva);
+    }
     else if (array->v.enabled)
     {
         p = array->v.p + index * array->v.stride;
@@ -567,7 +576,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 #ifdef CR_ARB_vertex_buffer_object
         if (array->v.buffer && array->v.buffer->data)
         {
-            p = (unsigned char *)(array->v.buffer->data) + (unsigned long)p;
+            p = (unsigned char *)(array->v.buffer->data) + (uintptr_t)p;
         }
 #endif
         switch (array->v.type)
@@ -612,7 +621,7 @@ crPackExpandArrayElement(GLint index, CRClientState *c)
 
 
 void
-crPackExpandDrawArrays(GLenum mode, GLint first, GLsizei count, CRClientState *c)
+crPackExpandDrawArrays(GLenum mode, GLint first, GLsizei count, CRClientState *c, const GLfloat *pZva)
 {
     int i;
 
@@ -631,7 +640,7 @@ crPackExpandDrawArrays(GLenum mode, GLint first, GLsizei count, CRClientState *c
     crPackBegin(mode);
     for (i=0; i<count; i++)
     {
-        crPackExpandArrayElement(first + i, c);
+        crPackExpandArrayElement(first + i, c, pZva);
     }
     crPackEnd();
 }
@@ -659,11 +668,11 @@ crPackDrawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid *indice
     int packet_length = sizeof(int) + sizeof(mode) + sizeof(count) + sizeof(type) + sizeof(GLuint);
     GLsizei indexsize;
 #ifdef CR_ARB_vertex_buffer_object
-    CRBufferObject *elementsBuffer = crStateGetCurrent()->bufferobject.elementsBuffer;
+    CRBufferObject *elementsBuffer = crStateGetCurrent(g_pStateTracker)->bufferobject.elementsBuffer;
     packet_length += sizeof(GLint);
     if (elementsBuffer && elementsBuffer->id)
     {
-        /*@todo not sure it's possible, and not sure what to do*/
+        /** @todo not sure it's possible, and not sure what to do*/
         if (!elementsBuffer->data)
         {
             crWarning("crPackDrawElements: trying to use bound but empty elements buffer, ignoring.");
@@ -698,7 +707,7 @@ crPackDrawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid *indice
 
 void PACK_APIENTRY
 crPackDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei count,
-                                                GLenum type, const GLvoid *indices)
+                        GLenum type, const GLvoid *indices)
 {
     unsigned char *data_ptr, *start_ptr;
     int packet_length = sizeof(int) + sizeof(mode) + sizeof(start)
@@ -706,11 +715,11 @@ crPackDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei count,
     GLsizei indexsize;
 
 #ifdef CR_ARB_vertex_buffer_object
-    CRBufferObject *elementsBuffer = crStateGetCurrent()->bufferobject.elementsBuffer;
+    CRBufferObject *elementsBuffer = crStateGetCurrent(g_pStateTracker)->bufferobject.elementsBuffer;
     packet_length += sizeof(GLint);
     if (elementsBuffer && elementsBuffer->id)
     {
-        /*@todo not sure it's possible, and not sure what to do*/
+        /** @todo not sure it's possible, and not sure what to do*/
         if (!elementsBuffer->data)
         {
             crWarning("crPackDrawElements: trying to use bound but empty elements buffer, ignoring.");
@@ -752,12 +761,12 @@ crPackDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei count,
  */
 void
 crPackExpandDrawElements(GLenum mode, GLsizei count, GLenum type,
-                                                 const GLvoid *indices, CRClientState *c)
+                         const GLvoid *indices, CRClientState *c, const GLfloat *pZva)
 {
     int i;
     GLubyte *p = (GLubyte *)indices;
 #ifdef CR_ARB_vertex_buffer_object
-    CRBufferObject *elementsBuffer = crStateGetCurrent()->bufferobject.elementsBuffer;
+    CRBufferObject *elementsBuffer = crStateGetCurrent(g_pStateTracker)->bufferobject.elementsBuffer;
 #endif
 
     if (count < 0)
@@ -786,34 +795,34 @@ crPackExpandDrawElements(GLenum mode, GLsizei count, GLenum type,
 #ifdef CR_ARB_vertex_buffer_object
     if (elementsBuffer && elementsBuffer->data)
     {
-        p = (unsigned char *)(elementsBuffer->data) + (unsigned long)p;
+        p = (unsigned char *)(elementsBuffer->data) + (uintptr_t)p;
     }
 #endif
 
     if (mode != 999)
         crPackBegin(mode);
 
-    //crDebug("crPackExpandDrawElements mode:0x%x, count:%d, type:0x%x", mode, count, type);
+    /*crDebug("crPackExpandDrawElements mode:0x%x, count:%d, type:0x%x", mode, count, type);*/
 
     switch (type)
     {
         case GL_UNSIGNED_BYTE:
             for (i=0; i<count; i++)
             {
-                crPackExpandArrayElement((GLint) *p++, c);
+                crPackExpandArrayElement((GLint) *p++, c, pZva);
             }
             break;
         case GL_UNSIGNED_SHORT:
             for (i=0; i<count; i++)
             {
-                crPackExpandArrayElement((GLint) * (GLushort *) p, c);
+                crPackExpandArrayElement((GLint) * (GLushort *) p, c, pZva);
                 p+=sizeof (GLushort);
             }
             break;
         case GL_UNSIGNED_INT:
             for (i=0; i<count; i++)
             {
-                crPackExpandArrayElement((GLint) * (GLuint *) p, c);
+                crPackExpandArrayElement((GLint) * (GLuint *) p, c, pZva);
                 p+=sizeof (GLuint);
             }
             break;
@@ -871,7 +880,7 @@ crPackUnrollDrawElements(GLsizei count, GLenum type,
  * glDrawRangeElements, expanded into crPackBegin/Vertex/End/etc.
  */
 void
-crPackExpandDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid *indices, CRClientState *c)
+crPackExpandDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid *indices, CRClientState *c, const GLfloat *pZva)
 {
     if (start>end)
     {
@@ -879,7 +888,7 @@ crPackExpandDrawRangeElements(GLenum mode, GLuint start, GLuint end, GLsizei cou
         return;
     }
 
-    crPackExpandDrawElements(mode, count, type, indices, c);
+    crPackExpandDrawElements(mode, count, type, indices, c, pZva);
 }
 
 
@@ -905,12 +914,12 @@ crPackMultiDrawArraysEXT( GLenum mode, GLint *first, GLsizei *count,
  */
 void
 crPackExpandMultiDrawArraysEXT( GLenum mode, GLint *first, GLsizei *count,
-                                                                GLsizei primcount, CRClientState *c )
+                                                                GLsizei primcount, CRClientState *c, const GLfloat *pZva )
 {
    GLint i;
    for (i = 0; i < primcount; i++) {
       if (count[i] > 0) {
-         crPackExpandDrawArrays(mode, first[i], count[i], c);
+         crPackExpandDrawArrays(mode, first[i], count[i], c, pZva);
       }
    }
 }
@@ -938,12 +947,12 @@ crPackMultiDrawElementsEXT( GLenum mode, const GLsizei *count, GLenum type,
 void
 crPackExpandMultiDrawElementsEXT( GLenum mode, const GLsizei *count,
                                   GLenum type, const GLvoid **indices,
-                                  GLsizei primcount, CRClientState *c )
+                                  GLsizei primcount, CRClientState *c, const GLfloat *pZva )
 {
    GLint i;
    for (i = 0; i < primcount; i++) {
       if (count[i] > 0) {
-         crPackExpandDrawElements(mode, count[i], type, indices[i], c);
+         crPackExpandDrawElements(mode, count[i], type, indices[i], c, pZva);
       }
    }
 }
@@ -1055,7 +1064,7 @@ static void crPackLockClientPointer(GLint first, GLint count, unsigned char **pp
 
 void PACK_APIENTRY crPackLockArraysEXT(GLint first, GLint count)
 {
-    CRContext *g = crStateGetCurrent();
+    CRContext *g = crStateGetCurrent(g_pStateTracker);
     CRClientState *c = &g->client;
     unsigned char *data_ptr, *start_ptr;
     int packet_length = sizeof(int); /*extopcode*/
