@@ -18,7 +18,7 @@ void PACK_APIENTRY crPackDrawPixels(GLsizei width, GLsizei height,
 {
     unsigned char *data_ptr;
     int packet_length, imagesize;
-    int noimagedata = (pixels == NULL) || crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB);
+    int noimagedata = (pixels == NULL) || crStateIsBufferBound(g_pStateTracker, GL_PIXEL_UNPACK_BUFFER_ARB);
 
     packet_length = 
         sizeof( width ) +
@@ -103,6 +103,7 @@ void PACK_APIENTRY crPackReadPixels(GLint x, GLint y, GLsizei width,
     WRITE_DATA( 44, GLint, packstate->rowLength );
     WRITE_NETWORK_POINTER( 48, (char *) pixels );
     WRITE_OPCODE( pc, CR_READPIXELS_OPCODE );
+    CR_CMDBLOCK_CHECK_FLUSH(pc);
     CR_UNLOCK_PACKER_CONTEXT(pc);
 }
 
@@ -113,10 +114,10 @@ void PACK_APIENTRY crPackBitmap(GLsizei width, GLsizei height,
                                 GLfloat xorig, GLfloat yorig, GLfloat xmove, GLfloat ymove,
                                 const GLubyte *bitmap, const CRPixelPackState *unpack )
 {
-    const int noimagedata = (bitmap == NULL) || crStateIsBufferBound(GL_PIXEL_UNPACK_BUFFER_ARB);
+    const int noimagedata = (bitmap == NULL) || crStateIsBufferBound(g_pStateTracker, GL_PIXEL_UNPACK_BUFFER_ARB);
     unsigned char *data_ptr;
     int data_length = 0;
-    GLubyte *destBitmap = NULL;
+    /*GLubyte *destBitmap = NULL; - unused */
     int packet_length = 
         sizeof( width ) + 
         sizeof( height ) +
@@ -208,7 +209,7 @@ crPackGetTexImage( GLenum target, GLint level, GLenum format, GLenum type,
 {
     CR_GET_PACKER_CONTEXT(pc);
     unsigned char *data_ptr;
-    (void) pc;
+    (void) pc; (void) packstate;
     CR_GET_BUFFERED_POINTER( pc, 40 );
     WRITE_DATA( 0, GLint, 40 );
     WRITE_DATA( 4, GLenum, CR_GETTEXIMAGE_EXTEND_OPCODE );
@@ -219,5 +220,6 @@ crPackGetTexImage( GLenum target, GLint level, GLenum format, GLenum type,
     WRITE_NETWORK_POINTER( 24, (void *) pixels );
     WRITE_NETWORK_POINTER( 32, (void *) writeback );
     WRITE_OPCODE( pc, CR_EXTEND_OPCODE );
+    CR_CMDBLOCK_CHECK_FLUSH(pc);
     CR_UNLOCK_PACKER_CONTEXT(pc);
 }
