@@ -5,34 +5,23 @@
 
 # This script generates the packer.c file from the gl_header.parsed file.
 
+from __future__ import print_function
 import sys, string, re
 
 import apiutil
 
 
 
-def WriteData( offset, arg_type, arg_name, is_swapped ):
+def WriteData( offset, arg_type, arg_name ):
     """Return a string to write a variable to the packing buffer."""
     retval = 9
     if apiutil.IsPointer(arg_type):
-        retval = "\tWRITE_NETWORK_POINTER( %d, (void *) %s );" % (offset, arg_name )
+        retval = "\tWRITE_NETWORK_POINTER(%d, (void *) %s);" % (offset, arg_name )
     else:   
-        if is_swapped:
-            if arg_type == "GLfloat" or arg_type == "GLclampf":
-                retval = "\tWRITE_DATA( %d, GLuint, SWAPFLOAT(%s) );" % (offset, arg_name)
-            elif arg_type == "GLdouble" or arg_type == "GLclampd":
-                retval = "\tWRITE_SWAPPED_DOUBLE( %d, %s );" % (offset, arg_name)
-            elif apiutil.sizeof(arg_type) == 1:
-                retval = "\tWRITE_DATA( %d, %s, %s );" % (offset, arg_type, arg_name)
-            elif apiutil.sizeof(arg_type) == 2:
-                retval = "\tWRITE_DATA( %d, %s, SWAP16(%s) );" % (offset, arg_type, arg_name)
-            elif apiutil.sizeof(arg_type) == 4:
-                retval = "\tWRITE_DATA( %d, %s, SWAP32(%s) );" % (offset, arg_type, arg_name)
+        if arg_type == "GLdouble" or arg_type == "GLclampd":
+            retval = "\tWRITE_DOUBLE(%d, %s);" % (offset, arg_name)
         else:
-            if arg_type == "GLdouble" or arg_type == "GLclampd":
-                retval = "\tWRITE_DOUBLE( %d, %s );" % (offset, arg_name)
-            else:
-                retval = "\tWRITE_DATA( %d, %s, %s );" % (offset, arg_type, arg_name)
+            retval = "\tWRITE_DATA(%d, %s, %s);" % (offset, arg_type, arg_name)
     if retval == 9:
         print >>sys.stderr, "no retval for %s %s" % (arg_name, arg_type)
         assert 0
@@ -45,7 +34,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = '%s%s' % (k[:1].lower(),k[1:])
         type = m.group(3) + m.group(2)
-        print "\tpc->current.c.%s.%s = data_ptr;" % (name,type)
+        print("\tpc->current.c.%s.%s = data_ptr;" % (name,type))
         return
 
     m = re.search( r"^(SecondaryColor)(3)(ub|b|us|s|ui|i|f|d)EXT$", func_name )
@@ -53,7 +42,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = 'secondaryColor'
         type = m.group(3) + m.group(2)
-        print "\tpc->current.c.%s.%s = data_ptr;" % (name,type)
+        print("\tpc->current.c.%s.%s = data_ptr;" % (name,type))
         return
 
     m = re.search( r"^(TexCoord)([1234])(ub|b|us|s|ui|i|f|d)$", func_name )
@@ -61,7 +50,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = 'texCoord'
         type = m.group(3) + m.group(2)
-        print "\tpc->current.c.%s.%s[0] = data_ptr;" % (name,type)
+        print("\tpc->current.c.%s.%s[0] = data_ptr;" % (name,type))
         return
 
     m = re.search( r"^(MultiTexCoord)([1234])(ub|b|us|s|ui|i|f|d)ARB$", func_name )
@@ -69,7 +58,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = 'texCoord'
         type = m.group(3) + m.group(2)
-        print "\tpc->current.c.%s.%s[texture-GL_TEXTURE0_ARB] = data_ptr + 4;" % (name,type)
+        print("\tpc->current.c.%s.%s[texture-GL_TEXTURE0_ARB] = data_ptr + 4;" % (name,type))
         return
 
     m = re.match( r"^(Index)(ub|b|us|s|ui|i|f|d)$", func_name )
@@ -77,7 +66,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = 'index'
         type = m.group(2) + "1"
-        print "\tpc->current.c.%s.%s = data_ptr;" % (name,type)
+        print("\tpc->current.c.%s.%s = data_ptr;" % (name,type))
         return
 
     m = re.match( r"^(EdgeFlag)$", func_name )
@@ -85,7 +74,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = 'edgeFlag'
         type = "l1"
-        print "\tpc->current.c.%s.%s = data_ptr;" % (name,type)
+        print("\tpc->current.c.%s.%s = data_ptr;" % (name,type))
         return
 
     m = re.match( r"^(FogCoord)(f|d)EXT$", func_name )
@@ -93,7 +82,7 @@ def UpdateCurrentPointer( func_name ):
         k = m.group(1)
         name = 'fogCoord'
         type = m.group(2) + "1"
-        print "\tpc->current.c.%s.%s = data_ptr;" % (name,type)
+        print("\tpc->current.c.%s.%s = data_ptr;" % (name,type))
         return
 
 
@@ -103,21 +92,19 @@ def UpdateCurrentPointer( func_name ):
         name = 'vertexAttrib'
         type = m.group(3) + m.group(2)
         # Add 12 to skip the packet length, opcode and index fields
-        print "\tpc->current.c.%s.%s[index] = data_ptr + 12;" % (name,type)
+        print("\tpc->current.c.%s.%s[index] = data_ptr + 4;" % (name,type))
         if m.group(4) == "ARB" or m.group(4) == "NV":
-            print "\tpc->current.attribsUsedMask |= (1 << index);"
+            print("\tpc->current.attribsUsedMask |= (1 << index);")
+            print("\tpc->current.changedVertexAttrib |= (1 << index);")
         return
 
 
 
-def PrintFunc( func_name, params, is_swapped, can_have_pointers ):
+def PrintFunc( func_name, params, can_have_pointers ):
     """Emit a packer function."""
-    if is_swapped:
-        print 'void PACK_APIENTRY crPack%sSWAP( %s )' % (func_name, apiutil.MakeDeclarationStringWithContext('CR_PACKER_CONTEXT', params))
-    else:
-        print 'void PACK_APIENTRY crPack%s( %s )' % (func_name, apiutil.MakeDeclarationStringWithContext('CR_PACKER_CONTEXT', params))
-    print '{'
-    print '\tCR_GET_PACKER_CONTEXT(pc);'
+    print('void PACK_APIENTRY crPack%s(%s)' % (func_name, apiutil.MakeDeclarationStringWithContext('CR_PACKER_CONTEXT', params)))
+    print('{')
+    print('\tCR_GET_PACKER_CONTEXT(pc);')
 
     # Save original function name
     orig_func_name = func_name
@@ -136,10 +123,10 @@ def PrintFunc( func_name, params, is_swapped, can_have_pointers ):
             bail_out = 1
     if bail_out:
         for (name, type, vecSize) in nonVecParams:
-            print '\t(void)%s;' % (name)
-        print '\tcrError ( "%s needs to be special cased %d %d!");' % (func_name, vecSize, can_have_pointers)
-        print '\t(void) pc;'
-        print '}'
+            print('\t(void)%s;' % (name))
+        print('\tcrError ( "%s needs to be special cased %d %d!");' % (func_name, vecSize, can_have_pointers))
+        print('\t(void) pc;')
+        print('}')
         # XXX we should really abort here
         return
 
@@ -149,40 +136,40 @@ def PrintFunc( func_name, params, is_swapped, can_have_pointers ):
         is_extended = 0
 
 
-    print "\tunsigned char *data_ptr;"
-    print '\t(void) pc;'
+    print("\tunsigned char *data_ptr = NULL;")
+    print('\t(void) pc;')
     #if func_name == "Enable" or func_name == "Disable":
     #   print "\tCRASSERT(!pc->buffer.geometry_only); /* sanity check */"
 
     for index in range(0,len(params)):
         (name, type, vecSize) = params[index]
         if vecSize>0 and func_name!=orig_func_name:
-            print "    if (!%s) {" % name
+            print("    if (!%s) {" % name)
             # Know the reason for this one, so avoid the spam.
             if orig_func_name != "SecondaryColor3fvEXT":
-                print "        crDebug(\"App passed NULL as %s for %s\");" % (name, orig_func_name)
-            print "        return;"
-            print "    }"
+                print("        crDebug(\"App passed NULL as %s for %s\");" % (name, orig_func_name))
+            print("        return;")
+            print("    }")
 
     packet_length = apiutil.PacketLength(nonVecParams)
 
     if packet_length == 0 and not is_extended:
-        print "\tCR_GET_BUFFERED_POINTER_NO_ARGS( pc );"
+        print("\tCR_GET_BUFFERED_POINTER_NO_ARGS(pc);")
     elif func_name[:9] == "Translate" or func_name[:5] == "Color":
         # XXX WTF is the purpose of this?
         if is_extended:
             packet_length += 8
-        print "\tCR_GET_BUFFERED_POINTER_NO_BEGINEND_FLUSH( pc, %d, GL_TRUE );" % packet_length
+        print("\tCR_GET_BUFFERED_POINTER_NO_BEGINEND_FLUSH(pc, %d, GL_TRUE);" % packet_length)
     else:
         if is_extended:
             packet_length += 8
-        print "\tCR_GET_BUFFERED_POINTER( pc, %d );" % packet_length
+        print("\tCR_GET_BUFFERED_POINTER(pc, %d);" % packet_length)
     UpdateCurrentPointer( func_name )
 
     if is_extended:
         counter = 8
-        print WriteData( 0, 'GLint', packet_length, is_swapped )
-        print WriteData( 4, 'GLenum', apiutil.ExtendedOpcodeName( func_name ), is_swapped )
+        print(WriteData( 0, 'GLint', packet_length ))
+        print(WriteData( 4, 'GLenum', apiutil.ExtendedOpcodeName( func_name )))
     else:
         counter = 0
 
@@ -193,11 +180,11 @@ def PrintFunc( func_name, params, is_swapped, can_have_pointers ):
         if vecSize > 0 and func_name != orig_func_name:
             ptrType = apiutil.PointerType(type)
             for i in range(0, vecSize):
-                print WriteData( counter + i * apiutil.sizeof(ptrType),
-                                 ptrType, "%s[%d]" % (name, i), is_swapped )
+                print(WriteData( counter + i * apiutil.sizeof(ptrType),
+                                 ptrType, "%s[%d]" % (name, i)))
             # XXX increment counter here?
         else:
-            print WriteData( counter, type, name, is_swapped )
+            print(WriteData( counter, type, name))
             if apiutil.IsPointer(type):
                 counter += apiutil.PointerSize()
             else:
@@ -205,19 +192,23 @@ def PrintFunc( func_name, params, is_swapped, can_have_pointers ):
 
     # finish up
     if is_extended:
-        print "\tWRITE_OPCODE( pc, CR_EXTEND_OPCODE );"
+        print("\tWRITE_OPCODE(pc, CR_EXTEND_OPCODE);")
     else:
-        print "\tWRITE_OPCODE( pc, %s );" % apiutil.OpcodeName( func_name )
+        print("\tWRITE_OPCODE(pc, %s);" % apiutil.OpcodeName( func_name ))
 
-    print '\tCR_UNLOCK_PACKER_CONTEXT(pc);'
-    print '}\n'
+    if "get" in apiutil.Properties(func_name):
+        print('\tCR_CMDBLOCK_CHECK_FLUSH(pc);')
+
+    print('\tCR_UNLOCK_PACKER_CONTEXT(pc);')
+    print('}\n')
 
 
+r0_funcs = [ 'ChromiumParameteriCR', 'WindowSize', 'WindowShow', 'WindowPosition' ]
 
 
 apiutil.CopyrightC()
 
-print """
+print("""
 /* DO NOT EDIT - THIS FILE GENERATED BY THE packer.py SCRIPT */
 
 /* For each of the OpenGL functions we have a packer function which
@@ -227,7 +218,7 @@ print """
 #include "packer.h"
 #include "cr_opcodes.h"
 
-"""
+""")
 
 
 keys = apiutil.GetDispatchedFunctions(sys.argv[1]+"/APIspec.txt")
@@ -246,7 +237,7 @@ for func_name in keys:
 
     if return_type != 'void':
         # Yet another gross hack for glGetString
-        if string.find( return_type, '*' ) == -1:
+        if return_type.find('*') == -1:
             return_type = return_type + " *"
         params.append(("return_value", return_type, 0))
 
@@ -257,5 +248,11 @@ for func_name in keys:
     if func_name == 'Writeback':
         pointers_ok = 1
 
-    PrintFunc( func_name, params, 0, pointers_ok )
-    PrintFunc( func_name, params, 1, pointers_ok )
+    if not func_name in r0_funcs:
+        print('#ifndef IN_RING0')
+        
+    PrintFunc( func_name, params, pointers_ok )
+
+    if not func_name in r0_funcs:
+        print('#endif')
+    
