@@ -1,10 +1,10 @@
-/* $Id$ */
-
+/* $Id: blitter.cpp $ */
 /** @file
  * Blitter API implementation
  */
+
 /*
- * Copyright (C) 2013 Oracle Corporation
+ * Copyright (C) 2013-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,38 +14,312 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
-#include "cr_blitter.h"
-#include "cr_spu.h"
-#include "chromium.h"
-#include "cr_error.h"
-#include "cr_net.h"
-#include "cr_rand.h"
-#include "cr_mem.h"
-#include "cr_string.h"
+
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
+#ifdef IN_VMSVGA3D
+# include <OpenGL/OpenGL.h>
+# include <OpenGL/gl3.h>
+# include "../include/cr_blitter.h"
+# include <iprt/assert.h>
+# define WARN       AssertMsgFailed
+# define CRASSERT   Assert
+DECLINLINE(void) crWarning(const char *format, ... ) {}
+#else
+# include "cr_blitter.h"
+# include "cr_spu.h"
+# include "chromium.h"
+# include "cr_error.h"
+# include "cr_net.h"
+# include "cr_mem.h"
+# include "cr_string.h"
+# include "cr_bmpscale.h"
+#endif
 
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
 #include <iprt/mem.h>
 
-/* @param pCtxBase      - contains the blitter context info. Its value is treated differently depending on the fCreateNewCtx value
- * @param fCreateNewCtx - if true  - the pCtxBase must NOT be NULL. its visualBits is used as a visual bits info for the new context,
- *                                   its id field is used to specified the shared context id to be used for blitter context.
- *                                   The id can be null to specify no shared context is needed
- *                        if false - if pCtxBase is NOT null AND its id field is NOT null -
- *                                     specified the blitter context to be used
- *                                     blitter treats it as if it has default ogl state.
- *                                   otherwise -
- *                                     the blitter works in a "no-context" mode, i.e. a caller is responsible
- *                                     to making a proper context current before calling the blitter.
- *                                     Note that BltEnter/Leave MUST still be called, but the proper context
- *                                     must be set before doing BltEnter, and ResoreContext info is ignored in that case.
- *                                     Also note that blitter caches the current window info, and assumes the current context's values are preserved
- *                                     wrt that window before the calls, so if one uses different contexts for one blitter,
- *                                     the blitter current window values must be explicitly reset by doing CrBltMuralSetCurrentInfo(pBlitter, NULL)
- * @param fForceDrawBlt - if true  - forces the blitter to always use glDrawXxx-based blits even if GL_EXT_framebuffer_blit.
- *                                   This is needed because BlitFramebufferEXT is known to be often buggy, and glDrawXxx-based blits appear to be more reliable
+
+
+static void crMClrFillMem(uint32_t *pu32Dst, int32_t cbDstPitch, uint32_t width, uint32_t height, uint32_t u32Color)
+{
+    for (uint32_t i = 0; i < height; ++i)
+    {
+        for (uint32_t j = 0; j < width; ++j)
+        {
+            pu32Dst[j] = u32Color;
+        }
+
+        pu32Dst = (uint32_t*)(((uint8_t*)pu32Dst) + cbDstPitch);
+    }
+}
+
+void CrMClrFillImgRect(CR_BLITTER_IMG *pDst, const RTRECT *pCopyRect, uint32_t u32Color)
+{
+    int32_t x = pCopyRect->xLeft;
+    int32_t y = pCopyRect->yTop;
+    int32_t width = pCopyRect->xRight - pCopyRect->xLeft;
+    int32_t height = pCopyRect->yBottom - pCopyRect->yTop;
+    Assert(x >= 0);
+    Assert(y >= 0);
+    uint8_t *pu8Dst = ((uint8_t*)pDst->pvData) + pDst->pitch * y + x * 4;
+
+    crMClrFillMem((uint32_t*)pu8Dst, pDst->pitch, width, height, u32Color);
+}
+
+void CrMClrFillImg(CR_BLITTER_IMG *pImg, uint32_t cRects, const RTRECT *pRects, uint32_t u32Color)
+{
+    RTRECT Rect;
+    Rect.xLeft = 0;
+    Rect.yTop = 0;
+    Rect.xRight = pImg->width;
+    Rect.yBottom = pImg->height;
+
+
+    RTRECT Intersection;
+    /*const RTPOINT ZeroPoint = {0, 0}; - unused */
+
+    for (uint32_t i = 0; i < cRects; ++i)
+    {
+        const RTRECT * pRect = &pRects[i];
+        VBoxRectIntersected(pRect, &Rect, &Intersection);
+
+        if (VBoxRectIsZero(&Intersection))
+            continue;
+
+        CrMClrFillImgRect(pImg, &Intersection, u32Color);
+    }
+}
+
+static void crMBltMem(const uint8_t *pu8Src, int32_t cbSrcPitch, uint8_t *pu8Dst, int32_t cbDstPitch, uint32_t width, uint32_t height)
+{
+    uint32_t cbCopyRow = width * 4;
+
+    for (uint32_t i = 0; i < height; ++i)
+    {
+        memcpy(pu8Dst, pu8Src, cbCopyRow);
+
+        pu8Src += cbSrcPitch;
+        pu8Dst += cbDstPitch;
+    }
+}
+
+void CrMBltImgRect(const CR_BLITTER_IMG *pSrc, const RTPOINT *pSrcDataPoint, bool fSrcInvert, const RTRECT *pCopyRect, CR_BLITTER_IMG *pDst)
+{
+    int32_t srcX = pCopyRect->xLeft - pSrcDataPoint->x;
+    int32_t srcY = pCopyRect->yTop - pSrcDataPoint->y;
+    Assert(srcX >= 0);
+    Assert(srcY >= 0);
+    Assert(srcX < (int32_t)pSrc->width);
+    Assert(srcY < (int32_t)pSrc->height);
+
+    int32_t dstX = pCopyRect->xLeft;
+    int32_t dstY = pCopyRect->yTop;
+    Assert(dstX >= 0);
+    Assert(dstY >= 0);
+    Assert(dstX < (int32_t)pDst->width);
+    Assert(dstY < (int32_t)pDst->height);
+
+    uint8_t *pu8Src = ((uint8_t*)pSrc->pvData) + pSrc->pitch * (!fSrcInvert ? srcY : pSrc->height - srcY - 1) + srcX * 4;
+    uint8_t *pu8Dst = ((uint8_t*)pDst->pvData) + pDst->pitch * dstY + dstX * 4;
+
+    crMBltMem(pu8Src, fSrcInvert ? -((int32_t)pSrc->pitch) : (int32_t)pSrc->pitch, pu8Dst, pDst->pitch, pCopyRect->xRight - pCopyRect->xLeft, pCopyRect->yBottom - pCopyRect->yTop);
+}
+
+void CrMBltImg(const CR_BLITTER_IMG *pSrc, const RTPOINT *pPos, uint32_t cRects, const RTRECT *pRects, CR_BLITTER_IMG *pDst)
+{
+    RTRECT Intersection;
+    RTRECT RestrictSrcRect;
+    RestrictSrcRect.xLeft = 0;
+    RestrictSrcRect.yTop = 0;
+    RestrictSrcRect.xRight = pSrc->width;
+    RestrictSrcRect.yBottom = pSrc->height;
+    RTRECT RestrictDstRect;
+    RestrictDstRect.xLeft = 0;
+    RestrictDstRect.yTop = 0;
+    RestrictDstRect.xRight = pDst->width;
+    RestrictDstRect.yBottom = pDst->height;
+
+    for (uint32_t i = 0; i < cRects; ++i)
+    {
+        const RTRECT * pRect = &pRects[i];
+        VBoxRectIntersected(pRect, &RestrictDstRect, &Intersection);
+
+        RTRECT TranslatedSrc;
+        VBoxRectTranslated(&RestrictSrcRect, pPos->x, pPos->y, &TranslatedSrc);
+
+        VBoxRectIntersect(&Intersection, &TranslatedSrc);
+
+        if (VBoxRectIsZero(&Intersection))
+            continue;
+
+        CrMBltImgRect(pSrc, pPos, false, &Intersection, pDst);
+    }
+}
+
+#ifndef IN_VMSVGA3D
+
+void CrMBltImgRectScaled(const CR_BLITTER_IMG *pSrc, const RTPOINT *pPos, bool fSrcInvert, const RTRECT *pCopyRect, float strX, float strY, CR_BLITTER_IMG *pDst)
+{
+    RTPOINT UnscaledPos;
+    UnscaledPos.x = CR_FLOAT_RCAST(int32_t, pPos->x / strX);
+    UnscaledPos.y = CR_FLOAT_RCAST(int32_t, pPos->y / strY);
+
+    RTRECT UnscaledCopyRect;
+
+    VBoxRectUnscaled(pCopyRect, strX, strY, &UnscaledCopyRect);
+
+    if (VBoxRectIsZero(&UnscaledCopyRect))
+    {
+        WARN(("ups"));
+        return;
+    }
+
+    int32_t srcX = UnscaledCopyRect.xLeft - UnscaledPos.x;
+    int32_t srcY = UnscaledCopyRect.yTop - UnscaledPos.y;
+    if (srcX < 0)
+    {
+        WARN(("ups"));
+        srcX = 0;
+    }
+    if (srcY < 0)
+    {
+        WARN(("ups"));
+        srcY = 0;
+    }
+
+    if ((GLuint)srcX >= pSrc->width)
+    {
+        WARN(("ups"));
+        return;
+    }
+
+    if ((GLuint)srcY >= pSrc->height)
+    {
+        WARN(("ups"));
+        return;
+    }
+
+    Assert(srcX >= 0);
+    Assert(srcY >= 0);
+    Assert(srcX < (int32_t)pSrc->width);
+    Assert(srcY < (int32_t)pSrc->height);
+
+    int32_t dstX = pCopyRect->xLeft;
+    int32_t dstY = pCopyRect->yTop;
+    Assert(dstX >= 0);
+    Assert(dstY >= 0);
+
+    int32_t UnscaledSrcWidth = UnscaledCopyRect.xRight - UnscaledCopyRect.xLeft;
+    int32_t UnscaledSrcHeight = UnscaledCopyRect.yBottom - UnscaledCopyRect.yTop;
+
+    if (UnscaledSrcWidth + srcX > (GLint)pSrc->width)
+        UnscaledSrcWidth = pSrc->width - srcX;
+
+    if (UnscaledSrcHeight + srcY > (GLint)pSrc->height)
+        UnscaledSrcHeight = pSrc->height - srcY;
+
+    uint8_t *pu8Src = ((uint8_t*)pSrc->pvData) + pSrc->pitch * (!fSrcInvert ? srcY : pSrc->height - srcY - 1) + srcX * 4;
+    uint8_t *pu8Dst = ((uint8_t*)pDst->pvData) + pDst->pitch * dstY + dstX * 4;
+
+    CrBmpScale32(pu8Dst, pDst->pitch,
+                        pCopyRect->xRight - pCopyRect->xLeft, pCopyRect->yBottom - pCopyRect->yTop,
+                        pu8Src,
+                        fSrcInvert ? -((int32_t)pSrc->pitch) : (int32_t)pSrc->pitch,
+                        UnscaledSrcWidth, UnscaledSrcHeight);
+}
+
+
+void CrMBltImgScaled(const CR_BLITTER_IMG *pSrc, const RTRECTSIZE *pSrcRectSize, const RTRECT *pDstRect, uint32_t cRects, const RTRECT *pRects, CR_BLITTER_IMG *pDst)
+{
+    int32_t srcWidth = pSrcRectSize->cx;
+    int32_t srcHeight = pSrcRectSize->cy;
+    int32_t dstWidth = pDstRect->xRight - pDstRect->xLeft;
+    int32_t dstHeight = pDstRect->yBottom - pDstRect->yTop;
+
+    float strX = ((float)dstWidth) / srcWidth;
+    float strY = ((float)dstHeight) / srcHeight;
+    Assert(dstWidth != srcWidth || dstHeight != srcHeight);
+
+    RTRECT Intersection;
+    RTRECT ScaledRestrictSrcRect;
+    ScaledRestrictSrcRect.xLeft = 0;
+    ScaledRestrictSrcRect.yTop = 0;
+    ScaledRestrictSrcRect.xRight = CR_FLOAT_RCAST(int32_t, pSrc->width * strX);
+    ScaledRestrictSrcRect.yBottom = CR_FLOAT_RCAST(int32_t, pSrc->height * strY);
+    RTRECT RestrictDstRect;
+    RestrictDstRect.xLeft = 0;
+    RestrictDstRect.yTop = 0;
+    RestrictDstRect.xRight = pDst->width;
+    RestrictDstRect.yBottom = pDst->height;
+
+    RTPOINT Pos = {pDstRect->xLeft, pDstRect->yTop};
+
+    for (uint32_t i = 0; i < cRects; ++i)
+    {
+        const RTRECT * pRect = &pRects[i];
+        VBoxRectIntersected(pRect, &RestrictDstRect, &Intersection);
+
+        RTRECT TranslatedSrc;
+        VBoxRectTranslated(&ScaledRestrictSrcRect, Pos.x, Pos.y, &TranslatedSrc);
+
+        VBoxRectIntersect(&Intersection, &TranslatedSrc);
+
+        if (VBoxRectIsZero(&Intersection))
+            continue;
+
+        CrMBltImgRectScaled(pSrc, &Pos, false, &Intersection, strX, strY, pDst);
+    }
+}
+
+#endif /* !IN_VMSVGA3D */
+
+
+/**
+ *
+ * @param   pBlitter        The blitter to initialize.
+ * @param   pCtxBase        Contains the blitter context info. Its value is
+ *                          treated differently depending on the fCreateNewCtx
+ *                          value.
+ * @param   fCreateNewCtx   If true, then @a pCtxBase must NOT be NULL. Its
+ *                          visualBits is used as a visual bits info for the new
+ *                          context, its id field is used to specified the
+ *                          shared context id to be used for blitter context.
+ *                          The id can be null to specify no shared context is
+ *                          needed
+ *
+ *                          If false and @a pCtxBase is NOT null AND its id
+ *                          field is NOT null, then specified the blitter
+ *                          context to be used blitter treats it as if it has
+ *                          default ogl state.
+ *
+ *                          Otherwise, the blitter works in a "no-context" mode,
+ *                          i.e. the caller is responsible for making a proper
+ *                          context current before calling the blitter. Note
+ *                          that BltEnter/Leave MUST still be called, but the
+ *                          proper context must be set before doing BltEnter,
+ *                          and ResoreContext info is ignored in that case. Also
+ *                          note that the blitter caches the current window
+ *                          info, and assumes the current context's values are
+ *                          preserved wrt that window before the calls, so if
+ *                          one uses different contexts for one blitter, the
+ *                          blitter current window values must be explicitly
+ *                          reset by doing CrBltMuralSetCurrentInfo(pBlitter,
+ *                          NULL).
+ * @param   fForceDrawBlt   If true this forces the blitter to always use
+ *                          glDrawXxx-based blits even if
+ *                          GL_EXT_framebuffer_blit.  This is needed because
+ *                          BlitFramebufferEXT is often known to be buggy, and
+ *                          glDrawXxx-based blits appear to be more reliable.
+ * @param   pShaders
+ * @param   pDispatch
  */
-VBOXBLITTERDECL(int) CrBltInit(PCR_BLITTER pBlitter, const CR_BLITTER_CONTEXT *pCtxBase, bool fCreateNewCtx, bool fForceDrawBlt, const CR_GLSL_CACHE *pShaders, SPUDispatchTable *pDispatch)
+VBOXBLITTERDECL(int) CrBltInit(PCR_BLITTER pBlitter, const CR_BLITTER_CONTEXT *pCtxBase,
+                               bool fCreateNewCtx, bool fForceDrawBlt, const CR_GLSL_CACHE *pShaders,
+                               SPUDispatchTable *pDispatch)
 {
     if (pCtxBase && pCtxBase->Base.id < 0)
     {
@@ -59,7 +333,7 @@ VBOXBLITTERDECL(int) CrBltInit(PCR_BLITTER pBlitter, const CR_BLITTER_CONTEXT *p
         return VERR_INVALID_PARAMETER;
     }
 
-    memset(pBlitter, 0, sizeof (*pBlitter));
+    RT_ZERO(*pBlitter);
 
     pBlitter->pDispatch = pDispatch;
     if (pCtxBase)
@@ -69,10 +343,15 @@ VBOXBLITTERDECL(int) CrBltInit(PCR_BLITTER pBlitter, const CR_BLITTER_CONTEXT *p
 
     if (fCreateNewCtx)
     {
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+        pBlitter->CtxInfo.Base.id = 0;
+#else
         pBlitter->CtxInfo.Base.id = pDispatch->CreateContext("", pCtxBase->Base.visualBits, pCtxBase->Base.id);
+#endif
         if (!pBlitter->CtxInfo.Base.id)
         {
-            memset(pBlitter, 0, sizeof (*pBlitter));
+            RT_ZERO(*pBlitter);
             crWarning("CreateContext failed!");
             return VERR_GENERAL_FAILURE;
         }
@@ -120,8 +399,12 @@ VBOXBLITTERDECL(int) CrBltCleanup(PCR_BLITTER pBlitter)
 
 void CrBltTerm(PCR_BLITTER pBlitter)
 {
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+#else
     if (pBlitter->Flags.CtxCreated)
         pBlitter->pDispatch->DestroyContext(pBlitter->CtxInfo.Base.id);
+#endif
     memset(pBlitter, 0, sizeof (*pBlitter));
 }
 
@@ -149,7 +432,8 @@ int CrBltMuralSetCurrentInfo(PCR_BLITTER pBlitter, const CR_BLITTER_WINDOW *pMur
 
     if (!CrBltIsEntered(pBlitter))
         return VINF_SUCCESS;
-    else if (!pBlitter->CtxInfo.Base.id)
+
+    if (!pBlitter->CtxInfo.Base.id)
     {
         WARN(("setting current mural for entered no-context blitter"));
         return VERR_INVALID_STATE;
@@ -157,13 +441,20 @@ int CrBltMuralSetCurrentInfo(PCR_BLITTER pBlitter, const CR_BLITTER_WINDOW *pMur
 
     WARN(("changing mural for entered blitter, is is somewhat expected?"));
 
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+#else
     pBlitter->pDispatch->Flush();
 
     pBlitter->pDispatch->MakeCurrent(pMural->Base.id, pBlitter->i32MakeCurrentUserData, pBlitter->CtxInfo.Base.id);
+#endif
 
     return VINF_SUCCESS;
 }
 
+
+#ifndef IN_VMSVGA3D
+
 static DECLCALLBACK(int) crBltBlitTexBufImplFbo(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, const RTRECT *paSrcRect, const RTRECTSIZE *pDstSize, const RTRECT *paDstRect, uint32_t cRects, uint32_t fFlags)
 {
     GLenum filter = CRBLT_FILTER_FROM_FLAGS(fFlags);
@@ -279,6 +570,8 @@ DECLINLINE(GLfloat*) crBltVtRectsTFNormalized(const RTRECT *paRects, uint32_t cR
 
 DECLINLINE(GLint*) crBltVtRectTF(const RTRECT *pRect, uint32_t normalX, uint32_t normalY, GLint* pBuff, uint32_t height)
 {
+    (void)normalX; (void)normalY;
+
     /* xLeft yTop */
     pBuff[0] = pRect->xLeft;
     pBuff[1] = height ? height - pRect->yTop : pRect->yTop;
@@ -366,7 +659,7 @@ static GLfloat* crBltVtRectsITNormalized(const RTRECT *paRects, uint32_t cRects,
     return pBuff;
 }
 
-static void* crBltBufGet(PCR_BLITTER_BUFFER pBuffer, GLuint cbBuffer)
+static void *crBltBufGet(PCR_BLITTER_BUFFER pBuffer, GLuint cbBuffer)
 {
     if (pBuffer->cbBuffer < cbBuffer)
     {
@@ -392,28 +685,42 @@ static void* crBltBufGet(PCR_BLITTER_BUFFER pBuffer, GLuint cbBuffer)
     return pBuffer->pvBuffer;
 }
 
+#endif /* !IN_VMSVGA3D */
+
+
 static void crBltCheckSetupViewport(PCR_BLITTER pBlitter, const RTRECTSIZE *pDstSize, bool fFBODraw)
 {
     bool fUpdateViewport = pBlitter->Flags.CurrentMuralChanged;
-    if (pBlitter->CurrentSetSize.cx != pDstSize->cx
-            || pBlitter->CurrentSetSize.cy != pDstSize->cy)
+    if (   pBlitter->CurrentSetSize.cx != pDstSize->cx
+        || pBlitter->CurrentSetSize.cy != pDstSize->cy)
     {
         pBlitter->CurrentSetSize = *pDstSize;
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pBlitter->pDispatch->MatrixMode(GL_PROJECTION);
         pBlitter->pDispatch->LoadIdentity();
         pBlitter->pDispatch->Ortho(0, pDstSize->cx, 0, pDstSize->cy, -1, 1);
+#endif
         fUpdateViewport = true;
     }
 
     if (fUpdateViewport)
     {
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pBlitter->pDispatch->Viewport(0, 0, pBlitter->CurrentSetSize.cx, pBlitter->CurrentSetSize.cy);
+#endif
         pBlitter->Flags.CurrentMuralChanged = 0;
     }
 
     pBlitter->Flags.LastWasFBODraw = fFBODraw;
 }
 
+
+#ifndef IN_VMSVGA3D
+
 static DECLCALLBACK(int) crBltBlitTexBufImplDraw2D(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, const RTRECT *paSrcRect, const RTRECTSIZE *pDstSize, const RTRECT *paDstRect, uint32_t cRects, uint32_t fFlags)
 {
     GLuint normalX, normalY;
@@ -459,10 +766,10 @@ static DECLCALLBACK(int) crBltBlitTexBufImplDraw2D(PCR_BLITTER pBlitter, const V
         crBltVtRectsTFNormalized(paSrcRect, cRects, normalX, normalY, pTexCoords, srcHeight);
 
         pBlitter->pDispatch->EnableClientState(GL_VERTEX_ARRAY);
-        pBlitter->pDispatch->VertexPointer(2, GL_FLOAT, 0, pVerticies);
+        pBlitter->pDispatch->VertexPointer(2, GL_FLOAT, 0, pVerticies CRVBOX_HOST_ONLY_PARAM(1 /*fRealPtr*/));
 
         pBlitter->pDispatch->EnableClientState(GL_TEXTURE_COORD_ARRAY);
-        pBlitter->pDispatch->TexCoordPointer(2, GL_FLOAT, 0, pTexCoords);
+        pBlitter->pDispatch->TexCoordPointer(2, GL_FLOAT, 0, pTexCoords CRVBOX_HOST_ONLY_PARAM(1 /*fRealPtr*/));
 
         pBlitter->pDispatch->Enable(pSrc->target);
 
@@ -487,10 +794,10 @@ static DECLCALLBACK(int) crBltBlitTexBufImplDraw2D(PCR_BLITTER pBlitter, const V
         crBltVtRectsITNormalized(paSrcRect, cRects, normalX, normalY, pTexCoords, NULL, NULL, srcHeight);
 
         pBlitter->pDispatch->EnableClientState(GL_VERTEX_ARRAY);
-        pBlitter->pDispatch->VertexPointer(2, GL_FLOAT, 0, pVerticies);
+        pBlitter->pDispatch->VertexPointer(2, GL_FLOAT, 0, pVerticies CRVBOX_HOST_ONLY_PARAM(1 /*fRealPtr*/));
 
         pBlitter->pDispatch->EnableClientState(GL_TEXTURE_COORD_ARRAY);
-        pBlitter->pDispatch->TexCoordPointer(2, GL_FLOAT, 0, pTexCoords);
+        pBlitter->pDispatch->TexCoordPointer(2, GL_FLOAT, 0, pTexCoords CRVBOX_HOST_ONLY_PARAM(1 /*fRealPtr*/));
 
         pBlitter->pDispatch->Enable(pSrc->target);
 
@@ -524,7 +831,7 @@ static int crBltInitOnMakeCurent(PCR_BLITTER pBlitter)
     else
         crWarning("GL_ARB_pixel_buffer_object not supported");
 
-    /* BlitFramebuffer seems to be buggy on Intel, 
+    /* BlitFramebuffer seems to be buggy on Intel,
      * try always glDrawXxx for now */
     if (!pBlitter->Flags.ForceDrawBlit && crStrstr(pszExtension, "GL_EXT_framebuffer_blit"))
     {
@@ -545,6 +852,9 @@ static int crBltInitOnMakeCurent(PCR_BLITTER pBlitter)
     return VINF_SUCCESS;
 }
 
+#endif /* !IN_VMSVGA3D */
+
+
 void CrBltLeave(PCR_BLITTER pBlitter)
 {
     if (!pBlitter->cEnters)
@@ -556,6 +866,9 @@ void CrBltLeave(PCR_BLITTER pBlitter)
     if (--pBlitter->cEnters)
         return;
 
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+#else
     if (pBlitter->Flags.SupportsFBO)
     {
         pBlitter->pDispatch->BindFramebufferEXT(GL_FRAMEBUFFER, 0);
@@ -567,6 +880,7 @@ void CrBltLeave(PCR_BLITTER pBlitter)
 
     if (pBlitter->CtxInfo.Base.id)
         pBlitter->pDispatch->MakeCurrent(0, 0, 0);
+#endif
 }
 
 int CrBltEnter(PCR_BLITTER pBlitter)
@@ -582,13 +896,22 @@ int CrBltEnter(PCR_BLITTER pBlitter)
 
     if (pBlitter->CurrentMural.Base.id) /* <- pBlitter->CurrentMural.Base.id can be null if the blitter is in a "no-context" mode (see comments to BltInit for detail)*/
     {
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+#else
         pBlitter->pDispatch->MakeCurrent(pBlitter->CurrentMural.Base.id, pBlitter->i32MakeCurrentUserData, pBlitter->CtxInfo.Base.id);
+#endif
     }
 
     if (pBlitter->Flags.Initialized)
         return VINF_SUCCESS;
 
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+    int rc = VINF_SUCCESS;
+#else
     int rc = crBltInitOnMakeCurent(pBlitter);
+#endif
     if (RT_SUCCESS(rc))
     {
         pBlitter->Flags.Initialized = 1;
@@ -600,8 +923,12 @@ int CrBltEnter(PCR_BLITTER pBlitter)
     return rc;
 }
 
+
 static void crBltBlitTexBuf(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, const RTRECT *paSrcRects, GLenum enmDstBuff, const RTRECTSIZE *pDstSize, const RTRECT *paDstRects, uint32_t cRects, uint32_t fFlags)
 {
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+#else
     pBlitter->pDispatch->DrawBuffer(enmDstBuff);
 
     crBltCheckSetupViewport(pBlitter, pDstSize, enmDstBuff == GL_DRAW_FRAMEBUFFER);
@@ -610,10 +937,9 @@ static void crBltBlitTexBuf(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, co
         pBlitter->pfnBlt(pBlitter, pSrc, paSrcRects, pDstSize, paDstRects, cRects, fFlags);
     else
     {
-        int rc = pBlitter->Flags.ShadersGloal ?
-                CrGlslProgUseNoAlpha(pBlitter->pGlslCache, pSrc->target)
-                :
-                CrGlslProgUseGenNoAlpha(&pBlitter->LocalGlslCache, pSrc->target);
+        int rc = pBlitter->Flags.ShadersGloal
+               ? CrGlslProgUseNoAlpha(pBlitter->pGlslCache, pSrc->target)
+               : CrGlslProgUseGenNoAlpha(&pBlitter->LocalGlslCache, pSrc->target);
 
         if (!RT_SUCCESS(rc))
         {
@@ -630,6 +956,7 @@ static void crBltBlitTexBuf(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, co
 
         CrGlslProgClear(pBlitter->pGlslCache);
     }
+#endif
 }
 
 void CrBltCheckUpdateViewport(PCR_BLITTER pBlitter)
@@ -648,11 +975,18 @@ void CrBltBlitTexMural(PCR_BLITTER pBlitter, bool fBb, const VBOXVR_TEXTURE *pSr
 
     RTRECTSIZE DstSize = {pBlitter->CurrentMural.width, pBlitter->CurrentMural.height};
 
+#ifdef IN_VMSVGA3D
+    /** @todo IN_VMSVGA3D */
+#else
     pBlitter->pDispatch->BindFramebufferEXT(GL_DRAW_FRAMEBUFFER, 0);
+#endif
 
     crBltBlitTexBuf(pBlitter, pSrc, paSrcRects, fBb ? GL_BACK : GL_FRONT, &DstSize, paDstRects, cRects, fFlags);
 }
 
+
+#ifndef IN_VMSVGA3D
+
 void CrBltBlitTexTex(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, const RTRECT *pSrcRect, const VBOXVR_TEXTURE *pDst, const RTRECT *pDstRect, uint32_t cRects, uint32_t fFlags)
 {
     if (!CrBltIsEntered(pBlitter))
@@ -665,7 +999,7 @@ void CrBltBlitTexTex(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *pSrc, const RTR
 
     pBlitter->pDispatch->BindFramebufferEXT(GL_DRAW_FRAMEBUFFER, pBlitter->idFBO);
 
-    /* TODO: mag/min filters ? */
+    /** @todo mag/min filters ? */
 
     pBlitter->pDispatch->FramebufferTexture2DEXT(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, pDst->target, pDst->hwid, 0);
 
@@ -780,13 +1114,14 @@ VBOXBLITTERDECL(int) CrBltImgGetTex(PCR_BLITTER pBlitter, const VBOXVR_TEXTURE *
 
 VBOXBLITTERDECL(int) CrBltImgGetMural(PCR_BLITTER pBlitter, bool fBb, CR_BLITTER_IMG *pDst)
 {
+    (void)fBb; (void)pDst;
     if (!CrBltIsEntered(pBlitter))
     {
         WARN(("CrBltImgGetMural: blitter not entered"));
         return VERR_INVALID_STATE;
     }
 
-    crWarning("NOT IMPLEMENTED");
+    WARN(("NOT IMPLEMENTED"));
     return VERR_NOT_IMPLEMENTED;
 }
 
@@ -824,7 +1159,7 @@ VBOXBLITTERDECL(bool) CrGlslIsSupported(CR_GLSL_CACHE *pCache)
 
     crWarning("GLSL unsuported, gl version %d", pCache->iGlVersion);
 
-    /* @todo: we could also check for GL_ARB_shader_objects and GL_ARB_fragment_shader,
+    /** @todo we could also check for GL_ARB_shader_objects and GL_ARB_fragment_shader,
      * but seems like chromium does not support properly gl*Object versions of shader functions used with those extensions */
     return false;
 }
@@ -1090,6 +1425,8 @@ VBOXBLITTERDECL(int) CrGlslProgUseGenNoAlpha(CR_GLSL_CACHE *pCache, GLenum enmTe
     return VINF_SUCCESS;
 }
 
+#endif /* !IN_VMSVGA3D */
+
 VBOXBLITTERDECL(bool) CrGlslNeedsCleanup(const CR_GLSL_CACHE *pCache)
 {
     return pCache->uNoAlpha2DProg || pCache->uNoAlpha2DRectProg;
@@ -1099,13 +1436,21 @@ VBOXBLITTERDECL(void) CrGlslCleanup(CR_GLSL_CACHE *pCache)
 {
     if (pCache->uNoAlpha2DProg)
     {
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pCache->pDispatch->DeleteProgram(pCache->uNoAlpha2DProg);
+#endif
         pCache->uNoAlpha2DProg = 0;
     }
 
     if (pCache->uNoAlpha2DRectProg)
     {
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pCache->pDispatch->DeleteProgram(pCache->uNoAlpha2DRectProg);
+#endif
         pCache->uNoAlpha2DRectProg = 0;
     }
 }
@@ -1120,6 +1465,7 @@ VBOXBLITTERDECL(void) CrGlslTerm(CR_GLSL_CACHE *pCache)
     memset(pCache, 0, sizeof (*pCache));
 }
 
+#ifndef IN_VMSVGA3D
 
 /*TdBlt*/
 static void crTdBltCheckPBO(PCR_TEXDATA pTex)
@@ -1186,6 +1532,9 @@ int crTdBltCheckInvertTex(PCR_TEXDATA pTex)
     return VINF_SUCCESS;
 }
 
+#endif /* !IN_VMSVGA3D */
+
+
 void crTdBltImgRelease(PCR_TEXDATA pTex)
 {
     pTex->Flags.DataValid = 0;
@@ -1209,9 +1558,13 @@ void crTdBltImgFree(PCR_TEXDATA pTex)
         PCR_BLITTER pBlitter = pTex->pBlitter;
 
         Assert(CrBltIsEntered(pBlitter));
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pBlitter->pDispatch->BindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, pTex->idPBO);
         pBlitter->pDispatch->UnmapBufferARB(GL_PIXEL_PACK_BUFFER_ARB);
         pBlitter->pDispatch->BindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, 0);
+#endif
     }
     else
     {
@@ -1222,6 +1575,9 @@ void crTdBltImgFree(PCR_TEXDATA pTex)
     pTex->Img.pvData = NULL;
 }
 
+
+#ifndef IN_VMSVGA3D
+
 int crTdBltImgAcquire(PCR_TEXDATA pTex, GLenum enmFormat, bool fInverted)
 {
     void *pvData = pTex->Img.pvData;
@@ -1289,6 +1645,9 @@ int crTdBltImgAcquire(PCR_TEXDATA pTex, GLenum enmFormat, bool fInverted)
     return VINF_SUCCESS;
 }
 
+#endif /* !IN_VMSVGA3D */
+
+
 /* release the texture data, the data remains cached in the CR_TEXDATA object until it is discarded with CrTdBltDataInvalidateNe or CrTdBltDataCleanup */
 VBOXBLITTERDECL(int) CrTdBltDataRelease(PCR_TEXDATA pTex)
 {
@@ -1388,14 +1747,22 @@ static void crTdBltDataCleanup(PCR_TEXDATA pTex)
     if (pTex->idPBO)
     {
         Assert(CrBltIsEntered(pBlitter));
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pBlitter->pDispatch->DeleteBuffersARB(1, &pTex->idPBO);
+#endif
         pTex->idPBO = 0;
     }
 
     if (pTex->idInvertTex)
     {
         Assert(CrBltIsEntered(pBlitter));
+#ifdef IN_VMSVGA3D
+        /** @todo IN_VMSVGA3D */
+#else
         pBlitter->pDispatch->DeleteTextures(1, &pTex->idInvertTex);
+#endif
         pTex->idInvertTex = 0;
     }
 
@@ -1439,6 +1806,9 @@ VBOXBLITTERDECL(int) CrTdBltDataCleanupNe(PCR_TEXDATA pTex)
     return VINF_SUCCESS;
 }
 
+
+#ifndef IN_VMSVGA3D
+
 /* acquire the texture data, returns the cached data in case it is cached.
  * the data remains cached in the CR_TEXDATA object until it is discarded with CrTdBltDataFree or CrTdBltDataCleanup.
  * */
@@ -1660,8 +2030,8 @@ static int ctTdBltSdGetUpdated(PCR_TEXDATA pTex, uint32_t width, uint32_t height
         return rc;
     }
 
-    Assert(width == pScaledCache->Tex.width);
-    Assert(height == pScaledCache->Tex.height);
+    Assert(width == (uint32_t)pScaledCache->Tex.width);
+    Assert(height == (uint32_t)pScaledCache->Tex.height);
 
     if (!pScaledCache->Flags.DataValid)
     {
@@ -1687,7 +2057,7 @@ static int ctTdBltSdGetUpdated(PCR_TEXDATA pTex, uint32_t width, uint32_t height
 
 VBOXBLITTERDECL(int) CrTdBltDataAcquireScaled(PCR_TEXDATA pTex, GLenum enmFormat, bool fInverted, uint32_t width, uint32_t height, const CR_BLITTER_IMG**ppImg)
 {
-    if (pTex->Tex.width == width && pTex->Tex.height == height)
+    if ((uint32_t)pTex->Tex.width == width && (uint32_t)pTex->Tex.height == height)
         return CrTdBltDataAcquire(pTex, enmFormat, fInverted, ppImg);
 
     if (!pTex->Flags.Entered)
@@ -1749,3 +2119,6 @@ VBOXBLITTERDECL(void) CrTdBltScaleCacheMoveTo(PCR_TEXDATA pTex, PCR_TEXDATA pDst
     pDstTex->pScaledCache = pTex->pScaledCache;
     pTex->pScaledCache = NULL;
 }
+
+#endif /* !IN_VMSVGA3D */
+
