@@ -2,8 +2,9 @@
 /** @file
  * VBox USB R0 runtime
  */
+
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -12,8 +13,21 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
  */
 
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "VBoxUsbCmn.h"
 #include "../cmn/VBoxUsbIdc.h"
 #include "../cmn/VBoxUsbTool.h"
@@ -21,12 +35,21 @@
 #include <VBox/usblib-win.h>
 #include <iprt/assert.h>
 #include <VBox/log.h>
-#define _USBD_
+
+
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
+#define _USBD_ /** @todo r=bird: What is this?? */
 
 #define USBD_DEFAULT_PIPE_TRANSFER 0x00000008
 
 #define VBOXUSB_MAGIC  0xABCF1423
 
+
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 typedef struct VBOXUSB_URB_CONTEXT
 {
     PURB pUrb;
@@ -46,32 +69,26 @@ typedef struct VBOXUSB_SETUP
     uint16_t wLength;
 } VBOXUSB_SETUP, *PVBOXUSB_SETUP;
 
+
+
 static bool vboxUsbRtCtxSetOwner(PVBOXUSBDEV_EXT pDevExt, PFILE_OBJECT pFObj)
 {
-    bool bRc = ASMAtomicCmpXchgPtr(&pDevExt->Rt.pOwner, pFObj, NULL);
-    if (bRc)
-    {
-        Log((__FUNCTION__": pDevExt (0x%x) Owner(0x%x) acquired\n", pFObj));
-    }
+    bool fRc = ASMAtomicCmpXchgPtr(&pDevExt->Rt.pOwner, pFObj, NULL);
+    if (fRc)
+        LogFunc(("pDevExt (0x%x) Owner(0x%x) acquired\n", pFObj));
     else
-    {
-        Log((__FUNCTION__": pDevExt (0x%x) Owner(0x%x) FAILED!!\n", pFObj));
-    }
-    return bRc;
+        LogFunc(("pDevExt (0x%x) Owner(0x%x) FAILED!!\n", pFObj));
+    return fRc;
 }
 
 static bool vboxUsbRtCtxReleaseOwner(PVBOXUSBDEV_EXT pDevExt, PFILE_OBJECT pFObj)
 {
-    bool bRc = ASMAtomicCmpXchgPtr(&pDevExt->Rt.pOwner, NULL, pFObj);
-    if (bRc)
-    {
-        Log((__FUNCTION__": pDevExt (0x%x) Owner(0x%x) released\n", pFObj));
-    }
+    bool fRc = ASMAtomicCmpXchgPtr(&pDevExt->Rt.pOwner, NULL, pFObj);
+    if (fRc)
+        LogFunc(("pDevExt (0x%x) Owner(0x%x) released\n", pFObj));
     else
-    {
-        Log((__FUNCTION__": pDevExt (0x%x) Owner(0x%x) release: is NOT an owner\n", pFObj));
-    }
-    return bRc;
+        LogFunc(("pDevExt (0x%x) Owner(0x%x) release: is NOT an owner\n", pFObj));
+    return fRc;
 }
 
 static bool vboxUsbRtCtxIsOwner(PVBOXUSBDEV_EXT pDevExt, PFILE_OBJECT pFObj)
@@ -99,8 +116,11 @@ static NTSTATUS vboxUsbRtIdcInit()
         vboxUsbRtIdcSubmit(VBOXUSBIDC_INTERNAL_IOCTL_GET_VERSION, &Version);
         if (NT_SUCCESS(Status))
         {
-            if (Version.u32Major == VBOXUSBIDC_VERSION_MAJOR
-                    && Version.u32Minor >= VBOXUSBIDC_VERSION_MINOR)
+            if (   Version.u32Major == VBOXUSBIDC_VERSION_MAJOR
+#if VBOXUSBIDC_VERSION_MINOR != 0
+                && Version.u32Minor >= VBOXUSBIDC_VERSION_MINOR
+#endif
+               )
                 return STATUS_SUCCESS;
             AssertFailed();
         }
@@ -238,7 +258,7 @@ static void vboxUsbRtFreeInterfaces(PVBOXUSBDEV_EXT pDevExt, BOOLEAN fAbortPipes
             {
                 if (fAbortPipes)
                 {
-                    for(j=0; j<pDevExt->Rt.pVBIfaceInfo[i].pInterfaceInfo->NumberOfPipes; j++)
+                    for (j=0; j<pDevExt->Rt.pVBIfaceInfo[i].pInterfaceInfo->NumberOfPipes; j++)
                     {
                         Log(("Aborting Pipe %d handle %x address %x\n", j,
                                  pDevExt->Rt.pVBIfaceInfo[i].pInterfaceInfo->Pipes[j].PipeHandle,
@@ -407,8 +427,8 @@ static NTSTATUS vboxUsbRtDispatchReleaseDevice(PVBOXUSBDEV_EXT pDevExt, PIRP pIr
     if (vboxUsbRtCtxIsOwner(pDevExt, pFObj))
     {
         vboxUsbRtFreeCachedDescriptors(pDevExt);
-        bool bRc = vboxUsbRtCtxReleaseOwner(pDevExt, pFObj);
-        Assert(bRc);
+        bool fRc = vboxUsbRtCtxReleaseOwner(pDevExt, pFObj);
+        Assert(fRc); NOREF(fRc);
     }
     else
     {
@@ -437,7 +457,7 @@ static NTSTATUS vboxUsbRtGetDeviceDescription(PVBOXUSBDEV_EXT pDevExt)
 
             if (pDr->iSerialNumber
 #ifdef DEBUG
-                    || pDr->iProduct || pDr->iManufacturer
+                || pDr->iProduct || pDr->iManufacturer
 #endif
                )
             {
@@ -445,7 +465,8 @@ static NTSTATUS vboxUsbRtGetDeviceDescription(PVBOXUSBDEV_EXT pDevExt)
                 Status = VBoxUsbToolGetLangID(pDevExt->pLowerDO, &langId, RT_INDEFINITE_WAIT);
                 if (NT_SUCCESS(Status))
                 {
-                    Status = VBoxUsbToolGetStringDescriptorA(pDevExt->pLowerDO, pDevExt->Rt.szSerial, sizeof (pDevExt->Rt.szSerial), pDr->iSerialNumber, langId, RT_INDEFINITE_WAIT);
+                    Status = VBoxUsbToolGetStringDescriptor(pDevExt->pLowerDO, pDevExt->Rt.szSerial, sizeof (pDevExt->Rt.szSerial),
+                                                            pDr->iSerialNumber, langId, RT_INDEFINITE_WAIT);
                 }
                 else
                 {
@@ -462,14 +483,14 @@ static NTSTATUS vboxUsbRtGetDeviceDescription(PVBOXUSBDEV_EXT pDevExt)
 static NTSTATUS vboxUsbRtDispatchGetDevice(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
 {
     PIO_STACK_LOCATION pSl = IoGetCurrentIrpStackLocation(pIrp);
-    PUSBSUP_GETDEV pDev  = (PUSBSUP_GETDEV)pIrp->AssociatedIrp.SystemBuffer;
-    NTSTATUS Status = STATUS_SUCCESS;
+    PUSBSUP_GETDEV pDev = (PUSBSUP_GETDEV)pIrp->AssociatedIrp.SystemBuffer;
     ULONG cbOut = 0;
 
     /* don't check for owner since this request is allowed for non-owners as well */
-
-    if (pDev && pSl->Parameters.DeviceIoControl.InputBufferLength == sizeof (*pDev)
-             && pSl->Parameters.DeviceIoControl.OutputBufferLength == sizeof (*pDev))
+    NTSTATUS Status;
+    if (   pDev
+        && pSl->Parameters.DeviceIoControl.InputBufferLength  == sizeof(*pDev)
+        && pSl->Parameters.DeviceIoControl.OutputBufferLength == sizeof(*pDev))
     {
         Status = VBoxUsbToolGetDeviceSpeed(pDevExt->pLowerDO, &pDevExt->Rt.fIsHighSpeed);
         if (NT_SUCCESS(Status))
@@ -481,9 +502,7 @@ static NTSTATUS vboxUsbRtDispatchGetDevice(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
         }
     }
     else
-    {
         Status = STATUS_INVALID_PARAMETER;
-    }
 
     Assert(Status != STATUS_PENDING);
     VBoxDrvToolIoComplete(pIrp, Status, cbOut);
@@ -493,44 +512,42 @@ static NTSTATUS vboxUsbRtDispatchGetDevice(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
 
 static NTSTATUS vboxUsbRtDispatchUsbReset(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
 {
-    PIO_STACK_LOCATION pSl = IoGetCurrentIrpStackLocation(pIrp);
-    PFILE_OBJECT pFObj = pSl->FileObject;
-    PUSBSUP_GETDEV pDev  = (PUSBSUP_GETDEV)pIrp->AssociatedIrp.SystemBuffer;
-    NTSTATUS Status = STATUS_SUCCESS;
-
-    do
+    PIO_STACK_LOCATION  pSl   = IoGetCurrentIrpStackLocation(pIrp);
+    PFILE_OBJECT        pFObj = pSl->FileObject;
+    NTSTATUS            rcNt;
+    if (pFObj)
     {
-        if (!pFObj)
+        if (vboxUsbRtCtxIsOwner(pDevExt, pFObj))
+        {
+            if (   pIrp->AssociatedIrp.SystemBuffer == NULL
+                && pSl->Parameters.DeviceIoControl.InputBufferLength == 0
+                && pSl->Parameters.DeviceIoControl.OutputBufferLength == 0)
+            {
+                rcNt = VBoxUsbToolIoInternalCtlSendSync(pDevExt->pLowerDO, IOCTL_INTERNAL_USB_RESET_PORT, NULL, NULL);
+                Assert(NT_SUCCESS(rcNt));
+            }
+            else
+            {
+                AssertFailed();
+                rcNt = STATUS_INVALID_PARAMETER;
+            }
+        }
+        else
         {
             AssertFailed();
-            Status = STATUS_INVALID_PARAMETER;
-            break;
+            rcNt = STATUS_ACCESS_DENIED;
         }
+    }
+    else
+    {
+        AssertFailed();
+        rcNt = STATUS_INVALID_PARAMETER;
+    }
 
-        if (!vboxUsbRtCtxIsOwner(pDevExt, pFObj))
-        {
-            AssertFailed();
-            Status = STATUS_ACCESS_DENIED;
-            break;
-        }
-
-        if (pIrp->AssociatedIrp.SystemBuffer
-                || pSl->Parameters.DeviceIoControl.InputBufferLength
-                || pSl->Parameters.DeviceIoControl.OutputBufferLength)
-        {
-            AssertFailed();
-            Status = STATUS_INVALID_PARAMETER;
-            break;
-        }
-
-        Status = VBoxUsbToolIoInternalCtlSendSync(pDevExt->pLowerDO, IOCTL_INTERNAL_USB_RESET_PORT, NULL, NULL);
-        Assert(NT_SUCCESS(Status));
-    } while (0);
-
-    Assert(Status != STATUS_PENDING);
-    VBoxDrvToolIoComplete(pIrp, Status, 0);
+    Assert(rcNt != STATUS_PENDING);
+    VBoxDrvToolIoComplete(pIrp, rcNt, 0);
     vboxUsbDdiStateRelease(pDevExt);
-    return Status;
+    return rcNt;
 }
 
 static PUSB_CONFIGURATION_DESCRIPTOR vboxUsbRtFindConfigDesc(PVBOXUSBDEV_EXT pDevExt, uint8_t uConfiguration)
@@ -561,7 +578,7 @@ static NTSTATUS vboxUsbRtSetConfig(PVBOXUSBDEV_EXT pDevExt, uint8_t uConfigurati
     if (!uConfiguration)
     {
         pUrb = VBoxUsbToolUrbAllocZ(URB_FUNCTION_SELECT_CONFIGURATION, sizeof (struct _URB_SELECT_CONFIGURATION));
-        if(!pUrb)
+        if (!pUrb)
         {
             AssertMsgFailed((__FUNCTION__": VBoxUsbToolUrbAlloc failed\n"));
             return STATUS_INSUFFICIENT_RESOURCES;
@@ -572,7 +589,7 @@ static NTSTATUS vboxUsbRtSetConfig(PVBOXUSBDEV_EXT pDevExt, uint8_t uConfigurati
         pUrb->UrbSelectConfiguration.ConfigurationDescriptor = NULL;
 
         Status = VBoxUsbToolUrbPost(pDevExt->pLowerDO, pUrb, RT_INDEFINITE_WAIT);
-        if(NT_SUCCESS(Status) && USBD_SUCCESS(pUrb->UrbHeader.Status))
+        if (NT_SUCCESS(Status) && USBD_SUCCESS(pUrb->UrbHeader.Status))
         {
             pDevExt->Rt.hConfiguration = pUrb->UrbSelectConfiguration.ConfigurationHandle;
             pDevExt->Rt.uConfigValue = uConfiguration;
@@ -587,6 +604,10 @@ static NTSTATUS vboxUsbRtSetConfig(PVBOXUSBDEV_EXT pDevExt, uint8_t uConfigurati
         return Status;
     }
 
+/** @todo r=bird: Need to write a script for fixing these kind of clueless use
+ *        of AssertMsgFailed (into AssertMsgReturn).  The __FUNCTION__ is just
+ *        the topping it off - the assertion message includes function, file and
+ *        line number. Duh! */
     PUSB_CONFIGURATION_DESCRIPTOR pCfgDr = vboxUsbRtFindConfigDesc(pDevExt, uConfiguration);
     if (!pCfgDr)
     {
@@ -604,7 +625,6 @@ static NTSTATUS vboxUsbRtSetConfig(PVBOXUSBDEV_EXT pDevExt, uint8_t uConfigurati
     for (i = 0; i < pCfgDr->bNumInterfaces; i++)
     {
         pIfLe[i].InterfaceDescriptor = USBD_ParseConfigurationDescriptorEx(pCfgDr, pCfgDr, i, 0, -1, -1, -1);
-        pIfLe[i].Interface = NULL;
         if (!pIfLe[i].InterfaceDescriptor)
         {
             AssertMsgFailed((__FUNCTION__": interface %d not found\n", i));
@@ -634,7 +654,7 @@ static NTSTATUS vboxUsbRtSetConfig(PVBOXUSBDEV_EXT pDevExt, uint8_t uConfigurati
                     Assert(NT_SUCCESS(Status));
                     for (i = 0; i < pDevExt->Rt.uNumInterfaces; i++)
                     {
-                        size_t uTotalIfaceInfoLength = RT_OFFSETOF(struct _USBD_INTERFACE_INFORMATION, Pipes[RT_MAX(pIfLe[i].Interface->NumberOfPipes, 1)]);
+                        size_t uTotalIfaceInfoLength = GET_USBD_INTERFACE_SIZE(pIfLe[i].Interface->NumberOfPipes);
                         pDevExt->Rt.pVBIfaceInfo[i].pInterfaceInfo = (PUSBD_INTERFACE_INFORMATION)vboxUsbMemAlloc(uTotalIfaceInfoLength);
                         if (!pDevExt->Rt.pVBIfaceInfo[i].pInterfaceInfo)
                         {
@@ -739,110 +759,101 @@ static NTSTATUS vboxUsbRtDispatchUsbSetConfig(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp
 
 static NTSTATUS vboxUsbRtSetInterface(PVBOXUSBDEV_EXT pDevExt, uint32_t InterfaceNumber, int AlternateSetting)
 {
-    if (!pDevExt->Rt.uConfigValue)
-    {
-        AssertMsgFailed((__FUNCTION__": Can't select an interface without an active configuration\n"));
-        return STATUS_INVALID_PARAMETER;
-    }
-
-    if (InterfaceNumber >= pDevExt->Rt.uNumInterfaces)
-    {
-        AssertMsgFailed((__FUNCTION__": InterfaceNumber %d too high!!\n", InterfaceNumber));
-        return STATUS_INVALID_PARAMETER;
-    }
-
+    AssertMsgReturn(pDevExt->Rt.uConfigValue, ("Can't select an interface without an active configuration\n"),
+                    STATUS_INVALID_PARAMETER);
+    AssertMsgReturn(InterfaceNumber < pDevExt->Rt.uNumInterfaces, ("InterfaceNumber %d too high!!\n", InterfaceNumber),
+                    STATUS_INVALID_PARAMETER);
     PUSB_CONFIGURATION_DESCRIPTOR pCfgDr = vboxUsbRtFindConfigDesc(pDevExt, pDevExt->Rt.uConfigValue);
-    if (!pCfgDr)
-    {
-        AssertMsgFailed((__FUNCTION__": configuration %d not found!!\n", pDevExt->Rt.uConfigValue));
-        return STATUS_INVALID_PARAMETER;
-    }
-
+    AssertMsgReturn(pCfgDr, ("configuration %d not found!!\n", pDevExt->Rt.uConfigValue),
+                    STATUS_INVALID_PARAMETER);
     PUSB_INTERFACE_DESCRIPTOR pIfDr = USBD_ParseConfigurationDescriptorEx(pCfgDr, pCfgDr, InterfaceNumber, AlternateSetting, -1, -1, -1);
-    if (!pIfDr)
-    {
-        AssertMsgFailed((__FUNCTION__": invalid interface %d or alternate setting %d\n", InterfaceNumber, AlternateSetting));
-        return STATUS_UNSUCCESSFUL;
-    }
+    AssertMsgReturn(pIfDr, ("invalid interface %d or alternate setting %d\n", InterfaceNumber, AlternateSetting),
+                    STATUS_UNSUCCESSFUL);
 
     USHORT uUrbSize = GET_SELECT_INTERFACE_REQUEST_SIZE(pIfDr->bNumEndpoints);
     ULONG uTotalIfaceInfoLength = GET_USBD_INTERFACE_SIZE(pIfDr->bNumEndpoints);
     NTSTATUS Status = STATUS_SUCCESS;
-    PURB pUrb = VBoxUsbToolUrbAllocZ(0, uUrbSize);
-    if (!pUrb)
-    {
-        AssertMsgFailed((__FUNCTION__": VBoxUsbToolUrbAlloc failed\n"));
-        return STATUS_NO_MEMORY;
-    }
+    PURB pUrb;
+    PUSBD_INTERFACE_INFORMATION pNewIFInfo = NULL;
+    VBOXUSB_PIPE_INFO *pNewPipeInfo = NULL;
 
-    /*
-     * Free old interface and pipe info, allocate new again
-     */
     if (pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo)
     {
         /* Clear pipes associated with the interface, else Windows may hang. */
-        for(ULONG i = 0; i < pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo->NumberOfPipes; i++)
-        {
+        for (ULONG i = 0; i < pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo->NumberOfPipes; i++)
             VBoxUsbToolPipeClear(pDevExt->pLowerDO, pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo->Pipes[i].PipeHandle, FALSE);
+    }
+
+    do {
+        /* First allocate all the structures we'll need. */
+        pUrb = VBoxUsbToolUrbAllocZ(0, uUrbSize);
+        if (!pUrb)
+        {
+            AssertMsgFailed((__FUNCTION__": VBoxUsbToolUrbAllocZ failed\n"));
+            Status = STATUS_NO_MEMORY;
+            break;
         }
-        vboxUsbMemFree(pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo);
-    }
 
-    if (pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo)
-    {
-        vboxUsbMemFree(pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo);
-    }
+        pNewIFInfo = (PUSBD_INTERFACE_INFORMATION)vboxUsbMemAlloc(uTotalIfaceInfoLength);
+        if (!pNewIFInfo)
+        {
+            AssertMsgFailed((__FUNCTION__": Failed allocating interface storage\n"));
+            Status = STATUS_NO_MEMORY;
+            break;
+        }
 
-    pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo = (PUSBD_INTERFACE_INFORMATION)vboxUsbMemAlloc(uTotalIfaceInfoLength);
-    if (pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo)
-    {
         if (pIfDr->bNumEndpoints > 0)
         {
-            pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo = (VBOXUSB_PIPE_INFO*)vboxUsbMemAlloc(pIfDr->bNumEndpoints * sizeof(VBOXUSB_PIPE_INFO));
-            if (!pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo)
+            pNewPipeInfo = (VBOXUSB_PIPE_INFO *)vboxUsbMemAlloc(pIfDr->bNumEndpoints * sizeof(VBOXUSB_PIPE_INFO));
+            if (!pNewPipeInfo)
             {
-                AssertMsgFailed(("VBoxUSBSetInterface: ExAllocatePool failed!\n"));
+                AssertMsgFailed((__FUNCTION__": Failed allocating pipe info storage\n"));
                 Status = STATUS_NO_MEMORY;
+                break;
+            }
+        }
+        else
+            pNewPipeInfo = NULL;
+
+        /* Now that we have all the bits, select the interface. */
+        UsbBuildSelectInterfaceRequest(pUrb, uUrbSize, pDevExt->Rt.hConfiguration, InterfaceNumber, AlternateSetting);
+        pUrb->UrbSelectInterface.Interface.Length = GET_USBD_INTERFACE_SIZE(pIfDr->bNumEndpoints);
+
+        Status = VBoxUsbToolUrbPost(pDevExt->pLowerDO, pUrb, RT_INDEFINITE_WAIT);
+        if (NT_SUCCESS(Status) && USBD_SUCCESS(pUrb->UrbHeader.Status))
+        {
+            /* Free the old memory and put new in. */
+            if (pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo)
+                vboxUsbMemFree(pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo);
+            pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo = pNewIFInfo;
+            if (pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo)
+                vboxUsbMemFree(pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo);
+            pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo = pNewPipeInfo;
+            pNewPipeInfo = NULL; pNewIFInfo = NULL; /* Don't try to free it again. */
+
+            USBD_INTERFACE_INFORMATION *pIfInfo = &pUrb->UrbSelectInterface.Interface;
+            memcpy(pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo, pIfInfo, GET_USBD_INTERFACE_SIZE(pIfDr->bNumEndpoints));
+
+            Assert(pIfInfo->NumberOfPipes == pIfDr->bNumEndpoints);
+            for (ULONG i = 0; i < pIfInfo->NumberOfPipes; i++)
+            {
+                pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo[i].EndpointAddress = pIfInfo->Pipes[i].EndpointAddress;
+                pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo[i].NextScheduledFrame = 0;
             }
         }
         else
         {
-            pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo = NULL;
+            AssertMsgFailed((__FUNCTION__": VBoxUsbToolUrbPost failed Status (0x%x) usb Status (0x%x)\n", Status, pUrb->UrbHeader.Status));
         }
+    } while (0);
 
-        if (NT_SUCCESS(Status))
-        {
-            UsbBuildSelectInterfaceRequest(pUrb, uUrbSize, pDevExt->Rt.hConfiguration, InterfaceNumber, AlternateSetting);
-            pUrb->UrbSelectInterface.Interface.Length = GET_USBD_INTERFACE_SIZE(pIfDr->bNumEndpoints);
-
-            Status = VBoxUsbToolUrbPost(pDevExt->pLowerDO, pUrb, RT_INDEFINITE_WAIT);
-            if (NT_SUCCESS(Status) && USBD_SUCCESS(pUrb->UrbHeader.Status))
-            {
-                USBD_INTERFACE_INFORMATION *pIfInfo = &pUrb->UrbSelectInterface.Interface;
-                memcpy(pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo, pIfInfo, GET_USBD_INTERFACE_SIZE(pIfDr->bNumEndpoints));
-
-                Assert(pIfInfo->NumberOfPipes == pIfDr->bNumEndpoints);
-                for(ULONG i = 0; i < pIfInfo->NumberOfPipes; i++)
-                {
-                    pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pInterfaceInfo->Pipes[i] = pIfInfo->Pipes[i];
-                    pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo[i].EndpointAddress = pIfInfo->Pipes[i].EndpointAddress;
-                    pDevExt->Rt.pVBIfaceInfo[InterfaceNumber].pPipeInfo[i].NextScheduledFrame = 0;
-                }
-            }
-            else
-            {
-                AssertMsgFailed((__FUNCTION__": VBoxUsbToolUrbPost failed Status (0x%x) usb Status (0x%x)\n", Status, pUrb->UrbHeader.Status));
-            }
-        }
-
-    }
-    else
-    {
-        AssertMsgFailed(("VBoxUSBSetInterface: ExAllocatePool failed!\n"));
-        Status = STATUS_NO_MEMORY;
-    }
-
-    VBoxUsbToolUrbFree(pUrb);
+    /* Clean up. */
+    if (pUrb)
+        VBoxUsbToolUrbFree(pUrb);
+    if (pNewIFInfo)
+        vboxUsbMemFree(pNewIFInfo);
+    if (pNewPipeInfo)
+        vboxUsbMemFree(pNewPipeInfo);
 
     return Status;
 }
@@ -890,6 +901,9 @@ static NTSTATUS vboxUsbRtDispatchUsbSelectInterface(PVBOXUSBDEV_EXT pDevExt, PIR
 
 static HANDLE vboxUsbRtGetPipeHandle(PVBOXUSBDEV_EXT pDevExt, uint32_t EndPointAddress)
 {
+    if (EndPointAddress == 0)
+        return pDevExt->Rt.hPipe0;
+
     for (ULONG i = 0; i < pDevExt->Rt.uNumInterfaces; i++)
     {
         for (ULONG j = 0; j < pDevExt->Rt.pVBIfaceInfo[i].pInterfaceInfo->NumberOfPipes; j++)
@@ -1014,6 +1028,8 @@ static NTSTATUS vboxUsbRtDispatchUsbAbortEndpoint(PVBOXUSBDEV_EXT pDevExt, PIRP
 
 static NTSTATUS vboxUsbRtUrbSendCompletion(PDEVICE_OBJECT pDevObj, IRP *pIrp, void *pvContext)
 {
+    RT_NOREF1(pDevObj);
+
     if (!pvContext)
     {
         AssertMsgFailed((__FUNCTION__":  context is NULL\n"));
@@ -1035,7 +1051,7 @@ static NTSTATUS vboxUsbRtUrbSendCompletion(PDEVICE_OBJECT pDevObj, IRP *pIrp, vo
     PUSBSUP_URB pUrbInfo = (PUSBSUP_URB)pContext->pOut;
     PVBOXUSBDEV_EXT pDevExt = pContext->pDevExt;
 
-    if (!pUrb || !pMdlBuf || !pUrbInfo | !pDevExt)
+    if (!pUrb || !pMdlBuf || !pUrbInfo || !pDevExt)
     {
         AssertMsgFailed((__FUNCTION__": Invalid args\n"));
         if (pDevExt)
@@ -1064,22 +1080,30 @@ static NTSTATUS vboxUsbRtUrbSendCompletion(PDEVICE_OBJECT pDevObj, IRP *pIrp, vo
             case USBD_STATUS_DEV_NOT_RESPONDING:
             default:
                 pUrbInfo->error = USBSUP_XFER_DNR;
-            break;
+                break;
         }
 
         switch(pContext->ulTransferType)
         {
-            case USBSUP_TRANSFER_TYPE_CTRL:
             case USBSUP_TRANSFER_TYPE_MSG:
                 pUrbInfo->len = pUrb->UrbControlTransfer.TransferBufferLength;
-                if (pContext->ulTransferType == USBSUP_TRANSFER_TYPE_MSG)
+                /* QUSB_TRANSFER_TYPE_MSG is a control transfer, but it is special
+                 * the first 8 bytes of the buffer is the setup packet so the real
+                 * data length is therefore urb->len - 8
+                 */
+                pUrbInfo->len += sizeof (pUrb->UrbControlTransfer.SetupPacket);
+
+                /* If a control URB was successfully completed on the default control
+                 * pipe, stash away the handle. When submitting the URB, we don't need
+                 * to know (and initially don't have) the handle. If we want to abort
+                 * the default control pipe, we *have* to have a handle. This is how we
+                 * find out what the handle is.
+                 */
+                if (!pUrbInfo->ep && (pDevExt->Rt.hPipe0 == NULL))
                 {
-                    /* QUSB_TRANSFER_TYPE_MSG is a control transfer, but it is special
-                     * the first 8 bytes of the buffer is the setup packet so the real
-                     * data length is therefore urb->len - 8
-                     */
-                    pUrbInfo->len += sizeof (pUrb->UrbControlTransfer.SetupPacket);
+                    pDevExt->Rt.hPipe0 = pUrb->UrbControlTransfer.PipeHandle;
                 }
+
                 break;
             case USBSUP_TRANSFER_TYPE_ISOC:
                 pUrbInfo->len = pUrb->UrbIsochronousTransfer.TransferBufferLength;
@@ -1107,13 +1131,12 @@ static NTSTATUS vboxUsbRtUrbSendCompletion(PDEVICE_OBJECT pDevObj, IRP *pIrp, vo
     {
         pUrbInfo->len = 0;
 
-        Log((__FUNCTION__": URB failed Status (0x%x) urb Status (0x%x)\n", Status, pUrb->UrbHeader.Status));
+        LogFunc(("URB failed Status (0x%x) urb Status (0x%x)\n", Status, pUrb->UrbHeader.Status));
 #ifdef DEBUG
         switch(pContext->ulTransferType)
         {
-            case USBSUP_TRANSFER_TYPE_CTRL:
             case USBSUP_TRANSFER_TYPE_MSG:
-                LogRel(("Ctrl/Msg length=%d\n", pUrb->UrbControlTransfer.TransferBufferLength));
+                LogRel(("Msg (CTRL) length=%d\n", pUrb->UrbControlTransfer.TransferBufferLength));
                 break;
             case USBSUP_TRANSFER_TYPE_ISOC:
                 LogRel(("ISOC length=%d\n", pUrb->UrbIsochronousTransfer.TransferBufferLength));
@@ -1135,11 +1158,12 @@ static NTSTATUS vboxUsbRtUrbSendCompletion(PDEVICE_OBJECT pDevObj, IRP *pIrp, vo
                 Status = STATUS_SUCCESS;
                 break;
             case USBD_STATUS_DEV_NOT_RESPONDING:
+            case USBD_STATUS_DEVICE_GONE:
                 pUrbInfo->error = USBSUP_XFER_DNR;
                 Status = STATUS_SUCCESS;
                 break;
             case ((USBD_STATUS)0xC0010000L): // USBD_STATUS_CANCELED - too bad usbdi.h and usb.h aren't consistent!
-                // TODO: What the heck are we really supposed to do here?
+                /// @todo What the heck are we really supposed to do here?
                 pUrbInfo->error = USBSUP_XFER_STALL;
                 Status = STATUS_SUCCESS;
                 break;
@@ -1264,7 +1288,6 @@ static NTSTATUS vboxUsbRtUrbSend(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp, PUSBSUP_URB
 
         switch (pUrbInfo->type)
         {
-            case USBSUP_TRANSFER_TYPE_CTRL:
             case USBSUP_TRANSFER_TYPE_MSG:
             {
                 pUrb->UrbHeader.Function = URB_FUNCTION_CONTROL_TRANSFER;
@@ -1277,34 +1300,25 @@ static NTSTATUS vboxUsbRtUrbSend(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp, PUSBSUP_URB
                 if (!hPipe)
                     pUrb->UrbControlTransfer.TransferFlags |= USBD_DEFAULT_PIPE_TRANSFER;
 
-                if (pUrbInfo->type == USBSUP_TRANSFER_TYPE_MSG)
-                {
-                   /* QUSB_TRANSFER_TYPE_MSG is a control transfer, but it is special
-                    * the first 8 bytes of the buffer is the setup packet so the real
-                    * data length is therefore pUrb->len - 8
-                    */
-                    PVBOXUSB_SETUP pSetup = (PVBOXUSB_SETUP)pUrb->UrbControlTransfer.SetupPacket;
-                    memcpy(pUrb->UrbControlTransfer.SetupPacket, pBuffer, min(sizeof (pUrb->UrbControlTransfer.SetupPacket), pUrbInfo->len));
+                /* QUSB_TRANSFER_TYPE_MSG is a control transfer, but it is special
+                 * the first 8 bytes of the buffer is the setup packet so the real
+                 * data length is therefore pUrb->len - 8
+                 */
+                //PVBOXUSB_SETUP pSetup = (PVBOXUSB_SETUP)pUrb->UrbControlTransfer.SetupPacket;
+                memcpy(pUrb->UrbControlTransfer.SetupPacket, pBuffer, min(sizeof (pUrb->UrbControlTransfer.SetupPacket), pUrbInfo->len));
 
-                    if (pUrb->UrbControlTransfer.TransferBufferLength <= sizeof (pUrb->UrbControlTransfer.SetupPacket))
-                        pUrb->UrbControlTransfer.TransferBufferLength = 0;
-                    else
-                        pUrb->UrbControlTransfer.TransferBufferLength -= sizeof (pUrb->UrbControlTransfer.SetupPacket);
-
-                    pUrb->UrbControlTransfer.TransferBuffer = (uint8_t *)pBuffer + sizeof(pUrb->UrbControlTransfer.SetupPacket);
-                    pUrb->UrbControlTransfer.TransferBufferMDL = 0;
-                    pUrb->UrbControlTransfer.TransferFlags |= USBD_SHORT_TRANSFER_OK;
-                }
+                if (pUrb->UrbControlTransfer.TransferBufferLength <= sizeof (pUrb->UrbControlTransfer.SetupPacket))
+                    pUrb->UrbControlTransfer.TransferBufferLength = 0;
                 else
-                {
-                    pUrb->UrbControlTransfer.TransferBuffer = 0;
-                    pUrb->UrbControlTransfer.TransferBufferMDL = pMdlBuf;
-                }
+                    pUrb->UrbControlTransfer.TransferBufferLength -= sizeof (pUrb->UrbControlTransfer.SetupPacket);
+
+                pUrb->UrbControlTransfer.TransferBuffer = (uint8_t *)pBuffer + sizeof(pUrb->UrbControlTransfer.SetupPacket);
+                pUrb->UrbControlTransfer.TransferBufferMDL = 0;
+                pUrb->UrbControlTransfer.TransferFlags |= USBD_SHORT_TRANSFER_OK;
                 break;
             }
             case USBSUP_TRANSFER_TYPE_ISOC:
             {
-                Assert(pUrbInfo->dir == USBSUP_DIRECTION_IN || pUrbInfo->type == USBSUP_TRANSFER_TYPE_BULK);
                 Assert(hPipe);
                 VBOXUSB_PIPE_INFO *pPipeInfo = vboxUsbRtGetPipeInfo(pDevExt, pUrbInfo->ep | ((pUrbInfo->dir == USBSUP_DIRECTION_IN) ? 0x80 : 0x00));
                 if (pPipeInfo == NULL)
@@ -1344,7 +1358,14 @@ static NTSTATUS vboxUsbRtUrbSend(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp, PUSBSUP_URB
                 iStartFrame = pPipeInfo->NextScheduledFrame;
                 if ((iFrame < iStartFrame) || (iStartFrame > iFrame + 512))
                     iFrame = iStartFrame;
-                pPipeInfo->NextScheduledFrame = iFrame + pUrbInfo->numIsoPkts;
+                /* For full-speed devices, there must be one transfer per frame (Windows USB
+                 * stack requirement), but URBs can contain multiple packets. For high-speed or
+                 * faster transfers, we expect one URB per frame, regardless of the interval.
+                 */
+                if (pDevExt->Rt.devdescr->bcdUSB < 0x300 && !pDevExt->Rt.fIsHighSpeed)
+                    pPipeInfo->NextScheduledFrame = iFrame + pUrbInfo->numIsoPkts;
+                else
+                    pPipeInfo->NextScheduledFrame = iFrame + 1;
                 pUrb->UrbIsochronousTransfer.StartFrame = iFrame;
                 break;
             }
@@ -1471,8 +1492,9 @@ static NTSTATUS vboxUsbRtDispatchGetVersion(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
     PUSBSUP_VERSION pVer= (PUSBSUP_VERSION)pIrp->AssociatedIrp.SystemBuffer;
     NTSTATUS Status = STATUS_SUCCESS;
 
-    if (pVer && pSl->Parameters.DeviceIoControl.InputBufferLength == 0
-             && pSl->Parameters.DeviceIoControl.OutputBufferLength == sizeof (*pVer))
+    if (   pVer
+        && pSl->Parameters.DeviceIoControl.InputBufferLength == 0
+        && pSl->Parameters.DeviceIoControl.OutputBufferLength == sizeof(*pVer))
     {
         pVer->u32Major = USBDRV_MAJOR_VERSION;
         pVer->u32Minor = USBDRV_MINOR_VERSION;
@@ -1498,14 +1520,10 @@ static NTSTATUS vboxUsbRtDispatchDefault(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
 
 DECLHIDDEN(NTSTATUS) vboxUsbRtCreate(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
 {
+    RT_NOREF1(pDevExt);
     PIO_STACK_LOCATION pSl = IoGetCurrentIrpStackLocation(pIrp);
     PFILE_OBJECT pFObj = pSl->FileObject;
-    if (!pFObj)
-    {
-        AssertFailed();
-        return STATUS_INVALID_PARAMETER;
-    }
-
+    AssertReturn(pFObj, STATUS_INVALID_PARAMETER);
     return STATUS_SUCCESS;
 }
 
@@ -1526,52 +1544,39 @@ DECLHIDDEN(NTSTATUS) vboxUsbRtDispatch(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp)
     switch (pSl->Parameters.DeviceIoControl.IoControlCode)
     {
         case SUPUSB_IOCTL_USB_CLAIM_DEVICE:
-        {
             return vboxUsbRtDispatchClaimDevice(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_USB_RELEASE_DEVICE:
-        {
             return vboxUsbRtDispatchReleaseDevice(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_GET_DEVICE:
-        {
             return vboxUsbRtDispatchGetDevice(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_USB_RESET:
-        {
             return vboxUsbRtDispatchUsbReset(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_USB_SET_CONFIG:
-        {
             return vboxUsbRtDispatchUsbSetConfig(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_USB_SELECT_INTERFACE:
-        {
             return vboxUsbRtDispatchUsbSelectInterface(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_USB_CLEAR_ENDPOINT:
-        {
             return vboxUsbRtDispatchUsbClearEndpoint(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_USB_ABORT_ENDPOINT:
-        {
             return vboxUsbRtDispatchUsbAbortEndpoint(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_SEND_URB:
-        {
             return vboxUsbRtDispatchSendUrb(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_IS_OPERATIONAL:
-        {
             return vboxUsbRtDispatchIsOperational(pDevExt, pIrp);
-        }
+
         case SUPUSB_IOCTL_GET_VERSION:
-        {
             return vboxUsbRtDispatchGetVersion(pDevExt, pIrp);
-        }
+
         default:
-        {
             return vboxUsbRtDispatchDefault(pDevExt, pIrp);
-        }
     }
 }
