@@ -3,7 +3,7 @@
  * VBox USB Monitor Device Filtering functionality
  */
 /*
- * Copyright (C) 2011-2015 Oracle Corporation
+ * Copyright (C) 2011-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -12,12 +12,21 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "VBoxUsbMon.h"
 #include "../cmn/VBoxUsbTool.h"
 
@@ -39,30 +48,11 @@
 #include <VBox/usblib.h>
 #include <devguid.h>
 
-/*
- * Note: Must match the VID & PID in the USB driver .inf file!!
- */
-/*
-  BusQueryDeviceID USB\Vid_80EE&Pid_CAFE
-  BusQueryInstanceID 2
-  BusQueryHardwareIDs USB\Vid_80EE&Pid_CAFE&Rev_0100
-  BusQueryHardwareIDs USB\Vid_80EE&Pid_CAFE
-  BusQueryCompatibleIDs USB\Class_ff&SubClass_00&Prot_00
-  BusQueryCompatibleIDs USB\Class_ff&SubClass_00
-  BusQueryCompatibleIDs USB\Class_ff
-*/
 
-#define szBusQueryDeviceId                  L"USB\\Vid_80EE&Pid_CAFE"
-#define szBusQueryHardwareIDs               L"USB\\Vid_80EE&Pid_CAFE&Rev_0100\0USB\\Vid_80EE&Pid_CAFE\0\0"
-#define szBusQueryCompatibleIDs             L"USB\\Class_ff&SubClass_00&Prot_00\0USB\\Class_ff&SubClass_00\0USB\\Class_ff\0\0"
-
-#define szDeviceTextDescription             L"VirtualBox USB"
-
-/* Possible USB bus driver names. */
-static LPWSTR lpszStandardControllerName[1] =
-{
-    L"\\Driver\\usbhub",
-};
+/* We should be including ntifs.h but that's not as easy as it sounds. */
+extern "C" {
+NTKERNELAPI PDEVICE_OBJECT IoGetDeviceAttachmentBaseRef(__in PDEVICE_OBJECT DeviceObject);
+}
 
 /*
  * state transitions:
@@ -111,6 +101,8 @@ typedef struct VBOXUSBFLT_DEVICE
     uintptr_t uFltId;
     /* true iff device is filtered with a one-shot filter */
     bool fIsFilterOneShot;
+    /* true if descriptors could not be read and only inferred from PnP Manager data */
+    bool fInferredDesc;
     /* The device state. If the non-owner session is requesting the state while the device is grabbed,
      * the USBDEVICESTATE_USED_BY_HOST is returned. */
     VBOXUSBFLT_DEVSTATE  enmState;
@@ -319,35 +311,6 @@ static PVBOXUSBFLT_DEVICE vboxUsbFltDevGetLocked(PDEVICE_OBJECT pPdo)
     return NULL;
 }
 
-PVBOXUSBFLT_DEVICE vboxUsbFltDevGet(PDEVICE_OBJECT pPdo)
-{
-    PVBOXUSBFLT_DEVICE pDevice;
-
-    VBOXUSBFLT_LOCK_ACQUIRE();
-    pDevice = vboxUsbFltDevGetLocked(pPdo);
-    /*
-     * Prevent a host crash when vboxUsbFltDevGetLocked fails to locate the matching PDO
-     * in g_VBoxUsbFltGlobals.DeviceList (see @bugref{6509}).
-     */
-    if (pDevice == NULL)
-    {
-        WARN(("failed to get device for PDO(0x%p)", pPdo));
-    }
-    else if (pDevice->enmState > VBOXUSBFLT_DEVSTATE_ADDED)
-    {
-        vboxUsbFltDevRetain(pDevice);
-        LOG(("found device (0x%p), state(%d) for PDO(0x%p)", pDevice, pDevice->enmState, pPdo));
-    }
-    else
-    {
-        LOG(("found replugging device (0x%p), state(%d) for PDO(0x%p)", pDevice, pDevice->enmState, pPdo));
-        pDevice = NULL;
-    }
-    VBOXUSBFLT_LOCK_RELEASE();
-
-    return pDevice;
-}
-
 static NTSTATUS vboxUsbFltPdoReplug(PDEVICE_OBJECT pDo)
 {
     LOG(("Replugging PDO(0x%p)", pDo));
@@ -383,12 +346,32 @@ static PVBOXUSBFLTCTX vboxUsbFltDevMatchLocked(PVBOXUSBFLT_DEVICE pDevice, uintp
     USBFilterSetNumExact(&DevFlt, USBFILTERIDX_VENDOR_ID, pDevice->idVendor, true);
     USBFilterSetNumExact(&DevFlt, USBFILTERIDX_PRODUCT_ID, pDevice->idProduct, true);
     USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_REV, pDevice->bcdDevice, true);
-    USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_CLASS, pDevice->bClass, true);
-    USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_SUB_CLASS, pDevice->bSubClass, true);
-    USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_PROTOCOL, pDevice->bProtocol, true);
-    USBFilterSetStringExact(&DevFlt, USBFILTERIDX_MANUFACTURER_STR, pDevice->szMfgName, true);
-    USBFilterSetStringExact(&DevFlt, USBFILTERIDX_PRODUCT_STR, pDevice->szProduct, true);
-    USBFilterSetStringExact(&DevFlt, USBFILTERIDX_SERIAL_NUMBER_STR, pDevice->szSerial, true);
+
+    /* If we could not read a string descriptor, don't set the filter item at all. */
+    if (pDevice->szMfgName[0])
+        USBFilterSetStringExact(&DevFlt, USBFILTERIDX_MANUFACTURER_STR, pDevice->szMfgName, true /*fMustBePresent*/, true /*fPurge*/);
+    if (pDevice->szProduct[0])
+        USBFilterSetStringExact(&DevFlt, USBFILTERIDX_PRODUCT_STR, pDevice->szProduct, true /*fMustBePresent*/, true /*fPurge*/);
+    if (pDevice->szSerial[0])
+        USBFilterSetStringExact(&DevFlt, USBFILTERIDX_SERIAL_NUMBER_STR, pDevice->szSerial, true /*fMustBePresent*/, true /*fPurge*/);
+
+    /* If device descriptor had to be inferred from PnP Manager data, the class/subclass/protocol may be wrong.
+     * When Windows reports CompatibleIDs 'USB\Class_03&SubClass_00&Prot_00', the device descriptor might be
+     * reporting class 3 (HID), *or* the device descriptor might be reporting class 0 (specified by interface)
+     * and the device's interface reporting class 3. Ignore the class/subclass/protocol in such case, since
+     * we are more or less guaranteed to rely on VID/PID anyway.
+     * See @bugref{9479}.
+     */
+    if (pDevice->fInferredDesc)
+    {
+        LOG(("Device descriptor was not read, only inferred; ignoring class/subclass/protocol!"));
+    }
+    else
+    {
+        USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_CLASS, pDevice->bClass, true);
+        USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_SUB_CLASS, pDevice->bSubClass, true);
+        USBFilterSetNumExact(&DevFlt, USBFILTERIDX_DEVICE_PROTOCOL, pDevice->bProtocol, true);
+    }
 
     /* Run filters on the thing. */
     PVBOXUSBFLTCTX pOwner = VBoxUSBFilterMatchEx(&DevFlt, puId, fRemoveFltIfOneShot, pfFilter, pfIsOneShot);
@@ -412,6 +395,131 @@ static bool vboxUsbFltDevStateIsFiltered(PVBOXUSBFLT_DEVICE pDevice)
     return pDevice->enmState >= VBOXUSBFLT_DEVSTATE_CAPTURING;
 }
 
+static uint16_t vboxUsbParseHexNumU16(WCHAR **ppStr)
+{
+    WCHAR       *pStr = *ppStr;
+    WCHAR       wc;
+    uint16_t    num = 0;
+    unsigned    u;
+
+    for (int i = 0; i < 4; ++i)
+    {
+        if (!*pStr)     /* Just in case the string is too short. */
+            break;
+
+        wc = *pStr;
+        u = wc >= 'A' ? wc - 'A' + 10 : wc - '0';   /* Hex digit to number. */
+        num |= u << (12 - 4 * i);
+        pStr++;
+    }
+    *ppStr = pStr;
+
+    return num;
+}
+
+static uint8_t vboxUsbParseHexNumU8(WCHAR **ppStr)
+{
+    WCHAR       *pStr = *ppStr;
+    WCHAR       wc;
+    uint16_t    num = 0;
+    unsigned    u;
+
+    for (int i = 0; i < 2; ++i)
+    {
+        if (!*pStr)     /* Just in case the string is too short. */
+            break;
+
+        wc = *pStr;
+        u = wc >= 'A' ? wc - 'A' + 10 : wc - '0';   /* Hex digit to number. */
+        num |= u << (4 - 4 * i);
+        pStr++;
+    }
+    *ppStr = pStr;
+
+    return num;
+}
+
+static bool vboxUsbParseHardwareID(WCHAR *pchIdStr, uint16_t *pVid, uint16_t *pPid, uint16_t *pRev)
+{
+#define VID_PREFIX  L"USB\\VID_"
+#define PID_PREFIX  L"&PID_"
+#define REV_PREFIX  L"&REV_"
+
+    *pVid = *pPid = *pRev = 0xFFFF;
+
+    /* The Hardware ID is in the format USB\VID_xxxx&PID_xxxx&REV_xxxx, with 'xxxx'
+     * being 16-bit hexadecimal numbers. The string is coming from the
+     * Windows PnP manager so OEMs should have no opportunity to mess it up.
+     */
+
+    if (wcsncmp(pchIdStr, VID_PREFIX, wcslen(VID_PREFIX)))
+        return false;
+    /* Point to the start of the vendor ID number and parse it. */
+    pchIdStr += wcslen(VID_PREFIX);
+    *pVid = vboxUsbParseHexNumU16(&pchIdStr);
+
+    if (wcsncmp(pchIdStr, PID_PREFIX, wcslen(PID_PREFIX)))
+        return false;
+    /* Point to the start of the product ID number and parse it. */
+    pchIdStr += wcslen(PID_PREFIX);
+    *pPid = vboxUsbParseHexNumU16(&pchIdStr);
+
+    /* The revision might not be there; the Windows documentation is not
+     * entirely clear if it will be always present for USB devices or not.
+     * If it's not there, still consider this a success. */
+    if (wcsncmp(pchIdStr, REV_PREFIX, wcslen(REV_PREFIX)))
+        return true;
+
+    /* Point to the start of the revision number and parse it. */
+    pchIdStr += wcslen(REV_PREFIX);
+    *pRev = vboxUsbParseHexNumU16(&pchIdStr);
+
+    return true;
+#undef VID_PREFIX
+#undef PID_PREFIX
+#undef REV_PREFIX
+}
+
+static bool vboxUsbParseCompatibleIDs(WCHAR *pchIdStr, uint8_t *pClass, uint8_t *pSubClass, uint8_t *pProt)
+{
+#define CLS_PREFIX  L"USB\\Class_"
+#define SUB_PREFIX  L"&SubClass_"
+#define PRO_PREFIX  L"&Prot_"
+
+    *pClass = *pSubClass = *pProt = 0xFF;
+
+    /* The Compatible IDs string is in the format USB\Class_xx&SubClass_xx&Prot_xx,
+     * with 'xx' being 8-bit hexadecimal numbers. Since this string is provided by the
+     * PnP manager and USB devices always report these as part of the basic USB device
+     * descriptor, we assume all three must be present.
+     */
+
+    if (wcsncmp(pchIdStr, CLS_PREFIX, wcslen(CLS_PREFIX)))
+        return false;
+    /* Point to the start of the device class and parse it. */
+    pchIdStr += wcslen(CLS_PREFIX);
+    *pClass = vboxUsbParseHexNumU8(&pchIdStr);
+
+    if (wcsncmp(pchIdStr, SUB_PREFIX, wcslen(SUB_PREFIX)))
+        return false;
+
+    /* Point to the start of the subclass and parse it. */
+    pchIdStr += wcslen(SUB_PREFIX);
+    *pSubClass = vboxUsbParseHexNumU8(&pchIdStr);
+
+    if (wcsncmp(pchIdStr, PRO_PREFIX, wcslen(PRO_PREFIX)))
+        return false;
+
+    /* Point to the start of the protocol and parse it. */
+    pchIdStr += wcslen(PRO_PREFIX);
+    *pProt = vboxUsbParseHexNumU8(&pchIdStr);
+
+    return true;
+#undef CLS_PREFIX
+#undef SUB_PREFIX
+#undef PRO_PREFIX
+}
+
 #define VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS 10000
 
 static NTSTATUS vboxUsbFltDevPopulate(PVBOXUSBFLT_DEVICE pDevice, PDEVICE_OBJECT pDo /*, BOOLEAN bPopulateNonFilterProps*/)
@@ -432,11 +540,66 @@ static NTSTATUS vboxUsbFltDevPopulate(PVBOXUSBFLT_DEVICE pDevice, PDEVICE_OBJECT
 
     do
     {
+        pDevice->fInferredDesc = false;
         Status = VBoxUsbToolGetDescriptor(pDo, pDevDr, sizeof(*pDevDr), USB_DEVICE_DESCRIPTOR_TYPE, 0, 0, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
         if (!NT_SUCCESS(Status))
         {
-            WARN(("getting device descriptor failed, Status (0x%x)", Status));
-            break;
+            WCHAR       wchPropBuf[256];
+            ULONG       ulResultLen;
+            bool        rc;
+            uint16_t    vid, pid, rev;
+            uint8_t     cls, sub, prt;
+
+            WARN(("getting device descriptor failed, Status (0x%x); falling back to IoGetDeviceProperty", Status));
+
+            /* Try falling back to IoGetDeviceProperty. */
+            Status = IoGetDeviceProperty(pDo, DevicePropertyHardwareID, sizeof(wchPropBuf), wchPropBuf, &ulResultLen);
+            if (!NT_SUCCESS(Status))
+            {
+                /* This just isn't our day. We have no idea what the device is. */
+                WARN(("IoGetDeviceProperty failed for DevicePropertyHardwareID, Status (0x%x)", Status));
+                break;
+            }
+            rc = vboxUsbParseHardwareID(wchPropBuf, &vid, &pid, &rev);
+            if (!rc)
+            {
+                /* This *really* should not happen. */
+                WARN(("Failed to parse Hardware ID"));
+                break;
+            }
+
+            /* Now grab the Compatible IDs to get the class/subclass/protocol. */
+            Status = IoGetDeviceProperty(pDo, DevicePropertyCompatibleIDs, sizeof(wchPropBuf), wchPropBuf, &ulResultLen);
+            if (!NT_SUCCESS(Status))
+            {
+                /* We really kind of need these. */
+                WARN(("IoGetDeviceProperty failed for DevicePropertyCompatibleIDs, Status (0x%x)", Status));
+                break;
+            }
+            rc = vboxUsbParseCompatibleIDs(wchPropBuf, &cls, &sub, &prt);
+            if (!rc)
+            {
+                /* This *really* should not happen. */
+                WARN(("Failed to parse Hardware ID"));
+                break;
+            }
+
+            LOG(("Parsed HardwareID: vid=%04X, pid=%04X, rev=%04X, class=%02X, subcls=%02X, prot=%02X", vid, pid, rev, cls, sub, prt));
+            if (vid == 0xFFFF || pid == 0xFFFF)
+                break;
+
+            LOG(("Successfully fell back to IoGetDeviceProperty result"));
+            pDevDr->idVendor  = vid;
+            pDevDr->idProduct = pid;
+            pDevDr->bcdDevice = rev;
+            pDevDr->bDeviceClass    = cls;
+            pDevDr->bDeviceSubClass = sub;
+            pDevDr->bDeviceProtocol = prt;
+
+            /* The USB device class/subclass/protocol may not be accurate. We have to be careful when comparing
+             * and not take mismatches too seriously.
+             */
+            pDevice->fInferredDesc = true;
         }
 
         if (vboxUsbFltBlDevMatchLocked(pDevDr->idVendor, pDevDr->idProduct, pDevDr->bcdDevice))
@@ -477,7 +640,7 @@ static NTSTATUS vboxUsbFltDevPopulate(PVBOXUSBFLT_DEVICE pDevice, PDEVICE_OBJECT
 
             if (pDevDr->iSerialNumber)
             {
-                Status = VBoxUsbToolGetStringDescriptorA(pDo, pDevice->szSerial, sizeof (pDevice->szSerial), pDevDr->iSerialNumber, langId, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
+                Status = VBoxUsbToolGetStringDescriptor(pDo, pDevice->szSerial, sizeof (pDevice->szSerial), pDevDr->iSerialNumber, langId, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
                 if (!NT_SUCCESS(Status))
                 {
                     WARN(("reading serial number failed"));
@@ -496,7 +659,7 @@ static NTSTATUS vboxUsbFltDevPopulate(PVBOXUSBFLT_DEVICE pDevice, PDEVICE_OBJECT
 
             if (pDevDr->iManufacturer)
             {
-                Status = VBoxUsbToolGetStringDescriptorA(pDo, pDevice->szMfgName, sizeof (pDevice->szMfgName), pDevDr->iManufacturer, langId, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
+                Status = VBoxUsbToolGetStringDescriptor(pDo, pDevice->szMfgName, sizeof (pDevice->szMfgName), pDevDr->iManufacturer, langId, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
                 if (!NT_SUCCESS(Status))
                 {
                     WARN(("reading manufacturer name failed"));
@@ -515,7 +678,7 @@ static NTSTATUS vboxUsbFltDevPopulate(PVBOXUSBFLT_DEVICE pDevice, PDEVICE_OBJECT
 
             if (pDevDr->iProduct)
             {
-                Status = VBoxUsbToolGetStringDescriptorA(pDo, pDevice->szProduct, sizeof (pDevice->szProduct), pDevDr->iProduct, langId, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
+                Status = VBoxUsbToolGetStringDescriptor(pDo, pDevice->szProduct, sizeof (pDevice->szProduct), pDevDr->iProduct, langId, VBOXUSBMON_POPULATE_REQUEST_TIMEOUT_MS);
                 if (!NT_SUCCESS(Status))
                 {
                     WARN(("reading product name failed"));
@@ -584,24 +747,6 @@ static NTSTATUS vboxUsbFltDevPopulate(PVBOXUSBFLT_DEVICE pDevice, PDEVICE_OBJECT
     return Status;
 }
 
-static void vboxUsbFltSignalChangeLocked()
-{
-    for (PLIST_ENTRY pEntry = g_VBoxUsbFltGlobals.ContextList.Flink;
-            pEntry != &g_VBoxUsbFltGlobals.ContextList;
-            pEntry = pEntry->Flink)
-    {
-        PVBOXUSBFLTCTX pCtx = PVBOXUSBFLTCTX_FROM_LE(pEntry);
-        /* the removed context can not be in a list */
-        Assert(!pCtx->bRemoved);
-        if (pCtx->pChangeEvent)
-        {
-            KeSetEvent(pCtx->pChangeEvent,
-                    0, /* increment*/
-                    FALSE /* wait */);
-        }
-    }
-}
-
 static bool vboxUsbFltDevCheckReplugLocked(PVBOXUSBFLT_DEVICE pDevice, PVBOXUSBFLTCTX pContext)
 {
     ASSERT_WARN(pContext, ("context is NULL!"));
@@ -782,18 +927,19 @@ typedef struct VBOXUSBFLTCHECKWALKER
     PVBOXUSBFLTCTX pContext;
 } VBOXUSBFLTCHECKWALKER, *PVBOXUSBFLTCHECKWALKER;
 
-static DECLCALLBACK(BOOLEAN) vboxUsbFltFilterCheckWalker(PFILE_OBJECT pFile, PDEVICE_OBJECT pTopDo, PDEVICE_OBJECT pHubDo, PVOID pvContext)
+static DECLCALLBACK(BOOLEAN) vboxUsbFltFilterCheckWalker(PFILE_OBJECT pHubFile,
+                                                         PDEVICE_OBJECT pHubDo, PVOID pvContext)
 {
     PVBOXUSBFLTCHECKWALKER pData = (PVBOXUSBFLTCHECKWALKER)pvContext;
     PVBOXUSBFLTCTX pContext = pData->pContext;
 
-    LOG(("Visiting pFile(0x%p), pTopDo(0x%p), pHubDo(0x%p), oContext(0x%p)", pFile, pTopDo, pHubDo, pContext));
+    LOG(("Visiting pHubFile(0x%p), pHubDo(0x%p), oContext(0x%p)", pHubFile, pHubDo, pContext));
     KIRQL Irql = KeGetCurrentIrql();
     ASSERT_WARN(Irql == PASSIVE_LEVEL, ("unexpected IRQL (%d)", Irql));
 
     PDEVICE_RELATIONS pDevRelations = NULL;
 
-    NTSTATUS Status = VBoxUsbMonQueryBusRelations(pTopDo, pFile, &pDevRelations);
+    NTSTATUS Status = VBoxUsbMonQueryBusRelations(pHubDo, pHubFile, &pDevRelations);
     if (Status == STATUS_SUCCESS && pDevRelations)
     {
         ULONG cReplugPdos = pDevRelations->Count;
@@ -801,7 +947,15 @@ static DECLCALLBACK(BOOLEAN) vboxUsbFltFilterCheckWalker(PFILE_OBJECT pFile, PDE
         InitializeListHead(&ReplugDevList);
         for (ULONG k = 0; k < pDevRelations->Count; ++k)
         {
-            PDEVICE_OBJECT pDevObj = pDevRelations->Objects[k];
+            PDEVICE_OBJECT pDevObj;
+
+            /* Grab the PDO+reference. We won't need the upper layer device object
+             * anymore, so dereference that right here, and drop the PDO ref later.
+             */
+            pDevObj = IoGetDeviceAttachmentBaseRef(pDevRelations->Objects[k]);
+            LOG(("DevObj=%p, PDO=%p\n", pDevRelations->Objects[k], pDevObj));
+            ObDereferenceObject(pDevRelations->Objects[k]);
+            pDevRelations->Objects[k] = pDevObj;
 
             LOG(("Found existing USB PDO 0x%p", pDevObj));
             VBOXUSBFLT_LOCK_ACQUIRE();
@@ -842,9 +996,10 @@ static DECLCALLBACK(BOOLEAN) vboxUsbFltFilterCheckWalker(PFILE_OBJECT pFile, PDE
                 bool fIsOneShot = false;
                 VBOXUSBFLT_LOCK_ACQUIRE();
                 PVBOXUSBFLTCTX pCtx = vboxUsbFltDevMatchLocked(&Device, &uId,
-                        false, /* do not remove a one-shot filter */
-                        &fFilter, &fIsOneShot);
+                                                               false, /* do not remove a one-shot filter */
+                                                               &fFilter, &fIsOneShot);
                 VBOXUSBFLT_LOCK_RELEASE();
+                NOREF(pCtx);
                 LOG(("Matching Info: Filter (0x%p), pCtx(0x%p), fFilter(%d), fIsOneShot(%d)", uId, pCtx, (int)fFilter, (int)fIsOneShot));
                 if (fFilter)
                 {
@@ -878,13 +1033,13 @@ static DECLCALLBACK(BOOLEAN) vboxUsbFltFilterCheckWalker(PFILE_OBJECT pFile, PDE
                     continue;
 
                 Status = vboxUsbFltPdoReplug(pDevRelations->Objects[k]);
-                ASSERT_WARN(Status == STATUS_SUCCESS, ("vboxUsbFltPdoReplug ailed Status(0x%x)", Status));
+                ASSERT_WARN(Status == STATUS_SUCCESS, ("vboxUsbFltPdoReplug failed! Status(0x%x)", Status));
                 ObDereferenceObject(pDevRelations->Objects[k]);
                 if (!--cReplugPdos)
                     break;
             }
 
-            ASSERT_WARN(!cReplugPdos, ("cReplugPdosreached zero!"));
+            ASSERT_WARN(!cReplugPdos, ("cReplugPdos reached zero!"));
         }
 
         vboxUsbFltReplugList(&ReplugDevList);
@@ -893,11 +1048,11 @@ static DECLCALLBACK(BOOLEAN) vboxUsbFltFilterCheckWalker(PFILE_OBJECT pFile, PDE
     }
     else
     {
-        WARN(("VBoxUsbMonQueryBusRelations failed for DO(0x%p), Status(0x%x), pDevRelations(0x%p)",
-                pTopDo, Status, pDevRelations));
+        WARN(("VBoxUsbMonQueryBusRelations failed for hub DO(0x%p), Status(0x%x), pDevRelations(0x%p)",
+                pHubDo, Status, pDevRelations));
     }
 
-    LOG(("Done Visiting pFile(0x%p), pTopDo(0x%p), pHubDo(0x%p), oContext(0x%p)", pFile, pTopDo, pHubDo, pContext));
+    LOG(("Done Visiting pHubFile(0x%p), pHubDo(0x%p), oContext(0x%p)", pHubFile, pHubDo, pContext));
 
     return TRUE;
 }
@@ -911,7 +1066,7 @@ NTSTATUS VBoxUsbFltFilterCheck(PVBOXUSBFLTCTX pContext)
 
     VBOXUSBFLTCHECKWALKER Data;
     Data.pContext = pContext;
-    vboxUsbMonHubDevWalk(vboxUsbFltFilterCheckWalker, &Data, VBOXUSBMONHUBWALK_F_FDO);
+    vboxUsbMonHubDevWalk(vboxUsbFltFilterCheckWalker, &Data);
 
     LOG(("DONE Running filters, Context (0x%p)", pContext));
 
@@ -930,21 +1085,8 @@ NTSTATUS VBoxUsbFltClose(PVBOXUSBFLTCTX pContext)
     ASSERT_WARN(Irql == PASSIVE_LEVEL, ("irql==(%d)", Irql));
 
     VBOXUSBFLT_LOCK_ACQUIRE();
-    uint32_t cActiveFilters = pContext->cActiveFilters;
+
     pContext->bRemoved = TRUE;
-    if (pContext->pChangeEvent)
-    {
-        LOG(("seting & closing change event (0x%p)", pContext->pChangeEvent));
-        KeSetEvent(pContext->pChangeEvent,
-                0, /* increment*/
-                FALSE /* wait */);
-        ObDereferenceObject(pContext->pChangeEvent);
-        pContext->pChangeEvent = NULL;
-    }
-    else
-    {
-        LOG(("no change event"));
-    }
     RemoveEntryList(&pContext->ListEntry);
 
     LOG(("removing owner filters"));
@@ -955,8 +1097,8 @@ NTSTATUS VBoxUsbFltClose(PVBOXUSBFLTCTX pContext)
     LOG(("enumerating devices.."));
     /* 2. check if there are devices owned */
     for (PLIST_ENTRY pEntry = g_VBoxUsbFltGlobals.DeviceList.Flink;
-            pEntry != &g_VBoxUsbFltGlobals.DeviceList;
-            pEntry = pEntry->Flink)
+         pEntry != &g_VBoxUsbFltGlobals.DeviceList;
+         pEntry = pEntry->Flink)
     {
         PVBOXUSBFLT_DEVICE pDevice = PVBOXUSBFLT_DEVICE_FROM_LE(pEntry);
         if (pDevice->pOwner != pContext)
@@ -983,6 +1125,7 @@ NTSTATUS VBoxUsbFltClose(PVBOXUSBFLTCTX pContext)
             LOG(("device does NOT need replug"));
         }
     }
+
     VBOXUSBFLT_LOCK_RELEASE();
 
     /* this should replug all devices that were either skipped or grabbed due to the context's */
@@ -1030,6 +1173,22 @@ int VBoxUsbFltAdd(PVBOXUSBFLTCTX pContext, PUSBFILTER pFilter, uintptr_t *pId)
     USBFilterSetMustBePresent(pFilter, USBFILTERIDX_BUS, false);
     USBFilterSetMustBePresent(pFilter, USBFILTERIDX_PORT, false);
 
+    /* We may not be able to reconstruct the class/subclass/protocol if we aren't able to
+     * read the device descriptor. Don't require these to be present. See also the fInferredDesc flag.
+     */
+    USBFilterSetMustBePresent(pFilter, USBFILTERIDX_DEVICE_CLASS, false);
+    USBFilterSetMustBePresent(pFilter, USBFILTERIDX_DEVICE_SUB_CLASS, false);
+    USBFilterSetMustBePresent(pFilter, USBFILTERIDX_DEVICE_PROTOCOL, false);
+
+    /* We may also be unable to read string descriptors. Often the userland can't read the
+     * string descriptors either because the device is in a low-power state, but it can happen
+     * that the userland gets lucky and reads the strings, but by the time we get to read them
+     * they're inaccessible due to power management. So, don't require the strings to be present.
+     */
+    USBFilterSetMustBePresent(pFilter, USBFILTERIDX_MANUFACTURER_STR, false);
+    USBFilterSetMustBePresent(pFilter, USBFILTERIDX_PRODUCT_STR, false);
+    USBFilterSetMustBePresent(pFilter, USBFILTERIDX_SERIAL_NUMBER_STR, false);
+
     uintptr_t uId = 0;
     VBOXUSBFLT_LOCK_ACQUIRE();
     /* Add the filter. */
@@ -1037,7 +1196,7 @@ int VBoxUsbFltAdd(PVBOXUSBFLTCTX pContext, PUSBFILTER pFilter, uintptr_t *pId)
     VBOXUSBFLT_LOCK_RELEASE();
     if (RT_SUCCESS(rc))
     {
-        LOG(("ADDED filer id 0x%p", uId));
+        LOG(("ADDED filter id 0x%p", uId));
         ASSERT_WARN(uId, ("uid is NULL"));
 #ifdef VBOX_USBMON_WITH_FILTER_AUTOAPPLY
         VBoxUsbFltFilterCheck();
@@ -1105,36 +1264,6 @@ int VBoxUsbFltRemove(PVBOXUSBFLTCTX pContext, uintptr_t uId)
     return rc;
 }
 
-NTSTATUS VBoxUsbFltSetNotifyEvent(PVBOXUSBFLTCTX pContext, HANDLE hEvent)
-{
-    NTSTATUS Status = STATUS_SUCCESS;
-    PKEVENT pEvent = NULL;
-    PKEVENT pOldEvent = NULL;
-    if (hEvent)
-    {
-        Status = ObReferenceObjectByHandle(hEvent,
-                    EVENT_MODIFY_STATE,
-                    *ExEventObjectType, UserMode,
-                    (PVOID*)&pEvent,
-                    NULL);
-        Assert(Status == STATUS_SUCCESS);
-        if (!NT_SUCCESS(Status))
-            return Status;
-    }
-
-    VBOXUSBFLT_LOCK_ACQUIRE();
-    pOldEvent = pContext->pChangeEvent;
-    pContext->pChangeEvent = pEvent;
-    VBOXUSBFLT_LOCK_RELEASE();
-
-    if (pOldEvent)
-    {
-        ObDereferenceObject(pOldEvent);
-    }
-
-    return STATUS_SUCCESS;
-}
-
 static USBDEVICESTATE vboxUsbDevGetUserState(PVBOXUSBFLTCTX pContext, PVBOXUSBFLT_DEVICE pDevice)
 {
     if (vboxUsbFltDevStateIsNotFiltered(pDevice))
@@ -1191,12 +1320,15 @@ static void vboxUsbDevToUserInfo(PVBOXUSBFLTCTX pContext, PVBOXUSBFLT_DEVICE pDe
         strcpy(pDevInfo->szObjName, pDevice->szObjName);
     }
     pDevInfo->fHighSpeed = pDevice->fHighSpeed;
+#else
+    RT_NOREF3(pContext, pDevice, pDevInfo);
 #endif
 }
 
 NTSTATUS VBoxUsbFltGetDevice(PVBOXUSBFLTCTX pContext, HVBOXUSBDEVUSR hDevice, PUSBSUP_GETDEV_MON pInfo)
 {
-    Assert(hDevice);
+    if (!hDevice)
+       return STATUS_INVALID_PARAMETER;
 
     memset (pInfo, 0, sizeof (*pInfo));
     VBOXUSBFLT_LOCK_ACQUIRE();
@@ -1219,9 +1351,7 @@ NTSTATUS VBoxUsbFltGetDevice(PVBOXUSBFLTCTX pContext, HVBOXUSBDEVUSR hDevice, PU
 
     VBOXUSBFLT_LOCK_RELEASE();
 
-    /* this should not occur */
-    AssertFailed();
-
+    /* We should not get this far with valid input. */
     return STATUS_INVALID_PARAMETER;
 }
 
@@ -1230,6 +1360,13 @@ NTSTATUS VBoxUsbFltPdoAdd(PDEVICE_OBJECT pPdo, BOOLEAN *pbFiltered)
     *pbFiltered = FALSE;
     PVBOXUSBFLT_DEVICE pDevice;
 
+    /* Find the real PDO+reference. Dereference when we're done with it. Note that
+     * the input pPdo was not explicitly referenced so we're not dropping its ref.
+     */
+    PDEVICE_OBJECT pDevObj = IoGetDeviceAttachmentBaseRef(pPdo);
+    LOG(("DevObj=%p, real PDO=%p\n", pPdo, pDevObj));
+    pPdo = pDevObj;
+
     /* first check if device is in the a already */
     VBOXUSBFLT_LOCK_ACQUIRE();
     pDevice = vboxUsbFltDevGetLocked(pPdo);
@@ -1240,6 +1377,7 @@ NTSTATUS VBoxUsbFltPdoAdd(PDEVICE_OBJECT pPdo, BOOLEAN *pbFiltered)
         ASSERT_WARN(pDevice->enmState != VBOXUSBFLT_DEVSTATE_REMOVED, ("VBOXUSBFLT_DEVSTATE_REMOVED state for device(0x%p)", pDevice));
         *pbFiltered = pDevice->enmState >= VBOXUSBFLT_DEVSTATE_CAPTURING;
         VBOXUSBFLT_LOCK_RELEASE();
+        ObDereferenceObject(pPdo);
         return STATUS_SUCCESS;
     }
     VBOXUSBFLT_LOCK_RELEASE();
@@ -1247,6 +1385,7 @@ NTSTATUS VBoxUsbFltPdoAdd(PDEVICE_OBJECT pPdo, BOOLEAN *pbFiltered)
     if (!pDevice)
     {
         WARN(("VBoxUsbMonMemAllocZ failed"));
+        ObDereferenceObject(pPdo);
         return STATUS_NO_MEMORY;
     }
 
@@ -1256,6 +1395,7 @@ NTSTATUS VBoxUsbFltPdoAdd(PDEVICE_OBJECT pPdo, BOOLEAN *pbFiltered)
     if (!NT_SUCCESS(Status))
     {
         WARN(("vboxUsbFltDevPopulate failed, Status 0x%x", Status));
+        ObDereferenceObject(pPdo);
         VBoxUsbMonMemFree(pDevice);
         return Status;
     }
@@ -1268,6 +1408,10 @@ NTSTATUS VBoxUsbFltPdoAdd(PDEVICE_OBJECT pPdo, BOOLEAN *pbFiltered)
     VBOXUSBFLT_LOCK_ACQUIRE();
     /* (paranoia) re-check the device is still not here */
     pTmpDev = vboxUsbFltDevGetLocked(pPdo);
+
+    /* Drop the PDO ref, now we won't need it anymore. */
+    ObDereferenceObject(pPdo);
+
     if (pTmpDev)
     {
         LOG(("second try: found device (0x%p), state(%d) for PDO(0x%p)", pDevice, pDevice->enmState, pPdo));
@@ -1313,24 +1457,25 @@ NTSTATUS VBoxUsbFltPdoAdd(PDEVICE_OBJECT pPdo, BOOLEAN *pbFiltered)
     return STATUS_SUCCESS;
 }
 
-NTSTATUS VBoxUsbFltPdoAddCompleted(PDEVICE_OBJECT pPdo)
-{
-    VBOXUSBFLT_LOCK_ACQUIRE();
-    vboxUsbFltSignalChangeLocked();
-    VBOXUSBFLT_LOCK_RELEASE();
-    return STATUS_SUCCESS;
-}
-
 BOOLEAN VBoxUsbFltPdoIsFiltered(PDEVICE_OBJECT pPdo)
 {
     VBOXUSBFLT_DEVSTATE enmState = VBOXUSBFLT_DEVSTATE_REMOVED;
+
+    /* Find the real PDO+reference. Dereference when we're done with it. Note that
+     * the input pPdo was not explicitly referenced so we're not dropping its ref.
+     */
+    PDEVICE_OBJECT pDevObj = IoGetDeviceAttachmentBaseRef(pPdo);
+    LOG(("DevObj=%p, real PDO=%p\n", pPdo, pDevObj));
+    pPdo = pDevObj;
+
     VBOXUSBFLT_LOCK_ACQUIRE();
+
     PVBOXUSBFLT_DEVICE pDevice = vboxUsbFltDevGetLocked(pPdo);
     if (pDevice)
-    {
         enmState = pDevice->enmState;
-    }
+
     VBOXUSBFLT_LOCK_RELEASE();
+    ObDereferenceObject(pPdo);
 
     return enmState >= VBOXUSBFLT_DEVSTATE_CAPTURING;
 }
@@ -1340,6 +1485,13 @@ NTSTATUS VBoxUsbFltPdoRemove(PDEVICE_OBJECT pPdo)
     PVBOXUSBFLT_DEVICE pDevice;
     VBOXUSBFLT_DEVSTATE enmOldState;
 
+    /* Find the real PDO+reference. Dereference when we're done with it. Note that
+     * the input pPdo was not explicitly referenced so we're not dropping its ref.
+     */
+    PDEVICE_OBJECT pDevObj = IoGetDeviceAttachmentBaseRef(pPdo);
+    LOG(("DevObj=%p, real PDO=%p\n", pPdo, pDevObj));
+    pPdo = pDevObj;
+
     VBOXUSBFLT_LOCK_ACQUIRE();
     pDevice = vboxUsbFltDevGetLocked(pPdo);
     if (pDevice)
@@ -1347,20 +1499,9 @@ NTSTATUS VBoxUsbFltPdoRemove(PDEVICE_OBJECT pPdo)
         RemoveEntryList(&pDevice->GlobalLe);
         enmOldState = pDevice->enmState;
         pDevice->enmState = VBOXUSBFLT_DEVSTATE_REMOVED;
-        if (enmOldState != VBOXUSBFLT_DEVSTATE_REPLUGGING)
-        {
-            vboxUsbFltSignalChangeLocked();
-        }
-        else
-        {
-            /* the device *should* reappear, do signlling on re-appear only
-             * to avoid extra signaling. still there might be a situation
-             * when the device will not re-appear if it gets physically removed
-             * before it re-appears
-             * @todo: set a timer callback to do a notification from it */
-        }
     }
     VBOXUSBFLT_LOCK_RELEASE();
+    ObDereferenceObject(pPdo);
     if (pDevice)
         vboxUsbFltDevRelease(pDevice);
     return STATUS_SUCCESS;
@@ -1370,6 +1511,8 @@ HVBOXUSBFLTDEV VBoxUsbFltProxyStarted(PDEVICE_OBJECT pPdo)
 {
     PVBOXUSBFLT_DEVICE pDevice;
     VBOXUSBFLT_LOCK_ACQUIRE();
+
+    /* NB: The USB proxy (VBoxUSB.sys) passes us the real PDO, not anything above that. */
     pDevice = vboxUsbFltDevGetLocked(pPdo);
     /*
      * Prevent a host crash when vboxUsbFltDevGetLocked fails to locate the matching PDO
@@ -1379,12 +1522,11 @@ HVBOXUSBFLTDEV VBoxUsbFltProxyStarted(PDEVICE_OBJECT pPdo)
     {
         WARN(("failed to get device for PDO(0x%p)", pPdo));
     }
-    else if (pDevice->enmState = VBOXUSBFLT_DEVSTATE_CAPTURING)
+    else if (pDevice->enmState == VBOXUSBFLT_DEVSTATE_CAPTURING)
     {
         pDevice->enmState = VBOXUSBFLT_DEVSTATE_CAPTURED;
-        LOG(("The proxy notified proxy start for the captured device 0x%x", pDevice));
+        LOG(("The proxy notified proxy start for the captured device 0x%p", pDevice));
         vboxUsbFltDevRetain(pDevice);
-        vboxUsbFltSignalChangeLocked();
     }
     else
     {
@@ -1412,13 +1554,12 @@ void VBoxUsbFltProxyStopped(HVBOXUSBFLTDEV hDev)
             || pDevice->enmState == VBOXUSBFLT_DEVSTATE_USED_BY_GUEST)
     {
         /* this is due to devie was physically removed */
-        LOG(("The proxy notified proxy stop for the captured device 0x%x, current state %d", pDevice, pDevice->enmState));
+        LOG(("The proxy notified proxy stop for the captured device 0x%p, current state %d", pDevice, pDevice->enmState));
         pDevice->enmState = VBOXUSBFLT_DEVSTATE_CAPTURING;
-        vboxUsbFltSignalChangeLocked();
     }
     else
     {
-        if(pDevice->enmState != VBOXUSBFLT_DEVSTATE_REPLUGGING)
+        if (pDevice->enmState != VBOXUSBFLT_DEVSTATE_REPLUGGING)
         {
             WARN(("invalid state, %d", pDevice->enmState));
         }
