@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008 Oracle Corporation
+ * Copyright (C) 2008-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -34,9 +34,10 @@
  * is changed or when the request times out.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_HGCM
 #include <VBox/HostServices/GuestPropertySvc.h>
 
@@ -51,8 +52,8 @@
 #include <iprt/string.h>
 #include <iprt/thread.h>
 #include <iprt/time.h>
+#include <VBox/vmm/dbgf.h>
 
-#include <memory>  /* for auto_ptr */
 #include <string>
 #include <list>
 
@@ -265,6 +266,28 @@ private:
         return VINF_SUCCESS;
     }
 
+    /**
+     * Check whether the property name is reserved for host changes only.
+     *
+     * @returns Boolean true (host reserved) or false (available to guest).
+     *
+     * @param   pszName  The property name to check.
+     */
+    bool checkHostReserved(const char *pszName)
+    {
+        if (RTStrStartsWith(pszName, "/VirtualBox/GuestAdd/VBoxService/"))
+            return true;
+        if (RTStrStartsWith(pszName, "/VirtualBox/GuestAdd/PAM/"))
+            return true;
+        if (RTStrStartsWith(pszName, "/VirtualBox/GuestAdd/Greeter/"))
+            return true;
+        if (RTStrStartsWith(pszName, "/VirtualBox/GuestAdd/SharedFolders/"))
+            return true;
+        if (RTStrStartsWith(pszName, "/VirtualBox/HostInfo/"))
+            return true;
+        return false;
+    }
+
     /**
      * Gets a property.
      *
@@ -289,12 +312,12 @@ public:
         , mcTimestampAdjustments(0)
 #ifdef ASYNC_HOST_NOTIFY
         , mhThreadNotifyHost(NIL_RTTHREAD)
-        , mpReqQNotifyHost(NULL)
+        , mhReqQNotifyHost(NIL_RTREQQUEUE)
 #endif
     { }
 
     /**
-     * @copydoc VBOXHGCMSVCHELPERS::pfnUnload
+     * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnUnload}
      * Simply deletes the service object
      */
     static DECLCALLBACK(int) svcUnload(void *pvService)
@@ -309,7 +332,7 @@ public:
     }
 
     /**
-     * @copydoc VBOXHGCMSVCHELPERS::pfnConnect
+     * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnConnect}
      * Stub implementation of pfnConnect and pfnDisconnect.
      */
     static DECLCALLBACK(int) svcConnectDisconnect(void * /* pvService */,
@@ -320,7 +343,7 @@ public:
     }
 
     /**
-     * @copydoc VBOXHGCMSVCHELPERS::pfnCall
+     * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnCall}
      * Wraps to the call member function
      */
     static DECLCALLBACK(void) svcCall(void * pvService,
@@ -339,7 +362,7 @@ public:
     }
 
     /**
-     * @copydoc VBOXHGCMSVCHELPERS::pfnHostCall
+     * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnHostCall}
      * Wraps to the hostCall member function
      */
     static DECLCALLBACK(int) svcHostCall(void *pvService,
@@ -356,7 +379,7 @@ public:
     }
 
     /**
-     * @copydoc VBOXHGCMSVCHELPERS::pfnRegisterExtension
+     * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnRegisterExtension}
      * Installs a host callback for notifications of property changes.
      */
     static DECLCALLBACK(int) svcRegisterExtension(void *pvService,
@@ -369,6 +392,7 @@ public:
         pSelf->mpvHostData = pvExtension;
         return VINF_SUCCESS;
     }
+
 #ifdef ASYNC_HOST_NOTIFY
     int initialize();
 #endif
@@ -397,14 +421,18 @@ private:
               VBOXHGCMSVCPARM paParms[]);
     int hostCall(uint32_t eFunction, uint32_t cParms, VBOXHGCMSVCPARM paParms[]);
     int uninit();
+    void dbgInfoShow(PCDBGFINFOHLP pHlp);
+    static DECLCALLBACK(void) dbgInfo(void *pvUser, PCDBGFINFOHLP pHlp, const char *pszArgs);
 
 #ifdef ASYNC_HOST_NOTIFY
     /* Thread for handling host notifications. */
     RTTHREAD mhThreadNotifyHost;
     /* Queue for handling requests for notifications. */
-    RTREQQUEUE *mpReqQNotifyHost;
+    RTREQQUEUE mhReqQNotifyHost;
     static DECLCALLBACK(int) threadNotifyHost(RTTHREAD self, void *pvUser);
 #endif
+
+    DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(Service);
 };
 
 
@@ -467,7 +495,7 @@ int Service::validateName(const char *pszName, uint32_t cbName)
  */
 int Service::validateValue(const char *pszValue, uint32_t cbValue)
 {
-    LogFlowFunc(("cbValue=%d\n", cbValue));
+    LogFlowFunc(("cbValue=%d\n", cbValue)); RT_NOREF1(pszValue);
 
     int rc = VINF_SUCCESS;
     if (RT_SUCCESS(rc) && cbValue == 0)
@@ -507,56 +535,62 @@ int Service::setPropertyBlock(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
         )
         rc = VERR_INVALID_PARAMETER;
     /** @todo validate the array sizes... */
-
-    for (unsigned i = 0; RT_SUCCESS(rc) && papszNames[i] != NULL; ++i)
+    else
     {
-        if (   !RT_VALID_PTR(papszNames[i])
-            || !RT_VALID_PTR(papszValues[i])
-            || !RT_VALID_PTR(papszFlags[i])
-            )
-            rc = VERR_INVALID_POINTER;
-        else
+        for (unsigned i = 0; RT_SUCCESS(rc) && papszNames[i] != NULL; ++i)
         {
-            uint32_t fFlagsIgn;
-            rc = validateFlags(papszFlags[i], &fFlagsIgn);
-        }
-    }
-
-    if (RT_SUCCESS(rc))
-    {
-        /*
-         * Add the properties.  No way to roll back here.
-         */
-        for (unsigned i = 0; papszNames[i] != NULL; ++i)
-        {
-            uint32_t fFlags;
-            rc = validateFlags(papszFlags[i], &fFlags);
-            AssertRCBreak(rc);
-
-            Property *pProp = getPropertyInternal(papszNames[i]);
-            if (pProp)
-            {
-                /* Update existing property. */
-                pProp->mValue     = papszValues[i];
-                pProp->mTimestamp = pau64Timestamps[i];
-                pProp->mFlags     = fFlags;
-            }
+            if (   !RT_VALID_PTR(papszNames[i])
+                || !RT_VALID_PTR(papszValues[i])
+                || !RT_VALID_PTR(papszFlags[i])
+                )
+                rc = VERR_INVALID_POINTER;
             else
             {
-                /* Create a new property */
-                pProp = new Property(papszNames[i], papszValues[i], pau64Timestamps[i], fFlags);
-                if (!pProp)
+                uint32_t fFlagsIgn;
+                rc = validateFlags(papszFlags[i], &fFlagsIgn);
+            }
+        }
+        if (RT_SUCCESS(rc))
+        {
+            /*
+             * Add the properties.  No way to roll back here.
+             */
+            for (unsigned i = 0; papszNames[i] != NULL; ++i)
+            {
+                uint32_t fFlags;
+                rc = validateFlags(papszFlags[i], &fFlags);
+                AssertRCBreak(rc);
+                /*
+                 * Handle names which are read-only for the guest.
+                 */
+                if (checkHostReserved(papszNames[i]))
+                    fFlags |= RDONLYGUEST;
+
+                Property *pProp = getPropertyInternal(papszNames[i]);
+                if (pProp)
                 {
-                    rc = VERR_NO_MEMORY;
-                    break;
+                    /* Update existing property. */
+                    pProp->mValue     = papszValues[i];
+                    pProp->mTimestamp = pau64Timestamps[i];
+                    pProp->mFlags     = fFlags;
                 }
-                if (RTStrSpaceInsert(&mhProperties, &pProp->mStrCore))
-                    mcProperties++;
                 else
                 {
-                    delete pProp;
-                    rc = VERR_INTERNAL_ERROR_3;
-                    AssertFailedBreak();
+                    /* Create a new property */
+                    pProp = new Property(papszNames[i], papszValues[i], pau64Timestamps[i], fFlags);
+                    if (!pProp)
+                    {
+                        rc = VERR_NO_MEMORY;
+                        break;
+                    }
+                    if (RTStrSpaceInsert(&mhProperties, &pProp->mStrCore))
+                        mcProperties++;
+                    else
+                    {
+                        delete pProp;
+                        rc = VERR_INTERNAL_ERROR_3;
+                        AssertFailedBreak();
+                    }
                 }
             }
         }
@@ -581,8 +615,9 @@ int Service::getProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
 {
     int         rc;
     const char *pcszName = NULL;        /* shut up gcc */
-    char       *pchBuf;
-    uint32_t    cbName, cbBuf;
+    char       *pchBuf = NULL;          /* shut up MSC */
+    uint32_t    cbName;
+    uint32_t    cbBuf = 0;              /* shut up MSC */
 
     /*
      * Get and validate the parameters
@@ -706,6 +741,16 @@ int Service::setProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGues
      */
     Property *pProp = getPropertyInternal(pcszName);
     rc = checkPermission(pProp ? (ePropFlags)pProp->mFlags : NILFLAG, isGuest);
+    /*
+     * Handle names which are read-only for the guest.
+     */
+    if (rc == VINF_SUCCESS && checkHostReserved(pcszName))
+    {
+        if (isGuest)
+            rc = VERR_PERMISSION_DENIED;
+        else
+            fFlags |= RDONLYGUEST;
+    }
     if (rc == VINF_SUCCESS)
     {
         /*
@@ -735,7 +780,7 @@ int Service::setProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGues
                     rc = VERR_ALREADY_EXISTS;
                 }
             }
-            catch (std::bad_alloc)
+            catch (std::bad_alloc &)
             {
                 rc = VERR_NO_MEMORY;
             }
@@ -1017,18 +1062,19 @@ int Service::getNotificationWriteOut(uint32_t cParms, VBOXHGCMSVCPARM paParms[],
             buffer += szFlags;
             buffer += '\0';
             u64Timestamp = prop.mTimestamp;
+
+            /* Write out the data. */
+            if (RT_SUCCESS(rc))
+            {
+                paParms[1].setUInt64(u64Timestamp);
+                paParms[3].setUInt32((uint32_t)buffer.size());
+                if (buffer.size() <= cbBuf)
+                    buffer.copy(pchBuf, cbBuf);
+                else
+                    rc = VERR_BUFFER_OVERFLOW;
+            }
         }
     }
-    /* Write out the data. */
-    if (RT_SUCCESS(rc))
-    {
-        paParms[1].setUInt64(u64Timestamp);
-        paParms[3].setUInt32((uint32_t)buffer.size());
-        if (buffer.size() <= cbBuf)
-            buffer.copy(pchBuf, cbBuf);
-        else
-            rc = VERR_BUFFER_OVERFLOW;
-    }
     return rc;
 }
 
@@ -1037,8 +1083,10 @@ int Service::getNotificationWriteOut(uint32_t cParms, VBOXHGCMSVCPARM paParms[],
  * Get the next guest notification.
  *
  * @returns iprt status value
- * @param   cParms  the number of HGCM parameters supplied
- * @param   paParms the array of HGCM parameters
+ * @param   u32ClientId the client ID
+ * @param   callHandle  handle
+ * @param   cParms      the number of HGCM parameters supplied
+ * @param   paParms     the array of HGCM parameters
  * @thread  HGCM
  * @throws  can throw std::bad_alloc
  */
@@ -1056,63 +1104,64 @@ int Service::getNotification(uint32_t u32ClientId, VBOXHGCMCALLHANDLE callHandle
      * Get the HGCM function arguments and perform basic verification.
      */
     LogFlowThisFunc(("\n"));
-    if (   (cParms != 4)  /* Hardcoded value as the next lines depend on it. */
+    if (   cParms != 4  /* Hardcoded value as the next lines depend on it. */
         || RT_FAILURE(paParms[0].getString(&pszPatterns, &cchPatterns))  /* patterns */
         || RT_FAILURE(paParms[1].getUInt64(&u64Timestamp))  /* timestamp */
         || RT_FAILURE(paParms[2].getBuffer((void **)&pchBuf, &cbBuf))  /* return buffer */
        )
         rc = VERR_INVALID_PARAMETER;
-
-    if (RT_SUCCESS(rc))
+    else
+    {
         LogFlow(("pszPatterns=%s, u64Timestamp=%llu\n", pszPatterns, u64Timestamp));
 
-    /*
-     * If no timestamp was supplied or no notification was found in the queue
-     * of old notifications, enqueue the request in the waiting queue.
-     */
-    Property prop;
-    if (RT_SUCCESS(rc) && u64Timestamp != 0)
-        rc = getOldNotification(pszPatterns, u64Timestamp, &prop);
-    if (RT_SUCCESS(rc))
-    {
-        if (prop.isNull())
-        {
-            /*
-             * Check if the client already had the same request.
-             * Complete the old request with an error in this case.
-             * Protection against clients, which cancel and resubmits requests.
-             */
-            CallList::iterator it = mGuestWaiters.begin();
-            while (it != mGuestWaiters.end())
-            {
-                const char *pszPatternsExisting;
-                uint32_t cchPatternsExisting;
-                int rc3 = it->mParms[0].getString(&pszPatternsExisting, &cchPatternsExisting);
-
-                if (   RT_SUCCESS(rc3)
-                    && u32ClientId == it->u32ClientId
-                    && RTStrCmp(pszPatterns, pszPatternsExisting) == 0)
-                {
-                    /* Complete the old request. */
-                    mpHelpers->pfnCallComplete(it->mHandle, VERR_INTERRUPTED);
-                    it = mGuestWaiters.erase(it);
-                }
-                else
-                    ++it;
-            }
-
-            mGuestWaiters.push_back(GuestCall(u32ClientId, callHandle, GET_NOTIFICATION,
-                                              cParms, paParms, rc));
-            rc = VINF_HGCM_ASYNC_EXECUTE;
-        }
         /*
-         * Otherwise reply at once with the enqueued notification we found.
+         * If no timestamp was supplied or no notification was found in the queue
+         * of old notifications, enqueue the request in the waiting queue.
          */
-        else
+        Property prop;
+        if (RT_SUCCESS(rc) && u64Timestamp != 0)
+            rc = getOldNotification(pszPatterns, u64Timestamp, &prop);
+        if (RT_SUCCESS(rc))
         {
-            int rc2 = getNotificationWriteOut(cParms, paParms, prop);
-            if (RT_FAILURE(rc2))
-                rc = rc2;
+            if (prop.isNull())
+            {
+                /*
+                 * Check if the client already had the same request.
+                 * Complete the old request with an error in this case.
+                 * Protection against clients, which cancel and resubmits requests.
+                 */
+                CallList::iterator it = mGuestWaiters.begin();
+                while (it != mGuestWaiters.end())
+                {
+                    const char *pszPatternsExisting;
+                    uint32_t cchPatternsExisting;
+                    int rc3 = it->mParms[0].getString(&pszPatternsExisting, &cchPatternsExisting);
+
+                    if (   RT_SUCCESS(rc3)
+                        && u32ClientId == it->u32ClientId
+                        && RTStrCmp(pszPatterns, pszPatternsExisting) == 0)
+                    {
+                        /* Complete the old request. */
+                        mpHelpers->pfnCallComplete(it->mHandle, VERR_INTERRUPTED);
+                        it = mGuestWaiters.erase(it);
+                    }
+                    else
+                        ++it;
+                }
+
+                mGuestWaiters.push_back(GuestCall(u32ClientId, callHandle, GET_NOTIFICATION,
+                                                  cParms, paParms, rc));
+                rc = VINF_HGCM_ASYNC_EXECUTE;
+            }
+            /*
+             * Otherwise reply at once with the enqueued notification we found.
+             */
+            else
+            {
+                int rc2 = getNotificationWriteOut(cParms, paParms, prop);
+                if (RT_FAILURE(rc2))
+                    rc = rc2;
+            }
         }
     }
 
@@ -1185,7 +1234,7 @@ int Service::doNotifications(const char *pszProperty, uint64_t u64Timestamp)
         if (mGuestNotifications.size() > MAX_GUEST_NOTIFICATIONS)
             mGuestNotifications.pop_front();
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         rc = VERR_NO_MEMORY;
     }
@@ -1279,9 +1328,9 @@ int Service::notifyHost(const char *pszName, const char *pszValue,
         pu8 += cbFlags;
         *pu8++ = 0;
 
-        rc = RTReqCallEx(mpReqQNotifyHost, NULL, 0, RTREQFLAGS_VOID | RTREQFLAGS_NO_WAIT,
-                         (PFNRT)notifyHostAsyncWorker, 3,
-                         mpfnHostCallback, mpvHostData, pHostCallbackData);
+        rc = RTReqQueueCallEx(mhReqQNotifyHost, NULL, 0, RTREQFLAGS_VOID | RTREQFLAGS_NO_WAIT,
+                              (PFNRT)notifyHostAsyncWorker, 3,
+                              mpfnHostCallback, mpvHostData, pHostCallbackData);
         if (RT_FAILURE(rc))
         {
             RTMemFree(pHostCallbackData);
@@ -1309,7 +1358,7 @@ int Service::notifyHost(const char *pszName, const char *pszValue,
 
 /**
  * Handle an HGCM service call.
- * @copydoc VBOXHGCMSVCFNTABLE::pfnCall
+ * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnCall}
  * @note    All functions which do not involve an unreasonable delay will be
  *          handled synchronously.  If needed, we will add a request handler
  *          thread in future for those which do.
@@ -1368,7 +1417,7 @@ void Service::call (VBOXHGCMCALLHANDLE callHandle, uint32_t u32ClientID,
                 rc = VERR_NOT_IMPLEMENTED;
         }
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         rc = VERR_NO_MEMORY;
     }
@@ -1379,10 +1428,56 @@ void Service::call (VBOXHGCMCALLHANDLE callHandle, uint32_t u32ClientID,
     }
 }
 
+/**
+ * Enumeration data shared between dbgInfoCallback and Service::dbgInfoShow.
+ */
+typedef struct ENUMDBGINFO
+{
+    PCDBGFINFOHLP pHlp;
+} ENUMDBGINFO;
+
+static DECLCALLBACK(int) dbgInfoCallback(PRTSTRSPACECORE pStr, void *pvUser)
+{
+    Property *pProp = (Property *)pStr;
+    PCDBGFINFOHLP pHlp = ((ENUMDBGINFO*)pvUser)->pHlp;
+
+    char szFlags[MAX_FLAGS_LEN];
+    int rc = writeFlags(pProp->mFlags, szFlags);
+    if (RT_FAILURE(rc))
+        RTStrPrintf(szFlags, sizeof(szFlags), "???");
+
+    pHlp->pfnPrintf(pHlp, "%s: '%s', %RU64",
+                    pProp->mName.c_str(), pProp->mValue.c_str(), pProp->mTimestamp);
+    if (strlen(szFlags))
+        pHlp->pfnPrintf(pHlp, " (%s)", szFlags);
+    pHlp->pfnPrintf(pHlp, "\n");
+    return 0;
+}
+
+void Service::dbgInfoShow(PCDBGFINFOHLP pHlp)
+{
+    ENUMDBGINFO EnumData = { pHlp };
+    RTStrSpaceEnumerate(&mhProperties, dbgInfoCallback, &EnumData);
+}
+
+/**
+ * Handler for debug info.
+ *
+ * @param   pvUser      user pointer.
+ * @param   pHlp        The info helper functions.
+ * @param   pszArgs     Arguments, ignored.
+ */
+void Service::dbgInfo(void *pvUser, PCDBGFINFOHLP pHlp, const char *pszArgs)
+{
+    RT_NOREF1(pszArgs);
+    SELF *pSelf = reinterpret_cast<SELF *>(pvUser);
+    pSelf->dbgInfoShow(pHlp);
+}
+
 
 /**
  * Service call handler for the host.
- * @copydoc VBOXHGCMSVCFNTABLE::pfnHostCall
+ * @interface_method_impl{VBOXHGCMSVCFNTABLE,pfnHostCall}
  * @thread  hgcm
  */
 int Service::hostCall (uint32_t eFunction, uint32_t cParms, VBOXHGCMSVCPARM paParms[])
@@ -1446,12 +1541,19 @@ int Service::hostCall (uint32_t eFunction, uint32_t cParms, VBOXHGCMSVCPARM paPa
                     rc = VERR_INVALID_PARAMETER;
                 break;
 
+            case GET_DBGF_INFO_FN:
+                if (cParms != 2)
+                    return VERR_INVALID_PARAMETER;
+                paParms[0].u.pointer.addr = (void*)(uintptr_t)dbgInfo;
+                paParms[1].u.pointer.addr = (void*)this;
+                break;
+
             default:
                 rc = VERR_NOT_SUPPORTED;
                 break;
         }
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         rc = VERR_NO_MEMORY;
     }
@@ -1462,8 +1564,9 @@ int Service::hostCall (uint32_t eFunction, uint32_t cParms, VBOXHGCMSVCPARM paPa
 
 #ifdef ASYNC_HOST_NOTIFY
 /* static */
-DECLCALLBACK(int) Service::threadNotifyHost(RTTHREAD self, void *pvUser)
+DECLCALLBACK(int) Service::threadNotifyHost(RTTHREAD hThreadSelf, void *pvUser)
 {
+    RT_NOREF1(hThreadSelf);
     Service *pThis = (Service *)pvUser;
     int rc = VINF_SUCCESS;
 
@@ -1471,7 +1574,7 @@ DECLCALLBACK(int) Service::threadNotifyHost(RTTHREAD self, void *pvUser)
 
     for (;;)
     {
-        rc = RTReqProcess(pThis->mpReqQNotifyHost, RT_INDEFINITE_WAIT);
+        rc = RTReqQueueProcess(pThis->mhReqQNotifyHost, RT_INDEFINITE_WAIT);
 
         AssertMsg(rc == VWRN_STATE_CHANGED,
                   ("Left RTReqProcess and error code is not VWRN_STATE_CHANGED rc=%Rrc\n",
@@ -1488,14 +1591,14 @@ DECLCALLBACK(int) Service::threadNotifyHost(RTTHREAD self, void *pvUser)
 
 static DECLCALLBACK(int) wakeupNotifyHost(void)
 {
-    /* Returning a VWRN_* will cause RTReqProcess return. */
+    /* Returning a VWRN_* will cause RTReqQueueProcess return. */
     return VWRN_STATE_CHANGED;
 }
 
 int Service::initialize()
 {
     /* The host notification thread and queue. */
-    int rc = RTReqCreateQueue(&mpReqQNotifyHost);
+    int rc = RTReqQueueCreate(&mhReqQNotifyHost);
     if (RT_SUCCESS(rc))
     {
         rc = RTThreadCreate(&mhThreadNotifyHost,
@@ -1503,38 +1606,53 @@ int Service::initialize()
                             this,
                             0 /* default stack size */,
                             RTTHREADTYPE_DEFAULT,
-                            0, /* no flags. */
+                            RTTHREADFLAGS_WAITABLE,
                             "GSTPROPNTFY");
     }
 
     if (RT_FAILURE(rc))
     {
-        if (mpReqQNotifyHost != NULL)
+        if (mhReqQNotifyHost != NIL_RTREQQUEUE)
         {
-            RTReqDestroyQueue(mpReqQNotifyHost);
-            mpReqQNotifyHost = NULL;
+            RTReqQueueDestroy(mhReqQNotifyHost);
+            mhReqQNotifyHost = NIL_RTREQQUEUE;
         }
     }
 
     return rc;
 }
+
+/**
+ * @callback_method_impl{FNRTSTRSPACECALLBACK, Destroys Property.}
+ */
+static DECLCALLBACK(int) destroyProperty(PRTSTRSPACECORE pStr, void *pvUser)
+{
+    RT_NOREF(pvUser);
+    Property *pProp = RT_FROM_CPP_MEMBER(pStr, struct Property, mStrCore); /* clang objects to offsetof on non-POD.*/
+    delete pProp;
+    return 0;
+}
+
 #endif
 
 int Service::uninit()
 {
 #ifdef ASYNC_HOST_NOTIFY
-    if (mpReqQNotifyHost != NULL)
+    if (mhReqQNotifyHost != NIL_RTREQQUEUE)
     {
         /* Stop the thread */
         PRTREQ pReq;
-        int rc = RTReqCall(mpReqQNotifyHost, &pReq, 10000, (PFNRT)wakeupNotifyHost, 0);
+        int rc = RTReqQueueCall(mhReqQNotifyHost, &pReq, 10000, (PFNRT)wakeupNotifyHost, 0);
         if (RT_SUCCESS(rc))
-            RTReqFree(pReq);
-
-        rc = RTReqDestroyQueue(mpReqQNotifyHost);
+            RTReqRelease(pReq);
+        rc = RTThreadWait(mhThreadNotifyHost, 10000, NULL);
         AssertRC(rc);
-        mpReqQNotifyHost = NULL;
+        rc = RTReqQueueDestroy(mhReqQNotifyHost);
+        AssertRC(rc);
+        mhReqQNotifyHost = NIL_RTREQQUEUE;
         mhThreadNotifyHost = NIL_RTTHREAD;
+        RTStrSpaceDestroy(&mhProperties, destroyProperty, NULL);
+        mhProperties = NULL;
     }
 #endif
 
