@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,8 +19,10 @@
 #endif
 
 #include "mappings.h"
+#include "vbsfpath.h"
 #include <iprt/alloc.h>
 #include <iprt/assert.h>
+#include <iprt/path.h>
 #include <iprt/string.h>
 
 #ifdef UNITTEST
@@ -78,7 +80,14 @@ int vbsfMappingLoaded(const PMAPPING pLoadedMapping, SHFLROOT root)
         }
     }
 
-    return VERR_INVALID_PARAMETER;
+    /* No corresponding mapping on the host but the guest still uses it.
+     * Add a 'placeholder' mapping.
+     */
+    LogRel2(("SharedFolders: mapping a placeholder for '%ls' -> '%s'\n",
+              pLoadedMapping->pMapName->String.ucs2, pLoadedMapping->pszFolderName));
+    return vbsfMappingsAdd(pLoadedMapping->pszFolderName, pLoadedMapping->pMapName,
+                           pLoadedMapping->fWritable, pLoadedMapping->fAutoMount,
+                           pLoadedMapping->fSymlinksCreate, /* fMissing = */ true, /* fPlaceholder = */ true);
 }
 
 MAPPING *vbsfMappingGetByRoot(SHFLROOT root)
@@ -190,12 +199,12 @@ void testMappingsAdd(RTTEST hTest)
 /*
  * We are always executed from one specific HGCM thread. So thread safe.
  */
-int vbsfMappingsAdd(PSHFLSTRING pFolderName, PSHFLSTRING pMapName,
-                    bool fWritable, bool fAutoMount, bool fSymlinksCreate, bool fMissing)
+int vbsfMappingsAdd(const char *pszFolderName, PSHFLSTRING pMapName,
+                    bool fWritable, bool fAutoMount, bool fSymlinksCreate, bool fMissing, bool fPlaceholder)
 {
     unsigned i;
 
-    Assert(pFolderName && pMapName);
+    Assert(pszFolderName && pMapName);
 
     Log(("vbsfMappingsAdd %ls\n", pMapName->String.ucs2));
 
@@ -216,9 +225,18 @@ int vbsfMappingsAdd(PSHFLSTRING pFolderName, PSHFLSTRING pMapName,
     {
         if (FolderMapping[i].fValid == false)
         {
-            int rc = RTUtf16ToUtf8(pFolderName->String.ucs2, &FolderMapping[i].pszFolderName);
+            /* Make sure the folder name is an absolute path, otherwise we're
+               likely to get into trouble with buffer sizes in vbsfPathGuestToHost. */
+            char szAbsFolderName[RTPATH_MAX];
+            int rc = vbsfPathAbs(NULL, pszFolderName, szAbsFolderName, sizeof(szAbsFolderName));
             AssertRCReturn(rc, rc);
 
+            FolderMapping[i].pszFolderName = RTStrDup(szAbsFolderName);
+            if (!FolderMapping[i].pszFolderName)
+            {
+                return VERR_NO_MEMORY;
+            }
+
             FolderMapping[i].pMapName = (PSHFLSTRING)RTMemAlloc(ShflStringSizeOfBuffer(pMapName));
             if (!FolderMapping[i].pMapName)
             {
@@ -237,19 +255,13 @@ int vbsfMappingsAdd(PSHFLSTRING pFolderName, PSHFLSTRING pMapName,
             FolderMapping[i].fAutoMount      = fAutoMount;
             FolderMapping[i].fSymlinksCreate = fSymlinksCreate;
             FolderMapping[i].fMissing        = fMissing;
+            FolderMapping[i].fPlaceholder    = fPlaceholder;
 
             /* Check if the host file system is case sensitive */
             RTFSPROPERTIES prop;
-            char *pszAsciiRoot;
-
-            rc = RTStrUtf8ToCurrentCP(&pszAsciiRoot, FolderMapping[i].pszFolderName);
-            if (RT_SUCCESS(rc))
-            {
-                rc = RTFsQueryProperties(pszAsciiRoot, &prop);
-                AssertRC(rc);
-                RTStrFree(pszAsciiRoot);
-            }
-
+            prop.fCaseSensitive = false; /* Shut up MSC. */
+            rc = RTFsQueryProperties(FolderMapping[i].pszFolderName, &prop);
+            AssertRC(rc);
             FolderMapping[i].fHostCaseSensitive = RT_SUCCESS(rc) ? prop.fCaseSensitive : false;
             vbsfRootHandleAdd(i);
             break;
@@ -257,11 +269,11 @@ int vbsfMappingsAdd(PSHFLSTRING pFolderName, PSHFLSTRING pMapName,
     }
     if (i == SHFL_MAX_MAPPINGS)
     {
-        AssertMsgFailed(("vbsfMappingsAdd: no more room to add mapping %ls to %ls!!\n", pFolderName->String.ucs2, pMapName->String.ucs2));
+        AssertLogRelMsgFailed(("vbsfMappingsAdd: no more room to add mapping %s to %ls!!\n", pszFolderName, pMapName->String.ucs2));
         return VERR_TOO_MUCH_DATA;
     }
 
-    Log(("vbsfMappingsAdd: added mapping %ls to %ls\n", pFolderName->String.ucs2, pMapName->String.ucs2));
+    Log(("vbsfMappingsAdd: added mapping %s to %ls\n", pszFolderName, pMapName->String.ucs2));
     return VINF_SUCCESS;
 }
 
@@ -290,28 +302,31 @@ int vbsfMappingsRemove(PSHFLSTRING pMapName)
             {
                 if (FolderMapping[i].cMappings != 0)
                 {
-                    Log(("vbsfMappingsRemove: trying to remove active share %ls\n", pMapName->String.ucs2));
-                    return VERR_PERMISSION_DENIED;
+                    LogRel2(("SharedFolders: removing '%ls' -> '%s', which is still used by the guest\n",
+                             pMapName->String.ucs2, FolderMapping[i].pszFolderName));
+                    FolderMapping[i].fMissing = true;
+                    FolderMapping[i].fPlaceholder = true;
+                    return VINF_PERMISSION_DENIED;
                 }
 
+                /* pMapName can be the same as FolderMapping[i].pMapName,
+                 * log it before deallocating the memory.
+                 */
+                Log(("vbsfMappingsRemove: mapping %ls removed\n", pMapName->String.ucs2));
+
                 RTStrFree(FolderMapping[i].pszFolderName);
                 RTMemFree(FolderMapping[i].pMapName);
                 FolderMapping[i].pszFolderName = NULL;
                 FolderMapping[i].pMapName      = NULL;
                 FolderMapping[i].fValid        = false;
                 vbsfRootHandleRemove(i);
-                break;
+                return VINF_SUCCESS;
             }
         }
     }
 
-    if (i == SHFL_MAX_MAPPINGS)
-    {
-        AssertMsgFailed(("vbsfMappingsRemove: mapping %ls not found!!!!\n", pMapName->String.ucs2));
-        return VERR_FILE_NOT_FOUND;
-    }
-    Log(("vbsfMappingsRemove: mapping %ls removed\n", pMapName->String.ucs2));
-    return VINF_SUCCESS;
+    AssertMsgFailed(("vbsfMappingsRemove: mapping %ls not found!!!!\n", pMapName->String.ucs2));
+    return VERR_FILE_NOT_FOUND;
 }
 
 const char* vbsfMappingsQueryHostRoot(SHFLROOT root)
@@ -323,6 +338,20 @@ const char* vbsfMappingsQueryHostRoot(SHFLROOT root)
     return pFolderMapping->pszFolderName;
 }
 
+int vbsfMappingsQueryHostRootEx(SHFLROOT hRoot, const char **ppszRoot, uint32_t *pcbRootLen)
+{
+    MAPPING *pFolderMapping = vbsfMappingGetByRoot(hRoot);
+    AssertReturn(pFolderMapping, VERR_INVALID_PARAMETER);
+    if (pFolderMapping->fMissing)
+        return VERR_NOT_FOUND;
+    if (   pFolderMapping->pszFolderName == NULL
+        || pFolderMapping->pszFolderName[0] == 0)
+        return VERR_NOT_FOUND;
+    *ppszRoot = pFolderMapping->pszFolderName;
+    *pcbRootLen = (uint32_t)strlen(pFolderMapping->pszFolderName);
+    return VINF_SUCCESS;
+}
+
 bool vbsfIsGuestMappingCaseSensitive(SHFLROOT root)
 {
     MAPPING *pFolderMapping = vbsfMappingGetByRoot(root);
@@ -453,8 +482,11 @@ int vbsfMappingsQueryName(PSHFLCLIENTDATA pClient, SHFLROOT root, SHFLSTRING *pS
     return rc;
 }
 
+/** Queries fWritable flag for the given root. Returns error if the root is not accessible.
+ */
 int vbsfMappingsQueryWritable(PSHFLCLIENTDATA pClient, SHFLROOT root, bool *fWritable)
 {
+    RT_NOREF1(pClient);
     int rc = VINF_SUCCESS;
 
     LogFlow(("vbsfMappingsQueryWritable: pClient = %p, root = %d\n", pClient, root));
@@ -475,6 +507,7 @@ int vbsfMappingsQueryWritable(PSHFLCLIENTDATA pClient, SHFLROOT root, bool *fWri
 
 int vbsfMappingsQueryAutoMount(PSHFLCLIENTDATA pClient, SHFLROOT root, bool *fAutoMount)
 {
+    RT_NOREF1(pClient);
     int rc = VINF_SUCCESS;
 
     LogFlow(("vbsfMappingsQueryAutoMount: pClient = %p, root = %d\n", pClient, root));
@@ -494,6 +527,7 @@ int vbsfMappingsQueryAutoMount(PSHFLCLIENTDATA pClient, SHFLROOT root, bool *fAu
 
 int vbsfMappingsQuerySymlinksCreate(PSHFLCLIENTDATA pClient, SHFLROOT root, bool *fSymlinksCreate)
 {
+    RT_NOREF1(pClient);
     int rc = VINF_SUCCESS;
 
     LogFlow(("vbsfMappingsQueryAutoMount: pClient = %p, root = %d\n", pClient, root));
@@ -534,7 +568,7 @@ void testMapFolder(RTTEST hTest)
 }
 #endif
 int vbsfMapFolder(PSHFLCLIENTDATA pClient, PSHFLSTRING pszMapName,
-                  RTUTF16 pwszDelimiter, bool fCaseSensitive, SHFLROOT *pRoot)
+                  RTUTF16 wcDelimiter, bool fCaseSensitive, SHFLROOT *pRoot)
 {
     MAPPING *pFolderMapping = NULL;
 
@@ -547,13 +581,18 @@ int vbsfMapFolder(PSHFLCLIENTDATA pClient, PSHFLSTRING pszMapName,
         Log(("vbsfMapFolder %ls\n", pszMapName->String.ucs2));
     }
 
+    AssertMsgReturn(wcDelimiter == '/' || wcDelimiter == '\\',
+                    ("Invalid path delimiter: %#x\n", wcDelimiter),
+                    VERR_INVALID_PARAMETER);
     if (pClient->PathDelimiter == 0)
     {
-        pClient->PathDelimiter = pwszDelimiter;
+        pClient->PathDelimiter = wcDelimiter;
     }
     else
     {
-        Assert(pwszDelimiter == pClient->PathDelimiter);
+        AssertMsgReturn(wcDelimiter == pClient->PathDelimiter,
+                        ("wcDelimiter=%#x PathDelimiter=%#x", wcDelimiter, pClient->PathDelimiter),
+                        VERR_INVALID_PARAMETER);
     }
 
     if (BIT_FLAG(pClient->fu32Flags, SHFL_CF_UTF8))
@@ -600,6 +639,7 @@ void testUnmapFolder(RTTEST hTest)
 #endif
 int vbsfUnmapFolder(PSHFLCLIENTDATA pClient, SHFLROOT root)
 {
+    RT_NOREF1(pClient);
     int rc = VINF_SUCCESS;
 
     MAPPING *pFolderMapping = vbsfMappingGetByRoot(root);
@@ -613,6 +653,16 @@ int vbsfUnmapFolder(PSHFLCLIENTDATA pClient, SHFLROOT root)
     if (pFolderMapping->cMappings > 0)
         pFolderMapping->cMappings--;
 
+    if (   pFolderMapping->cMappings == 0
+        && pFolderMapping->fPlaceholder)
+    {
+        /* Automatically remove, it is not used by the guest anymore. */
+        Assert(pFolderMapping->fMissing);
+        LogRel2(("SharedFolders: unmapping placeholder '%ls' -> '%s'\n",
+                pFolderMapping->pMapName->String.ucs2, pFolderMapping->pszFolderName));
+        vbsfMappingsRemove(pFolderMapping->pMapName);
+    }
+
     Log(("vbsfUnmapFolder\n"));
     return rc;
 }
