@@ -3,6 +3,7 @@
 #
 # See the file LICENSE.txt for information on redistributing this software.
 
+from __future__ import print_function
 import sys
 
 import apiutil
@@ -10,14 +11,14 @@ import apiutil
 
 apiutil.CopyrightC()
 
-print """#include "cr_spu.h"
+print("""#include "cr_spu.h"
 #include "chromium.h"
 #include "cr_error.h"
 #include "cr_mem.h"
 #include "cr_net.h"
 #include "server_dispatch.h"
 #include "server.h"
-"""
+""")
 
 from get_sizes import *;
 
@@ -28,11 +29,11 @@ types = [ 'GLint', 'GLfloat', 'GLdouble', 'GLboolean' ]
 for index in range(len(funcs)):
     func_name = funcs[index]
     params = apiutil.Parameters(func_name)
-    print 'void SERVER_DISPATCH_APIENTRY crServerDispatch%s( %s )' % ( func_name, apiutil.MakeDeclarationString(params))
-    print '{'
-    print '\t%s *get_values;' % types[index]
-    print '\tint tablesize;'
-    print """
+    print('void SERVER_DISPATCH_APIENTRY crServerDispatch%s(%s)' % ( func_name, apiutil.MakeDeclarationString(params)))
+    print('{')
+    print('\t%s *get_values;' % types[index])
+    print('\tint tablesize;')
+    print("""
     #ifdef CR_ARB_texture_compression
     if (GL_COMPRESSED_TEXTURE_FORMATS_ARB == pname)
     {
@@ -45,12 +46,16 @@ for index in range(len(funcs)):
     {
         tablesize = __numValues( pname ) * sizeof(%s);
     }
-    """ % (types[index], types[index])
-    print '\t(void) params;'
-    print '\tget_values = (%s *) crAlloc( tablesize );' % types[index]
-    print '\tif (tablesize>0)'
-    print '\tcr_server.head_spu->dispatch_table.%s( pname, get_values );' % func_name
-    print """
+    """ % (types[index], types[index]))
+    print('\t(void) params;')
+    print('\tif (tablesize == 0)')
+    print('\t{')
+    print('\t\tcrServerReturnValue(NULL, 0);')
+    print('\t\treturn;')
+    print('\t}')
+    print('\tget_values = (%s *) crAlloc( tablesize );' % types[index])
+    print('\tcr_server.head_spu->dispatch_table.%s( pname, get_values );' % func_name)
+    print("""
     if (GL_TEXTURE_BINDING_1D==pname
         || GL_TEXTURE_BINDING_2D==pname
         || GL_TEXTURE_BINDING_3D==pname
@@ -60,45 +65,51 @@ for index in range(len(funcs)):
         GLuint texid;
         CRASSERT(tablesize/sizeof(%s)==1);
         texid = (GLuint) *get_values;
-        *get_values = (%s) crStateTextureHWIDtoID(texid);
+        *get_values = (%s) crStateTextureHWIDtoID(&cr_server.StateTracker, texid);
     }
     else if (GL_CURRENT_PROGRAM==pname)
     {
         GLuint programid;
         CRASSERT(tablesize/sizeof(%s)==1);
         programid = (GLuint) *get_values;
-        *get_values = (%s) crStateGLSLProgramHWIDtoID(programid);
+        *get_values = (%s) crStateGLSLProgramHWIDtoID(&cr_server.StateTracker, programid);
     }
     else if (GL_FRAMEBUFFER_BINDING_EXT==pname
              ||GL_READ_FRAMEBUFFER_BINDING==pname)
     {
         GLuint fboid;
         CRASSERT(tablesize/sizeof(%s)==1);
-        fboid = crStateFBOHWIDtoID((GLuint) *get_values);
-        if (cr_server.curClient->currentMural->bUseFBO
-            && crServerIsRedirectedToFBO()
-            && fboid==cr_server.curClient->currentMural->idFBO)
+        fboid = (GLuint) *get_values;
+        if (crServerIsRedirectedToFBO()
+            && (fboid==cr_server.curClient->currentMural->aidFBOs[0]
+            || fboid==cr_server.curClient->currentMural->aidFBOs[1]))
         {
             fboid = 0;
         }
+        else
+        {
+        	fboid = crStateFBOHWIDtoID(&cr_server.StateTracker, fboid);
+        }
         *get_values = (%s) fboid;
     }
     else if (GL_READ_BUFFER==pname)
     {
-        if (cr_server.curClient->currentMural->bUseFBO && crServerIsRedirectedToFBO()
-            && cr_server.curClient->currentMural->idFBO
-            && !crStateGetCurrent()->framebufferobject.readFB)
+        if (crServerIsRedirectedToFBO()
+            && CR_SERVER_FBO_FOR_IDX(cr_server.curClient->currentMural, cr_server.curClient->currentMural->iCurReadBuffer)
+            && !crStateGetCurrent(&cr_server.StateTracker)->framebufferobject.readFB)
         {
-            *get_values = (%s) crStateGetCurrent()->buffer.readBuffer;
+            *get_values = (%s) crStateGetCurrent(&cr_server.StateTracker)->buffer.readBuffer;
+            Assert(crStateGetCurrent(&cr_server.StateTracker)->buffer.readBuffer == GL_BACK || crStateGetCurrent(&cr_server.StateTracker)->buffer.readBuffer == GL_FRONT);
         }
     }
     else if (GL_DRAW_BUFFER==pname)
     {
-        if (cr_server.curClient->currentMural->bUseFBO && crServerIsRedirectedToFBO()
-            && cr_server.curClient->currentMural->idFBO
-            && !crStateGetCurrent()->framebufferobject.drawFB)
+        if (crServerIsRedirectedToFBO()
+            && CR_SERVER_FBO_FOR_IDX(cr_server.curClient->currentMural, cr_server.curClient->currentMural->iCurDrawBuffer)
+            && !crStateGetCurrent(&cr_server.StateTracker)->framebufferobject.drawFB)
         {
-            *get_values = (%s) crStateGetCurrent()->buffer.drawBuffer;
+            *get_values = (%s) crStateGetCurrent(&cr_server.StateTracker)->buffer.drawBuffer;
+            Assert(crStateGetCurrent(&cr_server.StateTracker)->buffer.drawBuffer == GL_BACK || crStateGetCurrent(&cr_server.StateTracker)->buffer.drawBuffer == GL_FRONT);
         }
     }
     else if (GL_RENDERBUFFER_BINDING_EXT==pname)
@@ -106,7 +117,7 @@ for index in range(len(funcs)):
         GLuint rbid;
         CRASSERT(tablesize/sizeof(%s)==1);
         rbid = (GLuint) *get_values;
-        *get_values = (%s) crStateRBOHWIDtoID(rbid);
+        *get_values = (%s) crStateRBOHWIDtoID(&cr_server.StateTracker, rbid);
     }
     else if (GL_ARRAY_BUFFER_BINDING_ARB==pname
              || GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB==pname
@@ -123,7 +134,7 @@ for index in range(len(funcs)):
         GLuint bufid;
         CRASSERT(tablesize/sizeof(%s)==1);
         bufid = (GLuint) *get_values;
-        *get_values = (%s) crStateBufferHWIDtoID(bufid);
+        *get_values = (%s) crStateBufferHWIDtoID(&cr_server.StateTracker, bufid);
     }
     else if (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS==pname)
     {
@@ -132,7 +143,14 @@ for index in range(len(funcs)):
     		*get_values = (%s)CR_MAX_TEXTURE_UNITS;
     	} 
     }
-    """ % (types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index])
-    print '\tcrServerReturnValue( get_values, tablesize );'
-    print '\tcrFree(get_values);'
-    print '}\n'
+    else if (GL_MAX_VERTEX_ATTRIBS_ARB==pname)
+    {
+        if (CR_MAX_VERTEX_ATTRIBS < (GLuint)*get_values)
+        {
+            *get_values = (%s)CR_MAX_VERTEX_ATTRIBS;
+        } 
+    }
+    """ % (types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index], types[index]))
+    print('\tcrServerReturnValue( get_values, tablesize );')
+    print('\tcrFree(get_values);')
+    print('}\n')
