@@ -4,17 +4,17 @@
  * See the file LICENSE.txt for information on redistributing this software.
  */
 
-#include "cr_environment.h"
 #include "cr_string.h"
 #include "cr_error.h"
 #include "cr_mem.h"
 #include "cr_spu.h"
-#include "cr_environment.h"
 #include "renderspu.h"
 #include "cr_extstring.h"
 
 #include <iprt/asm.h>
 
+#include <stdio.h> /*sprintf*/
+
 uint32_t renderspuContextRelease(ContextInfo *context);
 uint32_t renderspuContextRetain(ContextInfo *context);
 
@@ -25,26 +25,10 @@ DoSync(void)
 
     out.header.type = CR_MESSAGE_OOB;
 
-    if (render_spu.is_swap_master)
-    {
-        int a;
+    crNetSend( render_spu.swap_conns[0], NULL, &out, sizeof(CRMessage));
 
-        for (a = 0; a < render_spu.num_swap_clients; a++)
-        {
-            crNetGetMessage( render_spu.swap_conns[a], &in );
-            crNetFree( render_spu.swap_conns[a], in);
-        }
-
-        for (a = 0; a < render_spu.num_swap_clients; a++)
-            crNetSend( render_spu.swap_conns[a], NULL, &out, sizeof(CRMessage));
-    }
-    else
-    {
-        crNetSend( render_spu.swap_conns[0], NULL, &out, sizeof(CRMessage));
-
-        crNetGetMessage( render_spu.swap_conns[0], &in );
-        crNetFree( render_spu.swap_conns[0], in);
-    }
+    crNetGetMessage( render_spu.swap_conns[0], &in );
+    crNetFree( render_spu.swap_conns[0], in);
 }
 
 
@@ -258,7 +242,7 @@ uint32_t renderspuContextMarkDeletedAndRelease( ContextInfo *context )
     return renderspuContextRelease( context );
 }
 
-ContextInfo * renderspuDefaultSharedContextAcquire()
+ContextInfo * renderspuDefaultSharedContextAcquire(void)
 {
     ContextInfo * pCtx = render_spu.defaultSharedContext;
     if (!pCtx)
@@ -315,22 +299,108 @@ renderspuDestroyContext( GLint ctx )
     renderspuContextMarkDeletedAndRelease(context);
 }
 
+WindowInfo* renderspuWinCreate(GLint visBits, GLint id)
+{
+    WindowInfo* window = (WindowInfo *)crAlloc(sizeof (*window));
+    if (!window)
+    {
+        crWarning("crAlloc failed");
+        return NULL;
+    }
+
+    if (!renderspuWinInit(window, NULL, visBits, id))
+    {
+        crWarning("renderspuWinInit failed");
+        crFree(window);
+        return NULL;
+    }
+
+    return window;
+}
+
+void renderspuWinTermOnShutdown(WindowInfo *window)
+{
+    renderspuVBoxCompositorSet(window, NULL);
+    renderspuVBoxPresentBlitterCleanup(window);
+    window->BltInfo.Base.id = -1;
+    renderspu_SystemDestroyWindow( window );
+}
+
+static void renderspuCheckCurrentCtxWindowCB(unsigned long key, void *data1, void *data2)
+{
+    ContextInfo *pCtx = (ContextInfo *) data1;
+    WindowInfo *pWindow = data2;
+    (void) key;
+
+    if (pCtx->currentWindow==pWindow)
+    {
+        WindowInfo* pDummy = renderspuGetDummyWindow(pCtx->BltInfo.Base.visualBits);
+        if (pDummy)
+        {
+            renderspuPerformMakeCurrent(pDummy, 0, pCtx);
+        }
+        else
+        {
+            crWarning("failed to get dummy window");
+            renderspuMakeCurrent(CR_RENDER_DEFAULT_WINDOW_ID, 0, pCtx->BltInfo.Base.id);
+        }
+    }
+}
+
+void renderspuWinTerm( WindowInfo *window )
+{
+    if (!renderspuWinIsTermed(window))
+    {
+
+    GET_CONTEXT(pOldCtx);
+    WindowInfo * pOldWindow = pOldCtx ? pOldCtx->currentWindow : NULL;
+    CRASSERT(!pOldCtx == !pOldWindow);
+    /* ensure no concurrent draws can take place */
+    renderspuWinTermOnShutdown(window);
+    /* check if this window is bound to some ctx. Note: window pointer is already freed here */
+    crHashtableWalk(render_spu.contextTable, renderspuCheckCurrentCtxWindowCB, window);
+    /* restore current context */
+    {
+        GET_CONTEXT(pNewCtx);
+        WindowInfo * pNewWindow = pNewCtx ? pNewCtx->currentWindow : NULL;
+        CRASSERT(!pNewCtx == !pNewWindow);
+
+        if (pOldWindow == window)
+            renderspuMakeCurrent(CR_RENDER_DEFAULT_WINDOW_ID, 0, CR_RENDER_DEFAULT_CONTEXT_ID);
+        else if (pNewCtx != pOldCtx || pOldWindow != pNewWindow)
+        {
+            if (pOldCtx)
+                renderspuPerformMakeCurrent(pOldWindow, 0, pOldCtx);
+            else
+                renderspuMakeCurrent(CR_RENDER_DEFAULT_WINDOW_ID, 0, CR_RENDER_DEFAULT_CONTEXT_ID);
+        }
+    }
+
+    }
+}
+
+void renderspuWinCleanup(WindowInfo *window)
+{
+    renderspuWinTerm( window );
+    crFree(window->title);
+    RTCritSectDelete(&window->CompositorLock);
+}
+
+void renderspuWinDestroy(WindowInfo *window)
+{
+    renderspuWinCleanup(window);
+    crFree(window);
+}
+
 WindowInfo* renderspuGetDummyWindow(GLint visBits)
 {
     WindowInfo *window = (WindowInfo *) crHashtableSearch(render_spu.dummyWindowTable, visBits);
     if (!window)
     {
-        window = (WindowInfo *)crAlloc(sizeof (*window));
+        window = renderspuWinCreate(visBits, -1);
         if (!window)
         {
-            crWarning("crAlloc failed");
-            return NULL;
-        }
-
-        if (!renderspuWindowInit(window, NULL, visBits, -1))
-        {
-            crWarning("renderspuWindowInit failed");
-            crFree(window);
+            WARN(("renderspuWinCreate failed"));
             return NULL;
         }
 
@@ -340,29 +410,68 @@ WindowInfo* renderspuGetDummyWindow(GLint visBits)
     return window;
 }
 
+/* Check that OpenGL extensions listed in pszRequiredExts string also exist in the pszAvailableExts string. */
+static void renderCompareGLExtensions(const char *pszAvailableExts, const char *pszRequiredExts)
+{
+    unsigned char fPrintHeader = 1;
+    const char *pszExt = pszRequiredExts;
+
+    for (;;)
+    {
+        const char *pszSrc = pszAvailableExts;
+        size_t offExtEnd;
+
+        while (*pszExt == ' ')
+            ++pszExt;
+
+        if (!*pszExt)
+            break;
+
+        offExtEnd = RTStrOffCharOrTerm(pszExt, ' ');
+
+        for (;;)
+        {
+            size_t offSrcEnd;
+
+            while (*pszSrc == ' ')
+                ++pszSrc;
+
+            if (!*pszSrc)
+                break;
+
+            offSrcEnd = RTStrOffCharOrTerm(pszSrc, ' ');
+
+            if (   offSrcEnd == offExtEnd
+                && memcmp(pszSrc, pszExt, offSrcEnd) == 0)
+                    break;
+
+            pszSrc += offSrcEnd;
+        }
+
+        if (!*pszSrc)
+        {
+            if (fPrintHeader)
+            {
+                fPrintHeader = 0;
+                crInfo("Host does not support OpenGL extension(s):");
+            }
+            crInfo("  %.*s", offExtEnd, pszExt);
+        }
+
+        pszExt += offExtEnd;
+    }
+}
+
 void renderspuPerformMakeCurrent(WindowInfo *window, GLint nativeWindow, ContextInfo *context)
 {
     if (window && context)
     {
-#ifdef CHROMIUM_THREADSAFE
         crSetTSD(&_RenderTSD, context);
-#else
-        render_spu.currentContext = context;
-#endif
         context->currentWindow = window;
-        if (!window)
-        {
-            crDebug("Render SPU: MakeCurrent invalid window id: %d", window->BltInfo.Base.id);
-            return;
-        }
-        if (!context)
-        {
-            crDebug("Render SPU: MakeCurrent invalid context id: %d", context->BltInfo.Base.id);
-            return;
-        }
 
         renderspu_SystemMakeCurrent( window, nativeWindow, context );
         if (!context->everCurrent) {
+            static volatile uint32_t u32ExtCompared = 0;
             /* print OpenGL info */
             const char *extString = (const char *) render_spu.ws.glGetString( GL_EXTENSIONS );
             /*
@@ -372,6 +481,10 @@ void renderspuPerformMakeCurrent(WindowInfo *window, GLint nativeWindow, Context
             crInfo( "Render SPU: GL_RENDERER: %s", render_spu.ws.glGetString( GL_RENDERER ) );
             crInfo( "Render SPU: GL_VERSION:  %s", render_spu.ws.glGetString( GL_VERSION ) );
             crInfo( "Render SPU: GL_EXTENSIONS: %s", render_spu.ws.glGetString( GL_EXTENSIONS ) );
+
+            if (ASMAtomicCmpXchgU32(&u32ExtCompared, 1, 0))
+                renderCompareGLExtensions(extString, crExtensions);
+
             if (crStrstr(extString, "GL_ARB_window_pos"))
                 context->haveWindowPosARB = GL_TRUE;
             else
@@ -392,11 +505,7 @@ void renderspuPerformMakeCurrent(WindowInfo *window, GLint nativeWindow, Context
     else if (!window && !context)
     {
         renderspu_SystemMakeCurrent( NULL, 0, NULL );
-#ifdef CHROMIUM_THREADSAFE
         crSetTSD(&_RenderTSD, NULL);
-#else
-        render_spu.currentContext = NULL;
-#endif
     }
     else
     {
@@ -445,11 +554,10 @@ renderspuMakeCurrent(GLint crWindow, GLint nativeWindow, GLint ctx)
     renderspuPerformMakeCurrent(window, nativeWindow, context);
 }
 
-GLboolean renderspuWindowInitWithVisual( WindowInfo *window, VisualInfo *visual, GLboolean showIt, GLint id )
+GLboolean renderspuWinInitWithVisual( WindowInfo *window, VisualInfo *visual, GLboolean showIt, GLint id )
 {
     crMemset(window, 0, sizeof (*window));
     RTCritSectInit(&window->CompositorLock);
-    window->fCompositorPresentEmpty = GL_FALSE;
     window->pCompositor = NULL;
 
     window->BltInfo.Base.id = id;
@@ -481,6 +589,7 @@ GLboolean renderspuWindowInitWithVisual( WindowInfo *window, VisualInfo *visual,
         
     window->BltInfo.Base.visualBits = visual->visAttribs;
     
+    window->cRefs = 1;
     
     /*
     crDebug("Render SPU: Creating window (visBits=0x%x, id=%d)", visBits, window->BltInfo.Base.id);
@@ -501,7 +610,7 @@ GLboolean renderspuWindowInitWithVisual( WindowInfo *window, VisualInfo *visual,
 /*
  * Window functions
  */
-GLboolean renderspuWindowInit(WindowInfo *pWindow, const char *dpyName, GLint visBits, GLint id)
+GLboolean renderspuWinInit(WindowInfo *pWindow, const char *dpyName, GLint visBits, GLint id)
 {
     VisualInfo *visual;
 
@@ -521,7 +630,7 @@ GLboolean renderspuWindowInit(WindowInfo *pWindow, const char *dpyName, GLint vi
     crDebug("Render SPU: Creating window (visBits=0x%x, id=%d)", visBits, window->BltInfo.Base.id);
     */
     /* Have GLX/WGL/AGL create the window */
-    if (!renderspuWindowInitWithVisual( pWindow, visual, 0, id ))
+    if (!renderspuWinInitWithVisual( pWindow, visual, 0, id ))
     {
         crWarning( "Render SPU: Couldn't create a window, renderspu_SystemCreateWindow failed" );
         return GL_FALSE;
@@ -534,6 +643,8 @@ GLint renderspuWindowCreateEx( const char *dpyName, GLint visBits, GLint id )
 {
     WindowInfo *window;
 
+    RT_NOREF(dpyName);
+
     if (id <= 0)
     {
         id = (GLint)crHashtableAllocKeys(render_spu.windowTable, 1);
@@ -553,16 +664,11 @@ GLint renderspuWindowCreateEx( const char *dpyName, GLint visBits, GLint id )
     }
 
     /* Allocate WindowInfo */
-    window = (WindowInfo *) crCalloc(sizeof(WindowInfo));
+    window = renderspuWinCreate(visBits, id);
+
     if (!window)
     {
-        crWarning( "Render SPU: Couldn't create a window" );
-        return -1;
-    }
-    
-    if (!renderspuWindowInit(window, dpyName, visBits, id))
-    {
-        crWarning("renderspuWindowInit failed");
+        crWarning("renderspuWinCreate failed");
         crFree(window);
         return -1;
     }
@@ -577,55 +683,9 @@ renderspuWindowCreate( const char *dpyName, GLint visBits )
     return renderspuWindowCreateEx( dpyName, visBits, 0 );
 }
 
-static void renderspuCheckCurrentCtxWindowCB(unsigned long key, void *data1, void *data2)
+void renderspuWinReleaseCb(void*pvWindow)
 {
-    ContextInfo *pCtx = (ContextInfo *) data1;
-    WindowInfo *pWindow = data2;
-    (void) key;
-
-    if (pCtx->currentWindow==pWindow)
-    {
-        WindowInfo* pDummy = renderspuGetDummyWindow(pCtx->BltInfo.Base.visualBits);
-        if (pDummy)
-        {
-            renderspuPerformMakeCurrent(pDummy, 0, pCtx);
-        }
-        else
-        {
-            crWarning("failed to get dummy window");
-            renderspuMakeCurrent(CR_RENDER_DEFAULT_WINDOW_ID, 0, pCtx->BltInfo.Base.id);
-        }
-    }
-}
-
-void renderspuWindowTerm( WindowInfo *window )
-{
-    GET_CONTEXT(pOldCtx);
-    WindowInfo * pOldWindow = pOldCtx ? pOldCtx->currentWindow : NULL;
-    CRASSERT(!pOldCtx == !pOldWindow);
-    /* ensure no concurrent draws can take place */
-    renderspuVBoxCompositorSet(window, NULL);
-    renderspuVBoxPresentBlitterCleanup(window);
-    renderspu_SystemDestroyWindow( window );
-    RTCritSectDelete(&window->CompositorLock);
-    /* check if this window is bound to some ctx. Note: window pointer is already freed here */
-    crHashtableWalk(render_spu.contextTable, renderspuCheckCurrentCtxWindowCB, window);
-    /* restore current context */
-    {
-        GET_CONTEXT(pNewCtx);
-        WindowInfo * pNewWindow = pNewCtx ? pNewCtx->currentWindow : NULL;
-        CRASSERT(!pNewCtx == !pNewWindow);
-
-        if (pOldWindow == window)
-            renderspuMakeCurrent(CR_RENDER_DEFAULT_WINDOW_ID, 0, CR_RENDER_DEFAULT_CONTEXT_ID);
-        else if (pNewCtx != pOldCtx || pOldWindow != pNewWindow)
-        {
-            if (pOldCtx)
-                renderspuPerformMakeCurrent(pOldWindow, 0, pOldCtx);
-            else
-                renderspuMakeCurrent(CR_RENDER_DEFAULT_WINDOW_ID, 0, CR_RENDER_DEFAULT_CONTEXT_ID);
-        }
-    }
+    renderspuWinRelease((WindowInfo*)pvWindow);
 }
 
 void
@@ -642,10 +702,14 @@ RENDER_APIENTRY renderspuWindowDestroy( GLint win )
     window = (WindowInfo *) crHashtableSearch(render_spu.windowTable, win);
     if (window) {
         crDebug("Render SPU: Destroy window (%d)", win);
-        renderspuWindowTerm( window );
+        /* since os-specific backend can hold its own reference to the window object (e.g. on OSX),
+         * we need to explicitly issue a window destroy command
+         * this ensures the backend will eventually release the reference,
+         * the window object itself will remain valid until its ref count reaches zero */
+        renderspuWinTerm( window );
 
         /* remove window info from hash table, and free it */
-        crHashtableDelete(render_spu.windowTable, win, crFree);
+        crHashtableDelete(render_spu.windowTable, win, renderspuWinReleaseCb);
 
     }
     else {
@@ -661,8 +725,8 @@ renderspuWindowSize( GLint win, GLint w, GLint h )
     CRASSERT(win >= 0);
     window = (WindowInfo *) crHashtableSearch(render_spu.windowTable, win);
     if (window) {
-        if (w != window->BltInfo.width
-                || h != window->BltInfo.height)
+        if (   (GLuint)w != window->BltInfo.width
+            || (GLuint)h != window->BltInfo.height)
         {
             /* window is resized, compositor data is no longer valid
              * this set also ensures all redraw operations are done in the redraw thread
@@ -674,7 +738,7 @@ renderspuWindowSize( GLint win, GLint w, GLint h )
         }
     }
     else {
-        crDebug("Render SPU: Attempt to resize invalid window (%d)", win);
+        WARN(("Render SPU: Attempt to resize invalid window (%d)", win));
     }
 }
 
@@ -753,7 +817,7 @@ renderspuWindowShow( GLint win, GLint flag )
         
         visible = !!flag;
         
-        if (window->visible != visible)
+//        if (window->visible != visible)
         {
             renderspu_SystemShowWindow( window, visible );
             window->visible = visible;
@@ -771,14 +835,7 @@ renderspuVBoxPresentComposition( GLint win, const struct VBOXVR_SCR_COMPOSITOR *
     CRASSERT(win >= 0);
     window = (WindowInfo *) crHashtableSearch(render_spu.windowTable, win);
     if (window) {
-        if (pCompositor && CrVrScrCompositorIsEmpty(pCompositor) && !window->fCompositorPresentEmpty)
-            pCompositor = NULL;
-
-        if (pCompositor)
-            window->fCompositorPresentEmpty = GL_FALSE;
-
-        renderspuVBoxCompositorSet( window, pCompositor);
-        if (pCompositor)
+        if (renderspuVBoxCompositorSet(window, pCompositor))
         {
             renderspu_SystemVBoxPresentComposition(window, pChangedEntry);
         }
@@ -940,9 +997,9 @@ int renderspuVBoxPresentBlitterEnter( PCR_BLITTER pBlitter, int32_t i32MakeCurre
     return VINF_SUCCESS;
 }
 
-PCR_BLITTER renderspuVBoxPresentBlitterGetAndEnter( WindowInfo *window, int32_t i32MakeCurrentUserData )
+PCR_BLITTER renderspuVBoxPresentBlitterGetAndEnter( WindowInfo *window, int32_t i32MakeCurrentUserData, bool fRedraw )
 {
-    PCR_BLITTER pBlitter = renderspuVBoxPresentBlitterGet(window);
+    PCR_BLITTER pBlitter = fRedraw ? window->pBlitter : renderspuVBoxPresentBlitterGet(window);
     if (pBlitter)
     {
         int rc = renderspuVBoxPresentBlitterEnter(pBlitter, i32MakeCurrentUserData);
@@ -995,9 +1052,13 @@ PCR_BLITTER renderspuVBoxPresentBlitterEnsureCreated( WindowInfo *window, int32_
     return window->pBlitter;
 }
 
-void renderspuVBoxPresentCompositionGeneric( WindowInfo *window, const struct VBOXVR_SCR_COMPOSITOR * pCompositor, const struct VBOXVR_SCR_COMPOSITOR_ENTRY *pChangedEntry, int32_t i32MakeCurrentUserData )
+void renderspuVBoxPresentCompositionGeneric( WindowInfo *window, const struct VBOXVR_SCR_COMPOSITOR * pCompositor,
+        const struct VBOXVR_SCR_COMPOSITOR_ENTRY *pChangedEntry, int32_t i32MakeCurrentUserData,
+        bool fRedraw )
 {
-    PCR_BLITTER pBlitter = renderspuVBoxPresentBlitterGetAndEnter(window, i32MakeCurrentUserData);
+    PCR_BLITTER pBlitter = renderspuVBoxPresentBlitterGetAndEnter(window, i32MakeCurrentUserData, fRedraw);
+
+    RT_NOREF(pChangedEntry);
     if (!pBlitter)
         return;
 
@@ -1008,34 +1069,50 @@ void renderspuVBoxPresentCompositionGeneric( WindowInfo *window, const struct VB
     CrBltLeave(pBlitter);
 }
 
-void renderspuVBoxCompositorSet( WindowInfo *window, const struct VBOXVR_SCR_COMPOSITOR * pCompositor)
+GLboolean renderspuVBoxCompositorSet( WindowInfo *window, const struct VBOXVR_SCR_COMPOSITOR * pCompositor)
 {
     int rc;
+    GLboolean fEmpty = pCompositor && CrVrScrCompositorIsEmpty(pCompositor);
+    GLboolean fNeedPresent;
+
     /* renderspuVBoxCompositorSet can be invoked from the chromium thread only and is not reentrant,
      * no need to synch here
      * the lock is actually needed to ensure we're in synch with the redraw thread */
-    if (window->pCompositor == pCompositor)
-        return;
+    if (window->pCompositor == pCompositor && !fEmpty)
+        return !!pCompositor;
+
     rc = RTCritSectEnter(&window->CompositorLock);
     if (RT_SUCCESS(rc))
     {
-        window->pCompositor = pCompositor;
+        if (!fEmpty)
+            fNeedPresent = !!pCompositor;
+        else
+        {
+            fNeedPresent = renderspu_SystemWindowNeedEmptyPresent(window);
+            pCompositor = NULL;
+        }
+
+        window->pCompositor = !fEmpty ? pCompositor : NULL;
         RTCritSectLeave(&window->CompositorLock);
-        return;
+        return fNeedPresent;
     }
     else
     {
-        crWarning("RTCritSectEnter failed rc %d", rc);
+        WARN(("RTCritSectEnter failed rc %d", rc));
     }
+
+    return GL_FALSE;
 }
 
 static void renderspuVBoxCompositorClearAllCB(unsigned long key, void *data1, void *data2)
 {
     WindowInfo *window = (WindowInfo *) data1;
+    RT_NOREF(key, data2);
+
     renderspuVBoxCompositorSet(window, NULL);
 }
 
-void renderspuVBoxCompositorClearAll()
+void renderspuVBoxCompositorClearAll(void)
 {
     /* we need to clear window compositor, which is not that trivial though,
      * since the lock order used in presentation thread is compositor lock() -> hash table lock (aquired for id->window resolution)
@@ -1052,7 +1129,10 @@ const struct VBOXVR_SCR_COMPOSITOR * renderspuVBoxCompositorAcquire( WindowInfo
     {
         const VBOXVR_SCR_COMPOSITOR * pCompositor = window->pCompositor;
         if (pCompositor)
+        {
+            Assert(!CrVrScrCompositorIsEmpty(window->pCompositor));
             return pCompositor;
+        }
 
         /* if no compositor is set, release the lock and return */
         RTCritSectLeave(&window->CompositorLock);
@@ -1064,10 +1144,16 @@ const struct VBOXVR_SCR_COMPOSITOR * renderspuVBoxCompositorAcquire( WindowInfo
     return NULL;
 }
 
-int renderspuVBoxCompositorLock(WindowInfo *window)
+int renderspuVBoxCompositorLock(WindowInfo *window, const struct VBOXVR_SCR_COMPOSITOR **ppCompositor)
 {
     int rc = RTCritSectEnter(&window->CompositorLock);
-    AssertRC(rc);
+    if (RT_SUCCESS(rc))
+    {
+        if (ppCompositor)
+            *ppCompositor = window->pCompositor;
+    }
+    else
+        WARN(("RTCritSectEnter failed %d", rc));
     return rc;
 }
 
@@ -1085,7 +1171,10 @@ int renderspuVBoxCompositorTryAcquire(WindowInfo *window, const struct VBOXVR_SC
     {
         *ppCompositor = window->pCompositor;
         if (*ppCompositor)
+        {
+            Assert(!CrVrScrCompositorIsEmpty(window->pCompositor));
             return VINF_SUCCESS;
+        }
 
         /* if no compositor is set, release the lock and return */
         RTCritSectLeave(&window->CompositorLock);
@@ -1102,8 +1191,7 @@ void renderspuVBoxCompositorRelease( WindowInfo *window)
 {
     int rc;
     Assert(window->pCompositor);
-    if (CrVrScrCompositorIsEmpty(window->pCompositor) && RTCritSectGetRecursion(&window->CompositorLock) == 1)
-        window->pCompositor = NULL;
+    Assert(!CrVrScrCompositorIsEmpty(window->pCompositor));
     rc = RTCritSectLeave(&window->CompositorLock);
     if (!RT_SUCCESS(rc))
     {
@@ -1255,9 +1343,6 @@ void RENDER_APIENTRY renderspuSwapBuffers( GLint window, GLint flags )
     if (render_spu.drawCursor)
         DrawCursor( render_spu.cursorX, render_spu.cursorY );
 
-    if (render_spu.swap_master_url)
-        DoSync();
-
     renderspu_SystemSwapBuffers( w, flags );
 }
 
@@ -1366,10 +1451,8 @@ void renderspuSetDefaultSharedContext(ContextInfo *pCtx)
     render_spu.defaultSharedContext = pCtx;
 }
 
-
 static void RENDER_APIENTRY renderspuChromiumParameteriCR(GLenum target, GLint value)
 {
-
     switch (target)
     {
         case GL_HH_SET_DEFAULT_SHARED_CTX:
@@ -1383,6 +1466,23 @@ static void RENDER_APIENTRY renderspuChromiumParameteriCR(GLenum target, GLint v
             renderspuSetDefaultSharedContext(pCtx);
             break;
         }
+        case GL_HH_RENDERTHREAD_INFORM:
+        {
+            if (value)
+            {
+                int rc = renderspuDefaultCtxInit();
+                if (RT_FAILURE(rc))
+                {
+                    WARN(("renderspuDefaultCtxInit failed"));
+                    break;
+                }
+            }
+            else
+            {
+                renderspuCleanupBase(false);
+            }
+            break;
+        }
         default:
 //            crWarning("Unhandled target in renderspuChromiumParameteriCR()");
             break;
@@ -1404,6 +1504,20 @@ renderspuChromiumParameterfCR(GLenum target, GLfloat value)
 #endif
 }
 
+bool renderspuCalloutAvailable(void)
+{
+    return render_spu.pfnClientCallout != NULL;
+}
+
+bool renderspuCalloutClient(PFNVCRSERVER_CLIENT_CALLOUT_CB pfnCb, void *pvCb)
+{
+    if (render_spu.pfnClientCallout)
+    {
+        render_spu.pfnClientCallout(pfnCb, pvCb);
+        return true;
+    }
+    return false;
+}
 
 static void RENDER_APIENTRY
 renderspuChromiumParametervCR(GLenum target, GLenum type, GLsizei count,
@@ -1415,7 +1529,9 @@ renderspuChromiumParametervCR(GLenum target, GLenum type, GLsizei count,
     unsigned char *privbuf = NULL;
 
     switch (target) {
-
+        case GL_HH_SET_CLIENT_CALLOUT:
+            render_spu.pfnClientCallout = (PFNVCRSERVER_CLIENT_CALLOUT)values;
+            break;
         case GL_GATHER_CONNECT_CR:
             if (render_spu.gather_userbuf_size)
                 privbuf = (unsigned char *)crAlloc(1024*768*4);
@@ -1433,18 +1549,6 @@ renderspuChromiumParametervCR(GLenum target, GLenum type, GLsizei count,
             {
                 switch (render_spu.server->clients[client_num]->conn->type)
                 {
-                    case CR_TCPIP:
-                        crDebug("Render SPU: AcceptClient from %s on %d",
-                            render_spu.server->clients[client_num]->conn->hostname, render_spu.gather_port);
-                        render_spu.gather_conns[client_num] =
-                                crNetAcceptClient("tcpip", NULL, port, 1024*1024,  1);
-                        break;
-
-                    case CR_GM:
-                        render_spu.gather_conns[client_num] =
-                                crNetAcceptClient("gm", NULL, port, 1024*1024,  1);
-                        break;
-
                     default:
                         crError("Render SPU: Unknown Network Type to Open Gather Connection");
                 }
@@ -1463,7 +1567,7 @@ renderspuChromiumParametervCR(GLenum target, GLenum type, GLsizei count,
 
                 if (render_spu.gather_conns[client_num])
                 {
-                    crDebug("Render SPU: success! from %s", render_spu.gather_conns[client_num]->hostname);
+                    crDebug("Render SPU: success!");
                 }
             }
 
@@ -1492,10 +1596,6 @@ renderspuChromiumParametervCR(GLenum target, GLenum type, GLsizei count,
          * call to DoSync() we really want [this one, or the one
          * in SwapBuffers above] is not necessary -- karl
          */
-
-        if (render_spu.swap_master_url)
-            DoSync();
-
         for (client_num=0; client_num< render_spu.server->numClients; client_num++)
             crNetSend(render_spu.gather_conns[client_num], NULL, &pingback,
                                         sizeof(CRMessageHeader));
@@ -1752,6 +1852,7 @@ renderspuGetString(GLenum pname)
 static void renderspuReparentWindowCB(unsigned long key, void *data1, void *data2)
 {
     WindowInfo *pWindow = (WindowInfo *)data1;
+    RT_NOREF(key, data2);
 
     renderspu_SystemReparentWindow(pWindow);
 }
@@ -1778,6 +1879,11 @@ DECLEXPORT(void) renderspuReparentWindow(GLint window)
     }
 }
 
+DECLEXPORT(void) renderspuSetUnscaledHiDPI(bool fEnable)
+{
+    render_spu.fUnscaledHiDPI = fEnable;
+}
+
 #define FILLIN( NAME, FUNC ) \
   table[i].name = crStrdup(NAME); \
   table[i].fn = (SPUGenericFunction) FUNC; \
