@@ -6,18 +6,18 @@
 
 
 #define WIN32_LEAN_AND_MEAN
-#include <windows.h>
+#include <iprt/win/windows.h>
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <memory.h>
 
-#include "cr_environment.h"
 #include "cr_error.h"
 #include "cr_string.h"
 #include "renderspu.h"
 #include "cr_mem.h"
 
+#include <iprt/env.h>
 
 /* IAT patcher stuff */
 #define RVA2PTR(_t, _base, _off) ((_t*)(((uint8_t*)(_base)) + (_off)))
@@ -98,9 +98,9 @@ int renderspuIatPatcherGetImportAddress(HMODULE hModule, LPCSTR pszLib, LPCSTR p
     return VERR_NOT_FOUND;
 }
 
-int renderspuIatPatcherPatchEntry(void *pvEntry, void *pvValue, void **ppvOldVal)
+int renderspuIatPatcherPatchEntry(void *pvEntry, PFNRT pfnValue, PFNRT *ppfnOldVal)
 {
-    void **ppfn = (void**)pvEntry;
+    PFNRT *ppfn = (PFNRT *)pvEntry;
     DWORD dwOldProtect = 0;
 
     if (!VirtualProtect(pvEntry, sizeof (pvEntry), PAGE_READWRITE, &dwOldProtect))
@@ -109,9 +109,9 @@ int renderspuIatPatcherPatchEntry(void *pvEntry, void *pvValue, void **ppvOldVal
         return VERR_ACCESS_DENIED;
     }
 
-    if (ppvOldVal)
-        *ppvOldVal = *ppfn;
-    *ppfn = pvValue;
+    if (ppfnOldVal)
+        *ppfnOldVal = *ppfn;
+    *ppfn = pfnValue;
 
     if (!VirtualProtect(pvEntry, sizeof (pvEntry), dwOldProtect, &dwOldProtect))
     {
@@ -122,7 +122,7 @@ int renderspuIatPatcherPatchEntry(void *pvEntry, void *pvValue, void **ppvOldVal
 }
 
 
-int renderspuIatPatcherPatchFunction(HMODULE hModule, LPCSTR pszLib, LPCSTR pszName, void* pfn)
+int renderspuIatPatcherPatchFunction(HMODULE hModule, LPCSTR pszLib, LPCSTR pszName, PFNRT pfn)
 {
     void* pAdr;
     int rc = renderspuIatPatcherGetImportAddress(hModule, pszLib, pszName, &pAdr);
@@ -143,7 +143,7 @@ int renderspuIatPatcherPatchFunction(HMODULE hModule, LPCSTR pszLib, LPCSTR pszN
 }
 
 /* patch */
-static HWND __stdcall renderspuAtiQuirk_GetForegroundWindow()
+static HWND __stdcall renderspuAtiQuirk_GetForegroundWindow(void)
 {
     crDebug("renderspuAtiQuirk_GetForegroundWindow");
     return NULL;
@@ -278,7 +278,7 @@ static int renderspuAtiQuirk_ApplyForModule(LPCSTR pszAtiDll)
         return VERR_NOT_FOUND;
     }
 
-    rc = renderspuIatPatcherPatchFunction(hAtiDll, "user32.dll", "GetForegroundWindow", (void*)renderspuAtiQuirk_GetForegroundWindow);
+    rc = renderspuIatPatcherPatchFunction(hAtiDll, "user32.dll", "GetForegroundWindow", (PFNRT)renderspuAtiQuirk_GetForegroundWindow);
     if (RT_FAILURE(rc))
     {
         crDebug("renderspuIatPatcherPatchFunction failed, %d", rc);
@@ -286,7 +286,7 @@ static int renderspuAtiQuirk_ApplyForModule(LPCSTR pszAtiDll)
     }
 
     crDebug("renderspuAtiQuirk_ApplyForModule SUCCEEDED!");
-    crInfo("ATI Fullscreen qwirk SUCCEEDED!");
+    crInfo("ATI Fullscreen quirk patch SUCCEEDED!");
 
     return VINF_SUCCESS;
 }
@@ -310,7 +310,7 @@ static LPCSTR renderspuRegMultiSzCurVal(LPCSTR pszBuf)
 }
 
 
-static int renderspuAtiQuirk_Apply()
+static int renderspuAtiQuirk_Apply(void)
 {
     char aBuf[4096];
     DWORD cbResult = 0;
@@ -336,18 +336,18 @@ static int renderspuAtiQuirk_Apply()
     return VINF_SUCCESS;
 }
 
-static GLboolean renderspuAtiQuirk_Needed()
+static GLboolean renderspuAtiQuirk_Needed(void)
 {
-    const char * pszString = render_spu.ws.glGetString(GL_VENDOR);
+    const char * pszString = (const char *)render_spu.ws.glGetString(GL_VENDOR);
     if (pszString && strstr(pszString, "ATI"))
         return GL_TRUE;
-    pszString = render_spu.ws.glGetString(GL_RENDERER);
+    pszString = (const char *)render_spu.ws.glGetString(GL_RENDERER);
     if (pszString && strstr(pszString, "ATI"))
         return GL_TRUE;
     return GL_FALSE;
 }
 
-static void renderspuAtiQuirk_ChkApply()
+static void renderspuAtiQuirk_ChkApply(void)
 {
     static GLboolean fChecked = GL_FALSE;
     if (fChecked)
@@ -445,7 +445,7 @@ MainWndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
                         BOOL bRc;
                         pWindow->redraw_device_context = hDC;
 
-                        renderspuVBoxPresentCompositionGeneric(pWindow, pCompositor, NULL, 1);
+                        renderspuVBoxPresentCompositionGeneric(pWindow, pCompositor, NULL, 1, true);
 
                         bRc = EndPaint(pWindow->hWnd, &Paint);
 
@@ -505,7 +505,7 @@ bSetupPixelFormatEXT( HDC hdc, GLbitfield visAttribs)
     int pixelFormat;
     int attribList[100];
     float fattribList[] = { 0.0, 0.0 };
-    int numFormats;
+    unsigned numFormats;
     int i = 0;
     BOOL vis;
 
@@ -663,7 +663,7 @@ bSetupPixelFormatNormal( HDC hdc, GLbitfield visAttribs )
      * versions.
      */
 #ifdef VBOX_CR_SERVER_FORCE_WGL
-    if (crGetenv( "CR_WGL_DO_NOT_USE_GDI" ) != NULL)
+    if (RTEnvGet( "CR_WGL_DO_NOT_USE_GDI" ) != NULL)
     {
         pixelformat = render_spu.ws.wglChoosePixelFormat( hdc, ppfd );
         /* doing this twice is normal Win32 magic */
@@ -1226,7 +1226,7 @@ GLboolean renderspu_SystemVBoxCreateWindow( VisualInfo *visual, GLboolean showIt
     {
         LONG_PTR oldVal = SetWindowLongPtr(window->hWnd, GWLP_USERDATA, (LONG_PTR)window);
         DWORD winEr = GetLastError();
-        Assert(!oldVal && winEr == NO_ERROR);
+        Assert(!oldVal && winEr == NO_ERROR); RT_NOREF(oldVal, winEr);
     }
 
     return GL_TRUE;
@@ -1284,33 +1284,32 @@ void renderspu_SystemShowWindow( WindowInfo *window, GLboolean showIt )
 
 void renderspu_SystemVBoxPresentComposition( WindowInfo *window, const struct VBOXVR_SCR_COMPOSITOR_ENTRY *pChangedEntry )
 {
-    /* the CR_RENDER_FORCE_PRESENT_MAIN_THREAD is actually inherited from cocoa backend impl,
-     * here it forces rendering in WinCmd thread rather than a Main thread.
-     * it is used for debugging only in any way actually.
-     * @todo: change to some more generic macro name */
-#ifndef CR_RENDER_FORCE_PRESENT_MAIN_THREAD
-    const struct VBOXVR_SCR_COMPOSITOR *pCompositor;
-    /* we do not want to be blocked with the GUI thread here, so only draw her eif we are really able to do that w/o bllocking */
-    int rc = renderspuVBoxCompositorTryAcquire(window, &pCompositor);
-    if (RT_SUCCESS(rc))
+    /* The !render_spu.force_present_main_thread code flow is actually inspired
+     * by cocoa backend impl, here it forces rendering in WndProc, i.e. main
+     * thread. It defaults to 0, because it is for debugging and working around
+     * bugs. In principle would need root cause investigation. */
+    if (!render_spu.force_present_main_thread)
     {
-        renderspuVBoxPresentCompositionGeneric(window, pCompositor, pChangedEntry, 0);
-        renderspuVBoxCompositorRelease(window);
+        const struct VBOXVR_SCR_COMPOSITOR *pCompositor;
+        /* we do not want to be blocked with the GUI thread here, so only draw here if we are really able to do that w/o blocking */
+        int rc = renderspuVBoxCompositorTryAcquire(window, &pCompositor);
+        if (RT_SUCCESS(rc))
+        {
+            renderspuVBoxPresentCompositionGeneric(window, pCompositor, pChangedEntry, 0, false);
+            renderspuVBoxCompositorRelease(window);
+        }
+        else if (rc != VERR_SEM_BUSY)
+        {
+            /* this is somewhat we do not expect */
+            crWarning("renderspuVBoxCompositorTryAcquire failed rc %d", rc);
+        }
     }
-    else if (rc == VERR_SEM_BUSY)
-#endif
+
     {
         render_spu.self.Flush();
         renderspuVBoxPresentBlitterEnsureCreated(window, 0);
         RedrawWindow(window->hWnd, NULL, NULL, RDW_INTERNALPAINT);
     }
-#ifndef CR_RENDER_FORCE_PRESENT_MAIN_THREAD
-    else
-    {
-        /* this is somewhat we do not expect */
-        crWarning("renderspuVBoxCompositorTryAcquire failed rc %d", rc);
-    }
-#endif
 }
 
 GLboolean renderspu_SystemCreateContext( VisualInfo *visual, ContextInfo *context, ContextInfo *sharedContext )
@@ -1390,7 +1389,7 @@ void renderspu_SystemMakeCurrent( WindowInfo *window, GLint nativeWindow, Contex
              * and re-create it with the context's visual abilities
              */
 
-            /*@todo Chromium has no correct code to remove window ids and associated info from 
+            /** @todo Chromium has no correct code to remove window ids and associated info from 
              * various tables. This is hack which just hides the root case.
              */
             crWarning("Recreating window in renderspu_SystemMakeCurrent\n");
@@ -1398,6 +1397,7 @@ void renderspu_SystemMakeCurrent( WindowInfo *window, GLint nativeWindow, Contex
             renderspu_SystemVBoxCreateWindow( context->visual, window->visible, window );
         }
 
+#if 0
         if (0/*render_spu.render_to_app_window && nativeWindow*/)
         {
             /* The render_to_app_window option 
@@ -1437,6 +1437,7 @@ void renderspu_SystemMakeCurrent( WindowInfo *window, GLint nativeWindow, Contex
             render_spu.ws.wglMakeCurrent( window->nativeWindow, context->hRC );
         }
         else
+#endif
         {
             if (!context->hRC) {
                 CRASSERT(!nativeWindow);
@@ -1579,6 +1580,12 @@ void renderspu_SystemWindowPosition( WindowInfo *window, GLint x, GLint y )
     window->y = y;
 }
 
+GLboolean renderspu_SystemWindowNeedEmptyPresent(WindowInfo *window)
+{
+    RT_NOREF(window);
+    return GL_FALSE;
+}
+
 void renderspu_SystemWindowVisibleRegion(WindowInfo *window, GLint cRects, const GLint* pRects)
 {
     GLint i;
@@ -1626,6 +1633,8 @@ void renderspu_SystemSwapBuffers( WindowInfo *w, GLint flags )
 {
     int return_value;
 
+    RT_NOREF(flags);
+
     /* peek at the windows message queue */
 //    renderspuHandleWindowMessages( w->hWnd );
 
@@ -1704,22 +1713,23 @@ void renderspu_SystemReparentWindow(WindowInfo *window)
     SetParent(window->hWnd, (HWND)render_spu_parent_window_id);
 }
 
-int renderspu_SystemInit()
+int renderspu_SystemInit(void)
 {
     return VINF_SUCCESS;
 }
 
-int renderspu_SystemTerm()
+int renderspu_SystemTerm(void)
 {
     return VINF_SUCCESS;
 }
 
 void renderspu_SystemDefaultSharedContextChanged(ContextInfo *fromContext, ContextInfo *toContext)
 {
-
+    RT_NOREF(fromContext, toContext);
 }
 
 uint32_t renderspu_SystemPostprocessFunctions(SPUNamedFunctionTable *aFunctions, uint32_t cFunctions, uint32_t cTable)
 {
+    RT_NOREF(aFunctions, cTable);
     return cFunctions;
 }
