@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2014-2015 Oracle Corporation
+ * Copyright (C) 2014-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,9 +18,15 @@
 #ifndef ____H_GUESTDNDSOURCEIMPL
 #define ____H_GUESTDNDSOURCEIMPL
 
+#include <VBox/GuestHost/DragAndDrop.h>
+#include <VBox/HostServices/DragAndDropSvc.h>
+
+using namespace DragAndDropSvc;
+
 #include "GuestDnDSourceWrap.h"
 #include "GuestDnDPrivate.h"
 
+class RecvDataTask;
 struct RECVDATACTX;
 typedef struct RECVDATACTX *PRECVDATACTX;
 
@@ -45,16 +51,16 @@ private:
     /** Private wrapped @name IDnDBase methods.
      * @{ */
     HRESULT isFormatSupported(const com::Utf8Str &aFormat, BOOL *aSupported);
-    HRESULT getFormats(std::vector<com::Utf8Str> &aFormats);
-    HRESULT addFormats(const std::vector<com::Utf8Str> &aFormats);
-    HRESULT removeFormats(const std::vector<com::Utf8Str> &aFormats);
+    HRESULT getFormats(GuestDnDMIMEList &aFormats);
+    HRESULT addFormats(const GuestDnDMIMEList &aFormats);
+    HRESULT removeFormats(const GuestDnDMIMEList &aFormats);
 
     HRESULT getProtocolVersion(ULONG *aProtocolVersion);
     /** @}  */
 
     /** Private wrapped @name IDnDSource methods.
      * @{ */
-    HRESULT dragIsPending(ULONG uScreenId, std::vector<com::Utf8Str> &aFormats, std::vector<DnDAction_T> &aAllowedActions, DnDAction_T *aDefaultAction);
+    HRESULT dragIsPending(ULONG uScreenId, GuestDnDMIMEList &aFormats, std::vector<DnDAction_T> &aAllowedActions, DnDAction_T *aDefaultAction);
     HRESULT drop(const com::Utf8Str &aFormat, DnDAction_T aAction, ComPtr<IProgress> &aProgress);
     HRESULT receiveData(std::vector<BYTE> &aData);
     /** @}  */
@@ -64,10 +70,11 @@ protected:
 #ifdef VBOX_WITH_DRAG_AND_DROP_GH
     /** @name Dispatch handlers for the HGCM callbacks.
      * @{ */
-    int i_onReceiveData(PRECVDATACTX pCtx, const void *pvData, uint32_t cbData, uint64_t cbTotalSize);
+    int i_onReceiveDataHdr(PRECVDATACTX pCtx, PVBOXDNDSNDDATAHDR pDataHdr);
+    int i_onReceiveData(PRECVDATACTX pCtx, PVBOXDNDSNDDATA pSndData);
     int i_onReceiveDir(PRECVDATACTX pCtx, const char *pszPath, uint32_t cbPath, uint32_t fMode);
     int i_onReceiveFileHdr(PRECVDATACTX pCtx, const char *pszPath, uint32_t cbPath, uint64_t cbSize, uint32_t fMode, uint32_t fFlags);
-    int i_onReceiveFileData(PRECVDATACTX pCtx, const void *pvData, uint32_t cbData);
+    int i_onReceiveFileData(PRECVDATACTX pCtx,const void *pvData, uint32_t cbData);
     /** @}  */
 #endif
 
@@ -76,9 +83,9 @@ protected:
     static Utf8Str i_guestErrorToString(int guestRc);
     static Utf8Str i_hostErrorToString(int hostRc);
 
-    /** @name Thread callbacks.
+    /** @name Thread task .
      * @{ */
-    static DECLCALLBACK(int) i_receiveDataThread(RTTHREAD Thread, void *pvUser);
+    static void i_receiveDataThreadTask(RecvDataTask *pTask);
     /** @}  */
 
     /** @name Callbacks for dispatch handler.
@@ -92,7 +99,6 @@ protected:
     int i_receiveData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout);
     int i_receiveRawData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout);
     int i_receiveURIData(PRECVDATACTX pCtx, RTMSINTERVAL msTimeout);
-    int i_updateProcess(PRECVDATACTX pCtx, uint64_t cbDataAdd);
 
 protected:
 
@@ -100,9 +106,12 @@ protected:
     {
         /** Maximum data block size (in bytes) the source can handle. */
         uint32_t    mcbBlockSize;
-        /** The context for receiving data from the guest. */
+        /** The context for receiving data from the guest.
+         *  At the moment only one transfer at a time is supported. */
         RECVDATACTX mRecvCtx;
     } mData;
+
+    friend class RecvDataTask;
 };
 
 #endif /* !____H_GUESTDNDSOURCEIMPL */
