@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -61,20 +61,30 @@ typedef std::list<Utf8Str> StringsList;
 
 #if !defined(VBOX_WITH_XPCOM)
 
-#include <atlcom.h>
-
 /* use a special version of the singleton class factory,
  * see KB811591 in msdn for more info. */
 
 #undef DECLARE_CLASSFACTORY_SINGLETON
 #define DECLARE_CLASSFACTORY_SINGLETON(obj) DECLARE_CLASSFACTORY_EX(CMyComClassFactorySingleton<obj>)
 
+/**
+ * @todo r=bird: This CMyComClassFactorySingleton stuff is probably obsoleted by
+ *                microatl.h? Right?
+ */
+
 template <class T>
-class CMyComClassFactorySingleton : public CComClassFactory
+class CMyComClassFactorySingleton : public ATL::CComClassFactory
 {
 public:
-    CMyComClassFactorySingleton() : m_hrCreate(S_OK){}
-    virtual ~CMyComClassFactorySingleton(){}
+    CMyComClassFactorySingleton() :
+        m_hrCreate(S_OK), m_spObj(NULL)
+    {
+    }
+    virtual ~CMyComClassFactorySingleton()
+    {
+        if (m_spObj)
+            m_spObj->Release();
+    }
     // IClassFactory
     STDMETHOD(CreateInstance)(LPUNKNOWN pUnkOuter, REFIID riid, void** ppvObj)
     {
@@ -82,57 +92,52 @@ public:
         if (ppvObj != NULL)
         {
             *ppvObj = NULL;
-            // Aggregation is not supported in singleton objects.
-            ATLASSERT(pUnkOuter == NULL);
-            if (pUnkOuter != NULL)
-                hRes = CLASS_E_NOAGGREGATION;
+            // no aggregation for singletons
+            AssertReturn(pUnkOuter == NULL, CLASS_E_NOAGGREGATION);
+            if (m_hrCreate == S_OK && m_spObj == NULL)
+            {
+                Lock();
+                __try
+                {
+                    // Fix:  The following If statement was moved inside the __try statement.
+                    // Did another thread arrive here first?
+                    if (m_hrCreate == S_OK && m_spObj == NULL)
+                    {
+                        // lock the module to indicate activity
+                        // (necessary for the monitor shutdown thread to correctly
+                        // terminate the module in case when CreateInstance() fails)
+                        ATL::_pAtlModule->Lock();
+                        ATL::CComObjectCached<T> *p;
+                        m_hrCreate = ATL::CComObjectCached<T>::CreateInstance(&p);
+                        if (SUCCEEDED(m_hrCreate))
+                        {
+                            m_hrCreate = p->QueryInterface(IID_IUnknown, (void **)&m_spObj);
+                            if (FAILED(m_hrCreate))
+                            {
+                                delete p;
+                            }
+                        }
+                        ATL::_pAtlModule->Unlock();
+                    }
+                }
+                __finally
+                {
+                    Unlock();
+                }
+            }
+            if (m_hrCreate == S_OK)
+            {
+                hRes = m_spObj->QueryInterface(riid, ppvObj);
+            }
             else
             {
-                if (m_hrCreate == S_OK && m_spObj == NULL)
-                {
-                    Lock();
-                    __try
-                    {
-                        // Fix:  The following If statement was moved inside the __try statement.
-                        // Did another thread arrive here first?
-                        if (m_hrCreate == S_OK && m_spObj == NULL)
-                        {
-                            // lock the module to indicate activity
-                            // (necessary for the monitor shutdown thread to correctly
-                            // terminate the module in case when CreateInstance() fails)
-                            _pAtlModule->Lock();
-                            CComObjectCached<T> *p;
-                            m_hrCreate = CComObjectCached<T>::CreateInstance(&p);
-                            if (SUCCEEDED(m_hrCreate))
-                            {
-                                m_hrCreate = p->QueryInterface(IID_IUnknown, (void**)&m_spObj);
-                                if (FAILED(m_hrCreate))
-                                {
-                                    delete p;
-                                }
-                            }
-                            _pAtlModule->Unlock();
-                        }
-                    }
-                    __finally
-                    {
-                        Unlock();
-                    }
-                }
-                if (m_hrCreate == S_OK)
-                {
-                    hRes = m_spObj->QueryInterface(riid, ppvObj);
-                }
-                else
-                {
-                    hRes = m_hrCreate;
-                }
+                hRes = m_hrCreate;
             }
         }
         return hRes;
     }
     HRESULT m_hrCreate;
-    CComPtr<IUnknown> m_spObj;
+    IUnknown *m_spObj;
 };
 
 #endif /* !defined(VBOX_WITH_XPCOM) */
@@ -527,7 +532,7 @@ public:
  *
  *  @param      cls     class to declare a ctor and dtor for
  */
-#define DECLARE_EMPTY_CTOR_DTOR(cls) cls(); ~cls();
+#define DECLARE_EMPTY_CTOR_DTOR(cls) cls(); virtual ~cls();
 
 /**
  *  Defines an empty constructor and destructor for the given class.
@@ -587,7 +592,7 @@ public:
      * the name of the C class as the first argument ("context of
      * translation"). See VirtualBoxBase::translate() for more info.
      *
-     * @param aSourceText   String to translate.
+     * @param pcszSourceText String to translate.
      * @param aComment      Comment to the string to resolve possible
      *                      ambiguities (NULL means no comment).
      *
@@ -635,18 +640,18 @@ public:
  * getComponentName() methods. If this macro is not present, instances
  * of a class derived from VirtualBoxBase cannot be instantiated.
  *
- * @param X The class name, e.g. "Class".
- * @param IX The interface name which this class implements, e.g. "IClass".
+ * @param cls The class name, e.g. "Class".
+ * @param iface The interface name which this class implements, e.g. "IClass".
  */
 #ifdef VBOX_WITH_XPCOM
   #define VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT(cls, iface) \
     VIRTUALBOXBASE_ADD_VIRTUAL_COMPONENT_METHODS(cls, iface)
-#else // #ifdef VBOX_WITH_XPCOM
+#else // !VBOX_WITH_XPCOM
   #define VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT(cls, iface) \
     VIRTUALBOXBASE_ADD_VIRTUAL_COMPONENT_METHODS(cls, iface) \
     STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid) \
     { \
-        const _ATL_INTMAP_ENTRY* pEntries = cls::_GetEntries(); \
+        const ATL::_ATL_INTMAP_ENTRY* pEntries = cls::_GetEntries(); \
         Assert(pEntries); \
         if (!pEntries) \
             return S_FALSE; \
@@ -663,7 +668,20 @@ public:
         Assert(bISupportErrorInfoFound); \
         return bSupports ? S_OK : S_FALSE; \
     }
-#endif // #ifdef VBOX_WITH_XPCOM
+#endif // !VBOX_WITH_XPCOM
+
+/**
+ * VBOX_TWEAK_INTERFACE_ENTRY:
+ * Macro for defining magic interface entries needed for all interfaces
+ * implemented by any subclass of VirtualBoxBase.
+ */
+#ifdef VBOX_WITH_XPCOM
+#define VBOX_TWEAK_INTERFACE_ENTRY(iface)
+#else // !VBOX_WITH_XPCOM
+#define VBOX_TWEAK_INTERFACE_ENTRY(iface)                                   \
+        COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.m_p)
+#endif // !VBOX_WITH_XPCOM
+
 
 /**
  * Abstract base class for all component classes implementing COM
@@ -675,33 +693,18 @@ public:
  */
 class ATL_NO_VTABLE VirtualBoxBase
     : public VirtualBoxTranslatable,
-      public CComObjectRootEx<CComMultiThreadModel>
+      public ATL::CComObjectRootEx<ATL::CComMultiThreadModel>
 #if !defined (VBOX_WITH_XPCOM)
     , public ISupportErrorInfo
 #endif
 {
 protected:
 #ifdef RT_OS_WINDOWS
-     CComPtr <IUnknown>   m_pUnkMarshaler;
+     ComPtr<IUnknown> m_pUnkMarshaler;
 #endif
 
-     HRESULT   BaseFinalConstruct()
-     {
-#ifdef RT_OS_WINDOWS
-        return CoCreateFreeThreadedMarshaler(this, //GetControllingUnknown(),
-                                             &m_pUnkMarshaler.p);
-#else
-        return S_OK;
-#endif
-     }
-
-     void   BaseFinalRelease()
-     {
-#ifdef RT_OS_WINDOWS
-         m_pUnkMarshaler.Release();
-#endif
-     }
-
+     HRESULT BaseFinalConstruct();
+     void BaseFinalRelease();
 
 public:
     VirtualBoxBase();
@@ -804,9 +807,35 @@ private:
     ObjectState mState;
 
     /** User-level object lock for subclasses */
-    mutable RWLockHandle *mObjectLock;
+    RWLockHandle *mObjectLock;
+
+    /** Slot of this object in the g_aClassFactoryStats array */
+    uint32_t iFactoryStat;
+
+private:
+    DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(VirtualBoxBase); /* Shuts up MSC warning C4625. */
 };
 
+/** Structure for counting the currently existing and ever created objects
+ * for each component name. */
+typedef struct CLASSFACTORY_STAT
+{
+    const char *psz;
+    uint64_t current;
+    uint64_t overall;
+} CLASSFACTORY_STAT;
+
+/** Maximum number of component names to deal with. There will be debug
+ * assertions if the value is too low. Since the table is global and its
+ * entries are reasonably small, it's not worth squeezing out the last bit. */
+#define CLASSFACTORYSTATS_MAX 128
+
+/* global variables (defined in VirtualBoxBase.cpp) */
+extern CLASSFACTORY_STAT g_aClassFactoryStats[CLASSFACTORYSTATS_MAX];
+extern RWLockHandle *g_pClassFactoryStatsLock;
+
+extern void APIDumpComponentFactoryStats();
+
 /**
  * Dummy macro that is used to shut down Qt's lupdate tool warnings in some
  * situations. This macro needs to be present inside (better at the very
@@ -832,7 +861,7 @@ class Shareable
 public:
 
     Shareable() : mData(NULL), mIsShared(FALSE) {}
-    ~Shareable() { free(); }
+    virtual ~Shareable() { free(); }
 
     void allocate() { attach(new D); }
 
