@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2012 Oracle Corporation
+ * Copyright (C) 2010-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +16,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "../include/ExtPackUtil.h"
 
 #include <iprt/ctype.h>
@@ -73,6 +73,7 @@ static void vboxExtPackClearDesc(PVBOXEXTPACKDESC a_pExtPackDesc)
     a_pExtPackDesc->strEdition.setNull();
     a_pExtPackDesc->uRevision = 0;
     a_pExtPackDesc->strMainModule.setNull();
+    a_pExtPackDesc->strMainVMModule.setNull();
     a_pExtPackDesc->strVrdeModule.setNull();
     a_pExtPackDesc->cPlugIns = 0;
     a_pExtPackDesc->paPlugIns = NULL;
@@ -110,7 +111,7 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
         return new RTCString("No VirtualBoxExtensionPack element");
 
     RTCString strFormatVersion;
-    if (!pVBoxExtPackElm->getAttributeValue("version", strFormatVersion))
+    if (!pVBoxExtPackElm->getAttributeValueN("version", strFormatVersion, RT_XML_ATTR_TINY))
         return new RTCString("Missing format version");
     if (!strFormatVersion.equals("1.0"))
         return &(new RTCString("Unsupported format version: "))->append(strFormatVersion);
@@ -121,14 +122,14 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
     const xml::ElementNode *pNameElm = pVBoxExtPackElm->findChildElement("Name");
     if (!pNameElm)
         return new RTCString("The 'Name' element is missing");
-    const char *pszName = pNameElm->getValue();
+    const char *pszName = pNameElm->getValueN(RT_XML_CONTENT_SMALL);
     if (!VBoxExtPackIsValidName(pszName))
         return &(new RTCString("Invalid name: "))->append(pszName);
 
     const xml::ElementNode *pDescElm = pVBoxExtPackElm->findChildElement("Description");
     if (!pDescElm)
         return new RTCString("The 'Description' element is missing");
-    const char *pszDesc = pDescElm->getValue();
+    const char *pszDesc = pDescElm->getValueN(RT_XML_CONTENT_LARGE);
     if (!pszDesc || *pszDesc == '\0')
         return new RTCString("The 'Description' element is empty");
     if (strpbrk(pszDesc, "\n\r\t\v\b") != NULL)
@@ -137,7 +138,7 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
     const xml::ElementNode *pVersionElm = pVBoxExtPackElm->findChildElement("Version");
     if (!pVersionElm)
         return new RTCString("The 'Version' element is missing");
-    const char *pszVersion = pVersionElm->getValue();
+    const char *pszVersion = pVersionElm->getValueN(RT_XML_CONTENT_SMALL);
     if (!pszVersion || *pszVersion == '\0')
         return new RTCString("The 'Version' element is empty");
     if (!VBoxExtPackIsValidVersionString(pszVersion))
@@ -148,7 +149,7 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
         uRevision = 0;
 
     const char *pszEdition;
-    if (!pVersionElm->getAttributeValue("edition", pszEdition))
+    if (!pVersionElm->getAttributeValueN("edition", pszEdition, RT_XML_ATTR_TINY))
         pszEdition = "";
     if (!VBoxExtPackIsValidEditionString(pszEdition))
         return &(new RTCString("Invalid edition string: "))->append(pszEdition);
@@ -156,12 +157,27 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
     const xml::ElementNode *pMainModuleElm = pVBoxExtPackElm->findChildElement("MainModule");
     if (!pMainModuleElm)
         return new RTCString("The 'MainModule' element is missing");
-    const char *pszMainModule = pMainModuleElm->getValue();
+    const char *pszMainModule = pMainModuleElm->getValueN(RT_XML_CONTENT_SMALL);
     if (!pszMainModule || *pszMainModule == '\0')
         return new RTCString("The 'MainModule' element is empty");
     if (!VBoxExtPackIsValidModuleString(pszMainModule))
         return &(new RTCString("Invalid main module string: "))->append(pszMainModule);
 
+    /*
+     * The main VM module, optional.
+     * Accept both none and empty as tokens of no main VM module.
+     */
+    const char *pszMainVMModule = NULL;
+    const xml::ElementNode *pMainVMModuleElm = pVBoxExtPackElm->findChildElement("MainVMModule");
+    if (pMainVMModuleElm)
+    {
+        pszMainVMModule = pMainVMModuleElm->getValueN(RT_XML_CONTENT_SMALL);
+        if (!pszMainVMModule || *pszMainVMModule == '\0')
+            pszMainVMModule = NULL;
+        else if (!VBoxExtPackIsValidModuleString(pszMainVMModule))
+            return &(new RTCString("Invalid main VM module string: "))->append(pszMainVMModule);
+    }
+
     /*
      * The VRDE module, optional.
      * Accept both none and empty as tokens of no VRDE module.
@@ -170,7 +186,7 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
     const xml::ElementNode *pVrdeModuleElm = pVBoxExtPackElm->findChildElement("VRDEModule");
     if (pVrdeModuleElm)
     {
-        pszVrdeModule = pVrdeModuleElm->getValue();
+        pszVrdeModule = pVrdeModuleElm->getValueN(RT_XML_CONTENT_SMALL);
         if (!pszVrdeModule || *pszVrdeModule == '\0')
             pszVrdeModule = NULL;
         else if (!VBoxExtPackIsValidModuleString(pszVrdeModule))
@@ -204,6 +220,7 @@ static RTCString *vboxExtPackLoadDescFromDoc(xml::Document *a_pDoc, PVBOXEXTPACK
     a_pExtPackDesc->strEdition      = pszEdition;
     a_pExtPackDesc->uRevision       = uRevision;
     a_pExtPackDesc->strMainModule   = pszMainModule;
+    a_pExtPackDesc->strMainVMModule = pszMainVMModule;
     a_pExtPackDesc->strVrdeModule   = pszVrdeModule;
     a_pExtPackDesc->cPlugIns        = cPlugIns;
     a_pExtPackDesc->paPlugIns       = paPlugIns;
@@ -254,16 +271,23 @@ RTCString *VBoxExtPackLoadDesc(const char *a_pszDir, PVBOXEXTPACKDESC a_pExtPack
         {
             Parser.read(szFilePath, Doc);
         }
-        catch (xml::XmlError Err)
+        catch (xml::XmlError &rErr)
         {
-            return new RTCString(Err.what());
+            return new RTCString(rErr.what());
         }
     }
 
     /*
      * Hand the xml doc over to the common code.
      */
-    return vboxExtPackLoadDescFromDoc(&Doc, a_pExtPackDesc);
+    try
+    {
+        return vboxExtPackLoadDescFromDoc(&Doc, a_pExtPackDesc);
+    }
+    catch (RTCError &rXcpt)      // includes all XML exceptions
+    {
+        return new RTCString(rXcpt.what());
+    }
 }
 
 /**
@@ -271,7 +295,7 @@ RTCString *VBoxExtPackLoadDesc(const char *a_pszDir, PVBOXEXTPACKDESC a_pExtPack
  *
  * @returns NULL on success, pointer to an error message on failure (caller
  *          deletes it).
- * @param   a_pszDir        The directory containing the description file.
+ * @param   hVfsFile        The file handle of the description file.
  * @param   a_pExtPackDesc  Where to store the extension pack descriptor.
  * @param   a_pObjInfo      Where to store the object info for the file (unix
  *                          attribs). Optional.
@@ -327,9 +351,9 @@ RTCString *VBoxExtPackLoadDescFromVfsFile(RTVFSFILE hVfsFile, PVBOXEXTPACKDESC a
         {
             Parser.read(pvFile, cbFile, strFileName, Doc);
         }
-        catch (xml::XmlError Err)
+        catch (xml::XmlError &rErr)
         {
-            pstrErr = new RTCString(Err.what());
+            pstrErr = new RTCString(rErr.what());
             rc = VERR_PARSE_ERROR;
         }
     }
@@ -339,7 +363,14 @@ RTCString *VBoxExtPackLoadDescFromVfsFile(RTVFSFILE hVfsFile, PVBOXEXTPACKDESC a
      * Hand the xml doc over to the common code.
      */
     if (RT_SUCCESS(rc))
-        pstrErr = vboxExtPackLoadDescFromDoc(&Doc, a_pExtPackDesc);
+        try
+        {
+            pstrErr = vboxExtPackLoadDescFromDoc(&Doc, a_pExtPackDesc);
+        }
+        catch (RTCError &rXcpt)      // includes all XML exceptions
+        {
+            return new RTCString(rXcpt.what());
+        }
 
     return pstrErr;
 }
@@ -361,6 +392,7 @@ void VBoxExtPackFreeDesc(PVBOXEXTPACKDESC a_pExtPackDesc)
     a_pExtPackDesc->strEdition.setNull();
     a_pExtPackDesc->uRevision = 0;
     a_pExtPackDesc->strMainModule.setNull();
+    a_pExtPackDesc->strMainVMModule.setNull();
     a_pExtPackDesc->strVrdeModule.setNull();
     a_pExtPackDesc->cPlugIns = 0;
     RTMemFree(a_pExtPackDesc->paPlugIns);
@@ -677,7 +709,7 @@ static void vboxExtPackSetError(char *pszError, size_t cbError, const char *pszF
  * Verifies the manifest and its signature.
  *
  * @returns VBox status code, failures with message.
- * @param   hManifestFile       The xml from the extension pack.
+ * @param   hXmlFile            The xml from the extension pack.
  * @param   pszExtPackName      The expected extension pack name.  This can be
  *                              NULL, in which we don't have any expectations.
  * @param   pszError            Where to store an error message on failure.
@@ -843,7 +875,7 @@ static int vboxExtPackVerifyFileDigest(RTMANIFEST hFileManifest, const char *psz
                 {
                     *pStrDigest = szCalculatedDigest;
                 }
-                catch (std::bad_alloc)
+                catch (std::bad_alloc &)
                 {
                     rc = VERR_NO_MEMORY;
                 }
