@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,7 +24,7 @@
 #include <typeinfo>
 
 #if !defined(VBOX_WITH_XPCOM)
-#include <windows.h>
+#include <iprt/win/windows.h>
 #include <dbghelp.h>
 #else /* !defined(VBOX_WITH_XPCOM) */
 /// @todo remove when VirtualBoxErrorInfo goes away from here
@@ -47,17 +47,121 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-VirtualBoxBase::VirtualBoxBase() : mState(this)
+CLASSFACTORY_STAT g_aClassFactoryStats[CLASSFACTORYSTATS_MAX] =
+{
+    { "--- totals ---", 0 },
+    { NULL, 0 }
+};
+
+RWLockHandle *g_pClassFactoryStatsLock = NULL;
+
+
+VirtualBoxBase::VirtualBoxBase() :
+    mState(this),
+    iFactoryStat(~0U)
 {
     mObjectLock = NULL;
+
+    if (!g_pClassFactoryStatsLock)
+    {
+        RWLockHandle *lock = new RWLockHandle(LOCKCLASS_OBJECTSTATE);
+        if (!ASMAtomicCmpXchgPtr(&g_pClassFactoryStatsLock, lock, NULL))
+            delete lock;
+    }
+    Assert(g_pClassFactoryStatsLock);
 }
 
 VirtualBoxBase::~VirtualBoxBase()
 {
+    Assert(iFactoryStat == ~0U);
     if (mObjectLock)
         delete mObjectLock;
 }
 
+HRESULT VirtualBoxBase::BaseFinalConstruct()
+{
+    Assert(iFactoryStat == ~0U);
+    if (g_pClassFactoryStatsLock)
+    {
+        AutoWriteLock alock(g_pClassFactoryStatsLock COMMA_LOCKVAL_SRC_POS);
+        g_aClassFactoryStats[0].current++;
+        g_aClassFactoryStats[0].overall++;
+        const char *pszName = getComponentName();
+        uint32_t i = 1;
+        while (i < CLASSFACTORYSTATS_MAX && g_aClassFactoryStats[i].psz)
+        {
+            if (g_aClassFactoryStats[i].psz == pszName)
+                break;
+            i++;
+        }
+        if (i < CLASSFACTORYSTATS_MAX)
+        {
+            if (!g_aClassFactoryStats[i].psz)
+            {
+                g_aClassFactoryStats[i].psz = pszName;
+                g_aClassFactoryStats[i].current = 0;
+                g_aClassFactoryStats[i].overall = 0;
+            }
+            iFactoryStat = i;
+            g_aClassFactoryStats[i].current++;
+            g_aClassFactoryStats[i].overall++;
+        }
+        else
+            AssertMsg(i < CLASSFACTORYSTATS_MAX, ("%u exhausts size of factory housekeeping array\n", i));
+    }
+    else
+        Assert(g_pClassFactoryStatsLock);
+
+#ifdef RT_OS_WINDOWS
+    return CoCreateFreeThreadedMarshaler(this, //GetControllingUnknown(),
+                                         m_pUnkMarshaler.asOutParam());
+#else
+    return S_OK;
+#endif
+}
+
+void VirtualBoxBase::BaseFinalRelease()
+{
+    if (g_pClassFactoryStatsLock)
+    {
+        AutoWriteLock alock(g_pClassFactoryStatsLock COMMA_LOCKVAL_SRC_POS);
+        g_aClassFactoryStats[0].current--;
+        const char *pszName = getComponentName();
+        if (iFactoryStat < CLASSFACTORYSTATS_MAX)
+        {
+            if (g_aClassFactoryStats[iFactoryStat].psz == pszName)
+            {
+                g_aClassFactoryStats[iFactoryStat].current--;
+                iFactoryStat = ~0U;
+            }
+            else
+                AssertMsgFailed(("could not find factory housekeeping array entry for %s (index %u contains %s)\n", pszName, iFactoryStat, g_aClassFactoryStats[iFactoryStat].psz));
+        }
+        else
+            AssertMsgFailed(("factory housekeeping array corruption, index %u is too large\n", iFactoryStat));
+    }
+    else
+        Assert(g_pClassFactoryStatsLock);
+
+#ifdef RT_OS_WINDOWS
+     m_pUnkMarshaler.setNull();
+#endif
+}
+
+void APIDumpComponentFactoryStats()
+{
+    if (g_pClassFactoryStatsLock)
+    {
+        AutoReadLock alock(g_pClassFactoryStatsLock COMMA_LOCKVAL_SRC_POS);
+        for (uint32_t i = 0; i < CLASSFACTORYSTATS_MAX && g_aClassFactoryStats[i].psz; i++)
+            LogRel(("CFS: component %-30s current %-10u total %-10u\n",
+                    g_aClassFactoryStats[i].psz, g_aClassFactoryStats[i].current,
+                    g_aClassFactoryStats[i].overall));
+    }
+    else
+        Assert(g_pClassFactoryStatsLock);
+}
+
 /**
  * This virtual method returns an RWLockHandle that can be used to
  * protect instance data. This RWLockHandle is generally referred to
@@ -110,8 +214,8 @@ RWLockHandle *VirtualBoxBase::lockHandle() const
         }
  * @endcode
  *
- * @param aThis             object where the exception happened
- * @param RT_SRC_POS_DECL   "RT_SRC_POS" macro instantiation.
+ * @param aThis     object where the exception happened
+ * @param SRC_POS   "RT_SRC_POS" macro instantiation.
  *  */
 /* static */
 HRESULT VirtualBoxBase::handleUnexpectedExceptions(VirtualBoxBase *const aThis, RT_SRC_POS_DECL)
@@ -148,9 +252,11 @@ HRESULT VirtualBoxBase::handleUnexpectedExceptions(VirtualBoxBase *const aThis,
                                 true /* aLogIt */);
     }
 
+#ifndef _MSC_VER /* (unreachable) */
     /* should not get here */
     AssertFailed();
     return E_FAIL;
+#endif
 }
 
 /**
@@ -160,6 +266,14 @@ HRESULT VirtualBoxBase::handleUnexpectedExceptions(VirtualBoxBase *const aThis,
  *  indicates the error severity is reset to zero to make sure the receiver will
  *  recognize that the created error info object represents a warning rather
  *  than an error.
+ *
+ *  @param  aResultCode
+ *  @param  aIID
+ *  @param  pcszComponent
+ *  @param  aText
+ *  @param  aWarning
+ *  @param  aLogIt
+ *  @param  aResultDetail
  */
 /* static */
 HRESULT VirtualBoxBase::setErrorInternal(HRESULT aResultCode,
@@ -181,7 +295,6 @@ HRESULT VirtualBoxBase::setErrorInternal(HRESULT aResultCode,
                 &aIID,
                 pcszComponent,
                 aText.c_str(),
-                aWarning,
                 preserve,
                 aResultDetail));
 
@@ -341,8 +454,7 @@ HRESULT VirtualBoxBase::setErrorInternal(HRESULT aResultCode,
  * class interface ID and component name inserted correctly. This uses the
  * virtual getClassIID() and getComponentName() methods which are automatically
  * defined by the VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT macro.
- * @param aResultCode
- * @param pcsz
+ * @param   aResultCode
  * @return
  */
 HRESULT VirtualBoxBase::setError(HRESULT aResultCode)
@@ -360,7 +472,8 @@ HRESULT VirtualBoxBase::setError(HRESULT aResultCode)
  * class interface ID and component name inserted correctly. This uses the
  * virtual getClassIID() and getComponentName() methods which are automatically
  * defined by the VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT macro.
- * @param aResultCode
+ * @param   aResultCode
+ * @param   pcsz
  * @return
  */
 HRESULT VirtualBoxBase::setError(HRESULT aResultCode, const char *pcsz, ...)
@@ -382,7 +495,7 @@ HRESULT VirtualBoxBase::setError(HRESULT aResultCode, const char *pcsz, ...)
  * class interface ID and component name inserted correctly. This uses the
  * virtual getClassIID() and getComponentName() methods which are automatically
  * defined by the VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT macro.
- * @param ei
+ * @param   ei
  * @return
  */
 HRESULT VirtualBoxBase::setError(const com::ErrorInfo &ei)
@@ -518,7 +631,7 @@ HRESULT VirtualBoxBase::setErrorVrc(int vrc)
     return setErrorInternal(Global::vboxStatusCodeToCOM(vrc),
                             this->getClassIID(),
                             this->getComponentName(),
-                            Utf8Str("%Rrc", vrc),
+                            Utf8StrFmt("%Rrc", vrc),
                             false /* aWarning */,
                             true /* aLogIt */,
                             vrc /* aResultDetail */);
@@ -557,7 +670,7 @@ HRESULT VirtualBoxBase::setErrorVrc(int vrc, const char *pcszMsgFmt, ...)
  *
  * @param   hrc             The COM status code to return.
  * @param   vrc             The VBox status code.
- * @return  Most likely @hrc, see setErrorInternal.
+ * @return  Most likely @a hrc, see setErrorInternal.
  *
  * @sa      VirtualBoxBase::setError(HRESULT)
  */
@@ -566,7 +679,7 @@ HRESULT VirtualBoxBase::setErrorBoth(HRESULT hrc, int vrc)
     return setErrorInternal(hrc,
                             this->getClassIID(),
                             this->getComponentName(),
-                            Utf8Str("%Rrc", vrc),
+                            Utf8StrFmt("%Rrc", vrc),
                             false /* aWarning */,
                             true /* aLogIt */,
                             vrc /* aResultDetail */);
@@ -582,7 +695,7 @@ HRESULT VirtualBoxBase::setErrorBoth(HRESULT hrc, int vrc)
  * @param   vrc             The VBox status code.
  * @param   pcszMsgFmt      Error message format string.
  * @param   ...             Argument specified in the @a pcszMsgFmt
- * @return  Most likely @hrc, see setErrorInternal.
+ * @return  Most likely @a hrc, see setErrorInternal.
  *
  * @sa      VirtualBoxBase::setError(HRESULT, const char *, ...)
  */
