@@ -1,10 +1,11 @@
+/* $Id: VirtualBoxErrorInfoImpl.cpp $ */
 /** @file
  *
  * VirtualBoxErrorInfo COM class implementation
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -30,6 +31,7 @@ HRESULT VirtualBoxErrorInfo::init(HRESULT aResultCode,
                                   IVirtualBoxErrorInfo *aNext)
 {
     m_resultCode = aResultCode;
+    m_resultDetail = 0; /* Not being used. */
     m_IID = aIID;
     m_strComponent = pcszComponent;
     m_strText = strText;
@@ -38,10 +40,24 @@ HRESULT VirtualBoxErrorInfo::init(HRESULT aResultCode,
     return S_OK;
 }
 
+HRESULT VirtualBoxErrorInfo::initEx(HRESULT aResultCode,
+                                    LONG aResultDetail,
+                                    const GUID &aIID,
+                                    const char *pcszComponent,
+                                    const Utf8Str &strText,
+                                    IVirtualBoxErrorInfo *aNext)
+{
+    HRESULT hr = init(aResultCode, aIID, pcszComponent, strText, aNext);
+    m_resultDetail = aResultDetail;
+
+    return hr;
+}
+
 HRESULT VirtualBoxErrorInfo::init(const com::ErrorInfo &info,
                                   IVirtualBoxErrorInfo *aNext)
 {
     m_resultCode = info.getResultCode();
+    m_resultDetail = info.getResultDetail();
     m_IID = info.getInterfaceID();
     m_strComponent = info.getComponent();
     m_strText = info.getText();
@@ -74,6 +90,14 @@ STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(ResultCode)(LONG *aResultCode)
     return S_OK;
 }
 
+STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(ResultDetail)(LONG *aResultDetail)
+{
+    CheckComArgOutPointerValid(aResultDetail);
+
+    *aResultDetail = m_resultDetail;
+    return S_OK;
+}
+
 STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(InterfaceID)(BSTR *aIID)
 {
     CheckComArgOutPointerValid(aIID);
@@ -123,6 +147,7 @@ HRESULT VirtualBoxErrorInfo::init(IErrorInfo *aInfo)
      * corresponding fields will simply remain null in this case). */
 
     m_resultCode = S_OK;
+    m_resultDetail = 0;
     rc = aInfo->GetGUID(m_IID.asOutParam());
     AssertComRC(rc);
     Bstr bstrComponent;
@@ -156,17 +181,19 @@ STDMETHODIMP VirtualBoxErrorInfo::GetGUID(GUID *guid)
 
 STDMETHODIMP VirtualBoxErrorInfo::GetHelpContext(DWORD *pdwHelpContext)
 {
+    RT_NOREF(pdwHelpContext);
     return E_NOTIMPL;
 }
 
-STDMETHODIMP VirtualBoxErrorInfo::GetHelpFile(BSTR *pbstrHelpFile)
+STDMETHODIMP VirtualBoxErrorInfo::GetHelpFile(BSTR *pBstrHelpFile)
 {
+    RT_NOREF(pBstrHelpFile);
     return E_NOTIMPL;
 }
 
-STDMETHODIMP VirtualBoxErrorInfo::GetSource(BSTR *source)
+STDMETHODIMP VirtualBoxErrorInfo::GetSource(BSTR *pBstrSource)
 {
-    return COMGETTER(Component)(source);
+    return COMGETTER(Component)(pBstrSource);
 }
 
 #else // defined(VBOX_WITH_XPCOM)
@@ -187,6 +214,7 @@ HRESULT VirtualBoxErrorInfo::init(nsIException *aInfo)
 
     rc = aInfo->GetResult(&m_resultCode);
     AssertComRC(rc);
+    m_resultDetail = 0; /* Not being used. */
 
     char *pszMsg;             /* No Utf8Str.asOutParam, different allocator! */
     rc = aInfo->GetMessage(&pszMsg);
