@@ -1,11 +1,10 @@
 /* $Id: GuestDnDPrivate.cpp $ */
 /** @file
- * Private guest drag and drop code, used by GuestDnDTarget +
- * GuestDnDSource.
+ * Private guest drag and drop code, used by GuestDnDTarget + GuestDnDSource.
  */
 
 /*
- * Copyright (C) 2011-2015 Oracle Corporation
+ * Copyright (C) 2011-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,6 +15,9 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#define LOG_GROUP LOG_GROUP_GUEST_DND
+#include "LoggingNew.h"
+
 #include "GuestImpl.h"
 #include "AutoCaller.h"
 
@@ -38,13 +40,7 @@
 # include <VBox/HostServices/DragAndDropSvc.h>
 # include <VBox/version.h>
 
-# ifdef LOG_GROUP
-#  undef LOG_GROUP
-# endif
-# define LOG_GROUP LOG_GROUP_GUEST_DND
-# include <VBox/log.h>
-
-/**
+/** @page pg_main_dnd  Dungeons & Dragons - Overview
  * Overview:
  *
  * Drag and Drop is handled over the internal HGCM service for the host <->
@@ -193,7 +189,7 @@ GuestDnDResponse::GuestDnDResponse(const ComObjPtr<Guest>& pGuest)
     : m_EventSem(NIL_RTSEMEVENT)
     , m_defAction(0)
     , m_allActions(0)
-    , m_parent(pGuest)
+    , m_pParent(pGuest)
 {
     int rc = RTSemEventCreate(&m_EventSem);
     if (RT_FAILURE(rc))
@@ -220,19 +216,19 @@ void GuestDnDResponse::reset(void)
     m_defAction  = 0;
     m_allActions = 0;
 
-    m_strFmtReq  = "";
+    m_lstFormats.clear();
 }
 
 HRESULT GuestDnDResponse::resetProgress(const ComObjPtr<Guest>& pParent)
 {
-    m_progress.setNull();
+    m_pProgress.setNull();
 
-    HRESULT hr = m_progress.createObject();
+    HRESULT hr = m_pProgress.createObject();
     if (SUCCEEDED(hr))
     {
-        hr = m_progress->init(static_cast<IGuest *>(pParent),
-                              Bstr(pParent->tr("Dropping data")).raw(),
-                              TRUE /* aCancelable */);
+        hr = m_pProgress->init(static_cast<IGuest *>(pParent),
+                               Bstr(pParent->tr("Dropping data")).raw(),
+                               TRUE /* aCancelable */);
     }
 
     return hr;
@@ -241,9 +237,9 @@ HRESULT GuestDnDResponse::resetProgress(const ComObjPtr<Guest>& pParent)
 bool GuestDnDResponse::isProgressCanceled(void) const
 {
     BOOL fCanceled;
-    if (!m_progress.isNull())
+    if (!m_pProgress.isNull())
     {
-        HRESULT hr = m_progress->COMGETTER(Canceled)(&fCanceled);
+        HRESULT hr = m_pProgress->COMGETTER(Canceled)(&fCanceled);
         AssertComRC(hr);
     }
     else
@@ -280,18 +276,19 @@ int GuestDnDResponse::setProgress(unsigned uPercentage,
                                   uint32_t uStatus,
                                   int rcOp /* = VINF_SUCCESS */, const Utf8Str &strMsg /* = "" */)
 {
+    RT_NOREF(rcOp);
     LogFlowFunc(("uStatus=%RU32, uPercentage=%RU32, rcOp=%Rrc, strMsg=%s\n",
                  uStatus, uPercentage, rcOp, strMsg.c_str()));
 
     int rc = VINF_SUCCESS;
-    if (!m_progress.isNull())
+    if (!m_pProgress.isNull())
     {
         BOOL fCompleted;
-        HRESULT hr = m_progress->COMGETTER(Completed)(&fCompleted);
+        HRESULT hr = m_pProgress->COMGETTER(Completed)(&fCompleted);
         AssertComRC(hr);
 
         BOOL fCanceled;
-        hr = m_progress->COMGETTER(Canceled)(&fCanceled);
+        hr = m_pProgress->COMGETTER(Canceled)(&fCanceled);
         AssertComRC(hr);
 
         LogFlowFunc(("Current: fCompleted=%RTbool, fCanceled=%RTbool\n", fCompleted, fCanceled));
@@ -302,16 +299,18 @@ int GuestDnDResponse::setProgress(unsigned uPercentage,
             {
                 case DragAndDropSvc::DND_PROGRESS_ERROR:
                 {
-                    hr = m_progress->i_notifyComplete(VBOX_E_IPRT_ERROR,
-                                                      COM_IIDOF(IGuest),
-                                                      m_parent->getComponentName(), strMsg.c_str());
+                    hr = m_pProgress->i_notifyComplete(VBOX_E_IPRT_ERROR,
+                                                       COM_IIDOF(IGuest),
+                                                       m_pParent->getComponentName(), strMsg.c_str());
                     reset();
                     break;
                 }
 
                 case DragAndDropSvc::DND_PROGRESS_CANCELLED:
                 {
-                    hr = m_progress->i_notifyComplete(S_OK);
+                    hr = m_pProgress->Cancel();
+                    AssertComRC(hr);
+                    hr = m_pProgress->i_notifyComplete(S_OK);
                     AssertComRC(hr);
 
                     reset();
@@ -323,12 +322,12 @@ int GuestDnDResponse::setProgress(unsigned uPercentage,
                 {
                     if (!fCanceled)
                     {
-                        hr = m_progress->SetCurrentOperationProgress(uPercentage);
+                        hr = m_pProgress->SetCurrentOperationProgress(uPercentage);
                         AssertComRC(hr);
                         if (   uStatus     == DragAndDropSvc::DND_PROGRESS_COMPLETE
                             || uPercentage >= 100)
                         {
-                            hr = m_progress->i_notifyComplete(S_OK);
+                            hr = m_pProgress->i_notifyComplete(S_OK);
                             AssertComRC(hr);
                         }
                     }
@@ -340,9 +339,9 @@ int GuestDnDResponse::setProgress(unsigned uPercentage,
             }
         }
 
-        hr = m_progress->COMGETTER(Completed)(&fCompleted);
+        hr = m_pProgress->COMGETTER(Completed)(&fCompleted);
         AssertComRC(hr);
-        hr = m_progress->COMGETTER(Canceled)(&fCanceled);
+        hr = m_pProgress->COMGETTER(Canceled)(&fCanceled);
         AssertComRC(hr);
 
         LogFlowFunc(("New: fCompleted=%RTbool, fCanceled=%RTbool\n", fCompleted, fCanceled));
@@ -361,12 +360,28 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
 
     switch (u32Function)
     {
+        case DragAndDropSvc::GUEST_DND_CONNECT:
+        {
+            LogThisFunc(("Client connected\n"));
+
+            /* Nothing to do here (yet). */
+            rc = VINF_SUCCESS;
+            break;
+        }
+
+        case DragAndDropSvc::GUEST_DND_DISCONNECT:
+        {
+            LogThisFunc(("Client disconnected\n"));
+            rc = setProgress(100, DND_PROGRESS_CANCELLED, VINF_SUCCESS);
+            break;
+        }
+
         case DragAndDropSvc::GUEST_DND_HG_ACK_OP:
         {
             DragAndDropSvc::PVBOXDNDCBHGACKOPDATA pCBData = reinterpret_cast<DragAndDropSvc::PVBOXDNDCBHGACKOPDATA>(pvParms);
             AssertPtr(pCBData);
             AssertReturn(sizeof(DragAndDropSvc::VBOXDNDCBHGACKOPDATA) == cbParms, VERR_INVALID_PARAMETER);
-            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_ACK_OP == pCBData->hdr.u32Magic, VERR_INVALID_PARAMETER);
+            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_ACK_OP == pCBData->hdr.uMagic, VERR_INVALID_PARAMETER);
 
             setDefAction(pCBData->uAction);
             rc = notifyAboutGuestResponse();
@@ -378,17 +393,21 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
             DragAndDropSvc::PVBOXDNDCBHGREQDATADATA pCBData = reinterpret_cast<DragAndDropSvc::PVBOXDNDCBHGREQDATADATA>(pvParms);
             AssertPtr(pCBData);
             AssertReturn(sizeof(DragAndDropSvc::VBOXDNDCBHGREQDATADATA) == cbParms, VERR_INVALID_PARAMETER);
-            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_REQ_DATA == pCBData->hdr.u32Magic, VERR_INVALID_PARAMETER);
+            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_REQ_DATA == pCBData->hdr.uMagic, VERR_INVALID_PARAMETER);
 
-            if (   pCBData->cbFormat == 0
-                || pCBData->cbFormat > _64K)
+            if (   pCBData->cbFormat  == 0
+                || pCBData->cbFormat  > _64K /** @todo Make this configurable? */
+                || pCBData->pszFormat == NULL)
+            {
+                rc = VERR_INVALID_PARAMETER;
+            }
+            else if (!RTStrIsValidEncoding(pCBData->pszFormat))
             {
                 rc = VERR_INVALID_PARAMETER;
             }
             else
             {
-                setFmtReq(pCBData->pszFormat);
-
+                setFormats(GuestDnD::toFormatList(pCBData->pszFormat));
                 rc = VINF_SUCCESS;
             }
 
@@ -404,7 +423,7 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
                reinterpret_cast<DragAndDropSvc::PVBOXDNDCBHGEVTPROGRESSDATA>(pvParms);
             AssertPtr(pCBData);
             AssertReturn(sizeof(DragAndDropSvc::VBOXDNDCBHGEVTPROGRESSDATA) == cbParms, VERR_INVALID_PARAMETER);
-            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_EVT_PROGRESS == pCBData->hdr.u32Magic, VERR_INVALID_PARAMETER);
+            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_EVT_PROGRESS == pCBData->hdr.uMagic, VERR_INVALID_PARAMETER);
 
             rc = setProgress(pCBData->uPercentage, pCBData->uStatus, pCBData->rc);
             if (RT_SUCCESS(rc))
@@ -418,16 +437,21 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
                reinterpret_cast<DragAndDropSvc::PVBOXDNDCBGHACKPENDINGDATA>(pvParms);
             AssertPtr(pCBData);
             AssertReturn(sizeof(DragAndDropSvc::VBOXDNDCBGHACKPENDINGDATA) == cbParms, VERR_INVALID_PARAMETER);
-            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_GH_ACK_PENDING == pCBData->hdr.u32Magic, VERR_INVALID_PARAMETER);
+            AssertReturn(DragAndDropSvc::CB_MAGIC_DND_GH_ACK_PENDING == pCBData->hdr.uMagic, VERR_INVALID_PARAMETER);
 
-            if (   pCBData->cbFormat == 0
-                || pCBData->cbFormat >  _64K)
+            if (   pCBData->cbFormat  == 0
+                || pCBData->cbFormat  > _64K /** @todo Make the maximum size configurable? */
+                || pCBData->pszFormat == NULL)
+            {
+                rc = VERR_INVALID_PARAMETER;
+            }
+            else if (!RTStrIsValidEncoding(pCBData->pszFormat))
             {
                 rc = VERR_INVALID_PARAMETER;
             }
             else
             {
-                setFmtReq    (pCBData->pszFormat);
+                setFormats   (GuestDnD::toFormatList(pCBData->pszFormat));
                 setDefAction (pCBData->uDefAction);
                 setAllActions(pCBData->uAllActions);
 
@@ -458,7 +482,10 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
             rc = it->second.pfnCallback(u32Function, pvParms, cbParms, it->second.pvUser);
         }
         else
-            rc = VERR_NO_DATA; /* Tell the guest. */
+        {
+            LogFlowFunc(("No callback for function %RU32 defined (%zu callbacks total)\n", u32Function, m_mapCallbacks.size()));
+            rc = VERR_NOT_SUPPORTED; /* Tell the guest. */
+        }
     }
 
     LogFlowFunc(("Returning rc=%Rrc\n", rc));
@@ -467,7 +494,7 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
 
 HRESULT GuestDnDResponse::queryProgressTo(IProgress **ppProgress)
 {
-    return m_progress.queryInterfaceTo(ppProgress);
+    return m_pProgress.queryInterfaceTo(ppProgress);
 }
 
 int GuestDnDResponse::waitForGuestResponse(RTMSINTERVAL msTimeout /*= 500 */) const
@@ -508,7 +535,7 @@ GuestDnD::~GuestDnD(void)
         delete m_pResponse;
 }
 
-int GuestDnD::adjustScreenCoordinates(ULONG uScreenId, ULONG *puX, ULONG *puY) const
+HRESULT GuestDnD::adjustScreenCoordinates(ULONG uScreenId, ULONG *puX, ULONG *puY) const
 {
     /** @todo r=andy Save the current screen's shifting coordinates to speed things up.
      *               Only query for new offsets when the screen ID has changed. */
@@ -534,11 +561,8 @@ int GuestDnD::adjustScreenCoordinates(ULONG uScreenId, ULONG *puX, ULONG *puY) c
     if (puY)
         *puY += yShift;
 
-#ifdef DEBUG_andy
-    LogFlowFunc(("uScreenId=%RU32, x=%RU32, y=%RU32\n",
-                 uScreenId, puX ? *puX : 0, puY ? *puY : 0));
-#endif
-    return VINF_SUCCESS;
+    LogFlowFunc(("uScreenId=%RU32, x=%RU32, y=%RU32\n", uScreenId, puX ? *puX : 0, puY ? *puY : 0));
+    return S_OK;
 }
 
 int GuestDnD::hostCall(uint32_t u32Function, uint32_t cParms, PVBOXHGCMSVCPARM paParms) const
@@ -552,9 +576,7 @@ int GuestDnD::hostCall(uint32_t u32Function, uint32_t cParms, PVBOXHGCMSVCPARM p
     if (!pVMMDev)
         return VERR_COM_OBJECT_NOT_FOUND;
 
-    int rc = pVMMDev->hgcmHostCall("VBoxDragAndDropSvc", u32Function, cParms, paParms);
-    LogFlowFunc(("uMsg=%RU32, cParms=%RU32, rc=%Rrc\n", u32Function, cParms, rc));
-    return rc;
+    return pVMMDev->hgcmHostCall("VBoxDragAndDropSvc", u32Function, cParms, paParms);
 }
 
 /* static */
@@ -577,48 +599,74 @@ DECLCALLBACK(int) GuestDnD::notifyDnDDispatcher(void *pvExtension, uint32_t u32F
     return VERR_NOT_SUPPORTED;
 }
 
+/* static */
+bool GuestDnD::isFormatInFormatList(const com::Utf8Str &strFormat, const GuestDnDMIMEList &lstFormats)
+{
+    return std::find(lstFormats.begin(), lstFormats.end(), strFormat) != lstFormats.end();
+}
 
 /* static */
-com::Utf8Str GuestDnD::toFormatString(const std::vector<com::Utf8Str> &lstSupportedFormats,
-                                      const std::vector<com::Utf8Str> &lstWantedFormats)
+GuestDnDMIMEList GuestDnD::toFormatList(const com::Utf8Str &strFormats)
+{
+    GuestDnDMIMEList lstFormats;
+    RTCList<RTCString> lstFormatsTmp = strFormats.split("\r\n");
+
+    for (size_t i = 0; i < lstFormatsTmp.size(); i++)
+        lstFormats.push_back(com::Utf8Str(lstFormatsTmp.at(i)));
+
+    return lstFormats;
+}
+
+/* static */
+com::Utf8Str GuestDnD::toFormatString(const GuestDnDMIMEList &lstFormats)
 {
     com::Utf8Str strFormat;
-    for (size_t i = 0; i < lstWantedFormats.size(); ++i)
+    for (size_t i = 0; i < lstFormats.size(); i++)
     {
-        const com::Utf8Str &f = lstWantedFormats.at(i);
-        /* Only keep allowed format types. */
-        if (std::find(lstSupportedFormats.begin(),
-                      lstSupportedFormats.end(), f) != lstSupportedFormats.end())
-            strFormat += f + "\r\n";
+        const com::Utf8Str &f = lstFormats.at(i);
+        strFormat += f + "\r\n";
     }
 
     return strFormat;
 }
 
 /* static */
-void GuestDnD::toFormatVector(const std::vector<com::Utf8Str> &lstSupportedFormats,
-                              const com::Utf8Str &strFormats,
-                              std::vector<com::Utf8Str> &vecformats)
+GuestDnDMIMEList GuestDnD::toFilteredFormatList(const GuestDnDMIMEList &lstFormatsSupported, const GuestDnDMIMEList &lstFormatsWanted)
 {
-    RTCList<RTCString> lstFormats = strFormats.split("\r\n");
+    GuestDnDMIMEList lstFormats;
+
+    for (size_t i = 0; i < lstFormatsWanted.size(); i++)
+    {
+        /* Only keep supported format types. */
+        if (std::find(lstFormatsSupported.begin(),
+                      lstFormatsSupported.end(), lstFormatsWanted.at(i)) != lstFormatsSupported.end())
+        {
+            lstFormats.push_back(lstFormatsWanted[i]);
+        }
+    }
+
+    return lstFormats;
+}
+
+/* static */
+GuestDnDMIMEList GuestDnD::toFilteredFormatList(const GuestDnDMIMEList &lstFormatsSupported, const com::Utf8Str &strFormatsWanted)
+{
+    GuestDnDMIMEList lstFmt;
+
+    RTCList<RTCString> lstFormats = strFormatsWanted.split("\r\n");
     size_t i = 0;
     while (i < lstFormats.size())
     {
         /* Only keep allowed format types. */
-        if (std::find(lstSupportedFormats.begin(),
-                      lstSupportedFormats.end(), lstFormats.at(i)) == lstSupportedFormats.end())
-            lstFormats.removeAt(i);
-        else
-            ++i;
+        if (std::find(lstFormatsSupported.begin(),
+                      lstFormatsSupported.end(), lstFormats.at(i)) != lstFormatsSupported.end())
+        {
+            lstFmt.push_back(lstFormats[i]);
+        }
+        i++;
     }
 
-    for (i = 0; i < lstFormats.size(); i++)
-    {
-        const Utf8Str &f = lstFormats.at(i);
-        if (std::find(lstSupportedFormats.begin(),
-                      lstSupportedFormats.end(), f) != lstSupportedFormats.end())
-            vecformats.push_back(lstFormats[i]);
-    }
+    return lstFmt;
 }
 
 /* static */
@@ -692,9 +740,10 @@ DnDAction_T GuestDnD::toMainAction(uint32_t uAction)
 }
 
 /* static */
-void GuestDnD::toMainActions(uint32_t uActions,
-                             std::vector<DnDAction_T> &vecActions)
+std::vector<DnDAction_T> GuestDnD::toMainActions(uint32_t uActions)
 {
+    std::vector<DnDAction_T> vecActions;
+
     /* For now it doesn't seems useful to allow a
      * link action between host & guest. Maybe later! */
     RTCList<DnDAction_T> lstActions;
@@ -705,59 +754,61 @@ void GuestDnD::toMainActions(uint32_t uActions,
 
     for (size_t i = 0; i < lstActions.size(); ++i)
         vecActions.push_back(lstActions.at(i));
+
+    return vecActions;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 
 GuestDnDBase::GuestDnDBase(void)
 {
-    mDataBase.mfTransferIsPending = false;
+    /* Initialize public attributes. */
+    m_lstFmtSupported = GuestDnDInst()->defaultFormats();
 
-    /*
-     * Initialize public attributes.
-     */
-    m_vecFmtSup = GuestDnDInst()->defaultFormats();
+    /* Initialzie private stuff. */
+    mDataBase.m_cTransfersPending = 0;
+    mDataBase.m_uProtocolVersion  = 0;
 }
 
 HRESULT GuestDnDBase::i_isFormatSupported(const com::Utf8Str &aFormat, BOOL *aSupported)
 {
-    *aSupported = std::find(m_vecFmtSup.begin(),
-                            m_vecFmtSup.end(), aFormat) != m_vecFmtSup.end()
+    *aSupported = std::find(m_lstFmtSupported.begin(),
+                            m_lstFmtSupported.end(), aFormat) != m_lstFmtSupported.end()
                 ? TRUE : FALSE;
     return S_OK;
 }
 
-HRESULT GuestDnDBase::i_getFormats(std::vector<com::Utf8Str> &aFormats)
+HRESULT GuestDnDBase::i_getFormats(GuestDnDMIMEList &aFormats)
 {
-    aFormats = m_vecFmtSup;
+    aFormats = m_lstFmtSupported;
 
     return S_OK;
 }
 
-HRESULT GuestDnDBase::i_addFormats(const std::vector<com::Utf8Str> &aFormats)
+HRESULT GuestDnDBase::i_addFormats(const GuestDnDMIMEList &aFormats)
 {
     for (size_t i = 0; i < aFormats.size(); ++i)
     {
         Utf8Str strFormat = aFormats.at(i);
-        if (std::find(m_vecFmtSup.begin(),
-                      m_vecFmtSup.end(), strFormat) == m_vecFmtSup.end())
+        if (std::find(m_lstFmtSupported.begin(),
+                      m_lstFmtSupported.end(), strFormat) == m_lstFmtSupported.end())
         {
-            m_vecFmtSup.push_back(strFormat);
+            m_lstFmtSupported.push_back(strFormat);
         }
     }
 
     return S_OK;
 }
 
-HRESULT GuestDnDBase::i_removeFormats(const std::vector<com::Utf8Str> &aFormats)
+HRESULT GuestDnDBase::i_removeFormats(const GuestDnDMIMEList &aFormats)
 {
     for (size_t i = 0; i < aFormats.size(); ++i)
     {
         Utf8Str strFormat = aFormats.at(i);
-        std::vector<com::Utf8Str>::iterator itFormat = std::find(m_vecFmtSup.begin(),
-                                                                 m_vecFmtSup.end(), strFormat);
-        if (itFormat != m_vecFmtSup.end())
-            m_vecFmtSup.erase(itFormat);
+        GuestDnDMIMEList::iterator itFormat = std::find(m_lstFmtSupported.begin(),
+                                                        m_lstFmtSupported.end(), strFormat);
+        if (itFormat != m_lstFmtSupported.end())
+            m_lstFmtSupported.erase(itFormat);
     }
 
     return S_OK;
@@ -769,103 +820,148 @@ HRESULT GuestDnDBase::i_getProtocolVersion(ULONG *puVersion)
     return RT_SUCCESS(rc) ? S_OK : E_FAIL;
 }
 
-int GuestDnDBase::getProtocolVersion(uint32_t *puVersion)
+/**
+ * Tries to guess the DnD protocol version to use on the guest, based on the
+ * installed Guest Additions version + revision.
+ *
+ * If unable to retrieve the protocol version, VERR_NOT_FOUND is returned along
+ * with protocol version 1.
+ *
+ * @return  IPRT status code.
+ * @param   puProto                 Where to store the protocol version.
+ */
+int GuestDnDBase::getProtocolVersion(uint32_t *puProto)
 {
-    AssertPtrReturn(puVersion, VERR_INVALID_POINTER);
+    AssertPtrReturn(puProto, VERR_INVALID_POINTER);
 
     int rc;
 
-    uint32_t uVer, uVerAdditions = 0;
+    uint32_t uProto = 0;
+    uint32_t uVerAdditions;
+    uint32_t uRevAdditions;
     if (   m_pGuest
-        && (uVerAdditions = m_pGuest->i_getAdditionsVersion()) > 0)
+        && (uVerAdditions = m_pGuest->i_getAdditionsVersion())  > 0
+        && (uRevAdditions = m_pGuest->i_getAdditionsRevision()) > 0)
     {
-        uint32_t uVBoxMajor = VBOX_FULL_VERSION_GET_MAJOR(uVerAdditions);
-        uint32_t uVBoxMinor = VBOX_FULL_VERSION_GET_MINOR(uVerAdditions);
-
-#if 0 /*def DEBUG_andy*/
+#if 0 && defined(DEBUG)
         /* Hardcode the to-used protocol version; nice for testing side effects. */
-        uVer = 2;
-#else
-        uVer = (  uVBoxMajor  >= 5)
-             ? 2  /* VBox 5.0 and up: Protocol version 2. */
-             : 1; /* VBox <= 4.3:     Protocol version 1. */
-        /* Build revision is ignored. */
+        if (true)
+            uProto = 3;
+        else
 #endif
+        if (uVerAdditions >= VBOX_FULL_VERSION_MAKE(5, 0, 0))
+        {
+/** @todo
+ *  r=bird: This is just too bad for anyone using an OSE additions build...
+ */
+            if (uRevAdditions >= 103344) /* Since r103344: Protocol v3. */
+                uProto = 3;
+            else
+                uProto = 2; /* VBox 5.0.0 - 5.0.8: Protocol v2. */
+        }
+        /* else: uProto: 0 */
 
-        LogFlowThisFunc(("uVerAdditions=%RU32 (%RU32.%RU32)\n", uVerAdditions, uVBoxMajor, uVBoxMinor));
+        LogFlowFunc(("uVerAdditions=%RU32 (%RU32.%RU32.%RU32), r%RU32\n",
+                     uVerAdditions, VBOX_FULL_VERSION_GET_MAJOR(uVerAdditions), VBOX_FULL_VERSION_GET_MINOR(uVerAdditions),
+                                    VBOX_FULL_VERSION_GET_BUILD(uVerAdditions), uRevAdditions));
         rc = VINF_SUCCESS;
     }
     else
     {
-        uVer = 1; /* Fallback. */
+        uProto = 1; /* Fallback. */
         rc = VERR_NOT_FOUND;
     }
 
-    LogFlowThisFunc(("uVer=%RU32, uVerAdditions=%RU32, rc=%Rrc\n", uVer, uVerAdditions, rc));
+    LogRel2(("DnD: Guest is using protocol v%RU32, rc=%Rrc\n", uProto, rc));
 
-    *puVersion = uVer;
+    *puProto = uProto;
     return rc;
 }
 
 int GuestDnDBase::msgQueueAdd(GuestDnDMsg *pMsg)
 {
-    mDataBase.mListOutgoing.push_back(pMsg);
+    mDataBase.m_lstMsgOut.push_back(pMsg);
     return VINF_SUCCESS;
 }
 
 GuestDnDMsg *GuestDnDBase::msgQueueGetNext(void)
 {
-    if (mDataBase.mListOutgoing.empty())
+    if (mDataBase.m_lstMsgOut.empty())
         return NULL;
-    return mDataBase.mListOutgoing.front();
+    return mDataBase.m_lstMsgOut.front();
 }
 
 void GuestDnDBase::msgQueueRemoveNext(void)
 {
-    if (!mDataBase.mListOutgoing.empty())
+    if (!mDataBase.m_lstMsgOut.empty())
     {
-        GuestDnDMsg *pMsg = mDataBase.mListOutgoing.front();
+        GuestDnDMsg *pMsg = mDataBase.m_lstMsgOut.front();
         if (pMsg)
             delete pMsg;
-        mDataBase.mListOutgoing.pop_front();
+        mDataBase.m_lstMsgOut.pop_front();
     }
 }
 
 void GuestDnDBase::msgQueueClear(void)
 {
-    GuestDnDMsgList::iterator itMsg = mDataBase.mListOutgoing.begin();
-    while (itMsg != mDataBase.mListOutgoing.end())
+    LogFlowFunc(("cMsg=%zu\n", mDataBase.m_lstMsgOut.size()));
+
+    GuestDnDMsgList::iterator itMsg = mDataBase.m_lstMsgOut.begin();
+    while (itMsg != mDataBase.m_lstMsgOut.end())
     {
-        delete *itMsg;
+        GuestDnDMsg *pMsg = *itMsg;
+        if (pMsg)
+            delete pMsg;
+
+        itMsg++;
     }
 
-    mDataBase.mListOutgoing.clear();
+    mDataBase.m_lstMsgOut.clear();
 }
 
 int GuestDnDBase::sendCancel(void)
 {
-    int rc;
-    try
-    {
-        GuestDnDMsg *pMsgCancel = new GuestDnDMsg();
-        pMsgCancel->setType(DragAndDropSvc::HOST_DND_HG_EVT_CANCEL);
-
-        rc = msgQueueAdd(pMsgCancel);
-    }
-    catch(std::bad_alloc & /*e*/)
-    {
-        rc = VERR_NO_MEMORY;
-    }
+    int rc = GuestDnDInst()->hostCall(HOST_DND_HG_EVT_CANCEL,
+                                      0 /* cParms */, NULL /* paParms */);
 
     LogFlowFunc(("Generated cancelling request, rc=%Rrc\n", rc));
     return rc;
 }
 
-/** @todo GuestDnDResponse *pResp needs to go. */
-int GuestDnDBase::waitForEvent(RTMSINTERVAL msTimeout, GuestDnDCallbackEvent &Event, GuestDnDResponse *pResp)
+int GuestDnDBase::updateProgress(GuestDnDData *pData, GuestDnDResponse *pResp,
+                                 uint32_t cbDataAdd /* = 0 */)
 {
+    AssertPtrReturn(pData, VERR_INVALID_POINTER);
+    AssertPtrReturn(pResp, VERR_INVALID_POINTER);
+    /* cbDataAdd is optional. */
+
+    LogFlowFunc(("cbTotal=%RU64, cbProcessed=%RU64, cbRemaining=%RU64, cbDataAdd=%RU32\n",
+                 pData->getTotal(), pData->getProcessed(), pData->getRemaining(), cbDataAdd));
+
+    if (!pResp)
+        return VINF_SUCCESS;
+
+    if (cbDataAdd)
+        pData->addProcessed(cbDataAdd);
+
+    int rc = pResp->setProgress(pData->getPercentComplete(),
+                                  pData->isComplete()
+                                ? DND_PROGRESS_COMPLETE
+                                : DND_PROGRESS_RUNNING);
+    LogFlowFuncLeaveRC(rc);
+    return rc;
+}
+
+/** @todo GuestDnDResponse *pResp needs to go. */
+int GuestDnDBase::waitForEvent(GuestDnDCallbackEvent *pEvent, GuestDnDResponse *pResp, RTMSINTERVAL msTimeout)
+{
+    AssertPtrReturn(pEvent, VERR_INVALID_POINTER);
+    AssertPtrReturn(pResp, VERR_INVALID_POINTER);
+
     int rc;
 
+    LogFlowFunc(("msTimeout=%RU32\n", msTimeout));
+
     uint64_t tsStart = RTTimeMilliTS();
     do
     {
@@ -874,10 +970,10 @@ int GuestDnDBase::waitForEvent(RTMSINTERVAL msTimeout, GuestDnDCallbackEvent &Ev
          * wait event. As we don't want to block if the guest does not
          * respond, do busy waiting here.
          */
-        rc = Event.Wait(500 /* ms */);
+        rc = pEvent->Wait(500 /* ms */);
         if (RT_SUCCESS(rc))
         {
-            rc = Event.Result();
+            rc = pEvent->Result();
             LogFlowFunc(("Callback done, result is %Rrc\n", rc));
             break;
         }
