@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,6 +15,9 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#define LOG_GROUP LOG_GROUP_MAIN_GUEST
+#include "LoggingNew.h"
+
 #include "GuestImpl.h"
 #ifdef VBOX_WITH_GUEST_CONTROL
 # include "GuestSessionImpl.h"
@@ -28,7 +31,6 @@
 #include "VMMDev.h"
 
 #include "AutoCaller.h"
-#include "Logging.h"
 #include "Performance.h"
 #include "VBoxEvents.h"
 
@@ -205,9 +207,11 @@ DECLCALLBACK(void) Guest::i_staticUpdateStats(RTTIMERLR hTimerLR, void *pvUser,
 }
 
 /* static */
-int Guest::i_staticEnumStatsCallback(const char *pszName, STAMTYPE enmType, void *pvSample, STAMUNIT enmUnit,
-                                     STAMVISIBILITY enmVisiblity, const char *pszDesc, void *pvUser)
+DECLCALLBACK(int)  Guest::i_staticEnumStatsCallback(const char *pszName, STAMTYPE enmType, void *pvSample,
+                                                    STAMUNIT enmUnit, STAMVISIBILITY enmVisiblity,
+                                                    const char *pszDesc, void *pvUser)
 {
+    RT_NOREF(enmVisiblity, pszDesc);
     AssertLogRelMsgReturn(enmType == STAMTYPE_COUNTER, ("Unexpected sample type %d ('%s')\n", enmType, pszName), VINF_SUCCESS);
     AssertLogRelMsgReturn(enmUnit == STAMUNIT_BYTES, ("Unexpected sample unit %d ('%s')\n", enmUnit, pszName), VINF_SUCCESS);
 
@@ -256,6 +260,8 @@ int Guest::i_staticEnumStatsCallback(const char *pszName, STAMTYPE enmType, void
 
 void Guest::i_updateStats(uint64_t iTick)
 {
+    RT_NOREF(iTick);
+
     uint64_t cbFreeTotal      = 0;
     uint64_t cbAllocTotal     = 0;
     uint64_t cbBalloonedTotal = 0;
@@ -487,6 +493,7 @@ HRESULT Guest::getAdditionsRevision(ULONG *aAdditionsRevision)
 HRESULT Guest::getDnDSource(ComPtr<IGuestDnDSource> &aDnDSource)
 {
 #ifndef VBOX_WITH_DRAG_AND_DROP
+    RT_NOREF(aDnDSource);
     ReturnComNotImplemented();
 #else
     LogFlowThisFuncEnter();
@@ -502,6 +509,7 @@ HRESULT Guest::getDnDSource(ComPtr<IGuestDnDSource> &aDnDSource)
 HRESULT Guest::getDnDTarget(ComPtr<IGuestDnDTarget> &aDnDTarget)
 {
 #ifndef VBOX_WITH_DRAG_AND_DROP
+    RT_NOREF(aDnDTarget);
     ReturnComNotImplemented();
 #else
     LogFlowThisFuncEnter();
@@ -735,9 +743,10 @@ HRESULT Guest::i_setStatistic(ULONG aCpuId, GUESTSTATTYPE enmType, ULONG aVal)
 /**
  * Returns the status of a specified Guest Additions facility.
  *
- * @return  aStatus         Current status of specified facility.
- * @param   aType           Facility to get the status from.
+ * @return  COM status code
+ * @param   aFacility       Facility to get the status from.
  * @param   aTimestamp      Timestamp of last facility status update in ms (optional).
+ * @param   aStatus         Current status of the specified facility.
  */
 HRESULT Guest::getFacilityStatus(AdditionsFacilityType_T aFacility, LONG64 *aTimestamp, AdditionsFacilityStatus_T *aStatus)
 {
@@ -842,7 +851,7 @@ HRESULT Guest::setCredentials(const com::Utf8Str &aUserName, const com::Utf8Str
  * @param aInterfaceVersion
  * @param aOsType
  */
-void Guest::i_setAdditionsInfo(com::Utf8Str aInterfaceVersion, VBOXOSTYPE aOsType)
+void Guest::i_setAdditionsInfo(const com::Utf8Str &aInterfaceVersion, VBOXOSTYPE aOsType)
 {
     RTTIMESPEC TimeSpecTS;
     RTTimeNow(&TimeSpecTS);
@@ -852,7 +861,6 @@ void Guest::i_setAdditionsInfo(com::Utf8Str aInterfaceVersion, VBOXOSTYPE aOsTyp
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-
     /*
      * Note: The Guest Additions API (interface) version is deprecated
      * and will not be used anymore!  We might need it to at least report
@@ -887,7 +895,7 @@ void Guest::i_setAdditionsInfo(com::Utf8Str aInterfaceVersion, VBOXOSTYPE aOsTyp
              * "graphics" (feature) facility to active as soon as we got the Guest Additions
              * interface version.
              */
-            i_facilityUpdate(VBoxGuestFacilityType_Graphics, VBoxGuestFacilityStatus_Active,  0 /*fFlags*/, &TimeSpecTS);
+            i_facilityUpdate(VBoxGuestFacilityType_Graphics, VBoxGuestFacilityStatus_Active, 0 /*fFlags*/, &TimeSpecTS);
         }
     }
 
@@ -919,6 +927,14 @@ void Guest::i_setAdditionsInfo(com::Utf8Str aInterfaceVersion, VBOXOSTYPE aOsTyp
      */
     mData.mOSType = aOsType;
     mData.mOSTypeId = Global::OSTypeId(aOsType);
+
+    /*
+     * Always fire an event here.
+     */
+    AdditionsRunLevelType_T const enmRunLevel = mData.mAdditionsRunLevel;
+    alock.release();
+    fireGuestAdditionsStatusChangedEvent(mEventSource, AdditionsFacilityType_None, AdditionsFacilityStatus_Active,
+                                         enmRunLevel, RTTimeSpecGetMilli(&TimeSpecTS));
 }
 
 /**
@@ -977,17 +993,18 @@ bool Guest::i_facilityIsActive(VBoxGuestFacilityType enmFacility)
     return false;
 }
 
-void Guest::i_facilityUpdate(VBoxGuestFacilityType a_enmFacility, VBoxGuestFacilityStatus a_enmStatus,
+bool Guest::i_facilityUpdate(VBoxGuestFacilityType a_enmFacility, VBoxGuestFacilityStatus a_enmStatus,
                              uint32_t a_fFlags, PCRTTIMESPEC a_pTimeSpecTS)
 {
-    AssertReturnVoid(   a_enmFacility < VBoxGuestFacilityType_All
-                     && a_enmFacility > VBoxGuestFacilityType_Unknown);
+    AssertReturn(   a_enmFacility < VBoxGuestFacilityType_All
+                 && a_enmFacility > VBoxGuestFacilityType_Unknown, false);
 
+    bool fChanged;
     FacilityMapIter it = mData.mFacilityMap.find((AdditionsFacilityType_T)a_enmFacility);
     if (it != mData.mFacilityMap.end())
     {
         AdditionsFacility *pFac = it->second;
-        pFac->i_update((AdditionsFacilityStatus_T)a_enmStatus, a_fFlags, a_pTimeSpecTS);
+        fChanged = pFac->i_update((AdditionsFacilityStatus_T)a_enmStatus, a_fFlags, a_pTimeSpecTS);
     }
     else
     {
@@ -995,18 +1012,20 @@ void Guest::i_facilityUpdate(VBoxGuestFacilityType a_enmFacility, VBoxGuestFacil
         {
             /* The easy way out for now. We could automatically destroy
                inactive facilities like VMMDev does if we like... */
-            AssertFailedReturnVoid();
+            AssertFailedReturn(false);
         }
 
         ComObjPtr<AdditionsFacility> ptrFac;
         ptrFac.createObject();
-        AssertReturnVoid(!ptrFac.isNull());
+        AssertReturn(!ptrFac.isNull(), false);
 
         HRESULT hrc = ptrFac->init(this, (AdditionsFacilityType_T)a_enmFacility, (AdditionsFacilityStatus_T)a_enmStatus,
                                    a_fFlags, a_pTimeSpecTS);
-        if (SUCCEEDED(hrc))
-            mData.mFacilityMap.insert(std::make_pair((AdditionsFacilityType_T)a_enmFacility, ptrFac));
+        AssertComRCReturn(hrc, false);
+        mData.mFacilityMap.insert(std::make_pair((AdditionsFacilityType_T)a_enmFacility, ptrFac));
+        fChanged = true;
     }
+    return fChanged;
 }
 
 /**
@@ -1022,6 +1041,7 @@ void Guest::i_facilityUpdate(VBoxGuestFacilityType a_enmFacility, VBoxGuestFacil
 void Guest::i_onUserStateChange(Bstr aUser, Bstr aDomain, VBoxGuestUserState enmState,
                                 const uint8_t *pbDetails, uint32_t cbDetails)
 {
+    RT_NOREF(pbDetails, cbDetails);
     LogFlowThisFunc(("\n"));
 
     AutoCaller autoCaller(this);
@@ -1039,7 +1059,6 @@ void Guest::i_onUserStateChange(Bstr aUser, Bstr aDomain, VBoxGuestUserState enm
  *
  * Gets called by vmmdevUpdateGuestStatus, which just passes the report along.
  *
- * @param   a_pInterface        Pointer to this interface.
  * @param   a_enmFacility       The facility.
  * @param   a_enmStatus         The status.
  * @param   a_fFlags            Flags assoicated with the update. Currently
@@ -1062,15 +1081,17 @@ void Guest::i_setAdditionsStatus(VBoxGuestFacilityType a_enmFacility, VBoxGuestF
     /*
      * Set a specific facility status.
      */
+    bool fFireEvent = false;
     if (a_enmFacility == VBoxGuestFacilityType_All)
         for (FacilityMapIter it = mData.mFacilityMap.begin(); it != mData.mFacilityMap.end(); ++it)
-            i_facilityUpdate((VBoxGuestFacilityType)it->first, a_enmStatus, a_fFlags, a_pTimeSpecTS);
+            fFireEvent |= i_facilityUpdate((VBoxGuestFacilityType)it->first, a_enmStatus, a_fFlags, a_pTimeSpecTS);
     else /* Update one facility only. */
-        i_facilityUpdate(a_enmFacility, a_enmStatus, a_fFlags, a_pTimeSpecTS);
+        fFireEvent = i_facilityUpdate(a_enmFacility, a_enmStatus, a_fFlags, a_pTimeSpecTS);
 
     /*
      * Recalc the runlevel.
      */
+    AdditionsRunLevelType_T const enmOldRunLevel = mData.mAdditionsRunLevel;
     if (i_facilityIsActive(VBoxGuestFacilityType_VBoxTrayClient))
         mData.mAdditionsRunLevel = AdditionsRunLevelType_Desktop;
     else if (i_facilityIsActive(VBoxGuestFacilityType_VBoxService))
@@ -1079,12 +1100,24 @@ void Guest::i_setAdditionsStatus(VBoxGuestFacilityType a_enmFacility, VBoxGuestF
         mData.mAdditionsRunLevel = AdditionsRunLevelType_System;
     else
         mData.mAdditionsRunLevel = AdditionsRunLevelType_None;
+
+    /*
+     * Fire event if something actually changed.
+     */
+    AdditionsRunLevelType_T const enmNewRunLevel = mData.mAdditionsRunLevel;
+    if (fFireEvent || enmNewRunLevel != enmOldRunLevel)
+    {
+        alock.release();
+        fireGuestAdditionsStatusChangedEvent(mEventSource, (AdditionsFacilityType_T)a_enmFacility,
+                                             (AdditionsFacilityStatus_T)a_enmStatus, enmNewRunLevel,
+                                             RTTimeSpecGetMilli(a_pTimeSpecTS));
+    }
 }
 
 /**
  * Sets the supported features (and whether they are active or not).
  *
- * @param   fCaps       Guest capability bit mask (VMMDEV_GUEST_SUPPORTS_XXX).
+ * @param   aCaps   Guest capability bit mask (VMMDEV_GUEST_SUPPORTS_XXX).
  */
 void Guest::i_setSupportedFeatures(uint32_t aCaps)
 {
