@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,16 +15,19 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
+#define LOG_GROUP LOG_GROUP_MAIN_MACHINEDEBUGGER
+#include "LoggingNew.h"
+
 #include "MachineDebuggerImpl.h"
 
 #include "Global.h"
 #include "ConsoleImpl.h"
 
 #include "AutoCaller.h"
-#include "Logging.h"
 
 #include <VBox/vmm/em.h>
 #include <VBox/vmm/patm.h>
@@ -69,7 +72,7 @@ void MachineDebugger::FinalRelease()
  * @returns COM result indicator
  * @param aParent handle of our parent object
  */
-HRESULT MachineDebugger::init (Console *aParent)
+HRESULT MachineDebugger::init(Console *aParent)
 {
     LogFlowThisFunc(("aParent=%p\n", aParent));
 
@@ -83,13 +86,13 @@ HRESULT MachineDebugger::init (Console *aParent)
 
     for (unsigned i = 0; i < RT_ELEMENTS(maiQueuedEmExecPolicyParams); i++)
         maiQueuedEmExecPolicyParams[i] = UINT8_MAX;
-    mSingleStepQueued = ~0;
-    mRecompileUserQueued = ~0;
-    mRecompileSupervisorQueued = ~0;
-    mPatmEnabledQueued = ~0;
-    mCsamEnabledQueued = ~0;
-    mLogEnabledQueued = ~0;
-    mVirtualTimeRateQueued = ~0;
+    mSingleStepQueued = -1;
+    mRecompileUserQueued = -1;
+    mRecompileSupervisorQueued = -1;
+    mPatmEnabledQueued = -1;
+    mCsamEnabledQueued = -1;
+    mLogEnabledQueued = -1;
+    mVirtualTimeRateQueued = UINT32_MAX;
     mFlushMode = false;
 
     /* Confirm a successful initialization */
@@ -122,7 +125,7 @@ void MachineDebugger::uninit()
  * Returns the current singlestepping flag.
  *
  * @returns COM status code
- * @param   a_fEnabled      Where to store the result.
+ * @param   aSingleStep     Where to store the result.
  */
 HRESULT MachineDebugger::getSingleStep(BOOL *aSingleStep)
 {
@@ -131,7 +134,7 @@ HRESULT MachineDebugger::getSingleStep(BOOL *aSingleStep)
     HRESULT hrc = ptrVM.rc();
     if (SUCCEEDED(hrc))
     {
-        /** @todo */
+        RT_NOREF(aSingleStep); /** @todo */
         ReturnComNotImplemented();
     }
     return hrc;
@@ -141,7 +144,7 @@ HRESULT MachineDebugger::getSingleStep(BOOL *aSingleStep)
  * Sets the singlestepping flag.
  *
  * @returns COM status code
- * @param   a_fEnable       The new state.
+ * @param   aSingleStep     The new state.
  */
 HRESULT MachineDebugger::setSingleStep(BOOL aSingleStep)
 {
@@ -150,7 +153,7 @@ HRESULT MachineDebugger::setSingleStep(BOOL aSingleStep)
     HRESULT hrc = ptrVM.rc();
     if (SUCCEEDED(hrc))
     {
-        /** @todo */
+        NOREF(aSingleStep); /** @todo */
         ReturnComNotImplemented();
     }
     return hrc;
@@ -222,7 +225,7 @@ HRESULT MachineDebugger::i_setEmExecPolicyProperty(EMEXECPOLICY enmPolicy, BOOL
  * Returns the current recompile user mode code flag.
  *
  * @returns COM status code
- * @param   a_fEnabled address of result variable
+ * @param   aRecompileUser  address of result variable
  */
 HRESULT MachineDebugger::getRecompileUser(BOOL *aRecompileUser)
 {
@@ -233,7 +236,7 @@ HRESULT MachineDebugger::getRecompileUser(BOOL *aRecompileUser)
  * Sets the recompile user mode code flag.
  *
  * @returns COM status
- * @param   aEnable new user mode code recompile flag.
+ * @param   aRecompileUser  new user mode code recompile flag.
  */
 HRESULT MachineDebugger::setRecompileUser(BOOL aRecompileUser)
 {
@@ -245,7 +248,7 @@ HRESULT MachineDebugger::setRecompileUser(BOOL aRecompileUser)
  * Returns the current recompile supervisor code flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aRecompileSupervisor    address of result variable
  */
 HRESULT MachineDebugger::getRecompileSupervisor(BOOL *aRecompileSupervisor)
 {
@@ -256,7 +259,7 @@ HRESULT MachineDebugger::getRecompileSupervisor(BOOL *aRecompileSupervisor)
  * Sets the new recompile supervisor code flag.
  *
  * @returns COM status code
- * @param   aEnable new recompile supervisor code flag
+ * @param   aRecompileSupervisor    new recompile supervisor code flag
  */
 HRESULT MachineDebugger::setRecompileSupervisor(BOOL aRecompileSupervisor)
 {
@@ -268,7 +271,7 @@ HRESULT MachineDebugger::setRecompileSupervisor(BOOL aRecompileSupervisor)
  * Returns the current execute-all-in-IEM setting.
  *
  * @returns COM status code
- * @param   aEnabled    Address of result variable.
+ * @param   aExecuteAllInIEM    Address of result variable.
  */
 HRESULT MachineDebugger::getExecuteAllInIEM(BOOL *aExecuteAllInIEM)
 {
@@ -279,7 +282,7 @@ HRESULT MachineDebugger::getExecuteAllInIEM(BOOL *aExecuteAllInIEM)
  * Changes the execute-all-in-IEM setting.
  *
  * @returns COM status code
- * @param   aEnable     New setting.
+ * @param   aExecuteAllInIEM    New setting.
  */
 HRESULT MachineDebugger::setExecuteAllInIEM(BOOL aExecuteAllInIEM)
 {
@@ -291,7 +294,7 @@ HRESULT MachineDebugger::setExecuteAllInIEM(BOOL aExecuteAllInIEM)
  * Returns the current patch manager enabled flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aPATMEnabled    address of result variable
  */
 HRESULT MachineDebugger::getPATMEnabled(BOOL *aPATMEnabled)
 {
@@ -300,7 +303,7 @@ HRESULT MachineDebugger::getPATMEnabled(BOOL *aPATMEnabled)
 
     Console::SafeVMPtrQuiet ptrVM(mParent);
     if (ptrVM.isOk())
-        *aPATMEnabled = PATMR3IsEnabled (ptrVM.rawUVM());
+        *aPATMEnabled = PATMR3IsEnabled(ptrVM.rawUVM());
     else
 #endif
         *aPATMEnabled = false;
@@ -312,7 +315,7 @@ HRESULT MachineDebugger::getPATMEnabled(BOOL *aPATMEnabled)
  * Set the new patch manager enabled flag.
  *
  * @returns COM status code
- * @param   aEnable new patch manager enabled flag
+ * @param   aPATMEnabled    new patch manager enabled flag
  */
 HRESULT MachineDebugger::setPATMEnabled(BOOL aPATMEnabled)
 {
@@ -347,7 +350,7 @@ HRESULT MachineDebugger::setPATMEnabled(BOOL aPATMEnabled)
  * Returns the current code scanner enabled flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aCSAMEnabled    address of result variable
  */
 HRESULT MachineDebugger::getCSAMEnabled(BOOL *aCSAMEnabled)
 {
@@ -369,7 +372,7 @@ HRESULT MachineDebugger::getCSAMEnabled(BOOL *aCSAMEnabled)
  * Sets the new code scanner enabled flag.
  *
  * @returns COM status code
- * @param   aEnable new code scanner enabled flag
+ * @param   aCSAMEnabled    new code scanner enabled flag
  */
 HRESULT MachineDebugger::setCSAMEnabled(BOOL aCSAMEnabled)
 {
@@ -404,7 +407,7 @@ HRESULT MachineDebugger::setCSAMEnabled(BOOL aCSAMEnabled)
  * Returns the log enabled / disabled status.
  *
  * @returns COM status code
- * @param   aEnabled     address of result variable
+ * @param   aLogEnabled     address of result variable
  */
 HRESULT MachineDebugger::getLogEnabled(BOOL *aLogEnabled)
 {
@@ -424,7 +427,7 @@ HRESULT MachineDebugger::getLogEnabled(BOOL *aLogEnabled)
  * Enables or disables logging.
  *
  * @returns COM status code
- * @param   aEnabled    The new code log state.
+ * @param   aLogEnabled    The new code log state.
  */
 HRESULT MachineDebugger::setLogEnabled(BOOL aLogEnabled)
 {
@@ -531,7 +534,7 @@ HRESULT MachineDebugger::getLogRelDestinations(com::Utf8Str &aLogRelDestinations
  * Returns the current hardware virtualization flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aHWVirtExEnabled    address of result variable
  */
 HRESULT MachineDebugger::getHWVirtExEnabled(BOOL *aHWVirtExEnabled)
 {
@@ -551,7 +554,7 @@ HRESULT MachineDebugger::getHWVirtExEnabled(BOOL *aHWVirtExEnabled)
  * Returns the current nested paging flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aHWVirtExNestedPagingEnabled    address of result variable
  */
 HRESULT MachineDebugger::getHWVirtExNestedPagingEnabled(BOOL *aHWVirtExNestedPagingEnabled)
 {
@@ -571,7 +574,7 @@ HRESULT MachineDebugger::getHWVirtExNestedPagingEnabled(BOOL *aHWVirtExNestedPag
  * Returns the current VPID flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aHWVirtExVPIDEnabled address of result variable
  */
 HRESULT MachineDebugger::getHWVirtExVPIDEnabled(BOOL *aHWVirtExVPIDEnabled)
 {
@@ -591,7 +594,7 @@ HRESULT MachineDebugger::getHWVirtExVPIDEnabled(BOOL *aHWVirtExVPIDEnabled)
  * Returns the current unrestricted execution setting.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aHWVirtExUXEnabled  address of result variable
  */
 HRESULT MachineDebugger::getHWVirtExUXEnabled(BOOL *aHWVirtExUXEnabled)
 {
@@ -627,7 +630,7 @@ HRESULT MachineDebugger::getOSName(com::Utf8Str &aOSName)
                 Bstr bstrName(szName);
                 aOSName = Utf8Str(bstrName);
             }
-            catch (std::bad_alloc)
+            catch (std::bad_alloc &)
             {
                 hrc = E_OUTOFMEMORY;
             }
@@ -658,7 +661,7 @@ HRESULT MachineDebugger::getOSVersion(com::Utf8Str &aOSVersion)
                 Bstr bstrVersion(szVersion);
                 aOSVersion = Utf8Str(bstrVersion);
             }
-            catch (std::bad_alloc)
+            catch (std::bad_alloc &)
             {
                 hrc = E_OUTOFMEMORY;
             }
@@ -673,7 +676,7 @@ HRESULT MachineDebugger::getOSVersion(com::Utf8Str &aOSVersion)
  * Returns the current PAE flag.
  *
  * @returns COM status code
- * @param   aEnabled address of result variable
+ * @param   aPAEEnabled     address of result variable.
  */
 HRESULT MachineDebugger::getPAEEnabled(BOOL *aPAEEnabled)
 {
@@ -697,7 +700,7 @@ HRESULT MachineDebugger::getPAEEnabled(BOOL *aPAEEnabled)
  * Returns the current virtual time rate.
  *
  * @returns COM status code.
- * @param   a_puPct      Where to store the rate.
+ * @param   aVirtualTimeRate    Where to store the rate.
  */
 HRESULT MachineDebugger::getVirtualTimeRate(ULONG *aVirtualTimeRate)
 {
@@ -712,10 +715,10 @@ HRESULT MachineDebugger::getVirtualTimeRate(ULONG *aVirtualTimeRate)
 }
 
 /**
- * Returns the current virtual time rate.
+ * Set the virtual time rate.
  *
  * @returns COM status code.
- * @param   aPct     Where to store the rate.
+ * @param   aVirtualTimeRate    The new rate.
  */
 HRESULT MachineDebugger::setVirtualTimeRate(ULONG aVirtualTimeRate)
 {
@@ -748,7 +751,7 @@ HRESULT MachineDebugger::setVirtualTimeRate(ULONG aVirtualTimeRate)
  * This is only temporary (promise) while prototyping the debugger.
  *
  * @returns COM status code
- * @param   a_u64Vm     Where to store the vm handle. Since there is no
+ * @param   aVM         Where to store the vm handle. Since there is no
  *                      uintptr_t in COM, we're using the max integer.
  *                      (No, ULONG is not pointer sized!)
  * @remarks The returned handle must be passed to VMR3ReleaseUVM()!
@@ -773,6 +776,24 @@ HRESULT MachineDebugger::getVM(LONG64 *aVM)
     return hrc;
 }
 
+/**
+ * Get the VM uptime in milliseconds.
+ *
+ * @returns COM status code
+ * @param   aUptime     Where to store the uptime.
+ */
+HRESULT MachineDebugger::getUptime(LONG64 *aUptime)
+{
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    Console::SafeVMPtr ptrVM(mParent);
+    HRESULT hrc = ptrVM.rc();
+    if (SUCCEEDED(hrc))
+        *aUptime = (int64_t)TMR3TimeVirtGetMilli(ptrVM.rawUVM());
+
+    return hrc;
+}
+
 // IMachineDebugger methods
 /////////////////////////////////////////////////////////////////////////////
 
@@ -798,6 +819,7 @@ HRESULT MachineDebugger::dumpGuestCore(const com::Utf8Str &aFilename, const com:
 
 HRESULT MachineDebugger::dumpHostProcessCore(const com::Utf8Str &aFilename, const com::Utf8Str &aCompression)
 {
+    RT_NOREF(aFilename, aCompression);
     ReturnComNotImplemented();
 }
 
@@ -858,23 +880,26 @@ static DECLCALLBACK(size_t) MachineDebuggerInfoOutput(void *pvArg, const char *p
     /*
      * Copy the bytes into the buffer and terminate it.
      */
-    memcpy(&pHlp->pszBuf[pHlp->offBuf], pachChars, cbChars);
-    pHlp->offBuf += cbChars;
+    if (cbChars)
+    {
+        memcpy(&pHlp->pszBuf[pHlp->offBuf], pachChars, cbChars);
+        pHlp->offBuf += cbChars;
+    }
     pHlp->pszBuf[pHlp->offBuf] = '\0';
     Assert(pHlp->offBuf < pHlp->cbBuf);
     return cbChars;
 }
 
 /**
- * @interface_method_impl{DBGFINFOHLP, pfnPrintfV}
+ * @interface_method_impl{DBGFINFOHLP,pfnPrintfV}
  */
-static DECLCALLBACK(void) MachineDebuggerInfoPrintfV(PCDBGFINFOHLP pHlp, const char *pszFormat, va_list va)
+static DECLCALLBACK(void) MachineDebuggerInfoPrintfV(PCDBGFINFOHLP pHlp, const char *pszFormat, va_list args)
 {
-    RTStrFormatV(MachineDebuggerInfoOutput, (void *)pHlp, NULL,  NULL, pszFormat, va);
+    RTStrFormatV(MachineDebuggerInfoOutput, (void *)pHlp, NULL,  NULL, pszFormat, args);
 }
 
 /**
- * @interface_method_impl{DBGFINFOHLP, pfnPrintf}
+ * @interface_method_impl{DBGFINFOHLP,pfnPrintf}
  */
 static DECLCALLBACK(void) MachineDebuggerInfoPrintf(PCDBGFINFOHLP pHlp, const char *pszFormat, ...)
 {
@@ -943,7 +968,7 @@ HRESULT MachineDebugger::info(const com::Utf8Str &aName, const com::Utf8Str &aAr
                         Bstr bstrInfo(Hlp.pszBuf);
                         aInfo = bstrInfo;
                     }
-                    catch (std::bad_alloc)
+                    catch (std::bad_alloc &)
                     {
                         hrc = E_OUTOFMEMORY;
                     }
@@ -1030,21 +1055,25 @@ HRESULT MachineDebugger::modifyLogDestinations(const com::Utf8Str &aSettings)
 
 HRESULT MachineDebugger::readPhysicalMemory(LONG64 aAddress, ULONG aSize, std::vector<BYTE> &aBytes)
 {
+    RT_NOREF(aAddress, aSize, aBytes);
     ReturnComNotImplemented();
 }
 
 HRESULT MachineDebugger::writePhysicalMemory(LONG64 aAddress, ULONG aSize, const std::vector<BYTE> &aBytes)
 {
+    RT_NOREF(aAddress, aSize, aBytes);
     ReturnComNotImplemented();
 }
 
 HRESULT MachineDebugger::readVirtualMemory(ULONG aCpuId, LONG64 aAddress, ULONG aSize, std::vector<BYTE> &aBytes)
 {
+    RT_NOREF(aCpuId, aAddress, aSize, aBytes);
     ReturnComNotImplemented();
 }
 
 HRESULT MachineDebugger::writeVirtualMemory(ULONG aCpuId, LONG64 aAddress, ULONG aSize, const std::vector<BYTE> &aBytes)
 {
+    RT_NOREF(aCpuId, aAddress, aSize, aBytes);
     ReturnComNotImplemented();
 }
 
@@ -1069,7 +1098,7 @@ HRESULT MachineDebugger::loadPlugIn(const com::Utf8Str &aName, com::Utf8Str &aPl
                 aPlugInName = "all";
                 hrc = S_OK;
             }
-            catch (std::bad_alloc)
+            catch (std::bad_alloc &)
             {
                 hrc = E_OUTOFMEMORY;
             }
@@ -1083,10 +1112,10 @@ HRESULT MachineDebugger::loadPlugIn(const com::Utf8Str &aName, com::Utf8Str &aPl
             {
                 try
                 {
-                    aPlugInName = "all";
+                    aPlugInName = szName;
                     hrc = S_OK;
                 }
-                catch (std::bad_alloc)
+                catch (std::bad_alloc &)
                 {
                     hrc = E_OUTOFMEMORY;
                 }
@@ -1155,7 +1184,7 @@ HRESULT MachineDebugger::detectOS(com::Utf8Str &aOs)
             {
                 aOs = szName;
             }
-            catch (std::bad_alloc)
+            catch (std::bad_alloc &)
             {
                 hrc = E_OUTOFMEMORY;
             }
@@ -1259,7 +1288,7 @@ HRESULT MachineDebugger::getRegister(ULONG aCpuId, const com::Utf8Str &aName, co
                 if (SUCCEEDED(hrc))
                     aValue = Utf8Str(bstrValue);
             }
-            catch (std::bad_alloc)
+            catch (std::bad_alloc &)
             {
                 hrc = E_OUTOFMEMORY;
             }
@@ -1279,6 +1308,8 @@ HRESULT MachineDebugger::getRegister(ULONG aCpuId, const com::Utf8Str &aName, co
 
 HRESULT MachineDebugger::getRegisters(ULONG aCpuId, std::vector<com::Utf8Str> &aNames, std::vector<com::Utf8Str> &aValues)
 {
+    RT_NOREF(aCpuId); /** @todo fix missing aCpuId usage! */
+
     /*
      * The prologue.
      */
@@ -1311,12 +1342,12 @@ HRESULT MachineDebugger::getRegisters(ULONG aCpuId, std::vector<com::Utf8Str> &a
                             szHex[159] = szHex[0] = '\0';
                             ssize_t cch = DBGFR3RegFormatValue(szHex, sizeof(szHex), &paRegs[iReg].Val,
                                                                paRegs[iReg].enmType, true /*fSpecial*/);
-                            Assert(cch > 0);
+                            Assert(cch > 0); NOREF(cch);
                             aNames[iReg] = Utf8Str(paRegs[iReg].pszName);
                             aValues[iReg] = Utf8Str(szHex);
                         }
                     }
-                    catch (std::bad_alloc)
+                    catch (std::bad_alloc &)
                     {
                         hrc = E_OUTOFMEMORY;
                     }
@@ -1337,18 +1368,158 @@ HRESULT MachineDebugger::getRegisters(ULONG aCpuId, std::vector<com::Utf8Str> &a
 
 HRESULT MachineDebugger::setRegister(ULONG aCpuId, const com::Utf8Str &aName, const com::Utf8Str &aValue)
 {
+    RT_NOREF(aCpuId, aName, aValue);
     ReturnComNotImplemented();
 }
 
 HRESULT MachineDebugger::setRegisters(ULONG aCpuId, const std::vector<com::Utf8Str> &aNames,
                                       const std::vector<com::Utf8Str> &aValues)
 {
+    RT_NOREF(aCpuId, aNames, aValues);
     ReturnComNotImplemented();
 }
 
 HRESULT MachineDebugger::dumpGuestStack(ULONG aCpuId, com::Utf8Str &aStack)
 {
-    ReturnComNotImplemented();
+    /*
+     * The prologue.
+     */
+    LogFlowThisFunc(("\n"));
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+    Console::SafeVMPtr ptrVM(mParent);
+    HRESULT hrc = ptrVM.rc();
+    if (SUCCEEDED(hrc))
+    {
+        /*
+         * There is currently a problem with the windows diggers and SMP, where
+         * guest driver memory is being read from CPU zero in order to ensure that
+         * we've got a consisten virtual memory view.  If one of the other CPUs
+         * initiates a rendezvous while we're unwinding the stack and trying to
+         * read guest driver memory, we will deadlock.
+         *
+         * So, check the VM state and maybe suspend the VM before we continue.
+         */
+        int  vrc     = VINF_SUCCESS;
+        bool fPaused = false;
+        if (aCpuId != 0)
+        {
+            VMSTATE enmVmState = VMR3GetStateU(ptrVM.rawUVM());
+            if (   enmVmState == VMSTATE_RUNNING
+                || enmVmState == VMSTATE_RUNNING_LS
+                || enmVmState == VMSTATE_RUNNING_FT)
+            {
+                alock.release();
+                vrc = VMR3Suspend(ptrVM.rawUVM(), VMSUSPENDREASON_USER);
+                alock.acquire();
+                fPaused = RT_SUCCESS(vrc);
+            }
+        }
+        if (RT_SUCCESS(vrc))
+        {
+            PCDBGFSTACKFRAME pFirstFrame;
+            vrc = DBGFR3StackWalkBegin(ptrVM.rawUVM(), aCpuId, DBGFCODETYPE_GUEST, &pFirstFrame);
+            if (RT_SUCCESS(vrc))
+            {
+                /*
+                 * Print header.
+                 */
+                try
+                {
+                    uint32_t fBitFlags = 0;
+                    for (PCDBGFSTACKFRAME pFrame = pFirstFrame;
+                         pFrame;
+                         pFrame = DBGFR3StackWalkNext(pFrame))
+                    {
+                        uint32_t const fCurBitFlags = pFrame->fFlags & (DBGFSTACKFRAME_FLAGS_16BIT | DBGFSTACKFRAME_FLAGS_32BIT | DBGFSTACKFRAME_FLAGS_64BIT);
+                        if (fCurBitFlags & DBGFSTACKFRAME_FLAGS_16BIT)
+                        {
+                            if (fCurBitFlags != fBitFlags)
+                                aStack.append("SS:BP     Ret SS:BP Ret CS:EIP    Arg0     Arg1     Arg2     Arg3     CS:EIP / Symbol [line]\n");
+                            aStack.append(Utf8StrFmt("%04RX16:%04RX16 %04RX16:%04RX16 %04RX32:%08RX32 %08RX32 %08RX32 %08RX32 %08RX32",
+                                                     pFrame->AddrFrame.Sel,
+                                                     (uint16_t)pFrame->AddrFrame.off,
+                                                     pFrame->AddrReturnFrame.Sel,
+                                                     (uint16_t)pFrame->AddrReturnFrame.off,
+                                                     (uint32_t)pFrame->AddrReturnPC.Sel,
+                                                     (uint32_t)pFrame->AddrReturnPC.off,
+                                                     pFrame->Args.au32[0],
+                                                     pFrame->Args.au32[1],
+                                                     pFrame->Args.au32[2],
+                                                     pFrame->Args.au32[3]));
+                        }
+                        else if (fCurBitFlags & DBGFSTACKFRAME_FLAGS_32BIT)
+                        {
+                            if (fCurBitFlags != fBitFlags)
+                                aStack.append("EBP      Ret EBP  Ret CS:EIP    Arg0     Arg1     Arg2     Arg3     CS:EIP / Symbol [line]\n");
+                            aStack.append(Utf8StrFmt("%08RX32 %08RX32 %04RX32:%08RX32 %08RX32 %08RX32 %08RX32 %08RX32",
+                                                     (uint32_t)pFrame->AddrFrame.off,
+                                                     (uint32_t)pFrame->AddrReturnFrame.off,
+                                                     (uint32_t)pFrame->AddrReturnPC.Sel,
+                                                     (uint32_t)pFrame->AddrReturnPC.off,
+                                                     pFrame->Args.au32[0],
+                                                     pFrame->Args.au32[1],
+                                                     pFrame->Args.au32[2],
+                                                     pFrame->Args.au32[3]));
+                        }
+                        else if (fCurBitFlags & DBGFSTACKFRAME_FLAGS_64BIT)
+                        {
+                            if (fCurBitFlags != fBitFlags)
+                                aStack.append("RBP              Ret SS:RBP            Ret RIP          CS:RIP / Symbol [line]\n");
+                            aStack.append(Utf8StrFmt("%016RX64 %04RX16:%016RX64 %016RX64",
+                                                     (uint64_t)pFrame->AddrFrame.off,
+                                                     pFrame->AddrReturnFrame.Sel,
+                                                     (uint64_t)pFrame->AddrReturnFrame.off,
+                                                     (uint64_t)pFrame->AddrReturnPC.off));
+                        }
+
+                        if (!pFrame->pSymPC)
+                            aStack.append(Utf8StrFmt(fCurBitFlags & DBGFSTACKFRAME_FLAGS_64BIT
+                                                     ? " %RTsel:%016RGv"
+                                                     : fCurBitFlags & DBGFSTACKFRAME_FLAGS_32BIT
+                                                     ? " %RTsel:%08RGv"
+                                                     : " %RTsel:%04RGv"
+                                                     , pFrame->AddrPC.Sel, pFrame->AddrPC.off));
+                        else
+                        {
+                            RTGCINTPTR offDisp = pFrame->AddrPC.FlatPtr - pFrame->pSymPC->Value; /** @todo this isn't 100% correct for segmented stuff. */
+                            if (offDisp > 0)
+                                aStack.append(Utf8StrFmt(" %s+%llx", pFrame->pSymPC->szName, (int64_t)offDisp));
+                            else if (offDisp < 0)
+                                aStack.append(Utf8StrFmt(" %s-%llx", pFrame->pSymPC->szName, -(int64_t)offDisp));
+                            else
+                                aStack.append(Utf8StrFmt(" %s", pFrame->pSymPC->szName));
+                        }
+                        if (pFrame->pLinePC)
+                            aStack.append(Utf8StrFmt(" [%s @ 0i%d]", pFrame->pLinePC->szFilename, pFrame->pLinePC->uLineNo));
+                        aStack.append(Utf8StrFmt("\n"));
+
+                        fBitFlags = fCurBitFlags;
+                    }
+                }
+                catch (std::bad_alloc &)
+                {
+                    hrc = E_OUTOFMEMORY;
+                }
+
+                DBGFR3StackWalkEnd(pFirstFrame);
+            }
+            else
+                hrc = setError(E_FAIL, tr("DBGFR3StackWalkBegin failed with %Rrc"), vrc);
+
+            /*
+             * Resume the VM if we suspended it.
+             */
+            if (fPaused)
+            {
+                alock.release();
+                VMR3Resume(ptrVM.rawUVM(), VMRESUMEREASON_USER);
+            }
+        }
+        else
+            hrc = setError(E_FAIL, tr("Suspending the VM failed with %Rrc\n"), vrc);
+    }
+
+    return hrc;
 }
 
 /**
@@ -1397,7 +1568,7 @@ HRESULT MachineDebugger::dumpStats(const com::Utf8Str &aPattern)
  */
 HRESULT MachineDebugger::getStats(const com::Utf8Str &aPattern, BOOL aWithDescriptions, com::Utf8Str &aStats)
 {
-    Console::SafeVMPtrQuiet ptrVM (mParent);
+    Console::SafeVMPtrQuiet ptrVM(mParent);
 
     if (!ptrVM.isOk())
         return setError(VBOX_E_INVALID_VM_STATE, "Machine is not running");
@@ -1425,10 +1596,10 @@ HRESULT MachineDebugger::getStats(const com::Utf8Str &aPattern, BOOL aWithDescri
 void MachineDebugger::i_flushQueuedSettings()
 {
     mFlushMode = true;
-    if (mSingleStepQueued != ~0)
+    if (mSingleStepQueued != -1)
     {
         COMSETTER(SingleStep)(mSingleStepQueued);
-        mSingleStepQueued = ~0;
+        mSingleStepQueued = -1;
     }
     for (unsigned i = 0; i < EMEXECPOLICY_END; i++)
         if (maiQueuedEmExecPolicyParams[i] != UINT8_MAX)
@@ -1436,25 +1607,25 @@ void MachineDebugger::i_flushQueuedSettings()
             i_setEmExecPolicyProperty((EMEXECPOLICY)i, RT_BOOL(maiQueuedEmExecPolicyParams[i]));
             maiQueuedEmExecPolicyParams[i] = UINT8_MAX;
         }
-    if (mPatmEnabledQueued != ~0)
+    if (mPatmEnabledQueued != -1)
     {
         COMSETTER(PATMEnabled)(mPatmEnabledQueued);
-        mPatmEnabledQueued = ~0;
+        mPatmEnabledQueued = -1;
     }
-    if (mCsamEnabledQueued != ~0)
+    if (mCsamEnabledQueued != -1)
     {
         COMSETTER(CSAMEnabled)(mCsamEnabledQueued);
-        mCsamEnabledQueued = ~0;
+        mCsamEnabledQueued = -1;
     }
-    if (mLogEnabledQueued != ~0)
+    if (mLogEnabledQueued != -1)
     {
         COMSETTER(LogEnabled)(mLogEnabledQueued);
-        mLogEnabledQueued = ~0;
+        mLogEnabledQueued = -1;
     }
-    if (mVirtualTimeRateQueued != ~(uint32_t)0)
+    if (mVirtualTimeRateQueued != UINT32_MAX)
     {
         COMSETTER(VirtualTimeRate)(mVirtualTimeRateQueued);
-        mVirtualTimeRateQueued = ~0;
+        mVirtualTimeRateQueued = UINT32_MAX;
     }
     mFlushMode = false;
 }
