@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2013 Oracle Corporation
+ * Copyright (C) 2011-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,32 +17,39 @@
 
 
 #define LOG_GROUP LOG_GROUP_USB_WEBCAM
+#include "LoggingNew.h"
+
 #include "UsbWebcamInterface.h"
 #include "ConsoleImpl.h"
 #include "ConsoleVRDPServer.h"
+#include "EmulatedUSBImpl.h"
 
 #include <VBox/vmm/pdmwebcaminfs.h>
 
 
-typedef struct EMWEBCAMDRV
-{
-    EmWebcam *pEmWebcam;
-    PPDMIWEBCAMUP  pIWebcamUp;
-    PDMIWEBCAMDOWN IWebcamDown;
-} EMWEBCAMDRV, *PEMWEBCAMDRV;
-
 typedef struct EMWEBCAMREMOTE
 {
     EmWebcam *pEmWebcam;
 
     VRDEVIDEOINDEVICEHANDLE deviceHandle; /* The remote identifier. */
+
+    /* Received from the remote client. */
     uint32_t u32Version;                  /* VRDE_VIDEOIN_NEGOTIATE_VERSION */
     uint32_t fu32Capabilities;            /* VRDE_VIDEOIN_NEGOTIATE_CAP_* */
+    VRDEVIDEOINDEVICEDESC *pDeviceDesc;
+    uint32_t cbDeviceDesc;
 
     /* The device identifier for the PDM device.*/
     uint64_t u64DeviceId;
 } EMWEBCAMREMOTE;
 
+typedef struct EMWEBCAMDRV
+{
+    EMWEBCAMREMOTE *pRemote;
+    PPDMIWEBCAMDEV  pIWebcamUp;
+    PDMIWEBCAMDRV   IWebcamDrv;
+} EMWEBCAMDRV, *PEMWEBCAMDRV;
+
 typedef struct EMWEBCAMREQCTX
 {
     EMWEBCAMREMOTE *pRemote;
@@ -50,23 +57,45 @@ typedef struct EMWEBCAMREQCTX
 } EMWEBCAMREQCTX;
 
 
-static DECLCALLBACK(int) drvEmWebcamControl(PPDMIWEBCAMDOWN pInterface,
+static DECLCALLBACK(void) drvEmWebcamReady(PPDMIWEBCAMDRV pInterface,
+                                           bool fReady)
+{
+    NOREF(fReady);
+
+    PEMWEBCAMDRV pThis = RT_FROM_MEMBER(pInterface, EMWEBCAMDRV, IWebcamDrv);
+    EMWEBCAMREMOTE *pRemote = pThis->pRemote;
+
+    LogFlowFunc(("pRemote:%p\n", pThis->pRemote));
+
+    if (pThis->pIWebcamUp)
+    {
+        pThis->pIWebcamUp->pfnAttached(pThis->pIWebcamUp,
+                                       pRemote->u64DeviceId,
+                                       pRemote->pDeviceDesc,
+                                       pRemote->cbDeviceDesc,
+                                       pRemote->u32Version,
+                                       pRemote->fu32Capabilities);
+    }
+}
+
+static DECLCALLBACK(int) drvEmWebcamControl(PPDMIWEBCAMDRV pInterface,
                                             void *pvUser,
                                             uint64_t u64DeviceId,
-                                            const PDMIWEBCAM_CTRLHDR *pCtrl,
+                                            const struct VRDEVIDEOINCTRLHDR *pCtrl,
                                             uint32_t cbCtrl)
 {
-    PEMWEBCAMDRV pThis = RT_FROM_MEMBER(pInterface, EMWEBCAMDRV, IWebcamDown);
+    PEMWEBCAMDRV pThis = RT_FROM_MEMBER(pInterface, EMWEBCAMDRV, IWebcamDrv);
+    EMWEBCAMREMOTE *pRemote = pThis->pRemote;
 
-    LogFlowFunc(("pEmWebcam:%p, u64DeviceId %lld\n", pThis->pEmWebcam, u64DeviceId));
+    LogFlowFunc(("pRemote:%p, u64DeviceId %lld\n", pRemote, u64DeviceId));
 
-    return pThis->pEmWebcam->SendControl(pThis, pvUser, u64DeviceId, (const VRDEVIDEOINCTRLHDR *)pCtrl, cbCtrl);
+    return pRemote->pEmWebcam->SendControl(pThis, pvUser, u64DeviceId, pCtrl, cbCtrl);
 }
 
 
-EmWebcam::EmWebcam(Console *console)
+EmWebcam::EmWebcam(ConsoleVRDPServer *pServer)
     :
-    mParent(console),
+    mParent(pServer),
     mpDrv(NULL),
     mpRemote(NULL),
     mu64DeviceIdSrc(0)
@@ -77,7 +106,7 @@ EmWebcam::~EmWebcam()
 {
     if (mpDrv)
     {
-        mpDrv->pEmWebcam = NULL;
+        mpDrv->pRemote = NULL;
         mpDrv = NULL;
     }
 }
@@ -95,11 +124,17 @@ void EmWebcam::EmWebcamDestruct(EMWEBCAMDRV *pDrv)
 
     if (mpRemote)
     {
-        mParent->consoleVRDPServer()->VideoInDeviceDetach(&mpRemote->deviceHandle);
+        mParent->VideoInDeviceDetach(&mpRemote->deviceHandle);
+
+        RTMemFree(mpRemote->pDeviceDesc);
+        mpRemote->pDeviceDesc = NULL;
+        mpRemote->cbDeviceDesc = 0;
+
         RTMemFree(mpRemote);
         mpRemote = NULL;
     }
 
+    mpDrv->pRemote = NULL;
     mpDrv = NULL;
 }
 
@@ -117,15 +152,11 @@ void EmWebcam::EmWebcamCbNotify(uint32_t u32Id, const void *pvData, uint32_t cbD
             uint32_t u32Version = 1;
             uint32_t fu32Capabilities = VRDE_VIDEOIN_NEGOTIATE_CAP_VOID;
 
-            if (cbData >= RT_OFFSETOF(VRDEVIDEOINNOTIFYATTACH, u32Version) + sizeof(p->u32Version))
-            {
+            if (cbData >= RT_UOFFSETOF(VRDEVIDEOINNOTIFYATTACH, u32Version) + sizeof(p->u32Version))
                 u32Version = p->u32Version;
-            }
 
-            if (cbData >= RT_OFFSETOF(VRDEVIDEOINNOTIFYATTACH, fu32Capabilities) + sizeof(p->fu32Capabilities))
-            {
+            if (cbData >= RT_UOFFSETOF(VRDEVIDEOINNOTIFYATTACH, fu32Capabilities) + sizeof(p->fu32Capabilities))
                 fu32Capabilities = p->fu32Capabilities;
-            }
 
             LogFlowFunc(("ATTACH[%d,%d] version %d, caps 0x%08X\n",
                          p->deviceHandle.u32ClientId, p->deviceHandle.u32DeviceId,
@@ -150,12 +181,14 @@ void EmWebcam::EmWebcamCbNotify(uint32_t u32Id, const void *pvData, uint32_t cbD
             pRemote->deviceHandle     = p->deviceHandle;
             pRemote->u32Version       = u32Version;
             pRemote->fu32Capabilities = fu32Capabilities;
+            pRemote->pDeviceDesc      = NULL;
+            pRemote->cbDeviceDesc     = 0;
             pRemote->u64DeviceId      = ASMAtomicIncU64(&mu64DeviceIdSrc);
 
             mpRemote = pRemote;
 
             /* Tell the server that this webcam will be used. */
-            rc = mParent->consoleVRDPServer()->VideoInDeviceAttach(&mpRemote->deviceHandle, mpRemote);
+            rc = mParent->VideoInDeviceAttach(&mpRemote->deviceHandle, mpRemote);
             if (RT_FAILURE(rc))
             {
                 RTMemFree(mpRemote);
@@ -164,11 +197,11 @@ void EmWebcam::EmWebcamCbNotify(uint32_t u32Id, const void *pvData, uint32_t cbD
             }
 
             /* Get the device description. */
-            rc = mParent->consoleVRDPServer()->VideoInGetDeviceDesc(NULL, &mpRemote->deviceHandle);
+            rc = mParent->VideoInGetDeviceDesc(NULL, &mpRemote->deviceHandle);
 
             if (RT_FAILURE(rc))
             {
-                mParent->consoleVRDPServer()->VideoInDeviceDetach(&mpRemote->deviceHandle);
+                mParent->VideoInDeviceDetach(&mpRemote->deviceHandle);
                 RTMemFree(mpRemote);
                 mpRemote = NULL;
                 break;
@@ -179,23 +212,17 @@ void EmWebcam::EmWebcamCbNotify(uint32_t u32Id, const void *pvData, uint32_t cbD
 
         case VRDE_VIDEOIN_NOTIFY_ID_DETACH:
         {
-            VRDEVIDEOINNOTIFYDETACH *p = (VRDEVIDEOINNOTIFYDETACH *)pvData;
+            VRDEVIDEOINNOTIFYDETACH *p = (VRDEVIDEOINNOTIFYDETACH *)pvData; NOREF(p);
             Assert(cbData == sizeof(VRDEVIDEOINNOTIFYDETACH));
 
             LogFlowFunc(("DETACH[%d,%d]\n", p->deviceHandle.u32ClientId, p->deviceHandle.u32DeviceId));
 
-            /* @todo */
+            /** @todo */
             if (mpRemote)
             {
                 if (mpDrv && mpDrv->pIWebcamUp)
-                {
-                    mpDrv->pIWebcamUp->pfnWebcamUpDetached(mpDrv->pIWebcamUp,
-                                                           mpRemote->u64DeviceId);
-                }
-
-                /* No need to tell the server by calling VideoInDeviceDetach because the server is telling. */
-                RTMemFree(mpRemote);
-                mpRemote = NULL;
+                    mpDrv->pIWebcamUp->pfnDetached(mpDrv->pIWebcamUp, mpRemote->u64DeviceId);
+                /* mpRemote is deallocated in EmWebcamDestruct */
             }
         } break;
 
@@ -211,6 +238,7 @@ void EmWebcam::EmWebcamCbNotify(uint32_t u32Id, const void *pvData, uint32_t cbD
 void EmWebcam::EmWebcamCbDeviceDesc(int rcRequest, void *pDeviceCtx, void *pvUser,
                                     const VRDEVIDEOINDEVICEDESC *pDeviceDesc, uint32_t cbDeviceDesc)
 {
+    RT_NOREF(pvUser);
     EMWEBCAMREMOTE *pRemote = (EMWEBCAMREMOTE *)pDeviceCtx;
     Assert(pRemote == mpRemote);
 
@@ -219,19 +247,18 @@ void EmWebcam::EmWebcamCbDeviceDesc(int rcRequest, void *pDeviceCtx, void *pvUse
 
     if (RT_SUCCESS(rcRequest))
     {
-        if (mpDrv && mpDrv->pIWebcamUp)
-        {
-            mpDrv->pIWebcamUp->pfnWebcamUpAttached(mpDrv->pIWebcamUp,
-                                                   pRemote->u64DeviceId,
-                                                   (const PDMIWEBCAM_DEVICEDESC *)pDeviceDesc,
-                                                   cbDeviceDesc,
-                                                   pRemote->u32Version,
-                                                   pRemote->fu32Capabilities);
-        }
+        /* Save device description. */
+        Assert(pRemote->pDeviceDesc == NULL);
+        pRemote->pDeviceDesc = (VRDEVIDEOINDEVICEDESC *)RTMemDup(pDeviceDesc, cbDeviceDesc);
+        pRemote->cbDeviceDesc = cbDeviceDesc;
+
+        /* Try to attach the device. */
+        EmulatedUSB *pEUSB = mParent->getConsole()->i_getEmulatedUSB();
+        pEUSB->i_webcamAttachInternal("", "", "EmWebcam", pRemote);
     }
     else
     {
-        mParent->consoleVRDPServer()->VideoInDeviceDetach(&mpRemote->deviceHandle);
+        mParent->VideoInDeviceDetach(&mpRemote->deviceHandle);
         RTMemFree(mpRemote);
         mpRemote = NULL;
     }
@@ -240,7 +267,8 @@ void EmWebcam::EmWebcamCbDeviceDesc(int rcRequest, void *pDeviceCtx, void *pvUse
 void EmWebcam::EmWebcamCbControl(int rcRequest, void *pDeviceCtx, void *pvUser,
                                  const VRDEVIDEOINCTRLHDR *pControl, uint32_t cbControl)
 {
-    EMWEBCAMREMOTE *pRemote = (EMWEBCAMREMOTE *)pDeviceCtx;
+    RT_NOREF(rcRequest);
+    EMWEBCAMREMOTE *pRemote = (EMWEBCAMREMOTE *)pDeviceCtx; NOREF(pRemote);
     Assert(pRemote == mpRemote);
 
     LogFlowFunc(("rcRequest %Rrc %p %p %p %d\n",
@@ -250,12 +278,12 @@ void EmWebcam::EmWebcamCbControl(int rcRequest, void *pDeviceCtx, void *pvUser,
 
     if (mpDrv && mpDrv->pIWebcamUp)
     {
-        mpDrv->pIWebcamUp->pfnWebcamUpControl(mpDrv->pIWebcamUp,
-                                              fResponse,
-                                              pvUser,
-                                              mpRemote->u64DeviceId,
-                                              (const PDMIWEBCAM_CTRLHDR *)pControl,
-                                              cbControl);
+        mpDrv->pIWebcamUp->pfnControl(mpDrv->pIWebcamUp,
+                                      fResponse,
+                                      pvUser,
+                                      mpRemote->u64DeviceId,
+                                      pControl,
+                                      cbControl);
     }
 
     RTMemFree(pvUser);
@@ -264,15 +292,25 @@ void EmWebcam::EmWebcamCbControl(int rcRequest, void *pDeviceCtx, void *pvUser,
 void EmWebcam::EmWebcamCbFrame(int rcRequest, void *pDeviceCtx,
                                const VRDEVIDEOINPAYLOADHDR *pFrame, uint32_t cbFrame)
 {
+    RT_NOREF(rcRequest, pDeviceCtx);
     LogFlowFunc(("rcRequest %Rrc %p %p %d\n",
                  rcRequest, pDeviceCtx, pFrame, cbFrame));
 
     if (mpDrv && mpDrv->pIWebcamUp)
     {
-        mpDrv->pIWebcamUp->pfnWebcamUpFrame(mpDrv->pIWebcamUp,
-                                            mpRemote->u64DeviceId,
-                                            (const uint8_t *)pFrame,
-                                            cbFrame);
+        if (   cbFrame >= sizeof(VRDEVIDEOINPAYLOADHDR)
+            && cbFrame >= pFrame->u8HeaderLength)
+        {
+            uint32_t cbImage = cbFrame - pFrame->u8HeaderLength;
+            const uint8_t *pu8Image = cbImage > 0? (const uint8_t *)pFrame + pFrame->u8HeaderLength: NULL;
+
+            mpDrv->pIWebcamUp->pfnFrame(mpDrv->pIWebcamUp,
+                                        mpRemote->u64DeviceId,
+                                        pFrame,
+                                        pFrame->u8HeaderLength,
+                                        pu8Image,
+                                        cbImage);
+        }
     }
 }
 
@@ -306,7 +344,7 @@ int EmWebcam::SendControl(EMWEBCAMDRV *pDrv, void *pvUser, uint64_t u64DeviceId,
         pCtx->pRemote = mpRemote;
         pCtx->pvUser = pvUser;
 
-        rc = mParent->consoleVRDPServer()->VideoInControl(pCtx, &mpRemote->deviceHandle, pControl, cbControl);
+        rc = mParent->VideoInControl(pCtx, &mpRemote->deviceHandle, pControl, cbControl);
 
         if (RT_FAILURE(rc))
         {
@@ -325,25 +363,39 @@ int EmWebcam::SendControl(EMWEBCAMDRV *pDrv, void *pvUser, uint64_t u64DeviceId,
     LogFlowFunc(("pszIID:%s\n", pszIID));
 
     PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pDrvIns->IBase);
-    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIWEBCAMDOWN, &pThis->IWebcamDown);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIWEBCAMDRV, &pThis->IWebcamDrv);
     return NULL;
 }
 
+/* static */ DECLCALLBACK(void) EmWebcam::drvDestruct(PPDMDRVINS pDrvIns)
+{
+    PDMDRV_CHECK_VERSIONS_RETURN_VOID(pDrvIns);
+    PEMWEBCAMDRV pThis = PDMINS_2_DATA(pDrvIns, PEMWEBCAMDRV);
+    EMWEBCAMREMOTE *pRemote = pThis->pRemote;
+
+    LogFlowFunc(("iInstance %d, pRemote %p, pIWebcamUp %p\n",
+                 pDrvIns->iInstance, pRemote, pThis->pIWebcamUp));
+
+    if (pRemote && pRemote->pEmWebcam)
+    {
+        pRemote->pEmWebcam->EmWebcamDestruct(pThis);
+    }
+}
+
 /* static */ DECLCALLBACK(int) EmWebcam::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
 {
+    RT_NOREF(fFlags);
+    PDMDRV_CHECK_VERSIONS_RETURN(pDrvIns);
     LogFlowFunc(("iInstance:%d, pCfg:%p, fFlags:%x\n", pDrvIns->iInstance, pCfg, fFlags));
 
     PEMWEBCAMDRV pThis = PDMINS_2_DATA(pDrvIns, PEMWEBCAMDRV);
 
-    if (!CFGMR3AreValuesValid(pCfg, "Object\0"))
-        return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
-
     AssertMsgReturn(PDMDrvHlpNoAttach(pDrvIns) == VERR_PDM_NO_ATTACHED_DRIVER,
                     ("Configuration error: Not possible to attach anything to this driver!\n"),
                     VERR_PDM_DRVINS_NO_ATTACH);
 
     /* Check early that there is a device. No need to init anything if there is no device. */
-    pThis->pIWebcamUp = PDMIBASE_QUERY_INTERFACE(pDrvIns->pUpBase, PDMIWEBCAMUP);
+    pThis->pIWebcamUp = PDMIBASE_QUERY_INTERFACE(pDrvIns->pUpBase, PDMIWEBCAMDEV);
     if (pThis->pIWebcamUp == NULL)
     {
         LogRel(("USBWEBCAM: Emulated webcam device does not exist.\n"));
@@ -358,30 +410,17 @@ int EmWebcam::SendControl(EMWEBCAMDRV *pDrv, void *pvUser, uint64_t u64DeviceId,
                     ("Configuration error: No/bad \"Object\" %p value! rc=%Rrc\n", pv, rc), rc);
 
     /* Everything ok. Initialize. */
-    pThis->pEmWebcam = (EmWebcam *)pv;
-    pThis->pEmWebcam->EmWebcamConstruct(pThis);
+    pThis->pRemote = (EMWEBCAMREMOTE *)pv;
+    pThis->pRemote->pEmWebcam->EmWebcamConstruct(pThis);
 
     pDrvIns->IBase.pfnQueryInterface = drvQueryInterface;
 
-    pThis->IWebcamDown.pfnWebcamDownControl = drvEmWebcamControl;
+    pThis->IWebcamDrv.pfnReady   = drvEmWebcamReady;
+    pThis->IWebcamDrv.pfnControl = drvEmWebcamControl;
 
     return VINF_SUCCESS;
 }
 
-/* static */ DECLCALLBACK(void) EmWebcam::drvDestruct(PPDMDRVINS pDrvIns)
-{
-    PEMWEBCAMDRV pThis = PDMINS_2_DATA(pDrvIns, PEMWEBCAMDRV);
-
-    LogFlowFunc(("iInstance %d, pEmWebcam %p, pIWebcamUp %p\n",
-                 pDrvIns->iInstance, pThis->pEmWebcam, pThis->pIWebcamUp));
-
-    if (pThis->pEmWebcam)
-    {
-        pThis->pEmWebcam->EmWebcamDestruct(pThis);
-        pThis->pEmWebcam = NULL;
-    }
-}
-
 /* static */ const PDMDRVREG EmWebcam::DrvReg =
 {
     /* u32Version */
