@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2013 Oracle Corporation
+ * Copyright (C) 2008-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -30,6 +30,7 @@
 #include "SystemPropertiesImpl.h"
 #include "AutoCaller.h"
 #include "Logging.h"
+#include "CertificateImpl.h"
 
 #include "ApplianceImplPrivate.h"
 
@@ -147,6 +148,8 @@ g_osTypes[] =
     { ovf::CIMOSType_CIMOS_MACOS,                                VBOXOSTYPE_MacOS109_x64 },
     { ovf::CIMOSType_CIMOS_MACOS,                                VBOXOSTYPE_MacOS1010_x64 },
     { ovf::CIMOSType_CIMOS_MACOS,                                VBOXOSTYPE_MacOS1011_x64 },
+    { ovf::CIMOSType_CIMOS_MACOS,                                VBOXOSTYPE_MacOS1012_x64 },
+    { ovf::CIMOSType_CIMOS_MACOS,                                VBOXOSTYPE_MacOS1013_x64 },
 
     // Linuxes
     { ovf::CIMOSType_CIMOS_RedHatEnterpriseLinux,                VBOXOSTYPE_RedHat },
@@ -268,7 +271,7 @@ static const osTypePattern g_osTypesPattern64[] =
 /**
  * Private helper func that suggests a VirtualBox guest OS type
  * for the given OVF operating system type.
- * @param osTypeVBox
+ * @param strType
  * @param c
  * @param cStr
  */
@@ -371,26 +374,25 @@ Utf8Str convertNetworkAttachmentTypeToString(NetworkAttachmentType_T type)
 /**
  * Implementation for IVirtualBox::createAppliance.
  *
- * @param anAppliance IAppliance object created if S_OK is returned.
+ * @param aAppliance IAppliance object created if S_OK is returned.
  * @return S_OK or error.
  */
 HRESULT VirtualBox::createAppliance(ComPtr<IAppliance> &aAppliance)
 {
-    HRESULT rc;
-
     ComObjPtr<Appliance> appliance;
-    appliance.createObject();
-    rc = appliance->init(this);
-
-    if (SUCCEEDED(rc))
-        appliance.queryInterfaceTo(aAppliance.asOutParam());
-
-    return rc;
+    HRESULT hrc = appliance.createObject();
+    if (SUCCEEDED(hrc))
+    {
+        hrc = appliance->init(this);
+        if (SUCCEEDED(hrc))
+            hrc = appliance.queryInterfaceTo(aAppliance.asOutParam());
+    }
+    return hrc;
 }
 
 /**
  * Appliance COM initializer.
- * @param
+ * @param   aVirtualBox     The VirtualBox object.
  * @return
  */
 HRESULT Appliance::init(VirtualBox *aVirtualBox)
@@ -444,7 +446,7 @@ void Appliance::uninit()
 
 /**
  * Public method implementation.
- * @param
+ * @param   aPath
  * @return
  */
 HRESULT Appliance::getPath(com::Utf8Str &aPath)
@@ -461,7 +463,7 @@ HRESULT Appliance::getPath(com::Utf8Str &aPath)
 
 /**
  * Public method implementation.
- * @param
+ * @param aDisks
  * @return
  */
 HRESULT Appliance::getDisks(std::vector<com::Utf8Str> &aDisks)
@@ -511,9 +513,25 @@ HRESULT Appliance::getDisks(std::vector<com::Utf8Str> &aDisks)
     return S_OK;
 }
 
+/**
+* Public method implementation.
+ * @return
+ */
+HRESULT Appliance::getCertificate(ComPtr<ICertificate> &aCertificateInfo)
+{
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (!i_isApplianceIdle())
+        return E_ACCESSDENIED;
+
+    /* Can be NULL at this point, queryInterfaceto handles that. */
+    m->ptrCertificateInfo.queryInterfaceTo(aCertificateInfo.asOutParam());
+    return S_OK;
+}
+
 /**
  * Public method implementation.
- * @param
+ * @param   aVirtualSystemDescriptions
  * @return
  */
 HRESULT Appliance::getVirtualSystemDescriptions(std::vector<ComPtr<IVirtualSystemDescription> > &aVirtualSystemDescriptions)
@@ -535,7 +553,7 @@ HRESULT Appliance::getVirtualSystemDescriptions(std::vector<ComPtr<IVirtualSyste
 
 /**
  * Public method implementation.
- * @param aDisks
+ * @param aMachines
  * @return
  */
 HRESULT Appliance::getMachines(std::vector<com::Utf8Str> &aMachines)
@@ -761,9 +779,9 @@ HRESULT Appliance::i_initApplianceIONameMap()
     if (!applianceIONameMap.empty())
         return rc;
 
-        applianceIONameMap.insert(std::make_pair(applianceIOTar, applianceIOTarName));
-        applianceIONameMap.insert(std::make_pair(applianceIOFile, applianceIOFileName));
-        applianceIONameMap.insert(std::make_pair(applianceIOSha, applianceIOShaName));
+    applianceIONameMap.insert(std::make_pair(applianceIOTar, applianceIOTarName));
+    applianceIONameMap.insert(std::make_pair(applianceIOFile, applianceIOFileName));
+    applianceIONameMap.insert(std::make_pair(applianceIOSha, applianceIOShaName));
 
     return rc;
 }
@@ -853,6 +871,43 @@ HRESULT Appliance::i_findMediumFormatFromDiskImage(const ovf::DiskImage &di, Com
     return rc;
 }
 
+/**
+ * Setup automatic I/O stream digest calculation, adding it to hOurManifest.
+ *
+ * @returns Passthru I/O stream, of @a hVfsIos if no digest calc needed.
+ * @param   hVfsIos             The stream to wrap. Always consumed.
+ * @param   pszManifestEntry    The manifest entry.
+ * @param   fRead               Set if read stream, clear if write.
+ * @throws  Nothing.
+ */
+RTVFSIOSTREAM Appliance::i_manifestSetupDigestCalculationForGivenIoStream(RTVFSIOSTREAM hVfsIos, const char *pszManifestEntry,
+                                                                          bool fRead /*= true */)
+{
+    int vrc;
+    Assert(!RTManifestPtIosIsInstanceOf(hVfsIos));
+
+    if (m->fDigestTypes == 0)
+        return hVfsIos;
+
+    /* Create the manifest if necessary. */
+    if (m->hOurManifest == NIL_RTMANIFEST)
+    {
+        vrc = RTManifestCreate(0 /*fFlags*/, &m->hOurManifest);
+        AssertRCReturnStmt(vrc, RTVfsIoStrmRelease(hVfsIos), NIL_RTVFSIOSTREAM);
+    }
+
+    /* Setup the stream. */
+    RTVFSIOSTREAM hVfsIosPt;
+    vrc = RTManifestEntryAddPassthruIoStream(m->hOurManifest, hVfsIos, pszManifestEntry, m->fDigestTypes, fRead, &hVfsIosPt);
+
+    RTVfsIoStrmRelease(hVfsIos);        /* always consumed! */
+    if (RT_SUCCESS(vrc))
+        return hVfsIosPt;
+
+    setErrorVrc(vrc, "RTManifestEntryAddPassthruIoStream failed with rc=%Rrc", vrc);
+    return NIL_RTVFSIOSTREAM;
+}
+
 /**
  * Returns true if the appliance is in "idle" state. This should always be the
  * case unless an import or export is currently in progress. Similar to machine
@@ -884,7 +939,7 @@ HRESULT Appliance::i_searchUniqueVMName(Utf8Str& aName) const
     IMachine *machine = NULL;
     char *tmpName = RTStrDup(aName.c_str());
     int i = 1;
-    /** @todo: Maybe too cost-intensive; try to find a lighter way */
+    /** @todo Maybe too cost-intensive; try to find a lighter way */
     while (mVirtualBox->FindMachine(Bstr(tmpName).raw(), &machine) != VBOX_E_OBJECT_NOT_FOUND)
     {
         RTStrFree(tmpName);
@@ -904,7 +959,7 @@ HRESULT Appliance::i_searchUniqueDiskImageFilePath(Utf8Str& aName) const
     int i = 1;
     /* Check if the file exists or if a file with this path is registered
      * already */
-    /** @todo: Maybe too cost-intensive; try to find a lighter way */
+    /** @todo Maybe too cost-intensive; try to find a lighter way */
     while (    RTPathExists(tmpName)
             || mVirtualBox->OpenMedium(Bstr(tmpName).raw(), DeviceType_HardDisk, AccessMode_ReadWrite,
                                        FALSE /* fForceNewUuid */,  &harddisk) != VBOX_E_OBJECT_NOT_FOUND)
@@ -931,12 +986,12 @@ HRESULT Appliance::i_searchUniqueDiskImageFilePath(Utf8Str& aName) const
  * progress object with the proper weights and maximum progress values.
  *
  * @param pProgress
- * @param bstrDescription
+ * @param strDescription
  * @param mode
  * @return
  */
 HRESULT Appliance::i_setUpProgress(ComObjPtr<Progress> &pProgress,
-                                   const Bstr &bstrDescription,
+                                   const Utf8Str &strDescription,
                                    SetUpProgressMode mode)
 {
     HRESULT rc;
@@ -1031,11 +1086,11 @@ HRESULT Appliance::i_setUpProgress(ComObjPtr<Progress> &pProgress,
          m->ulTotalDisksMB, m->cDisks, cOperations, ulTotalOperationsWeight, m->ulWeightForXmlOperation));
 
     rc = pProgress->init(mVirtualBox, static_cast<IAppliance*>(this),
-                         bstrDescription.raw(),
+                         Bstr(strDescription).raw(),
                          TRUE /* aCancelable */,
                          cOperations, // ULONG cOperations,
                          ulTotalOperationsWeight, // ULONG ulTotalOperationsWeight,
-                         bstrDescription.raw(), // CBSTR bstrFirstOperationDescription,
+                         Bstr(strDescription).raw(), // CBSTR bstrFirstOperationDescription,
                          m->ulWeightForXmlOperation); // ULONG ulFirstOperationWeight,
     return rc;
 }
@@ -1118,11 +1173,18 @@ void Appliance::i_waitForAsyncProgress(ComObjPtr<Progress> &pProgressThis,
 
 void Appliance::i_addWarning(const char* aWarning, ...)
 {
-    va_list args;
-    va_start(args, aWarning);
-    Utf8Str str(aWarning, args);
-    va_end(args);
-    m->llWarnings.push_back(str);
+    try
+    {
+        va_list args;
+        va_start(args, aWarning);
+        Utf8Str str(aWarning, args);
+        va_end(args);
+        m->llWarnings.push_back(str);
+    }
+    catch (...)
+    {
+        AssertFailed();
+    }
 }
 
 /**
@@ -1184,99 +1246,83 @@ void Appliance::i_parseBucket(Utf8Str &aPath, Utf8Str &aBucket)
 }
 
 /**
- * Starts the worker thread for the task.
+ * Worker for TaskOVF::handler.
  *
- * @return COM status code.
- */
-HRESULT Appliance::TaskOVF::startThread()
-{
-    /* Pick a thread name suitable for logging (<= 8 chars). */
-    const char *pszTaskNm;
-    switch (taskType)
-    {
-        case TaskOVF::Read:     pszTaskNm = "ApplRead"; break;
-        case TaskOVF::Import:   pszTaskNm = "ApplImp"; break;
-        case TaskOVF::Write:    pszTaskNm = "ApplWrit"; break;
-        default:                pszTaskNm = "ApplTask"; break;
-    }
-
-    int vrc = RTThreadCreate(NULL, Appliance::i_taskThreadImportOrExport, this,
-                             0, RTTHREADTYPE_MAIN_HEAVY_WORKER, 0, pszTaskNm);
-    if (RT_SUCCESS(vrc))
-        return S_OK;
-    return Appliance::i_setErrorStatic(E_FAIL, Utf8StrFmt("Could not create OVF task thread (%Rrc)\n", vrc));
-}
-
-/**
- * Thread function for the thread started in Appliance::readImpl() and Appliance::importImpl()
+ * The TaskOVF is started in Appliance::readImpl() and Appliance::importImpl()
  * and Appliance::writeImpl().
- * This will in turn call Appliance::readFS() or Appliance::readS3() or Appliance::importFS()
- * or Appliance::importS3() or Appliance::writeFS() or Appliance::writeS3().
  *
- * @param aThread
- * @param pvUser
+ * This will in turn call Appliance::readFS() or Appliance::importFS() or
+ * Appliance::writeFS().
+ *
+ * @thread  pTask       The task.
  */
 /* static */
-DECLCALLBACK(int) Appliance::i_taskThreadImportOrExport(RTTHREAD /* aThread */, void *pvUser)
+void Appliance::i_importOrExportThreadTask(TaskOVF *pTask)
 {
-    std::auto_ptr<TaskOVF> task(static_cast<TaskOVF*>(pvUser));
-    AssertReturn(task.get(), VERR_GENERAL_FAILURE);
-
-    Appliance *pAppliance = task->pAppliance;
-
     LogFlowFuncEnter();
-    LogFlowFunc(("Appliance %p taskType=%d\n", pAppliance, task->taskType));
+    AssertReturnVoid(pTask);
 
-    HRESULT taskrc = S_OK;
+    Appliance *pAppliance = pTask->pAppliance;
+    LogFlowFunc(("Appliance %p taskType=%d\n", pAppliance, pTask->taskType));
 
-    switch (task->taskType)
+    switch (pTask->taskType)
     {
         case TaskOVF::Read:
-            if (task->locInfo.storageType == VFSType_File)
-                taskrc = pAppliance->i_readFS(task.get());
-            else if (task->locInfo.storageType == VFSType_S3)
-#ifdef VBOX_WITH_S3
-                taskrc = pAppliance->i_readS3(task.get());
-#else
-                taskrc = VERR_NOT_IMPLEMENTED;
-#endif
-        break;
+            pAppliance->m->resetReadData();
+            if (pTask->locInfo.storageType == VFSType_File)
+                pTask->rc = pAppliance->i_readFS(pTask);
+            else
+                pTask->rc = E_NOTIMPL;
+            break;
 
         case TaskOVF::Import:
-            if (task->locInfo.storageType == VFSType_File)
-                taskrc = pAppliance->i_importFS(task.get());
-            else if (task->locInfo.storageType == VFSType_S3)
-#ifdef VBOX_WITH_S3
-                taskrc = pAppliance->i_importS3(task.get());
-#else
-                taskrc = VERR_NOT_IMPLEMENTED;
-#endif
-        break;
+            /** @todo allow overriding these? */
+            if (!pAppliance->m->fSignatureValid && pAppliance->m->pbSignedDigest)
+                pTask->rc = pAppliance->setError(E_FAIL, tr("The manifest signature for '%s' is not valid"),
+                                                 pTask->locInfo.strPath.c_str());
+            else if (!pAppliance->m->fCertificateValid && pAppliance->m->pbSignedDigest)
+            {
+                if (pAppliance->m->strCertError.isNotEmpty())
+                    pTask->rc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is not valid: %s"),
+                                                     pTask->locInfo.strPath.c_str(), pAppliance->m->strCertError.c_str());
+                else
+                    pTask->rc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is not valid"),
+                                                     pTask->locInfo.strPath.c_str());
+            }
+            // fusion does not consider this a show stopper (we've filed a warning during read).
+            //else if (pAppliance->m->fCertificateMissingPath && pAppliance->m->pbSignedDigest)
+            //    pTask->rc = pAppliance->setError(E_FAIL, tr("The certificate used to signed '%s' is does not have a valid CA path"),
+            //                                     pTask->locInfo.strPath.c_str());
+            else
+            {
+                if (pTask->locInfo.storageType == VFSType_File)
+                    pTask->rc = pAppliance->i_importFS(pTask);
+                else
+                    pTask->rc = E_NOTIMPL;
+            }
+            break;
 
         case TaskOVF::Write:
-            if (task->locInfo.storageType == VFSType_File)
-                taskrc = pAppliance->i_writeFS(task.get());
-            else if (task->locInfo.storageType == VFSType_S3)
-#ifdef VBOX_WITH_S3
-                taskrc = pAppliance->i_writeS3(task.get());
-#else
-                taskrc = VERR_NOT_IMPLEMENTED;
-#endif
-        break;
+            if (pTask->locInfo.storageType == VFSType_File)
+                pTask->rc = pAppliance->i_writeFS(pTask);
+            else
+                pTask->rc = E_NOTIMPL;
+            break;
+
+        default:
+            AssertFailed();
+            pTask->rc = E_FAIL;
+            break;
     }
 
-    task->rc = taskrc;
-
-    if (!task->pProgress.isNull())
-        task->pProgress->i_notifyComplete(taskrc);
+    if (!pTask->pProgress.isNull())
+        pTask->pProgress->i_notifyComplete(pTask->rc);
 
     LogFlowFuncLeave();
-
-    return VINF_SUCCESS;
 }
 
 /* static */
-int Appliance::TaskOVF::updateProgress(unsigned uPercent, void *pvUser)
+DECLCALLBACK(int) Appliance::TaskOVF::updateProgress(unsigned uPercent, void *pvUser)
 {
     Appliance::TaskOVF* pTask = *(Appliance::TaskOVF**)pvUser;
 
@@ -1385,7 +1431,7 @@ void VirtualSystemDescription::uninit()
 
 /**
  * Public method implementation.
- * @param
+ * @param   aCount
  * @return
  */
 HRESULT VirtualSystemDescription::getCount(ULONG *aCount)
@@ -1555,8 +1601,8 @@ HRESULT VirtualSystemDescription::addDescription(VirtualSystemDescriptionType_T
  * Internal method; adds a new description item to the member list.
  * @param aType Type of description for the new item.
  * @param strRef Reference item; only used with hard disk controllers.
- * @param aOrigValue Corresponding original value from OVF.
- * @param aAutoValue Initial configuration value (can be overridden by caller with setFinalValues).
+ * @param aOvfValue Corresponding original value from OVF.
+ * @param aVBoxValue Initial configuration value (can be overridden by caller with setFinalValues).
  * @param ulSizeMB Weight for IProgress
  * @param strExtraConfig Extra configuration; meaning dependent on type.
  */
@@ -1646,7 +1692,8 @@ const VirtualSystemDescriptionEntry* VirtualSystemDescription::i_findControllerF
             case VirtualSystemDescriptionType_HardDiskControllerSAS:
                 if (d.strRef == strRef)
                     return &d;
-            break;
+                break;
+            default: break; /* Shut up MSC. */
         }
     }
 
