@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2008-2012 Oracle Corporation
+ * Copyright (C) 2008-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -34,8 +34,8 @@
 #endif
 #ifdef RT_OS_WINDOWS
 # define _WIN32_DCOM
-# include <objidl.h>
-# include <objbase.h>
+# include <iprt/win/objidl.h>
+# include <iprt/win/objbase.h>
 # include "../src-server/win/PerformanceWin.cpp"
 #endif
 
@@ -50,31 +50,39 @@
 #define RUN_TIME_MS        1000
 
 #define N_CALLS(n, fn) \
+  do {\
     for (int call = 0; call < n; ++call) \
         rc = collector->fn; \
     if (RT_FAILURE(rc)) \
-        RTPrintf("tstCollector: "#fn" -> %Rrc\n", rc)
+        RTPrintf("tstCollector: "#fn" -> %Rrc\n", rc); \
+  } while (0)
 
-#define CALLS_PER_SECOND(fn) \
+#define CALLS_PER_SECOND(fn, args) \
+  do { \
     nCalls = 0; \
     start = RTTimeMilliTS(); \
     do { \
-        rc = collector->fn; \
+        rc = collector->fn args; \
         if (RT_FAILURE(rc)) \
             break; \
         ++nCalls; \
     } while (RTTimeMilliTS() - start < RUN_TIME_MS); \
     if (RT_FAILURE(rc)) \
-    { \
         RTPrintf("tstCollector: "#fn" -> %Rrc\n", rc); \
-    } \
     else \
-        RTPrintf("%70s -- %u calls per second\n", #fn, nCalls)
+        RTPrintf("%70s -- %u calls per second\n", #fn, nCalls); \
+  } while (0)
+
+void shutdownProcessList(std::vector<RTPROCESS> const &rProcesses)
+{
+    for (size_t i = 0; i < rProcesses.size(); i++)
+        RTProcTerminate(rProcesses[i]);
+}
 
 void measurePerformance(pm::CollectorHAL *collector, const char *pszName, int cVMs)
 {
 
-    static const char * const args[] = { pszName, "-child", NULL };
+    const char * const args[] = { pszName, "-child", NULL };
     pm::CollectorHints hints;
     std::vector<RTPROCESS> processes;
 
@@ -88,7 +96,8 @@ void measurePerformance(pm::CollectorHAL *collector, const char *pszName, int cV
         if (RT_FAILURE(rc))
         {
             hints.getProcesses(processes);
-            std::for_each(processes.begin(), processes.end(), std::ptr_fun(RTProcTerminate));
+            shutdownProcessList(processes);
+
             RTPrintf("tstCollector: RTProcCreate() -> %Rrc\n", rc);
             return;
         }
@@ -105,17 +114,17 @@ void measurePerformance(pm::CollectorHAL *collector, const char *pszName, int cV
     uint64_t start;
     unsigned int nCalls;
     /* Pre-collect */
-    CALLS_PER_SECOND(preCollect(hints, 0));
+    CALLS_PER_SECOND(preCollect, (hints, 0));
     /* Host CPU load */
-    CALLS_PER_SECOND(getRawHostCpuLoad(&tmp64, &tmp64, &tmp64));
+    CALLS_PER_SECOND(getRawHostCpuLoad, (&tmp64, &tmp64, &tmp64));
     /* Process CPU load */
-    CALLS_PER_SECOND(getRawProcessCpuLoad(processes[nCalls%cVMs], &tmp64, &tmp64, &tmp64));
+    CALLS_PER_SECOND(getRawProcessCpuLoad, (processes[nCalls % cVMs], &tmp64, &tmp64, &tmp64));
     /* Host CPU speed */
-    CALLS_PER_SECOND(getHostCpuMHz(&tmp));
+    CALLS_PER_SECOND(getHostCpuMHz, (&tmp));
     /* Host RAM usage */
-    CALLS_PER_SECOND(getHostMemoryUsage(&tmp, &tmp, &tmp));
+    CALLS_PER_SECOND(getHostMemoryUsage, (&tmp, &tmp, &tmp));
     /* Process RAM usage */
-    CALLS_PER_SECOND(getProcessMemoryUsage(processes[nCalls%cVMs], &tmp));
+    CALLS_PER_SECOND(getProcessMemoryUsage, (processes[nCalls % cVMs], &tmp));
 
     start = RTTimeNanoTS();
 
@@ -135,10 +144,10 @@ void measurePerformance(pm::CollectorHAL *collector, const char *pszName, int cV
         /* Process RAM usage */
         N_CALLS(cVMs, getProcessMemoryUsage(processes[call], &tmp));
     }
-    printf("\n%u VMs -- %.2f%% of CPU time\n", cVMs, (RTTimeNanoTS() - start) / 10000000. / times);
+    printf("\n%d VMs -- %.2f%% of CPU time\n", cVMs, (RTTimeNanoTS() - start) / 10000000. / times);
 
     /* Shut down fake VMs */
-    std::for_each(processes.begin(), processes.end(), std::ptr_fun(RTProcTerminate));
+    shutdownProcessList(processes);
 }
 
 #ifdef RT_OS_SOLARIS
@@ -566,6 +575,6 @@ int main(int argc, char *argv[])
 
     printf ("\ntstCollector FINISHED.\n");
 
-    return rc;
+    return RTEXITCODE_SUCCESS;
 }
 
