@@ -1,11 +1,14 @@
-/**
+/* $Id: vboxweb.cpp $ */
+/** @file
  * vboxweb.cpp:
  *      hand-coded parts of the webservice server. This is linked with the
  *      generated code in out/.../src/VBox/Main/webservice/methodmaps.cpp
  *      (plus static gSOAP server code) to implement the actual webservice
  *      server, to which clients can connect.
- *
- * Copyright (C) 2007-2015 Oracle Corporation
+ */
+
+/*
+ * Copyright (C) 2007-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -50,6 +53,10 @@
 #include <iprt/stream.h>
 #include <iprt/asm.h>
 
+#ifdef WITH_OPENSSL
+# include <openssl/opensslv.h>
+#endif
+
 #ifndef RT_OS_WINDOWS
 # include <signal.h>
 #endif
@@ -99,7 +106,7 @@ typedef WebsessionsMap::iterator                WebsessionsMapIterator;
 
 typedef std::map<RTTHREAD, com::Utf8Str> ThreadsMap;
 
-static int fntWatchdog(RTTHREAD ThreadSelf, void *pvUser);
+static DECLCALLBACK(int) fntWatchdog(RTTHREAD ThreadSelf, void *pvUser);
 
 /****************************************************************************
  *
@@ -398,15 +405,17 @@ public:
     /**
      * Static function that can be passed to RTThreadCreate and that calls
      * process() on the SoapThread instance passed as the thread parameter.
-     * @param pThread
-     * @param pvThread
+     *
+     * @param   hThreadSelf
+     * @param   pvThread
      * @return
      */
-    static int fntWrapper(RTTHREAD pThread, void *pvThread)
+    static DECLCALLBACK(int) fntWrapper(RTTHREAD hThreadSelf, void *pvThread)
     {
+        RT_NOREF(hThreadSelf);
         SoapThread *pst = (SoapThread*)pvThread;
         pst->process();
-        return 0;
+        return VINF_SUCCESS;
     }
 
     size_t          m_u;            // thread number
@@ -451,7 +460,7 @@ public:
                 RTSemEventMultiSignal(m_event);
                 qlock.acquire();
             }
-            WebLog("ending queue processing (%d out of %d threads idle)\n", m_cIdleThreads, m_llAllThreads.size());
+            LogRel(("ending queue processing (%d out of %d threads idle)\n", m_cIdleThreads, m_llAllThreads.size()));
         }
 
         RTSemEventMultiDestroy(m_event);
@@ -589,7 +598,7 @@ public:
  */
 void SoapThread::process()
 {
-    WebLog("New SOAP thread started\n");
+    LogRel(("New SOAP thread started\n"));
 
     while (g_fKeepRunning)
     {
@@ -600,19 +609,18 @@ void SoapThread::process()
         if (!soap_valid_socket(m_soap->socket))
             continue;
 
-        WebLog("Processing connection from IP=%lu.%lu.%lu.%lu socket=%d (%d out of %d threads idle)\n",
-               (m_soap->ip >> 24) & 0xFF,
-               (m_soap->ip >> 16) & 0xFF,
-               (m_soap->ip >> 8)  & 0xFF,
-               m_soap->ip         & 0xFF,
-               m_soap->socket,
-               cIdleThreads,
-               cThreads);
+        LogRel(("Processing connection from IP=%RTnaipv4 socket=%d (%d out of %d threads idle)\n",
+                RT_H2N_U32(m_soap->ip), m_soap->socket, cIdleThreads, cThreads));
 
         // Ensure that we don't get stuck indefinitely for connections using
         // keepalive, otherwise stale connections tie up worker threads.
         m_soap->send_timeout = 60;
         m_soap->recv_timeout = 60;
+        // Limit the maximum SOAP request size to a generous amount, just to
+        // be on the safe side (SOAP is quite wordy when representing arrays,
+        // and some API uses need to deal with large arrays). Good that binary
+        // data is no longer represented by byte arrays...
+        m_soap->recv_maxlength = _16M;
         // process the request; this goes into the COM code in methodmaps.cpp
         do {
 #ifdef WITH_OPENSSL
@@ -673,7 +681,7 @@ public:
                 pVSACEv->COMGETTER(Available)(&fAvailable);
                 if (!fAvailable)
                 {
-                    WebLog("VBoxSVC became unavailable\n");
+                    LogRel(("VBoxSVC became unavailable\n"));
                     {
                         util::AutoWriteLock vlock(g_pVirtualBoxLockHandle COMMA_LOCKVAL_SRC_POS);
                         g_pVirtualBox.setNull();
@@ -696,7 +704,7 @@ public:
                 }
                 else
                 {
-                    WebLog("VBoxSVC became available\n");
+                    LogRel(("VBoxSVC became available\n"));
                     util::AutoWriteLock vlock(g_pVirtualBoxLockHandle COMMA_LOCKVAL_SRC_POS);
                     HRESULT hrc = g_pVirtualBoxClient->COMGETTER(VirtualBox)(g_pVirtualBox.asOutParam());
                     AssertComRC(hrc);
@@ -717,24 +725,6 @@ typedef ListenerImpl<VirtualBoxClientEventListener> VirtualBoxClientEventListene
 
 VBOX_LISTENER_DECLARE(VirtualBoxClientEventListenerImpl)
 
-/**
- * Prints a message to the webservice log file.
- * @param pszFormat
- * @todo eliminate, has no significant additional value over direct calls to LogRel.
- */
-void WebLog(const char *pszFormat, ...)
-{
-    va_list args;
-    va_start(args, pszFormat);
-    char *psz = NULL;
-    RTStrAPrintfV(&psz, pszFormat, args);
-    va_end(args);
-
-    LogRel(("%s", psz));
-
-    RTStrFree(psz);
-}
-
 /**
  * Helper for printing SOAP error messages.
  * @param soap
@@ -744,21 +734,37 @@ void WebLogSoapError(struct soap *soap)
 {
     if (soap_check_state(soap))
     {
-        WebLog("Error: soap struct not initialized\n");
+        LogRel(("Error: soap struct not initialized\n"));
         return;
     }
 
     const char *pcszFaultString = *soap_faultstring(soap);
     const char **ppcszDetail = soap_faultcode(soap);
-    WebLog("#### SOAP FAULT: %s [%s]\n",
-           pcszFaultString ? pcszFaultString : "[no fault string available]",
-           (ppcszDetail && *ppcszDetail) ? *ppcszDetail : "no details available");
+    LogRel(("#### SOAP FAULT: %s [%s]\n",
+            pcszFaultString ? pcszFaultString : "[no fault string available]",
+            (ppcszDetail && *ppcszDetail) ? *ppcszDetail : "no details available"));
 }
 
-#ifdef WITH_OPENSSL
+/**
+ * Helper for decoding AuthResult.
+ * @param result AuthResult
+ */
+static const char * decodeAuthResult(AuthResult result)
+{
+    switch (result)
+    {
+        case AuthResultAccessDenied:    return "access DENIED";
+        case AuthResultAccessGranted:   return "access granted";
+        case AuthResultDelegateToGuest: return "delegated to guest";
+        default:                        return "unknown AuthResult";
+    }
+}
+
+#if defined(WITH_OPENSSL) && (OPENSSL_VERSION_NUMBER < 0x10100000 || defined(LIBRESSL_VERSION_NUMBER))
 /****************************************************************************
  *
- * OpenSSL convenience functions for multithread support
+ * OpenSSL convenience functions for multithread support.
+ * Not required for OpenSSL 1.1+
  *
  ****************************************************************************/
 
@@ -860,7 +866,7 @@ static void CRYPTO_thread_cleanup()
     RTMemFree(g_pSSLMutexes);
     g_pSSLMutexes = NULL;
 }
-#endif /* WITH_OPENSSL */
+#endif /* WITH_OPENSSL && (OPENSSL_VERSION_NUMBER < 0x10100000 || defined(LIBRESSL_VERSION_NUMBER)) */
 
 /****************************************************************************
  *
@@ -870,13 +876,13 @@ static void CRYPTO_thread_cleanup()
 
 static void doQueuesLoop()
 {
-#ifdef WITH_OPENSSL
+#if defined(WITH_OPENSSL) && (OPENSSL_VERSION_NUMBER < 0x10100000 || defined(LIBRESSL_VERSION_NUMBER))
     if (g_fSSL && CRYPTO_thread_setup())
     {
-        WebLog("Failed to set up OpenSSL thread mutex!");
+        LogRel(("Failed to set up OpenSSL thread mutex!"));
         exit(RTEXITCODE_FAILURE);
     }
-#endif /* WITH_OPENSSL */
+#endif
 
     // set up gSOAP
     struct soap soap;
@@ -900,29 +906,53 @@ static void doQueuesLoop()
                   g_pcszBindToHost ? g_pcszBindToHost : "localhost",    // safe default host
                   g_uBindToPort,    // port
                   g_uBacklog);      // backlog = max queue size for requests
-    if (m < 0)
+    if (m == SOAP_INVALID_SOCKET)
         WebLogSoapError(&soap);
     else
     {
-        WebLog("Socket connection successful: host = %s, port = %u, %smaster socket = %d\n",
-               (g_pcszBindToHost) ? g_pcszBindToHost : "default (localhost)",
-               g_uBindToPort,
 #ifdef WITH_OPENSSL
-               g_fSSL ? "SSL, " : "",
+        const char *pszSsl = g_fSSL ? "SSL, " : "";
 #else /* !WITH_OPENSSL */
-               "",
+        const char *pszSsl = "";
 #endif /*!WITH_OPENSSL */
-               m);
+        LogRel(("Socket connection successful: host = %s, port = %u, %smaster socket = %d\n",
+                (g_pcszBindToHost) ? g_pcszBindToHost : "default (localhost)",
+                g_uBindToPort, pszSsl, m));
 
         // initialize thread queue, mutex and eventsem
         g_pSoapQ = new SoapQ(&soap);
 
-        for (uint64_t i = 1;
-             g_fKeepRunning;
-             i++)
+        uint64_t cAccepted = 1;
+        while (g_fKeepRunning)
         {
+            struct timeval timeout;
+            fd_set ReadFds, WriteFds, XcptFds;
+            int rv;
+            for (;;)
+            {
+                timeout.tv_sec = 60;
+                timeout.tv_usec = 0;
+                FD_ZERO(&ReadFds);
+                FD_SET(soap.master, &ReadFds);
+                FD_ZERO(&WriteFds);
+                FD_SET(soap.master, &WriteFds);
+                FD_ZERO(&XcptFds);
+                FD_SET(soap.master, &XcptFds);
+                rv = select((int)soap.master + 1, &ReadFds, &WriteFds, &XcptFds, &timeout);
+                if (rv > 0)
+                    break; // work is waiting
+                if (rv == 0)
+                    continue; // timeout, not necessary to bother gsoap
+                // r < 0, errno
+                if (soap_socket_errno(soap.master) == SOAP_EINTR)
+                    rv = 0; // re-check if we should terminate
+                break;
+            }
+            if (rv == 0)
+                continue;
+
             // call gSOAP to handle incoming SOAP connection
-            soap.accept_timeout = 10;
+            soap.accept_timeout = -1; // 1usec timeout, actual waiting is above
             s = soap_accept(&soap);
             if (!soap_valid_socket(s))
             {
@@ -934,13 +964,14 @@ static void doQueuesLoop()
             // add the socket to the queue and tell worker threads to
             // pick up the job
             size_t cItemsOnQ = g_pSoapQ->add(s);
-            WebLog("Request %llu on socket %d queued for processing (%d items on Q)\n", i, s, cItemsOnQ);
+            LogRel(("Request %llu on socket %d queued for processing (%d items on Q)\n", cAccepted, s, cItemsOnQ));
+            cAccepted++;
         }
 
         delete g_pSoapQ;
         g_pSoapQ = NULL;
 
-        WebLog("ending SOAP request handling\n");
+        LogRel(("ending SOAP request handling\n"));
 
         delete g_pSoapQ;
         g_pSoapQ = NULL;
@@ -948,10 +979,10 @@ static void doQueuesLoop()
     }
     soap_done(&soap); // close master socket and detach environment
 
-#ifdef WITH_OPENSSL
+#if defined(WITH_OPENSSL) && (OPENSSL_VERSION_NUMBER < 0x10100000 || defined(LIBRESSL_VERSION_NUMBER))
     if (g_fSSL)
         CRYPTO_thread_cleanup();
-#endif /* WITH_OPENSSL */
+#endif
 }
 
 /**
@@ -959,8 +990,10 @@ static void doQueuesLoop()
  * the loop that takes SOAP calls from HTTP and serves them by handing sockets to the
  * SOAP queue worker threads.
  */
-static int fntQPumper(RTTHREAD ThreadSelf, void *pvUser)
+static DECLCALLBACK(int) fntQPumper(RTTHREAD hThreadSelf, void *pvUser)
 {
+    RT_NOREF(hThreadSelf, pvUser);
+
     // store a log prefix for this thread
     util::AutoWriteLock thrLock(g_pThreadsLockHandle COMMA_LOCKVAL_SRC_POS);
     g_mapThreads[RTThreadSelf()] = "[ P ]";
@@ -970,13 +1003,10 @@ static int fntQPumper(RTTHREAD ThreadSelf, void *pvUser)
 
     thrLock.acquire();
     g_mapThreads.erase(RTThreadSelf());
-    return 0;
+    return VINF_SUCCESS;
 }
 
 #ifdef RT_OS_WINDOWS
-// Required for ATL
-static CComModule _Module;
-
 /**
  * "Signal" handler for cleanly terminating the event loop.
  */
@@ -992,35 +1022,28 @@ static BOOL WINAPI websrvSignalHandler(DWORD dwCtrlType)
         case CTRL_C_EVENT:
         case CTRL_LOGOFF_EVENT:
         case CTRL_SHUTDOWN_EVENT:
+        {
             ASMAtomicWriteBool(&g_fKeepRunning, false);
+            com::NativeEventQueue *pQ = com::NativeEventQueue::getMainEventQueue();
+            pQ->interruptEventQueueProcessing();
             fEventHandled = TRUE;
             break;
+        }
         default:
             break;
     }
     return fEventHandled;
 }
 #else
-class ForceQuitEvent : public com::NativeEvent
-{
-    void *handler()
-    {
-        LogFlowFunc(("\n"));
-
-        ASMAtomicWriteBool(&g_fKeepRunning, false);
-
-        return NULL;
-    }
-};
-
 /**
  * Signal handler for cleanly terminating the event loop.
  */
 static void websrvSignalHandler(int iSignal)
 {
     NOREF(iSignal);
+    ASMAtomicWriteBool(&g_fKeepRunning, false);
     com::NativeEventQueue *pQ = com::NativeEventQueue::getMainEventQueue();
-    pQ->postEvent(new ForceQuitEvent());
+    pQ->interruptEventQueueProcessing();
 }
 #endif
 
@@ -1041,6 +1064,9 @@ int main(int argc, char *argv[])
     int rc = RTR3InitExe(argc, &argv, 0);
     if (RT_FAILURE(rc))
         return RTMsgInitFailure(rc);
+#ifdef RT_OS_WINDOWS
+    ATL::CComModule _Module; /* Required internally by ATL (constructor records instance in global variable). */
+#endif
 
     // store a log prefix for this thread
     g_mapThreads[RTThreadSelf()] = "[M  ]";
@@ -1189,15 +1215,15 @@ int main(int argc, char *argv[])
     }
 
     /* create release logger, to stdout */
-    char szError[RTPATH_MAX + 128];
+    RTERRINFOSTATIC ErrInfo;
     rc = com::VBoxLogRelCreate("web service", g_fDaemonize ? NULL : pszLogFile,
                                RTLOGFLAGS_PREFIX_THREAD | RTLOGFLAGS_PREFIX_TIME_PROG,
                                "all", "VBOXWEBSRV_RELEASE_LOG",
                                RTLOGDEST_STDOUT, UINT32_MAX /* cMaxEntriesPerGroup */,
                                g_cHistory, g_uHistoryFileTime, g_uHistoryFileSize,
-                               szError, sizeof(szError));
+                               RTErrInfoInitStatic(&ErrInfo));
     if (RT_FAILURE(rc))
-        return RTMsgErrorExit(RTEXITCODE_FAILURE, "failed to open release log (%s, %Rrc)", szError, rc);
+        return RTMsgErrorExit(RTEXITCODE_FAILURE, "failed to open release log (%s, %Rrc)", ErrInfo.Core.pszMsg, rc);
 
 #if defined(RT_OS_DARWIN) || defined(RT_OS_LINUX) || defined (RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
     if (g_fDaemonize)
@@ -1226,9 +1252,9 @@ int main(int argc, char *argv[])
                                    "all", "VBOXWEBSRV_RELEASE_LOG",
                                    RTLOGDEST_FILE, UINT32_MAX /* cMaxEntriesPerGroup */,
                                    g_cHistory, g_uHistoryFileTime, g_uHistoryFileSize,
-                                   szError, sizeof(szError));
+                                   RTErrInfoInitStatic(&ErrInfo));
         if (RT_FAILURE(rc))
-            return RTMsgErrorExit(RTEXITCODE_FAILURE, "failed to open release log (%s, %Rrc)", szError, rc);
+            return RTMsgErrorExit(RTEXITCODE_FAILURE, "failed to open release log (%s, %Rrc)", ErrInfo.Core.pszMsg, rc);
     }
 #endif
 
@@ -1339,6 +1365,7 @@ int main(int argc, char *argv[])
     }
 #else
     signal(SIGINT,   websrvSignalHandler);
+    signal(SIGTERM,  websrvSignalHandler);
 # ifdef SIGBREAK
     signal(SIGBREAK, websrvSignalHandler);
 # endif
@@ -1354,7 +1381,7 @@ int main(int argc, char *argv[])
             RTMsgError("processEventQueue -> %Rrc", rc);
     }
 
-    WebLog("requested termination, cleaning up\n");
+    LogRel(("requested termination, cleaning up\n"));
 
 #ifdef RT_OS_WINDOWS
     if (!SetConsoleCtrlHandler((PHANDLER_ROUTINE)websrvSignalHandler, FALSE /* Remove handler */))
@@ -1364,14 +1391,23 @@ int main(int argc, char *argv[])
     }
 #else
     signal(SIGINT,   SIG_DFL);
+    signal(SIGTERM,  SIG_DFL);
 # ifdef SIGBREAK
     signal(SIGBREAK, SIG_DFL);
 # endif
 #endif
 
+#ifndef RT_OS_WINDOWS
+    RTThreadPoke(threadQPumper);
+#endif
     RTThreadWait(threadQPumper, 30000, NULL);
     if (threadWatchdog != NIL_RTTHREAD)
+    {
+#ifndef RT_OS_WINDOWS
+        RTThreadPoke(threadWatchdog);
+#endif
         RTThreadWait(threadWatchdog, g_iWatchdogCheckInterval * 1000 + 10000, NULL);
+    }
 
     /* VirtualBoxClient events unregistration. */
     if (vboxClientListener)
@@ -1419,9 +1455,10 @@ int main(int argc, char *argv[])
  * for whether they have been no requests in a configurable timeout period. In
  * that case, the websession is automatically logged off.
  */
-/* static */
-int fntWatchdog(RTTHREAD ThreadSelf, void *pvUser)
+static DECLCALLBACK(int) fntWatchdog(RTTHREAD hThreadSelf, void *pvUser)
 {
+    RT_NOREF(hThreadSelf, pvUser);
+
     // store a log prefix for this thread
     util::AutoWriteLock thrLock(g_pThreadsLockHandle COMMA_LOCKVAL_SRC_POS);
     g_mapThreads[RTThreadSelf()] = "[W  ]";
@@ -1429,13 +1466,14 @@ int fntWatchdog(RTTHREAD ThreadSelf, void *pvUser)
 
     WEBDEBUG(("Watchdog thread started\n"));
 
+    uint32_t tNextStat = 0;
+
     while (g_fKeepRunning)
     {
         WEBDEBUG(("Watchdog: sleeping %d seconds\n", g_iWatchdogCheckInterval));
         RTThreadSleep(g_iWatchdogCheckInterval * 1000);
 
-        time_t                      tNow;
-        time(&tNow);
+        uint32_t tNow = RTTimeProgramSecTS();
 
         // we're messing with websessions, so lock them
         util::AutoWriteLock lock(g_pWebsessionsLockHandle COMMA_LOCKVAL_SRC_POS);
@@ -1465,12 +1503,34 @@ int fntWatchdog(RTTHREAD ThreadSelf, void *pvUser)
             if (pSystemProperties)
                 pSystemProperties->COMSETTER(WebServiceAuthLibrary)(com::Bstr(g_pcszAuthentication).raw());
         }
+
+        // Log some MOR usage statistics every 5 minutes, but only if there's
+        // something worth logging (at least one reference or a transition to
+        // zero references). Avoids useless log spamming in idle webservice.
+        if (tNow >= tNextStat)
+        {
+            size_t cMOR = 0;
+            it = g_mapWebsessions.begin();
+            itEnd = g_mapWebsessions.end();
+            while (it != itEnd)
+            {
+                cMOR += it->second->CountRefs();
+                ++it;
+            }
+            static bool fLastZero = false;
+            if (cMOR || !fLastZero)
+                LogRel(("Statistics: %zu websessions, %zu references\n",
+                        g_mapWebsessions.size(), cMOR));
+            fLastZero = (cMOR == 0);
+            while (tNextStat <= tNow)
+                tNextStat += 5 * 60; /* 5 minutes */
+        }
     }
 
     thrLock.acquire();
     g_mapThreads.erase(RTThreadSelf());
 
-    WebLog("ending Watchdog thread\n");
+    LogRel(("ending Watchdog thread\n"));
     return 0;
 }
 
@@ -1485,7 +1545,7 @@ int fntWatchdog(RTTHREAD ThreadSelf, void *pvUser)
  * functions, which raise specific SOAP faults.
  *
  * @param soap
- * @param str
+ * @param pcsz
  * @param extype
  * @param ex
  */
@@ -1550,19 +1610,19 @@ void RaiseSoapInvalidObjectFault(struct soap *soap,
 std::string ConvertComString(const com::Bstr &bstr)
 {
     com::Utf8Str ustr(bstr);
-    return ustr.c_str();        // @todo r=dj since the length is known, we can probably use a better std::string allocator
+    return ustr.c_str();        /// @todo r=dj since the length is known, we can probably use a better std::string allocator
 }
 
 /**
  * Return a safe C++ string from the given COM UUID,
  * without crashing if the UUID is empty.
- * @param bstr
+ * @param uuid
  * @return
  */
 std::string ConvertComString(const com::Guid &uuid)
 {
     com::Utf8Str ustr(uuid.toString());
-    return ustr.c_str();        // @todo r=dj since the length is known, we can probably use a better std::string allocator
+    return ustr.c_str();        /// @todo r=dj since the length is known, we can probably use a better std::string allocator
 }
 
 /** Code to handle string <-> byte arrays base64 conversion. */
@@ -1597,7 +1657,7 @@ void Base64DecodeByteArray(struct soap *soap, const std::string& aStr, ComSafeAr
 
     if (cbOut > DECODE_STR_MAX)
     {
-        WebLog("Decode string too long.\n");
+        LogRel(("Decode string too long.\n"));
         RaiseSoapRuntimeFault(soap, idThis, pszMethodName, E_INVALIDARG, pObj, iid);
     }
 
@@ -1605,7 +1665,7 @@ void Base64DecodeByteArray(struct soap *soap, const std::string& aStr, ComSafeAr
     int rc = RTBase64Decode(pszStr, result.raw(), cbOut, NULL, NULL);
     if (FAILED(rc))
     {
-        WebLog("String Decoding Failed. Error code: %Rrc\n", rc);
+        LogRel(("String Decoding Failed. Error code: %Rrc\n", rc));
         RaiseSoapRuntimeFault(soap, idThis, pszMethodName, E_INVALIDARG, pObj, iid);
     }
 
@@ -1633,15 +1693,15 @@ void RaiseSoapRuntimeFault(struct soap *soap,
 
     WEBDEBUG(("   error, raising SOAP exception\n"));
 
-    WebLog("API method name:            %s\n", pcszMethodName);
-    WebLog("API return code:            %#10lx (%Rhrc)\n", apirc, apirc);
+    LogRel(("API method name:            %s\n", pcszMethodName));
+    LogRel(("API return code:            %#10lx (%Rhrc)\n", apirc, apirc));
     if (info.isFullAvailable() || info.isBasicAvailable())
     {
         const com::ErrorInfo *pInfo = &info;
         do
         {
-            WebLog("COM error info result code: %#10lx (%Rhrc)\n", pInfo->getResultCode(), pInfo->getResultCode());
-            WebLog("COM error info text:        %ls\n", pInfo->getText().raw());
+            LogRel(("COM error info result code: %#10lx (%Rhrc)\n", pInfo->getResultCode(), pInfo->getResultCode()));
+            LogRel(("COM error info text:        %ls\n", pInfo->getText().raw()));
 
             pInfo = pInfo->getNext();
         }
@@ -1747,9 +1807,6 @@ class WebServiceSessionPrivate
  * Constructor for the websession object.
  *
  * Preconditions: Caller must have locked g_pWebsessionsLockHandle.
- *
- * @param username
- * @param password
  */
 WebServiceSession::WebServiceSession()
     : _uNextObjectID(1),        // avoid 0 for no real reason
@@ -1800,7 +1857,6 @@ WebServiceSession::~WebServiceSession()
  *  @return 0 if the user was successfully authenticated, or an error code
  *  otherwise.
  */
-
 int WebServiceSession::authenticate(const char *pcszUsername,
                                     const char *pcszPassword,
                                     IVirtualBox **ppVirtualBox)
@@ -1832,7 +1888,7 @@ int WebServiceSession::authenticate(const char *pcszUsername,
         systemProperties->COMGETTER(WebServiceAuthLibrary)(authLibrary.asOutParam());
         com::Utf8Str filename = authLibrary;
 
-        WEBDEBUG(("external authentication library is '%ls'\n", authLibrary.raw()));
+        LogRel(("External authentication library is '%ls'\n", authLibrary.raw()));
 
         if (filename == "null")
             // authentication disabled, let everyone in:
@@ -1849,20 +1905,24 @@ int WebServiceSession::authenticate(const char *pcszUsername,
 
                 if (RT_FAILURE(rc))
                 {
-                    WEBDEBUG(("%s() Failed to load external authentication library. Error code: %Rrc\n", __FUNCTION__, rc));
+                    WEBDEBUG(("%s() Failed to load external authentication library '%s'. Error code: %Rrc\n",
+                              __FUNCTION__, filename.c_str(), rc));
                     break;
                 }
 
                 if (RT_FAILURE(rc = RTLdrGetSymbol(hlibAuth, AUTHENTRY3_NAME, (void**)&pfnAuthEntry3)))
                 {
-                    WEBDEBUG(("%s(): Could not resolve import '%s'. Error code: %Rrc\n", __FUNCTION__, AUTHENTRY3_NAME, rc));
+                    WEBDEBUG(("%s(): Could not resolve import '%s'. Error code: %Rrc\n",
+                              __FUNCTION__, AUTHENTRY3_NAME, rc));
 
                     if (RT_FAILURE(rc = RTLdrGetSymbol(hlibAuth, AUTHENTRY2_NAME, (void**)&pfnAuthEntry2)))
                     {
-                        WEBDEBUG(("%s(): Could not resolve import '%s'. Error code: %Rrc\n", __FUNCTION__, AUTHENTRY2_NAME, rc));
+                        WEBDEBUG(("%s(): Could not resolve import '%s'. Error code: %Rrc\n",
+                                  __FUNCTION__, AUTHENTRY2_NAME, rc));
 
                         if (RT_FAILURE(rc = RTLdrGetSymbol(hlibAuth, AUTHENTRY_NAME, (void**)&pfnAuthEntry)))
-                            WEBDEBUG(("%s(): Could not resolve import '%s'. Error code: %Rrc\n", __FUNCTION__, AUTHENTRY_NAME, rc));
+                            WEBDEBUG(("%s(): Could not resolve import '%s'. Error code: %Rrc\n",
+                                      __FUNCTION__, AUTHENTRY_NAME, rc));
                     }
                 }
 
@@ -1873,36 +1933,59 @@ int WebServiceSession::authenticate(const char *pcszUsername,
         }
     }
 
-    rc = VERR_WEB_NOT_AUTHENTICATED;
-    AuthResult result;
-    if (pfnAuthEntry3)
+    if (strlen(pcszUsername) >= _1K)
     {
-        result = pfnAuthEntry3("webservice", NULL, AuthGuestNotAsked, pcszUsername, pcszPassword, NULL, true, 0);
-        WEBDEBUG(("%s(): result of AuthEntry(): %d\n", __FUNCTION__, result));
-        if (result == AuthResultAccessGranted)
-            rc = 0;
+        LogRel(("Access denied, excessive username length: %zu\n", strlen(pcszUsername)));
+        rc = VERR_WEB_NOT_AUTHENTICATED;
     }
-    else if (pfnAuthEntry2)
+    else if (strlen(pcszPassword) >= _1K)
     {
-        result = pfnAuthEntry2(NULL, AuthGuestNotAsked, pcszUsername, pcszPassword, NULL, true, 0);
-        WEBDEBUG(("%s(): result of VRDPAuth2(): %d\n", __FUNCTION__, result));
-        if (result == AuthResultAccessGranted)
-            rc = 0;
+        LogRel(("Access denied, excessive password length: %zu\n", strlen(pcszPassword)));
+        rc = VERR_WEB_NOT_AUTHENTICATED;
     }
-    else if (pfnAuthEntry)
+    else if (pfnAuthEntry3 || pfnAuthEntry2 || pfnAuthEntry)
     {
-        result = pfnAuthEntry(NULL, AuthGuestNotAsked, pcszUsername, pcszPassword, NULL);
-        WEBDEBUG(("%s(): result of VRDPAuth(%s, [%d]): %d\n", __FUNCTION__, pcszUsername, strlen(pcszPassword), result));
+        const char *pszFn;
+        AuthResult result;
+        if (pfnAuthEntry3)
+        {
+            result = pfnAuthEntry3("webservice", NULL, AuthGuestNotAsked, pcszUsername, pcszPassword, NULL, true, 0);
+            pszFn = AUTHENTRY3_NAME;
+        }
+        else if (pfnAuthEntry2)
+        {
+            result = pfnAuthEntry2(NULL, AuthGuestNotAsked, pcszUsername, pcszPassword, NULL, true, 0);
+            pszFn = AUTHENTRY2_NAME;
+        }
+        else
+        {
+            result = pfnAuthEntry(NULL, AuthGuestNotAsked, pcszUsername, pcszPassword, NULL);
+            pszFn = AUTHENTRY_NAME;
+        }
+        WEBDEBUG(("%s(): result of %s('%s', [%d]): %d (%s)\n",
+                  __FUNCTION__, pszFn, pcszUsername, strlen(pcszPassword), result, decodeAuthResult(result)));
         if (result == AuthResultAccessGranted)
-            rc = 0;
+        {
+            LogRel(("Access for user '%s' granted\n", pcszUsername));
+            rc = VINF_SUCCESS;
+        }
+        else
+        {
+            if (result == AuthResultAccessDenied)
+                LogRel(("Access for user '%s' denied\n", pcszUsername));
+            rc = VERR_WEB_NOT_AUTHENTICATED;
+        }
     }
     else if (fAuthLibLoaded)
-        // fAuthLibLoaded = true but both pointers are NULL:
-        // then the authlib was "null" and auth was disabled
-        rc = 0;
+    {
+        // fAuthLibLoaded = true but all pointers are NULL:
+        // The authlib was "null" and auth was disabled
+        rc = VINF_SUCCESS;
+    }
     else
     {
         WEBDEBUG(("Could not resolve AuthEntry, VRDPAuth2 or VRDPAuth entry point"));
+        rc = VERR_WEB_NOT_AUTHENTICATED;
     }
 
     lock.release();
@@ -1922,7 +2005,7 @@ int WebServiceSession::authenticate(const char *pcszUsername,
  *
  * Preconditions: Caller must have locked g_pWebsessionsLockHandle.
  *
- * @param pcu pointer to a COM object.
+ * @param pObject pointer to a COM object.
  * @return The existing ManagedObjectRef that represents the COM object, or NULL if there's none yet.
  */
 ManagedObjectRef* WebServiceSession::findRefFromPtr(const IUnknown *pObject)
@@ -1979,7 +2062,15 @@ WebServiceSession *WebServiceSession::findWebsessionFromRef(const WSDLT_ID &id)
  */
 void WebServiceSession::touch()
 {
-    time(&_tLastObjectLookup);
+    _tLastObjectLookup = RTTimeProgramSecTS();
+}
+
+/**
+ * Counts the number of managed object references in this websession.
+ */
+size_t WebServiceSession::CountRefs()
+{
+    return _pp->_mapManagedObjectsById.size();
 }
 
 
@@ -2118,8 +2209,9 @@ ManagedObjectRef::~ManagedObjectRef()
  *
  * Preconditions: Caller must have locked g_pWebsessionsLockHandle.
  *
- * @param strId
- * @param iter
+ * @param   id
+ * @param   pRef
+ * @param   fNullAllowed
  * @return
  */
 int ManagedObjectRef::findRefFromId(const WSDLT_ID &id,
@@ -2197,6 +2289,7 @@ int __vbox__IManagedObjectRef_USCOREgetInterfaceName(
     _vbox__IManagedObjectRef_USCOREgetInterfaceName *req,
     _vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse *resp)
 {
+    RT_NOREF(soap);
     HRESULT rc = S_OK;
     WEBDEBUG(("-- entering %s\n", __FUNCTION__));
 
@@ -2232,6 +2325,7 @@ int __vbox__IManagedObjectRef_USCORErelease(
     _vbox__IManagedObjectRef_USCORErelease *req,
     _vbox__IManagedObjectRef_USCOREreleaseResponse *resp)
 {
+    RT_NOREF(resp);
     HRESULT rc = S_OK;
     WEBDEBUG(("-- entering %s\n", __FUNCTION__));
 
@@ -2287,9 +2381,9 @@ int __vbox__IManagedObjectRef_USCORErelease(
  * a much better solution, both for performance and cleanliness, for the webservice
  * client to clean up itself.
  *
- * @param
- * @param vbox__IWebsessionManager_USCORElogon
- * @param vbox__IWebsessionManager_USCORElogonResponse
+ * @param soap
+ * @param req
+ * @param resp
  * @return
  */
 int __vbox__IWebsessionManager_USCORElogon(
@@ -2297,6 +2391,7 @@ int __vbox__IWebsessionManager_USCORElogon(
         _vbox__IWebsessionManager_USCORElogon *req,
         _vbox__IWebsessionManager_USCORElogonResponse *resp)
 {
+    RT_NOREF(soap);
     HRESULT rc = S_OK;
     WEBDEBUG(("-- entering %s\n", __FUNCTION__));
 
@@ -2376,9 +2471,8 @@ int __vbox__IWebsessionManager_USCOREgetSessionObject(
 /**
  * hard-coded implementation for IWebsessionManager::logoff.
  *
- * @param
- * @param vbox__IWebsessionManager_USCORElogon
- * @param vbox__IWebsessionManager_USCORElogonResponse
+ * @param req
+ * @param resp
  * @return
  */
 int __vbox__IWebsessionManager_USCORElogoff(
@@ -2386,16 +2480,16 @@ int __vbox__IWebsessionManager_USCORElogoff(
         _vbox__IWebsessionManager_USCORElogoff *req,
         _vbox__IWebsessionManager_USCORElogoffResponse *resp)
 {
+    RT_NOREF(resp);
     HRESULT rc = S_OK;
     WEBDEBUG(("-- entering %s\n", __FUNCTION__));
 
-    do
     {
         // findWebsessionFromRef and the websession destructor require the lock
         util::AutoWriteLock lock(g_pWebsessionsLockHandle COMMA_LOCKVAL_SRC_POS);
 
-        WebServiceSession* pWebsession;
-        if ((pWebsession = WebServiceSession::findWebsessionFromRef(req->refIVirtualBox)))
+        WebServiceSession *pWebsession = WebServiceSession::findWebsessionFromRef(req->refIVirtualBox);
+        if (pWebsession)
         {
             WEBDEBUG(("websession logoff, deleting websession %#llx\n", pWebsession->getID()));
             delete pWebsession;
@@ -2403,7 +2497,7 @@ int __vbox__IWebsessionManager_USCORElogoff(
 
             WEBDEBUG(("websession destroyed, %d websessions left open\n", g_mapWebsessions.size()));
         }
-    } while (0);
+    }
 
     WEBDEBUG(("-- leaving %s, rc: %#lx\n", __FUNCTION__, rc));
     if (FAILED(rc))
