@@ -1,10 +1,10 @@
-/* $Id$ */
+/* $Id: lwipopts.h $ */
 /** @file
  * NAT Network - lwIP configuration options.
  */
 
 /*
- * Copyright (C) 2013-2014 Oracle Corporation
+ * Copyright (C) 2013-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,6 +18,7 @@
 #ifndef _VBOX_NETNAT_LWIP_OPTS_H_
 #define _VBOX_NETNAT_LWIP_OPTS_H_
 
+#include <VBox/cdefs.h>     /* For VBOX_STRICT. */
 #include <iprt/mem.h>
 #include <iprt/alloca.h>    /* This may include malloc.h (msc), which is something that has
                              * to be done before redefining any of the functions therein. */
@@ -120,7 +121,7 @@
 
 /* PBUF_POOL_BUFSIZE: the size of each pbuf in the pbuf pool.
    Use default that is based on TCP_MSS and PBUF_LINK_HLEN.  */
-#undef PBUF_POOL_BUFSIZE  
+#undef PBUF_POOL_BUFSIZE
 
 /** Turn on support for lightweight critical region protection. Leaving this
  * off uses synchronization code in pbuf.c which is totally polluted with
@@ -148,17 +149,17 @@
 
 #define LWIP_ND6_ALLOW_RA_UPDATES       (!LWIP_IPV6_FORWARD)
 #define LWIP_IPV6_SEND_ROUTER_SOLICIT   (!LWIP_IPV6_FORWARD)
-/* IPv6 autoconfig we don't need in proxy, but it required for very seldom cases 
+/* IPv6 autoconfig we don't need in proxy, but it required for very seldom cases
  * iSCSI over intnet with IPv6
  */
-#define LWIP_IPV6_AUTOCONFIG            1 
+#define LWIP_IPV6_AUTOCONFIG            1
 #if LWIP_IPV6_FORWARD /* otherwise use the default from lwip/opt.h */
 #define LWIP_IPV6_DUP_DETECT_ATTEMPTS   0
 #endif
 
 #define LWIP_IPV6_FRAG                  1
 
-/** 
+/**
  * aka Slirp mode.
  */
 #define LWIP_CONNECTION_PROXY 1
@@ -166,7 +167,7 @@
 
 /* MEMP_NUM_SYS_TIMEOUT: the number of simultaneously active
    timeouts. */
-#define MEMP_NUM_SYS_TIMEOUT    16
+#define MEMP_NUM_SYS_TIMEOUT    256
 
 
 /* this is required for IPv6 and IGMP needs */
@@ -184,13 +185,18 @@
 #define U16_F "hu"
 #define S16_F "hd"
 #define X16_F "hx"
-#define U32_F "lu"
-#define S32_F "ld"
-#define X32_F "lx"
+#define U32_F "u"
+#define S32_F "d"
+#define X32_F "x"
 
 /* Redirect libc memory alloc functions to IPRT. */
 #define malloc(x) RTMemAlloc(x)
 #define realloc(x,y) RTMemRealloc((x), (y))
 #define free(x) RTMemFree(x)
 
+/* Align VBOX_STRICT and LWIP_NOASSERT. */
+#ifndef VBOX_STRICT
+# define LWIP_NOASSERT 1
+#endif
+
 #endif /* _VBOX_NETNAT_LWIP_OPTS_H_ */
