@@ -1,4 +1,20 @@
-/* -*- indent-tabs-mode: nil; -*- */
+/* $Id: proxy_tftpd.c $ */
+/** @file
+ * NAT Network - TFTP server.
+ */
+
+/*
+ * Copyright (C) 2013-2017 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
 #define LOG_GROUP LOG_GROUP_NAT_SERVICE
 
 #include "winutils.h"
@@ -265,7 +281,8 @@ tftpd_rrq(struct pbuf *p, ip_addr_t *addr, u16_t port)
             }
 
             if (xfer->oack != NULL) {
-                pbuf_realloc(xfer->oack, xfer->oack->len - len);
+                Assert((u16_t)(xfer->oack->len - len) == xfer->oack->len - len);
+                pbuf_realloc(xfer->oack, (u16_t)(xfer->oack->len - len));
             }
         }
     }
@@ -601,10 +618,13 @@ tftp_parse_filename(struct xfer *xfer, char **ps, size_t *plen)
         return tftp_internal_error(xfer);
     }
 
-    status = RTStrPrintf(pathname, len, "%s/%s", tftpd.root, xfer->filename);
-    if (status < 0) {
-        return tftp_internal_error(xfer);
-    }
+    RTStrPrintf(pathname, len, "%s/%s", tftpd.root, xfer->filename);
+/** @todo fix RTStrPrintf because this does not currently work:
+ *   status = RTStrPrintf(pathname, len, "%s/%s", tftpd.root, xfer->filename);
+ *   if (status < 0) {
+ *       return tftp_internal_error(xfer);
+ *   }
+ */
 
     DPRINTF(("%s: full pathname: %s\n", __func__, pathname));
     xfer->fd = open(pathname, O_RDONLY);
@@ -804,8 +824,13 @@ tftp_add_oack(char **ps, size_t *plen,
     va_list ap;
     int sz;
 
-    sz = RTStrPrintf(*ps, *plen, "%s", optname);
-    if (sz < 0 || (size_t)sz >= *plen) {
+/** @todo Fix RTStrPrintf because this doesn't really work.
+ *   sz = RTStrPrintf(*ps, *plen, "%s", optname);
+ *   if (sz < 0 || (size_t)sz >= *plen) {
+ *       return -1;
+ *   } */
+    sz = (int)RTStrPrintf(*ps, *plen, "%s", optname);
+    if (/*sz < 0 ||*/ (size_t)sz >= *plen) {
         return -1;
     }
 
