@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2015 Oracle Corporation
+ * Copyright (C) 2009-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP RTLOGGROUP_DBG
 #include <iprt/dbg.h>
 #include "internal/iprt.h"
@@ -51,9 +51,9 @@
 #include "internal/magics.h"
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /** Debug info interpreter registration record. */
 typedef struct RTDBGMODREGDBG
 {
@@ -79,9 +79,9 @@ typedef struct RTDBGMODREGIMG
 typedef RTDBGMODREGIMG *PRTDBGMODREGIMG;
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** Validates a debug module handle and returns rc if not valid. */
 #define RTDBGMOD_VALID_RETURN_RC(pDbgMod, rc) \
     do { \
@@ -105,9 +105,9 @@ typedef RTDBGMODREGIMG *PRTDBGMODREGIMG;
     } while (0)
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** Init once object for lazy registration of the built-in image and debug
  * info interpreters. */
 static RTONCE           g_rtDbgModOnce = RTONCE_INITIALIZER;
@@ -368,6 +368,8 @@ RT_EXPORT_SYMBOL(RTDbgModCreate);
 RTDECL(int) RTDbgModCreateFromMap(PRTDBGMOD phDbgMod, const char *pszFilename, const char *pszName,
                                   RTUINTPTR uSubtrahend, RTDBGCFG hDbgCfg)
 {
+    RT_NOREF_PV(hDbgCfg);
+
     /*
      * Input validation and lazy initialization.
      */
@@ -504,7 +506,8 @@ static DECLCALLBACK(int) rtDbgModExtDbgInfoOpenCallback(RTDBGCFG hDbgCfg, const
 {
     PRTDBGMODINT        pDbgMod   = (PRTDBGMODINT)pvUser1;
     PCRTLDRDBGINFO      pDbgInfo  = (PCRTLDRDBGINFO)pvUser2;
-    NOREF(pDbgInfo); /** @todo consider a more direct search for a interpreter. */
+    RT_NOREF_PV(pDbgInfo); /** @todo consider a more direct search for a interpreter. */
+    RT_NOREF_PV(hDbgCfg);
 
     Assert(!pDbgMod->pDbgVt);
     Assert(!pDbgMod->pvDbgPriv);
@@ -562,6 +565,7 @@ static DECLCALLBACK(int)
 rtDbgModOpenDebugInfoExternalToImageCallback(RTLDRMOD hLdrMod, PCRTLDRDBGINFO pDbgInfo, void *pvUser)
 {
     RTDBGMODOPENDIETI *pArgs = (RTDBGMODOPENDIETI *)pvUser;
+    RT_NOREF_PV(hLdrMod);
 
     Assert(pDbgInfo->enmType > RTLDRDBGINFOTYPE_INVALID && pDbgInfo->enmType < RTLDRDBGINFOTYPE_END);
     const char *pszExtFile = pDbgInfo->pszExtFile;
@@ -674,7 +678,8 @@ static int rtDbgModOpenDebugInfoExternalToImage(PRTDBGMODINT pDbgMod, RTDBGCFG h
 static DECLCALLBACK(int) rtDbgModExtDbgInfoOpenCallback2(RTDBGCFG hDbgCfg, const char *pszFilename, void *pvUser1, void *pvUser2)
 {
     PRTDBGMODINT        pDbgMod   = (PRTDBGMODINT)pvUser1;
-    NOREF(pvUser2); /** @todo image matching string or smth. */
+    RT_NOREF_PV(pvUser2); /** @todo image matching string or smth. */
+    RT_NOREF_PV(hDbgCfg);
 
     Assert(!pDbgMod->pDbgVt);
     Assert(!pDbgMod->pvDbgPriv);
@@ -944,6 +949,7 @@ static DECLCALLBACK(int) rtDbgModFromPeImageOpenCallback(RTDBGCFG hDbgCfg, const
     PRTDBGMODINT        pDbgMod   = (PRTDBGMODINT)pvUser1;
     PRTDBGMODDEFERRED   pDeferred = (PRTDBGMODDEFERRED)pvUser2;
     LogFlow(("rtDbgModFromPeImageOpenCallback: %s\n", pszFilename));
+    RT_NOREF_PV(hDbgCfg);
 
     Assert(pDbgMod->pImgVt == NULL);
     Assert(pDbgMod->pvImgPriv == NULL);
@@ -1211,6 +1217,7 @@ rtDbgModFromMachOImageOpenDsymMachOCallback(RTDBGCFG hDbgCfg, const char *pszFil
 {
     PRTDBGMODINT        pDbgMod = (PRTDBGMODINT)pvUser1;
     PCRTDBGMODMACHOARGS pArgs   = (PCRTDBGMODMACHOARGS)pvUser2;
+    RT_NOREF_PV(hDbgCfg);
 
     Assert(!pDbgMod->pDbgVt);
     Assert(!pDbgMod->pvDbgPriv);
@@ -1330,6 +1337,8 @@ rtDbgModFromMachOImageOpenDsymMachOCallback(RTDBGCFG hDbgCfg, const char *pszFil
 static int rtDbgModFromMachOImageWorker(PRTDBGMODINT pDbgMod, RTLDRARCH enmArch, uint32_t cbImage,
                                         uint32_t cSegs, PCRTDBGSEGMENT paSegs, PCRTUUID pUuid, RTDBGCFG hDbgCfg)
 {
+    RT_NOREF_PV(cbImage); RT_NOREF_PV(cSegs); RT_NOREF_PV(paSegs);
+
     RTDBGMODMACHOARGS Args;
     Args.enmArch    = enmArch;
     Args.pUuid      = pUuid && RTUuidIsNull(pUuid) ? pUuid : NULL;
@@ -1432,7 +1441,7 @@ RTDECL(int) RTDbgModCreateFromMachOImage(PRTDBGMOD phDbgMod, const char *pszFile
                      */
                     PRTDBGMODDEFERRED pDeferred;
                     rc = rtDbgModDeferredCreate(pDbgMod, rtDbgModFromMachOImageDeferredCallback, cbImage, hDbgCfg,
-                                                RT_OFFSETOF(RTDBGMODDEFERRED, u.MachO.aSegs[cSegs]),
+                                                RT_UOFFSETOF_DYN(RTDBGMODDEFERRED, u.MachO.aSegs[cSegs]),
                                                 &pDeferred);
                     if (RT_SUCCESS(rc))
                     {
@@ -1756,7 +1765,7 @@ RTDECL(int) RTDbgModSymbolAdd(RTDBGMOD hDbgMod, const char *pszSymbol, RTDBGSEGI
      */
     PRTDBGMODINT pDbgMod = hDbgMod;
     RTDBGMOD_VALID_RETURN_RC(pDbgMod, VERR_INVALID_HANDLE);
-    AssertPtr(pszSymbol);
+    AssertPtrReturn(pszSymbol, VERR_INVALID_POINTER);
     size_t cchSymbol = strlen(pszSymbol);
     AssertReturn(cchSymbol, VERR_DBG_SYMBOL_NAME_OUT_OF_RANGE);
     AssertReturn(cchSymbol < RTDBG_SYMBOL_NAME_LENGTH, VERR_DBG_SYMBOL_NAME_OUT_OF_RANGE);
