@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2015 Oracle Corporation
+ * Copyright (C) 2011-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "internal/iprt.h"
 #include <iprt/trace.h>
 
@@ -51,9 +51,9 @@
 #include "internal/magics.h"
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /** Alignment used to place the trace buffer members, this should be a multiple
  * of the cache line size if possible.  (We should dynamically determine it.) */
 #define RTTRACEBUF_ALIGNMENT        64
@@ -134,9 +134,9 @@ typedef struct RTTRACEBUFINT
 typedef RTTRACEBUFINT const *PCRTTRACEBUFINT;
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /**
  * Get the current CPU Id.
  */
@@ -264,7 +264,7 @@ typedef RTTRACEBUFINT const *PCRTTRACEBUFINT;
     pEntry->idCpu  = RTTRACEBUF_CUR_CPU(); \
     pszBuf  = &pEntry->szMsg[0]; \
     *pszBuf = '\0'; \
-    cchBuf  = pThis->cbEntry - RT_OFFSETOF(RTTRACEBUFENTRY, szMsg) - 1; \
+    cchBuf  = pThis->cbEntry - RT_UOFFSETOF(RTTRACEBUFENTRY, szMsg) - 1; \
     rc      = VINF_SUCCESS
 
 
