@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/rand.h>
 #include "internal/iprt.h"
 
@@ -42,9 +42,9 @@
 #include "internal/rand.h"
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** For lazily initializing of the random generator. */
 static RTONCE g_rtRandOnce = RTONCE_INITIALIZER;
 /** The default random generator. */
@@ -103,7 +103,7 @@ static DECLCALLBACK(void) rtRandTermOnce(void *pvUser, bool fLazyCleanUpOk)
 }
 
 
-RTDECL(void) RTRandBytes(void *pv, size_t cb) RT_NO_THROW
+RTDECL(void) RTRandBytes(void *pv, size_t cb) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     RTRandAdvBytes(g_hRand, pv, cb);
@@ -111,7 +111,7 @@ RTDECL(void) RTRandBytes(void *pv, size_t cb) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandBytes);
 
 
-RTDECL(uint32_t) RTRandU32Ex(uint32_t u32First, uint32_t u32Last) RT_NO_THROW
+RTDECL(uint32_t) RTRandU32Ex(uint32_t u32First, uint32_t u32Last) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvU32Ex(g_hRand, u32First, u32Last);
@@ -119,7 +119,7 @@ RTDECL(uint32_t) RTRandU32Ex(uint32_t u32First, uint32_t u32Last) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandU32Ex);
 
 
-RTDECL(uint32_t) RTRandU32(void) RT_NO_THROW
+RTDECL(uint32_t) RTRandU32(void) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvU32(g_hRand);
@@ -127,7 +127,7 @@ RTDECL(uint32_t) RTRandU32(void) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandU32);
 
 
-RTDECL(int32_t) RTRandS32Ex(int32_t i32First, int32_t i32Last) RT_NO_THROW
+RTDECL(int32_t) RTRandS32Ex(int32_t i32First, int32_t i32Last) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvS32Ex(g_hRand, i32First, i32Last);
@@ -135,7 +135,7 @@ RTDECL(int32_t) RTRandS32Ex(int32_t i32First, int32_t i32Last) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandS32Ex);
 
 
-RTDECL(int32_t) RTRandS32(void) RT_NO_THROW
+RTDECL(int32_t) RTRandS32(void) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvS32(g_hRand);
@@ -143,7 +143,7 @@ RTDECL(int32_t) RTRandS32(void) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandS32);
 
 
-RTDECL(uint64_t) RTRandU64Ex(uint64_t u64First, uint64_t u64Last) RT_NO_THROW
+RTDECL(uint64_t) RTRandU64Ex(uint64_t u64First, uint64_t u64Last) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvU64Ex(g_hRand, u64First, u64Last);
@@ -151,7 +151,7 @@ RTDECL(uint64_t) RTRandU64Ex(uint64_t u64First, uint64_t u64Last) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandU64Ex);
 
 
-RTDECL(uint64_t) RTRandU64(void) RT_NO_THROW
+RTDECL(uint64_t) RTRandU64(void) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvU64(g_hRand);
@@ -159,7 +159,7 @@ RTDECL(uint64_t) RTRandU64(void) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandU64);
 
 
-RTDECL(int64_t) RTRandS64Ex(int64_t i64First, int64_t i64Last) RT_NO_THROW
+RTDECL(int64_t) RTRandS64Ex(int64_t i64First, int64_t i64Last) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvS64Ex(g_hRand, i64First, i64Last);
@@ -167,7 +167,7 @@ RTDECL(int64_t) RTRandS64Ex(int64_t i64First, int64_t i64Last) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTRandS64Ex);
 
 
-RTDECL(int64_t) RTRandS64(void) RT_NO_THROW
+RTDECL(int64_t) RTRandS64(void) RT_NO_THROW_DEF
 {
     RTOnceEx(&g_rtRandOnce, rtRandInitOnce, rtRandTermOnce, NULL);
     return RTRandAdvS32(g_hRand);
