@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2015 Oracle Corporation
+ * Copyright (C) 2009-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define RTCRITSECTRW_WITHOUT_REMAPPING
 #define RTASSERT_QUIET
 #include <iprt/critsect.h>
@@ -44,6 +44,39 @@
 #include "internal/magics.h"
 #include "internal/strict.h"
 
+/* Two issues here, (1) the tracepoint generator uses IPRT, and (2) only one .d
+   file per module. */
+#ifdef IPRT_WITH_DTRACE
+# include IPRT_DTRACE_INCLUDE
+# ifdef IPRT_DTRACE_PREFIX
+#  define IPRT_CRITSECTRW_EXCL_ENTERED          RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_ENTERED)
+#  define IPRT_CRITSECTRW_EXCL_ENTERED_ENABLED  RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_ENTERED_ENABLED)
+#  define IPRT_CRITSECTRW_EXCL_LEAVING          RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_LEAVING)
+#  define IPRT_CRITSECTRW_EXCL_LEAVING_ENABLED  RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_LEAVING_ENABLED)
+#  define IPRT_CRITSECTRW_EXCL_BUSY             RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_BUSY)
+#  define IPRT_CRITSECTRW_EXCL_WAITING          RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_WAITING)
+#  define IPRT_CRITSECTRW_EXCL_ENTERED_SHARED   RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_ENTERED_SHARED)
+#  define IPRT_CRITSECTRW_EXCL_LEAVING_SHARED   RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_EXCL_LEAVING_SHARED)
+#  define IPRT_CRITSECTRW_SHARED_ENTERED        RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_SHARED_ENTERED)
+#  define IPRT_CRITSECTRW_SHARED_LEAVING        RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_SHARED_LEAVING)
+#  define IPRT_CRITSECTRW_SHARED_BUSY           RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_SHARED_BUSY)
+#  define IPRT_CRITSECTRW_SHARED_WAITING        RT_CONCAT(IPRT_DTRACE_PREFIX,IPRT_CRITSECTRW_SHARED_WAITING)
+# endif
+#else
+# define IPRT_CRITSECTRW_EXCL_ENTERED(a_pvCritSect, a_pszName, a_cNestings, a_cWaitingReaders, a_cWriters) do {} while (0)
+# define IPRT_CRITSECTRW_EXCL_ENTERED_ENABLED() (false)
+# define IPRT_CRITSECTRW_EXCL_LEAVING(a_pvCritSect, a_pszName, a_cNestings, a_cWaitingReaders, a_cWriters) do {} while (0)
+# define IPRT_CRITSECTRW_EXCL_LEAVING_ENABLED() (false)
+# define IPRT_CRITSECTRW_EXCL_BUSY(   a_pvCritSect, a_pszName, a_fWriteMode, a_cWaitingReaders, a_cReaders, cWriters, a_pvNativeOwnerThread) do {} while (0)
+# define IPRT_CRITSECTRW_EXCL_WAITING(a_pvCritSect, a_pszName, a_fWriteMode, a_cWaitingReaders, a_cReaders, cWriters, a_pvNativeOwnerThread) do {} while (0)
+# define IPRT_CRITSECTRW_EXCL_ENTERED_SHARED(a_pvCritSect, a_pszName, a_cNestings, a_cWaitingReaders, a_cWriters) do {} while (0)
+# define IPRT_CRITSECTRW_EXCL_LEAVING_SHARED(a_pvCritSect, a_pszName, a_cNestings, a_cWaitingReaders, a_cWriters) do {} while (0)
+# define IPRT_CRITSECTRW_SHARED_ENTERED(a_pvCritSect, a_pszName, a_cReaders, a_cWaitingWriters)     do {} while (0)
+# define IPRT_CRITSECTRW_SHARED_LEAVING(a_pvCritSect, a_pszName, a_cReaders, a_cWaitingWriters)     do {} while (0)
+# define IPRT_CRITSECTRW_SHARED_BUSY(   a_pvCritSect, a_pszName, a_pvNativeOwnerThread, a_cWaitingReaders, a_cWriters) do {} while (0)
+# define IPRT_CRITSECTRW_SHARED_WAITING(a_pvCritSect, a_pszName, a_pvNativeOwnerThread, a_cWaitingReaders, a_cWriters) do {} while (0)
+#endif
+
 
 
 RTDECL(int) RTCritSectRwInit(PRTCRITSECTRW pThis)
@@ -53,6 +86,7 @@ RTDECL(int) RTCritSectRwInit(PRTCRITSECTRW pThis)
 RT_EXPORT_SYMBOL(RTCritSectRwInit);
 
 
+
 RTDECL(int) RTCritSectRwInitEx(PRTCRITSECTRW pThis, uint32_t fFlags,
                                RTLOCKVALCLASS hClass, uint32_t uSubClass, const char *pszNameFmt, ...)
 {
@@ -60,12 +94,19 @@ RTDECL(int) RTCritSectRwInitEx(PRTCRITSECTRW pThis, uint32_t fFlags,
     AssertReturn(!(fFlags & ~( RTCRITSECT_FLAGS_NO_NESTING | RTCRITSECT_FLAGS_NO_LOCK_VAL | RTCRITSECT_FLAGS_BOOTSTRAP_HACK
                               | RTCRITSECT_FLAGS_NOP )),
                  VERR_INVALID_PARAMETER);
+    RT_NOREF_PV(hClass); RT_NOREF_PV(uSubClass); RT_NOREF_PV(pszNameFmt);
+
 
     /*
      * Initialize the structure, allocate the lock validator stuff and sems.
      */
     pThis->u32Magic         = RTCRITSECTRW_MAGIC_DEAD;
     pThis->fNeedReset       = false;
+#ifdef IN_RING0
+    pThis->fFlags           = (uint16_t)(fFlags | RTCRITSECT_FLAGS_RING0);
+#else
+    pThis->fFlags           = (uint16_t)(fFlags & ~RTCRITSECT_FLAGS_RING0);
+#endif
     pThis->u64State         = 0;
     pThis->hNativeWriter    = NIL_RTNATIVETHREAD;
     pThis->cWriterReads     = 0;
@@ -137,6 +178,11 @@ RTDECL(uint32_t) RTCritSectRwSetSubClass(PRTCRITSECTRW pThis, uint32_t uSubClass
 {
     AssertPtrReturn(pThis, RTLOCKVAL_SUB_CLASS_INVALID);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, RTLOCKVAL_SUB_CLASS_INVALID);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
 #ifdef RTCRITSECTRW_STRICT
     AssertReturn(!(pThis->fFlags & RTCRITSECT_FLAGS_NOP), RTLOCKVAL_SUB_CLASS_INVALID);
 
@@ -157,6 +203,12 @@ static int rtCritSectRwEnterShared(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPo
      */
     AssertPtr(pThis);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, VERR_SEM_DESTROYED);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
+    RT_NOREF_PV(pSrcPos);
 
 #ifdef RTCRITSECTRW_STRICT
     RTTHREAD hThreadSelf = RTThreadSelfAutoAdopt();
@@ -226,13 +278,24 @@ static int rtCritSectRwEnterShared(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPo
                     return rc9;
 #endif
                 Assert(pThis->cWriterReads < UINT32_MAX / 2);
-                ASMAtomicIncU32(&pThis->cWriterReads);
+                uint32_t const cReads = ASMAtomicIncU32(&pThis->cWriterReads); NOREF(cReads);
+                IPRT_CRITSECTRW_EXCL_ENTERED_SHARED(pThis, NULL,
+                                                    cReads + pThis->cWriteRecursions,
+                                                    (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                                    (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
+
                 return VINF_SUCCESS; /* don't break! */
             }
 
             /* If we're only trying, return already. */
             if (fTryOnly)
+            {
+                IPRT_CRITSECTRW_SHARED_BUSY(pThis, NULL,
+                                            (void *)pThis->hNativeWriter,
+                                            (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                            (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
                 return VERR_SEM_BUSY;
+            }
 
             /* Add ourselves to the queue and wait for the direction to change. */
             uint64_t c = (u64State & RTCSRW_CNT_RD_MASK) >> RTCSRW_CNT_RD_SHIFT;
@@ -249,6 +312,10 @@ static int rtCritSectRwEnterShared(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPo
 
             if (ASMAtomicCmpXchgU64(&pThis->u64State, u64State, u64OldState))
             {
+                IPRT_CRITSECTRW_SHARED_WAITING(pThis, NULL,
+                                               (void *)pThis->hNativeWriter,
+                                               (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                               (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
                 for (uint32_t iLoop = 0; ; iLoop++)
                 {
                     int rc;
@@ -256,13 +323,15 @@ static int rtCritSectRwEnterShared(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPo
                     rc = RTLockValidatorRecSharedCheckBlocking(pThis->pValidatorRead, hThreadSelf, pSrcPos, true,
                                                                RT_INDEFINITE_WAIT, RTTHREADSTATE_RW_READ, false);
                     if (RT_SUCCESS(rc))
-#else
+#elif defined(IN_RING3)
                     RTTHREAD hThreadSelf = RTThreadSelf();
                     RTThreadBlocking(hThreadSelf, RTTHREADSTATE_RW_READ, false);
 #endif
                     {
                         rc = RTSemEventMultiWait(pThis->hEvtRead, RT_INDEFINITE_WAIT);
+#ifdef IN_RING3
                         RTThreadUnblocked(hThreadSelf, RTTHREADSTATE_RW_READ);
+#endif
                         if (pThis->u32Magic != RTCRITSECTRW_MAGIC)
                             return VERR_SEM_DESTROYED;
                     }
@@ -334,8 +403,10 @@ static int rtCritSectRwEnterShared(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPo
 
     /* got it! */
     Assert((ASMAtomicReadU64(&pThis->u64State) & RTCSRW_DIR_MASK) == (RTCSRW_DIR_READ << RTCSRW_DIR_SHIFT));
+    IPRT_CRITSECTRW_SHARED_ENTERED(pThis, NULL,
+                                   (uint32_t)((u64State & RTCSRW_CNT_RD_MASK) >> RTCSRW_CNT_RD_SHIFT),
+                                   (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
     return VINF_SUCCESS;
-
 }
 
 
@@ -387,6 +458,11 @@ RTDECL(int) RTCritSectRwLeaveShared(PRTCRITSECTRW pThis)
      */
     AssertPtr(pThis);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, VERR_SEM_DESTROYED);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
 
     /*
      * Check the direction and take action accordingly.
@@ -400,6 +476,10 @@ RTDECL(int) RTCritSectRwLeaveShared(PRTCRITSECTRW pThis)
         if (RT_FAILURE(rc9))
             return rc9;
 #endif
+        IPRT_CRITSECTRW_SHARED_LEAVING(pThis, NULL,
+                                       (uint32_t)((u64State & RTCSRW_CNT_RD_MASK) >> RTCSRW_CNT_RD_SHIFT) - 1,
+                                       (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
+
         for (;;)
         {
             uint64_t c = (u64State & RTCSRW_CNT_RD_MASK) >> RTCSRW_CNT_RD_SHIFT;
@@ -445,7 +525,11 @@ RTDECL(int) RTCritSectRwLeaveShared(PRTCRITSECTRW pThis)
         if (RT_FAILURE(rc))
             return rc;
 #endif
-        ASMAtomicDecU32(&pThis->cWriterReads);
+        uint32_t cReads = ASMAtomicDecU32(&pThis->cWriterReads); NOREF(cReads);
+        IPRT_CRITSECTRW_EXCL_LEAVING_SHARED(pThis, NULL,
+                                            cReads + pThis->cWriteRecursions,
+                                            (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                            (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
     }
 
     return VINF_SUCCESS;
@@ -460,6 +544,12 @@ static int rtCritSectRwEnterExcl(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPos,
      */
     AssertPtr(pThis);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, VERR_SEM_DESTROYED);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
+    RT_NOREF_PV(pSrcPos);
 
 #ifdef RTCRITSECTRW_STRICT
     RTTHREAD hThreadSelf = NIL_RTTHREAD;
@@ -487,14 +577,24 @@ static int rtCritSectRwEnterExcl(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPos,
             return rc9;
 #endif
         Assert(pThis->cWriteRecursions < UINT32_MAX / 2);
-        ASMAtomicIncU32(&pThis->cWriteRecursions);
+        uint32_t cNestings = ASMAtomicIncU32(&pThis->cWriteRecursions); NOREF(cNestings);
+
+#ifdef IPRT_WITH_DTRACE
+        if (IPRT_CRITSECTRW_EXCL_ENTERED_ENABLED())
+        {
+            uint64_t u64State = ASMAtomicReadU64(&pThis->u64State);
+            IPRT_CRITSECTRW_EXCL_ENTERED(pThis, NULL, cNestings + pThis->cWriterReads,
+                                         (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                         (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
+        }
+#endif
         return VINF_SUCCESS;
     }
 
     /*
      * Get cracking.
      */
-    uint64_t u64State    = ASMAtomicReadU64(&pThis->u64State);
+    uint64_t u64State = ASMAtomicReadU64(&pThis->u64State);
     uint64_t u64OldState = u64State;
 
     for (;;)
@@ -568,12 +668,24 @@ static int rtCritSectRwEnterExcl(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPos,
                 if (ASMAtomicCmpXchgU64(&pThis->u64State, u64State, u64OldState))
                     break;
             }
+            IPRT_CRITSECTRW_EXCL_BUSY(pThis, NULL,
+                                      (u64State & RTCSRW_DIR_MASK) == (RTCSRW_DIR_WRITE << RTCSRW_DIR_SHIFT) /*fWrite*/,
+                                      (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                      (uint32_t)((u64State & RTCSRW_CNT_RD_MASK) >> RTCSRW_CNT_RD_SHIFT),
+                                      (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT),
+                                      (void *)pThis->hNativeWriter);
             return VERR_SEM_BUSY;
         }
 
         /*
          * Wait for our turn.
          */
+        IPRT_CRITSECTRW_EXCL_WAITING(pThis, NULL,
+                                     (u64State & RTCSRW_DIR_MASK) == (RTCSRW_DIR_WRITE << RTCSRW_DIR_SHIFT) /*fWrite*/,
+                                     (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                     (uint32_t)((u64State & RTCSRW_CNT_RD_MASK) >> RTCSRW_CNT_RD_SHIFT),
+                                     (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT),
+                                     (void *)pThis->hNativeWriter);
         for (uint32_t iLoop = 0; ; iLoop++)
         {
             int rc;
@@ -583,13 +695,15 @@ static int rtCritSectRwEnterExcl(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPos,
             rc = RTLockValidatorRecExclCheckBlocking(pThis->pValidatorWrite, hThreadSelf, pSrcPos, true,
                                                      RT_INDEFINITE_WAIT, RTTHREADSTATE_RW_WRITE, false);
             if (RT_SUCCESS(rc))
-#else
+#elif defined(IN_RING3)
             RTTHREAD hThreadSelf = RTThreadSelf();
             RTThreadBlocking(hThreadSelf, RTTHREADSTATE_RW_WRITE, false);
 #endif
             {
                 rc = RTSemEventWait(pThis->hEvtWrite, RT_INDEFINITE_WAIT);
+#ifdef IN_RING3
                 RTThreadUnblocked(hThreadSelf, RTTHREADSTATE_RW_WRITE);
+#endif
                 if (pThis->u32Magic != RTCRITSECTRW_MAGIC)
                     return VERR_SEM_DESTROYED;
             }
@@ -629,6 +743,9 @@ static int rtCritSectRwEnterExcl(PRTCRITSECTRW pThis, PCRTLOCKVALSRCPOS pSrcPos,
 #ifdef RTCRITSECTRW_STRICT
     RTLockValidatorRecExclSetOwner(pThis->pValidatorWrite, hThreadSelf, pSrcPos, true);
 #endif
+    IPRT_CRITSECTRW_EXCL_ENTERED(pThis, NULL, 1,
+                                 (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                 (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
 
     return VINF_SUCCESS;
 }
@@ -681,6 +798,11 @@ RTDECL(int) RTCritSectRwLeaveExcl(PRTCRITSECTRW pThis)
      */
     AssertPtr(pThis);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, VERR_SEM_DESTROYED);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
 
     RTNATIVETHREAD hNativeSelf = RTThreadNativeSelf();
     RTNATIVETHREAD hNativeWriter;
@@ -704,9 +826,13 @@ RTDECL(int) RTCritSectRwLeaveExcl(PRTCRITSECTRW pThis)
         ASMAtomicWriteU32(&pThis->cWriteRecursions, 0);
         ASMAtomicWriteHandle(&pThis->hNativeWriter, NIL_RTNATIVETHREAD);
 
+        uint64_t u64State = ASMAtomicReadU64(&pThis->u64State);
+        IPRT_CRITSECTRW_EXCL_LEAVING(pThis, NULL, 0,
+                                     (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                     (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
+
         for (;;)
         {
-            uint64_t u64State    = ASMAtomicReadU64(&pThis->u64State);
             uint64_t u64OldState = u64State;
 
             uint64_t c = (u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT;
@@ -747,6 +873,7 @@ RTDECL(int) RTCritSectRwLeaveExcl(PRTCRITSECTRW pThis)
             ASMNopPause();
             if (pThis->u32Magic != RTCRITSECTRW_MAGIC)
                 return VERR_SEM_DESTROYED;
+            u64State = ASMAtomicReadU64(&pThis->u64State);
         }
     }
     else
@@ -757,7 +884,16 @@ RTDECL(int) RTCritSectRwLeaveExcl(PRTCRITSECTRW pThis)
         if (RT_FAILURE(rc9))
             return rc9;
 #endif
-        ASMAtomicDecU32(&pThis->cWriteRecursions);
+        uint32_t cNestings = ASMAtomicDecU32(&pThis->cWriteRecursions); NOREF(cNestings);
+#ifdef IPRT_WITH_DTRACE
+        if (IPRT_CRITSECTRW_EXCL_LEAVING_ENABLED())
+        {
+            uint64_t u64State = ASMAtomicReadU64(&pThis->u64State);
+            IPRT_CRITSECTRW_EXCL_LEAVING(pThis, NULL, cNestings + pThis->cWriterReads,
+                                         (uint32_t)((u64State & RTCSRW_WAIT_CNT_RD_MASK) >> RTCSRW_WAIT_CNT_RD_SHIFT),
+                                         (uint32_t)((u64State & RTCSRW_CNT_WR_MASK) >> RTCSRW_CNT_WR_SHIFT));
+        }
+#endif
     }
 
     return VINF_SUCCESS;
@@ -772,6 +908,11 @@ RTDECL(bool) RTCritSectRwIsWriteOwner(PRTCRITSECTRW pThis)
      */
     AssertPtr(pThis);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, false);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
 
     /*
      * Check ownership.
@@ -786,11 +927,18 @@ RT_EXPORT_SYMBOL(RTCritSectRwIsWriteOwner);
 
 RTDECL(bool) RTCritSectRwIsReadOwner(PRTCRITSECTRW pThis, bool fWannaHear)
 {
+    RT_NOREF_PV(fWannaHear);
+
     /*
      * Validate handle.
      */
     AssertPtr(pThis);
     AssertReturn(pThis->u32Magic == RTCRITSECTRW_MAGIC, false);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
 
     /*
      * Inspect the state.
@@ -890,6 +1038,11 @@ RTDECL(int) RTCritSectRwDelete(PRTCRITSECTRW pThis)
     //Assert(pThis->cNestings == 0);
     //Assert(pThis->cLockers == -1);
     Assert(pThis->hNativeWriter == NIL_RTNATIVETHREAD);
+#ifdef IN_RING0
+    Assert(pThis->fFlags & RTCRITSECT_FLAGS_RING0);
+#else
+    Assert(!(pThis->fFlags & RTCRITSECT_FLAGS_RING0));
+#endif
 
     /*
      * Invalidate the structure and free the semaphores.
@@ -908,8 +1061,10 @@ RTDECL(int) RTCritSectRwDelete(PRTCRITSECTRW pThis)
     int rc1 = RTSemEventDestroy(hEvtWrite);     AssertRC(rc1);
     int rc2 = RTSemEventMultiDestroy(hEvtRead); AssertRC(rc2);
 
+#ifndef IN_RING0
     RTLockValidatorRecSharedDestroy(&pThis->pValidatorRead);
     RTLockValidatorRecExclDestroy(&pThis->pValidatorWrite);
+#endif
 
     return RT_SUCCESS(rc1) ? rc2 : rc1;
 }
