@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2012 Oracle Corporation
+ * Copyright (C) 2008-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -227,6 +227,18 @@ typedef struct RTDBGMODVTIMG
      */
     DECLCALLBACKMEMBER(RTLDRARCH, pfnGetArch)(PRTDBGMODINT pMod);
 
+    /**
+     * Generic method for querying image properties.
+     *
+     * @returns IPRT status code.
+     * @param   pMod            Pointer to the module structure.
+     * @param   enmProp         The property to query.
+     * @param   pvBuf           Pointer to the return buffer.
+     * @param   cbBuf           The size of the return buffer.
+     * @sa      RTLdrQueryProp
+     */
+    DECLCALLBACKMEMBER(int, pfnQueryProp)(PRTDBGMODINT pMod, RTLDRPROP enmProp, void *pvBuf, size_t cbBuf);
+
     /** For catching initialization errors (RTDBGMODVTIMG_MAGIC). */
     uint32_t    u32EndMagic;
 } RTDBGMODVTIMG;
@@ -511,10 +523,10 @@ typedef RTDBGMODVTDBG const *PCRTDBGMODVTDBG;
  *
  * @returns IPRT status code. On success the necessary method tables should be
  *          installed in @a pMod.
- * @param   pMod            Pointer to the debug module structure.
+ * @param   pDbgMod         Pointer to the debug module structure.
  * @param   pDeferred       The deferred load data.
  */
-typedef DECLCALLBACK(int) FNRTDBGMODDEFERRED(PRTDBGMODINT pMod, struct RTDBGMODDEFERRED *pDeferred);
+typedef DECLCALLBACK(int) FNRTDBGMODDEFERRED(PRTDBGMODINT pDbgMod, struct RTDBGMODDEFERRED *pDeferred);
 /** Pointer to a deferred loading callback. */
 typedef FNRTDBGMODDEFERRED *PFNRTDBGMODDEFERRED;
 
@@ -558,6 +570,18 @@ typedef struct RTDBGMODDEFERRED
             /** The CRC-32 value found in the .gnu_debuglink section. */
             uint32_t    uCrc32;
         }   GnuDebugLink;
+
+        struct
+        {
+            /** The image UUID. */
+            RTUUID          Uuid;
+            /** Image architecture. */
+            RTLDRARCH       enmArch;
+            /** Number of segment mappings. */
+            uint32_t        cSegs;
+            /** Segment mappings. */
+            RTDBGSEGMENT    aSegs[1];
+        }   MachO;
     } u;
 } RTDBGMODDEFERRED;
 /** Pointer to the deferred loading data. */
@@ -636,7 +660,7 @@ DECLHIDDEN(int) rtDbgModContainer_RemoveAll(PRTDBGMODINT pMod);
 
 DECLHIDDEN(int) rtDbgModCreateForExports(PRTDBGMODINT pDbgMod);
 DECLHIDDEN(int) rtDbgModDeferredCreate(PRTDBGMODINT pDbgMod, PFNRTDBGMODDEFERRED pfnDeferred, RTUINTPTR cbImage,
-                                       RTDBGCFG hDbgCfg, PRTDBGMODDEFERRED *ppDeferred);
+                                       RTDBGCFG hDbgCfg, size_t cbDeferred, PRTDBGMODDEFERRED *ppDeferred);
 
 DECLHIDDEN(int) rtDbgModLdrOpenFromHandle(PRTDBGMODINT pDbgMod, RTLDRMOD hLdrMod);
 
