@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,13 +25,17 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <ntstatus.h>
 typedef long NTSTATUS;                  /** @todo figure out which headers to include to get this one typedef... */
 
 #include <iprt/err.h>
+#ifdef VBOX
+# include <VBox/err.h>
+#endif
+#include <iprt/log.h>
 #include <iprt/assert.h>
 
 
@@ -52,6 +56,18 @@ RTDECL(int)  RTErrConvertFromNtStatus(long lNativeCode)
 
         case STATUS_INVALID_HANDLE:         return VERR_INVALID_HANDLE;
         case STATUS_INVALID_PARAMETER:      return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_1:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_2:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_3:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_4:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_5:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_6:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_7:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_8:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_9:    return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_10:   return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_11:   return VERR_INVALID_PARAMETER;
+        case STATUS_INVALID_PARAMETER_12:   return VERR_INVALID_PARAMETER;
         case STATUS_NO_SUCH_DEVICE:         return VERR_FILE_NOT_FOUND;
         case STATUS_NO_SUCH_FILE:           return VERR_FILE_NOT_FOUND;
         case STATUS_INVALID_DEVICE_REQUEST: return VERR_IO_BAD_COMMAND;
@@ -64,10 +80,25 @@ RTDECL(int)  RTErrConvertFromNtStatus(long lNativeCode)
         case STATUS_OBJECT_PATH_SYNTAX_BAD: return VERR_INVALID_NAME;
         case STATUS_BAD_NETWORK_PATH:       return VERR_NET_PATH_NOT_FOUND;
         case STATUS_NOT_A_DIRECTORY:        return VERR_NOT_A_DIRECTORY;
+        case STATUS_DIRECTORY_NOT_EMPTY:    return VERR_DIR_NOT_EMPTY;
+        case STATUS_SHARING_VIOLATION:      return VERR_SHARING_VIOLATION;
+        case STATUS_FILE_IS_A_DIRECTORY:    return VERR_IS_A_DIRECTORY;
+
+        case STATUS_UNEXPECTED_NETWORK_ERROR:
+                                            return VERR_NET_IO_ERROR;
+        case STATUS_INVALID_IMAGE_HASH:     return VERR_LDR_IMAGE_HASH;
+#ifdef VBOX
+        case STATUS_TRUST_FAILURE:          return VERR_SUPLIB_NT_PROCESS_UNTRUSTED_5;
+#endif
     }
 
     /* unknown error. */
+#ifndef IN_SUP_HARDENED_R3
+    AssertLogRelMsgFailed(("Unhandled error %#lx (%lu)\n", lNativeCode, lNativeCode));
+#else
+    /* hardened main has no LogRel */
     AssertMsgFailed(("Unhandled error %#lx (%lu)\n", lNativeCode, lNativeCode));
+#endif
     return VERR_UNRESOLVED_ERROR;
 }
 
