@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2015 Oracle Corporation
+ * Copyright (C) 2008-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "the-darwin-kernel.h"
 #include "internal/iprt.h"
 #include <iprt/mp.h>
@@ -40,14 +40,16 @@
 #include "r0drv/mp-r0drv.h"
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 static int32_t volatile g_cMaxCpus = -1;
 
 
 static int rtMpDarwinInitMaxCpus(void)
 {
+    IPRT_DARWIN_SAVE_EFL_AC();
+
     int32_t cCpus = -1;
     size_t  oldLen = sizeof(cCpus);
     int rc = sysctlbyname("hw.ncpu", &cCpus, &oldLen, NULL, NULL);
@@ -58,6 +60,8 @@ static int rtMpDarwinInitMaxCpus(void)
     }
 
     ASMAtomicWriteS32(&g_cMaxCpus, cCpus);
+
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return cCpus;
 }
 
@@ -159,6 +163,7 @@ RTDECL(bool) RTMpIsCpuOnline(RTCPUID idCpu)
 RTDECL(uint32_t) RTMpGetCurFrequency(RTCPUID idCpu)
 {
     /** @todo darwin R0 MP (rainy day) */
+    RT_NOREF(idCpu);
     return 0;
 }
 
@@ -166,6 +171,7 @@ RTDECL(uint32_t) RTMpGetCurFrequency(RTCPUID idCpu)
 RTDECL(uint32_t) RTMpGetMaxFrequency(RTCPUID idCpu)
 {
     /** @todo darwin R0 MP (rainy day) */
+    RT_NOREF(idCpu);
     return 0;
 }
 
@@ -186,13 +192,16 @@ RTDECL(bool) RTMpIsCpuWorkPending(void)
 static void rtmpOnAllDarwinWrapper(void *pvArg)
 {
     PRTMPARGS pArgs = (PRTMPARGS)pvArg;
+    IPRT_DARWIN_SAVE_EFL_AC();
     pArgs->pfnWorker(cpu_number(), pArgs->pvUser1, pArgs->pvUser2);
+    IPRT_DARWIN_RESTORE_EFL_AC();
 }
 
 
 RTDECL(int) RTMpOnAll(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
     RT_ASSERT_INTS_ON();
+    IPRT_DARWIN_SAVE_EFL_AC();
 
     RTMPARGS Args;
     Args.pfnWorker = pfnWorker;
@@ -201,6 +210,8 @@ RTDECL(int) RTMpOnAll(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
     Args.idCpu = NIL_RTCPUID;
     Args.cHits = 0;
     mp_rendezvous_no_intrs(rtmpOnAllDarwinWrapper, &Args);
+
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 
@@ -216,13 +227,18 @@ static void rtmpOnOthersDarwinWrapper(void *pvArg)
     PRTMPARGS pArgs = (PRTMPARGS)pvArg;
     RTCPUID idCpu = cpu_number();
     if (pArgs->idCpu != idCpu)
+    {
+        IPRT_DARWIN_SAVE_EFL_AC();
         pArgs->pfnWorker(idCpu, pArgs->pvUser1, pArgs->pvUser2);
+        IPRT_DARWIN_RESTORE_EFL_AC();
+    }
 }
 
 
 RTDECL(int) RTMpOnOthers(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
     RT_ASSERT_INTS_ON();
+    IPRT_DARWIN_SAVE_EFL_AC();
 
     RTMPARGS Args;
     Args.pfnWorker = pfnWorker;
@@ -231,6 +247,8 @@ RTDECL(int) RTMpOnOthers(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
     Args.idCpu = RTMpCpuId();
     Args.cHits = 0;
     mp_rendezvous_no_intrs(rtmpOnOthersDarwinWrapper, &Args);
+
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 
@@ -247,8 +265,10 @@ static void rtmpOnSpecificDarwinWrapper(void *pvArg)
     RTCPUID idCpu = cpu_number();
     if (pArgs->idCpu == idCpu)
     {
+        IPRT_DARWIN_SAVE_EFL_AC();
         pArgs->pfnWorker(idCpu, pArgs->pvUser1, pArgs->pvUser2);
         ASMAtomicIncU32(&pArgs->cHits);
+        IPRT_DARWIN_RESTORE_EFL_AC();
     }
 }
 
@@ -256,6 +276,7 @@ static void rtmpOnSpecificDarwinWrapper(void *pvArg)
 RTDECL(int) RTMpOnSpecific(RTCPUID idCpu, PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
     RT_ASSERT_INTS_ON();
+    IPRT_DARWIN_SAVE_EFL_AC();
 
     RTMPARGS Args;
     Args.pfnWorker = pfnWorker;
@@ -264,6 +285,8 @@ RTDECL(int) RTMpOnSpecific(RTCPUID idCpu, PFNRTMPWORKER pfnWorker, void *pvUser1
     Args.idCpu = idCpu;
     Args.cHits = 0;
     mp_rendezvous_no_intrs(rtmpOnSpecificDarwinWrapper, &Args);
+
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return Args.cHits == 1
          ? VINF_SUCCESS
          : VERR_CPU_NOT_FOUND;
@@ -276,7 +299,10 @@ RTDECL(int) RTMpPokeCpu(RTCPUID idCpu)
 
     if (g_pfnR0DarwinCpuInterrupt == NULL)
         return VERR_NOT_SUPPORTED;
+    IPRT_DARWIN_SAVE_EFL_AC(); /* paranoia */
+    /// @todo use mp_cpus_kick() when available (since 10.10)?  It's probably slower (locks, mask iteration, checks), though...
     g_pfnR0DarwinCpuInterrupt(idCpu);
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 
