@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "the-solaris-kernel.h"
 #include "internal/iprt.h"
 #include <iprt/timer.h>
@@ -46,9 +46,9 @@
 #include "internal/magics.h"
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * The internal representation of a Solaris timer handle.
  */
@@ -120,9 +120,9 @@ typedef struct RTTIMER
 } RTTIMER;
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** Validates that the timer is valid. */
 #define RTTIMER_ASSERT_VALID_RET(pTimer) \
     do \
@@ -133,9 +133,9 @@ typedef struct RTTIMER
     } while (0)
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static void rtTimerSolSingleCallbackWrapper(void *pvArg);
 static void rtTimerSolStopIt(PRTTIMER pTimer);
 
@@ -376,7 +376,7 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
      * variable sized array of ticks counts, thus the size calculation.
      */
     PRTTIMER pTimer = (PRTTIMER)RTMemAllocZ(  (fFlags & RTTIMER_FLAGS_CPU_ALL) == RTTIMER_FLAGS_CPU_ALL
-                                            ? RT_OFFSETOF(RTTIMER, u.Omni.aPerCpu[RTMpGetCount()])
+                                            ? RT_UOFFSETOF_DYN(RTTIMER, u.Omni.aPerCpu[RTMpGetCount()])
                                             : sizeof(RTTIMER));
     if (!pTimer)
         return VERR_NO_MEMORY;
@@ -531,7 +531,7 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
         /*
          * Use a large interval (1 hour) so that we don't get a timer-callback between
          * cyclic_add() and cyclic_bind(). Program the correct interval once cyclic_bind() is done.
-         * See @bugref{7691} comment #20.
+         * See @bugref{7691#c20}.
          */
         if (!pTimer->fSpecificCpu)
             FireTime.cyt_when = RTTimeSystemNanoTS() + u64First;
