@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,9 +24,10 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "internal/iprt.h"
 #include <iprt/file.h>
 
@@ -38,9 +39,9 @@
 #include "internal/file.h"
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** Set of forced set open flags for files opened read-only. */
 static unsigned g_fOpenReadSet = 0;
 
@@ -129,15 +130,25 @@ int rtFileRecalcAndValidateFlags(uint64_t *pfOpen)
             fOpen |= g_fOpenReadWriteSet;
             fOpen &= ~g_fOpenReadWriteMask;
             break;
+#ifdef RT_OS_WINDOWS
+        case RTFILE_O_ATTR_ONLY:
+            if (fOpen & RTFILE_O_ACCESS_ATTR_MASK)
+                break;
+#endif
         default:
-            AssertMsgFailed(("Invalid RW value, fOpen=%#llx\n", fOpen));
+            AssertMsgFailed(("Invalid access mode value, fOpen=%#llx\n", fOpen));
             return VERR_INVALID_PARAMETER;
     }
 
     /*
      * Validate                                                                                                                                       .
      */
+#ifdef RT_OS_WINDOWS
+    AssertMsgReturn((fOpen & RTFILE_O_ACCESS_MASK) || (fOpen & RTFILE_O_ACCESS_ATTR_MASK),
+                    ("Missing RTFILE_O_READ/WRITE/ACCESS_ATTR: fOpen=%#llx\n", fOpen), VERR_INVALID_PARAMETER);
+#else
     AssertMsgReturn(fOpen & RTFILE_O_ACCESS_MASK, ("Missing RTFILE_O_READ/WRITE: fOpen=%#llx\n", fOpen), VERR_INVALID_PARAMETER);
+#endif
 #if defined(RT_OS_WINDOWS) || defined(RT_OS_OS2)
     AssertMsgReturn(!(fOpen & (~(uint64_t)RTFILE_O_VALID_MASK | RTFILE_O_NON_BLOCK)), ("%#llx\n", fOpen), VERR_INVALID_PARAMETER);
 #else
@@ -489,13 +500,13 @@ RTDECL(int) RTFileCopyByHandlesEx(RTFILE FileSrc, RTFILE FileDst, PFNRTPROGRESS
 
                     /* advance */
                     off += cbBlock;
-                    if (pfnProgress && offNextPercent < off)
+                    if (pfnProgress && offNextPercent < off && uPercentage < 100)
                     {
-                        while (offNextPercent < off)
+                        do
                         {
                             uPercentage++;
                             offNextPercent += cbPercent;
-                        }
+                        } while (offNextPercent < off && uPercentage < 100);
                         rc = pfnProgress(uPercentage, pvUser);
                         if (RT_FAILURE(rc))
                             break;
