@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,9 +24,10 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/semaphore.h>
 #include "internal/iprt.h"
 
@@ -48,15 +49,15 @@
 # define pthread_yield() pthread_yield_np()
 #endif
 
-#if defined(RT_OS_SOLARIS) || defined(RT_OS_HAIKU)
+#if defined(RT_OS_SOLARIS) || defined(RT_OS_HAIKU) || defined(RT_OS_FREEBSD) || defined(RT_OS_NETBSD)
 # include <sched.h>
 # define pthread_yield() sched_yield()
 #endif
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 
 /** Internal representation of the POSIX implementation of an Event semaphore.
  * The POSIX implementation uses a mutex and a condition variable to implement
@@ -119,58 +120,44 @@ RTDECL(int)  RTSemEventCreateEx(PRTSEMEVENT phEventSem, uint32_t fFlags, RTLOCKV
         /*
          * Create the condition variable.
          */
-        pthread_condattr_t CondAttr;
-        rc = pthread_condattr_init(&CondAttr);
+        rc = pthread_cond_init(&pThis->Cond, NULL);
         if (!rc)
         {
-            rc = pthread_cond_init(&pThis->Cond, &CondAttr);
+            /*
+             * Create the semaphore.
+             */
+            rc = pthread_mutex_init(&pThis->Mutex, NULL);
             if (!rc)
             {
-                /*
-                 * Create the semaphore.
-                 */
-                pthread_mutexattr_t MutexAttr;
-                rc = pthread_mutexattr_init(&MutexAttr);
-                if (!rc)
-                {
-                    rc = pthread_mutex_init(&pThis->Mutex, &MutexAttr);
-                    if (!rc)
-                    {
-                        pthread_mutexattr_destroy(&MutexAttr);
-                        pthread_condattr_destroy(&CondAttr);
-
-                        ASMAtomicWriteU32(&pThis->u32State, EVENT_STATE_NOT_SIGNALED);
-                        ASMAtomicWriteU32(&pThis->cWaiters, 0);
-                        pThis->fFlags = fFlags;
+                ASMAtomicWriteU32(&pThis->u32State, EVENT_STATE_NOT_SIGNALED);
+                ASMAtomicWriteU32(&pThis->cWaiters, 0);
+                pThis->fFlags = fFlags;
 #ifdef RTSEMEVENT_STRICT
-                        if (!pszNameFmt)
-                        {
-                            static uint32_t volatile s_iSemEventAnon = 0;
-                            RTLockValidatorRecSharedInit(&pThis->Signallers, hClass, RTLOCKVAL_SUB_CLASS_ANY, pThis,
-                                                         true /*fSignaller*/, !(fFlags & RTSEMEVENT_FLAGS_NO_LOCK_VAL),
-                                                         "RTSemEvent-%u", ASMAtomicIncU32(&s_iSemEventAnon) - 1);
-                        }
-                        else
-                        {
-                            va_list va;
-                            va_start(va, pszNameFmt);
-                            RTLockValidatorRecSharedInitV(&pThis->Signallers, hClass, RTLOCKVAL_SUB_CLASS_ANY, pThis,
-                                                          true /*fSignaller*/, !(fFlags & RTSEMEVENT_FLAGS_NO_LOCK_VAL),
-                                                          pszNameFmt, va);
-                            va_end(va);
-                        }
-                        pThis->fEverHadSignallers = false;
+                if (!pszNameFmt)
+                {
+                    static uint32_t volatile s_iSemEventAnon = 0;
+                    RTLockValidatorRecSharedInit(&pThis->Signallers, hClass, RTLOCKVAL_SUB_CLASS_ANY, pThis,
+                                                 true /*fSignaller*/, !(fFlags & RTSEMEVENT_FLAGS_NO_LOCK_VAL),
+                                                 "RTSemEvent-%u", ASMAtomicIncU32(&s_iSemEventAnon) - 1);
+                }
+                else
+                {
+                    va_list va;
+                    va_start(va, pszNameFmt);
+                    RTLockValidatorRecSharedInitV(&pThis->Signallers, hClass, RTLOCKVAL_SUB_CLASS_ANY, pThis,
+                                                  true /*fSignaller*/, !(fFlags & RTSEMEVENT_FLAGS_NO_LOCK_VAL),
+                                                  pszNameFmt, va);
+                    va_end(va);
+                }
+                pThis->fEverHadSignallers = false;
+#else
+                RT_NOREF_PV(hClass); RT_NOREF_PV(pszNameFmt);
 #endif
 
-                        *phEventSem = pThis;
-                        return VINF_SUCCESS;
-                    }
-
-                    pthread_mutexattr_destroy(&MutexAttr);
-                }
-                pthread_cond_destroy(&pThis->Cond);
+                *phEventSem = pThis;
+                return VINF_SUCCESS;
             }
-            pthread_condattr_destroy(&CondAttr);
+            pthread_cond_destroy(&pThis->Cond);
         }
 
         rc = RTErrConvertFromErrno(rc);
@@ -514,6 +501,8 @@ RTDECL(void) RTSemEventSetSignaller(RTSEMEVENT hEventSem, RTTHREAD hThread)
 
     ASMAtomicWriteBool(&pThis->fEverHadSignallers, true);
     RTLockValidatorRecSharedResetOwner(&pThis->Signallers, hThread, NULL);
+#else
+    RT_NOREF_PV(hEventSem); RT_NOREF_PV(hThread);
 #endif
 }
 
@@ -528,6 +517,8 @@ RTDECL(void) RTSemEventAddSignaller(RTSEMEVENT hEventSem, RTTHREAD hThread)
 
     ASMAtomicWriteBool(&pThis->fEverHadSignallers, true);
     RTLockValidatorRecSharedAddOwner(&pThis->Signallers, hThread, NULL);
+#else
+    RT_NOREF_PV(hEventSem); RT_NOREF_PV(hThread);
 #endif
 }
 
@@ -541,6 +532,8 @@ RTDECL(void) RTSemEventRemoveSignaller(RTSEMEVENT hEventSem, RTTHREAD hThread)
     AssertReturnVoid(u32 == EVENT_STATE_NOT_SIGNALED || u32 == EVENT_STATE_SIGNALED);
 
     RTLockValidatorRecSharedRemoveOwner(&pThis->Signallers, hThread);
+#else
+    RT_NOREF_PV(hEventSem); RT_NOREF_PV(hThread);
 #endif
 }
 
