@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,12 +25,12 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #ifdef RT_OS_WINDOWS
-# include <winsock2.h>
-# include <ws2tcpip.h>
+# include <iprt/win/winsock2.h>
+# include <iprt/win/ws2tcpip.h>
 #else /* !RT_OS_WINDOWS */
 # include <errno.h>
 # include <sys/select.h>
@@ -72,9 +72,9 @@
 #include "internal/string.h"
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /* non-standard linux stuff (it seems). */
 #ifndef MSG_NOSIGNAL
 # define MSG_NOSIGNAL           0
@@ -121,9 +121,9 @@
 #endif
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * Socket handle data.
  *
@@ -211,7 +211,7 @@ DECLINLINE(void) rtSocketErrorReset(void)
  *
  * @returns iprt status code.
  */
-int rtSocketResolverError(void)
+DECLHIDDEN(int) rtSocketResolverError(void)
 {
 #ifdef RT_OS_WINDOWS
     return RTErrConvertFromWin32(WSAGetLastError());
@@ -228,6 +228,7 @@ int rtSocketResolverError(void)
             return VERR_TRY_AGAIN;
 
         default:
+            AssertLogRelMsgFailed(("Unhandled error %u\n", h_errno));
             return VERR_UNRESOLVED_ERROR;
     }
 #endif
@@ -281,7 +282,7 @@ static int rtSocketNetAddrFromAddr(RTSOCKADDRUNION const *pSrc, size_t cbSrc, PR
  * @returns IPRT status code.
  * @param   pAddr               Pointer to the generic IPRT network address.
  * @param   pDst                The source address.
- * @param   cbSrc               The size of the source address.
+ * @param   cbDst               The size of the source address.
  * @param   pcbAddr             Where to store the size of the returned address.
  *                              Optional
  */
@@ -399,7 +400,7 @@ DECLINLINE(int) rtSocketSwitchBlockingMode(RTSOCKETINT *pThis, bool fBlocking)
  * @param   ppSocket        Where to return the IPRT socket handle.
  * @param   hNative         The native handle.
  */
-int rtSocketCreateForNative(RTSOCKETINT **ppSocket, RTSOCKETNATIVE hNative)
+DECLHIDDEN(int) rtSocketCreateForNative(RTSOCKETINT **ppSocket, RTSOCKETNATIVE hNative)
 {
     RTSOCKETINT *pThis = (RTSOCKETINT *)RTMemPoolAlloc(RTMEMPOOL_DEFAULT, sizeof(*pThis));
     if (!pThis)
@@ -416,6 +417,7 @@ int rtSocketCreateForNative(RTSOCKETINT **ppSocket, RTSOCKETNATIVE hNative)
     pThis->hEvent           = WSA_INVALID_EVENT;
     pThis->fPollEvts        = 0;
     pThis->fSubscribedEvts  = 0;
+    pThis->fEventsSaved     = 0;
 #endif
     *ppSocket = pThis;
     return VINF_SUCCESS;
@@ -443,7 +445,7 @@ RTDECL(int) RTSocketFromNative(PRTSOCKET phSocket, RTHCINTPTR uNative)
  * @param   iType               The socket type (SOCK_XXX).
  * @param   iProtocol           Socket parameter, usually 0.
  */
-int rtSocketCreate(PRTSOCKET phSocket, int iDomain, int iType, int iProtocol)
+DECLHIDDEN(int) rtSocketCreate(PRTSOCKET phSocket, int iDomain, int iType, int iProtocol)
 {
     /*
      * Create the socket.
@@ -514,9 +516,9 @@ static int rtSocketCloseIt(RTSOCKETINT *pThis, bool fDestroy)
             {
                 rc = rtSocketError();
 #ifdef RT_OS_WINDOWS
-                AssertMsgFailed(("\"%s\": closesocket(%p) -> %Rrc\n", (uintptr_t)hNative, rc));
+                AssertMsgFailed(("closesocket(%p) -> %Rrc\n", (uintptr_t)hNative, rc));
 #else
-                AssertMsgFailed(("\"%s\": close(%d) -> %Rrc\n", hNative, rc));
+                AssertMsgFailed(("close(%d) -> %Rrc\n", hNative, rc));
 #endif
             }
         }
@@ -665,7 +667,7 @@ RTDECL(int) RTSocketParseInetAddress(const char *pszAddress, unsigned uPort, PRT
     RTNETADDRIPV4 IPv4Quad;
     if (rtSocketIsIPv4Numerical(pszAddress, &IPv4Quad))
     {
-        Log3(("rtSocketIsIPv4Numerical: %#x (%RTnaipv4)\n", pszAddress, IPv4Quad.u, IPv4Quad));
+        Log3(("rtSocketIsIPv4Numerical: %s -> %#x (%RTnaipv4)\n", pszAddress, IPv4Quad.u, IPv4Quad));
         RT_ZERO(*pAddr);
         pAddr->enmType      = RTNETADDRTYPE_IPV4;
         pAddr->uPort        = uPort;
@@ -775,7 +777,6 @@ RTDECL(int) RTSocketQueryAddressStr(const char *pszHost, char *pszResult, size_t
         return VERR_NET_ADDRESS_NOT_AVAILABLE;
     }
 
-    uint8_t const  *pbDummy;
     RTNETADDRTYPE   enmAddrType = RTNETADDRTYPE_INVALID;
     size_t          cchIpAddress;
     char            szIpAddress[48];
@@ -1069,6 +1070,60 @@ RTDECL(int) RTSocketWriteTo(RTSOCKET hSocket, const void *pvBuffer, size_t cbBuf
 }
 
 
+RTDECL(int) RTSocketWriteToNB(RTSOCKET hSocket, const void *pvBuffer, size_t cbBuffer, PCRTNETADDR pAddr)
+{
+    /*
+     * Validate input.
+     */
+    RTSOCKETINT *pThis = hSocket;
+    AssertPtrReturn(pThis, VERR_INVALID_HANDLE);
+    AssertReturn(pThis->u32Magic == RTSOCKET_MAGIC, VERR_INVALID_HANDLE);
+
+    /* no locking since UDP reads may be done concurrently to writes, and
+     * this is the normal use case of this code. */
+
+    int rc = rtSocketSwitchBlockingMode(pThis, false /* fBlocking */);
+    if (RT_FAILURE(rc))
+        return rc;
+
+    /* Figure out destination address. */
+    struct sockaddr *pSA = NULL;
+#ifdef RT_OS_WINDOWS
+    int cbSA = 0;
+#else
+    socklen_t cbSA = 0;
+#endif
+    RTSOCKADDRUNION u;
+    if (pAddr)
+    {
+        rc = rtSocketAddrFromNetAddr(pAddr, &u, sizeof(u), NULL);
+        if (RT_FAILURE(rc))
+            return rc;
+        pSA = &u.Addr;
+        cbSA = sizeof(u);
+    }
+
+    /*
+     * Must write all at once, otherwise it is a failure.
+     */
+#ifdef RT_OS_WINDOWS
+    int     cbNow     = cbBuffer >= RTSOCKET_MAX_WRITE ? RTSOCKET_MAX_WRITE : (int)cbBuffer;
+#else
+    size_t  cbNow     = cbBuffer >= SSIZE_MAX   ? SSIZE_MAX   :      cbBuffer;
+#endif
+    ssize_t cbWritten = sendto(pThis->hNative, (const char *)pvBuffer, cbNow, MSG_NOSIGNAL, pSA, cbSA);
+    if (RT_LIKELY((size_t)cbWritten == cbBuffer && cbWritten >= 0))
+        rc = VINF_SUCCESS;
+    else if (cbWritten < 0)
+        rc = rtSocketError();
+    else
+        rc = VERR_TOO_MUCH_DATA;
+
+    rtSocketUnlock(pThis);
+    return rc;
+}
+
+
 RTDECL(int) RTSocketSgWrite(RTSOCKET hSocket, PCRTSGBUF pSgBuf)
 {
     /*
@@ -1211,15 +1266,26 @@ RTDECL(int) RTSocketReadNB(RTSOCKET hSocket, void *pvBuffer, size_t cbBuffer, si
         rc = VINF_SUCCESS;
     }
     else
+    {
         rc = rtSocketError();
+        if (rc == VERR_TRY_AGAIN)
+        {
+            *pcbRead = 0;
+            rc = VINF_TRY_AGAIN;
+        }
+    }
 
-    if (rc == VERR_TRY_AGAIN)
-        rc = VINF_TRY_AGAIN;
 #else
     ssize_t cbRead = recv(pThis->hNative, pvBuffer, cbNow, MSG_NOSIGNAL);
     if (cbRead >= 0)
         *pcbRead = cbRead;
-    else if (errno == EAGAIN)
+    else if (   errno == EAGAIN
+# ifdef EWOULDBLOCK
+#  if EWOULDBLOCK != EAGAIN
+             || errno == EWOULDBLOCK
+#  endif
+# endif
+             )
     {
         *pcbRead = 0;
         rc = VINF_TRY_AGAIN;
@@ -1249,13 +1315,12 @@ RTDECL(int) RTSocketWriteNB(RTSOCKET hSocket, const void *pvBuffer, size_t cbBuf
         return rc;
 
     rtSocketErrorReset();
-#ifdef RTSOCKET_MAX_WRITE
-    int    cbNow = cbBuffer >= RTSOCKET_MAX_WRITE ? RTSOCKET_MAX_WRITE : (int)cbBuffer;
-#else
-    size_t cbNow = cbBuffer;
-#endif
-
 #ifdef RT_OS_WINDOWS
+# ifdef RTSOCKET_MAX_WRITE
+    int    cbNow = cbBuffer >= RTSOCKET_MAX_WRITE ? RTSOCKET_MAX_WRITE : (int)cbBuffer;
+# else
+    size_t cbNow = cbBuffer;
+# endif
     int cbWritten = send(pThis->hNative, (const char *)pvBuffer, cbNow, MSG_NOSIGNAL);
     if (cbWritten >= 0)
     {
@@ -1263,15 +1328,25 @@ RTDECL(int) RTSocketWriteNB(RTSOCKET hSocket, const void *pvBuffer, size_t cbBuf
         rc = VINF_SUCCESS;
     }
     else
+    {
         rc = rtSocketError();
-
-    if (rc == VERR_TRY_AGAIN)
-        rc = VINF_TRY_AGAIN;
+        if (rc == VERR_TRY_AGAIN)
+        {
+            *pcbWritten = 0;
+            rc = VINF_TRY_AGAIN;
+        }
+    }
 #else
     ssize_t cbWritten = send(pThis->hNative, pvBuffer, cbBuffer, MSG_NOSIGNAL);
     if (cbWritten >= 0)
         *pcbWritten = cbWritten;
-    else if (errno == EAGAIN)
+    else if (   errno == EAGAIN
+# ifdef EWOULDBLOCK
+#  if EWOULDBLOCK != EAGAIN
+             || errno == EWOULDBLOCK
+#  endif
+# endif
+            )
     {
         *pcbWritten = 0;
         rc = VINF_TRY_AGAIN;
@@ -1392,7 +1467,7 @@ RTDECL(int) RTSocketSelectOne(RTSOCKET hSocket, RTMSINTERVAL cMillies)
     AssertReturn(pThis->u32Magic == RTSOCKET_MAGIC, VERR_INVALID_HANDLE);
     AssertReturn(RTMemPoolRefCount(pThis) >= (pThis->cUsers ? 2U : 1U), VERR_CALLER_NO_REFERENCE);
     int const fdMax = (int)pThis->hNative + 1;
-    AssertReturn(fdMax - 1 == pThis->hNative, VERR_INTERNAL_ERROR_5);
+    AssertReturn((RTSOCKETNATIVE)(fdMax - 1) == pThis->hNative, VERR_INTERNAL_ERROR_5);
 
     /*
      * Set up the file descriptor sets and do the select.
@@ -1424,8 +1499,7 @@ RTDECL(int) RTSocketSelectOne(RTSOCKET hSocket, RTMSINTERVAL cMillies)
 }
 
 
-RTDECL(int) RTSocketSelectOneEx(RTSOCKET hSocket, uint32_t fEvents, uint32_t *pfEvents,
-                                RTMSINTERVAL cMillies)
+RTDECL(int) RTSocketSelectOneEx(RTSOCKET hSocket, uint32_t fEvents, uint32_t *pfEvents, RTMSINTERVAL cMillies)
 {
     /*
      * Validate input.
@@ -1436,8 +1510,18 @@ RTDECL(int) RTSocketSelectOneEx(RTSOCKET hSocket, uint32_t fEvents, uint32_t *pf
     AssertPtrReturn(pfEvents, VERR_INVALID_PARAMETER);
     AssertReturn(!(fEvents & ~RTSOCKET_EVT_VALID_MASK), VERR_INVALID_PARAMETER);
     AssertReturn(RTMemPoolRefCount(pThis) >= (pThis->cUsers ? 2U : 1U), VERR_CALLER_NO_REFERENCE);
-    int const fdMax = (int)pThis->hNative + 1;
-    AssertReturn(fdMax - 1 == pThis->hNative, VERR_INTERNAL_ERROR_5);
+
+    RTSOCKETNATIVE hNative = pThis->hNative;
+    if (hNative == NIL_RTSOCKETNATIVE)
+    {
+        /* Socket is already closed? Possible we raced someone calling rtSocketCloseIt.
+           Should we return a different status code? */
+        *pfEvents = RTSOCKET_EVT_ERROR;
+        return VINF_SUCCESS;
+    }
+
+    int const fdMax = (int)hNative + 1;
+    AssertReturn((RTSOCKETNATIVE)(fdMax - 1) == hNative, VERR_INTERNAL_ERROR_5);
 
     *pfEvents = 0;
 
@@ -1452,11 +1536,11 @@ RTDECL(int) RTSocketSelectOneEx(RTSOCKET hSocket, uint32_t fEvents, uint32_t *pf
     FD_ZERO(&fdsetE);
 
     if (fEvents & RTSOCKET_EVT_READ)
-        FD_SET(pThis->hNative, &fdsetR);
+        FD_SET(hNative, &fdsetR);
     if (fEvents & RTSOCKET_EVT_WRITE)
-        FD_SET(pThis->hNative, &fdsetW);
+        FD_SET(hNative, &fdsetW);
     if (fEvents & RTSOCKET_EVT_ERROR)
-        FD_SET(pThis->hNative, &fdsetE);
+        FD_SET(hNative, &fdsetE);
 
     int rc;
     if (cMillies == RT_INDEFINITE_WAIT)
@@ -1470,14 +1554,22 @@ RTDECL(int) RTSocketSelectOneEx(RTSOCKET hSocket, uint32_t fEvents, uint32_t *pf
     }
     if (rc > 0)
     {
-        if (FD_ISSET(pThis->hNative, &fdsetR))
-            *pfEvents |= RTSOCKET_EVT_READ;
-        if (FD_ISSET(pThis->hNative, &fdsetW))
-            *pfEvents |= RTSOCKET_EVT_WRITE;
-        if (FD_ISSET(pThis->hNative, &fdsetE))
-            *pfEvents |= RTSOCKET_EVT_ERROR;
-
-        rc = VINF_SUCCESS;
+        if (pThis->hNative == hNative)
+        {
+            if (FD_ISSET(hNative, &fdsetR))
+                *pfEvents |= RTSOCKET_EVT_READ;
+            if (FD_ISSET(hNative, &fdsetW))
+                *pfEvents |= RTSOCKET_EVT_WRITE;
+            if (FD_ISSET(hNative, &fdsetE))
+                *pfEvents |= RTSOCKET_EVT_ERROR;
+            rc = VINF_SUCCESS;
+        }
+        else
+        {
+            /* Socket was closed while we waited (rtSocketCloseIt).  Different status code? */
+            *pfEvents = RTSOCKET_EVT_ERROR;
+            rc = VINF_SUCCESS;
+        }
     }
     else if (rc == 0)
         rc = VERR_TIMEOUT;
@@ -1586,7 +1678,7 @@ RTDECL(int) RTSocketGetPeerAddress(RTSOCKET hSocket, PRTNETADDR pAddr)
  * @param   hSocket             The socket handle.
  * @param   pAddr               The address to bind to.
  */
-int rtSocketBind(RTSOCKET hSocket, PCRTNETADDR pAddr)
+DECLHIDDEN(int) rtSocketBind(RTSOCKET hSocket, PCRTNETADDR pAddr)
 {
     RTSOCKADDRUNION u;
     int             cbAddr;
@@ -1606,7 +1698,7 @@ int rtSocketBind(RTSOCKET hSocket, PCRTNETADDR pAddr)
  *                              friends).
  * @param   cbAddr              The size of the address.
  */
-int rtSocketBindRawAddr(RTSOCKET hSocket, void const *pvAddr, size_t cbAddr)
+DECLHIDDEN(int) rtSocketBindRawAddr(RTSOCKET hSocket, void const *pvAddr, size_t cbAddr)
 {
     /*
      * Validate input.
@@ -1636,7 +1728,7 @@ int rtSocketBindRawAddr(RTSOCKET hSocket, void const *pvAddr, size_t cbAddr)
  * @param   hSocket             The socket handle.
  * @param   cMaxPending         The max number of pending connections.
  */
-int rtSocketListen(RTSOCKET hSocket, int cMaxPending)
+DECLHIDDEN(int) rtSocketListen(RTSOCKET hSocket, int cMaxPending)
 {
     /*
      * Validate input.
@@ -1667,7 +1759,7 @@ int rtSocketListen(RTSOCKET hSocket, int cMaxPending)
  *                              @a pAddr point to.  On return this contains the
  *                              size of what's stored at @a pAddr.
  */
-int rtSocketAccept(RTSOCKET hSocket, PRTSOCKET phClient, struct sockaddr *pAddr, size_t *pcbAddr)
+DECLHIDDEN(int) rtSocketAccept(RTSOCKET hSocket, PRTSOCKET phClient, struct sockaddr *pAddr, size_t *pcbAddr)
 {
     /*
      * Validate input.
@@ -1726,7 +1818,7 @@ int rtSocketAccept(RTSOCKET hSocket, PRTSOCKET phClient, struct sockaddr *pAddr,
  *                              Use RT_TCPCLIENTCONNECT_DEFAULT_WAIT to wait for the default time
  *                              configured on the running system.
  */
-int rtSocketConnect(RTSOCKET hSocket, PCRTNETADDR pAddr, RTMSINTERVAL cMillies)
+DECLHIDDEN(int) rtSocketConnect(RTSOCKET hSocket, PCRTNETADDR pAddr, RTMSINTERVAL cMillies)
 {
     /*
      * Validate input.
@@ -1815,6 +1907,35 @@ int rtSocketConnect(RTSOCKET hSocket, PCRTNETADDR pAddr, RTMSINTERVAL cMillies)
 }
 
 
+/**
+ * Wrapper around connect, raw address, no timeout.
+ *
+ * @returns IPRT status code.
+ * @param   hSocket             The socket handle.
+ * @param   pvAddr              The raw socket address to connect to.
+ * @param   cbAddr              The size of the raw address.
+ */
+DECLHIDDEN(int) rtSocketConnectRaw(RTSOCKET hSocket, void const *pvAddr, size_t cbAddr)
+{
+    /*
+     * Validate input.
+     */
+    RTSOCKETINT *pThis = hSocket;
+    AssertPtrReturn(pThis, VERR_INVALID_HANDLE);
+    AssertReturn(pThis->u32Magic == RTSOCKET_MAGIC, VERR_INVALID_HANDLE);
+    AssertReturn(rtSocketTryLock(pThis), VERR_CONCURRENT_ACCESS);
+
+    int rc;
+    if (connect(pThis->hNative, (const struct sockaddr *)pvAddr, (int)cbAddr) == 0)
+        rc = VINF_SUCCESS;
+    else
+        rc = rtSocketError();
+
+    rtSocketUnlock(pThis);
+    return rc;
+}
+
+
 /**
  * Wrapper around setsockopt.
  *
@@ -1825,7 +1946,7 @@ int rtSocketConnect(RTSOCKET hSocket, PCRTNETADDR pAddr, RTMSINTERVAL cMillies)
  * @param   pvValue             The value buffer.
  * @param   cbValue             The size of the value pointed to by pvValue.
  */
-int rtSocketSetOpt(RTSOCKET hSocket, int iLevel, int iOption, void const *pvValue, int cbValue)
+DECLHIDDEN(int) rtSocketSetOpt(RTSOCKET hSocket, int iLevel, int iOption, void const *pvValue, int cbValue)
 {
     /*
      * Validate input.
@@ -1853,9 +1974,10 @@ int rtSocketSetOpt(RTSOCKET hSocket, int iLevel, int iOption, void const *pvValu
  * @param   fEvents             The events we're polling for.
  * @param   phNative            Where to put the primary handle.
  */
-int rtSocketPollGetHandle(RTSOCKET hSocket, uint32_t fEvents, PRTHCINTPTR phNative)
+DECLHIDDEN(int) rtSocketPollGetHandle(RTSOCKET hSocket, uint32_t fEvents, PRTHCINTPTR phNative)
 {
     RTSOCKETINT *pThis = hSocket;
+    RT_NOREF_PV(fEvents);
     AssertPtrReturn(pThis, VERR_INVALID_HANDLE);
     AssertReturn(pThis->u32Magic == RTSOCKET_MAGIC, VERR_INVALID_HANDLE);
 #ifdef RT_OS_WINDOWS
@@ -2052,7 +2174,7 @@ static uint32_t rtSocketPollCheck(RTSOCKETINT *pThis, uint32_t fEvents)
  * @remarks There is a potential race wrt duplicate handles when @a fNoWait is
  *          @c true, we don't currently care about that oddity...
  */
-uint32_t rtSocketPollStart(RTSOCKET hSocket, RTPOLLSET hPollSet, uint32_t fEvents, bool fFinalEntry, bool fNoWait)
+DECLHIDDEN(uint32_t) rtSocketPollStart(RTSOCKET hSocket, RTPOLLSET hPollSet, uint32_t fEvents, bool fFinalEntry, bool fNoWait)
 {
     RTSOCKETINT *pThis = hSocket;
     AssertPtrReturn(pThis, UINT32_MAX);
@@ -2124,13 +2246,14 @@ uint32_t rtSocketPollStart(RTSOCKET hSocket, RTPOLLSET hPollSet, uint32_t fEvent
  *                              set was processed).
  * @param   fHarvestEvents      Set if we should check for pending events.
  */
-uint32_t rtSocketPollDone(RTSOCKET hSocket, uint32_t fEvents, bool fFinalEntry, bool fHarvestEvents)
+DECLHIDDEN(uint32_t) rtSocketPollDone(RTSOCKET hSocket, uint32_t fEvents, bool fFinalEntry, bool fHarvestEvents)
 {
     RTSOCKETINT *pThis = hSocket;
     AssertPtrReturn(pThis, 0);
     AssertReturn(pThis->u32Magic == RTSOCKET_MAGIC, 0);
     Assert(pThis->cUsers > 0);
     Assert(pThis->hPollSet != NIL_RTPOLLSET);
+    RT_NOREF_PV(fFinalEntry);
 
     /* Harvest events and clear the event mask for the next round of polling. */
     uint32_t fRetEvents = rtSocketPollCheck(pThis, fEvents);
