@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2015 Oracle Corporation
+ * Copyright (C) 2009-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/initterm.h>
 
 #include <iprt/test.h>
@@ -35,15 +35,17 @@
 #include <iprt/err.h>
 #include <iprt/initterm.h>
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 static uint32_t g_cCalls;
 static uint32_t g_fCalled;
 
 
 static DECLCALLBACK(void) tstTermCallback0(RTTERMREASON enmReason, int32_t iStatus, void *pvUser)
 {
+    RT_NOREF_PV(enmReason); RT_NOREF_PV(iStatus);
     RTTESTI_CHECK(pvUser == (void *)0);
     g_cCalls++;
     g_fCalled |= RT_BIT_32(0);
@@ -52,6 +54,7 @@ static DECLCALLBACK(void) tstTermCallback0(RTTERMREASON enmReason, int32_t iStat
 
 static DECLCALLBACK(void) tstTermCallback1(RTTERMREASON enmReason, int32_t iStatus, void *pvUser)
 {
+    RT_NOREF_PV(enmReason); RT_NOREF_PV(iStatus);
     RTTESTI_CHECK(pvUser == (void *)1);
     g_cCalls++;
     g_fCalled |= RT_BIT_32(1);
@@ -60,6 +63,7 @@ static DECLCALLBACK(void) tstTermCallback1(RTTERMREASON enmReason, int32_t iStat
 
 static DECLCALLBACK(void) tstTermCallback2(RTTERMREASON enmReason, int32_t iStatus, void *pvUser)
 {
+    RT_NOREF_PV(enmReason); RT_NOREF_PV(iStatus);
     RTTESTI_CHECK(pvUser == (void *)2);
     g_cCalls++;
     g_fCalled |= RT_BIT_32(2);
@@ -68,13 +72,14 @@ static DECLCALLBACK(void) tstTermCallback2(RTTERMREASON enmReason, int32_t iStat
 
 static DECLCALLBACK(void) tstTermCallback3(RTTERMREASON enmReason, int32_t iStatus, void *pvUser)
 {
+    RT_NOREF_PV(enmReason); RT_NOREF_PV(iStatus);
     RTTESTI_CHECK(pvUser == (void *)3);
     g_cCalls++;
     g_fCalled |= RT_BIT_32(3);
 }
 
 
-int main(int argc, char **argv)
+int main()
 {
     RTTEST hTest;
     int rc = RTTestInitAndCreate("tstTermCallback", &hTest);
