@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2015 Oracle Corporation
+ * Copyright (C) 2013-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/zip.h>
 
 #include <iprt/buildconfig.h>
@@ -47,9 +47,9 @@
 #include <iprt/zip.h>
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * Cache file type.
  */
@@ -80,9 +80,9 @@ typedef struct RTDBGSYMCACHEADDCFG
 typedef RTDBGSYMCACHEADDCFG const *PCRTDBGSYMCACHEADDCFG;
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** Bundle suffixes. */
 static const char * const g_apszBundleSuffixes[] =
 {
@@ -143,9 +143,9 @@ static const char * const g_apszDSymBundleSuffixes[] =
 };
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static int rtDbgSymCacheAddDirWorker(char *pszPath, size_t cchPath, PRTDIRENTRYEX pDirEntry, PCRTDBGSYMCACHEADDCFG pCfg);
 
 
@@ -219,7 +219,6 @@ static int rtDbgSymCacheAddCreateUuidMapping(const char *pszCacheFile, PRTUUID p
     /*
      * Create the directories in the path.
      */
-    char chSaved = RTPATH_SLASH;
     for (unsigned i = 0; i < 6; i++, cch += 5)
     {
         Assert(szMapPath[cch] == '-');
@@ -575,6 +574,7 @@ static int rtDbgSymCacheAddImageFile(const char *pszPath, const char *pszExtraSu
 static int rtDbgSymCacheAddDebugMachO(const char *pszPath, PCRTDBGSYMCACHEADDCFG pCfg)
 {
     /* This shouldn't happen, figure out what to do if it does. */
+    RT_NOREF_PV(pCfg);
     return RTMsgErrorRc(VERR_NOT_IMPLEMENTED,
                         "'%s' is an OS X image file, did you point me to a file inside a .dSYM or .sym file?",
                         pszPath);
@@ -591,7 +591,8 @@ static int rtDbgSymCacheAddDebugMachO(const char *pszPath, PCRTDBGSYMCACHEADDCFG
  */
 static int rtDbgSymCacheAddDebugPdb(const char *pszPath, PCRTDBGSYMCACHEADDCFG pCfg, RTFILE hFile)
 {
-    return RTMsgErrorRc(VERR_NOT_IMPLEMENTED, "PDB support not implemented: '%s'");
+    RT_NOREF2(pCfg, hFile);
+    return RTMsgErrorRc(VERR_NOT_IMPLEMENTED, "PDB support not implemented: '%s'", pszPath);
 }
 
 
@@ -946,7 +947,7 @@ static int rtDbgSymCacheAddDirWorker(char *pszPath, size_t cchPath, PRTDIRENTRYE
     /*
      * Open the directory.
      */
-    PRTDIR pDir;
+    RTDIR hDir;
     int rc, rc2;
     if (pCfg->pszFilter)
     {
@@ -956,10 +957,10 @@ static int rtDbgSymCacheAddDirWorker(char *pszPath, size_t cchPath, PRTDIRENTRYE
             pszPath[cchPath] = '\0';
             return RTMsgErrorRc(rc, "Filename too long (%Rrc): '%s" RTPATH_SLASH_STR "%s'", rc, pszPath, pCfg->pszFilter);
         }
-        rc = RTDirOpenFiltered(&pDir, pszPath, RTDIRFILTER_WINNT, 0 /*fFlags*/);
+        rc = RTDirOpenFiltered(&hDir, pszPath, RTDIRFILTER_WINNT, 0 /*fFlags*/);
     }
     else
-        rc = RTDirOpen(&pDir, pszPath);
+        rc = RTDirOpen(&hDir, pszPath);
     if (RT_FAILURE(rc))
         return RTMsgErrorRc(rc, "RTDirOpen%s failed on '%s': %Rrc", pCfg->pszFilter ? "Filtered" : "", pszPath, rc);
 
@@ -968,7 +969,7 @@ static int rtDbgSymCacheAddDirWorker(char *pszPath, size_t cchPath, PRTDIRENTRYE
      */
     for (;;)
     {
-        rc2 = RTDirReadEx(pDir, pDirEntry, NULL, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
+        rc2 = RTDirReadEx(hDir, pDirEntry, NULL, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
         if (RT_FAILURE(rc2))
         {
             pszPath[cchPath] = '\0';
@@ -1047,7 +1048,7 @@ static int rtDbgSymCacheAddDirWorker(char *pszPath, size_t cchPath, PRTDIRENTRYE
     /*
      * Clean up.
      */
-    rc2 = RTDirClose(pDir);
+    rc2 = RTDirClose(hDir);
     if (RT_FAILURE(rc2))
     {
         RTMsgError("RTDirClose failed in '%s': %Rrc", pszPath, rc);
@@ -1108,6 +1109,7 @@ static int rtDbgSymCacheAddDir(const char *pszPath, PCRTDBGSYMCACHEADDCFG pCfg)
 static RTEXITCODE rtDbgSymCacheAddFileOrDir(const char *pszPath, const char *pszCache, bool fRecursive,
                                             bool fOverwriteOnConflict)
 {
+    RT_NOREF1(fOverwriteOnConflict);
     RTDBGSYMCACHEADDCFG Cfg;
     Cfg.fRecursive      = fRecursive;
     Cfg.pszCache        = pszCache;
@@ -1123,7 +1125,7 @@ static RTEXITCODE rtDbgSymCacheAddFileOrDir(const char *pszPath, const char *psz
 
         case RTDBGSYMCACHEFILETYPE_DIR_FILTER:
             Cfg.pszFilter = RTPathFilename(pszPath);
-            /* fall thru */
+            RT_FALL_THRU();
         case RTDBGSYMCACHEFILETYPE_DIR:
             rc = rtDbgSymCacheAddDir(pszPath, &Cfg);
             break;
@@ -1159,7 +1161,7 @@ static RTEXITCODE rtDbgSymCacheAddFileOrDir(const char *pszPath, const char *psz
         }
 
         case RTDBGSYMCACHEFILETYPE_IGNORE:
-            rc = RTMsgErrorRc(VERR_INVALID_PARAMETER, "Invalid file: '%s'");
+            rc = RTMsgErrorRc(VERR_INVALID_PARAMETER, "Invalid file: '%s'", pszPath);
             break;
     }
     return RT_SUCCESS(rc) ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
@@ -1195,7 +1197,7 @@ static RTEXITCODE rtDbgSymCacheCmdAdd(const char *pszArg0, int cArgs, char **pap
     if (RT_FAILURE(rc))
         return RTMsgErrorExit(RTEXITCODE_FAILURE, "RTGetOptInit failed: %Rrc", rc);
 
-    uint32_t        cAdded = 0;
+    //uint32_t        cAdded = 0;
     RTGETOPTUNION   ValueUnion;
     int             chOpt;
     while ((chOpt = RTGetOpt(&State, &ValueUnion)) != 0)
