@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2015 Oracle Corporation
+ * Copyright (C) 2008-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +16,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_DBGF
 #include <VBox/vmm/dbgf.h>
 #include <VBox/vmm/mm.h>
@@ -39,9 +39,9 @@
 #include <iprt/path.h>
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 
 #define DBGF_PLUG_IN_READ_LOCK(pUVM) \
     do { int rcLock = RTCritSectRwEnterShared(&pUVM->dbgf.s.CritSect); AssertRC(rcLock); } while (0)
@@ -57,9 +57,9 @@
 #define DBGFPLUGIN_MAX_NAME     64
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * Plug-in tracking record.
  */
@@ -80,9 +80,9 @@ typedef struct DBGFPLUGIN
 typedef DBGFPLUGIN *PDBGFPLUGIN;
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static DECLCALLBACK(void) dbgfPlugInUnloadAll(PUVM pUVM);
 static FNDBGFHANDLERINT dbgfR3PlugInInfoList;
 
@@ -226,7 +226,7 @@ static int dbgfR3PlugInTryLoad(PDBGFPLUGIN pPlugIn, const char *pszModule, PRTER
         rc = RTLdrGetSymbol(pPlugIn->hLdrMod, DBGF_PLUG_IN_ENTRYPOINT, (void **)&pPlugIn->pfnEntry);
         if (RT_SUCCESS(rc))
         {
-            LogRel(("DBGF: Loaded Plug-In '%s' (%s).\n", pPlugIn->szName, pszModule));
+            LogRel(("DBGF: Loaded Plug-In '%s' (%s)\n", pPlugIn->szName, pszModule));
             return VINF_SUCCESS;
         }
 
@@ -312,7 +312,7 @@ static DECLCALLBACK(int) dbgfR3PlugInLoad(PUVM pUVM, const char *pszName, const
      * Create a module structure and we can pass around via RTPathTraverseList if needed.
      */
     size_t cbName = strlen(pszName) + 1;
-    pPlugIn = (PDBGFPLUGIN)MMR3HeapAllocZU(pUVM, MM_TAG_DBGF, RT_UOFFSETOF(DBGFPLUGIN, szName[cbName]));
+    pPlugIn = (PDBGFPLUGIN)MMR3HeapAllocZU(pUVM, MM_TAG_DBGF, RT_UOFFSETOF_DYN(DBGFPLUGIN, szName[cbName]));
     if (RT_UNLIKELY(!pPlugIn))
     {
         DBGF_PLUG_IN_WRITE_UNLOCK(pUVM);
@@ -380,7 +380,7 @@ static DECLCALLBACK(int) dbgfR3PlugInLoad(PUVM pUVM, const char *pszName, const
         }
 
         RTErrInfoSet(pErrInfo, rc, "Plug-in init failed");
-        LogRel(("DBGF: Plug-in '%s' failed during init: %Rrc\n", pPlugIn->szName));
+        LogRel(("DBGF: Plug-in '%s' failed during init: %Rrc\n", pPlugIn->szName, rc));
         RTLdrClose(pPlugIn->hLdrMod);
     }
     MMR3HeapFree(pPlugIn);
@@ -470,15 +470,15 @@ VMMR3DECL(void) DBGFR3PlugInLoadAll(PUVM pUVM)
     AssertRCReturnVoid(rc);
     strcat(szPath, pszSuff);
 
-    PRTDIR pDir;
-    rc = RTDirOpenFiltered(&pDir, szPath, RTDIRFILTER_WINNT, 0);
+    RTDIR hDir;
+    rc = RTDirOpenFiltered(&hDir, szPath, RTDIRFILTER_WINNT, 0 /*fFlags*/);
     if (RT_SUCCESS(rc))
     {
         /*
          * Now read it and try load each of the plug-in modules.
          */
         RTDIRENTRY DirEntry;
-        while (RT_SUCCESS(RTDirRead(pDir, &DirEntry, NULL)))
+        while (RT_SUCCESS(RTDirRead(hDir, &DirEntry, NULL)))
         {
             szPath[offDir] = '\0';
             rc = RTPathAppend(szPath, sizeof(szPath), DirEntry.szName);
@@ -495,7 +495,7 @@ VMMR3DECL(void) DBGFR3PlugInLoadAll(PUVM pUVM)
             }
         }
 
-        RTDirClose(pDir);
+        RTDirClose(hDir);
     }
 }
 
@@ -543,6 +543,7 @@ VMMR3DECL(int) DBGFR3PlugInUnload(PUVM pUVM, const char *pszName)
         pPlugIn->pfnEntry = NULL;
         pPlugIn->hLdrMod  = NIL_RTLDRMOD;
         MMR3HeapFree(pPlugIn->pNext);
+        rc = VINF_SUCCESS;
     }
     else
         rc = VERR_NOT_FOUND;
@@ -595,11 +596,12 @@ VMMR3DECL(void) DBGFR3PlugInUnloadAll(PUVM pUVM)
 
 
 /**
- * @interface_method_impl{FNDBGFHANDLERINT, The 'plugins' info item.}
+ * @callback_method_impl{FNDBGFHANDLERINT, The 'plugins' info item.}
  */
 static DECLCALLBACK(void) dbgfR3PlugInInfoList(PVM pVM, PCDBGFINFOHLP pHlp, const char *pszArgs)
 {
     PDBGFPLUGIN pPlugIn = pVM->pUVM->dbgf.s.pPlugInHead;
+    RT_NOREF_PV(pszArgs);
     if (pPlugIn)
     {
         pHlp->pfnPrintf(pHlp, "Debugging plug-in%s: %s", pPlugIn->pNext ? "s" : "", pPlugIn->szName);
