@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +16,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_PDM//_CRITSECT
 #include "PDMInternal.h"
 #include <VBox/vmm/pdmcritsect.h>
@@ -37,9 +37,9 @@
 #include <iprt/thread.h>
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static int pdmR3CritSectDeleteOne(PVM pVM, PUVM pUVM, PPDMCRITSECTINT pCritSect, PPDMCRITSECTINT pPrev, bool fFinal);
 static int pdmR3CritSectRwDeleteOne(PVM pVM, PUVM pUVM, PPDMCRITSECTRWINT pCritSect, PPDMCRITSECTRWINT pPrev, bool fFinal);
 
@@ -49,10 +49,11 @@ static int pdmR3CritSectRwDeleteOne(PVM pVM, PUVM pUVM, PPDMCRITSECTRWINT pCritS
  * Register statistics related to the critical sections.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 int pdmR3CritSectBothInitStats(PVM pVM)
 {
+    RT_NOREF_PV(pVM);
     STAM_REG(pVM, &pVM->pdm.s.StatQueuedCritSectLeaves, STAMTYPE_COUNTER, "/PDM/QueuedCritSectLeaves", STAMUNIT_OCCURENCES,
              "Number of times a critical section leave request needed to be queued for ring-3 execution.");
     return VINF_SUCCESS;
@@ -62,7 +63,7 @@ int pdmR3CritSectBothInitStats(PVM pVM)
 /**
  * Relocates all the critical sections.
  *
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 void pdmR3CritSectBothRelocate(PVM pVM)
 {
@@ -92,9 +93,9 @@ void pdmR3CritSectBothRelocate(PVM pVM)
  * latter call because other components expect the critical sections to be
  * automatically deleted.
  *
- * @returns VBox status.
+ * @returns VBox status code.
  *          First error code, rest is lost.
- * @param   pVMU        The user mode VM handle.
+ * @param   pVM             The cross context VM structure.
  * @remark  Don't confuse this with PDMR3CritSectDelete.
  */
 VMMR3_INT_DECL(int) PDMR3CritSectBothTerm(PVM pVM)
@@ -128,11 +129,10 @@ VMMR3_INT_DECL(int) PDMR3CritSectBothTerm(PVM pVM)
  * Initializes a critical section and inserts it into the list.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pCritSect       The critical section.
  * @param   pvKey           The owner key.
- * @param   RT_SRC_POS_DECL The source position.
- * @param   pszName         The name of the critical section (for statistics).
+ * @param   SRC_POS         The source position.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   va              Arguments for the format string.
@@ -154,6 +154,7 @@ static int pdmR3CritSectInitOne(PVM pVM, PPDMCRITSECTINT pCritSect, void *pvKey,
         char *pszName = RTStrAPrintf2V(pszNameFmt, va); /** @todo plug the "leak"... */
         if (pszName)
         {
+            RT_SRC_POS_NOREF();
 #ifndef PDMCRITSECT_STRICT
             pCritSect->Core.pValidatorRec = NULL;
 #else
@@ -215,11 +216,10 @@ static int pdmR3CritSectInitOne(PVM pVM, PPDMCRITSECTINT pCritSect, void *pvKey,
  * Initializes a read/write critical section and inserts it into the list.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pCritSect       The read/write critical section.
  * @param   pvKey           The owner key.
- * @param   RT_SRC_POS_DECL The source position.
- * @param   pszName         The name of the critical section (for statistics).
+ * @param   SRC_POS         The source position.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   va              Arguments for the format string.
@@ -247,6 +247,7 @@ static int pdmR3CritSectRwInitOne(PVM pVM, PPDMCRITSECTRWINT pCritSect, void *pv
             {
                 pCritSect->Core.pValidatorRead  = NULL;
                 pCritSect->Core.pValidatorWrite = NULL;
+                RT_SRC_POS_NOREF();
 #ifdef PDMCRITSECTRW_STRICT
 # ifdef RT_LOCK_STRICT_ORDER
                 RTLOCKVALCLASS hClass = RTLockValidatorClassForSrcPos(RT_SRC_POS_ARGS, "%s", pszName);
@@ -321,10 +322,9 @@ static int pdmR3CritSectRwInitOne(PVM pVM, PPDMCRITSECTRWINT pCritSect, void *pv
  * works in ring-0 and raw-mode context as well.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
- * @param   pDevIns         Device instance.
+ * @param   pVM             The cross context VM structure.
  * @param   pCritSect       Pointer to the critical section.
- * @param   RT_SRC_POS_DECL Use RT_SRC_POS.
+ * @param   SRC_POS         Use RT_SRC_POS.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   ...             Arguments for the format string.
@@ -351,10 +351,9 @@ VMMR3DECL(int) PDMR3CritSectInit(PVM pVM, PPDMCRITSECT pCritSect, RT_SRC_POS_DEC
  * critical sections, but works in ring-0 and raw-mode context as well.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
- * @param   pDevIns         Device instance.
+ * @param   pVM             The cross context VM structure.
  * @param   pCritSect       Pointer to the read/write critical section.
- * @param   RT_SRC_POS_DECL Use RT_SRC_POS.
+ * @param   SRC_POS         Use RT_SRC_POS.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   ...             Arguments for the format string.
@@ -378,9 +377,10 @@ VMMR3DECL(int) PDMR3CritSectRwInit(PVM pVM, PPDMCRITSECTRW pCritSect, RT_SRC_POS
  * Initializes a PDM critical section for a device.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pDevIns         Device instance.
  * @param   pCritSect       Pointer to the critical section.
+ * @param   SRC_POS         The source position.  Optional.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   va              Arguments for the format string.
@@ -396,9 +396,10 @@ int pdmR3CritSectInitDevice(PVM pVM, PPDMDEVINS pDevIns, PPDMCRITSECT pCritSect,
  * Initializes a PDM read/write critical section for a device.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pDevIns         Device instance.
  * @param   pCritSect       Pointer to the read/write critical section.
+ * @param   SRC_POS         The source position.  Optional.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   va              Arguments for the format string.
@@ -414,9 +415,13 @@ int pdmR3CritSectRwInitDevice(PVM pVM, PPDMDEVINS pDevIns, PPDMCRITSECTRW pCritS
  * Initializes the automatic default PDM critical section for a device.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pDevIns         Device instance.
+ * @param   SRC_POS         The source position.  Optional.
  * @param   pCritSect       Pointer to the critical section.
+ * @param   pszNameFmt      Format string for naming the critical section.  For
+ *                          statistics and lock validation.
+ * @param   ...             Arguments for the format string.
  */
 int pdmR3CritSectInitDeviceAuto(PVM pVM, PPDMDEVINS pDevIns, PPDMCRITSECT pCritSect, RT_SRC_POS_DECL,
                                 const char *pszNameFmt, ...)
@@ -435,9 +440,10 @@ int pdmR3CritSectInitDeviceAuto(PVM pVM, PPDMDEVINS pDevIns, PPDMCRITSECT pCritS
  * Initializes a PDM critical section for a driver.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pDrvIns         Driver instance.
  * @param   pCritSect       Pointer to the critical section.
+ * @param   SRC_POS         The source position.  Optional.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   ...             Arguments for the format string.
@@ -457,9 +463,10 @@ int pdmR3CritSectInitDriver(PVM pVM, PPDMDRVINS pDrvIns, PPDMCRITSECT pCritSect,
  * Initializes a PDM read/write critical section for a driver.
  *
  * @returns VBox status code.
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pDrvIns         Driver instance.
  * @param   pCritSect       Pointer to the read/write critical section.
+ * @param   SRC_POS         The source position.  Optional.
  * @param   pszNameFmt      Format string for naming the critical section.  For
  *                          statistics and lock validation.
  * @param   ...             Arguments for the format string.
@@ -480,7 +487,8 @@ int pdmR3CritSectRwInitDriver(PVM pVM, PPDMDRVINS pDrvIns, PPDMCRITSECTRW pCritS
  *
  * @returns Return code from RTCritSectDelete.
  *
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
+ * @param   pUVM        The user mode VM handle.
  * @param   pCritSect   The critical section.
  * @param   pPrev       The previous critical section in the list.
  * @param   fFinal      Set if this is the final call and statistics shouldn't be deregistered.
@@ -493,7 +501,7 @@ static int pdmR3CritSectDeleteOne(PVM pVM, PUVM pUVM, PPDMCRITSECTINT pCritSect,
      * Assert free waiters and so on (c&p from RTCritSectDelete).
      */
     Assert(pCritSect->Core.u32Magic == RTCRITSECT_MAGIC);
-    Assert(pCritSect->Core.cNestings == 0);
+    //Assert(pCritSect->Core.cNestings == 0); - we no longer reset this when leaving.
     Assert(pCritSect->Core.cLockers == -1);
     Assert(pCritSect->Core.NativeThreadOwner == NIL_RTNATIVETHREAD);
     Assert(RTCritSectIsOwner(&pUVM->pdm.s.ListCritSect));
@@ -537,7 +545,8 @@ static int pdmR3CritSectDeleteOne(PVM pVM, PUVM pUVM, PPDMCRITSECTINT pCritSect,
  *
  * @returns VBox status code.
  *
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
+ * @param   pUVM        The user mode VM handle.
  * @param   pCritSect   The read/write critical section.
  * @param   pPrev       The previous critical section in the list.
  * @param   fFinal      Set if this is the final call and statistics shouldn't be deregistered.
@@ -610,7 +619,7 @@ static int pdmR3CritSectRwDeleteOne(PVM pVM, PUVM pUVM, PPDMCRITSECTRWINT pCritS
  *          The entire list is processed on failure, so we'll only
  *          return the first error code. This shouldn't be a problem
  *          since errors really shouldn't happen here.
- * @param   pVM     Pointer to the VM.
+ * @param   pVM     The cross context VM structure.
  * @param   pvKey   The initializer key.
  */
 static int pdmR3CritSectDeleteByKey(PVM pVM, void *pvKey)
@@ -649,7 +658,7 @@ static int pdmR3CritSectDeleteByKey(PVM pVM, void *pvKey)
  *          The entire list is processed on failure, so we'll only
  *          return the first error code. This shouldn't be a problem
  *          since errors really shouldn't happen here.
- * @param   pVM     Pointer to the VM.
+ * @param   pVM     The cross context VM structure.
  * @param   pvKey   The initializer key.
  */
 static int pdmR3CritSectRwDeleteByKey(PVM pVM, void *pvKey)
@@ -686,7 +695,7 @@ static int pdmR3CritSectRwDeleteByKey(PVM pVM, void *pvKey)
  * device.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDevIns     The device handle.
  */
 int pdmR3CritSectBothDeleteDevice(PVM pVM, PPDMDEVINS pDevIns)
@@ -702,7 +711,7 @@ int pdmR3CritSectBothDeleteDevice(PVM pVM, PPDMDEVINS pDevIns)
  * driver.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDrvIns     The driver handle.
  */
 int pdmR3CritSectBothDeleteDriver(PVM pVM, PPDMDRVINS pDrvIns)
@@ -949,7 +958,7 @@ static void pdmR3CritSectAppendNameToList(char const *pszName, char **ppszNames,
  *
  * @returns Lock count.
  *
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   pszNames        Where to return the critical section names.
  * @param   cbNames         The size of the buffer.
  */
@@ -1006,7 +1015,7 @@ VMMR3DECL(uint32_t) PDMR3CritSectCountOwned(PVM pVM, char *pszNames, size_t cbNa
  * This is only used when entering guru meditation in order to prevent other
  * EMTs and I/O threads from deadlocking.
  *
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 VMMR3_INT_DECL(void) PDMR3CritSectLeaveAll(PVM pVM)
 {
@@ -1033,7 +1042,7 @@ VMMR3_INT_DECL(void) PDMR3CritSectLeaveAll(PVM pVM)
  * all enter immediately and concurrently.
  *
  * @returns The address of the NOP critical section.
- * @param   pVM                 Pointer to the VM.
+ * @param   pVM                 The cross context VM structure.
  */
 VMMR3DECL(PPDMCRITSECT)             PDMR3CritSectGetNop(PVM pVM)
 {
@@ -1046,7 +1055,7 @@ VMMR3DECL(PPDMCRITSECT)             PDMR3CritSectGetNop(PVM pVM)
  * Gets the ring-0 address of the NOP critical section.
  *
  * @returns The ring-0 address of the NOP critical section.
- * @param   pVM                 Pointer to the VM.
+ * @param   pVM                 The cross context VM structure.
  */
 VMMR3DECL(R0PTRTYPE(PPDMCRITSECT))  PDMR3CritSectGetNopR0(PVM pVM)
 {
@@ -1059,7 +1068,7 @@ VMMR3DECL(R0PTRTYPE(PPDMCRITSECT))  PDMR3CritSectGetNopR0(PVM pVM)
  * Gets the raw-mode context address of the NOP critical section.
  *
  * @returns The raw-mode context address of the NOP critical section.
- * @param   pVM                 Pointer to the VM.
+ * @param   pVM                 The cross context VM structure.
  */
 VMMR3DECL(RCPTRTYPE(PPDMCRITSECT))  PDMR3CritSectGetNopRC(PVM pVM)
 {
