@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007-2015 Oracle Corporation
+ * Copyright (C) 2007-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
-///@todo #define LOG_GROUP LOG_GROUP_PDM_THREAD
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
+/// @todo \#define LOG_GROUP LOG_GROUP_PDM_THREAD
 #include "PDMInternal.h"
 #include <VBox/vmm/pdm.h>
 #include <VBox/vmm/mm.h>
@@ -33,9 +34,9 @@
 #include <iprt/thread.h>
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static DECLCALLBACK(int) pdmR3ThreadMain(RTTHREAD Thread, void *pvUser);
 
 
@@ -97,7 +98,7 @@ static DECLCALLBACK(int) pdmR3ThreadWakeUp(PPDMTHREAD pThread)
  * Allocates new thread instance.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where to store the pointer to the instance.
  */
 static int pdmR3ThreadNew(PVM pVM, PPPDMTHREAD ppThread)
@@ -122,7 +123,7 @@ static int pdmR3ThreadNew(PVM pVM, PPPDMTHREAD ppThread)
  * Initialize a new thread, this actually creates the thread.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where the thread instance data handle is.
  * @param   cbStack     The stack size, see RTThreadCreate().
  * @param   enmType     The thread type, see RTThreadCreate().
@@ -184,7 +185,7 @@ static int pdmR3ThreadInit(PVM pVM, PPPDMTHREAD ppThread, size_t cbStack, RTTHRE
         RTSemEventMultiDestroy(pThread->Internal.s.BlockEvent);
         pThread->Internal.s.BlockEvent = NIL_RTSEMEVENTMULTI;
     }
-    MMHyperFree(pVM, pThread);
+    MMR3HeapFree(pThread);
     *ppThread = NULL;
 
     return rc;
@@ -195,7 +196,7 @@ static int pdmR3ThreadInit(PVM pVM, PPPDMTHREAD ppThread, size_t cbStack, RTTHRE
  * Device Helper for creating a thread associated with a device.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDevIns     The device instance.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
@@ -228,7 +229,7 @@ int pdmR3ThreadCreateDevice(PVM pVM, PPDMDEVINS pDevIns, PPPDMTHREAD ppThread, v
  * USB Device Helper for creating a thread associated with an USB device.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pUsbIns     The USB device instance.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
@@ -261,7 +262,7 @@ int pdmR3ThreadCreateUsb(PVM pVM, PPDMUSBINS pUsbIns, PPPDMTHREAD ppThread, void
  * Driver Helper for creating a thread associated with a driver.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDrvIns     The driver instance.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
@@ -294,7 +295,7 @@ int pdmR3ThreadCreateDriver(PVM pVM, PPDMDRVINS pDrvIns, PPPDMTHREAD ppThread, v
  * Creates a PDM thread for internal use in the VM.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
  * @param   pfnThread   The thread function.
@@ -325,7 +326,7 @@ VMMR3DECL(int) PDMR3ThreadCreate(PVM pVM, PPPDMTHREAD ppThread, void *pvUser, PF
  * Creates a PDM thread for VM use by some external party.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
  * @param   pfnThread   The thread function.
@@ -473,7 +474,7 @@ VMMR3DECL(int) PDMR3ThreadDestroy(PPDMTHREAD pThread, int *pRcThread)
  * destroyed (not currently implemented).
  *
  * @returns VBox status code of the first failure.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDevIns     the device instance.
  */
 int pdmR3ThreadDestroyDevice(PVM pVM, PPDMDEVINS pDevIns)
@@ -508,7 +509,7 @@ int pdmR3ThreadDestroyDevice(PVM pVM, PPDMDEVINS pDevIns)
  * This function is called by PDMUsb when a device is destroyed.
  *
  * @returns VBox status code of the first failure.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pUsbIns     The USB device instance.
  */
 int pdmR3ThreadDestroyUsb(PVM pVM, PPDMUSBINS pUsbIns)
@@ -543,7 +544,7 @@ int pdmR3ThreadDestroyUsb(PVM pVM, PPDMUSBINS pUsbIns)
  * This function is called by PDMDriver when a driver is destroyed.
  *
  * @returns VBox status code of the first failure.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDrvIns     The driver instance.
  */
 int pdmR3ThreadDestroyDriver(PVM pVM, PPDMDRVINS pDrvIns)
@@ -575,7 +576,7 @@ int pdmR3ThreadDestroyDriver(PVM pVM, PPDMDRVINS pDrvIns)
 /**
  * Called For VM power off.
  *
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 void pdmR3ThreadDestroyAll(PVM pVM)
 {
@@ -971,7 +972,7 @@ VMMR3DECL(int) PDMR3ThreadSuspend(PPDMTHREAD pThread)
  * and drivers have been notified about the suspend / power off.
  *
  * @return VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 int pdmR3ThreadSuspendAll(PVM pVM)
 {
@@ -983,7 +984,7 @@ int pdmR3ThreadSuspendAll(PVM pVM)
             case PDMTHREADSTATE_RUNNING:
             {
                 int rc = PDMR3ThreadSuspend(pThread);
-                AssertRCReturn(rc, rc);
+                AssertRCReturnStmt(rc, RTCritSectLeave(&pUVM->pdm.s.ListCritSect), rc);
                 break;
             }
 
@@ -1063,7 +1064,7 @@ VMMR3DECL(int) PDMR3ThreadResume(PPDMTHREAD pThread)
  * and drivers have been notified about the resume / power on .
  *
  * @return VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 int pdmR3ThreadResumeAll(PVM pVM)
 {
