@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,26 +25,32 @@
 #include <VBox/dis.h>
 #include <VBox/vmm/hm.h>
 #include <VBox/vmm/hm_vmx.h>
+#include <VBox/vmm/hm_svm.h>
 #include <VBox/vmm/pgm.h>
 #include <VBox/vmm/cpum.h>
+#include <VBox/vmm/trpm.h>
 #include <iprt/memobj.h>
 #include <iprt/cpuset.h>
 #include <iprt/mp.h>
 #include <iprt/avl.h>
 #include <iprt/string.h>
 
-#if HC_ARCH_BITS == 64 || defined(VBOX_WITH_HYBRID_32BIT_KERNEL) || defined (VBOX_WITH_64_BITS_GUESTS)
+#if defined(RT_OS_DARWIN) && HC_ARCH_BITS == 32
+# error "32-bit darwin is no longer supported. Go back to 4.3 or earlier!"
+#endif
+
+#if HC_ARCH_BITS == 64 || defined (VBOX_WITH_64_BITS_GUESTS)
 /* Enable 64 bits guest support. */
 # define VBOX_ENABLE_64_BITS_GUESTS
 #endif
 
-#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS) && !defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
+#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS)
 # define VMX_USE_CACHED_VMCS_ACCESSES
 #endif
 
 /** @def HM_PROFILE_EXIT_DISPATCH
  * Enables profiling of the VM exit handler dispatching. */
-#if 0
+#if 0 || defined(DOXYGEN_RUNNING)
 # define HM_PROFILE_EXIT_DISPATCH
 #endif
 
@@ -60,7 +66,7 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_CLEAR
  * Clears a HM-context flag.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlag   The flag to clear.
  */
 #define HMCPU_CF_CLEAR(pVCpu, fFlag)              (ASMAtomicUoAndU32(&(pVCpu)->hm.s.fContextUseFlags, ~(fFlag)))
@@ -68,7 +74,7 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_SET
  * Sets a HM-context flag.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlag   The flag to set.
  */
 #define HMCPU_CF_SET(pVCpu, fFlag)                (ASMAtomicUoOrU32(&(pVCpu)->hm.s.fContextUseFlags, (fFlag)))
@@ -76,7 +82,7 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_IS_SET
  * Checks if all the flags in the specified HM-context set is pending.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlag   The flag to check.
  */
 #define HMCPU_CF_IS_SET(pVCpu, fFlag)             ((ASMAtomicUoReadU32(&(pVCpu)->hm.s.fContextUseFlags) & (fFlag)) == (fFlag))
@@ -85,7 +91,7 @@ RT_C_DECLS_BEGIN
  * Checks if one or more of the flags in the specified HM-context set is
  * pending.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlags  The flags to check for.
  */
 #define HMCPU_CF_IS_PENDING(pVCpu, fFlags)        RT_BOOL(ASMAtomicUoReadU32(&(pVCpu)->hm.s.fContextUseFlags) & (fFlags))
@@ -93,7 +99,7 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_IS_PENDING_ONLY
  * Checks if -only- one or more of the specified HM-context flags is pending.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlags  The flags to check for.
  */
 #define HMCPU_CF_IS_PENDING_ONLY(pVCpu, fFlags)   !RT_BOOL(ASMAtomicUoReadU32(&(pVCpu)->hm.s.fContextUseFlags) & ~(fFlags))
@@ -101,7 +107,7 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_IS_SET_ONLY
  * Checks if -only- all the flags in the specified HM-context set is pending.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlags  The flags to check for.
  */
 #define HMCPU_CF_IS_SET_ONLY(pVCpu, fFlags)       (ASMAtomicUoReadU32(&(pVCpu)->hm.s.fContextUseFlags) == (fFlags))
@@ -109,7 +115,7 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_RESET_TO
  * Resets the HM-context flags to the specified value.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  * @param   fFlags  The new value.
  */
 #define HMCPU_CF_RESET_TO(pVCpu, fFlags)          (ASMAtomicUoWriteU32(&(pVCpu)->hm.s.fContextUseFlags, (fFlags)))
@@ -117,15 +123,15 @@ RT_C_DECLS_BEGIN
 /** @def HMCPU_CF_VALUE
  * Returns the current HM-context flags value.
  *
- * @param   pVCpu   Pointer to the VMCPU.
+ * @param   pVCpu   The cross context virtual CPU structure.
  */
 #define HMCPU_CF_VALUE(pVCpu)                     (ASMAtomicUoReadU32(&(pVCpu)->hm.s.fContextUseFlags))
 
 
-/** Resets/initializes the VM-exit/#VMEXIT history array. */
+/** Resets/initializes the VM-exit/\#VMEXIT history array. */
 #define HMCPU_EXIT_HISTORY_RESET(pVCpu)           (memset(&(pVCpu)->hm.s.auExitHistory, 0xff, sizeof((pVCpu)->hm.s.auExitHistory)))
 
-/** Updates the VM-exit/#VMEXIT history array. */
+/** Updates the VM-exit/\#VMEXIT history array. */
 #define HMCPU_EXIT_HISTORY_ADD(pVCpu, a_ExitReason) \
     do { \
         AssertMsg((pVCpu)->hm.s.idxExitHistoryFree < RT_ELEMENTS((pVCpu)->hm.s.auExitHistory), ("%u\n", (pVCpu)->hm.s.idxExitHistoryFree)); \
@@ -141,8 +147,12 @@ RT_C_DECLS_BEGIN
 #define MASK_INJECT_IRQ_STAT       0xff
 
 /** @name HM changed flags.
- * These flags are used to keep track of which important registers that
- * have been changed since last they were reset.
+ * These flags are used to keep track of which important registers that have
+ * been changed since last they were reset.
+ *
+ * Flags marked "shared" are used for registers that are common to both the host
+ * and guest (i.e. without dedicated VMCS/VMCB fields for guest bits).
+ *
  * @{
  */
 #define HM_CHANGED_GUEST_CR0                     RT_BIT(0)      /* Shared */
@@ -162,7 +172,7 @@ RT_C_DECLS_BEGIN
 #define HM_CHANGED_GUEST_SYSENTER_EIP_MSR        RT_BIT(14)
 #define HM_CHANGED_GUEST_SYSENTER_ESP_MSR        RT_BIT(15)
 #define HM_CHANGED_GUEST_EFER_MSR                RT_BIT(16)
-#define HM_CHANGED_GUEST_LAZY_MSRS               RT_BIT(17)     /* Shared */
+#define HM_CHANGED_GUEST_LAZY_MSRS               RT_BIT(17)     /* Shared */ /** @todo Move this to VT-x specific? */
 #define HM_CHANGED_GUEST_XCPT_INTERCEPTS         RT_BIT(18)
 /* VT-x specific state. */
 #define HM_CHANGED_VMX_GUEST_AUTO_MSRS           RT_BIT(19)
@@ -210,9 +220,6 @@ RT_C_DECLS_BEGIN
                                                   | HM_CHANGED_GUEST_LAZY_MSRS)
 /** @} */
 
-/** Maximum number of page flushes we are willing to remember before considering a full TLB flush. */
-#define HM_MAX_TLB_SHOOTDOWN_PAGES      8
-
 /** Size for the EPT identity page table (1024 4 MB pages to cover the entire address space). */
 #define HM_EPT_IDENTITY_PG_TABLE_SIZE   PAGE_SIZE
 /** Size of the TSS structure + 2 pages for the IO bitmap + end byte. */
@@ -264,6 +271,11 @@ typedef struct HMGLOBALCPUINFO
     RTCPUID             idCpu;
     /** The VM_HSAVE_AREA (AMD-V) / VMXON region (Intel) memory backing. */
     RTR0MEMOBJ          hMemObj;
+    /** The physical address of the first page in hMemObj (it's a
+     *  physcially contigous allocation if it spans multiple pages). */
+    RTHCPHYS            HCPhysMemObj;
+    /** The address of the memory (for pfnEnable). */
+    void               *pvMemObj;
     /** Current ASID (AMD-V) / VPID (Intel). */
     uint32_t            uCurrentAsid;
     /** TLB flush count. */
@@ -284,9 +296,8 @@ typedef enum
 {
     HMPENDINGIO_INVALID = 0,
     HMPENDINGIO_PORT_READ,
-    HMPENDINGIO_PORT_WRITE,
-    HMPENDINGIO_STRING_READ,
-    HMPENDINGIO_STRING_WRITE,
+    /* not implemented: HMPENDINGIO_STRING_READ, */
+    /* not implemented: HMPENDINGIO_STRING_WRITE, */
     /** The usual 32-bit paranoia. */
     HMPENDINGIO_32BIT_HACK   = 0x7fffffff
 } HMPENDINGIO;
@@ -330,10 +341,45 @@ typedef struct
 /** Pointer to HMTPRPATCH. */
 typedef HMTPRPATCH *PHMTPRPATCH;
 
+
+/**
+ * Makes a HMEXITSTAT::uKey value from a program counter and an exit code.
+ *
+ * @returns 64-bit key
+ * @param   a_uPC           The RIP + CS.BASE value of the exit.
+ * @param   a_uExit         The exit code.
+ * @todo    Add CPL?
+ */
+#define HMEXITSTAT_MAKE_KEY(a_uPC, a_uExit) (((a_uPC) & UINT64_C(0x0000ffffffffffff)) | (uint64_t)(a_uExit) << 48)
+
+typedef struct HMEXITINFO
+{
+    /** See HMEXITSTAT_MAKE_KEY(). */
+    uint64_t                uKey;
+    /** Number of recent hits (depreciates with time). */
+    uint32_t volatile       cHits;
+    /** The age + lock. */
+    uint16_t volatile       uAge;
+    /** Action or action table index. */
+    uint16_t                iAction;
+} HMEXITINFO;
+AssertCompileSize(HMEXITINFO, 16); /* Lots of these guys, so don't add any unnecessary stuff! */
+
+typedef struct HMEXITHISTORY
+{
+    /** The exit timestamp. */
+    uint64_t                uTscExit;
+    /** The index of the corresponding HMEXITINFO entry.
+     * UINT32_MAX if none (too many collisions, race, whatever). */
+    uint32_t                iExitInfo;
+    /** Figure out later, needed for padding now. */
+    uint32_t                uSomeClueOrSomething;
+} HMEXITHISTORY;
+
 /**
  * Switcher function, HC to the special 64-bit RC.
  *
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   offCpumVCpu     Offset from pVM->cpum to pVM->aCpus[idCpu].cpum.
  * @returns Return code indicating the action to take.
  */
@@ -357,18 +403,35 @@ typedef struct HM
     bool                        fLargePages;
     /** Set if we can support 64-bit guests or not. */
     bool                        fAllow64BitGuests;
-    /** Set if an IO-APIC is configured for this VM. */
-    bool                        fHasIoApic;
     /** Set when TPR patching is allowed. */
     bool                        fTprPatchingAllowed;
     /** Set when we initialize VT-x or AMD-V once for all CPUs. */
     bool                        fGlobalInit;
     /** Set when TPR patching is active. */
     bool                        fTPRPatchingActive;
-    bool                        u8Alignment[3];
-
-    /** Host kernel flags that HM might need to know (SUPKERNELFEATURES_XXX). */
-    uint32_t                    uHostKernelFeatures;
+    /** Set when the debug facility has breakpoints/events enabled that requires
+     *  us to use the debug execution loop in ring-0. */
+    bool                        fUseDebugLoop;
+    /** Set if hardware APIC virtualization is enabled. */
+    bool                        fVirtApicRegs;
+    /** Set if posted interrupt processing is enabled. */
+    bool                        fPostedIntrs;
+    /** Set if indirect branch prediction barrier on VM exit. */
+    bool                        fIbpbOnVmExit;
+    /** Set if indirect branch prediction barrier on VM entry. */
+    bool                        fIbpbOnVmEntry;
+    /** Set if level 1 data cache should be flushed on VM entry. */
+    bool                        fL1dFlushOnVmEntry;
+    /** Set if level 1 data cache should be flushed on EMT scheduling. */
+    bool                        fL1dFlushOnSched;
+    /** Set if host manages speculation control settings. */
+    bool                        fSpecCtrlByHost;
+    /** Set if MDS related buffers should be cleared on VM entry. */
+    bool                        fMdsClearOnVmEntry;
+    /** Set if MDS related buffers should be cleared on EMT scheduling. */
+    bool                        fMdsClearOnSched;
+    /** Alignment padding. */
+    bool                        afPaddingMinus1[6];
 
     /** Maximum ASID allowed. */
     uint32_t                    uMaxAsid;
@@ -376,15 +439,17 @@ typedef struct HM
      * This number is set much higher when RTThreadPreemptIsPending is reliable. */
     uint32_t                    cMaxResumeLoops;
 
+    /** Host kernel flags that HM might need to know (SUPKERNELFEATURES_XXX). */
+    uint32_t                    fHostKernelFeatures;
+
+    /** Size of the guest patch memory block. */
+    uint32_t                    cbGuestPatchMem;
     /** Guest allocated memory for patching purposes. */
     RTGCPTR                     pGuestPatchMem;
     /** Current free pointer inside the patch block. */
     RTGCPTR                     pFreeGuestPatchMem;
-    /** Size of the guest patch memory block. */
-    uint32_t                    cbGuestPatchMem;
-    uint32_t                    u32Alignment0;
 
-#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS) && !defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
+#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS)
     /** 32 to 64 bits switcher entrypoint. */
     R0PTRTYPE(PFNHMSWITCHERHC)  pfnHost32ToGuest64R0;
     RTR0PTR                     pvR0Alignment0;
@@ -430,17 +495,24 @@ typedef struct HM
 
         /** Internal Id of which flush-handler to use for tagged-TLB entries. */
         uint32_t                    uFlushTaggedTlb;
+
+        /** Pause-loop exiting (PLE) gap in ticks. */
+        uint32_t                    cPleGapTicks;
+        /** Pause-loop exiting (PLE) window in ticks. */
+        uint32_t                    cPleWindowTicks;
         uint32_t                    u32Alignment0;
+
         /** Host CR4 value (set by ring-0 VMX init) */
         uint64_t                    u64HostCr4;
-
+        /** Host SMM monitor control (set by ring-0 VMX init) */
+        uint64_t                    u64HostSmmMonitorCtl;
         /** Host EFER value (set by ring-0 VMX init) */
         uint64_t                    u64HostEfer;
         /** Whether the CPU supports VMCS fields for swapping EFER. */
         bool                        fSupportsVmcsEfer;
         uint8_t                     u8Alignment2[7];
 
-        /** VMX MSR values */
+        /** VMX MSR values. */
         VMXMSRS                     Msrs;
 
         /** Flush types for invept & invvpid; they depend on capabilities. */
@@ -478,6 +550,12 @@ typedef struct HM
         uint32_t                    u32Rev;
         /** SVM feature bits from cpuid 0x8000000a */
         uint32_t                    u32Features;
+
+        /** Pause filter counter. */
+        uint16_t                    cPauseFilter;
+        /** Pause filter treshold in ticks. */
+        uint16_t                    cPauseFilterThresholdTicks;
+        uint32_t                    u32Alignment0;
     } svm;
 
     /**
@@ -503,7 +581,8 @@ typedef struct HM
 
     STAMCOUNTER             StatTprPatchSuccess;
     STAMCOUNTER             StatTprPatchFailure;
-    STAMCOUNTER             StatTprReplaceSuccess;
+    STAMCOUNTER             StatTprReplaceSuccessCr8;
+    STAMCOUNTER             StatTprReplaceSuccessVmc;
     STAMCOUNTER             StatTprReplaceFailure;
 } HM;
 /** Pointer to HM VM instance data. */
@@ -576,7 +655,16 @@ typedef struct VMCSCACHE
 typedef VMCSCACHE *PVMCSCACHE;
 AssertCompileSizeAlignment(VMCSCACHE, 8);
 
-/** VMX StartVM function. */
+/**
+ * VMX StartVM function.
+ *
+ * @returns VBox status code (no informational stuff).
+ * @param   fResume     Whether to use VMRESUME (true) or VMLAUNCH (false).
+ * @param   pCtx        The CPU register context.
+ * @param   pCache      The VMCS cache.
+ * @param   pVM         Pointer to the cross context VM structure.
+ * @param   pVCpu       Pointer to the cross context per-CPU structure.
+ */
 typedef DECLCALLBACK(int) FNHMVMXSTARTVM(RTHCUINT fResume, PCPUMCTX pCtx, PVMCSCACHE pCache, PVM pVM, PVMCPU pVCpu);
 /** Pointer to a VMX StartVM function. */
 typedef R0PTRTYPE(FNHMVMXSTARTVM *) PFNHMVMXSTARTVM;
@@ -600,25 +688,34 @@ typedef struct HMCPU
     bool                        fActive;
     /** Set when the TLB has been checked until we return from the world switch. */
     volatile bool               fCheckedTLBFlush;
-    /** Whether we're executing a single instruction. */
-    bool                        fSingleInstruction;
-    /** Set if we need to clear the trap flag because of single stepping. */
-    bool                        fClearTrapFlag;
     /** Whether we've completed the inner HM leave function. */
     bool                        fLeaveDone;
     /** Whether we're using the hyper DR7 or guest DR7. */
     bool                        fUsingHyperDR7;
-    /** Whether to preload the guest-FPU state to avoid #NM VM-exit overhead. */
-    bool                        fPreloadGuestFpu;
     /** Set if XCR0 needs to be loaded and saved when entering and exiting guest
      * code execution. */
     bool                        fLoadSaveGuestXcr0;
 
-    /** Whether #UD needs to be intercepted (required by certain GIM providers). */
+    /** Whether we should use the debug loop because of single stepping or special
+     *  debug breakpoints / events are armed. */
+    bool                        fUseDebugLoop;
+    /** Whether we are currently executing in the debug loop.
+     *  Mainly for assertions. */
+    bool                        fUsingDebugLoop;
+    /** Set if we using the debug loop and wish to intercept RDTSC. */
+    bool                        fDebugWantRdTscExit;
+    /** Whether we're executing a single instruction. */
+    bool                        fSingleInstruction;
+    /** Set if we need to clear the trap flag because of single stepping. */
+    bool                        fClearTrapFlag;
+
+    /** Whether \#UD needs to be intercepted (required by certain GIM providers). */
     bool                        fGIMTrapXcptUD;
     /** Whether paravirt. hypercalls are enabled. */
     bool                        fHypercallsEnabled;
-    uint8_t                     u8Alignment0[5];
+    /** Whether \#GP needs to be intercept for mesa driver workaround. */
+    bool                        fTrapXcptGpForLovelyMesaDrv;
+    uint8_t                     u8Alignment0[2];
 
     /** World switch exit counter. */
     volatile uint32_t           cWorldSwitchExits;
@@ -673,8 +770,8 @@ typedef struct HMCPU
 
         /** Physical address of the virtual APIC page for TPR caching. */
         RTHCPHYS                    HCPhysVirtApic;
-        /** R0 memory object for the virtual APIC page for TPR caching. */
-        RTR0MEMOBJ                  hMemObjVirtApic;
+        /** Padding. */
+        R0PTRTYPE(void *)           pvAlignment0;
         /** Virtual address of the virtual APIC page for TPR caching. */
         R0PTRTYPE(uint8_t *)        pbVirtApic;
 
@@ -765,7 +862,10 @@ typedef struct HMCPU
 
         /** Set if guest was executing in real mode (extra checks). */
         bool                        fWasInRealMode;
-        uint8_t                     u8Alignment1[7];
+        /** Set if guest switched to 64-bit mode on a 32-bit host. */
+        bool                        fSwitchedTo64on32;
+
+        uint8_t                     u8Alignment1[6];
     } vmx;
 
     struct
@@ -780,27 +880,31 @@ typedef struct HMCPU
         RTHCPHYS                    HCPhysVmcbHost;
         /** R0 memory object for the host VMCB which holds additional host-state. */
         RTR0MEMOBJ                  hMemObjVmcbHost;
-        /** Virtual address of the host VMCB which holds additional host-state. */
-        R0PTRTYPE(void *)           pvVmcbHost;
+        /** Padding. */
+        R0PTRTYPE(void *)           pvPadding;
 
         /** Physical address of the guest VMCB. */
         RTHCPHYS                    HCPhysVmcb;
         /** R0 memory object for the guest VMCB. */
         RTR0MEMOBJ                  hMemObjVmcb;
-        /** Virtual address of the guest VMCB. */
-        R0PTRTYPE(void *)           pvVmcb;
+        /** Pointer to the guest VMCB. */
+        R0PTRTYPE(PSVMVMCB)         pVmcb;
 
         /** Physical address of the MSR bitmap (8 KB). */
         RTHCPHYS                    HCPhysMsrBitmap;
         /** R0 memory object for the MSR bitmap (8 KB). */
         RTR0MEMOBJ                  hMemObjMsrBitmap;
-        /** Virtual address of the MSR bitmap. */
+        /** Pointer to the MSR bitmap. */
         R0PTRTYPE(void *)           pvMsrBitmap;
 
         /** Whether VTPR with V_INTR_MASKING set is in effect, indicating
          *  we should check if the VTPR changed on every VM-exit. */
         bool                        fSyncVTpr;
         uint8_t                     u8Alignment0[7];
+
+        /** Cache of the nested-guest's VMCB fields that we modify in order to run the
+         *  nested-guest using AMD-V. This will be restored on \#VMEXIT. */
+        SVMNESTEDVMCBCACHE          NstGstVmcbCache;
     } svm;
 
     /** Event injection state. */
@@ -826,6 +930,7 @@ typedef struct HMCPU
         uint64_t                cr0;
     } EmulateIoBlock;
 
+    /* */
     struct
     {
         /** Pending IO operation type. */
@@ -856,15 +961,7 @@ typedef struct HMCPU
      * HMR0Enter and cleared in HMR0Leave. */
     RTCPUID                 idEnteredCpu;
 
-    /** To keep track of pending TLB shootdown pages. (SMP guest only) */
-    struct
-    {
-        RTGCPTR             aPages[HM_MAX_TLB_SHOOTDOWN_PAGES];
-        uint32_t            cPages;
-        uint32_t            u32Alignment0; /**< Explicit alignment padding. */
-    } TlbShootdown;
-
-    /** VT-x/AMD-V VM-exit/#VMXEXIT history, circular array. */
+    /** VT-x/AMD-V VM-exit/\#VMXEXIT history, circular array. */
     uint16_t                auExitHistory[31];
     /** The index of the next free slot in the history array. */
     uint16_t                idxExitHistoryFree;
@@ -882,7 +979,7 @@ typedef struct HMCPU
     STAMPROFILEADV          StatLoadGuestState;
     STAMPROFILEADV          StatInGC;
 
-#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS) && !defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
+#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS)
     STAMPROFILEADV          StatWorldSwitch3264;
 #endif
     STAMPROFILEADV          StatPoke;
@@ -892,11 +989,12 @@ typedef struct HMCPU
     STAMCOUNTER             StatInjectInterrupt;
     STAMCOUNTER             StatInjectXcpt;
     STAMCOUNTER             StatInjectPendingReflect;
+    STAMCOUNTER             StatInjectPendingInterpret;
 
     STAMCOUNTER             StatExitAll;
     STAMCOUNTER             StatExitShadowNM;
     STAMCOUNTER             StatExitGuestNM;
-    STAMCOUNTER             StatExitShadowPF;       /* Misleading, currently used for MMIO #PFs as well. */
+    STAMCOUNTER             StatExitShadowPF;       /**< Misleading, currently used for MMIO \#PFs as well. */
     STAMCOUNTER             StatExitShadowPFEM;
     STAMCOUNTER             StatExitGuestPF;
     STAMCOUNTER             StatExitGuestUD;
@@ -967,6 +1065,7 @@ typedef struct HMCPU
     STAMCOUNTER             StatTlbShootdown;
     STAMCOUNTER             StatTlbShootdownFlush;
 
+    STAMCOUNTER             StatSwitchTprMaskedIrq;
     STAMCOUNTER             StatSwitchGuestIrq;
     STAMCOUNTER             StatSwitchHmToR3FF;
     STAMCOUNTER             StatSwitchExitToR3;
@@ -999,7 +1098,7 @@ typedef struct HMCPU
     STAMCOUNTER             StatVmxCheckBadTr;
     STAMCOUNTER             StatVmxCheckPmOk;
 
-#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS) && !defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
+#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS)
     STAMCOUNTER             StatFpu64SwitchBack;
     STAMCOUNTER             StatDebug64SwitchBack;
 #endif
@@ -1020,40 +1119,22 @@ AssertCompileMemberAlignment(HMCPU, vmx, 8);
 AssertCompileMemberAlignment(HMCPU, svm, 8);
 AssertCompileMemberAlignment(HMCPU, Event, 8);
 
-
 #ifdef IN_RING0
-VMMR0DECL(PHMGLOBALCPUINFO) HMR0GetCurrentCpu(void);
-VMMR0DECL(PHMGLOBALCPUINFO) HMR0GetCurrentCpuEx(RTCPUID idCpu);
-
+VMMR0_INT_DECL(PHMGLOBALCPUINFO) hmR0GetCurrentCpu(void);
 
 # ifdef VBOX_STRICT
-VMMR0DECL(void) HMDumpRegs(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx);
-VMMR0DECL(void) HMR0DumpDescriptor(PCX86DESCHC pDesc, RTSEL Sel, const char *pszMsg);
+VMMR0_INT_DECL(void) hmR0DumpRegs(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx);
+VMMR0_INT_DECL(void) hmR0DumpDescriptor(PCX86DESCHC pDesc, RTSEL Sel, const char *pszMsg);
 # else
-#  define HMDumpRegs(a, b ,c)            do { } while (0)
-#  define HMR0DumpDescriptor(a, b, c)    do { } while (0)
+#  define hmR0DumpRegs(a, b ,c)          do { } while (0)
+#  define hmR0DumpDescriptor(a, b, c)    do { } while (0)
 # endif /* VBOX_STRICT */
 
 # ifdef VBOX_WITH_KERNEL_USING_XMM
-DECLASM(int) HMR0VMXStartVMWrapXMM(RTHCUINT fResume, PCPUMCTX pCtx, PVMCSCACHE pCache, PVM pVM, PVMCPU pVCpu, PFNHMVMXSTARTVM pfnStartVM);
-DECLASM(int) HMR0SVMRunWrapXMM(RTHCPHYS pVmcbHostPhys, RTHCPHYS pVmcbPhys, PCPUMCTX pCtx, PVM pVM, PVMCPU pVCpu, PFNHMSVMVMRUN pfnVMRun);
+DECLASM(int) hmR0VMXStartVMWrapXMM(RTHCUINT fResume, PCPUMCTX pCtx, PVMCSCACHE pCache, PVM pVM, PVMCPU pVCpu, PFNHMVMXSTARTVM pfnStartVM);
+DECLASM(int) hmR0SVMRunWrapXMM(RTHCPHYS pVmcbHostPhys, RTHCPHYS pVmcbPhys, PCPUMCTX pCtx, PVM pVM, PVMCPU pVCpu, PFNHMSVMVMRUN pfnVMRun);
 # endif
-
-# ifdef VBOX_WITH_HYBRID_32BIT_KERNEL
-/**
- * Gets 64-bit GDTR and IDTR on darwin.
- * @param  pGdtr        Where to store the 64-bit GDTR.
- * @param  pIdtr        Where to store the 64-bit IDTR.
- */
-DECLASM(void) HMR0Get64bitGdtrAndIdtr(PX86XDTR64 pGdtr, PX86XDTR64 pIdtr);
-
-/**
- * Gets 64-bit CR3 on darwin.
- * @returns CR3
- */
-DECLASM(uint64_t) HMR0Get64bitCR3(void);
-# endif  /* VBOX_WITH_HYBRID_32BIT_KERNEL */
-
+DECLASM(void) hmR0MdsClear(void);
 #endif /* IN_RING0 */
 
 /** @} */
