@@ -7,7 +7,7 @@ VirtualBox Test VMs
 
 __copyright__ = \
 """
-Copyright (C) 2010-2015 Oracle Corporation
+Copyright (C) 2010-2017 Oracle Corporation
 
 This file is part of VirtualBox Open Source Edition (OSE), as
 available from http://www.virtualbox.org. This file is free software;
@@ -26,7 +26,7 @@ CDDL are applicable instead of those of the GPL.
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
 """
-__version__ = "$Revision: 100967 $"
+__version__ = "$Revision: 118412 $"
 
 # Standard Python imports.
 import re;
@@ -51,15 +51,19 @@ g_dsVirtModeDescs  = {
     'hwvirt-np' : 'NestedPaging'
 };
 
-# Arch constants.
-g_k32    = 32;                          # pylint: disable=C0103
-g_k64    = 64;                          # pylint: disable=C0103
-g_k32_64 = 96;                          # pylint: disable=C0103
+## @name Flags.
+## @{
+g_k32           = 32;                   # pylint: disable=C0103
+g_k64           = 64;                   # pylint: disable=C0103
+g_k32_64        = 96;                   # pylint: disable=C0103
+g_kiArchMask    = 96;
+g_kiNoRaw       = 128;                  ##< No raw mode.
+## @}
 
 # Array indexes.
 g_iGuestOsType = 0;
 g_iKind        = 1;
-g_iArch        = 2;
+g_iFlags       = 2;
 g_iMinCpu      = 3;
 g_iMaxCpu      = 4;
 g_iRegEx       = 5;
@@ -74,14 +78,17 @@ g_aaNameToDetails = \
     [ 'WindowsXP_64',   'WindowsXP_64',          g_k64,    1,  32, ['xp64',   'xp64sp[0-9]']],
     [ 'Windows2003',    'Windows2003',           g_k32,    1,  32, ['w2k3',   'w2k3sp[0-9]', 'win2k3', 'win2k3sp[0-9]']],
     [ 'WindowsVista',   'WindowsVista',          g_k32,    1,  32, ['vista',  'vistasp[0-9]']],
+    [ 'WindowsVista_64','WindowsVista_64',       g_k64,    1,  64, ['vista-64', 'vistasp[0-9]-64',]],  # max cpus/cores??
     [ 'Windows2008',    'Windows2008',           g_k32,    1,  64, ['w2k8',   'w2k8sp[0-9]', 'win2k8', 'win2k8sp[0-9]']],     # max cpus/cores??
     [ 'Windows2008_64', 'Windows2008_64',        g_k64,    1,  64, ['w2k8r2', 'w2k8r2sp[0-9]', 'win2k8r2', 'win2k8r2sp[0-9]']], # max cpus/cores??
     [ 'Windows7',       'Windows7',              g_k32,    1,  32, ['w7',     'w7sp[0-9]', 'win7',]],        # max cpus/cores??
     [ 'Windows7_64',    'Windows7_64',           g_k64,    1,  64, ['w7-64',  'w7sp[0-9]-64', 'win7-64',]],  # max cpus/cores??
-    [ 'Windows8',       'Windows8',              g_k32,    1,  32, ['w8',     'w8sp[0-9]', 'win8',]],        # max cpus/cores??
+    [ 'Windows8',       'Windows8',     g_k32 | g_kiNoRaw, 1,  32, ['w8',     'w8sp[0-9]', 'win8',]],        # max cpus/cores??
     [ 'Windows8_64',    'Windows8_64',           g_k64,    1,  64, ['w8-64',  'w8sp[0-9]-64', 'win8-64',]],  # max cpus/cores??
-    [ 'Windows81',      'Windows81',             g_k32,    1,  32, ['w81',    'w81sp[0-9]', 'win81',]],       # max cpus/cores??
+    [ 'Windows81',      'Windows81',    g_k32 | g_kiNoRaw, 1,  32, ['w81',    'w81sp[0-9]', 'win81',]],       # max cpus/cores??
     [ 'Windows81_64',   'Windows81_64',          g_k64,    1,  64, ['w81-64', 'w81sp[0-9]-64', 'win81-64',]], # max cpus/cores??
+    [ 'Windows10',      'Windows10',    g_k32 | g_kiNoRaw, 1,  32, ['w10',    'w10sp[0-9]', 'win10',]],       # max cpus/cores??
+    [ 'Windows10_64',   'Windows10_64',          g_k64,    1,  64, ['w10-64', 'w10sp[0-9]-64', 'win10-64',]], # max cpus/cores??
     [ 'Linux',          'Debian',                g_k32,    1, 256, ['deb[0-9]*', 'debian[0-9]*', ]],
     [ 'Linux_64',       'Debian_64',             g_k64,    1, 256, ['deb[0-9]*-64', 'debian[0-9]*-64', ]],
     [ 'Linux',          'RedHat',                g_k32,    1, 256, ['rhel',   'rhel[0-9]', 'rhel[0-9]u[0-9]']],
@@ -171,13 +178,15 @@ class TestVm(object):
     def __init__(self, oSet, sVmName, sHd = None, sKind = None, acCpusSup = None, asVirtModesSup = None, # pylint: disable=R0913
                  fIoApic = None, fPae = None, sNic0AttachType = None, sHddControllerType = 'IDE Controller',
                  sFloppy = None, fVmmDevTestingPart = None, fVmmDevTestingMmio = False, asParavirtModesSup = None,
-                 fRandomPvPMode = False):
+                 fRandomPvPMode = False, sFirmwareType = 'bios', sChipsetType = 'piix3'):
         self.oSet                    = oSet;
         self.sVmName                 = sVmName;
         self.sHd                     = sHd;          # Relative to the testrsrc root.
         self.acCpusSup               = acCpusSup;
         self.asVirtModesSup          = asVirtModesSup;
         self.asParavirtModesSup      = asParavirtModesSup;
+        self.asParavirtModesSupOrg   = asParavirtModesSup; # HACK ALERT! Trick to make the 'effing random mess not get in the
+                                                           # way of actively selecting virtualization modes.
         self.sKind                   = sKind;
         self.sGuestOsType            = None;
         self.sDvdImage               = None;         # Relative to the testrsrc root.
@@ -188,6 +197,8 @@ class TestVm(object):
         self.sFloppy                 = sFloppy;      # Relative to the testrsrc root, except when it isn't...
         self.fVmmDevTestingPart      = fVmmDevTestingPart;
         self.fVmmDevTestingMmio      = fVmmDevTestingMmio;
+        self.sFirmwareType           = sFirmwareType;
+        self.sChipsetType            = sChipsetType;
 
         self.fSnapshotRestoreCurrent = False;        # Whether to restore execution on the current snapshot.
         self.fSkip                   = False;        # All VMs are included in the configured set by default.
@@ -233,7 +244,7 @@ class TestVm(object):
                 reporter.fatal('The OS of test VM "%s" cannot be guessed' % (self.sVmName,));
 
             # Check for 64-bit, if required and supported.
-            if self.aInfo[g_iArch] == g_k32_64  and  _intersects(asSplit, ['64', 'amd64']):
+            if (self.aInfo[g_iFlags] & g_kiArchMask) == g_k32_64  and  _intersects(asSplit, ['64', 'amd64']):
                 self.sKind = self.sKind + '_64';
         else:
             # Lookup the kind.
@@ -260,9 +271,9 @@ class TestVm(object):
         # Restrict modes and such depending on the OS.
         if self.asVirtModesSup is None:
             self.asVirtModesSup = list(g_asVirtModes);
-            if self.sGuestOsType in (g_ksGuestOsTypeOS2, g_ksGuestOsTypeDarwin):
-                self.asVirtModesSup = [sVirtMode for sVirtMode in self.asVirtModesSup if sVirtMode != 'raw'];
-            if self.sKind.find('_64') > 0:
+            if   self.sGuestOsType in (g_ksGuestOsTypeOS2, g_ksGuestOsTypeDarwin) \
+              or self.sKind.find('_64') > 0 \
+              or (self.aInfo is not None and (self.aInfo[g_iFlags] & g_kiNoRaw)):
                 self.asVirtModesSup = [sVirtMode for sVirtMode in self.asVirtModesSup if sVirtMode != 'raw'];
             # TEMPORARY HACK - START
             sHostName = socket.getfqdn();
@@ -284,6 +295,7 @@ class TestVm(object):
             self.asParavirtModesSup = g_kdaParavirtProvidersSupported[self.sGuestOsType];
             ## @todo Remove this hack as soon as we've got around to explictly configure test variations
             ## on the server side. Client side random is interesting but not the best option.
+            self.asParavirtModesSupOrg = self.asParavirtModesSup;
             if fRandomPvPMode:
                 random.seed();
                 self.asParavirtModesSup = (random.choice(self.asParavirtModesSup),);
@@ -309,12 +321,16 @@ class TestVm(object):
                     fRc = None; # Skip the test.
                 elif self.isViaIncompatible() and oTestDrv.isHostCpuVia():
                     fRc = None; # Skip the test.
+                elif self.isP4Incompatible() and oTestDrv.isHostCpuP4():
+                    fRc = None; # Skip the test.
                 else:
                     oSession = oTestDrv.openSession(oVM);
                     if oSession is not None:
                         fRc =         oSession.enableVirtEx(sVirtMode != 'raw');
                         fRc = fRc and oSession.enableNestedPaging(sVirtMode == 'hwvirt-np');
                         fRc = fRc and oSession.setCpuCount(cCpus);
+                        if cCpus > 1:
+                            fRc = fRc and oSession.enableIoApic(True);
 
                         if sParavirtMode is not None and oSession.fpApiVer >= 5.0:
                             adParavirtProviders = {
@@ -360,7 +376,7 @@ class TestVm(object):
 
     def is64bitRequired(self):
         """ Check if 64-bit is required or not. """
-        return (self.aInfo[g_iArch] & g_k64) != 0;
+        return (self.aInfo[g_iFlags] & g_k64) != 0;
 
     def isLoggedOntoDesktop(self):
         """ Checks if the test VM is logging onto a graphical desktop by default. """
@@ -395,8 +411,30 @@ class TestVm(object):
               or self.sVmName.find('sp2') >= 0 \
               or self.sVmName.find('sp3') >= 0:
                 return True;
+        # XP x64 on a phyical VIA box hangs exactly like a VM.
+        if self.aInfo[g_iKind] in ['WindowsXP_64', 'Windows2003_64']:
+            return True;
+        # Vista 64 throws BSOD 0x5D (UNSUPPORTED_PROCESSOR)
+        if self.aInfo[g_iKind] in ['WindowsVista_64']:
+            return True;
+        # Solaris 11 hangs on VIA, tested on a physical box (testboxvqc)
+        if self.aInfo[g_iKind] in ['Solaris11_64']:
+            return True;
         return False;
 
+    def isP4Incompatible(self):
+        """
+        Identifies VMs that doesn't work on Pentium 4 / Pentium D.
+
+        Returns True if NOT supported on P4, False if it IS supported.
+        """
+        # Stupid 1 kHz timer. Too much for antique CPUs.
+        if self.sVmName.find('rhel5') >= 0:
+            return True;
+        # Due to the boot animation the VM takes forever to boot.
+        if self.aInfo[g_iKind] == 'Windows2000':
+            return True;
+        return False;
 
 
 class BootSectorTestVm(TestVm):
@@ -588,9 +626,13 @@ class TestVmSet(object):
                 if sPvMode not in g_kasParavirtProviders:
                     raise base.InvalidOption('The "--paravirt-modes" value "%s" is not valid; valid values are: %s'
                                              % (sPvMode, ', '.join(g_kasParavirtProviders),));
-            if len(self.asParavirtModes) == 0:
+            if not self.asParavirtModes:
                 self.asParavirtModes = None;
 
+            # HACK ALERT! Reset the random paravirt selection for members.
+            for oTestVm in self.aoTestVms:
+                oTestVm.asParavirtModesSup = oTestVm.asParavirtModesSupOrg;
+
         else:
             return iArg;
         return iArg + 1;
@@ -653,7 +695,9 @@ class TestVmSet(object):
                                             sHddControllerType = oTestVm.sHddControllerType,
                                             sFloppy            = oTestVm.sFloppy,
                                             fVmmDevTestingPart = oTestVm.fVmmDevTestingPart,
-                                            fVmmDevTestingMmio = oTestVm.fVmmDevTestingPart);
+                                            fVmmDevTestingMmio = oTestVm.fVmmDevTestingPart,
+                                            sFirmwareType = oTestVm.sFirmwareType,
+                                            sChipsetType = oTestVm.sChipsetType);
             if oVM is None:
                 return False;
 
@@ -817,6 +861,23 @@ class TestVmManager(object):
 
         oSet = TestVmSet(oTestVmManager = self);
 
+        oTestVm = TestVm(oSet, 'tst-win10-efi', sHd = '4.2/efi/win10-efi-x86.vdi',
+                         sKind = 'Windows10', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi');
+        oSet.aoTestVms.append(oTestVm);
+
+        oTestVm = TestVm(oSet, 'tst-win10-64-efi', sHd = '4.2/efi/win10-efi-amd64.vdi',
+                         sKind = 'Windows10_64', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi');
+        oSet.aoTestVms.append(oTestVm);
+
+        #oTestVm = TestVm(oSet, 'tst-win10-64-efi-ich9', sHd = '4.2/efi/win10-efi-amd64.vdi',
+        #                 sKind = 'Windows10_64', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi',
+        #                 sChipsetType = 'ich9');
+        #oSet.aoTestVms.append(oTestVm);
+
+        oTestVm = TestVm(oSet, 'tst-ubuntu-15_10-64-efi', sHd = '4.2/efi/ubuntu-15_10-efi-amd64.vdi',
+                         sKind = 'Ubuntu_64', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi');
+        oSet.aoTestVms.append(oTestVm);
+
         oTestVm = TestVm(oSet, 'tst-nt4sp1', sHd = '4.2/' + sTxsTransport + '/nt4sp1/t-nt4sp1.vdi',
                          sKind = 'WindowsNT4', acCpusSup = [1]);
         oSet.aoTestVms.append(oTestVm);
@@ -861,10 +922,14 @@ class TestVmManager(object):
                          sKind = 'Windows7', acCpusSup = range(1, 33), fIoApic = True);
         oSet.aoTestVms.append(oTestVm);
 
-        oTestVm = TestVm(oSet, 'tst-win8', sHd = '4.2/win8-32/t-win8.vdi',
-                         sKind = 'Windows8', acCpusSup = range(1, 33), fIoApic = True);
+        oTestVm = TestVm(oSet, 'tst-win8-64', sHd = '4.2/win8-64/t-win8-64.vdi',
+                         sKind = 'Windows8_64', acCpusSup = range(1, 33), fIoApic = True);
         oSet.aoTestVms.append(oTestVm);
 
+        #oTestVm = TestVm(oSet, 'tst-win8-64-ich9', sHd = '4.2/win8-64/t-win8-64.vdi',
+        #                 sKind = 'Windows8_64', acCpusSup = range(1, 33), fIoApic = True, sChipsetType = 'ich9');
+        #oSet.aoTestVms.append(oTestVm);
+
         return oSet;
 
     def getSmokeVmSet(self):
@@ -874,6 +939,24 @@ class TestVmManager(object):
 
         oSet = TestVmSet(oTestVmManager = self);
 
+        oTestVm = TestVm(oSet, 'tst-win10-efi', sHd = '4.2/efi/win10-efi-x86.vdi',
+                         sKind = 'Windows10', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi');
+        oSet.aoTestVms.append(oTestVm);
+
+        oTestVm = TestVm(oSet, 'tst-win10-64-efi', sHd = '4.2/efi/win10-efi-amd64.vdi',
+                         sKind = 'Windows10_64', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi');
+        oSet.aoTestVms.append(oTestVm);
+
+        #oTestVm = TestVm(oSet, 'tst-win10-64-efi-ich9', sHd = '4.2/efi/win10-efi-amd64.vdi',
+        #                 sKind = 'Windows10_64', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi',
+        #                 sChipsetType = 'ich9');
+        #oSet.aoTestVms.append(oTestVm);
+
+        oTestVm = TestVm(oSet, 'tst-ubuntu-15_10-64-efi', sHd = '4.2/efi/ubuntu-15_10-efi-amd64.vdi',
+                         sKind = 'Ubuntu_64', acCpusSup = range(1, 33), fIoApic = True, sFirmwareType = 'efi',
+                         asParavirtModesSup = [g_ksParavirtProviderKVM,]);
+        oSet.aoTestVms.append(oTestVm);
+
         oTestVm = TestVm(oSet, 'tst-nt4sp1', sHd = '4.2/nat/nt4sp1/t-nt4sp1.vdi',
                          sKind = 'WindowsNT4', acCpusSup = [1], sNic0AttachType = 'nat');
         oSet.aoTestVms.append(oTestVm);
@@ -903,6 +986,11 @@ class TestVmManager(object):
                          fIoApic = True, sHddControllerType = 'SATA Controller');
         oSet.aoTestVms.append(oTestVm);
 
+        #oTestVm = TestVm(oSet, 'tst-sol11u1-ich9', sHd = '4.2/nat/sol11u1/t-sol11u1.vdi',
+        #                 sKind = 'Solaris11_64', acCpusSup = range(1, 33), sNic0AttachType = 'nat',
+        #                 fIoApic = True, sHddControllerType = 'SATA Controller', sChipsetType = 'ich9');
+        #oSet.aoTestVms.append(oTestVm);
+
         oTestVm = TestVm(oSet, 'tst-nt4sp6', sHd = '4.2/nt4sp6/t-nt4sp6.vdi',
                          sKind = 'WindowsNT4', acCpusSup = range(1, 33));
         oSet.aoTestVms.append(oTestVm);
@@ -939,10 +1027,14 @@ class TestVmManager(object):
                          sKind = 'Windows7', acCpusSup = range(1, 33), fIoApic = True);
         oSet.aoTestVms.append(oTestVm);
 
-        oTestVm = TestVm(oSet, 'tst-win8', sHd = '4.2/win8-32/t-win8.vdi',
-                         sKind = 'Windows8', acCpusSup = range(1, 33), fIoApic = True);
+        oTestVm = TestVm(oSet, 'tst-win8-64', sHd = '4.2/win8-64/t-win8-64.vdi',
+                         sKind = 'Windows8_64', acCpusSup = range(1, 33), fIoApic = True);
         oSet.aoTestVms.append(oTestVm);
 
+        #oTestVm = TestVm(oSet, 'tst-win8-64-ich9', sHd = '4.2/win8-64/t-win8-64.vdi',
+        #                 sKind = 'Windows8_64', acCpusSup = range(1, 33), fIoApic = True, sChipsetType = 'ich9');
+        #oSet.aoTestVms.append(oTestVm);
+
         return oSet;
 
     def shutUpPyLint(self):
