@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2015 Oracle Corporation
+ * Copyright (C) 2010-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,9 +15,10 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/assert.h>
 #include <iprt/ctype.h>
 #include <iprt/err.h>
@@ -91,9 +92,8 @@ int ScmStreamInitForWriting(PSCMSTREAM pStream, PCSCMSTREAM pRelatedStream)
     scmStreamInitInternal(pStream, true /*fWriteOrRead*/);
 
     /* allocate stuff */
-    size_t cbEstimate = pRelatedStream
-                      ? pRelatedStream->cb + pRelatedStream->cb / 10
-                      : _64K;
+    size_t cbEstimate = !pRelatedStream ? _64K
+                      : pRelatedStream->cb > 0 ? pRelatedStream->cb + pRelatedStream->cb / 10 : 64;
     cbEstimate = RT_ALIGN(cbEstimate, _4K);
     pStream->pch = (char *)RTMemAlloc(cbEstimate);
     if (pStream->pch)
@@ -102,6 +102,8 @@ int ScmStreamInitForWriting(PSCMSTREAM pStream, PCSCMSTREAM pRelatedStream)
                               ? pRelatedStream->cLines + pRelatedStream->cLines / 10
                               : cbEstimate / 24;
         cLinesEstimate = RT_ALIGN(cLinesEstimate, 512);
+        if (cLinesEstimate == 0)
+            cLinesEstimate = 16;
         pStream->paLines = (PSCMSTREAMLINE)RTMemAlloc(cLinesEstimate * sizeof(SCMSTREAMLINE));
         if (pStream->paLines)
         {
@@ -253,7 +255,7 @@ bool ScmStreamIsText(PSCMSTREAM pStream)
     if (RTStrEnd(pStream->pch, pStream->cb))
         return false;
     if (!pStream->cb)
-        return false;
+        return true;
     return true;
 }
 
@@ -325,6 +327,7 @@ int ScmStreamWriteToFile(PSCMSTREAM pStream, const char *pszFilenameFmt, ...)
         rc = RTFileWrite(hFile, pStream->pch, pStream->cb, NULL);
         RTFileClose(hFile);
     }
+    va_end(va);
     return rc;
 }
 
@@ -660,11 +663,9 @@ bool ScmStreamIsAtStartOfLine(PSCMSTREAM pStream)
 }
 
 /**
- * Get a numbered line from the stream (changes the position).
+ * Worker for ScmStreamGetLineByNo and ScmStreamGetLine.
  *
- * A line is always delimited by a LF character or the end of the stream.  The
- * delimiter is not included in returned line length, but instead returned via
- * the @a penmEol indicator.
+ * Works on a fully lineated stream.
  *
  * @returns Pointer to the first character in the line, not NULL terminated.
  *          NULL if the end of the stream has been reached or some problem
@@ -675,38 +676,71 @@ bool ScmStreamIsAtStartOfLine(PSCMSTREAM pStream)
  * @param   pcchLine            The length.
  * @param   penmEol             Where to return the end of line type indicator.
  */
+DECLINLINE(const char *) scmStreamGetLineByNoCommon(PSCMSTREAM pStream, size_t iLine, size_t *pcchLine, PSCMEOL penmEol)
+{
+    Assert(!pStream->fWriteOrRead);
+    Assert(pStream->fFullyLineated);
+
+    /* Check stream status. */
+    if (RT_SUCCESS(pStream->rc))
+    {
+        /* Not at the end of the stream yet? */
+        if (RT_LIKELY(iLine < pStream->cLines))
+        {
+            /* Get the data. */
+            const char *pchRet = &pStream->pch[pStream->paLines[iLine].off];
+            *pcchLine          = pStream->paLines[iLine].cch;
+            *penmEol           = pStream->paLines[iLine].enmEol;
+
+            /* update the stream position. */
+            pStream->off       = pStream->paLines[iLine].off + pStream->paLines[iLine].cch + pStream->paLines[iLine].enmEol;
+            pStream->iLine     = iLine + 1;
+            return pchRet;
+        }
+        pStream->off   = pStream->cb;
+        pStream->iLine = pStream->cLines;
+    }
+    *pcchLine = 0;
+    *penmEol  = SCMEOL_NONE;
+    return NULL;
+}
+
+
+/**
+ * Get a numbered line from the stream (changes the position).
+ *
+ * A line is always delimited by a LF character or the end of the stream.  The
+ * delimiter is not included in returned line length, but instead returned via
+ * the @a penmEol indicator.
+ *
+ * @returns Pointer to the first character in the line, not NULL terminated.
+ *          NULL if the end of the stream has been reached or some problem
+ *          occurred (*pcchLine set to zero and *penmEol to SCMEOL_NONE).
+ *
+ * @param   pStream             The stream.  Must be in read mode.
+ * @param   iLine               The line to get (0-based).
+ * @param   pcchLine            The length.
+ * @param   penmEol             Where to return the end of line type indicator.
+ */
 const char *ScmStreamGetLineByNo(PSCMSTREAM pStream, size_t iLine, size_t *pcchLine, PSCMEOL penmEol)
 {
     AssertReturn(!pStream->fWriteOrRead, NULL);
-    if (RT_FAILURE(pStream->rc))
-        return NULL;
 
     /* Make sure it's fully delineated so we can use the index. */
-    if (RT_UNLIKELY(!pStream->fFullyLineated))
+    if (RT_LIKELY(pStream->fFullyLineated))
+        return scmStreamGetLineByNoCommon(pStream, iLine, pcchLine, penmEol);
+
+    int rc = pStream->rc;
+    if (RT_SUCCESS(rc))
     {
-        int rc = scmStreamLineate(pStream);
-        if (RT_FAILURE(rc))
-            return NULL;
+        rc = scmStreamLineate(pStream);
+        if (RT_SUCCESS(rc))
+            return scmStreamGetLineByNoCommon(pStream, iLine, pcchLine, penmEol);
     }
 
-    /* End of stream? */
-    if (RT_UNLIKELY(iLine >= pStream->cLines))
-    {
-        pStream->off   = pStream->cb;
-        pStream->iLine = pStream->cLines;
-        return NULL;
-    }
-
-    /* Get the data. */
-    const char *pchRet = &pStream->pch[pStream->paLines[iLine].off];
-    *pcchLine          = pStream->paLines[iLine].cch;
-    *penmEol           = pStream->paLines[iLine].enmEol;
-
-    /* update the stream position. */
-    pStream->off       = pStream->paLines[iLine].off + pStream->paLines[iLine].cch + pStream->paLines[iLine].enmEol;
-    pStream->iLine     = iLine + 1;
-
-    return pchRet;
+    *pcchLine = 0;
+    *penmEol  = SCMEOL_NONE;
+    return NULL;
 }
 
 /**
@@ -718,7 +752,7 @@ const char *ScmStreamGetLineByNo(PSCMSTREAM pStream, size_t iLine, size_t *pcchL
  *
  * @returns Pointer to the first character in the line, not NULL terminated.
  *          NULL if the end of the stream has been reached or some problem
- *          occurred.
+ *          occurred (*pcchLine set to zero and *penmEol to SCMEOL_NONE).
  *
  * @param   pStream             The stream.  Must be in read mode.
  * @param   pcchLine            The length.
@@ -726,24 +760,25 @@ const char *ScmStreamGetLineByNo(PSCMSTREAM pStream, size_t iLine, size_t *pcchL
  */
 const char *ScmStreamGetLine(PSCMSTREAM pStream, size_t *pcchLine, PSCMEOL penmEol)
 {
-    if (!pStream->fFullyLineated)
-        return scmStreamGetLineInternal(pStream, pcchLine, penmEol);
-
-    size_t      offCur   = pStream->off;
-    size_t      iCurLine = pStream->iLine;
-    const char *pszLine  = ScmStreamGetLineByNo(pStream, iCurLine, pcchLine, penmEol);
-    if (   pszLine
-        && offCur > pStream->paLines[iCurLine].off)
+    if (RT_LIKELY(pStream->fFullyLineated))
     {
-        offCur -= pStream->paLines[iCurLine].off;
-        Assert(offCur <= pStream->paLines[iCurLine].cch + pStream->paLines[iCurLine].enmEol);
-        if (offCur < pStream->paLines[iCurLine].cch)
-            *pcchLine  -= offCur;
-        else
-            *pcchLine   = 0;
-        pszLine        += offCur;
+        size_t      offCur   = pStream->off;
+        size_t      iCurLine = pStream->iLine;
+        const char *pszLine  = scmStreamGetLineByNoCommon(pStream, iCurLine, pcchLine, penmEol);
+        if (   pszLine
+            && offCur > pStream->paLines[iCurLine].off)
+        {
+            offCur -= pStream->paLines[iCurLine].off;
+            Assert(offCur <= pStream->paLines[iCurLine].cch + pStream->paLines[iCurLine].enmEol);
+            if (offCur < pStream->paLines[iCurLine].cch)
+                *pcchLine  -= offCur;
+            else
+                *pcchLine   = 0;
+            pszLine        += offCur;
+        }
+        return pszLine;
     }
-    return pszLine;
+    return scmStreamGetLineInternal(pStream, pcchLine, penmEol);
 }
 
 /**
@@ -1159,6 +1194,24 @@ int ScmStreamPutCh(PSCMSTREAM pStream, char ch)
     return VINF_SUCCESS;
 }
 
+/**
+ * Puts an EOL marker to the stream.
+ *
+ * @returns IPRt status code.
+ * @param   pStream             The stream.  Must be in write mode.
+ * @param   enmEol              The end-of-line marker to write.
+ */
+int ScmStreamPutEol(PSCMSTREAM pStream, SCMEOL enmEol)
+{
+    if (enmEol == SCMEOL_LF)
+        return ScmStreamWrite(pStream, "\n", 1);
+    if (enmEol == SCMEOL_CRLF)
+        return ScmStreamWrite(pStream, "\r\n", 2);
+    if (enmEol == SCMEOL_NONE)
+        return VINF_SUCCESS;
+    AssertFailedReturn(VERR_INVALID_PARAMETER);
+}
+
 /**
  * Formats a string and writes it to the SCM stream.
  *
