@@ -65,6 +65,7 @@
 # include <VBox/vmm/vmm.h>
 # include <VBox/vmm/stam.h>
 # include <VBox/vmm/cpumctx.h>
+# undef MSR_IA32_APICBASE_BSP
 #endif /* VBOX */
 
 #define R_EAX 0
@@ -115,6 +116,10 @@
 #define DESC_W_MASK     (1 << 9)  /* data: writable */
 
 #define DESC_TSS_BUSY_MASK (1 << 9)
+#ifdef VBOX
+# define DESC_INTEL_UNUSABLE    RT_BIT_32(16+8)      /**< Internal VT-x bit for NULL sectors. */
+# define DESC_RAW_FLAG_BITS     UINT32_C(0x00ffffff) /**< Flag bits we load from the descriptor. */
+#endif
 
 /* eflags masks */
 #define CC_C   	0x0001
@@ -215,7 +220,7 @@
 #define CR0_NE_MASK  (1 << 5)
 #define CR0_WP_MASK  (1 << 16)
 #define CR0_AM_MASK  (1 << 18)
-#define CR0_PG_MASK  (1 << 31)
+#define CR0_PG_MASK  (1U << 31)
 
 #define CR4_VME_MASK  (1 << 0)
 #define CR4_PVI_MASK  (1 << 1)
@@ -700,6 +705,9 @@ typedef struct CPUX86State {
     /* exception/interrupt handling */
     int error_code;
     int exception_is_int;
+#ifdef VBOX
+# define EXCEPTION_IS_INT_VALUE_HARDWARE_IRQ 0x42 /**< Special CPUX86State::exception_is_int value indicating hardware irq.  (HACK ALERT) */
+#endif
     target_ulong exception_next_eip;
     target_ulong dr[8]; /* debug registers */
     union {
@@ -909,7 +917,7 @@ typedef struct CPUX86State_Ver16 {
 # define CPU_RAW_RING0            0x0002 /* Set after first time RawR0 is executed, never cleared. */
 # define CPU_EMULATE_SINGLE_INSTR 0x0040 /* Execute a single instruction in emulation mode */
 # define CPU_EMULATE_SINGLE_STEP  0x0080 /* go into single step mode */
-# define CPU_RAW_HWACC            0x0100 /* Set after first time HWACC is executed, never cleared. */
+# define CPU_RAW_HM            0x0100 /* Set after first time HWACC is executed, never cleared. */
 /** @} */
 #endif /* !VBOX */
 
@@ -930,11 +938,19 @@ void cpu_set_ferr(CPUX86State *s);
 
 /* this function must always be used to load data in the segment
    cache: it synchronizes the hflags with the segment cache values */
+#ifndef VBOX
 static inline void cpu_x86_load_seg_cache(CPUX86State *env,
                                           int seg_reg, unsigned int selector,
                                           target_ulong base,
                                           unsigned int limit,
                                           unsigned int flags)
+#else
+static inline void cpu_x86_load_seg_cache_with_clean_flags(CPUX86State *env,
+                                                           int seg_reg, unsigned int selector,
+                                                           target_ulong base,
+                                                           unsigned int limit,
+                                                           unsigned int flags)
+#endif
 {
     SegmentCache *sc;
     unsigned int new_hflags;
@@ -943,12 +959,8 @@ static inline void cpu_x86_load_seg_cache(CPUX86State *env,
     sc->selector = selector;
     sc->base = base;
     sc->limit = limit;
-#ifndef VBOX
-    sc->flags = flags;
-#else
-    if (flags & DESC_P_MASK)
-        flags |= DESC_A_MASK;           /* Make sure the A bit is set to avoid trouble. */
     sc->flags = flags;
+#ifdef VBOX
     sc->newselector = 0;
     sc->fVBoxFlags  = CPUMSELREG_FLAGS_VALID;
 #endif
@@ -995,6 +1007,23 @@ static inline void cpu_x86_load_seg_cache(CPUX86State *env,
     }
 }
 
+#ifdef VBOX
+/* Raw input, adjust the flags adding the stupid intel flag when applicable. */
+static inline void cpu_x86_load_seg_cache(CPUX86State *env,
+                                          int seg_reg, unsigned int selector,
+                                          target_ulong base,
+                                          unsigned int limit,
+                                          unsigned int flags)
+{
+    flags &= DESC_RAW_FLAG_BITS;
+    if (flags & DESC_P_MASK)
+        flags |= DESC_A_MASK;           /* Make sure the A bit is set to avoid trouble. */
+    else if (selector < 4U)
+        flags |= DESC_INTEL_UNUSABLE;
+    cpu_x86_load_seg_cache_with_clean_flags(env, seg_reg, selector, base, limit, flags);
+}
+#endif
+
 static inline void cpu_x86_load_seg_cache_sipi(CPUX86State *env,
                                                int sipi_vector)
 {
@@ -1169,7 +1198,10 @@ static inline void cpu_get_tb_cpu_state(CPUState *env, target_ulong *pc,
                                         target_ulong *cs_base, int *flags)
 {
     *cs_base = env->segs[R_CS].base;
-    *pc = *cs_base + env->eip;
+    if (env->hflags & HF_CS64_MASK)
+        *pc = *cs_base + env->eip;
+    else
+        *pc = (uint32_t)(*cs_base + env->eip);
     *flags = env->hflags |
         (env->eflags & (IOPL_MASK | TF_MASK | RF_MASK | VM_MASK));
 }
